package com.nordicpeak.flowengine.addflowinstance;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.StringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SessionUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.BaseFlowBrowserModule;
import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.accesscontrollers.SessionAccessController;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.enums.ShowMode;
import com.nordicpeak.flowengine.exceptions.FlowEngineException;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.evaluationprovider.EvaluationProviderException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.exceptions.flow.FlowDisabledException;
import com.nordicpeak.flowengine.exceptions.flow.FlowLimitExceededException;
import com.nordicpeak.flowengine.exceptions.flow.FlowNotAvailiableInRequestedFormat;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.FlowInstanceManagerClosedException;
import com.nordicpeak.flowengine.exceptions.queryinstance.QueryInstanceHTMLException;
import com.nordicpeak.flowengine.exceptions.queryinstance.QueryRequestException;
import com.nordicpeak.flowengine.exceptions.queryinstance.SubmitCheckException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToGetQueryInstanceShowHTMLException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToResetQueryInstanceException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToSaveQueryInstanceException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderException;
import com.nordicpeak.flowengine.interfaces.FlowInstanceAccessController;
import com.nordicpeak.flowengine.interfaces.FlowProcessCallback;
import com.nordicpeak.flowengine.interfaces.Icon;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstanceEvent;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MultiSigningHandler;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.interfaces.SigningProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager;
import com.nordicpeak.flowengine.utils.FlowIconUtils;
import com.nordicpeak.flowengine.utils.MultiSignUtils;

public class AddFlowInstanceModule extends BaseFlowBrowserModule implements FlowProcessCallback, FlowInstanceAccessController {

	public static final StringIntegerValidator stringIntegerValidator = new StringIntegerValidator();

	public static final String SESSION_TAG = AddFlowInstanceModule.class.getSimpleName();

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "CKEditor connector module alias", description = "The full alias of the CKEditor connector module (relative from the contextpath). Leave empty if you do not want to activate file manager for CKEditor")
	protected String ckConnectorModuleAlias;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Use profile design", description = "If set overrides the current design with the design provided by the specified profile (if supplied)")
	protected boolean useProfileDesign = false;

	@InstanceManagerDependency
	protected PDFProvider pdfProvider;
	
	@InstanceManagerDependency
	protected SigningProvider signingProvider;
	
	@InstanceManagerDependency
	protected MultiSigningHandler multiSigningHandler;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(AddFlowInstanceModule.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + AddFlowInstanceModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	public void unload() throws Exception {

		super.unload();

		systemInterface.getInstanceHandler().removeInstance(AddFlowInstanceModule.class, this);
	}

	private ForegroundModuleResponse showError(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, ValidationError validationError) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, FlowInstanceManagerClosedException {

		Document doc = createDocument(req, uriParser, user);

		Element showErrorsElement = doc.createElement("ShowErrors");
		doc.getDocumentElement().appendChild(showErrorsElement);

		showErrorsElement.appendChild(validationError.toXML(doc));

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), this.getDefaultBreadcrumb());
	}
	
	@WebPublic
	public ForegroundModuleResponse add(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, FlowInstanceManagerClosedException {

		Integer flowID = null;

		if (uriParser.size() > 2 && stringIntegerValidator.validateFormat(uriParser.get(2))) {

			flowID = Integer.parseInt(uriParser.get(2));

			AddRequest addRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + flowID, req);

			if (addRequest == null) {

				throw new URINotFoundException(uriParser);
			}
			
			SiteProfile profile = null;
			
			if (addRequest.getProfileID() != null) {
				
				profile = profileHandler.getProfile(addRequest.getProfileID());
				
				if (useProfileDesign && profile != null && profile.getDesign() != null) {
					
					// Is reset by SiteProfileFilterModule on each request so this will only affect the current request
					SessionUtils.setAttribute("preferedDesign", profile.getDesign(), req);
				}
			}

			MutableFlowInstanceManager instanceManager = null;

			try {
				flowID = addRequest.getFlowID();
				
				if (uriParser.size() == 3) {
					
					// Get instance from session
					instanceManager = getUnsavedMutableFlowInstanceManager(flowID, this, req.getSession(true), user, addRequest.getPoster(), profile, uriParser, req, false, false, true, false, addRequest.getRequestMetadata());
					
					if (instanceManager == null) {
						
						// Create new instance
						instanceManager = getUnsavedMutableFlowInstanceManager(flowID, this, req.getSession(true), user, addRequest.getPoster(), profile, uriParser, req, true, false, true, false, addRequest.getRequestMetadata());
						
						if (instanceManager == null) {
							
							log.info("User " + user + " requested non-existing flow with ID " + flowID);
							throw new URINotFoundException(uriParser);
						}
						
						if (addRequest.getAttributeHandler() != null) {
							
							MutableAttributeHandler flowInstanceAttributeHandler = (MutableAttributeHandler) instanceManager.getFlowInstance().getAttributeHandler();
							
							for (Entry<String, String> entry : addRequest.getAttributeHandler().getAttributeMap().entrySet()) {
								
								flowInstanceAttributeHandler.setAttribute(entry.getKey(), entry.getValue());
							}
							
							InstanceMetadata instanceMetadata = new DefaultInstanceMetadata(profile);
							
							instanceManager.initEvaluators(evaluationHandler, user, instanceMetadata, addRequest.getRequestMetadata(), req);
						}
					}
					
				} else {
					
//					Integer flowInstanceID = NumberUtils.toInt(uriParser.get(3));
//
//					if (flowInstanceID != null) {
//
//						//Get saved instance from DB or session
//						instanceManager = getSavedMutableFlowInstanceManager(flowID, flowInstanceID, new SessionAccessController(req.getSession(), SESSION_ACCESS_CONTROLLER_TAG), req.getSession(true), user, uriParser, req, false, true, true, DEFAULT_REQUEST_METADATA);
//					}
//
//					if (instanceManager == null) {
//
//						log.info("User " + user + " requested non-existing flow with ID " + flowID);
						throw new URINotFoundException(uriParser);
//					}
				}
				
				if (addRequest.isDisableSigning()) {
					
					((Flow) instanceManager.getFlowInstance().getFlow()).setRequireSigning(false);
				}
				
				if (addRequest.isDisablePayment()) {
					
					((Flow) instanceManager.getFlowInstance().getFlow()).setPaymentSupportEnabled(false);
				}
				
			} catch (FlowDisabledException e) {

				log.info("User " + user + " requested flow " + flowID + " which is disabled.");
				
				return showError(req, res, user, uriParser, FLOW_DISABLED_VALIDATION_ERROR);
			
			} catch (FlowNotAvailiableInRequestedFormat e) {

				log.info("User " + user + " requested flow " + flowID + " which is not availiable in the requested format.");
				
				return showError(req, res, user, uriParser, FLOW_NOT_AVAILIABLE_IN_REQUESTED_FORMAT_VALIDATION_ERROR);

			} catch (FlowLimitExceededException e) {

				log.info("User " + user + " has reached the flow instance limit for flow " + e.getFlow());

				return handleFlowLimitExceededException(req, res, user, uriParser, e);

			} catch (FlowEngineException e) {

				log.error("Unable to get flow instance manager for flowID " + flowID + " requested by user " + user, e);
				
				return showError(req, res, user, uriParser, ERROR_GETTING_FLOW_INSTANCE_MANAGER_VALIDATION_ERROR);
			}

			try {
				return processFlowRequest(instanceManager, this, this, req, res, user, addRequest.getPoster(), uriParser, true, addRequest.getRequestMetadata());

			} catch (FlowInstanceManagerClosedException e) {

				log.info("User " + user + " requested flow instance manager for flow instance " + e.getFlowInstance() + " which has already been closed. Removing flow instance manager from session.");

				removeMutableFlowInstanceManagerFromSession(instanceManager, req.getSession(false));

				redirectToMethod(req, res, "/add/" + flowID);

				return null;
				
			} catch (QueryInstanceHTMLException e) {

				return processFlowRequestException(instanceManager, req, res, user, addRequest.getPoster(), uriParser, addRequest.getRequestMetadata(), e);

			} catch (RuntimeException e) {

				return processFlowRequestException(instanceManager, req, res, user, addRequest.getPoster(), uriParser, addRequest.getRequestMetadata(), e);
				
			} catch (SubmitCheckException e) {

				return processFlowRequestException(instanceManager, req, res, user, addRequest.getPoster(), uriParser, addRequest.getRequestMetadata(), e);
			}
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "mquery")
	public ForegroundModuleResponse processMutableQueryRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, QueryRequestException, QueryProviderException, EvaluationProviderException, InvalidFlowInstanceStepException, MissingQueryInstanceDescriptor, DuplicateFlowInstanceManagerIDException, UnableToResetQueryInstanceException {

		Integer flowID = null;

		if ((flowID = NumberUtils.toInt(uriParser.get(2))) == null) {

			throw new URINotFoundException(uriParser);
		}

		AddRequest addRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + flowID, req);

		if (addRequest == null) {

			throw new URINotFoundException(uriParser);
		}

		return processMutableQueryRequest(req, res, user, addRequest.getPoster(), uriParser, this, false, false, false, addRequest.getRequestMetadata());
	}

	@WebPublic(alias = "iquery")
	public ForegroundModuleResponse processImmutableQueryRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, QueryRequestException, QueryProviderException, EvaluationProviderException, InvalidFlowInstanceStepException, MissingQueryInstanceDescriptor, DuplicateFlowInstanceManagerIDException {

		throw new UnsupportedOperationException();
	}

	@Override
	public void checkNewFlowInstanceAccess(Flow flow, User user, SiteProfile profile) throws AccessDeniedException {
	}

	@Override
	public void checkFlowInstanceAccess(ImmutableFlowInstance flowInstance, User user) throws AccessDeniedException {

		throw new UnsupportedOperationException();
	}

	protected ForegroundModuleResponse handleFlowLimitExceededException(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, FlowLimitExceededException e) throws IOException {

		return null;
	}

	@Override
	public boolean isMutable(ImmutableFlowInstance flowInstance, User user) {

		if (flowInstance.getStatus() == null || flowInstance.getStatus().isUserMutable()) {

			return true;
		}

		return false;
	}

	@Override
	public String getSubmitActionID() {

		return FlowBrowserModule.SUBMIT_ACTION_ID;
	}

	@Override
	public String getSaveActionID() {

		return FlowBrowserModule.SAVE_ACTION_ID;
	}

	@Override
	public String getPaymentActionID() {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getMultiSigningActionID() {

		return FlowBrowserModule.MULTI_SIGNING_ACTION_ID;
	}
	
	@Override
	protected SigningProvider getSigningProvider() {

		return signingProvider;
	}

	@Override
	public void appendFormData(Document doc, Element baseElement, MutableFlowInstanceManager instanceManager, HttpServletRequest req, User user) {

	}

	@Override
	public void appendShowFlowInstanceData(Document doc, Element baseElement, FlowInstanceManager instanceManager, HttpServletRequest req, User user) {

	}

	@WebPublic(alias = "pdf")
	public ForegroundModuleResponse getEventPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, IOException, AccessDeniedException, FlowDisabledException {

		sendEventPDF(req, res, user, uriParser, new SessionAccessController(req.getSession(), SESSION_TAG), pdfProvider, true);

		return null;
	}

	public void returnToRequester(HttpServletRequest req, HttpServletResponse res, FlowInstanceManager instanceManager) throws IOException {

		AddRequest addRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + instanceManager.getFlowID(), req);

		String returnURL = addRequest.getReturnURL(instanceManager);
		
		if (StringUtils.isEmpty(returnURL)) {
			
			log.error("Return URL is empty! " + addRequest);
		}
		
		res.sendRedirect(returnURL);

		SessionUtils.removeAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + instanceManager.getFlowID(), req);
	}

	@Override
	protected void redirectToSubmitMethod(MutableFlowInstanceManager flowInstanceManager, HttpServletRequest req, HttpServletResponse res) throws IOException {

		SessionAccessController.setSessionAttribute(flowInstanceManager.getFlowInstanceID(), req.getSession(), SESSION_TAG);

		returnToRequester(req, res, flowInstanceManager);
	}

	@Override
	protected void flowInstanceSavedAndClosed(FlowInstanceManager flowInstanceManager, HttpServletRequest req, HttpServletResponse res, User user, FlowInstanceEvent event) throws IOException {

		returnToRequester(req, res, flowInstanceManager);
	}
	
	@Override
	protected void reOpenFlowInstance(Integer flowID, Integer flowInstanceID, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) {

		throw new UnsupportedOperationException();
	}

	@Override
	protected FlowInstanceEvent save(MutableFlowInstanceManager instanceManager, User user, User poster, HttpServletRequest req, String actionID, EventType eventType, Map<String,String> eventAttributes, RequestMetadata requestMetadata) throws FlowInstanceManagerClosedException, UnableToSaveQueryInstanceException, SQLException, FlowDefaultStatusNotFound {

		AddRequest addRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + instanceManager.getFlowID(), req);

		if (addRequest.getCallback() != null) {
			
			addRequest.getCallback().beforeSave(instanceManager, eventType);
		}
		
		FlowInstanceEvent event = super.save(instanceManager, user, poster, req, actionID, eventType, eventAttributes, requestMetadata);
		
		if (addRequest.getCallback() != null) {
			
			addRequest.getCallback().afterSave(instanceManager, eventType, req.getSession(false), event, this);
		}
		
		return event;
	}

	@Override
	protected String getBaseUpdateURL(HttpServletRequest req, URIParser uriParser, User user, ImmutableFlowInstance flowInstance, FlowInstanceAccessController accessController) {

		if (!accessController.isMutable(flowInstance, user)) {

			return null;
		}

		return req.getContextPath() + uriParser.getFormattedURI();
	}

	@Override
	public String getSignFailURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		String preview = instanceManager.getFlowInstance().getFlow().usesPreview() ? "&preview=1" : "";
		
		return RequestUtils.getFullContextPathURL(req) + this.getFullAlias() + "/add/" + instanceManager.getFlowID() + "?signprovidererror=1" + preview;
	}
	
	@Override
	public String getStandalonePaymentURL(FlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getPaymentFailURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getSaveAndSubmitURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		return RequestUtils.getFullContextPathURL(req) + getFullAlias() + "/add/" + instanceManager.getFlowID() + "?save-submit=1&nopost=1";
	}

	@Override
	public void signingComplete(MutableFlowInstanceManager instanceManager, FlowInstanceEvent event, HttpServletRequest req, SiteProfile siteProfile, String actionID) throws FlowInstanceManagerClosedException, UnableToSaveQueryInstanceException, FlowDefaultStatusNotFound, SQLException {

		super.signingComplete(instanceManager, event, req, siteProfile, actionID);
		
		SessionAccessController.setSessionAttribute(instanceManager.getFlowInstanceID(), req.getSession(), SESSION_TAG);
	}
	
	@Override
	public String getSignSuccessURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		if (MultiSignUtils.requiresMultiSigning(instanceManager)) {

			return RequestUtils.getFullContextPathURL(req) + getFullAlias() + "/submittedmultisign/" + instanceManager.getFlowInstanceID();

		} else if (requiresPayment(instanceManager)) {

//			return RequestUtils.getFullContextPathURL(req) + getFullAlias() + "/pay/" + instanceManager.getFlowInstanceID();
			throw new UnsupportedOperationException();
		}

		AddRequest addRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + instanceManager.getFlowID(), req);
		return addRequest.getReturnURL(instanceManager);
	}
	
	@WebPublic(alias = "multisign")
	public ForegroundModuleResponse showMultiSignMessage(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws FlowInstanceManagerClosedException, UnableToGetQueryInstanceShowHTMLException, AccessDeniedException, ModuleConfigurationException, SQLException, URINotFoundException {

		return super.showMultiSignMessage(req, res, user, uriParser, new SessionAccessController(req.getSession(), SESSION_TAG), this, false, ShowMode.PREVIEW);
	}
	
	@WebPublic(alias = "submittedmultisign")
	public ForegroundModuleResponse showSubmittedMultiSignMessage(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws FlowInstanceManagerClosedException, UnableToGetQueryInstanceShowHTMLException, AccessDeniedException, ModuleConfigurationException, SQLException, URINotFoundException {
		
		return super.showMultiSignMessage(req, res, user, uriParser, new SessionAccessController(req.getSession(), SESSION_TAG), this, false, ShowMode.SUBMIT);
	}
	
	@Override
	protected MultiSigningHandler getMultiSigningHandler() {

		return multiSigningHandler;
	}

	@Override
	public String getPaymentSuccessURL(FlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	protected String getEventPDFLink(FlowInstanceManager instanceManager, ImmutableFlowInstanceEvent event, HttpServletRequest req, User user) {

		if (event.getAttributeHandler().getPrimitiveBoolean("pdf")) {

			return this.getModuleURI(req) + "/pdf/" + instanceManager.getFlowInstanceID() + "/" + event.getEventID();
		}

		return null;
	}

	@Override
	public String getAbsoluteFileURL(URIParser uriParser, Object object) {

		if (ckConnectorModuleAlias != null) {

			return uriParser.getContextPath() + ckConnectorModuleAlias;
		}

		return null;
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws ModuleConfigurationException, SQLException {

		if (validationErrors != null) {
			throw new UnsupportedOperationException(StringUtils.toCommaSeparatedString(validationErrors));
		}
		
		throw new UnsupportedOperationException();
	}

	public ForegroundModuleResponse createAddFlowInstanceRequest(HttpServletRequest req, HttpServletResponse res, Integer flowID, User user, User poster, SiteProfile profile, String returnURL, RequestMetadata requestMetadata, AddFlowInstanceCallback callback) throws IOException {
		
		return createAddFlowInstanceRequest(req, res, flowID, user, poster, true, false, profile, returnURL, requestMetadata, callback, null);
	}
	
	public ForegroundModuleResponse createAddFlowInstanceRequest(HttpServletRequest req, HttpServletResponse res, Integer flowID, User user, User poster, boolean disableSigning, boolean disablePayment, SiteProfile profile, String returnURL, RequestMetadata requestMetadata, AddFlowInstanceCallback callback, MutableAttributeHandler attributeHandler) throws IOException {

		AddRequest existingRequest = (AddRequest) SessionUtils.getAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + flowID, req);
		
		if (existingRequest != null) {
			
			log.info("Removing old add flowinstance session for flow ID " + flowID);
			
			try {
				MutableFlowInstanceManager instanceManager = getUnsavedMutableFlowInstanceManager(flowID, this, req.getSession(false), user, poster, profile, null, req, false, false, false, false, requestMetadata);
				
				if (instanceManager != null) {
					removeMutableFlowInstanceManagerFromSession(instanceManager, req.getSession(false));
				}
				
			} catch (Exception e) {
				
				log.warn("Error removing old instance manager", e);
			}
			
			SessionUtils.removeAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + flowID, req);
		}
		
		SessionUtils.setAttribute(getClass().getName() + "-" + moduleDescriptor.getModuleID() + "-" + flowID, new AddRequest(flowID, returnURL, poster, disableSigning, disablePayment, profile, requestMetadata, callback, attributeHandler), req);

		res.sendRedirect(getModuleURI(req) + "/add/" + flowID);
		return null;
	}

	@Override
	protected Icon getFlowTypeIcon(Integer flowTypeID) throws SQLException {

		return getBareFlowType(flowTypeID);
	}

	@Override
	protected Icon getFlowIcon(Integer flowID) throws SQLException {

		return FlowIconUtils.getFlowIcon(getBareFlow(flowID));
	}

	@Override
	public String getFlowInstanceSessionPrefix() {
		return SESSION_TAG;
	}
}
