package com.nordicpeak.flowengine.addflowinstance;

import java.io.Serializable;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.string.StringUtils;

import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;

public class AddRequest implements Serializable {

	private static final long serialVersionUID = 5899990452844912983L;

	private final Integer flowID;
	private final String returnURL;
	private final User poster;
	private final Integer siteProfileID;
	private final AddFlowInstanceCallback callback;
	private final RequestMetadata requestMetadata;
	private final MutableAttributeHandler attributeHandler;
	private final boolean disableSigning;
	private final boolean disablePayment;

	public AddRequest(Integer flowID, String returnURL, RequestMetadata requestMetadata) {
		this(flowID, returnURL, null, true, false, null, requestMetadata, null, null);
	}

	public AddRequest(Integer flowID, String returnURL, User poster, boolean disableSigning, boolean disablePayment, SiteProfile profile, RequestMetadata requestMetadata, AddFlowInstanceCallback callback, MutableAttributeHandler attributeHandler) {
		super();

		this.flowID = flowID;
		this.returnURL = returnURL;
		this.poster = poster;
		this.disableSigning = disableSigning;
		this.disablePayment = disablePayment;
		this.requestMetadata = requestMetadata;
		this.callback = callback;
		this.attributeHandler = attributeHandler;
		
		if (profile != null) {
			
			this.siteProfileID = profile.getProfileID();
			
		} else {
		
			siteProfileID = null;
		}
	}

	public Integer getFlowID() {

		return flowID;
	}

	public String getReturnURL(FlowInstanceManager instanceManager) {

		if (callback != null) {
			
			String callbackReturnURL = callback.getCustomReturnURL(instanceManager);
			
			if (!StringUtils.isEmpty(callbackReturnURL)) {
				
				return callbackReturnURL;
			}
		}

		return returnURL;
	}

	public User getPoster() {

		return poster;
	}

	public AddFlowInstanceCallback getCallback() {

		return callback;
	}

	public RequestMetadata getRequestMetadata() {
		
		return requestMetadata;
	}
	
	public Integer getProfileID() {
		
		return siteProfileID;
	}
	
	public synchronized MutableAttributeHandler getAttributeHandler() {
		
		return attributeHandler;
	}
	
	public boolean isDisableSigning() {
		return disableSigning;
	}
	
	public boolean isDisablePayment() {
		return disablePayment;
	}
	
}
