package com.nordicpeak.flowengine.queries.alternativelistcheckboxquery;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.annotations.TextTagReplace;
import com.nordicpeak.flowengine.interfaces.ExportAlternativeID;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.queries.checkboxquery.Columns;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "alternative_list_checkbox_queries")
@XMLElement
public class AlternativeListCheckboxQuery extends FixedAlternativesBaseQuery implements AlternativeListQuery, ExportAlternativeID {
	
	private static final long serialVersionUID = -842191226937409416L;
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer minChecked;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer maxChecked;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean requireAllChecked;
	
	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String listName;
	
	@TextTagReplace
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String emptyListDescription;
	
	@WebPopulate(maxLength = 255)
	@DAOManaged
	@XMLElement
	private String freeTextAlternative;
	
	@DAOManaged
	@WebPopulate(maxLength = 10, required = true)
	@XMLElement
	private Columns columns = Columns.ONE;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideTitle;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean exportAlternativeID;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<AlternativeListCheckboxQueryInstance> instances;
	
	private AlternativeListInstance alternativeList;
	
	public static long getSerialversionuid() {
		
		return serialVersionUID;
	}
	
	@Override
	public Integer getQueryID() {
		
		return queryID;
	}
	
	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {
		
		if (alternativeList != null) {
			
			return alternativeList.getAlternatives();
		}
		
		return null;
	}
	
	public String getListName() {
		return listName;
	}
	
	public void setListName(String listIdentifier) {
		this.listName = listIdentifier;
	}
	
	public AlternativeListInstance getAlternativeList() {
		return alternativeList;
	}
	
	public void setAlternativeList(AlternativeListInstance alternativeList) {
		this.alternativeList = alternativeList;
	}
	
	public List<AlternativeListCheckboxQueryInstance> getInstances() {
		
		return instances;
	}
	
	public void setInstances(List<AlternativeListCheckboxQueryInstance> instances) {
		
		this.instances = instances;
	}
	
	public void setQueryID(int queryID) {
		
		this.queryID = queryID;
	}
	
	@Override
	public String getFreeTextAlternative() {
		return freeTextAlternative;
	}
	
	public void setFreeTextAlternative(String freeTextAlternative) {
		this.freeTextAlternative = freeTextAlternative;
	}
	
	public Integer getMinChecked() {
		
		return minChecked;
	}
	
	public void setMinChecked(Integer minChecked) {
		
		this.minChecked = minChecked;
	}
	
	public Integer getMaxChecked() {
		
		return maxChecked;
	}
	
	public void setMaxChecked(Integer maxChecked) {
		
		this.maxChecked = maxChecked;
	}
	
	public Columns getColumns() {
		return columns;
	}
	
	public void setColumns(Columns columns) {
		this.columns = columns;
	}
	
	public boolean isHideTitle() {
		
		return hideTitle;
	}
	
	public void setHideTitle(boolean hideTitle) {
		
		this.hideTitle = hideTitle;
	}
	
	@Override
	public String toString() {
		
		if (this.queryDescriptor != null) {
			
			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}
		
		return "AlternativeListCheckboxQuery (queryID: " + queryID + ")";
	}
	
	@Override
	public String getXSDTypeName() {
		
		return "AlternativeListCheckboxQuery" + queryID;
	}
	
	@Override
	public void toXSD(Document doc) {
		
		if (maxChecked != null) {
			
			if (minChecked != null) {
				
				toXSD(doc, minChecked, maxChecked);
				
			} else {
				
				toXSD(doc, maxChecked);
			}
			
		} else {
			
			if (alternativeList != null) {
				
				if (minChecked != null) {
					
					toXSD(doc, minChecked, alternativeList.getAlternatives().size());
					
				} else {
					
					toXSD(doc, alternativeList.getAlternatives().size());
				}
				
			} else {
				
				toXSD(doc, 1);
				
			}
		}
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<>();
		
		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		freeTextAlternative = XMLValidationUtils.validateParameter("freeTextAlternative", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		hideTitle = xmlParser.getPrimitiveBoolean("hideTitle");
		
		minChecked = XMLValidationUtils.validateParameter("minChecked", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		maxChecked = XMLValidationUtils.validateParameter("maxChecked", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		requireAllChecked = xmlParser.getPrimitiveBoolean("requireAllChecked");
		
		listName = XMLValidationUtils.validateParameter("listName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		emptyListDescription = XMLValidationUtils.validateParameter("emptyListDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (attributeName != null) {
			
			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		exportAlternativeID  = xmlParser.getPrimitiveBoolean("exportAlternativeID");
		
		if (!StringUtils.isEmpty(xmlParser.getString("columns"))) {
			
			columns = XMLValidationUtils.validateParameter("columns", xmlParser, true, new EnumPopulator<Columns>(Columns.class), errors);
		}
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
	
	@Override
	public Element toXML(Document doc) {
		
		Element element = super.toXML(doc);
		
		if (alternativeList != null) {
			
			//TODO don't append alternatives on export
			XMLUtils.append(doc, element, "Alternatives", alternativeList.getAlternatives());
		}
		
		return element;
	}
	
	@Override
	public String getAlternativeListName() {
		return listName;
	}

	public boolean isSetAsAttribute() {
	
		return setAsAttribute;
	}
	
	public void setSetAsAttribute(boolean setAsAttribute) {
	
		this.setAsAttribute = setAsAttribute;
	}
	
	public String getAttributeName() {
	
		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	@Override
	public boolean isExportAlternativeID() {
	
		return exportAlternativeID;
	}

	
	public void setExportAlternativeID(boolean exportAlternativeID) {
	
		this.exportAlternativeID = exportAlternativeID;
	}

	public boolean isRequireAllChecked() {
		return requireAllChecked;
	}

	public void setRequireAllChecked(boolean requireAllChecked) {
		this.requireAllChecked = requireAllChecked;
	}

}
