package com.nordicpeak.flowengine.queries.alternativelistcheckboxquery;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class AlternativeListCheckboxQueryCRUD extends BaseQueryCRUD<AlternativeListCheckboxQuery, AlternativeListCheckboxQueryProviderModule> {
	
	
	protected AnnotatedDAOWrapper<AlternativeListCheckboxQuery, Integer> queryDAO;
	
	public AlternativeListCheckboxQueryCRUD(AnnotatedDAOWrapper<AlternativeListCheckboxQuery, Integer> queryDAO, BeanRequestPopulator<AlternativeListCheckboxQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, AlternativeListCheckboxQueryProviderModule callback) {
		
		super(AlternativeListCheckboxQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.queryDAO = queryDAO;
	}
	
	@Override
	protected void appendUpdateFormData(AlternativeListCheckboxQuery query, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		
		super.appendUpdateFormData(query, doc, updateTypeElement, user, req, uriParser);
		
		Element listsElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Lists");
		
		for (AlternativeListDescriptor list : callback.getAlternativeLists()) {
			
			AlternativeListProvider provider = callback.getAlternativeListProvider(list.getProviderID());
			
			Element providerElement = XMLUtils.appendNewElement(doc, listsElement, "List");
			
			XMLUtils.appendNewElement(doc, providerElement, "Name", provider.getProviderName() + ": " + list.getName());
			XMLUtils.appendNewElement(doc, providerElement, "ID", list.getName());
		}
		
		AlternativeListDescriptor currentListDescriptor = callback.getAlternativeListDescriptor(query);
		
		if (currentListDescriptor != null) {
			
			XMLUtils.appendNewElement(doc, updateTypeElement, "CurrentList", currentListDescriptor.getName());
			
		} else if (!StringUtils.isEmpty(query.getListName())) {
			
			Element providerElement = XMLUtils.appendNewElement(doc, listsElement, "List");
			
			XMLUtils.appendNewElement(doc, providerElement, "Name", callback.getMissingAlternativeList() + query.getListName());
			XMLUtils.appendNewElement(doc, providerElement, "ID", query.getListName());
		}
	}
	
	@Override
	protected AlternativeListCheckboxQuery populateFromUpdateRequest(AlternativeListCheckboxQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		AlternativeListCheckboxQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		if (req.getParameter("useFreeTextAlternative") != null && StringUtils.isEmpty(bean.getFreeTextAlternative())) {
			validationErrors.add(new ValidationError("freeTextAlternative", ValidationErrorType.RequiredField));
		}
		
		Integer minChecked = query.getMinChecked();
		Integer maxChecked = query.getMaxChecked();
		
		if (minChecked != null && maxChecked != null && (minChecked > maxChecked || maxChecked < minChecked)) {
			validationErrors.add(new ValidationError("MinCheckedBiggerThanMaxChecked"));
		}
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		return query;
	}
	
}
