<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/alternativelistpriorityquery.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/alternativelistpriorityquery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
	
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
							<xsl:if test="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/description"> 
								<xsl:text> hasDescription</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/queryID" />
					</xsl:call-template>
				</div>
				
				<xsl:if test="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:if test="AlternativeListPriorityQueryInstance/Alternatives">
					<xsl:apply-templates select="AlternativeListPriorityQueryInstance/Alternatives/Alternative" mode="show"/>
				</xsl:if>
				
			</article>
		
		</div>	
	
	</xsl:template>
	
	<xsl:template match="Alternative" mode="show">
		
		<div class="floatleft full">
			<div class="floatleft" style="min-width: 50px;">
				<strong><xsl:value-of select="position()"/></strong>
				<xsl:text>.&#160;</xsl:text>
			</div>
			
			<div class="floatleft">
				<xsl:value-of select="name"/>
			</div>
		</div>
	
	</xsl:template>
			
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/queryID)" />
	
		<div id="{$queryID}">
		
			<xsl:attribute name="class">
				<xsl:text>query alternativelistpriorityquery</xsl:text>
				<xsl:if test="EnableAjaxPosting"><xsl:text> enableAjaxPosting</xsl:text></xsl:if>
				<xsl:if test="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<div class="minChoices" style="display: none;">
				<xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/minChoices"/>
			</div>
			
			<xsl:call-template name="extendShowQueryForm"/>
	
			<a name="{$queryID}" />
	
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
						<xsl:apply-templates select="ValidationErrors/validationError[not(fieldName)]" />
						
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="AlternativeListPriorityQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/helpText">		
						<xsl:apply-templates select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/helpText" />
					</xsl:if>
					
				</div>
					
				<xsl:if test="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
				
				<xsl:choose>
					<xsl:when test="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/Alternatives/Alternative">
						
						<fieldset>
					
							<xsl:call-template name="dropdownsLoop">
								<xsl:with-param name="minChoices" select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/minChoices" />
								<xsl:with-param name="maxChoices" select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/maxChoices" />
								<xsl:with-param name="requestparameters" select="requestparameters" />
								<xsl:with-param name="validationErrors" select="ValidationErrors" />
							</xsl:call-template>
		
						</fieldset>
						
					</xsl:when>
					<xsl:otherwise>
					
						<span class="italic html-description">
							<p class="error">
								<xsl:call-template name="replaceLineBreak">
									<xsl:with-param name="string" select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/emptyListDescription" />
								</xsl:call-template>
							</p>
						</span>
					
					</xsl:otherwise>
				</xsl:choose>
				
				<div class="hidden CanOnlyChooseAlternativeOnceTemplate">
					<xsl:call-template name="CanOnlyChooseAlternativeOnceValidationError"/>
				</div>
				
			</article>
	
		</div>
		
		<xsl:call-template name="showQueryFormInit"/>
		
	</xsl:template>
	
	<xsl:template name="extendShowQueryForm"/>
	
	<xsl:template name="showQueryFormInit">
	
		<script type="text/javascript">$(function(){initAlternativeListPriorityQuery('<xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/queryID" />');});</script>
	
	</xsl:template>
	
	<xsl:template name="dropdownsLoop">
		<xsl:param name="index" select="0"/>
		<xsl:param name="minChoices"/>
		<xsl:param name="maxChoices"/>
		<xsl:param name="requestparameters"/>
		<xsl:param name="validationErrors"/>
		
		<xsl:if test="$maxChoices > $index">
		
			<xsl:variable name="dropDownName">
					<xsl:value-of select="'q'"/>
					<xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/queryID"/>
					<xsl:value-of select="'_alternative_'"/>
					<xsl:value-of select="$index"/>
				</xsl:variable>
		
			<xsl:variable name="class">
				<xsl:text>floatleft full marginbottom</xsl:text>
				
				<xsl:if test="$validationErrors/validationError[fieldName = $dropDownName]">
					<xsl:text> invalid input-error</xsl:text>
				</xsl:if>
			</xsl:variable>
	
			<div class="{$class}  split">
			
				<xsl:if test="$index + 1 > $minChoices">
					<xsl:attribute name="style">
						<xsl:text>display: none;</xsl:text>
					</xsl:attribute>
				</xsl:if>
		
				<div class="floatleft marginright namecolumn">
					<xsl:value-of select="$index + 1"/>
					<xsl:text>.&#160;</xsl:text>
				</div>
				
				<xsl:variable name="selectedAlternativeID" select="AlternativeListPriorityQueryInstance/Alternatives/Alternative[position() = ($index + 1)]/alternativeID" />
			
				<select name="{$dropDownName}" class="floatleft alternativelistpriorityquery fifty">
				
					<xsl:if test="$index + 1 > $minChoices">
						<xsl:attribute name="disabled">
							<xsl:value-of select="'true'" /> 
						</xsl:attribute>
					</xsl:if>
				
					<option value=""><xsl:value-of select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/shortDescription"/></option>
					
					<xsl:for-each select="AlternativeListPriorityQueryInstance/AlternativeListPriorityQuery/Alternatives/Alternative">
						<option value="{alternativeID}" data-multiple="{allowMultiple}">
						
							<xsl:choose>
								<xsl:when test="$requestparameters">
									<xsl:if test="$requestparameters/parameter[name = $dropDownName]/value = alternativeID">
										<xsl:attribute name="selected"/>
									</xsl:if>
								</xsl:when>
								<xsl:otherwise>
									<xsl:if test="$selectedAlternativeID = alternativeID">
										<xsl:attribute name="selected" />
									</xsl:if>
								</xsl:otherwise>
							</xsl:choose>
							
							<xsl:value-of select="name" />
							
						</option>
					</xsl:for-each>
	
				</select>
				
				<div class="floatleft margin bigmarginleft errorIconDiv">
					<xsl:apply-templates select="$validationErrors/validationError[fieldName = $dropDownName]" mode="inline"/>
				</div>
			
			</div>
		
			<xsl:call-template name="dropdownsLoop">
				<xsl:with-param name="index" select="$index + 1" />
				<xsl:with-param name="minChoices" select="$minChoices" />
				<xsl:with-param name="maxChoices" select="$maxChoices" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="validationErrors" select="$validationErrors" />
			</xsl:call-template>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'CanOnlyChooseAlternativeOnce']" mode="inline" name="CanOnlyChooseAlternativeOnceValidationError">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.CanOnlyChooseAlternativeOnce"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']" mode="inline">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.RequiredPrioritySelection"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TooManyAlternativesSelected']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part1"/>
				<xsl:value-of select="maxChecked"/>
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part2"/>
				<xsl:value-of select="checked"/>
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part3"/>
			</strong>
		</span>
		
	</xsl:template>		
	
	<xsl:template match="validationError[messageKey = 'TooFewAlternativesSelected']">
		
		<span>
			<strong data-icon-before="!">
			<xsl:value-of select="$i18n.TooFewAlternativesSelected.part1"/>
			<xsl:value-of select="minChecked"/>
			<xsl:value-of select="$i18n.TooFewAlternativesSelected.part2"/>
			<xsl:value-of select="checked"/>
			<xsl:value-of select="$i18n.TooFewAlternativesSelected.part3"/>
			</strong>
		</span>
		
	</xsl:template>	
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.RequiredQuery"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
	
</xsl:stylesheet>