package com.nordicpeak.flowengine.evaluators.anonymousposterevaluator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.querystateevaluator.SelectionMode;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

public class AnonymousPosterEvaluationProviderModule extends BaseEvaluationProviderModule<AnonymousPosterEvaluator> {
	
	private static final RelationQuery RELATION_QUERY = new RelationQuery(FlowInstance.OWNERS_RELATION, FlowInstance.ATTRIBUTES_RELATION);
	
	protected AnnotatedDAO<AnonymousPosterEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<AnonymousPosterEvaluator, Integer> evaluatorIDParamFactory;
	
	protected AnonymousPosterEvaluatorCRUD evaluatorCRUD;
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@InstanceManagerDependency(required = true)
	protected FlowInstanceAdminModule flowInstanceAdminModule;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AnonymousPosterEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(AnonymousPosterEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new AnonymousPosterEvaluatorCRUD(AnonymousPosterEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, AnonymousPosterEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (!(queryInstance instanceof FixedAlternativesQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " does not implement interface " + FixedAlternativesQueryInstance.class.getName());
			
			return null;
		}
		
		return null;
	}
	
	@EventListener(channel = FlowInstanceManager.class, priority = 1)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {
		
		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();
		
		List<FixedAlternativesQueryInstance> queryInstances = flowInstanceManager.getQueries(FixedAlternativesQueryInstance.class);
		
		if (queryInstances != null) {
			
			for (FixedAlternativesQueryInstance queryInstance : queryInstances) {
				
				if (queryInstance == null || !queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {
					
					continue;
				}
				
				User oldPoster = flowInstanceManager.getFlowInstance().getPoster();
				
				if (oldPoster != null) {
					
					outer2: for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {
						
						if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {
							
							AnonymousPosterEvaluator evaluator = getEvaluator(evaluatorDescriptor);
							
							List<? extends ImmutableAlternative> alternatives = queryInstance.getAlternatives();
							
							String freeTextAlternative = queryInstance.getFreeTextAlternativeValue();
							
							if (CollectionUtils.isEmpty(alternatives) && freeTextAlternative == null) {
								
								continue;
								
							} else if (evaluator.getSelectionMode() == SelectionMode.ANY) {
								
								if (freeTextAlternative != null && evaluator.isFreeTextAlternative()) {
									
									anonymizePoster(event, flowInstanceManager, oldPoster);
									return;
								}
								
								if (alternatives != null && !CollectionUtils.isEmpty(evaluator.getAlternativeIDs())) {
									
									for (ImmutableAlternative alternative : alternatives) {
										
										if (evaluator.getAlternativeIDs().contains(alternative.getAlternativeID())) {
											
											anonymizePoster(event, flowInstanceManager, oldPoster);
											return;
										}
									}
								}
								
								continue;
								
							} else if (evaluator.getSelectionMode() == SelectionMode.ALL) {
								
								if (evaluator.isFreeTextAlternative() && freeTextAlternative == null) {
									
									continue;
								}
								
								if (!CollectionUtils.isEmpty(evaluator.getAlternativeIDs())) {
									
									outer: for (Integer alternativeID : evaluator.getAlternativeIDs()) {
										
										if (alternatives != null) {
											
											for (ImmutableAlternative alternative : alternatives) {
												
												if (alternative.getAlternativeID().equals(alternativeID)) {
													
													continue outer;
												}
											}
										}
										
										continue outer2;
									}
								}
								
								anonymizePoster(event, flowInstanceManager, oldPoster);
								return;
								
							} else {
								
								throw new RuntimeException("Unknown selection mode: " + evaluator.getSelectionMode());
							}
						}
					}
				}
			}
		}
	}
	
	protected void anonymizePoster(SubmitEvent event, FlowInstanceManager flowInstanceManager, User oldPoster) throws SQLException {
		
		log.info("Anonymizing flow instance " + flowInstanceManager.getFlowInstance());
		
		FlowInstance flowInstance = (FlowInstance) flowInstanceManager.getFlowInstance();
		flowInstance.setPoster(null);
		flowInstance.setEditor(null);
		event.getEvent().setPoster(null);
		
		if (flowInstance.getOwners() != null) {
			
			flowInstance.getOwners().remove(oldPoster);
		}
		
		flowInstance.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_ANONYMIZED_ATTRIBUTE, true);
		
		TransactionHandler transactionHandler = null;
		
		try {
			transactionHandler = flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().createTransaction();
			
			flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().update(flowInstance, transactionHandler, RELATION_QUERY);
			
			LowLevelQuery<FlowInstance> clearUserQuery = new LowLevelQuery<FlowInstance>("UPDATE " + flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().getTableName() + " SET poster = NULL, editor = NULL WHERE flowInstanceID = ?");
			clearUserQuery.addParameter(flowInstance.getFlowInstanceID());
			flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().update(clearUserQuery, transactionHandler);
			
			LowLevelQuery<FlowInstanceEvent> clearEventPosterQuery = new LowLevelQuery<FlowInstanceEvent>("UPDATE " + flowInstanceAdminModule.getDAOFactory().getFlowInstanceEventDAO().getTableName() + " SET poster = null WHERE eventID = ?");
			clearEventPosterQuery.addParameter(event.getEvent().getEventID());
			flowInstanceAdminModule.getDAOFactory().getFlowInstanceEventDAO().update(clearEventPosterQuery, transactionHandler);
			
			transactionHandler.commit();
			
			systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(FlowInstance.class, CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
			
		} finally {
			TransactionHandler.autoClose(transactionHandler);
		}
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		AnonymousPosterEvaluator evaluator = new AnonymousPosterEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		evaluator.setSelectionMode(SelectionMode.ANY);
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	protected AnonymousPosterEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {
		
		return evaluatorDAO.get(new HighLevelQuery<AnonymousPosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		AnonymousPosterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<AnonymousPosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<AnonymousPosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return FixedAlternativesQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		AnonymousPosterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<AnonymousPosterEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		if (evaluator.getAlternativeIDs() != null) {
			
			List<? extends ImmutableAlternative> sourceAlternatives = ((FixedAlternativesQuery) sourceQuery).getAlternatives();
			List<? extends ImmutableAlternative> copyAlternatives = ((FixedAlternativesQuery) copyQuery).getAlternatives();

			if (!CollectionUtils.isEmpty(sourceAlternatives) && !CollectionUtils.isEmpty(copyAlternatives)) {
				
				List<Integer> newRequiredAlternativeIDs = new ArrayList<Integer>(evaluator.getAlternativeIDs().size());
				
				for (Integer requiredAlternativeID : evaluator.getAlternativeIDs()) {
					
					int alternativeIndex = 0;
					
					while (alternativeIndex < sourceAlternatives.size()) {
						
						if (sourceAlternatives.get(alternativeIndex).getAlternativeID().equals(requiredAlternativeID)) {
							
							newRequiredAlternativeIDs.add(copyAlternatives.get(alternativeIndex).getAlternativeID());
							
							break;
						}
						
						alternativeIndex++;
					}
				}
				
				evaluator.setAlternativeIDs(newRequiredAlternativeIDs);
			}
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	@Override
	public AnonymousPosterEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		AnonymousPosterEvaluator evaluator = new AnonymousPosterEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		if (evaluator.getAlternativeIDs() != null && query instanceof FixedAlternativesQuery && ((FixedAlternativesQuery) query).getAlternativeConversionMap() != null) {
			
			Map<Integer, Integer> conversionMap = ((FixedAlternativesQuery) query).getAlternativeConversionMap();
			
			List<Integer> newIDList = new ArrayList<Integer>(evaluator.getAlternativeIDs().size());
			
			for (Integer alternativeID : evaluator.getAlternativeIDs()) {
				
				Integer newID = conversionMap.get(alternativeID);
				
				if (newID != null) {
					
					newIDList.add(newID);
				}
			}
			
			evaluator.setAlternativeIDs(newIDList);
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
	
	public QueryHandler getQueryHandler() {
		
		return queryHandler;
	}
	
}
