package com.nordicpeak.flowengine.integration.artvise;

import java.io.File;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.SSLContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.ws.BindingProvider;

import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.ArrayOfCaseCategoryItemDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.ArrayOfCounterpartDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.ArrayOfDeedDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.ArrayOfFileDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.CaseCategoryItemDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.CaseDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.CounterpartDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.CreateObjectResult;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.DeedDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.FileDto;
import org.datacontract.schemas._2004._07.artvise_mercury_base_datacontracts.OperationResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.artvise.mercuryapi.IService;
import se.artvise.mercuryapi.Service;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.jaxws.WSUtils;
import se.unlogic.standardutils.ssl.SSLUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceAttribute;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.integration.artvise.beans.IntegrationEndpoint;
import com.nordicpeak.flowengine.integration.artvise.beans.IntegrationSettings;
import com.nordicpeak.flowengine.integration.artvise.beans.QueuedIntegrationRequest;
import com.nordicpeak.flowengine.integration.base.BaseIntegrationProviderModule;
import com.nordicpeak.flowengine.integration.base.exceptions.IntegrationRequestException;
import com.nordicpeak.flowengine.integration.base.exceptions.IntegrationSettingsException;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;
import com.nordicpeak.flowengine.utils.PDFByteAttachment;

public class ArtviseIntegrationProviderModule extends BaseIntegrationProviderModule<QueuedIntegrationRequest, IntegrationSettings, IntegrationEndpoint> {

	private static final AnnotatedRequestPopulator<IntegrationSettings> POPULATOR = new AnnotatedRequestPopulator<>(IntegrationSettings.class);

	private static final String SUPPORTED_TAGS;

	static {
		ArrayList<String> tagList = new ArrayList<>(40);

		tagList.add(StandardFlowNotificationHandler.CONTACT_TAG_SOURCE_FACTORY.getAvailableTags());
		tagList.add(StandardFlowNotificationHandler.FLOW_TAG_SOURCE_FACTORY.getAvailableTags());
		tagList.add(StandardFlowNotificationHandler.FLOWINSTANCE_TAG_SOURCE_FACTORY.getAvailableTags());
		tagList.add(StandardFlowNotificationHandler.STATUS_TAG_SOURCE_FACTORY.getAvailableTags());

		SUPPORTED_TAGS = StringUtils.toCommaSeparatedString(tagList);
	}

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Error email subject", description = "The subject of emails sent when requests fail")
	@XSLVariable(prefix = "java.")
	private String errorEmailSubject = "Not set";

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Error email message", description = "The message of emails sent when errors are received", required = true)
	@XSLVariable(prefix = "java.")
	private String errorEmailMessage = "Not set";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Disable certificate validation", description = "Controls whether certificate should be validated when using SSL or not")
	private boolean disableCertificateValidation = false;

	@InstanceManagerDependency(required = true)
	private PDFProvider pdfProvider;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, this.getClass().getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		settingsDAO = daoFactory.getDAO(IntegrationSettings.class);
		endpointDAO = daoFactory.getDAO(IntegrationEndpoint.class);
		queuedRequestDAO = daoFactory.getDAO(QueuedIntegrationRequest.class);

		flowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
		endpointIDParamFactory = endpointDAO.getParamFactory("endpointID", Integer.class);
		lastRetryParamFactory = queuedRequestDAO.getParamFactory("lastRetry", Timestamp.class);
		eventIDParamFactory = queuedRequestDAO.getParamFactory("eventID", Integer.class);
		flowInstanceIDParamFactory = queuedRequestDAO.getParamFactory("flowInstanceID", Integer.class);
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 49)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws Exception {

		FlowInstance flowInstance = (FlowInstance) event.getFlowInstanceManager().getFlowInstance();
		IntegrationSettings settings = getIntegrationSettings(flowInstance.getFlow().getFlowFamily().getFlowFamilyID());

		if (settings == null || !settings.isEnabled()) {

			return;
		}

		if (!shouldProcessEvent(event)) {

			return;
		}

		FlowInstanceEvent flowInstanceEvent = event.getEvent();

		try {

			processIntegrationEvent(flowInstance, flowInstanceEvent);

		} catch (IntegrationRequestException e) {

			if (maxRetryCount > 0) {

				log.info("Adding request to retry queue");

				addQueuedEvent(flowInstanceEvent, flowInstance);

			} else if (settings.isSendErrorEmail()) {

				log.info("Sending error emails");

				sendEmails(settings.getErrorEmailAddresses(), errorEmailSubject, errorEmailMessage, flowInstance, getTagReplacer(flowInstance));
			}

		} catch (IntegrationSettingsException e) {

			log.error("Settings error while processing event " + event, e);
		}
	}

	@Override
	public void processIntegrationEvent(FlowInstance flowInstance, FlowInstanceEvent event) throws Exception {

		IntegrationSettings settings = getIntegrationSettings(flowInstance.getFlow());

		if (settings == null) {

			throw new IntegrationSettingsException("Settings are null");
		}

		if (!settings.isEnabled()) {

			throw new IntegrationSettingsException("Settings are disabled");
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(StandardFlowNotificationHandler.FLOWINSTANCE_TAG_SOURCE_FACTORY.getTagSource(flowInstance));
		tagReplacer.addTagSource(StandardFlowNotificationHandler.FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
		tagReplacer.addTagSource(StandardFlowNotificationHandler.STATUS_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getStatus()));

		Contact posterContact = FlowInstanceUtils.getPosterContact(flowInstance);

		if (posterContact != null) {

			tagReplacer.addTagSource(StandardFlowNotificationHandler.CONTACT_TAG_SOURCE_FACTORY.getTagSource(posterContact));
		}

		File pdfFile = pdfProvider.getPDF(flowInstance.getFlowInstanceID(), event.getEventID());

		if (pdfFile == null) {

			log.error("No PDF found for event " + event + " belonging to flow instance " + flowInstance);
			return;
		}

		MutableAttributeHandler attributeHandler = flowInstance.getAttributeHandler();

		try {
			//Case
			CaseDto caseDto = new CaseDto();
			caseDto.setRefID(flowInstance.getFlowInstanceID().toString());
			caseDto.setInboxGuid(replaceTags(settings.getCaseInboxGuid(), attributeHandler, tagReplacer));
			caseDto.setIncomingMediaTypeGuid(replaceTags(settings.getCaseIncomingMediaTypeGuid(), attributeHandler, tagReplacer));
			caseDto.setName(flowInstance.getFlow().getName());
			caseDto.setDescription(replaceTags(settings.getCaseDescription(), attributeHandler, tagReplacer));
			caseDto.setRequiresAnswer(settings.isRequiresAnswer());

			//HTML description
			DeedDto htmlDeedDto = new DeedDto();
			htmlDeedDto.setInboxGuid(caseDto.getInboxGuid());
			htmlDeedDto.setObjectTypeGuid(replaceTags(settings.getCaseHTMLDescriptionObjectTypeGuid(), attributeHandler, tagReplacer));
			htmlDeedDto.setName(replaceTags(settings.getCaseHTMLDescriptionName(), attributeHandler, tagReplacer));
			htmlDeedDto.setHtmlContent(replaceTags(settings.getCaseHTMLDescriptionContent(), attributeHandler, tagReplacer));

			ArrayOfDeedDto arrayOfDeedDto = new ArrayOfDeedDto();
			arrayOfDeedDto.getDeedDto().add(htmlDeedDto);
			caseDto.setDeeds(arrayOfDeedDto);

			//Files Dto
			ArrayOfFileDto arrayOfFileDto = new ArrayOfFileDto();
			htmlDeedDto.setFiles(arrayOfFileDto);

			//Files
			FileDto fileDto = new FileDto();
			fileDto.setFileName(flowInstance.getFlow() + ", " + flowInstance.getFlowInstanceID() + ".pdf");
			fileDto.setExtension(".pdf");
			fileDto.setSize((int) pdfFile.length());
			fileDto.setData(FileUtils.getRawBytes(pdfFile));
			arrayOfFileDto.getFileDto().add(fileDto);

			if (settings.isAddAttachmentsSeparately()) {

				List<PDFByteAttachment> pdfAttachments = pdfProvider.getPDFAttachments(pdfFile, true);

				if (pdfAttachments != null) {

					for (PDFByteAttachment pdfAttachment : pdfAttachments) {

						FileDto attechmentDto = new FileDto();
						attechmentDto.setFileName(pdfAttachment.getFilename());
						attechmentDto.setExtension(FileUtils.getFileExtension(pdfAttachment.getFilename()));
						attechmentDto.setSize(pdfAttachment.getData().length);
						attechmentDto.setData(pdfAttachment.getData());
						arrayOfFileDto.getFileDto().add(attechmentDto);
					}
				}
			}

			//Contact
			CounterpartDto counterpartDto = new CounterpartDto();
			counterpartDto.setSocialSecurityNumber(replaceTags(settings.getContactSocialSecurityNumber(), attributeHandler, tagReplacer));
			counterpartDto.setFirstName(replaceTags(settings.getContactFirstName(), attributeHandler, tagReplacer));
			counterpartDto.setLastName(replaceTags(settings.getContactLastName(), attributeHandler, tagReplacer));
			counterpartDto.setAddress1(replaceTags(settings.getContactAddress(), attributeHandler, tagReplacer));
			counterpartDto.setCity(replaceTags(settings.getContactPostalAddress(), attributeHandler, tagReplacer));
			counterpartDto.setZip(replaceTags(settings.getContactZIPCode(), attributeHandler, tagReplacer));
			counterpartDto.setEMail(replaceTags(settings.getContactEmail(), attributeHandler, tagReplacer));
			counterpartDto.setMobilePhone(replaceTags(settings.getContactMobilePhone(), attributeHandler, tagReplacer));
			counterpartDto.setHomePhone(replaceTags(settings.getContactHomePhone(), attributeHandler, tagReplacer));
			counterpartDto.setCustomStringValue1(replaceTags(settings.getCustomStringValue1(), attributeHandler, tagReplacer));
			
			ArrayOfCounterpartDto arrayOfCounterpartDto = new ArrayOfCounterpartDto();
			arrayOfCounterpartDto.getCounterpartDto().add(counterpartDto);

			caseDto.setCounterparts(arrayOfCounterpartDto);

			IService service = new Service().getBasicHttpBindingIService();

			WSUtils.setEndpoint((BindingProvider) service, settings.getEndpoint().getAddress());

			if (disableCertificateValidation) {

				SSLContext sslContext = SSLContext.getInstance("SSL");
				sslContext.init(null, SSLUtils.INSECURE_TRUST_MANGERS, new java.security.SecureRandom());
				WSUtils.setSSLSocketFactory((BindingProvider) service, sslContext.getSocketFactory());
			}

			CreateObjectResult response = service.createCase(settings.getApiKey(), caseDto);

			if (!response.isOk()) {

				throw new RuntimeException("Got bad response, message: " + response.getMessage());
			}

			log.info("Successfully created Artvise case with ID " + response.getObjectInformation().getIdentifier() + " for event " + event + " from flow instance " + flowInstance);

			FlowInstanceAttribute attribute = new FlowInstanceAttribute(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE, response.getObjectInformation().getIdentifier());
			attribute.setFlowInstance(flowInstance);

			flowAdminModule.getDAOFactory().getFlowInstanceAttributeDAO().add(attribute);

			attributeHandler.setAttribute(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE, response.getObjectInformation().getIdentifier());

			if (settings.getCaseCategoryGuid() != null && settings.getDiaryPlanItemGuid() != null) {

				String caseCategoryGuid = replaceTags(settings.getCaseCategoryGuid(), attributeHandler, tagReplacer);
				String diaryPlanItemGuid = replaceTags(settings.getDiaryPlanItemGuid(), attributeHandler, tagReplacer);

				if (caseCategoryGuid != null && diaryPlanItemGuid != null) {

					try {
						CaseCategoryItemDto categoryItemDto = new CaseCategoryItemDto();
						categoryItemDto.setCaseCategoryGuid(caseCategoryGuid);
						categoryItemDto.setFullName("Not set");
						categoryItemDto.setDiaryPlanItemGuid(diaryPlanItemGuid);

						ArrayOfCaseCategoryItemDto arrayOfCaseCategoryItemDto = new ArrayOfCaseCategoryItemDto();
						arrayOfCaseCategoryItemDto.getCaseCategoryItemDto().add(categoryItemDto);

						OperationResult result = service.setCaseCategory(settings.getApiKey(), response.getObjectInformation().getObjectID(), categoryItemDto);

						if (result.isOk()) {

							log.info("Successfully set case category in Artvise for flow instance " + flowInstance);

						} else {

							log.error("Unable to set case category in Artvise for flow instance " + flowInstance + ", got message: " + result.getMessage());
						}

					} catch (Exception e) {

						log.error("Error setting case category in Artvise for flow instance " + flowInstance, e);
					}
				}
			}

		} catch (Exception e) {

			boolean shouldLogAsError = maxRetryCount == 0 && !settings.isSendErrorEmail();

			logResponse("Error creating Artvise case for event " + event + " from flow instance " + flowInstance + " to endpoint " + settings.getEndpoint(), e, shouldLogAsError);

			throw new IntegrationRequestException();
		}
	}

	@WebPublic(alias = "update")
	public ForegroundModuleResponse updateFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
		}

		if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		ValidationException validationException = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			try {

				updateIntegrationSettings(integrationSettings, flow, user, req);

				flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + getShowFlowModuleAnchorLink());

			} catch (ValidationException e) {

				validationException = e;
			}
		}

		log.info("User " + user + " requesting update integration settings form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element integrationSettingsElement = doc.createElement("UpdateIntegrationSettings");
		doc.getDocumentElement().appendChild(integrationSettingsElement);

		integrationSettingsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, integrationSettingsElement, integrationSettings);
		XMLUtils.append(doc, integrationSettingsElement, "Endpoints", endpointDAO.getAll());

		XMLUtils.appendNewElement(doc, integrationSettingsElement, "SupportedTags", SUPPORTED_TAGS);

		if (validationException != null) {

			integrationSettingsElement.appendChild(validationException.toXML(doc));
			integrationSettingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	private void updateIntegrationSettings(IntegrationSettings integrationSettings, Flow flow, User user, HttpServletRequest req) throws SQLException, ValidationException {

		if (integrationSettings == null) {

			integrationSettings = new IntegrationSettings();
			integrationSettings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		POPULATOR.populate(integrationSettings, req);

		IntegrationEndpoint endpoint = getEndpoint(req);

		if (endpoint == null) {

			validationErrors.add(new ValidationError("NoEndpointSelected"));

		} else {

			integrationSettings.setEndpoint(endpoint);
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		log.info("User " + user + " updating integration settings for flow " + flow);

		if (integrationSettings.isPopulated()) {

			settingsDAO.addOrUpdate(integrationSettings, null);

		} else {

			settingsDAO.delete(integrationSettings);
		}
	}

	@Override
	protected QueuedIntegrationRequest getNewQueuedIntegrationRequest() {

		return new QueuedIntegrationRequest();
	}

	@Override
	protected void handleMaxRetryCountExceeded(ImmutableFlowInstance flowInstance, FlowInstanceEvent event, QueuedIntegrationRequest queuedRequest, TransactionHandler transactionHandler, IntegrationRequestException lastRequestException) throws SQLException {

		super.handleMaxRetryCountExceeded(flowInstance, event, queuedRequest, transactionHandler, lastRequestException);

		IntegrationSettings settings;

		if (flowInstance != null && (settings = getIntegrationSettings(flowInstance.getFlow())) != null) {

			if (settings.isSendErrorEmail()) {

				log.info("Sending error emails due to max retries exceeded");

				sendEmails(settings.getErrorEmailAddresses(), errorEmailSubject, errorEmailMessage, flowInstance, getTagReplacer(flowInstance));

			} else {

				log.error(queuedRequest + " hit its max retry count and error email is disabled");
			}
		}
	}

	@Override
	protected String getShowFlowModuleAnchorLink() {

		return "#artviseintegrationsettings";
	}
}
