<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
		/js/confirmpost.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/artviseintegration.js
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/CKEditor.xsl"/>

	<xsl:template match="Document">
	
		<xsl:apply-templates select="ShowIntegrationSettings" />
		<xsl:apply-templates select="UpdateIntegrationSettings" />
			
	</xsl:template>
	
	<xsl:template match="ShowIntegrationSettings">
	
		<a name="artviseintegrationsettings" />
		
		<xsl:choose>
			<xsl:when test="IntegrationSettings">
					
				<xsl:value-of select="$i18n.IntegrationEnabled"/>
				<xsl:text>:&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="IntegrationSettings/enabled = 'true'">
						<xsl:value-of select="$i18n.Yes"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No"/>
					</xsl:otherwise>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoIntegrationSettings" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/update/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateIntegrationSettings" />
			<img class="alignbottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="IntegrationSettings">
			<a href="{/Document/ModuleURI}/delete/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteIntegrationSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteIntegrationSettings" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdateIntegrationSettings">
		
		<div id="ArtviseIntegrationProvider" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdateIntegrationSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError"/>
	
			<form method="post" action="{/Document/ModuleURI}/update/{Flow/flowID}">
			
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'enabled'" />
						<xsl:with-param name="id" select="'enabled'" />
						<xsl:with-param name="element" select="IntegrationSettings" />       
					</xsl:call-template>
					
					<label for="enabled">
						<xsl:value-of select="$i18n.Enabled" />
					</label>
				</div>
			
				<label for="endpointID" class="floatleft full">
					<xsl:value-of select="$i18n.Endpoint" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'endpointID'"/>
						<xsl:with-param name="name" select="'endpointID'"/>
						<xsl:with-param name="element" select="Endpoints/Endpoint" /> 
						<xsl:with-param name="labelElementName" select="'name'" />        
						<xsl:with-param name="valueElementName" select="'endpointID'" />
						<xsl:with-param name="selectedValue" select="IntegrationSettings/Endpoint/endpointID"/>
					</xsl:call-template>
				</div>
				
				<xsl:call-template name="createTextFieldSetting">
					<xsl:with-param name="name" select="'apiKey'" />
					<xsl:with-param name="label" select="$i18n.apiKey"/>
				</xsl:call-template>
									
				<div class="clearboth marginbottom">
					<br/>
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.SupportedTags.Title"/></h2>
					
					<p><xsl:value-of select="$i18n.SupportedTags.Description"/></p>
					
					<p><xsl:value-of select="SupportedTags"/></p>
				
				</div>				
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.CaseData.Title"/></h2>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'caseInboxGuid'" />
						<xsl:with-param name="label" select="$i18n.caseInboxGuid"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'caseIncomingMediaTypeGuid'" />
						<xsl:with-param name="label" select="$i18n.caseIncomingMediaTypeGuid"/>
					</xsl:call-template>									
					
					<xsl:call-template name="createTextAreaSetting">
						<xsl:with-param name="name" select="'caseDescription'" />
						<xsl:with-param name="label" select="$i18n.caseDescription"/>
					</xsl:call-template>
					
					<div class="floatleft full bigmargintop bigmarginbottom">
			
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="name" select="'requiresAnswer'" />
							<xsl:with-param name="id" select="'requiresAnswer'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
						
						<label class="marginright" for="requiresAnswer">
							<xsl:value-of select="$i18n.requiresAnswer" />
						</label>
						
					</div>				
					
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.CaseCategoryAndDiaryPlan.Title"/></h2>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'caseCategoryGuid'" />
						<xsl:with-param name="label" select="$i18n.caseCategoryGuid"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'diaryPlanItemGuid'" />
						<xsl:with-param name="label" select="$i18n.diaryPlanItemGuid"/>
					</xsl:call-template>									
					
				</div>				
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.CaseHTMLData.Title"/></h2>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'caseHTMLDescriptionObjectTypeGuid'" />
						<xsl:with-param name="label" select="$i18n.caseHTMLDescriptionObjectTypeGuid"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'caseHTMLDescriptionName'" />
						<xsl:with-param name="label" select="$i18n.caseHTMLDescriptionName"/>
					</xsl:call-template>
					
					<xsl:call-template name="createHTMLEditorSetting">
						<xsl:with-param name="name" select="'caseHTMLDescriptionContent'" />
						<xsl:with-param name="label" select="$i18n.caseHTMLDescriptionContent"/>
					</xsl:call-template>
					
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.Contact.Title"/></h2>					
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactSocialSecurityNumber'" />
						<xsl:with-param name="label" select="$i18n.contactSocialSecurityNumber"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactFirstName'" />
						<xsl:with-param name="label" select="$i18n.contactFirstName"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactLastName'" />
						<xsl:with-param name="label" select="$i18n.contactLastName"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactAddress'" />
						<xsl:with-param name="label" select="$i18n.contactAddress"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactZIPCode'" />
						<xsl:with-param name="label" select="$i18n.contactZIPCode"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactPostalAddress'" />
						<xsl:with-param name="label" select="$i18n.contactPostalAddress"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactEmail'" />
						<xsl:with-param name="label" select="$i18n.contactEmail"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactHomePhone'" />
						<xsl:with-param name="label" select="$i18n.contactHomePhone"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'contactMobilePhone'" />
						<xsl:with-param name="label" select="$i18n.contactMobilePhone"/>
					</xsl:call-template>
				
					<xsl:call-template name="createTextAreaSetting">
						<xsl:with-param name="name" select="'customStringValue1'" />
						<xsl:with-param name="label" select="$i18n.customStringValue1"/>
					</xsl:call-template>				
				
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
		
					<h2><xsl:value-of select="$i18n.Attachments"/></h2>
				
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'addAttachmentsSeparately'" />
						<xsl:with-param name="id" select="'addAttachmentsSeparately'" />
						<xsl:with-param name="element" select="IntegrationSettings" />
					</xsl:call-template>
					
					<label class="marginright" for="addAttachmentsSeparately">
						<xsl:value-of select="$i18n.addAttachmentsSeparately" />
					</label>
					
				</div>
								
				<div class="floatleft full bigmargintop bigmarginbottom">
		
					<h2><xsl:value-of select="$i18n.Notifications"/></h2>
				
					<xsl:call-template name="notificationEmail">
						<xsl:with-param name="toggleField" select="'sendErrorEmail'" />
						<xsl:with-param name="addressesField" select="'errorEmailAddresses'" />
						<xsl:with-param name="toggleLabel" select="$i18n.Notifications.ErrorEmail" />
						<xsl:with-param name="addressesLabel" select="$i18n.Notifications.ErrorEmail.EmailAddresses" />
						<xsl:with-param name="addresses" select="IntegrationSettings/ErrorEmailAddresses/Address" />
					</xsl:call-template>
					
				</div>
			
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
		
			<xsl:call-template name="initializeFCKEditor">
				<xsl:with-param name="basePath"><xsl:value-of select="/Document/StaticContentURL"/>/ckeditor/</xsl:with-param>
				<xsl:with-param name="customConfig">config.js</xsl:with-param>
				<xsl:with-param name="editorContainerClass">fckeditor</xsl:with-param>
				<xsl:with-param name="editorHeight">150</xsl:with-param>
			</xsl:call-template>		
		
		</div>
		
	</xsl:template>
	
	<xsl:template name="createTextFieldSetting">
	
		<xsl:param name="name" />
		<xsl:param name="label" />
		
		<div class="floatleft full">
		
			<label for="{$name}">
				<xsl:value-of select="$label" />
			</label>
			
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="$name"/>
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="element" select="IntegrationSettings" />          
			</xsl:call-template>
		</div>
	
	</xsl:template>
		
	<xsl:template name="createTextAreaSetting">
		
		<xsl:param name="name" />
		<xsl:param name="label" />
		
		<div class="floatleft full">
		
			<label for="{$name}">
				<xsl:value-of select="$label" />
			</label>
			
			<xsl:call-template name="createTextArea">
				<xsl:with-param name="id" select="$name"/>
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="element" select="IntegrationSettings" />
			</xsl:call-template>
		</div>
	
	</xsl:template>	
	
	<xsl:template name="createHTMLEditorSetting">
		
		<xsl:param name="name" />
		<xsl:param name="label" />
		
		<div class="floatleft full">
		
			<label for="{$name}">
				<xsl:value-of select="$label" />
			</label>
			
			<xsl:call-template name="createTextArea">
				<xsl:with-param name="id" select="$name"/>
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="class" select="'fckeditor'" />
				<xsl:with-param name="element" select="IntegrationSettings" />          
			</xsl:call-template>
		</div>
	
	</xsl:template>
		
	<xsl:template name="notificationEmail">
		<xsl:param name="toggleField" />
		<xsl:param name="addressesField" />
		<xsl:param name="toggleLabel" />
		<xsl:param name="addressesLabel" />
		<xsl:param name="addresses" />
	
		<div class="notification-email">
		
			<div>
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="$toggleField" />
					<xsl:with-param name="id" select="$toggleField" />
					<xsl:with-param name="element" select="IntegrationSettings" />
					<xsl:with-param name="class" select="'notification-email-toggle'" />
				</xsl:call-template>
				
				<label class="marginright" for="{$toggleField}">
					<xsl:value-of select="$toggleLabel" />
				</label>
			</div>
			
			<div class="notification-email-text">
			
				<xsl:if test="not(validationException/validationError/fieldName = $addressesField)">
					<xsl:attribute name="class">notification-email-text hidden</xsl:attribute>
				</xsl:if>
			
				<div>
				
					<label for="{$addressesField}">
						<xsl:value-of select="$addressesLabel" />
					</label>
					
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="$addressesField"/>
						<xsl:with-param name="name" select="$addressesField"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="$addresses" />
					</xsl:call-template>
				</div>
			</div>
		</div>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoEndpointSelected']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.NoEndpointSelected"/>
		</p>
	
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endpointID'">
						<xsl:value-of select="$i18n.Endpoint"/>
					</xsl:when>
					<xsl:when test="fieldName = 'apiKey'">
						<xsl:value-of select="$i18n.apiKey"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseInboxGuid'">
						<xsl:value-of select="$i18n.caseInboxGuid"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseIncomingMediaTypeGuid'">
						<xsl:value-of select="$i18n.caseIncomingMediaTypeGuid"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseDescription'">
						<xsl:value-of select="$i18n.caseDescription"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseHTMLDescriptionObjectTypeGuid'">
						<xsl:value-of select="$i18n.caseHTMLDescriptionObjectTypeGuid"/>
					</xsl:when>					
					<xsl:when test="fieldName = 'caseHTMLDescriptionName'">
						<xsl:value-of select="$i18n.caseHTMLDescriptionName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseHTMLDescriptionContent'">
						<xsl:value-of select="$i18n.caseHTMLDescriptionContent"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseCategoryGuid'">
						<xsl:value-of select="$i18n.caseCategoryGuid"/>
					</xsl:when>
					<xsl:when test="fieldName = 'diaryPlanItemGuid'">
						<xsl:value-of select="$i18n.diaryPlanItemGuid"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactSocialSecurityNumber'">
						<xsl:value-of select="$i18n.contactSocialSecurityNumber"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactFirstName'">
						<xsl:value-of select="$i18n.contactFirstName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactLastName'">
						<xsl:value-of select="$i18n.contactLastName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactAddress'">
						<xsl:value-of select="$i18n.contactAddress"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactZIPCode'">
						<xsl:value-of select="$i18n.contactZIPCode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactPostalAddress'">
						<xsl:value-of select="$i18n.contactPostalAddress"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactEmail'">
						<xsl:value-of select="$i18n.contactEmail"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactHomePhone'">
						<xsl:value-of select="$i18n.contactHomePhone"/>
					</xsl:when>
					<xsl:when test="fieldName = 'contactMobilePhone'">
						<xsl:value-of select="$i18n.contactMobilePhone"/>
					</xsl:when>
					<xsl:when test="fieldName = 'errorEmailAddresses'">
						<xsl:value-of select="$i18n.Notifications.ErrorEmail.EmailAddresses"/>
					</xsl:when>													
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>