package com.nordicpeak.flowengine.integration.artvise.beans;

import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;

@Table(name = "artvise_integration_settings")
@XMLElement
public class IntegrationSettings extends BaseIntegrationSettings {

	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private IntegrationEndpoint endpoint;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String apiKey;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String caseInboxGuid;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String caseIncomingMediaTypeGuid;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String caseDescription;
	
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String customStringValue1;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean requiresAnswer;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String caseHTMLDescriptionObjectTypeGuid;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String caseHTMLDescriptionName;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String caseHTMLDescriptionContent;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String caseCategoryGuid;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String diaryPlanItemGuid;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactSocialSecurityNumber;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactFirstName;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactLastName;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactAddress;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactZIPCode;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactPostalAddress;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactEmail;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactHomePhone;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String contactMobilePhone;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean addAttachmentsSeparately;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "artvise_integration_email_recipients_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "Address")
	private List<String> errorEmailAddresses;

	@Override
	public boolean isPopulated() {

		if (isEnabled() || !ObjectUtils.isNull(endpoint, apiKey, caseInboxGuid, caseDescription, caseHTMLDescriptionObjectTypeGuid, caseHTMLDescriptionName, caseHTMLDescriptionContent, contactSocialSecurityNumber, contactFirstName, contactLastName, contactAddress, contactZIPCode, contactPostalAddress, contactEmail, contactHomePhone, contactMobilePhone, addAttachmentsSeparately, sendErrorEmail, errorEmailAddresses)) {

			return true;
		}

		return false;
	}

	public IntegrationEndpoint getEndpoint() {

		return endpoint;
	}

	public void setEndpoint(IntegrationEndpoint endpoint) {

		this.endpoint = endpoint;
	}

	public String getApiKey() {

		return apiKey;
	}

	public void setApiKey(String apiKey) {

		this.apiKey = apiKey;
	}

	public String getCaseInboxGuid() {

		return caseInboxGuid;
	}

	public void setCaseInboxGuid(String caseInboxGuid) {

		this.caseInboxGuid = caseInboxGuid;
	}

	public String getCaseDescription() {

		return caseDescription;
	}

	public void setCaseDescription(String caseDescription) {

		this.caseDescription = caseDescription;
	}

	public String getCaseHTMLDescriptionObjectTypeGuid() {

		return caseHTMLDescriptionObjectTypeGuid;
	}

	public void setCaseHTMLDescriptionObjectTypeGuid(String caseHTMLDescriptionObjectTypeGuid) {

		this.caseHTMLDescriptionObjectTypeGuid = caseHTMLDescriptionObjectTypeGuid;
	}

	public String getCaseHTMLDescriptionName() {

		return caseHTMLDescriptionName;
	}

	public void setCaseHTMLDescriptionName(String caseHTMLDescriptionName) {

		this.caseHTMLDescriptionName = caseHTMLDescriptionName;
	}

	public String getCaseHTMLDescriptionContent() {

		return caseHTMLDescriptionContent;
	}

	public void setCaseHTMLDescriptionContent(String caseHTMLDescription) {

		this.caseHTMLDescriptionContent = caseHTMLDescription;
	}

	public String getContactSocialSecurityNumber() {

		return contactSocialSecurityNumber;
	}

	public void setContactSocialSecurityNumber(String caseSocialSecurityNumber) {

		this.contactSocialSecurityNumber = caseSocialSecurityNumber;
	}

	public String getContactFirstName() {

		return contactFirstName;
	}

	public void setContactFirstName(String contactFirstName) {

		this.contactFirstName = contactFirstName;
	}

	public String getContactLastName() {

		return contactLastName;
	}

	public void setContactLastName(String contactLasttName) {

		this.contactLastName = contactLasttName;
	}

	public String getContactAddress() {

		return contactAddress;
	}

	public void setContactAddress(String contactAddress) {

		this.contactAddress = contactAddress;
	}

	public String getContactZIPCode() {

		return contactZIPCode;
	}

	public void setContactZIPCode(String contactZIPCode) {

		this.contactZIPCode = contactZIPCode;
	}

	public String getContactPostalAddress() {

		return contactPostalAddress;
	}

	public void setContactPostalAddress(String contactPostalAddress) {

		this.contactPostalAddress = contactPostalAddress;
	}

	public String getContactEmail() {

		return contactEmail;
	}

	public void setContactEmail(String contactEmail) {

		this.contactEmail = contactEmail;
	}

	public String getContactHomePhone() {

		return contactHomePhone;
	}

	public void setContactHomePhone(String contactHomePhone) {

		this.contactHomePhone = contactHomePhone;
	}

	public String getContactMobilePhone() {

		return contactMobilePhone;
	}

	public void setContactMobilePhone(String contactMobilePhone) {

		this.contactMobilePhone = contactMobilePhone;
	}

	public boolean isAddAttachmentsSeparately() {

		return addAttachmentsSeparately;
	}

	public void setAddAttachmentsSeparately(boolean addAttachmentsSeparately) {

		this.addAttachmentsSeparately = addAttachmentsSeparately;
	}

	public boolean isSendErrorEmail() {

		return sendErrorEmail;
	}

	public void setSendErrorEmail(boolean sendErrorEmail) {

		this.sendErrorEmail = sendErrorEmail;
	}

	public List<String> getErrorEmailAddresses() {

		return errorEmailAddresses;
	}

	public void setErrorEmailAddresses(List<String> errorEmailAddresses) {

		this.errorEmailAddresses = errorEmailAddresses;
	}

	public String getCaseIncomingMediaTypeGuid() {

		return caseIncomingMediaTypeGuid;
	}

	public void setCaseIncomingMediaTypeGuid(String caseIncomingMediaTypeGuid) {

		this.caseIncomingMediaTypeGuid = caseIncomingMediaTypeGuid;
	}

	public boolean isRequiresAnswer() {

		return requiresAnswer;
	}

	public void setRequiresAnswer(boolean requiresAnswer) {

		this.requiresAnswer = requiresAnswer;
	}

	
	public String getCaseCategoryGuid() {
	
		return caseCategoryGuid;
	}

	
	public void setCaseCategoryGuid(String caseCategoryGuid) {
	
		this.caseCategoryGuid = caseCategoryGuid;
	}

	
	public String getDiaryPlanItemGuid() {
	
		return diaryPlanItemGuid;
	}

	
	public void setDiaryPlanItemGuid(String diaryPlanItemGuid) {
	
		this.diaryPlanItemGuid = diaryPlanItemGuid;
	}

	
	public String getCustomStringValue1() {
	
		return customStringValue1;
	}

	
	public void setCustomStringValue1(String customStringValue1) {
	
		this.customStringValue1 = customStringValue1;
	}

}
