package com.nordicpeak.flowengine.queries.attributesumlimitquery;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.AttributeSumLimitEndpointAdminModule;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.beans.AttributeSumLimitEndpoint;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class AttributeSumLimitQueryCRUD extends BaseQueryCRUD<AttributeSumLimitQuery, AttributeSumLimitQueryProviderModule> {

	protected AnnotatedDAOWrapper<AttributeSumLimitQuery, Integer> queryDAO;

	public AttributeSumLimitQueryCRUD(AnnotatedDAOWrapper<AttributeSumLimitQuery, Integer> queryDAO, BeanRequestPopulator<AttributeSumLimitQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, AttributeSumLimitQueryProviderModule callback) {

		super(AttributeSumLimitQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.queryDAO = queryDAO;
	}
	
	@Override
	protected AttributeSumLimitQuery populateFromUpdateRequest(AttributeSumLimitQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		AttributeSumLimitQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		if(query.isUseEndpoint()) {
			
			Integer endpointID = ValidationUtils.validateParameter("endpointID", req, true, IntegerPopulator.getPopulator(), validationErrors);
			
			if(endpointID != null) {
				
				AttributeSumLimitEndpoint endpoint = callback.getEndpoint(endpointID);
				
				if(endpoint == null) {

					validationErrors.add(new ValidationError("endpointID", ValidationErrorType.RequiredField));
					
				}else {
					
					query.setEndpoint(endpoint);
					query.setMinDate(null);
					query.setMaxDate(null);
					query.setLimit(null);
				}
			}
		
		}else {
		 
			query.setEndpoint(null);
			query.setApiErrorDescription(null);
		}
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

	@Override
	protected void appendUpdateFormData(AttributeSumLimitQuery bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		AttributeSumLimitEndpointAdminModule apiModule = callback.getApiModule();
		
		if(apiModule != null) {
			
			XMLUtils.append(doc, updateTypeElement,"Endpoints", apiModule.getEndpoints());
		}
	}

}
