package com.nordicpeak.flowengine.queries.attributesumlimitquery;

import java.lang.reflect.Field;
import java.sql.SQLException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.SubmitCheck;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.interfaces.LimitProvider;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@XMLElement
public class AttributeSumLimitQueryInstance extends BaseQueryInstance implements SubmitCheck {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(AttributeSumLimitQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private AttributeSumLimitQuery query;

	@XMLElement
	private Integer remainingAmount;

	private LimitProvider limitProvider;

	private ValidationError submitValidationError;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public AttributeSumLimitQuery getQuery() {

		return query;
	}

	public void setQuery(AttributeSumLimitQuery query) {

		this.query = query;
	}

	public ValidationError getSubmitValidationError() {

		return submitValidationError;
	}

	public void setSubmitValidationError(ValidationError submitError) {

		this.submitValidationError = submitError;
	}

	public Integer getRemainingAmount() {

		return remainingAmount;
	}

	public void setRemainingAmount(Integer remainingAmount) {

		this.remainingAmount = remainingAmount;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.limitProvider = null;
		this.remainingAmount = null;
		this.submitValidationError = null;
		super.reset(attributeHandler);
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		return null;
	}

	@Override
	public boolean isValidForSubmit(MutableAttributeHandler attributeHandler, User poster, QueryHandler queryHandler, FlowInstanceManager flowInstanceManager) throws SQLException {

		return queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), AttributeSumLimitQueryProviderModule.class).isValidForSubmit(attributeHandler, this, poster);
	}

	public LimitProvider getLimitProvider() {

		return limitProvider;
	}

	public void setLimitProvider(LimitProvider limitProvider) {

		this.limitProvider = limitProvider;
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);
		
		XMLUtils.appendNewElement(doc, element, "HasLimitProvider", limitProvider != null);
		
		return element;
	}
}
