package com.nordicpeak.flowengine.queries.attributesumlimitquery.api;

import java.io.IOException;
import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.menu.MenuItemDescriptor;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.userproviders.SimpleUser;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.string.URLEncodingTagReplacer;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPResponse;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.interfaces.APISource;
import com.nordicpeak.flowengine.interfaces.APISourceHandler;
import com.nordicpeak.flowengine.interfaces.APISourceProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.AttributeSumLimitQuery;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.AttributeSumLimitQueryProviderModule;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.beans.AttributeSumLimitEndpoint;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.beans.AttributeSumLimiter;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.interfaces.LimitProvider;
import com.nordicpeak.flowengine.utils.UserAttributeTagUtils;

public class AttributeSumLimitEndpointAdminModule extends AnnotatedForegroundModule implements CRUDCallback<User>, APISourceProvider {

	public static final AnnotatedBeanTagSourceFactory<User> USER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(User.class, "$user.");

	@XSLVariable(prefix = "java.")
	private String apiSourceShortTypeName;
	
	@XSLVariable(prefix = "java.")
	private String apiSourceFullTypeName;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Allowed encodings", description = "Encodings allowed to use")
	private List<String> allowedEncodings = new ArrayList<>(Arrays.asList("ISO-8859-1", "UTF-8"));

	@InstanceManagerDependency
	private QueryHandler queryHandler;

	private AnnotatedDAO<AttributeSumLimitEndpoint> endpointDAO;
	private AnnotatedDAO<AttributeSumLimitQuery> queryDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;

	private QueryParameterFactory<AttributeSumLimitQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<AttributeSumLimitEndpoint, Integer> endpointIDParamFactory;

	private AttributeSumLimitEndpointCRUD endpointCRUD;

	private APISourceHandler apiSourceHandler;
	
	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(AttributeSumLimitEndpointAdminModule.class, this)) {

			throw new RuntimeException("Unable to register module " + moduleDescriptor + " in global instance handler using key " + AttributeSumLimitEndpointAdminModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AttributeSumLimitQueryProviderModule.class.getName(), new XMLDBScriptProvider(AttributeSumLimitQueryProviderModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(AttributeSumLimitQuery.class);
		endpointDAO = daoFactory.getDAO(AttributeSumLimitEndpoint.class);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		endpointIDParamFactory = endpointDAO.getParamFactory("endpointID", Integer.class);

		AdvancedAnnotatedDAOWrapper<AttributeSumLimitEndpoint, Integer> endpointDAOWrapper = endpointDAO.getAdvancedWrapper(Integer.class);
		
		endpointDAOWrapper.getGetAllQuery().addRelations(AttributeSumLimitEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetAllQuery().disableAutoRelations(true);
		
		endpointDAOWrapper.getGetQuery().addRelations(AttributeSumLimitEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetQuery().setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
		endpointDAOWrapper.getGetQuery().disableAutoRelations(true);

		endpointCRUD = new AttributeSumLimitEndpointCRUD(endpointDAOWrapper, this);

		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(AttributeSumLimitEndpointAdminModule.class, this);

		if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		super.unload();
	}

	@InstanceManagerDependency
	public void setAPISourceHandler(APISourceHandler apiSourceHandler) throws SQLException {
		
		if (apiSourceHandler != null) {
			
			apiSourceHandler.addAPISourceProvider(this);
			
		} else if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		this.apiSourceHandler = apiSourceHandler;
		
		//Check if this module is fully cached, then trigger an update of the menu items
		if(sectionInterface.getForegroundModuleCache().isCached(moduleDescriptor)) {
			
			this.sectionInterface.getMenuCache().moduleUpdated(moduleDescriptor, this);
		}
	}
	
	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.list(req, res, user, uriParser, null);
	}

	@WebPublic
	public ForegroundModuleResponse show(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.show(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse add(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.add(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse update(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.update(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse delete(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.delete(req, res, user, uriParser);
	}

	public List<AttributeSumLimitQuery> getQueries(AttributeSumLimitEndpoint endpoint) {

		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE endpointID = ?", IntegerPopulator.getPopulator());
			idQuery.setInt(1, endpoint.getEndpointID());

			List<Integer> queryIDs = idQuery.executeQuery();

			if (!CollectionUtils.isEmpty(queryIDs)) {

				List<AttributeSumLimitQuery> queries = new ArrayList<>(queryIDs.size());

				for (Integer queryID : queryIDs) {

					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);

					if (queryDescriptor != null) {

						queries.add((AttributeSumLimitQuery) getQuery(queryDescriptor));
					}
				}

				return queries;
			}

		} catch (SQLException e) {

			log.error("Unable to get queries using endpoint " + endpoint, e);
		}

		return null;
	}

	public Query getQuery(MutableQueryDescriptor descriptor) throws SQLException {

		AttributeSumLimitQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	public AttributeSumLimitQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<AttributeSumLimitQuery> getQuery = new HighLevelQuery<>();

		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(getQuery);
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();

		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(documentElement);

		return doc;
	}

	public String getEndpointURL(AttributeSumLimitEndpoint endpoint, User poster, AttributeHandler attributeHandler) {

		if (endpoint == null) {
			return null;
		}

		String address = endpoint.getAddress();

		User usedPoster = poster;

		if (usedPoster == null) {

			usedPoster = new SimpleUser();
		}

		TagReplacer tagReplacer = new URLEncodingTagReplacer();
		tagReplacer.addTagSource(USER_TAG_SOURCE_FACTORY.getTagSource(usedPoster));

		address = tagReplacer.replace(address);

		address = UserAttributeTagUtils.replaceTags(address, usedPoster, true);

		address = AttributeTagUtils.replaceTags(address, attributeHandler, false, true);

		return address;
	}

	public LimitProvider getAttributeSumLimit(AttributeSumLimitEndpoint endpoint, User poster, AttributeHandler attributeHandler) throws AttributeSumLimitAPIRequestException {

		if (endpoint == null) {
			return null;
		}
		
		String address = getEndpointURL(endpoint, poster, attributeHandler);

		log.info("User " + poster + " getting attribute sum limits values for api endpoint " + endpoint + " from: " + address);

		SimpleRequest simpleRequest = new SimpleRequest(address);

		simpleRequest.setFollowRedirects(false);
		simpleRequest.setConnectionTimeout(connectionTimeout * 1000);
		simpleRequest.setReadTimeout(readTimeout * 1000);

		if (endpoint.getUsername() != null && endpoint.getPassword() != null) {

			simpleRequest.setUsername(endpoint.getUsername());
			simpleRequest.setPassword(endpoint.getPassword());
		}

		List<Entry<String, String>> headerEntries = new ArrayList<>(2);

		headerEntries.add(new SimpleEntry<>("NoLoginRedirect", "NoLoginRedirect"));
		simpleRequest.setHeaders(headerEntries);

		try {
			HTTPResponse response = HTTPUtils.sendHTTPGetRequest(simpleRequest, endpoint.getCharset());

			try {
				XMLParser parser = new XMLParser(XMLUtils.parseXML(response.getValue(), endpoint.getCharset().toString(), false, false));

				Integer limit = parser.getInteger("/AttributeSumLimiter/Limit");
				
				if(limit == null) {
					
					throw new RuntimeException("Unable to parse limit from response using XPath /AttributeSumLimiter/limit");
				}

				String minDateString = parser.getString("/AttributeSumLimiter/FromDate");

				Date minDate = null;
				
				if(minDateString != null) {
					
					minDate = DatePopulator.getPopulator().getValue(minDateString);
				}
				
				String maxDateString = parser.getString("/AttributeSumLimiter/ToDate");

				Date maxDate = null;
				
				if(maxDateString != null) {
					
					maxDate = DatePopulator.getPopulator().getValue(maxDateString);
				}				
				
				return new AttributeSumLimiter(limit, minDate, maxDate);

			} catch (Exception e) {

				log.warn("Error parsing response XML for " + endpoint, e);
				throw new AttributeSumLimitAPIRequestException();
			}

		} catch (IOException e) {

			log.warn("Error connecting to " + endpoint, e);
			throw new AttributeSumLimitAPIRequestException();
		}
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public List<String> getAllowedEncodings() {

		return allowedEncodings;
	}

	public List<AttributeSumLimitEndpoint> getEndpoints() throws SQLException {

		HighLevelQuery<AttributeSumLimitEndpoint> query = new HighLevelQuery<>();

		return endpointDAO.getAll(query);
	}

	public AttributeSumLimitEndpoint getEndpoint(Integer endpointID) throws SQLException {

		HighLevelQuery<AttributeSumLimitEndpoint> query = new HighLevelQuery<>();
		query.addParameter(endpointIDParamFactory.getParameter(endpointID));

		return endpointDAO.get(query);
	}
	
	@Override
	public List<? extends APISource> getAPISources() throws Exception {

		return endpointDAO.getAll();
	}
	
	@Override
	public String getShortTypeName() {

		return apiSourceShortTypeName;
	}
	
	@Override
	public String getFullTypeName() {

		return apiSourceFullTypeName;
	}

	@Override
	public String getAddURL() {

		return this.getFullAlias() + "/add";
	}
	
	@Override
	public String getBaseShowURL() {

		return this.getFullAlias() + "/show/";
	}
	
	@Override
	public String getBaseUpdateURL() {

		return this.getFullAlias() + "/update/";
	}

	@Override
	public String getBaseDeleteURL() {

		return this.getFullAlias() + "/delete/";
	}

	@Override
	public boolean isInUse(APISource apiSource) throws Exception {

		AttributeSumLimitEndpoint endpoint = (AttributeSumLimitEndpoint)apiSource;
		
		return endpoint.getQueries() != null;
	}

	
	public APISourceHandler getAPISourceHandler() {
	
		return apiSourceHandler;
	}

	@Override
	public List<? extends MenuItemDescriptor> getVisibleMenuItems() {

		if(this.apiSourceHandler != null) {
			
			return null;
		
		}else {
		
			return super.getVisibleMenuItems();
		}
	}
}