package com.nordicpeak.flowengine.integration.base;

import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.cron4jutils.CronStringValidator;
import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.UserMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleAccessInterface;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.SystemStatus;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryOperators;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationEndpoint;
import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;
import com.nordicpeak.flowengine.integration.base.beans.BaseQueuedIntegrationRequest;
import com.nordicpeak.flowengine.integration.base.exceptions.IntegrationRequestException;
import com.nordicpeak.flowengine.integration.base.exceptions.IntegrationSettingsException;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;

import it.sauronsoftware.cron4j.Scheduler;

public abstract class BaseIntegrationProviderModule<QueuedIntegrationRequest extends BaseQueuedIntegrationRequest, IntegrationSettings extends BaseIntegrationSettings, IntegrationEndpoint extends BaseIntegrationEndpoint> extends AnnotatedForegroundModule implements FlowAdminExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor>, Runnable {

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Provider priority", description = "The priority of this provider", required = true)
	protected Integer priority = 100;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	protected boolean debugFragmententXML;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Supported actionID's", description = "The action ID's which will trigger export XML to be generated and stored when a submit event is detected", required = true)
	protected List<String> supportedActionIDs;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "Connection timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	protected Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Request timeout", description = "Request timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	protected Integer requestTimeout = 15;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max retry count", description = "Maximum number of times a failed request is retried before being removed (0 disables retrying)", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	protected Integer maxRetryCount = 0;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Retry interval", description = "How often a failed request should be retried (minutes)", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	protected Integer retryInterval = 60;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Retry check expression", description = "Cron expression for when to check for queued requests to retry", required = true, formatValidator = CronStringValidator.class)
	protected String retryCronExp = "*/15 * * * *";

	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Admin groups", description = "Groups that have access to the administrative functions in this module.")
	protected List<Integer> adminGroupIDs;

	@ModuleSetting(allowsNull = true)
	@UserMultiListSettingDescriptor(name = "Admin users", description = "Users that have access to the administrative functions in this module.")
	protected List<Integer> adminUserIDs;

	@InstanceManagerDependency(required = true)
	protected StaticContentModule staticContentModule;

	@InstanceManagerDependency(required = true)
	protected StandardFlowNotificationHandler notificationHandler;

	protected FlowAdminModule flowAdminModule;

	protected ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	protected Scheduler taskScheduler;

	protected AnnotatedDAO<IntegrationSettings> settingsDAO;
	protected QueryParameterFactory<IntegrationSettings, Integer> flowFamilyIDParamFactory;

	protected AnnotatedDAO<IntegrationEndpoint> endpointDAO;
	protected QueryParameterFactory<IntegrationEndpoint, Integer> endpointIDParamFactory;

	protected AnnotatedDAO<QueuedIntegrationRequest> queuedRequestDAO;
	protected QueryParameterFactory<QueuedIntegrationRequest, Timestamp> lastRetryParamFactory;
	protected QueryParameterFactory<QueuedIntegrationRequest, Integer> eventIDParamFactory;
	protected QueryParameterFactory<QueuedIntegrationRequest, Integer> flowInstanceIDParamFactory;

	protected AccessInterface adminAccessInterface;

	protected boolean configured;
	protected boolean running;

	@Override
	public String getExtensionViewTitle() {

		return moduleDescriptor.getDescription();
	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	protected abstract QueuedIntegrationRequest getNewQueuedIntegrationRequest();

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		initTaskScheduler();

		super.init(moduleDescriptor, sectionInterface, dataSource);
	}

	protected IntegrationSettings getIntegrationSettings(ImmutableFlow flow) throws SQLException {

		if (flow != null && flow.getFlowFamily() != null) {

			return getIntegrationSettings(flow.getFlowFamily().getFlowFamilyID());
		}

		return null;
	}

	protected IntegrationSettings getIntegrationSettings(Integer flowFamilyID) throws SQLException {

		HighLevelQuery<IntegrationSettings> query = new HighLevelQuery<>();

		query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamilyID));

		return settingsDAO.get(query);
	}

	protected IntegrationEndpoint getEndpoint(HttpServletRequest req) throws SQLException {

		Integer endpointID = NumberUtils.toInt(req.getParameter("endpointID"));

		if (endpointID != null) {

			HighLevelQuery<IntegrationEndpoint> query = new HighLevelQuery<>();
			query.addParameter(endpointIDParamFactory.getParameter(endpointID));

			return endpointDAO.get(query);
		}

		return null;
	}

	protected void addQueuedEvent(FlowInstanceEvent event, ImmutableFlowInstance flowInstance) throws SQLException {

		QueuedIntegrationRequest queuedEvent = getNewQueuedIntegrationRequest();

		queuedEvent.setEventID(event.getEventID());
		queuedEvent.setFlowInstanceID(flowInstance.getFlowInstanceID());

		queuedRequestDAO.add(queuedEvent, 20);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		this.configured = ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.WARN);

		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);

		this.adminAccessInterface = new SimpleAccessInterface(adminGroupIDs, adminUserIDs);

		stopTaskScheduler();
		initTaskScheduler();
	}

	@Override
	public void update(ForegroundModuleDescriptor descriptor, DataSource dataSource) throws Exception {

		stopTaskScheduler();
		initTaskScheduler();

		super.update(descriptor, dataSource);
	}

	@Override
	public void unload() throws Exception {

		if (flowAdminModule != null) {

			setFlowAdminModule(null);
		}

		stopTaskScheduler();

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (flowAdminModule == null && this.flowAdminModule != null) {

			this.flowAdminModule.removeExtensionViewProvider(this);
		}

		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}

	private void initTaskScheduler() {

		if (maxRetryCount == 0) {

			return;
		}

		this.taskScheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());

		this.taskScheduler.setDaemon(true);

		this.taskScheduler.schedule(retryCronExp, this);

		this.taskScheduler.start();
	}

	private void stopTaskScheduler() {

		if (this.taskScheduler != null && this.taskScheduler.isStarted()) {

			this.taskScheduler.stop();
		}
	}

	protected Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");

		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));
		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(document);

		return doc;
	}

	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {

		if (!AccessUtils.checkAccess(user, this.moduleDescriptor)) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showViewElement = doc.createElement("ShowIntegrationSettings");
		doc.getDocumentElement().appendChild(showViewElement);

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		boolean enabled = false;

		if (integrationSettings != null) {

			XMLUtils.append(doc, showViewElement, integrationSettings);
			enabled = integrationSettings.isEnabled();
		}

		showViewElement.appendChild(flow.toXML(doc));

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
	}

	@Override
	public void run() {

		if (this.systemInterface.getSystemStatus() == SystemStatus.STARTED && !running) {

			try {

				running = true;

				Calendar calendar = Calendar.getInstance();
				calendar.add(Calendar.MINUTE, -retryInterval);
				Timestamp retryIntervalMinutesAgo = new Timestamp(calendar.getTimeInMillis());

				HighLevelQuery<QueuedIntegrationRequest> query = new HighLevelQuery<>();

				query.addParameter(lastRetryParamFactory.getParameter(retryIntervalMinutesAgo, QueryOperators.SMALLER_THAN_OR_EQUALS));

				List<QueuedIntegrationRequest> queuedRequests = queuedRequestDAO.getAll(query);

				retryQueuedRequests(queuedRequests);

			} catch (Exception e) {

				log.error("Error processing queued requests", e);

			} finally {

				running = false;
			}
		}
	}

	protected void retryQueuedRequests(List<QueuedIntegrationRequest> queuedRequests) throws Exception {

		if (!CollectionUtils.isEmpty(queuedRequests)) {

			log.info("Retrying " + queuedRequests.size() + " queued requests");

			TransactionHandler transactionHandler = queuedRequestDAO.createTransaction();

			try {

				for (QueuedIntegrationRequest queuedRequest : queuedRequests) {

					if (this.systemInterface.getSystemStatus() != SystemStatus.STARTED) {

						log.warn("System stopped while retrying queued requests, aborting");
						return;
					}

					retryQueuedRequest(queuedRequest, transactionHandler);
				}

				transactionHandler.commit();

			} finally {

				TransactionHandler.autoClose(transactionHandler);
			}
		}
	}

	protected void retryQueuedRequest(QueuedIntegrationRequest queuedRequest, TransactionHandler transactionHandler) throws Exception {

		FlowInstanceEvent event = flowAdminModule.getFlowInstanceEvent(queuedRequest.getEventID(), getFlowInstanceRelations());

		if (event == null) {

			log.warn("Event not found, removing " + queuedRequest);

			queuedRequestDAO.delete(queuedRequest, transactionHandler);

			return;
		}

		if (event.getEventType() != EventType.SUBMITTED) {

			log.warn("Wrong event type, removing " + queuedRequest);

			queuedRequestDAO.delete(queuedRequest, transactionHandler);

			return;
		}

		queuedRequest.setRetryCount(queuedRequest.getRetryCount() + 1);

		log.info("Retry " + queuedRequest.getRetryCount() + "/" + maxRetryCount + " for " + queuedRequest);

		try {

			retryEvent(event.getFlowInstance(), event, queuedRequest, transactionHandler);

		} catch (IntegrationRequestException e) {

			if (queuedRequest.getRetryCount() >= maxRetryCount) {

				log.info(queuedRequest + " has been retried the maximum number of times, deleting");

				handleMaxRetryCountExceeded(event.getFlowInstance(), event, queuedRequest, transactionHandler, e);

			} else {

				log.info(queuedRequest + " failed, updating retry count and last retry timestamp");

				queuedRequest.setLastRetry(TimeUtils.getCurrentTimestamp());

				queuedRequestDAO.update(queuedRequest, transactionHandler, null);
			}

		} catch (IntegrationSettingsException e) {

			log.warn("Skipping retry of " + queuedRequest + ": " + e.getMessage());
		}
	}

	public abstract void processIntegrationEvent(FlowInstance flowInstance, FlowInstanceEvent event) throws Exception;

	protected void retryEvent(FlowInstance flowInstance, FlowInstanceEvent event, QueuedIntegrationRequest queuedRequest, TransactionHandler transactionHandler) throws Exception {

		processIntegrationEvent(flowInstance, event);

		log.info(queuedRequest + " succeeded, removing from queue");

		queuedRequestDAO.delete(queuedRequest, transactionHandler);
	}

	protected void handleMaxRetryCountExceeded(ImmutableFlowInstance flowInstance, FlowInstanceEvent event, QueuedIntegrationRequest queuedRequest, TransactionHandler transactionHandler, IntegrationRequestException lastRequestException) throws SQLException {

		queuedRequestDAO.delete(queuedRequest, transactionHandler);
	}

	@EventListener(channel = FlowFamily.class)
	public void processFlowFamilyEvent(CRUDEvent<FlowFamily> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.DELETE) {

			try (TransactionHandler transaction = settingsDAO.createTransaction()) {
				for (FlowFamily flowFamily : event.getBeans()) {

					log.info("Deleting integration settings for flow family ID: " + flowFamily.getFlowFamilyID());

					try {
						HighLevelQuery<IntegrationSettings> query = new HighLevelQuery<>();

						query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));

						settingsDAO.delete(query, transaction);

					} catch (SQLException e) {

						log.error("Error deleting integration settings for flow family ID: " + flowFamily.getFlowFamilyID(), e);
					}
				}
				transaction.commit();
			} catch (Exception e) {
				log.error("Error deleting integration settings for settingsDAO on event: " + event, e);
			}
		}
	}

	@EventListener(channel = FlowInstance.class)
	public void processFlowInstanceEvent(CRUDEvent<FlowInstance> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.DELETE) {

			try (TransactionHandler transactionHandler = queuedRequestDAO.createTransaction()) {

				for (FlowInstance flowInstance : event.getBeans()) {

					log.info("Deleting queued requests for flow instance: " + flowInstance.getFlowInstanceID());

					HighLevelQuery<QueuedIntegrationRequest> query = new HighLevelQuery<>();

					query.addParameter(flowInstanceIDParamFactory.getParameter(flowInstance.getFlowInstanceID()));

					queuedRequestDAO.delete(query, transactionHandler, 10);
				}

				transactionHandler.commit();

			} catch (Exception e) {

				log.error("Error deleting queued requests for events for flow instances " + event.getBeans(), e);
			}

		}
	}

	public boolean shouldProcessEvent(SubmitEvent event) {

		if (!configured) {

			log.warn("Module " + moduleDescriptor + " is not properly configured, ignoring event " + event);

			return false;
		}

		if (event.getEvent().getEventType() != EventType.SUBMITTED || event.getActionID() == null || !supportedActionIDs.contains(event.getActionID())) {

			return false;
		}

		return true;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse triggerSubmit(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		if (!AccessUtils.checkAccess(user, adminAccessInterface)) {

			throw new AccessDeniedException("Submit trigger access denied.");
		}

		if (flowAdminModule == null) {

			return new SimpleForegroundModuleResponse("Missing required dependency");
		}

		Integer eventID = uriParser.getInt(2);

		log.info("User " + user + " manually triggering submit event ID: " + eventID);

		if (eventID == null) {

			throw new URINotFoundException(uriParser);
		}

		HighLevelQuery<QueuedIntegrationRequest> query = new HighLevelQuery<>();

		query.addParameter(eventIDParamFactory.getParameter(eventID));

		List<QueuedIntegrationRequest> queuedRequests = queuedRequestDAO.getAll(query);

		if (!CollectionUtils.isEmpty(queuedRequests)) {

			retryQueuedRequests(queuedRequests);

		} else {

			FlowInstanceEvent event = flowAdminModule.getFlowInstanceEvent(eventID, getFlowInstanceRelations());

			if (event == null) {

				return new SimpleForegroundModuleResponse("Event not found");
			}
			
			if (event.getEventType() != EventType.SUBMITTED) {

				return new SimpleForegroundModuleResponse("Wrong event type");
			}

			processIntegrationEvent(event.getFlowInstance(), event);
		}

		return new SimpleForegroundModuleResponse("Event triggered, check log.");
	}

	@WebPublic(alias = "delete")
	public ForegroundModuleResponse deleteFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		if (!HTTPUtils.isPost(req)) {

			throw new AccessDeniedException("Delete requests using method " + req.getMethod() + " are not allowed.");
		}

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
		}
		
		if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		if (integrationSettings != null) {

			log.info("User " + user + " deleting integration settings for flow " + flow);

			settingsDAO.delete(integrationSettings);

			//TODO add event to flow family history

		} else {

			log.warn("User " + user + " trying to deleting integration settings for flow " + flow + " which has no integration settings");
		}

		flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + getShowFlowModuleAnchorLink());

		return null;
	}

	protected abstract String getShowFlowModuleAnchorLink();

	protected void appendDataObject(JsonArray dataArray, String name, String value, MutableAttributeHandler attributeHandler, TagReplacer tagReplacer) {

		if (dataArray == null) {

			return;
		}

		String tagReplacedValue = replaceTags(value, attributeHandler, tagReplacer);

		if (!StringUtils.isEmpty(tagReplacedValue)) {

			JsonObject dataObject = new JsonObject();

			dataObject.putField("key", name);
			dataObject.putField("value", tagReplacedValue);

			dataArray.addNode(dataObject);
		}
	}

	protected String replaceTags(String text, AttributeHandler attributeHandler, TagReplacer tagReplacer) {

		if (text == null) {

			return null;
		}

		if (attributeHandler != null) {

			text = AttributeTagUtils.replaceTags(text, attributeHandler);
		}

		if (tagReplacer != null) {

			text = tagReplacer.replace(text);
		}

		return text;
	}

	protected void sendEmails(List<String> addresses, String subject, String message, ImmutableFlowInstance flowInstance, TagReplacer tagReplacer) {

		if (!CollectionUtils.isEmpty(addresses)) {

			for (String address : addresses) {

				sendEmail(address, subject, message, flowInstance, tagReplacer);
			}
		}
	}

	protected void sendEmail(String address, String subject, String message, ImmutableFlowInstance flowInstance, TagReplacer tagReplacer) {

		if (ObjectUtils.containsNull(address, subject, message)) {

			return;
		}

		subject = replaceTags(subject, flowInstance.getAttributeHandler(), tagReplacer);
		message = replaceTags(message, flowInstance.getAttributeHandler(), tagReplacer);

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {

			email.addRecipient(address);
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(notificationHandler.getEmailSenderName(flowInstance));
			email.setSenderAddress(notificationHandler.getEmailSenderAddress(flowInstance));
			email.setSubject(subject);
			email.setMessage(EmailUtils.addMessageBody(message));

			systemInterface.getEmailHandler().send(email);

		} catch (Exception e) {

			log.info("Error sending email " + email, e);
		}
	}

	protected void logResponse(String response, Exception e, boolean error) {

		if (error) {

			log.error(response, e);

		} else {

			log.warn(response, e);
		}
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

	protected TagReplacer getTagReplacer(ImmutableFlowInstance flowInstance) {

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(StandardFlowNotificationHandler.FLOWINSTANCE_TAG_SOURCE_FACTORY.getTagSource((FlowInstance) flowInstance));
		tagReplacer.addTagSource(new SingleTagSource("$flowInstance.url", notificationHandler.getFlowInstanceAdminModuleAlias(flowInstance) + "/overview/" + flowInstance.getFlowInstanceID()));
		tagReplacer.addTagSource(StandardFlowNotificationHandler.FLOW_TAG_SOURCE_FACTORY.getTagSource((Flow) flowInstance.getFlow()));
		tagReplacer.addTagSource(StandardFlowNotificationHandler.STATUS_TAG_SOURCE_FACTORY.getTagSource((Status) flowInstance.getStatus()));

		Contact posterContact = FlowInstanceUtils.getPosterContact(flowInstance);

		if (posterContact != null) {

			tagReplacer.addTagSource(StandardFlowNotificationHandler.CONTACT_TAG_SOURCE_FACTORY.getTagSource(posterContact));
		}

		return tagReplacer;
	}

	protected Field[] getFlowInstanceRelations() {

		return new Field[] { FlowInstanceEvent.FLOW_INSTANCE_RELATION, FlowInstance.ATTRIBUTES_RELATION, FlowInstance.FLOW_RELATION, FlowInstance.STATUS_RELATION, FlowInstance.MANAGERS_RELATION, Flow.FLOW_FAMILY_RELATION, FlowFamily.MANAGER_GROUPS_RELATION, FlowFamily.MANAGER_USERS_RELATION };
	}

}
