package com.nordicpeak.flowengine.queries.basetablequery;

import java.lang.reflect.Field;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.IntegerBasedCRUD;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.populators.AlternativesPopulator;

public abstract class TableColumnCRUD<DropAlternative extends BaseDropDownAlternative, CellValue extends TableCellValue<DropAlternative, Query, Row, Column>, Row extends TableRow<DropAlternative, Column, Query>, QueryInstance extends BaseTableQueryInstance<DropAlternative, Row, Column, Query, CellValue>, Column extends BaseTableColumn<DropAlternative, Column, Query>, Query extends BaseTableQuery<DropAlternative, Query, Column>, TableQueryProvider extends TableQueryProviderModule<DropAlternative, Column, Query, Row, CellValue, QueryInstance, TableQueryProvider>> extends IntegerBasedCRUD<Column, TableQueryProvider> {

	protected AnnotatedDAOWrapper<Column, Integer> tableColumnDAO;

	protected static EnumPopulator<ColumnType> COLUM_TYPE_POPULATOR = new EnumPopulator<>(ColumnType.class);

	public TableColumnCRUD(AnnotatedDAOWrapper<Column, Integer> tableColumnDAO, BeanRequestPopulator<Column> populator, String typeElementName, String typeLogName, String listMethodAlias, TableQueryProvider callback) {

		super(tableColumnDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.tableColumnDAO = tableColumnDAO;
	}

	@Override
	protected ForegroundModuleResponse beanAdded(Column bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventColumnAddedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(Column bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventColumnUpdatedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanDeleted(Column bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventColumnDeletedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	public Column getBean(Integer beanID) throws SQLException, AccessDeniedException {

		HighLevelQuery<Column> query = new HighLevelQuery<>(getColumnRelations());

		query.addParameter(tableColumnDAO.getParameterFactory().getParameter(beanID));

		Column tableRow = tableColumnDAO.getAnnotatedDAO().get(query);

		if (tableRow != null && tableRow.getQuery() != null) {

			tableRow.getQuery().init(callback.getFlowAdminModule().getQueryDescriptor(tableRow.getQuery().getQueryID()), null);

		}

		return tableRow;
	}

	protected abstract Field[] getColumnRelations();

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, addTypeElement);
		appendColumnTypes(doc, addTypeElement);
		appendColumns(null, doc, addTypeElement, req);

		appendAlternativeLists(null, doc, addTypeElement);
	}

	@Override
	protected void appendUpdateFormData(Column column, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, updateTypeElement);
		appendColumnTypes(doc, updateTypeElement);
		appendColumns(column, doc, updateTypeElement, req);

		Element listsElement = appendAlternativeLists(column, doc, updateTypeElement);

		appendCurrentAlternativeList(column, doc, updateTypeElement, listsElement);
	}

	private Element appendAlternativeLists(Column column, Document doc, Element updateTypeElement) {

		Element listsElement = XMLUtils.appendNewElement(doc, updateTypeElement, "AlternativeLists");

		for (AlternativeListDescriptor list : callback.getAlternativeLists()) {

			AlternativeListProvider provider = callback.getAlternativeListProvider(list.getProviderID());

			Element providerElement = XMLUtils.appendNewElement(doc, listsElement, "AlternativeList");

			XMLUtils.appendNewElement(doc, providerElement, "Name", provider.getProviderName() + ": " + list.getName());
			XMLUtils.appendNewElement(doc, providerElement, "ID", list.getName());
		}

		return listsElement;
	}

	private void appendCurrentAlternativeList(Column column, Document doc, Element updateTypeElement, Element listsElement) {

		AlternativeListDescriptor currentListDescriptor = callback.getAlternativeListDescriptor(column);

		if (currentListDescriptor != null) {

			XMLUtils.appendNewElement(doc, updateTypeElement, "CurrentAlternativeList", currentListDescriptor.getName());

		} else if (!StringUtils.isEmpty(column.getAlternativeListName())) {

			Element providerElement = (Element) listsElement.insertBefore(doc.createElement("AlternativeList"), listsElement.getFirstChild());

			XMLUtils.appendNewElement(doc, providerElement, "Name", callback.getMissingAlternativeList() + column.getAlternativeListName());
			XMLUtils.appendNewElement(doc, providerElement, "ID", column.getAlternativeListName());
		}
	}

	protected void appendFormatValidators(Document doc, Element element) {

		XMLUtils.append(doc, element, callback.getFormatValidators());
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Column populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		Query query = (Query) req.getAttribute("TableQuery");

		Column tableColumn = super.populateFromAddRequest(req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<>();

		List<DropAlternative> alternatives = populateDropDownAlternatives(tableColumn, req, validationErrors, null);

		validateRowSum(tableColumn, query, validationErrors);

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		tableColumn.setDropDownAlternatives(alternatives);
		tableColumn.setQuery(query);

		tableColumn.setSortIndex(this.getCurrentMaxSortIndex(tableColumn.getQuery()) + 1);

		return tableColumn;
	}

	@Override
	protected Column populateFromUpdateRequest(Column column, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		ColumnType oldColumnType = column.getColumnType();

		column = super.populateFromUpdateRequest(column, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<>();

		List<DropAlternative> alternatives = populateDropDownAlternatives(column, req, validationErrors, oldColumnType);

		validateRowSum(column, column.getQuery(), validationErrors);

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		column.setDropDownAlternatives(alternatives);

		return column;
	}

	private void validateRowSum(Column column, Query query, List<ValidationError> validationErrors) throws SQLException {

		List<Column> queryColumns = getQueryColumns(query);

		if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

			if (column.getRowSumAddedColumns() == null && column.getRowSumSubtractedColumns() == null) {

				validationErrors.add(new ValidationError("RowSumSelection", ValidationErrorType.RequiredField));

			} else {

				if (column.getRowSumAddedColumns() != null) {
					for (int columnID : column.getRowSumAddedColumns()) {

						Column selectedColumn = getQueryColumn(columnID, column, queryColumns);

						if (selectedColumn == null) {
							validationErrors.add(new ValidationError("rowsumadd", ValidationErrorType.InvalidFormat));
						}
					}
				}

				if (column.getRowSumSubtractedColumns() != null) {
					for (int columnID : column.getRowSumSubtractedColumns()) {

						Column selectedColumn = getQueryColumn(columnID, column, queryColumns);

						if (selectedColumn == null) {
							validationErrors.add(new ValidationError("rowsumsubtract", ValidationErrorType.InvalidFormat));
						}
					}
				}
			}
		}
	}

	private Column getQueryColumn(int columnID, Column currentColumn, List<Column> queryColumns) {

		for (Column column : queryColumns) {

			if (column.equals(currentColumn)) {
				return null;
			}

			if (column.getColumnID() == columnID) {
				return column;
			}
		}

		return null;
	}

	protected List<DropAlternative> populateDropDownAlternatives(Column column, HttpServletRequest req, List<ValidationError> validationErrors, ColumnType oldColumnType) {

		if (column.getColumnType() == ColumnType.DROPDOWN) {

			List<DropAlternative> alternatives;

			if (oldColumnType == ColumnType.DROPDOWN) {

				alternatives = getAlternativesPopulator().populate(column.getDropDownAlternatives(), req, validationErrors);

			} else {

				alternatives = getAlternativesPopulator().populate(null, req, validationErrors);
			}

			if (!CollectionUtils.isEmpty(alternatives) && alternatives.size() >= 2) {

				return alternatives;
			}

			validationErrors.add(new ValidationError("TooFewAlternatives"));
		}

		return null;
	}

	protected abstract AlternativesPopulator<DropAlternative> getAlternativesPopulator();

	@Override
	protected void checkUpdateAccess(Column bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	@Override
	protected void checkDeleteAccess(Column bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	private int getCurrentMaxSortIndex(Query tableQuery) throws SQLException {

		ObjectQuery<Integer> query = new ObjectQuery<>(tableColumnDAO.getAnnotatedDAO().getDataSource(), "SELECT MAX(sortIndex) FROM " + tableColumnDAO.getAnnotatedDAO().getTableName() + " WHERE queryID = ?", IntegerPopulator.getPopulator());

		query.setInt(1, tableQuery.getQueryID());

		Integer sortIndex = query.executeQuery();

		if (sortIndex == null) {

			return 0;
		}

		return sortIndex;
	}

	protected void appendColumnTypes(Document doc, Element element) {

		for (ColumnType layout : ColumnType.values()) {

			Element fieldLayoutElement = doc.createElement("ColumnType");
			element.appendChild(fieldLayoutElement);

			XMLUtils.appendNewElement(doc, fieldLayoutElement, "name", callback.getColumnTypeName(layout));
			XMLUtils.appendNewElement(doc, fieldLayoutElement, "value", layout);
		}
	}

	@SuppressWarnings("unchecked")
	protected void appendColumns(Column column, Document doc, Element element, HttpServletRequest req) throws SQLException {

		Query query;

		if (column != null) {

			query = column.getQuery();

		} else {

			query = (Query) req.getAttribute("TableQuery");
		}

		Element columnsElement = doc.createElement("Columns");
		element.appendChild(columnsElement);

		XMLUtils.append(doc, element, "Columns", getQueryColumns(query));
	}

	protected List<Column> getQueryColumns(Query query) throws SQLException {

		return callback.getQueryColumns(query);
	}
}
