package com.nordicpeak.flowengine.queries.basetablequery;

import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.listeners.SystemStartupListener;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.validationerrors.InvalidFormatValidationError;
import se.unlogic.standardutils.arrays.ArrayUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberStringyfier;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.StringFormatValidator;
import se.unlogic.standardutils.validation.TooLongContentValidationError;
import se.unlogic.standardutils.validation.TooShortContentValidationError;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.alternativelist.AlternativeListHandlerModule;
import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQueryProvider;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.formatvalidation.FormatValidationHandler;
import com.nordicpeak.flowengine.formatvalidation.FormatValidator;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public abstract class TableQueryProviderModule<DropAlternative extends BaseDropDownAlternative, Column extends BaseTableColumn<DropAlternative, Column, Query>, Query extends BaseTableQuery<DropAlternative, Query, Column>, Row extends TableRow<DropAlternative, Column, Query>, CellValue extends TableCellValue<DropAlternative, Query, Row, Column>, QueryInstance extends BaseTableQueryInstance<DropAlternative, Row, Column, Query, CellValue>, QueryProvider extends TableQueryProviderModule<DropAlternative, Column, Query, Row, CellValue, QueryInstance, QueryProvider>> extends BaseQueryProviderModule<QueryInstance> implements SystemStartupListener, AlternativeListQueryProvider {

	@XSLVariable(prefix = "java.")
	protected String columnTypeString = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeTextArea = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeCheckbox = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeDropdown = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeRowSumAdd = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeRowSumMultiply = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeDropdownAlternativeList = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String columnTypeListSearch = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String eventColumnAddedMessage = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String eventColumnUpdatedMessage = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String eventColumnDeletedMessage = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String missingList = "Missing";

	@XSLVariable(prefix = "java.")
	private String showingPartialList1 = "Showing";

	@XSLVariable(prefix = "java.")
	private String showingPartialList2 = "of";

	@XSLVariable(prefix = "java.")
	private String showingPartialList3 = "alternatives";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "List search max search hits", description = "Max number of search hits returned when using list search", required = true, formatValidator = PositiveStringIntegerPopulator.class)
	private int maxHits = 20;

	protected RelationQuery SAVE_QUERY_INSTANCE_RELATION_QUERY;

	protected AnnotatedDAO<Query> queryDAO;
	protected AnnotatedDAO<QueryInstance> queryInstanceDAO;
	protected AnnotatedDAO<Column> columnDAO;
	protected AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;

	protected TableQueryCRUD<DropAlternative, CellValue, Row, QueryInstance, Column, Query, QueryProvider> queryCRUD;
	protected TableColumnCRUD<DropAlternative, CellValue, Row, QueryInstance, Column, Query, QueryProvider> columnCRUD;

	protected QueryParameterFactory<Query, Integer> queryIDParamFactory;
	protected QueryParameterFactory<QueryInstance, Integer> queryInstanceIDParamFactory;
	protected QueryParameterFactory<Column, Query> columnQueryParamFactory;
	protected QueryParameterFactory<Column, String> columnListNameParamFactory;

	@InstanceManagerDependency(required = true)
	private FormatValidationHandler formatValidationHandler;

	@InstanceManagerDependency(required = true)
	private AlternativeListHandlerModule alternativeListHandlerModule;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		SAVE_QUERY_INSTANCE_RELATION_QUERY = getSaveQueryInstanceRelationQuery();

		systemInterface.addSystemStartupListener(this);
	}

	protected abstract RelationQuery getSaveQueryInstanceRelationQuery();

	@Override
	public void systemStarted() {

		if (formatValidationHandler == null) {

			log.error("No format validation handler found in instance handler.");
		}
	}

	protected abstract QueryInstance getNewQueryInstance();

	protected abstract Query getNewQuery();

	protected abstract CellValue getNewCellValue(Row row, Column column, String value);

	protected abstract CellValue getNewCellValue(Row row, Column column, String value, String displayValue, String xmlValue);

	protected abstract Column getNewTableColumn();

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		Query query = getNewQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@SuppressWarnings({ "unchecked" })
	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		Query query = getNewQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));

		contentFilter.filterHTML(query);

		List<Integer> oldColumnIDs = new ArrayList<>();
		List<Integer> oldDropDownAlternativeIDs = new ArrayList<>();
		Map<Column, List<Integer>> oldRowSumAddColumnIDs = new HashMap<>();
		Map<Column, List<Integer>> oldRowSumSubtractColumnIDs = new HashMap<>();

		if (query.getColumns() != null) {

			for (Column column : query.getColumns()) {

				oldColumnIDs.add(column.getColumnID());

				column.setColumnID(null);

				if (!CollectionUtils.isEmpty(column.getDropDownAlternatives())) {

					for (BaseDropDownAlternative alternative : (List<BaseDropDownAlternative>) column.getDropDownAlternatives()) {

						oldDropDownAlternativeIDs.add(alternative.getAlternativeID());

						alternative.setAlternativeID(null);
					}
				}

				if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

					if (!CollectionUtils.isEmpty(column.getRowSumAddedColumns())) {

						oldRowSumAddColumnIDs.put(column, column.getRowSumAddedColumns());
						column.setRowSumAddedColumns(null);
					}

					if (!CollectionUtils.isEmpty(column.getRowSumSubtractedColumns())) {

						oldRowSumSubtractColumnIDs.put(column, column.getRowSumSubtractedColumns());
						column.setRowSumSubtractedColumns(null);
					}
				}
			}
		}

		queryDAO.add(query, transactionHandler, null);

		boolean needsUpdate = false;

		if (query.getColumns() != null) {

			Map<Integer, Integer> columnConversionMap = new HashMap<>(oldColumnIDs.size());
			Map<Integer, Integer> dropDownAlternativeConversionMap = new HashMap<>(oldDropDownAlternativeIDs.size());

			int columnIndex = 0;
			int dropDownIndex = 0;

			for (Integer oldColumnID : oldColumnIDs) {

				Column column = query.getColumns().get(columnIndex);

				columnConversionMap.put(oldColumnID, column.getColumnID());

				List<DropAlternative> alternatives = column.getDropDownAlternatives();

				if (!CollectionUtils.isEmpty(alternatives)) {

					for (int i = 0; i < alternatives.size() && dropDownIndex < oldDropDownAlternativeIDs.size(); i++) {

						DropAlternative alternative = alternatives.get(i);
						Integer oldDropDownAlternativeID = oldDropDownAlternativeIDs.get(dropDownIndex);

						dropDownAlternativeConversionMap.put(oldDropDownAlternativeID, alternative.getAlternativeID());

						dropDownIndex++;
					}
				}

				columnIndex++;
			}

			for (Column column : query.getColumns()) {

				if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

					List<Integer> oldAddColumnIDs = oldRowSumAddColumnIDs.get(column);

					if (oldAddColumnIDs != null) {

						for (int i = 0; i < oldAddColumnIDs.size(); i++) {
							oldAddColumnIDs.set(i, columnConversionMap.get(oldAddColumnIDs.get(i)));
						}

						needsUpdate = true;
						column.setRowSumAddedColumns(oldAddColumnIDs);
					}

					List<Integer> oldSubtractColumnIDs = oldRowSumSubtractColumnIDs.get(column);

					if (oldSubtractColumnIDs != null) {

						for (int i = 0; i < oldSubtractColumnIDs.size(); i++) {
							oldSubtractColumnIDs.set(i, columnConversionMap.get(oldSubtractColumnIDs.get(i)));
						}

						needsUpdate = true;
						column.setRowSumSubtractedColumns(oldSubtractColumnIDs);
					}
				}
			}

			query.setColumnConversionMap(columnConversionMap);
			query.setDropDownAlternativeConversionMap(dropDownAlternativeConversionMap);
		}

		if (needsUpdate) {
			queryDAO.update(query, transactionHandler, new RelationQuery(getTableQueryRelations()));
		}

		return query;
	}

	public void copyQuery(Query query, MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		query.setQueryID(copyQueryDescriptor.getQueryID());

		List<Integer> oldColumnIDs = new ArrayList<>();
		Map<Column, List<Integer>> oldRowSumAddColumnIDs = new HashMap<>();
		Map<Column, List<Integer>> oldRowSumSubtractColumnIDs = new HashMap<>();

		if (query.getColumns() != null) {

			for (Column column : query.getColumns()) {

				oldColumnIDs.add(column.getColumnID());
				column.setColumnID(null);

				if (column.getDropDownAlternatives() != null) {

					for (BaseDropDownAlternative alternative : column.getDropDownAlternatives()) {

						alternative.setAlternativeID(null);
					}
				}

				if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

					if (!CollectionUtils.isEmpty(column.getRowSumAddedColumns())) {

						oldRowSumAddColumnIDs.put(column, column.getRowSumAddedColumns());
						column.setRowSumAddedColumns(null);
					}

					if (!CollectionUtils.isEmpty(column.getRowSumSubtractedColumns())) {

						oldRowSumSubtractColumnIDs.put(column, column.getRowSumSubtractedColumns());
						column.setRowSumSubtractedColumns(null);
					}
				}
			}
		}

		queryDAO.add(query, transactionHandler, new RelationQuery(getTableQueryRelations()));

		boolean needsUpdate = false;

		if (query.getColumns() != null) {

			Map<Integer, Integer> columnConversionMap = new HashMap<>(oldColumnIDs.size());

			int columnIndex = 0;

			for (Integer oldColumnID : oldColumnIDs) {

				Column column = query.getColumns().get(columnIndex);

				columnConversionMap.put(oldColumnID, column.getColumnID());
				columnIndex++;
			}

			for (Column column : query.getColumns()) {

				if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

					List<Integer> oldAddColumnIDs = oldRowSumAddColumnIDs.get(column);

					if (oldAddColumnIDs != null) {

						for (int i = 0; i < oldAddColumnIDs.size(); i++) {
							oldAddColumnIDs.set(i, columnConversionMap.get(oldAddColumnIDs.get(i)));
						}

						needsUpdate = true;
						column.setRowSumAddedColumns(oldAddColumnIDs);
					}

					List<Integer> oldSubtractColumnIDs = oldRowSumSubtractColumnIDs.get(column);

					if (oldSubtractColumnIDs != null) {

						for (int i = 0; i < oldSubtractColumnIDs.size(); i++) {
							oldSubtractColumnIDs.set(i, columnConversionMap.get(oldSubtractColumnIDs.get(i)));
						}

						needsUpdate = true;
						column.setRowSumSubtractedColumns(oldSubtractColumnIDs);
					}
				}
			}
		}

		if (needsUpdate) {
			queryDAO.update(query, transactionHandler, new RelationQuery(getTableQueryRelations()));
		}
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		Query query = getQuery(descriptor.getQueryID(), extraData);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		Query query = getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryResponse getFormHTML(QueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (queryInstance.getQuery().getColumns() != null) {

			for (Column column : queryInstance.getQuery().getColumns()) {

				if (column.getAlternativeList() == null || column.getAlternativeList().requiresReload(attributeHandler)) {

					column.setAlternativeList(getAlternativeListInstance(column, attributeHandler, poster));
				}
			}
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		QueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = getNewQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		if (descriptor.getQueryInstanceID() == null && queryInstance.getQuery().getColumns() != null) {

			for (Column column : queryInstance.getQuery().getColumns()) {

				column.setAlternativeList(getAlternativeListInstance(column, null, poster));
			}
		}

		return queryInstance;
	}

	protected Query getQuery(Integer queryID) throws SQLException {

		return getQuery(queryID, false);
	}

	protected Query getQuery(Integer queryID, boolean withAlternativeList) throws SQLException {

		HighLevelQuery<Query> highLevelQuery = new HighLevelQuery<>(getTableQueryRelations());

		highLevelQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		Query query = queryDAO.get(highLevelQuery);

		if (query != null && query.getColumns() != null) {

			for (Column column : query.getColumns()) {

				column.setQuery(query);

				if (withAlternativeList) {
					column.setAlternativeList(getAlternativeListInstance(column, null, null));
				}
			}
		}

		return query;
	}

	protected Query getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		return getQuery(queryID, transactionHandler, false);
	}

	protected Query getQuery(Integer queryID, TransactionHandler transactionHandler, boolean withAlternativeList) throws SQLException {

		HighLevelQuery<Query> highLevelQuery = new HighLevelQuery<>(getTableQueryRelations());

		highLevelQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		Query query = queryDAO.get(highLevelQuery, transactionHandler);

		if (query != null && query.getColumns() != null) {

			for (Column column : query.getColumns()) {

				column.setQuery(query);

				if (withAlternativeList) {
					column.setAlternativeList(getAlternativeListInstance(column, null, null));
				}
			}
		}

		return query;
	}

	protected abstract Field[] getTableQueryRelations();

	protected QueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<QueryInstance> query = new HighLevelQuery<>(getQueryInstanceRelations());

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	protected abstract Field[] getQueryInstanceRelations();

	@Override
	public void save(QueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
		}
	}

	protected CellValue populateValue(HttpServletRequest req, List<CellValue> cellValues, Column column, Row row, QueryInstance queryInstance, String cellID, boolean allowPartialPopulation, List<ValidationError> validationErrors) {

		if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {
			return null;
		}

		String value = req.getParameter("q" + queryInstance.getQuery().getQueryID() + cellID);
		String displayValue = null;
		String xmlValue = null;

		if (StringUtils.isEmpty(value)) {

			if (cellIsRequired(column, row, queryInstance, allowPartialPopulation)) {

				validationErrors.add(new ValidationError(cellID, ValidationErrorType.RequiredField));
			}

			return null;
		}

		value = value.trim();

		if (column.getColumnType() == ColumnType.CHECKBOX) {

			if (StringUtils.isEmpty(value)) {

				cellValues.add(getNewCellValue(row, column, "false"));

			} else {

				cellValues.add(getNewCellValue(row, column, "true"));
			}

			return null;
		}

		if (column.getColumnType() == ColumnType.STRING) {

			Integer maxLength = column.getMaxContentLength();

			if (maxLength == null || maxLength > 255) {

				maxLength = 255;
			}

			if (value.length() > maxLength) {

				validationErrors.add(new TooLongContentValidationError(cellID, value.length(), maxLength));
				return null;
			}

			Integer minLength = column.getMinContentLength();

			if (minLength != null && value.length() > 0 && value.length() < minLength) {

				validationErrors.add(new TooShortContentValidationError(cellID, value.length(), minLength));
				return null;
			}

			if (column.getFormatValidator() != null) {

				StringFormatValidator formatValidator = getStringFormatValidator(column);

				if (formatValidator == null) {

					log.warn("Unable to get format validator " + column.getFormatValidator() + " for table cell " + row + " " + column);

				} else if (!formatValidator.validateFormat(value)) {

					validationErrors.add(new InvalidFormatValidationError(cellID, column.getInvalidFormatMessage()));
					return null;
				}
			}

		} else if (column.getColumnType() == ColumnType.TEXTAREA) {

			Integer maxLength = column.getMaxContentLength();

			if (maxLength == null || maxLength > 1024) {

				maxLength = 1024;
			}

			if (value.length() > maxLength) {

				validationErrors.add(new TooLongContentValidationError(cellID, value.length(), maxLength));
				return null;
			}

		} else if (column.getColumnType() == ColumnType.DROPDOWN) {

			List<? extends ImmutableAlternative> availableAlternatives = column.getDropDownAlternatives();

			ImmutableAlternative alternative = populateAlternative(column, row, queryInstance, cellID, allowPartialPopulation, validationErrors, value, availableAlternatives);

			if (alternative == null) {

				return null;
			}

			value = alternative.getAlternativeID().toString();

		} else if (column.getColumnType() == ColumnType.DROPDOWN_ALTERNATIVE_LIST || column.getColumnType() == ColumnType.LIST_SEARCH) {

			List<? extends ImmutableAlternative> availableAlternatives = null;

			if (column.getAlternativeList() != null) {

				availableAlternatives = column.getAlternativeList().getAlternatives();
			}

			ImmutableAlternative alternative = populateAlternative(column, row, queryInstance, cellID, allowPartialPopulation, validationErrors, value, availableAlternatives);

			if (alternative == null) {

				return null;
			}

			value = alternative.getAlternativeID().toString();
			displayValue = alternative.getName();
			xmlValue = alternative.getExportXMLValue();
		}

		CellValue cellValue = getNewCellValue(row, column, value, displayValue, xmlValue);

		cellValues.add(cellValue);

		return cellValue;
	}

	private ImmutableAlternative populateAlternative(Column column, Row row, QueryInstance queryInstance, String cellID, boolean allowPartialPopulation, List<ValidationError> validationErrors, String value, List<? extends ImmutableAlternative> availableAlternatives) {

		if (CollectionUtils.isEmpty(availableAlternatives)) {

			//If the parent query doesn't have any alternatives then there is no population to do
			return null;
		}

		Integer alternativeID = NumberUtils.toInt(value);

		ImmutableAlternative selectedAlternative = null;

		if (alternativeID != null) {

			selectedAlternative = CollectionUtils.find(availableAlternatives, alternative -> alternative.getAlternativeID().equals(alternativeID));
		}

		if (selectedAlternative == null) {

			if (cellIsRequired(column, row, queryInstance, allowPartialPopulation)) {

				validationErrors.add(new ValidationError(cellID, ValidationErrorType.RequiredField));
			}

			return null;
		}

		return selectedAlternative;
	}

	private boolean cellIsRequired(Column column, Row row, QueryInstance queryInstance, boolean allowPartialPopulation) {

		return !allowPartialPopulation && row.isRequired() && column.isRequired() && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED;
	}

	protected void resetQueryInstanceSummary(QueryInstance queryInstance) {

		List<Column> columns = queryInstance.getQuery().getColumns();

		if (CollectionUtils.isEmpty(columns)) {
			return;
		}

		for (Column column : columns) {

			column.setSum(null);
		}
	}

	protected void calculateQueryInstanceSummary(QueryInstance queryInstance, List<Row> rows) {

		List<Column> columns = queryInstance.getQuery().getColumns();

		if (CollectionUtils.isEmpty(columns)) {
			return;
		}

		if (CollectionUtils.isEmpty(rows) || CollectionUtils.isEmpty(queryInstance.getValues())) {

			resetQueryInstanceSummary(queryInstance);
			return;
		}

		for (Column column : columns) {

			if (!column.isSummarise()) {
				continue;
			}

			BigDecimal sum = null;

			for (Row row : rows) {

				String value = queryInstance.getValue(row.getRowIdentifier(), column.getColumnID());

				if (!StringUtils.isEmpty(value)) {

					if (sum == null) {
						sum = new BigDecimal(0);
					}

					if (column.getColumnType() == ColumnType.CHECKBOX) {

						if ("true".equals(value)) {

							sum = sum.add(BigDecimal.ONE);
						}

					} else if (column.getColumnType() == ColumnType.STRING || column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

						value = value.replace(',', '.');

						try {
							sum = sum.add(new BigDecimal(value));

						} catch (NumberFormatException ignore) {

							sum = null;
							break;
						}
					}
				}
			}

			column.setSum(sum);
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getColumnSum(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer queryID = NumberUtils.toInt(req.getParameter("queryID"));

		if (ObjectUtils.isNull(queryID)) {

			log.warn("User " + user + " attempted to get columnsum with null queryID " + queryID);

			JsonObject jsonNode = new JsonObject(1);
			jsonNode.putField("Sum", "?");

			HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
			return null;
		}

		Query query = getQuery(queryID);

		if (query == null) {

			log.warn("User " + user + " attempted to get columnsum with invalid queryID: queryID " + queryID);

			JsonObject jsonNode = new JsonObject(1);
			jsonNode.putField("Sum", "?");

			HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
			return null;
		}

		log.info("User " + user + " requesting columnsum for queryID " + queryID);

		BigDecimal sum = new BigDecimal(0);

		String[] values = req.getParameterValues("value");
		String[] checkboxes = req.getParameterValues("checkbox");

		if (!ArrayUtils.isEmpty(values)) {

			for (String value : values) {

				if (!StringUtils.isEmpty(value)) {

					value = value.replace(',', '.');

					try {
						sum = sum.add(new BigDecimal(value));

					} catch (NumberFormatException ignore) {

						sum = null;
						break;
					}
				}
			}
		}

		if (!ArrayUtils.isEmpty(checkboxes)) {

			for (String value : checkboxes) {

				if (!StringUtils.isEmpty(value)) {

					if ("true".equals(value)) {

						sum = sum.add(BigDecimal.ONE);
					}
				}
			}
		}

		JsonObject jsonNode = new JsonObject(1);

		if (sum == null) {

			jsonNode.putField("Sum", "?");

		} else {

			Integer decimals = query.getDecimalsToShow();

			if (decimals != null) {

				jsonNode.putField("Sum", new NumberStringyfier(decimals, decimals, false).format(sum));

			} else if (sum.scale() > 0) {

				jsonNode.putField("Sum", BaseTableColumn.NUMBER_STRINGIFYER_WITH_DECIMALS.format(sum));

			} else {

				jsonNode.putField("Sum", BaseTableColumn.NUMBER_STRINGIFYER_WITHOUT_DECIMALS.format(sum));
			}
		}

		HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
		return null;
	}

	protected void resetQueryInstanceRowSums(QueryInstance queryInstance) {

		List<Column> columns = queryInstance.getQuery().getColumns();
		List<CellValue> values = queryInstance.getValues();

		if (CollectionUtils.isEmpty(columns) || CollectionUtils.isEmpty(values)) {
			return;
		}

		for (Column column : columns) {

			if (column.getColumnType() == ColumnType.ROWSUM_ADD || column.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

				Iterator<CellValue> it = values.iterator();

				while (it.hasNext()) {
					CellValue cell = it.next();

					if (cell.getColumn().getColumnID().equals(column.getColumnID())) {
						it.remove();
					}
				}
			}
		}
	}

	protected void calculateQueryInstanceRowSums(QueryInstance queryInstance, List<Row> rows) {

		List<Column> columns = queryInstance.getQuery().getColumns();

		if (CollectionUtils.isEmpty(columns)) {
			return;
		}

		if (CollectionUtils.isEmpty(rows) || CollectionUtils.isEmpty(queryInstance.getValues())) {

			resetQueryInstanceRowSums(queryInstance);
			return;
		}

		for (Column rowSumColumn : columns) {

			if (rowSumColumn.getColumnType() == ColumnType.ROWSUM_ADD || rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

				for (Row row : rows) {

					BigDecimal sum = BigDecimal.ZERO;
					BigDecimal mulAddSum = null;
					BigDecimal mulSubSum = null;

					for (Column column : columns) {

						if (CollectionUtils.contains(rowSumColumn.getRowSumAddedColumns(), column.getColumnID()) || CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {

							BigDecimal parsedValue = null;
							String value = queryInstance.getValue(row.getRowIdentifier(), column.getColumnID());

							if (!StringUtils.isEmpty(value)) {

								ColumnType columnType = column.getColumnType();

								if (columnType == ColumnType.CHECKBOX) {

									if ("true".equals(value)) {

										parsedValue = BigDecimal.ONE;
									}

								} else if (columnType == ColumnType.STRING || columnType == ColumnType.ROWSUM_ADD || columnType == ColumnType.ROWSUM_MULTIPLY) {

									value = value.replace(',', '.');

									try {
										parsedValue = new BigDecimal(value);

									} catch (NumberFormatException ignore) {

										sum = null;
										break;
									}
								}
							}

							if (parsedValue != null) {

								if (rowSumColumn.getColumnType() == ColumnType.ROWSUM_ADD) {

									if (CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {
										parsedValue = parsedValue.negate();
									}

									sum = sum.add(parsedValue);

								} else if (rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

									if (CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {

										if (mulSubSum == null) { // first value, multiply by 1
											mulSubSum = BigDecimal.ONE;
										}

										mulSubSum = mulSubSum.multiply(parsedValue);

									} else {

										if (mulAddSum == null) { // first value, multiply by 1
											mulAddSum = BigDecimal.ONE;
										}

										mulAddSum = mulAddSum.multiply(parsedValue);
									}
								}
							}
						}
					}

					if (sum != null && rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

						if (mulAddSum != null) {
							sum = sum.add(mulAddSum);
						}

						if (mulSubSum != null) {
							sum = sum.subtract(mulSubSum);
						}
					}

					String formattedValue = null;

					if (sum != null) {

						Integer decimals = queryInstance.getQuery().getDecimalsToShow();

						if (decimals != null) {

							formattedValue = new NumberStringyfier(decimals, decimals, false).format(sum);

						} else if (sum.scale() > 0) {

							formattedValue = BaseTableColumn.NUMBER_STRINGIFYER_WITH_DECIMALS.format(sum);

						} else {

							formattedValue = BaseTableColumn.NUMBER_STRINGIFYER_WITHOUT_DECIMALS.format(sum);
						}
					}

					if (formattedValue != null) {

						queryInstance.setValue(getNewCellValue(row, rowSumColumn, formattedValue));
					}
				}
			}
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getRowSum(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer queryID = NumberUtils.toInt(req.getParameter("queryID"));
		Integer columnID = NumberUtils.toInt(req.getParameter("columnID"));
		String rowID = req.getParameter("rowID");

		if (ObjectUtils.isNull(columnID, rowID, queryID)) {

			log.warn("User " + user + " attempted to get rowsum with null values: queryID " + queryID + ", columnID " + columnID + ", rowID " + rowID);

			JsonObject jsonNode = new JsonObject(1);
			jsonNode.putField("Sum", "?");

			HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
			return null;
		}

		Query query = getQuery(queryID);

		if (query == null) {

			log.warn("User " + user + " attempted to get rowsum with invalid queryID: queryID " + queryID + ", columnID " + columnID + ", rowID " + rowID);

			JsonObject jsonNode = new JsonObject(1);
			jsonNode.putField("Sum", "?");

			HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
			return null;
		}

		log.info("User " + user + " requesting rowsum for queryID " + queryID + ", columnID " + columnID + ", rowID " + rowID);

		BigDecimal sum = null;

		if (!CollectionUtils.isEmpty(query.getColumns())) {

			Column rowSumColumn = null;

			for (Column column : query.getColumns()) {
				if (column.getColumnID().equals(columnID)) {
					rowSumColumn = column;
					break;
				}
			}

			if (rowSumColumn != null && (rowSumColumn.getColumnType() == ColumnType.ROWSUM_ADD || rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) && (rowSumColumn.getRowSumAddedColumns() != null || rowSumColumn.getRowSumSubtractedColumns() != null)) {

				sum = BigDecimal.ZERO;
				BigDecimal mulAddSum = null;
				BigDecimal mulSubSum = null;

				for (Column column : query.getColumns()) {
					if (CollectionUtils.contains(rowSumColumn.getRowSumAddedColumns(), column.getColumnID()) || CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {

						String value = req.getParameter("q" + queryID + "cell_" + rowID + "_" + column.getColumnID());

						if (!StringUtils.isEmpty(value)) {

							ColumnType columnType = column.getColumnType();
							BigDecimal parsedValue = null;

							if (columnType == ColumnType.CHECKBOX) {

								if ("true".equals(value)) {

									parsedValue = BigDecimal.ONE;
								}

							} else if (columnType == ColumnType.STRING || columnType == ColumnType.ROWSUM_ADD || columnType == ColumnType.ROWSUM_MULTIPLY) {

								value = value.replace(',', '.');

								try {
									parsedValue = new BigDecimal(value);

								} catch (NumberFormatException ignore) {

									sum = null;
									break;
								}
							}

							if (parsedValue != null) {

								if (rowSumColumn.getColumnType() == ColumnType.ROWSUM_ADD) {

									if (CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {
										parsedValue = parsedValue.negate();
									}

									sum = sum.add(parsedValue);

								} else if (rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {

									if (CollectionUtils.contains(rowSumColumn.getRowSumSubtractedColumns(), column.getColumnID())) {

										if (mulSubSum == null) { // first value, multiply by 1
											mulSubSum = BigDecimal.ONE;
										}

										mulSubSum = mulSubSum.multiply(parsedValue);

									} else {

										if (mulAddSum == null) { // first value, multiply by 1
											mulAddSum = BigDecimal.ONE;
										}

										mulAddSum = mulAddSum.multiply(parsedValue);
									}
								}
							}
						}
					}
				}

				if (sum != null && rowSumColumn.getColumnType() == ColumnType.ROWSUM_MULTIPLY) {
					if (mulAddSum != null) {
						sum = sum.add(mulAddSum);
					}

					if (mulSubSum != null) {
						sum = sum.subtract(mulSubSum);
					}
				}
			}
		}

		JsonObject jsonNode = new JsonObject(1);

		if (sum == null) {

			jsonNode.putField("Sum", "?");

		} else {

			Integer decimals = query.getDecimalsToShow();

			if (decimals != null) {

				jsonNode.putField("Sum", new NumberStringyfier(decimals, decimals, false).format(sum));

			} else if (sum.scale() > 0) {

				jsonNode.putField("Sum", BaseTableColumn.NUMBER_STRINGIFYER_WITH_DECIMALS.format(sum));

			} else {

				jsonNode.putField("Sum", BaseTableColumn.NUMBER_STRINGIFYER_WITHOUT_DECIMALS.format(sum));
			}
		}

		HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
		return null;
	}

	private FormatValidator getStringFormatValidator(Column column) {

		if (formatValidationHandler != null) {

			return formatValidationHandler.getFormatValidator(column.getFormatValidator());
		}

		return null;
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "updatequery")
	public ForegroundModuleResponse updateQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "addcolumn")
	public ForegroundModuleResponse addColumn(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Query query = this.queryCRUD.getRequestedBean(req, res, user, uriParser, null);

		if (query != null) {

			this.checkUpdateQueryAccess(user, query);

			req.setAttribute("TableQuery", query);

			return columnCRUD.add(req, res, user, uriParser);

		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "updatecolumn")
	public ForegroundModuleResponse updateColumn(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return columnCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "deletecolumn")
	public ForegroundModuleResponse deleteColumn(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return columnCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "sortcolumns")
	public ForegroundModuleResponse sortColumns(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return queryCRUD.sortColumns(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		Query query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		QueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	public void checkUpdateQueryAccess(User user, Query query) throws AccessDeniedException, SQLException {

		flowAdminModule.checkFlowStructureManipulationAccess(user, (Flow) query.getQueryDescriptor().getStep().getFlow());

	}

	public void redirectToQueryConfig(Query query, HttpServletRequest req, HttpServletResponse res) throws IOException {

		res.sendRedirect(req.getContextPath() + this.getFullAlias() + "/config/" + query.getQueryID());

	}

	public List<FormatValidator> getFormatValidators() {

		if (formatValidationHandler != null) {

			return formatValidationHandler.getFormatValidators();
		}

		return null;
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, QueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	public String getColumnTypeName(ColumnType type) {

		if (type.equals(ColumnType.CHECKBOX)) {
			return columnTypeCheckbox;
		}

		if (type.equals(ColumnType.STRING)) {
			return columnTypeString;
		}

		if (type.equals(ColumnType.TEXTAREA)) {
			return columnTypeTextArea;
		}

		if (type.equals(ColumnType.DROPDOWN)) {
			return columnTypeDropdown;
		}

		if (type.equals(ColumnType.DROPDOWN_ALTERNATIVE_LIST)) {
			return columnTypeDropdownAlternativeList;
		}

		if (type.equals(ColumnType.ROWSUM_ADD)) {
			return columnTypeRowSumAdd;
		}

		if (type.equals(ColumnType.ROWSUM_MULTIPLY)) {
			return columnTypeRowSumMultiply;
		}

		if (type.equals(ColumnType.LIST_SEARCH)) {
			return columnTypeListSearch;
		}

		return type.toString();

	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		XMLUtils.appendNewElement(doc, (Element) doc.getFirstChild(), "ModuleURL", RequestUtils.getFullContextPathURL(req) + getFullAlias());

		return doc;
	}

	public String getMissingAlternativeList() {

		return missingList;
	}

	protected Collection<AlternativeListDescriptor> getAlternativeLists() {

		return alternativeListHandlerModule.getAlternativeLists();
	}

	protected AlternativeListProvider getAlternativeListProvider(String providerID) {

		return alternativeListHandlerModule.getProvider(providerID);
	}

	protected AlternativeListDescriptor getAlternativeListDescriptor(Column column) {

		if (!StringUtils.isEmpty(column.getAlternativeListName())) {

			return alternativeListHandlerModule.getAlternativeList(column.getAlternativeListName());
		}

		return null;
	}

	protected AlternativeListInstance getAlternativeListInstance(Column column, AttributeHandler attributeHandler, User poster) {

		AlternativeListDescriptor descriptor = getAlternativeListDescriptor(column);

		if (descriptor != null) {

			return alternativeListHandlerModule.getListInstance(descriptor, attributeHandler, poster);
		}

		return null;
	}

	@Override
	public Collection<? extends AlternativeListQuery> getQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) {

		try {

			HighLevelQuery<Column> getQuery = new HighLevelQuery<>(getColumnQueryRelation(), QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);

			getQuery.addParameter(columnListNameParamFactory.getParameter(descriptor.getName()));

			List<Column> columns = columnDAO.getAll(getQuery);

			if (columns == null) {

				return null;
			}

			Map<Integer, Query> queryIDToQueryMap = new HashMap<>();

			for (Column column : columns) {

				Query query = queryIDToQueryMap.computeIfAbsent(column.getQuery().getQueryID(), id -> column.getQuery());

				query.setColumns(CollectionUtils.addAndInstantiateIfNeeded(query.getColumns(), column));
			}

			if (!CollectionUtils.isEmpty(queryIDToQueryMap)) {

				for (Query query : queryIDToQueryMap.values()) {

					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(query.getQueryID());

					if (queryDescriptor != null) {

						query.init(queryDescriptor, getFullAlias() + "/config/" + query.getQueryID());
					}
				}

				return queryIDToQueryMap.values();
			}

		} catch (SQLException e) {

			log.error("Unable to get queries using alternative list " + descriptor, e);
		}

		return null;
	}

	@Override
	public boolean hasQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) throws SQLException {

		HighLevelQuery<Column> query = new HighLevelQuery<>();

		query.addParameter(columnListNameParamFactory.getParameter(descriptor.getName()));
		query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);

		return columnDAO.getBoolean(query);
	}

	protected abstract Field getColumnQueryRelation();

	protected String getEventColumnAddedMessage() {

		return eventColumnAddedMessage;
	}

	protected String getEventColumnUpdatedMessage() {

		return eventColumnUpdatedMessage;
	}

	protected String getEventColumnDeletedMessage() {

		return eventColumnDeletedMessage;
	}

	protected List<Column> getQueryColumns(Query query) throws SQLException {

		HighLevelQuery<Column> getQuery = new HighLevelQuery<>();
		getQuery.addParameter(columnQueryParamFactory.getParameter(query));

		return columnDAO.getAll(getQuery);
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(QueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws Exception {

		return new TableQueryRequestProcessor<DropAlternative, Column, Query, Row, CellValue, QueryInstance, QueryProvider>(queryInstance, maxHits, showingPartialList1, showingPartialList2, showingPartialList3);
	}

}
