<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-ui.js?v=1
		/timepicker/js/jquery.timepicker.js?v=1
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
		/timepicker/css/jquery.timepicker.css
	</xsl:variable>

<!-- 	<xsl:variable name="scripts"> -->
<!-- 		/js/jquery.ui.datepicker-sv.js -->
<!-- 		/js/tablequery.js -->
<!-- 	</xsl:variable> -->
	
<!-- 	<xsl:variable name="links"> -->
<!-- 		/css/tablequery.css -->
<!-- 	</xsl:variable> -->
	
	<xsl:template match="Column" mode="thead">
	
		<xsl:variable name="class">
			<xsl:value-of select="'paddingright'"/>
			
			<xsl:if test="required = 'true'">
				<xsl:value-of select="' required'"/>
			</xsl:if>
		</xsl:variable>
	
		<th class="{$class}" data-type="{columnType}">
		
			<xsl:if test="width">
				<xsl:attribute name="width">
					<xsl:value-of select="width"/>
					<xsl:value-of select="'%'"/>
				</xsl:attribute>
			</xsl:if>
			
			<xsl:value-of select="label"/>
			
			<xsl:if test="helpText">
				<i data-icon-after="?" title="{helpText}"/>
				
				<div class="help-box" data-help-box="tablequery_{columnID}">
					<div>
						<div> 
							<a class="close" href="#" data-icon-after="x"></a> 
							<xsl:value-of select="helpText" disable-output-escaping="yes" />
						</div>
					</div>
				</div>
			</xsl:if>
			
		</th>
	
	</xsl:template>
	
	<xsl:template match="Column" mode="thead-show">
	
		<th class="paddingright">
		
			<xsl:if test="width">
				<xsl:attribute name="width">
					<xsl:value-of select="width"/>
					<xsl:value-of select="'%'"/>
				</xsl:attribute>
			</xsl:if>
			
			<xsl:value-of select="label"/>
			
		</th>
	
	</xsl:template>
	
	<xsl:template match="Column" mode="tbody">
	
		<xsl:param name="rowID" select="null"/>
		<xsl:variable name="columnID" select="columnID"/>
		
		<xsl:variable name="cellID">
			<xsl:value-of select="'cell_'"/>
			<xsl:value-of select="$rowID"/>
			<xsl:value-of select="'_'"/>
			<xsl:value-of select="columnID"/>
		</xsl:variable>
		
		<xsl:variable name="name">
			<xsl:value-of select="'q'"/>
			<xsl:value-of select="../../queryID"/>
			<xsl:value-of select="$cellID"/>
		</xsl:variable>
		
		<xsl:variable name="required">
			<xsl:if test="required = 'true' and (not(../../Rows) or ../../Rows/Row[rowID = $rowID]/required = 'true')"><xsl:text>true</xsl:text></xsl:if>
		</xsl:variable>
		
		<xsl:variable name="title">
			<xsl:value-of select="label"/>
			<xsl:if test="$required = 'true'"> *</xsl:if>
		</xsl:variable>
		
		<td data-title="{$title}" data-column-id="{$columnID}">
			<xsl:attribute name="class">
				<xsl:text>paddingright</xsl:text>
				
				<xsl:if test="$required = 'true'">
					<xsl:text> required</xsl:text>
				</xsl:if>
				
				<xsl:if test="columnType='CHECKBOX'">
					<xsl:value-of select="' text-align-center'"/>
				</xsl:if>
				
				<xsl:if test="columnType='ROWSUM_ADD' or columnType='ROWSUM_MULTIPLY'">
					<xsl:value-of select="' rowsum'"/>
				</xsl:if>
				
				<xsl:if test="helpText"> mobilehelp</xsl:if>
			</xsl:attribute>
			
			<xsl:if test="helpText">
				<div class="mobilehelp floatleft only-mobile">
					<div class="help">
						<a class="open-help" href="#" data-icon-after="?" data-help-box="tablequery_{$columnID}" title="{$i18n.Help}" />
					</div>
					
					<div class="help-backdrop" data-help-box="tablequery_{$columnID}" />
				</div>
			</xsl:if>
			
			<div>
				<xsl:attribute name="class">
					<xsl:text>full</xsl:text>
					<xsl:if test="../../../../ValidationErrors/validationError[fieldName = $cellID]">
						<xsl:text> invalid input-error</xsl:text>
					</xsl:if>
				</xsl:attribute>
				
				<xsl:choose>
	
					<xsl:when test="columnType='STRING'">
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$name"/>
							<xsl:with-param name="name" select="$name"/>
							<xsl:with-param name="value" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
							<xsl:with-param name="class">
								<xsl:if test="summarise = 'true'">
									<xsl:text> text-align-right</xsl:text>
								</xsl:if>
							</xsl:with-param>
							<xsl:with-param name="maxlength">
								<xsl:if test="maxContentLength">
									<xsl:value-of select="maxContentLength"/>
								</xsl:if>
							</xsl:with-param>
							<xsl:with-param name="type">
								<xsl:choose>
									<xsl:when test="formatValidator = 'se.unlogic.standardutils.populators.DatePopulator'">
										<xsl:text>date</xsl:text>
									</xsl:when>
									<xsl:when test="formatValidator = 'se.unlogic.standardutils.populators.TimePopulator'">
										<xsl:text>time</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text>text</xsl:text>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:with-param>
							<xsl:with-param name="aria-label"><xsl:value-of select="label" /></xsl:with-param>
							<xsl:with-param name="aria-required"><xsl:if test="$required = 'true'">true</xsl:if></xsl:with-param>
							<xsl:with-param name="aria-describedby">
								<xsl:value-of select="$cellID" />
								<xsl:value-of select="'_error'" />
							</xsl:with-param>
						</xsl:call-template>
						
					</xsl:when>
	
					<xsl:when test="columnType='TEXTAREA'">
						
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="$name"/>
							<xsl:with-param name="name" select="$name"/>
							<xsl:with-param name="rows">
								<xsl:choose>
									<xsl:when test="textAreaRows">
										<xsl:value-of select="textAreaRows"/>
									</xsl:when>
									<xsl:otherwise>3</xsl:otherwise>
								</xsl:choose>
							</xsl:with-param>
							<xsl:with-param name="value" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
							<xsl:with-param name="class">
								<xsl:if test="summarise = 'true'">
									<xsl:text> text-align-right</xsl:text>
								</xsl:if>
							</xsl:with-param>
							<xsl:with-param name="maxlength">
								<xsl:if test="maxContentLength">
									<xsl:value-of select="maxContentLength"/>
								</xsl:if>
							</xsl:with-param>
							<xsl:with-param name="aria-label"><xsl:value-of select="label" /></xsl:with-param>
							<xsl:with-param name="aria-required"><xsl:if test="$required = 'true'">true</xsl:if></xsl:with-param>
							<xsl:with-param name="aria-describedby">
								<xsl:value-of select="$cellID" />
								<xsl:value-of select="'_error'" />
							</xsl:with-param>
						</xsl:call-template>
						
					</xsl:when>
					
					<xsl:when test="columnType='CHECKBOX'">
					
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="$name"/>
							<xsl:with-param name="name" select="$name"/>
							<xsl:with-param name="value" select="'true'" />
							<xsl:with-param name="checked" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
						</xsl:call-template>
						
						<label for="{$name}" class="checkbox">&#160;</label>
						
					</xsl:when>
					
					<xsl:when test="columnType='DROPDOWN'">
					
						<select id="{$name}" name="{$name}" aria-label="{label}" aria-describedby="{$cellID}_error">
						
							<xsl:if test="$required = 'true'"><xsl:attribute name="aria-required">true</xsl:attribute></xsl:if>
						
							<xsl:if test="dropdownChooseText">
								<option value="">
									<xsl:value-of select="dropdownChooseText"/>
								</option>
							</xsl:if>
							
							<xsl:for-each select="DropDownAlternatives/DropDownAlternative">
								<option value="{alternativeID}">
									<xsl:choose>
										<xsl:when test="../../../../../../requestparameters">
											<xsl:if test="../../../../../../requestparameters/parameter[name = $name]/value = alternativeID">
												<xsl:attribute name="selected"/>
											</xsl:if>
										</xsl:when>
										<xsl:otherwise>
											<xsl:if test="../../../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value = alternativeID">
												<xsl:attribute name="selected" />
											</xsl:if>
										</xsl:otherwise>
									</xsl:choose>
									
									<xsl:value-of select="name" />
									
								</option>
							</xsl:for-each>
							
						</select>
							
					</xsl:when>
					
					<xsl:when test="columnType='DROPDOWN_ALTERNATIVE_LIST'">
					
						<select id="{$name}" name="{$name}" aria-label="{label}" aria-describedby="{$cellID}_error">
						
							<xsl:if test="$required = 'true'"><xsl:attribute name="aria-required">true</xsl:attribute></xsl:if>
						
							<xsl:if test="dropdownChooseText">
								<option value="">
									<xsl:value-of select="dropdownChooseText"/>
								</option>
							</xsl:if>
							
							<xsl:for-each select="AlternativeListAlternatives/Alternative">
								<option value="{alternativeID}">
									<xsl:choose>
										<xsl:when test="../../../../../../requestparameters">
											<xsl:if test="../../../../../../requestparameters/parameter[name = $name]/value = alternativeID">
												<xsl:attribute name="selected"/>
											</xsl:if>
										</xsl:when>
										<xsl:otherwise>
											<xsl:if test="../../../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value = alternativeID">
												<xsl:attribute name="selected" />
											</xsl:if>
										</xsl:otherwise>
									</xsl:choose>
									
									<xsl:value-of select="name" />
									
								</option>
							</xsl:for-each>
							
						</select>
							
					</xsl:when>
					
					<xsl:when test="columnType='ROWSUM_ADD' or columnType='ROWSUM_MULTIPLY'">
						
						<span id="{$cellID}" data-columnid="{$columnID}" data-rowid="{$rowID}">
							<xsl:attribute name="class">
								<xsl:text>rowsum</xsl:text>
								<xsl:if test="summarise = 'true'"> text-align-right display-block</xsl:if>
							</xsl:attribute>
							
							<xsl:choose>
								<xsl:when test="../../../../requestparameters">
									<xsl:value-of select="../../../../requestparameters/parameter[name=$name]/value" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value" />
								</xsl:otherwise>
							</xsl:choose>
						</span>
						
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="name" select="$name"/>
							<xsl:with-param name="value" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
						</xsl:call-template>
						
					</xsl:when>					
					
					<xsl:when test="columnType='LIST_SEARCH'">
					
						<xsl:variable name="alternativeID" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]/value" />	
						<xsl:variable name="alternative" select="AlternativeListAlternatives/Alternative[alternativeID = $alternativeID]" />	
					
						<xsl:variable name="listSearchClass">
							<xsl:if test="showAlternativesOnEmptySearch = 'true'"> allowemptysearch</xsl:if>
						</xsl:variable>	
					
						<xsl:call-template name="createTextField">
							<xsl:with-param name="title" select="label"/>
							<xsl:with-param name="name" select="concat($name, '-name')"/>
							<xsl:with-param name="value" select="$alternative/name"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
							<xsl:with-param name="class" select="concat('list-search ', $listSearchClass)"/>
							<xsl:with-param name="placeholder" select="placeholderText" />
						</xsl:call-template>
						
						<i class="list-search-delete icon visibility-hidden">x</i>

						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="name" select="$name"/>
							<xsl:with-param name="value" select="$alternativeID"/>
							<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
							<xsl:with-param name="class" select="'hidden-list-search'"/>
						</xsl:call-template>
						
					</xsl:when>					
					
					<xsl:otherwise>Unknown columnType</xsl:otherwise>
				</xsl:choose>
				
				<xsl:apply-templates select="../../../../ValidationErrors/validationError[fieldName = $cellID]">
					<xsl:with-param name="id"><xsl:value-of select="$cellID" />_error</xsl:with-param>
				</xsl:apply-templates>
				
			</div>
			
		</td>
	
	</xsl:template>
	
	<xsl:template match="Column" mode="tbody-show">
	
		<xsl:param name="rowID" select="null"/>
		<xsl:variable name="columnID" select="columnID"/>
		
		<td data-title="{label}">
			<xsl:attribute name="class">
				<xsl:text>padding</xsl:text>
				
				<xsl:choose>
					<xsl:when test="summarise = 'true'">
						<xsl:text> text-align-right bigpaddingright</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<!-- <xsl:text> text-align-center</xsl:text> -->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		
			<xsl:variable name="cell" select="../../../Values/CellValue[Row/rowID = $rowID and Column/columnID = $columnID]"/>
			<xsl:variable name="value" select="$cell/value"/>
			<xsl:variable name="displayValue" select="$cell/displayValue"/>
			
			<xsl:choose>
			
				<xsl:when test="columnType='CHECKBOX'">
				
					<xsl:choose>
						<xsl:when test="$value='true'">
							<xsl:value-of select="$i18n.True"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.False"/>
						</xsl:otherwise>
					</xsl:choose>
				
				</xsl:when>
				
				<xsl:when test="columnType='DROPDOWN'">
				
					<xsl:value-of select="DropDownAlternatives/DropDownAlternative[alternativeID = $value]/name"/>
				
				</xsl:when>
				
				<xsl:when test="columnType='DROPDOWN_ALTERNATIVE_LIST' or columnType='LIST_SEARCH'">
				
					<xsl:value-of select="$displayValue"/>
				
				</xsl:when>
				
				<xsl:otherwise>
				
					<xsl:choose>
						<xsl:when test="$value">
							<xsl:value-of select="$value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>-</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
					
				</xsl:otherwise>
				
			</xsl:choose>
			
		</td>
	
	</xsl:template>
	
	<!-- Errors shown on Query -->
	<xsl:template name="fieldValidationErrorsMessage">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.FieldValidationErrors"/>
			</strong>
		</span>
		
	</xsl:template>
		
	<!-- Errors shown on fields -->
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!" title="{$i18n.RequiredField}">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
		</i>
		
	</xsl:template>	
	
	<xsl:template match="validationError[validationErrorType = 'TooShort']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.TooShortFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.TooShortFieldContent.part2"/>
				<xsl:value-of select="minLength"/>
				<xsl:value-of select="$i18n.TooShortFieldContent.part3"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.TooLongFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part2"/>
				<xsl:value-of select="maxLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part3"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="invalidFormatMessage">
						<xsl:value-of select="invalidFormatMessage"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.InvalidFormat"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</i>
		
	</xsl:template>
		
	<xsl:template match="validationError">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!" title="{$i18n.UnknownValidationError}">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
		</i>
		
	</xsl:template>
	
</xsl:stylesheet>