package com.nordicpeak.flowengine.biznodevehiclealternativelist;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeList;
import com.nordicpeak.flowengine.biznodevehiclealternativelist.biznoderesponse.VehicleType;

@Table(name = "biznode_alternative_list_sources")
@XMLElement(name = "AlternativeList")
public class ListSource extends BaseAlternativeList<ListSource, ListSourceEvent> implements Serializable {

	public static final Field EVENTS_RELATION = ReflectionUtils.getField(ListSource.class, "events");

	private static final long serialVersionUID = 3296400205039536096L;

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement(name = "listID")
	private Integer sourceID;

	@DAOManaged
	@OrderBy
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType1; // Personbil

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType2; // Lastbil

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType3; // Buss

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType4; // Slpvagn

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType5; // Motorcykel

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType6; // Traktor

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType7; // Motorredskap

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowVehicleType13; // Moped

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowTerrainVehicle; // Terrngmotorfordon (7, 9, 10, 14, 15)

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ListSourceEvent> events;

	public Integer getSourceID() {
		return sourceID;
	}

	public void setSourceID(Integer sourceID) {
		this.sourceID = sourceID;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String toString() {

		return StringUtils.toLogFormat(name, 30) + " (ID: " + sourceID + ")";
	}

	@Override
	public String getListName() {
		return name;
	}

	@Override
	public Integer getListID() {
		return sourceID;
	}

	@Override
	public List<ListSourceEvent> getEvents() {
		return events;
	}

	@Override
	public void setEvents(List<ListSourceEvent> events) {
		this.events = events;
	}

	public boolean isAllowVehicleType1() {
		return allowVehicleType1;
	}

	public void setAllowVehicleType1(boolean allowVehicleType1) {
		this.allowVehicleType1 = allowVehicleType1;
	}

	public boolean isAllowVehicleType2() {
		return allowVehicleType2;
	}

	public void setAllowVehicleType2(boolean allowVehicleType2) {
		this.allowVehicleType2 = allowVehicleType2;
	}

	public boolean isAllowVehicleType3() {
		return allowVehicleType3;
	}

	public void setAllowVehicleType3(boolean allowVehicleType3) {
		this.allowVehicleType3 = allowVehicleType3;
	}

	public boolean isAllowVehicleType4() {
		return allowVehicleType4;
	}

	public void setAllowVehicleType4(boolean allowVehicleType4) {
		this.allowVehicleType4 = allowVehicleType4;
	}

	public boolean isAllowVehicleType5() {
		return allowVehicleType5;
	}

	public void setAllowVehicleType5(boolean allowVehicleType5) {
		this.allowVehicleType5 = allowVehicleType5;
	}

	public boolean isAllowVehicleType6() {
		return allowVehicleType6;
	}

	public void setAllowVehicleType6(boolean allowVehicleType6) {
		this.allowVehicleType6 = allowVehicleType6;
	}

	public boolean isAllowVehicleType7() {
		return allowVehicleType7;
	}

	public void setAllowVehicleType7(boolean allowVehicleType7) {
		this.allowVehicleType7 = allowVehicleType7;
	}

	public boolean isAllowVehicleType13() {
		return allowVehicleType13;
	}

	public void setAllowVehicleType13(boolean allowVehicleType13) {
		this.allowVehicleType13 = allowVehicleType13;
	}

	public boolean isAllowTerrainVehicle() {
		return allowTerrainVehicle;
	}

	public void setAllowTerrainVehicle(boolean allowTerrainVehicle) {
		this.allowTerrainVehicle = allowTerrainVehicle;
	}

	public boolean allowsVehicle(VehicleType type) {
		
		if(allowVehicleType1 && type.equals(VehicleType.PB))
		{
			return true;
		}
		if(allowVehicleType2 && type.equals(VehicleType.LB))
		{
			return true;
		}
		if(allowVehicleType3 && type.equals(VehicleType.BUSS))
		{
			return true;
		}
		if(allowVehicleType4 && type.equals(VehicleType.SLAP))
		{
			return true;
		}
		if(allowVehicleType5 && type.equals(VehicleType.MC))
		{
			return true;
		}
		if(allowVehicleType6 && type.equals(VehicleType.TR))
		{
			return true;
		}
		if(allowVehicleType7 && type.equals(VehicleType.MRED))
		{
			return true;
		}
		if(allowVehicleType13 && type.equals(VehicleType.MOPED))
		{
			return true;
		}
		
		return allowTerrainVehicle && (type.equals(VehicleType.TGV) || type.equals(VehicleType.TGSK) 
				|| type.equals(VehicleType.TGSL)|| type.equals(VehicleType.TGSNO)
						|| type.equals(VehicleType.TGHJUL));
		
	}

}
