<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:template match="BookingEvent">
		
		<xsl:variable name="odd" select="(position() mod 2) != 0" />
		
		<tr>
			<xsl:if test="$odd">
				<xsl:attribute name="class">odd</xsl:attribute>
			</xsl:if>

			<td data-title="{$i18n.Action}" class="service">
				<xsl:call-template name="getBookingEventTypeText" />
			</td>
			
			<td data-title="{$i18n.Details}">
				<xsl:choose>
					<xsl:when test="details"><xsl:value-of select="details" /></xsl:when>
					<xsl:when test="eventType = 'CUSTOMER_MESSAGE_SENT'"><xsl:value-of select="$i18n.eventCustomerMessageSent"/></xsl:when>
					<xsl:when test="eventType = 'MANAGER_MESSAGE_SENT'"><xsl:value-of select="$i18n.eventManagerMessageSent"/></xsl:when>
					<xsl:otherwise>-</xsl:otherwise>
				</xsl:choose>
			</td>
			
			<td data-title="{$i18n.Date}" class="date"><xsl:value-of select="added" /></td>
			
			<td data-title="{$i18n.Person}" class="person">
				<xsl:choose>
					<xsl:when test="not(poster) and ../../PrimaryContact/firstname">
					
						<xsl:value-of select="../../PrimaryContact/firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="../../PrimaryContact/lastname" />
						
					</xsl:when>
					<xsl:otherwise>
					
						<xsl:call-template name="printUser">
							<xsl:with-param name="user" select="poster" />
						</xsl:call-template>
						
					</xsl:otherwise>
				</xsl:choose>
				
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="getBookingEventTypeText">
		
		<xsl:choose>
			<xsl:when test="eventType = 'SUBMITTED'">
				<xsl:value-of select="$i18n.SubmittedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'PAYED'">
				<xsl:value-of select="$i18n.PayedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'CUSTOMER_MESSAGE_SENT'">
				<xsl:value-of select="$i18n.MessageEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'MANAGER_MESSAGE_SENT'">
				<xsl:value-of select="$i18n.MessageEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'CHANGED_SLOT'">
				<xsl:value-of select="$i18n.ChangedSlotEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'ARCHIVED'">
				<xsl:value-of select="$i18n.ArchivedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'SLOT_PASSED'">
				<xsl:value-of select="$i18n.SlotPassedEvent" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.OtherEvent" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template name="getExternalMessageTypeText">
	
		<xsl:if test="type != ''">
					
			<strong>
				<xsl:choose>
					<xsl:when test="type = 'ENTRANCE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ENTRANCE"/>
					</xsl:when>
					<xsl:when test="type = 'MANUAL_ENTRANCE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.MANUAL_ENTRANCE"/>
					</xsl:when>
					<xsl:when test="type = 'RESTORE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.RESTORE"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_BY_USER'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ARCHIVE_BY_USER"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_BY_MANAGER'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ARCHIVE_BY_MANAGER"/>
					</xsl:when>
				</xsl:choose>
				
				<xsl:text>:&#160;</xsl:text>
			</strong>
		
			<br/>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template name="getInternalMessageTypeText">
	
		<xsl:if test="type != ''">
					
			<strong>
				<xsl:choose>
					<xsl:when test="type = 'RESTORE'">
						<xsl:value-of select="$i18n.InternalMessage.Type.RESTORE"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_BY_MANAGER'">
						<xsl:value-of select="$i18n.InternalMessage.Type.ARCHIVE_BY_MANAGER"/>
					</xsl:when>
				</xsl:choose>
				
				<xsl:text>:&#160;</xsl:text>
			</strong>
		
			<br/>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template name="printBookingSlotTime">
		<xsl:param name="bookingSlot" select="BookingSlot"/>
		<xsl:param name="calendar" select="BookingSlot/Calendar"/>
		
		<xsl:value-of select="$bookingSlot/startTime" />
				
		<xsl:choose>
			<xsl:when test="$calendar/showDuration = 'true'">
				<xsl:text> </xsl:text>
				<xsl:value-of select="$bookingSlot/duration" />
				<xsl:text>min</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> - </xsl:text>
				<xsl:value-of select="$bookingSlot/endTime" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="user" mode="manager">
		
		<xsl:choose>
			<xsl:when test="Attributes/Attribute[Name = 'bookingManagerName']">
				<xsl:value-of select="Attributes/Attribute[Name = 'bookingManagerName']/Value" />
			</xsl:when>
			<xsl:otherwise>
		<xsl:value-of select="firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="lastname" />
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:if test="position() != last()">
			<xsl:text>, </xsl:text>
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="user" mode="manager-admin">
		
		<xsl:value-of select="firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="lastname" />
		
		<xsl:if test="Attributes/Attribute[Name = 'bookingManagerName']">
			<xsl:text> (</xsl:text>
				<xsl:value-of select="Attributes/Attribute[Name = 'bookingManagerName']/Value" />
			<xsl:text>)</xsl:text>
		</xsl:if>		
		
		<xsl:if test="position() != last()">
			<xsl:text>, </xsl:text>
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template name="printUser">
		<xsl:param name="user" />
		
		<xsl:choose>
			<xsl:when test="$user/firstname or $user/lastname">
				<xsl:value-of select="$user/firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="$user/lastname" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.AnonymousUser"/>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template name="printBookingUser">
		<xsl:param name="booking" select="." />
		
		<xsl:choose>
			<xsl:when test="$booking/Poster">
				<xsl:value-of select="$booking/Poster/user/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$booking/Poster/user/lastname" />
			</xsl:when>
			<xsl:when test="$booking/ManualContact/firstname or $booking/ManualContact/lastname">
				<xsl:value-of select="$booking/ManualContact/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$booking/ManualContact/lastname" />
			</xsl:when>
			<xsl:when test="$booking/PrimaryContact">
				<xsl:value-of select="$booking/PrimaryContact/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$booking/PrimaryContact/lastname" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.AnonymousUser"/>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="ViewFragment">
		
		<xsl:value-of select="HTML" disable-output-escaping="yes"/>
		
	</xsl:template>
	
	<xsl:template name="BookingSizeSelector">
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="bookingSlot" />
		<xsl:param name="disabled" select="null" />
		<xsl:param name="maxSize" />
		<xsl:param name="hideText" select="false()" />
		<xsl:param name="readOnly" select="false()" />
		<xsl:param name="requestparameters" select="requestparameters" />
		
		<xsl:variable name="maxSizeOrFreeSlots">
			<xsl:choose>
				<xsl:when test="not($maxSize) or $maxSize > $bookingSlot/freeSlots">
					<xsl:value-of select="$bookingSlot/freeSlots"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$maxSize" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
	
		<xsl:variable name="size">
			<xsl:choose>
				<xsl:when test="$value">
					<xsl:value-of select="$value" />
				</xsl:when>
				<xsl:otherwise>1</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
	
		<div>
		
			<xsl:choose>
				<xsl:when test="$readOnly">
				
					<strong>
						<xsl:value-of select="$i18n.BookingSizeSelector.Size" />
					</strong>
				
					<xsl:text>: </xsl:text>
					
					<xsl:value-of select="$size" />
				
				</xsl:when>
				<xsl:otherwise>
				
					<xsl:if test="not($hideText)">
					
						<strong>
							<xsl:value-of select="$i18n.BookingSizeSelector.SelectSize" />
						</strong>
						
						<div>
							<xsl:value-of select="$bookingSlot/freeSlots" />
							<xsl:text> </xsl:text>
							<xsl:choose>
								<xsl:when test="$bookingSlot/freeSlots = 1">
									<xsl:value-of select="$i18n.BookingSizeSelector.OneSlot" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$i18n.BookingSizeSelector.MultipleSlots" />
								</xsl:otherwise>
							</xsl:choose>
						</div>
						
					</xsl:if>
		
					<div class="booking-size-selector d-flex align-items-center" data-max-size="{$maxSizeOrFreeSlots}">
					
						<button type="button" class="decrease btn btn-light">
							<i data-icon-before="-"></i>
						</button>
						
						<span class="size">
						
							<xsl:choose>
								<xsl:when test="$requestparameters">
									<xsl:value-of select="$requestparameters/parameter[name=$name]/value" />
								</xsl:when>			
								<xsl:otherwise>
									<xsl:value-of select="$size" />
								</xsl:otherwise>
							</xsl:choose>	
										
						</span>
									
						<button type="button" class="increase btn btn-light">
							<i data-icon-before="+"></i>
						</button>
						
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="name" select="$name" />
							<xsl:with-param name="value" select="$size" />
							<xsl:with-param name="disabled" select="$disabled" />
							<xsl:with-param name="requestparameters" select="$requestparameters" />
						</xsl:call-template>
					</div>
					
				</xsl:otherwise>
			</xsl:choose>

		</div>
	
	</xsl:template>
	
</xsl:stylesheet>