<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/plugins/jquery.qloader.js
		/featherlight/js/featherlight.min.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>

	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/flowengine.js?v=1
		/js/flowengine.helpdialog.js
		/js/flowengine.tablesorter.js
		/js/userflowinstancemodule.js
		/js/jquery.qloader-init.js
		/js/userbooking.js
		/js/js.cookie-2.0.4.min.js
		/js/messages.js?v=3
	</xsl:variable>

	<xsl:variable name="links">
		/css/flowengine.css
		/css/messages.css
		/css/userbooking.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<div id="UserBookingModule" class="contentitem">
			<xsl:apply-templates select="ListUserBookings" />
			<xsl:apply-templates select="ShowBookingOverview" />
			<xsl:apply-templates select="UpdateBooking" />
		</div>
		
	</xsl:template>
		
	<xsl:template match="ListUserBookings">
	
		<xsl:apply-templates select="validationError"/>
		
		<section class="my-errands">
			
			<div class="errands-wrapper">
				<div class="heading-wrapper">
					<h2><xsl:value-of select="$i18n.UserBookings" /></h2>
					<xsl:call-template name="createHelpDialog">
						<xsl:with-param name="id" select="'bookings-list'" />
						<xsl:with-param name="text" select="$i18n.UserBookingsHelp" />
					</xsl:call-template>
				</div>
				<table class="oep-table errand-table">
					<xsl:call-template name="ListUserBookingsTableHead" />
					<tbody>
						<xsl:choose>
							<xsl:when test="Booking[not(archived = 'true') and not(BookingSlot/HasPassed)]">
								<xsl:apply-templates select="Booking[not(archived = 'true') and not(BookingSlot/HasPassed)]" mode="list" />
							</xsl:when>
							<xsl:otherwise>
								<tr><td class="icon" /><td colspan="4"><xsl:value-of select="$i18n.NoBookings" /></td></tr>
							</xsl:otherwise>
						</xsl:choose>
					</tbody>
				</table>
			</div>
			
			<xsl:if test="Booking[archived = 'true' or BookingSlot/HasPassed]">
				<div class="errands-wrapper">
					<div class="heading-wrapper">
						<h2><xsl:value-of select="$i18n.UserArchivedBookings" /></h2>
						<xsl:call-template name="createHelpDialog">
							<xsl:with-param name="id" select="'archivedbookings-list'" />
							<xsl:with-param name="text" select="$i18n.UserArchivedBookingsHelp" />
						</xsl:call-template>
					</div>
					<table class="oep-table errand-table">
						<xsl:call-template name="ListUserBookingsTableHead" />
						<tbody>
							<xsl:apply-templates select="Booking[archived = 'true' or BookingSlot/HasPassed]" mode="list" />
						</tbody>
					</table>
				</div>
			</xsl:if>
					
		</section>
		
	</xsl:template>
	
	<xsl:template name="ListUserBookingsTableHead">
	
		<thead class="sortable">
			<tr>
				<th class="icon no-sort"></th>
				<th class="service"><span data-icon-after="_"><xsl:value-of select="$i18n.Calendar.Name" /></span></th>
				<th class="status"><span data-icon-after="_"><xsl:value-of select="$i18n.BookingSlot.DateAndTime" /></span></th>
				<th><span data-icon-after="_"><xsl:value-of select="$i18n.Booking.Size" /></span></th>
				<th class="date default-sort"><span data-icon-after="_"><xsl:value-of select="$i18n.LastEvent" /></span></th>
				<th class="link no-sort"></th>	
			</tr>
		</thead>
		
	</xsl:template>
	
	<xsl:template match="Booking" mode="list">
		
		<tr>
			<td class="icon"/>
			<td data-title="{$i18n.Calendar.Name}" class="service"><xsl:value-of select="BookingSlot/Calendar/name" /></td>
			<td data-title="{$i18n.BookingSlot.DateAndTime}" class="status">
				<xsl:value-of select="BookingSlot/startDate" />
				<xsl:text> </xsl:text>
				<xsl:call-template name="printBookingSlotTime">
					<xsl:with-param name="bookingSlot" select="BookingSlot"/>
					<xsl:with-param name="calendar" select="BookingSlot/Calendar"/>
				</xsl:call-template>
			</td>
			<td data-title="{$i18n.Booking.Size}">
				<xsl:value-of select="size" />
			</td>
			<td data-title="{$i18n.LastEvent}" class="date">
				<xsl:value-of select="Events/BookingEvent[position() = last()]/added" />
			</td>
			<td class="link">
				<a class="btn btn-green" href="{/Document/ModuleURL}/overview/{bookingID}"><xsl:value-of select="$i18n.ShowBooking" /></a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="ShowBookingOverview">
	
		<section class="child">
			<div class="section-inside step full">
  				<div class="heading-wrapper">
  					<div class="inner inner-less-padding">
	  					<!--
	  					<figure>
		  					<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{Flow/flowID}" alt="" />
		  				</figure>
		  				-->
		  				<div class="heading">
	  						<h1 class="xl">
	  							<xsl:value-of select="Booking/BookingSlot/Calendar/name" />
	  						</h1>
						</div>
					</div>
  				</div>
  			</div>
  			<div class="section-inside header-full no-pad-top">
  				<div class="description">
  					
  					<div>
  					
  						<div>
							<strong>
								<xsl:value-of select="$i18n.BookingSlot.DateAndTime" />
								<xsl:text>:&#160;</xsl:text>
							</strong>
							<xsl:value-of select="Booking/BookingSlot/startDate" />
							<xsl:text> </xsl:text>
							<xsl:call-template name="printBookingSlotTime">
								<xsl:with-param name="bookingSlot" select="Booking/BookingSlot"/>
								<xsl:with-param name="calendar" select="Booking/BookingSlot/Calendar"/>
							</xsl:call-template>
  						</div>
						
						
						<xsl:if test="Booking/BookingSlot/location">
							<div>
								<strong>
									<xsl:value-of select="$i18n.BookingSlot.Location" />
									<xsl:text>:&#160;</xsl:text>
								</strong>
								<xsl:value-of select="Booking/BookingSlot/location" />
							</div>
						</xsl:if>
						
												
						<xsl:if test="Booking/BookingSlot/Calendar/hideAddedDateFromUser != 'true'">
							<div>
								<strong>
									<xsl:value-of select="$i18n.Booking.Added" />
									<xsl:text>:&#160;</xsl:text>
								</strong>
								<xsl:value-of select="Booking/added" />
							</div>
						</xsl:if>
						
						<div>
							<strong>
								<xsl:value-of select="$i18n.Booking.Size" />
								<xsl:text>:&#160;</xsl:text>
							</strong>
							<xsl:value-of select="Booking/size" />
						</div>
						
						<xsl:if test="Booking/archived = 'true'">
							<div>
			  					<strong>
										<xsl:value-of select="$i18n.Archived" />
			  					</strong>
							</div>
	  					</xsl:if>
	  					
	  					<xsl:if test="Booking/BookingSlot/Calendar/showManagers = 'true' and Booking/BookingSlot/Managers">
	  						<div>
		  						<strong>
			  						<xsl:value-of select="$i18n.BookingSlot.Managers" />
										<xsl:text>:&#160;</xsl:text>
			  					</strong>
								<xsl:apply-templates select="Booking/BookingSlot/Managers/user" mode="manager"/>
	  						</div>
	  					</xsl:if>
  					</div>
  					
  					<xsl:if test="Booking/BookingSlot/description">
									
						<div class="floatleft bigmarginbottom">
							
							<label class="floatleft full">
								<xsl:value-of select="$i18n.BookingSlot.Description" />
							</label>
							
							<p >
								<xsl:call-template name="replaceLineBreak">
									<xsl:with-param name="string" select="Booking/BookingSlot/description"/>
								</xsl:call-template>
							</p>
						</div>
							
					</xsl:if>
		
					<xsl:apply-templates select="ExtensionFragments/ViewFragment"/>
		  				
  				</div>

  			</div>

			<div id="tabs">
 				<ul class="tabs">
 					<li class="active">
 						<a data-icon-before="o" href="#history"><xsl:value-of select="$i18n.BookingEvents" /></a>
 					</li>
 					
 					<xsl:if test="Booking/BookingSlot/Calendar/hideExternalMessages = 'false'">
	 					<li data-tabid="#messages">
	 						<a data-icon-before="m" href="#messages">
	 							<xsl:value-of select="$i18n.ExternalMessages" />
	 							<!-- TODO count how many unread messages since last login -->
	 							<xsl:if test="false()">
	 								<span class="count">0</span>
	 							</xsl:if>
	 						</a>
	 					</li>
 					</xsl:if>
 				</ul>
 				
				<xsl:if test="Booking/BookingSlot/Calendar/hideExternalMessages = 'false'">
	 				<div id="messages">
	  					
	  					<div id="new-message" class="tabs-content">
	  						
	  						<div class="heading-wrapper">
	  							<h2><xsl:value-of select="$i18n.NewMessage" /></h2>
	  							<a href="#" class="btn btn-light btn-right close_message"><xsl:value-of select="$i18n.Close" /><i data-icon-after="x"></i></a>
	  						</div>
	  						
	  						<form action="{/Document/requestinfo/uri}#messages" method="post" enctype="multipart/form-data">
	  						
		  						<label class="required" for="message"><xsl:value-of select="$i18n.Message" /></label>
		  						<xsl:apply-templates select="validationError[fieldName = 'externalmessage']" />
		  						<textarea id="message" name="externalmessage" class="full"></textarea>
		  						<div class="heading-wrapper">
		  							<label class="required"><xsl:value-of select="$i18n.AttachFiles" /></label>
		  						</div>
		  						
		  						<script>
									imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
									deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
								</script>				
								
								<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />
								
								<div class="full">
									
									<div class="upload clearboth">
										<span class="btn btn-upload btn-blue">
											<xsl:value-of select="$i18n.ChooseFiles" />
											<input id="external-message" type="file" name="attachments" multiple="multiple" size="55" class="qloader externalmessages bigmarginbottom" />
										</span>
										<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxFileSize" /></span>
									</div>
									
									<ul id="external-message-qloader-filelist" class="files" />
									
								</div>
								
		  						<input type="submit" value="{$i18n.SubmitMessage}" name="addmessage" class="btn btn-green btn-inline" />
		  						<a href="#" class="btn btn-light btn-inline close_message"><xsl:value-of select="$i18n.Cancel" /></a>
		  						
	  						</form>
	  						
	  					</div>
	  					
	  					<div class="tabs-content">
		  					
		  					<div class="heading-wrapper">
		  						<h2><xsl:value-of select="$i18n.ExternalMessages" /></h2>
		  						
		  						<xsl:if test="IsNewExternalMessagesAllowed = 'true'">
			  						<a href="#" class="btn btn-blue btn-right open_message"><i data-icon-before="+"></i><xsl:value-of select="$i18n.NewMessage" /></a>
		  						</xsl:if>
		  					</div>
		  					
		  					<xsl:choose>
		  						<xsl:when test="Booking/ExternalMessages/ExternalMessage">
									
									<xsl:call-template name="sortMessagesButton" />
									
		  							<ul class="messages">
		  								<xsl:apply-templates select="Booking/ExternalMessages/ExternalMessage" mode="user" />
		  							</ul>
		  						</xsl:when>
		  						<xsl:otherwise>
		  							<xsl:value-of select="$i18n.NoExternalMessages" />
		  						</xsl:otherwise>	  					
		  					</xsl:choose>
		  					
		  				</div>
		  				
	  				</div>
  				</xsl:if>
 				
 				<div id="history" class="tabs-content nopadding" >
 					
 					<div class="errands-wrapper">
  					
	  					<div class="heading-wrapper">
		  					<h2><xsl:value-of select="$i18n.BookingEvents" /></h2>
	  					</div>
	  					
	  					<table class="oep-table">
	  						<thead class="errand">
	  							<tr>
	  								<th class="service active"><span><xsl:value-of select="$i18n.Action" /></span></th>
		  							<th class="details"><span><xsl:value-of select="$i18n.Details" /></span></th>
	  								<th class="date"><span><xsl:value-of select="$i18n.Date" /></span></th>
	  								<th class="person"><span><xsl:value-of select="$i18n.Person" /></span></th>
	  							</tr>
	  						</thead>
	  						<tbody>
	  							<xsl:choose>
	  								<xsl:when test="HideSubmittedEvent = 'true' and Booking/Events/BookingEvent[eventType != 'SUBMITTED']">
	  									<xsl:apply-templates select="Booking/Events/BookingEvent[eventType != 'SUBMITTED']" />
	  								</xsl:when>
	  								<xsl:when test="HideSubmittedEvent != 'true' and Booking/Events/BookingEvent">
	  									<xsl:apply-templates select="Booking/Events/BookingEvent" />
	  								</xsl:when>
	  								<xsl:otherwise>
	  									<tr><td class="icon" /><td colspan="5"><xsl:value-of select="$i18n.NoEvents" /></td></tr>
	  								</xsl:otherwise>
	  							</xsl:choose>
	  						</tbody>
	  					</table>
  					
  					</div>
  				
  				</div>
  			
 			</div>
			
			<div class="navigator-buttons centered">
			
				<xsl:if test="Booking/Mutable = 'true'">
				
					<xsl:if test="not(Booking/BookingSlot/Calendar/disableRebooking = 'true')">
						<a href="{/Document/ModuleURL}/updatebooking/{Booking/bookingID}" class="btn btn-inline btn-green xl next">
							<i data-icon-before="w" class="xl"/>
							<xsl:value-of select="$i18n.ChangeBookingSlot" />
						</a>
					</xsl:if>
					
					<xsl:if test="not(Booking/BookingSlot/Calendar/disableLeaving = 'true')">
				
						<form class="display-inline marginleft" action="{/Document/ModuleURL}/leavebooking/{Booking/bookingID}" method="post">
					
							<input type="hidden" name="comment" />
					
							<div style="display: none;">
							
								<div class="comment-modal contentitem">
									<div class="modal-content">
									
										<div class="modal-header bigmarginbottom">
											<h1>
												<xsl:value-of select="$i18n.LeaveBookingSlot" />
											</h1>
										</div>
										
										<div class="modal-body">
										
											<xsl:if test="Booking/BookingSlot/Calendar/hideExternalMessages = 'false'">
										
												<div class="floatleft full bigmarginbottom">
												
													<p class="tiny floatleft full">
													<xsl:value-of select="$i18n.LeaveBookingSlotDescription" />
												  </p>
												  
													<label for="save-email" class="floatleft clearboth">
														<xsl:value-of select="$i18n.LeaveReason" />
													</label>
													
													<div class="floatleft full">
														<xsl:call-template name="createTextArea">
															<xsl:with-param name="name" select="'message'"/>
															<xsl:with-param name="maxlength" select="'1024'"/>
														</xsl:call-template>
												    </div>
												    
												</div>
											
											</xsl:if>
											
											<input class="btn btn-red close bigmargintop floatright" type="button" value="{$i18n.LeaveBookingSlot}" />
											
										</div>
										
									</div>
								</div>
							
							</div>
							
							<a href="#" class="btn btn-red xl prev" onclick="commentModal(event, this)">
								<i data-icon-before="t" class="xl"/>
								<xsl:value-of select="$i18n.LeaveBookingSlot" />
							</a>
								
						</form>
					
					</xsl:if>
				
				</xsl:if>

 			</div>
 			
		</section>
		
	</xsl:template>
	
	<xsl:template match="UpdateBooking">
	
		<section class="service">
		
			<div class="section-inside step">
 				<div class="heading-wrapper">
 					<div class="inner">
  					<!--
  					<figure>
	  					<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{Flow/flowID}" alt="" />
	  				</figure>
	  				-->
	  				<div class="heading">
  						<h1 class="xl">
  							<xsl:value-of select="Booking/BookingSlot/Calendar/name" />
  						</h1>
						</div>
					</div>
 				</div>
 			</div>
  			
 			<div class="section-full push">
 			
 				<form action="{/Document/requestinfo/uri}" method="post">
 			
 					<xsl:value-of select="QueryResponse/HTML" disable-output-escaping="yes"/>
 				
	 				<div class="navigator-buttons centered">
				
						<a href="#" class="btn btn-green xl next" onclick="$(this).closest('form').trigger('submit')">
							<i data-icon-before="c" class="xl marginright"/>
							<xsl:value-of select="$i18n.ChangeBookingSlot" />
						</a>
						
		 			</div>
		 			
 				</form>
 			
 			</div>
 			
		</section>
		
	</xsl:template>
	
	<xsl:template name="createHelpDialog">
		
		<xsl:param name="id" />
		<xsl:param name="text" />
		<xsl:param name="class" select="''" />
		
		<div class="help {$class}">
			<a class="open-help" href="#" data-icon-after="?" data-help-box="helpdialog_{$id}"><span><xsl:value-of select="$i18n.Help" /></span></a>
			<div class="help-box" data-help-box="helpdialog_{$id}">
				<div>
		  			<div> 
		  				<a class="close" href="#" data-icon-after="x"></a>
		  				<xsl:copy-of select="$text" />
		  			</div> 
				</div>
			</div>
		</div>
		
		<div class="help-backdrop" data-help-box="helpdialog_{$id}" />
		
	</xsl:template>
		
	<xsl:template match="validationError[messageKey='FileSizeLimitExceeded']">
	
		<div class="info-box error">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part1"/>
					<xsl:value-of select="filename"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part2"/>
					<xsl:value-of select="size"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part3"/>
					<xsl:value-of select="maxFileSize"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part4"/>
				</strong>
			</span>
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
	
		<div class="info-box error">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.ValidationError.UnableToParseRequest" />
				</strong>
			</span>
		</div>
		
	</xsl:template>						
	
</xsl:stylesheet>