package com.nordicpeak.flowengine.booking.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OneToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateTimeStringyfier;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.booking.BookingUtils;
import com.nordicpeak.flowengine.booking.enums.UserType;

@Table(name = "bookings")
@XMLElement
public class Booking extends GeneratedElementable implements Serializable, Cloneable {

	private static final long serialVersionUID = 9041413820791856640L;

	public static final Field EVENTS_RELATION = ReflectionUtils.getField(Booking.class, "events");
	public static final Field BOOKING_SLOT_RELATION = ReflectionUtils.getField(Booking.class, "bookingSlot");
	public static final Field CONTACT_RELATION = ReflectionUtils.getField(Booking.class, "primaryContact");
	public static final Field MANUAL_CONTACT_RELATION = ReflectionUtils.getField(Booking.class, "manualContact");
	public static final Field INTERNAL_MESSAGES_RELATION = ReflectionUtils.getField(Booking.class, "internalMessages");
	public static final Field EXTERNAL_MESSAGES_RELATION = ReflectionUtils.getField(Booking.class, "externalMessages");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer bookingID;

	@DAOManaged(columnName = "bookingSlotID")
	@ManyToOne
	@XMLElement
	private BookingSlot bookingSlot;

	@DAOManaged
	@XMLElement
	private Integer flowInstanceID;

	@XMLElement
	private FlowInstance flowInstance;

	@DAOManaged
	@XMLElement
	private boolean archived;

	@DAOManaged
	@XMLElement
	private boolean preliminary;

	@DAOManaged
	@XMLElement
	private boolean bookingPassedEventSent;

	@DAOManaged
	@StringTag(valueFormatter = DateTimeStringyfier.class)
	@XMLElement
	private Timestamp added;

	@DAOManaged
	@XMLElement
	private Timestamp updated;

	@DAOManaged
	@XMLElement
	private Integer size;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<BookingEvent> events;

	@DAOManaged
	@OneToOne
	@XMLElement(name = "PrimaryContact")
	private PrimaryContact primaryContact;

	@DAOManaged
	@OneToOne
	@XMLElement(name = "ManualContact")
	private ManualContact manualContact;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<InternalMessage> internalMessages;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<ExternalMessage> externalMessages;

	private User poster;

	public Integer getBookingID() {

		return bookingID;
	}

	public void setBookingID(Integer bookingSlotID) {

		this.bookingID = bookingSlotID;
	}

	public Integer getFlowInstanceID() {

		return flowInstanceID;
	}

	public void setFlowInstanceID(Integer flowInstanceID) {

		this.flowInstanceID = flowInstanceID;
	}

	public BookingSlot getBookingSlot() {

		return bookingSlot;
	}

	public void setBookingSlot(BookingSlot bookingSlot) {

		this.bookingSlot = bookingSlot;
	}

	public FlowInstance getFlowInstance() {

		return flowInstance;
	}

	public void setFlowInstance(FlowInstance flowInstance) {

		this.flowInstance = flowInstance;
	}

	public User getPoster() {

		return poster;
	}

	public void setPoster(User poster) {

		this.poster = poster;
	}

	public Timestamp getAdded() {

		return added;
	}

	public void setAdded(Timestamp added) {

		this.added = added;
	}

	public Timestamp getUpdated() {

		return updated;
	}

	public void setUpdated(Timestamp updated) {

		this.updated = updated;
	}

	public Integer getSize() {

		return size;
	}

	public void setSize(Integer size) {

		this.size = size;
	}

	public List<BookingEvent> getEvents() {

		return events;
	}

	public void setEvents(List<BookingEvent> events) {

		this.events = events;
	}

	public PrimaryContact getPrimaryContact() {

		return primaryContact;
	}

	public void setPrimaryContact(PrimaryContact primaryContact) {

		this.primaryContact = primaryContact;
	}

	public String getPrimaryContactCitizenIdentifier() {

		return primaryContact != null ? primaryContact.getCitizenIdentifier() : null;
	}

	public ManualContact getManualContact() {

		return manualContact;
	}

	public void setManualContact(ManualContact secondaryContact) {

		this.manualContact = secondaryContact;
	}

	public List<InternalMessage> getInternalMessages() {

		return internalMessages;
	}

	public void setInternalMessages(List<InternalMessage> internalMessages) {

		this.internalMessages = internalMessages;
	}

	public List<ExternalMessage> getExternalMessages() {

		return externalMessages;
	}

	public void setExternalMessages(List<ExternalMessage> externalMessages) {

		this.externalMessages = externalMessages;
	}

	public boolean isArchived() {

		return archived;
	}

	public void setArchived(boolean archived) {

		this.archived = archived;
	}

	public boolean isPreliminary() {

		return preliminary;
	}

	public void setPreliminary(boolean preliminary) {

		this.preliminary = preliminary;
	}

	public boolean isBookingPassedEventSent() {

		return bookingPassedEventSent;
	}

	public void setBookingPassedEventSent(boolean bookingPassedEventSent) {

		this.bookingPassedEventSent = bookingPassedEventSent;
	}

	@Override
	public String toString() {

		if (bookingID != null) {

			return "(ID: " + bookingID + (bookingSlot != null ? ", bookingSlotID: " + bookingSlot.getBookingSlotID() : "") + ")";
		}

		return null;
	}

	public User getPoster(UserHandler userHandler) {

		if (poster == null && userHandler != null && primaryContact != null && !StringUtils.isEmpty(primaryContact.getCitizenIdentifier())) {
			
			return getPoster(userHandler, bookingSlot.getCalendar());
		}
		
		return poster;
	}

	public User getPoster(UserHandler userHandler, BookingCalendar calendar) {
		
		if (poster == null && userHandler != null && primaryContact != null && !StringUtils.isEmpty(primaryContact.getCitizenIdentifier())) {

			if (calendar.isUseCalendarForInternalUsers()) {

				poster = userHandler.getUserByAttribute(Constants.MANAGER_CITIZEN_IDENTIFIER_ATTRIBUTE, primaryContact.getCitizenIdentifier(), false, true);

			} else {

				poster = userHandler.getUserByAttribute(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE, primaryContact.getCitizenIdentifier(), false, true);
			}
		}

		return poster;
	}
	
	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (poster != null) {

			Element posterElement = XMLUtils.appendNewElement(doc, element, "Poster");
			UserUtils.appendUsers(doc, posterElement, Collections.singletonList(poster), true);
		}

		if (bookingSlot != null && bookingSlot.getCalendar() != null) {

			XMLUtils.appendNewElement(doc, element, "Mutable", isMutable());
		}

		return element;
	}

	public Element toXML(Document doc, UserHandler userHandler, boolean includeSMS, boolean overrideContactBySMS) {

		getPoster(userHandler);

		Element element = toXML(doc);

		appendUserType(doc, element, includeSMS, overrideContactBySMS);

		return element;
	}

	public JsonNode toJson() {

		JsonObject bookingJson = new JsonObject();

		bookingJson.putField("id", bookingID);
		bookingJson.putField("name", primaryContact.getFirstname() + " " + primaryContact.getLastname());
		bookingJson.putField("added", DateUtils.DATE_TIME_FORMATTER.format(added));
		bookingJson.putField("slotDate", DateUtils.DATE_TIME_FORMATTER.format(bookingSlot.getStart()));
		bookingJson.putField("flowInstanceID", flowInstanceID);
		bookingJson.putField("archived", archived);

		return bookingJson;
	}

	public void appendUserType(Document doc, Element targetElement, boolean includeSMS, boolean overrideContactBySMS) {

		UserType userType = BookingUtils.getUserType(this, null, includeSMS, overrideContactBySMS);

		if (userType == UserType.DIGITAL) {

			XMLUtils.appendNewElement(doc, targetElement, "Digital");

		} else if (userType == UserType.DIGITAL_NOT_CONTACTABLE) {

			XMLUtils.appendNewElement(doc, targetElement, "DigitalNotContactable");

		} else {

			XMLUtils.appendNewElement(doc, targetElement, "Analog");
		}
	}

	public boolean isMutable() {

		return isMutable(bookingSlot.getCalendar());
	}

	public boolean isMutable(BookingCalendar calendar) {

		if (archived) {

			return false;
		}

		java.util.Calendar cal = java.util.Calendar.getInstance();
		cal.add(java.util.Calendar.HOUR_OF_DAY, calendar.getHoursBeforeBookingLocked());

		return new Date(cal.getTimeInMillis()).before(bookingSlot.getStart());
	}

	public void addEvent(BookingEvent event) {

		if (events == null) {

			events = new ArrayList<>();
		}

		events.add(event);
	}

	public void addInternalMessage(InternalMessage internalMessage) {

		if (internalMessages == null) {

			internalMessages = new ArrayList<>();
		}

		internalMessages.add(internalMessage);
	}

	public void addExternalMessage(ExternalMessage externalMessage) {

		if (externalMessages == null) {

			externalMessages = new ArrayList<>();
		}

		externalMessages.add(externalMessage);
	}

	@Override
	public Booking clone() throws CloneNotSupportedException {

		return (Booking) super.clone();
	}

}
