package com.nordicpeak.flowengine.booking.beans;

import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.util.List;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.booking.enums.InternalMessageType;

@Table(name = "bookings_internal_messages")
@XMLElement
public class InternalMessage extends BaseMessage {

	private static final long serialVersionUID = -934750439780152020L;

	public static final Field ATTACHMENTS_RELATION = ReflectionUtils.getField(InternalMessage.class, "attachments");
	public static final Field BOOKING_RELATION = ReflectionUtils.getField(InternalMessage.class, "booking");

	@DAOManaged(columnName = "bookingID")
	@ManyToOne
	@XMLElement
	private Booking booking;

	@DAOManaged
	@XMLElement
	private InternalMessageType type;

	@DAOManaged
	@OneToMany(autoAdd = true)
	@XMLElement
	private List<InternalMessageAttachment> attachments;

	public InternalMessage() {}

	public InternalMessage(String message, User poster, Timestamp added) {

		super(message, poster, added);
	}

	public InternalMessage(String message, User poster, Timestamp added, Booking booking, List<InternalMessageAttachment> attachments) {

		this(message, poster, added, booking, attachments, null);
	}

	public InternalMessage(String message, User poster, Timestamp added, Booking booking, List<InternalMessageAttachment> attachments, InternalMessageType type) {

		super(message, poster, added);
		this.booking = booking;
		this.attachments = attachments;
		this.type = type;
	}

	@Override
	public List<InternalMessageAttachment> getAttachments() {

		return attachments;
	}

	public void setAttachments(List<InternalMessageAttachment> attachments) {

		this.attachments = attachments;
	}

	@Override
	public Booking getBooking() {

		return booking;
	}

	public void setBooking(Booking booking) {

		this.booking = booking;
	}

	public InternalMessageType getType() {

		return type;
	}

	public void setType(InternalMessageType type) {

		this.type = type;
	}

}
