package com.nordicpeak.flowengine.booking.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Date;
import java.sql.Timestamp;
import java.util.List;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "booking_slots_repeating")
@XMLElement
public class Repeating extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = 9041413820791856640L;

	public static final Field CALENDAR_RELATION = ReflectionUtils.getField(Repeating.class, "calendar");
	public static final Field BOOKING_SLOTS_RELATION = ReflectionUtils.getField(Repeating.class, "bookingSlots");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer repeatID;

	@DAOManaged(columnName = "calendarID")
	@ManyToOne
	@XMLElement
	private BookingCalendar calendar;

	@DAOManaged
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date fromDate;

	@DAOManaged
	@WebPopulate(required = true, populator = DatePopulator.class)
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date toDate;

	@DAOManaged
	@XMLElement
	private Integer repetitionInterval;

	@DAOManaged
	@StringTag
	@XMLElement
	private Timestamp added;

	@DAOManaged
	@XMLElement
	private Timestamp updated;

	@DAOManaged
	@XMLElement
	private User poster;

	@DAOManaged
	@XMLElement
	private User updater;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<BookingSlot> bookingSlots;

	public Repeating() {}

	public Repeating(BookingCalendar calendar, Date fromDate, Date toDate, Integer repetitionInterval, Timestamp added, User poster) {

		this.calendar = calendar;
		this.fromDate = fromDate;
		this.toDate = toDate;
		this.repetitionInterval = repetitionInterval;
		this.added = added;
		this.poster = poster;
	}

	public Integer getRepeatID() {

		return repeatID;
	}

	public void setRepeatID(Integer bookingSlotID) {

		this.repeatID = bookingSlotID;
	}

	public Timestamp getAdded() {

		return added;
	}

	public void setAdded(Timestamp added) {

		this.added = added;
	}

	public Timestamp getUpdated() {

		return updated;
	}

	public void setUpdated(Timestamp updated) {

		this.updated = updated;
	}

	public BookingCalendar getCalendar() {

		return calendar;
	}

	public void setCalendar(BookingCalendar calendar) {

		this.calendar = calendar;
	}

	public User getPoster() {

		return poster;
	}

	public void setPoster(User poster) {

		this.poster = poster;
	}

	public User getUpdater() {

		return updater;
	}

	public void setUpdater(User updater) {

		this.updater = updater;
	}

	public List<BookingSlot> getBookingSlots() {

		return bookingSlots;
	}

	public void setBookingSlots(List<BookingSlot> bookingSlots) {

		this.bookingSlots = bookingSlots;
	}

	public Date getFromDate() {

		return fromDate;
	}

	public void setFromDate(Date fromDate) {

		this.fromDate = fromDate;
	}

	public Date getToDate() {

		return toDate;
	}

	public void setToDate(Date toDate) {

		this.toDate = toDate;
	}

	public Integer getRepetitionInterval() {

		return repetitionInterval;
	}

	public void setRepetitionInterval(Integer repetitionInterval) {

		this.repetitionInterval = repetitionInterval;
	}

	@Override
	public String toString() {

		if (calendar != null) {

			return "(ID: " + repeatID + ")" + " in " + calendar;
		}

		if (repeatID != null) {

			return "(ID: " + repeatID + ")";
		}

		return null;
	}

}
