package com.nordicpeak.flowengine.booking.notificationhandler;

import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleSMS;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.interfaces.sms.SMSSender;
import se.unlogic.hierarchy.core.settings.Setting;
import se.unlogic.hierarchy.core.settings.TextFieldSetting;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.ProfileSettingHandler;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileSettingProvider;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.string.TagSource;
import se.unlogic.standardutils.templates.TemplateUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.booking.BookingUtils;
import com.nordicpeak.flowengine.booking.CalendarAdminModule;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.events.BookingArchiveEvent;
import com.nordicpeak.flowengine.booking.events.BookingManuallyAddedEvent;
import com.nordicpeak.flowengine.booking.events.BookingSlotDeletedEvent;
import com.nordicpeak.flowengine.booking.events.BookingSlotRebookedEvent;
import com.nordicpeak.flowengine.booking.events.ExternalMessageAddedEvent;
import com.nordicpeak.flowengine.enums.SenderType;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;

import it.sauronsoftware.cron4j.Scheduler;

public class BookingNotificationHandler extends AnnotatedForegroundModule implements Runnable, SiteProfileSettingProvider {

	//@formatter:off
	private static final Field[] BOOKING_RELATIONS = { 
		Booking.CONTACT_RELATION, 
		Booking.MANUAL_CONTACT_RELATION, 
		Booking.BOOKING_SLOT_RELATION, 
		BookingSlot.CALENDAR_RELATION, 
		BookingSlot.MANAGERS_RELATION, 
		BookingCalendar.MANAGER_USERS_RELATION, 
		BookingCalendar.MANAGER_GROUPS_RELATION, 
		BookingCalendar.ADMIN_GROUPS_RELATION, 
	};
	//@formatter:on

	private static final AnnotatedBeanTagSourceFactory<BookingCalendar> CALENDAR_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(BookingCalendar.class, "$calendar.");
	private static final AnnotatedBeanTagSourceFactory<BookingSlot> BOOKING_SLOT_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(BookingSlot.class, "$bookingSlot.");
	private static final AnnotatedBeanTagSourceFactory<BookingSlot> BOOKING_SLOT_OLD_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(BookingSlot.class, "$bookingSlotOld.");
	private static final AnnotatedBeanTagSourceFactory<Booking> BOOKING_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(Booking.class, "$booking.");
	private static final AnnotatedBeanTagSourceFactory<Contact> CONTACT_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(Contact.class, "$contact.");
	private static final AnnotatedBeanTagSourceFactory<User> MANAGER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(User.class, "$manager.");

	@XSLVariable(prefix = "java.")
	private String i18nUserBookingModuleAlias = "User booking module URL";

	@XSLVariable(prefix = "java.")
	private String i18nUserBookingModuleAliasDescription = "The full URL of the user booking module";

	@XSLVariable(prefix = "java.")
	private String i18nBookingSlotAdminModuleAlias = "Booking slot admin module URL";

	@XSLVariable(prefix = "java.")
	private String i18nBookingSlotAdminModuleAliasDescription = "The full URL of the booking slot admin module";

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when new messages are received", description = "The SMS messages sent to the users when they receive new messages.", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New message received email subject (users)", description = "The subject of emails sent to the users when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New message received email message (users)", description = "The message of emails sent to the users when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedUserEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New message received email subject (managers)", description = "The subject of emails sent to the managers when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedManagerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New message received email message (managers)", description = "The message of emails sent to the managers when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedManagerEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New message received email subject (global)", description = "The subject of emails sent when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedGlobalEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New message received email message (global)", description = "The message of emails sent when new messages are received", required = true)
	@XSLVariable(prefix = "java.")
	private String externalMessageReceivedGlobalEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when manually added to booking", description = "The SMS messages sent to the users when they are manually added to booking.", required = true)
	@XSLVariable(prefix = "java.")
	private String manualEntranceSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Manually added to booking email subject (users)", description = "The subject of emails sent to the users when they are manually added to booking", required = true)
	@XSLVariable(prefix = "java.")
	private String manualEntranceEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Manually added to booking email message (users)", description = "The message of emails sent to the users when they are manually added to booking", required = true)
	@XSLVariable(prefix = "java.")
	private String manualEntranceEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Manually added to booking email subject (managers)", description = "The subject of emails sent to the managers when users are manually added to booking", required = true)
	@XSLVariable(prefix = "java.")
	private String manualEntranceManagerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Manually added to booking email message (managers)", description = "The message of emails sent to the managers when users are manually added to booking", required = true)
	@XSLVariable(prefix = "java.")
	private String manualEntranceManagerEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when manually sending reminder", description = "The SMS messages sent to the users when manager manually sends reminder.", required = true)
	@XSLVariable(prefix = "java.")
	private String manualReminderSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Manually sent reminder email subject (users)", description = "The subject of emails sent to the users when manager manually sends reminder", required = true)
	@XSLVariable(prefix = "java.")
	private String manualReminderEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Manually sent reminder email message (users)", description = "The message of emails sent to the users when manager manually sends reminder", required = true)
	@XSLVariable(prefix = "java.")
	private String manualReminderEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when rebooked", description = "The SMS messages sent to the users when they are rebooked.", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Rebooking email subject (user)", description = "The subject of emails sent to the user when user is rebooked", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Rebooking email message (user)", description = "The message of emails sent to the user when user is rebooked", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedUserEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when removed from booking", description = "The SMS messages sent to the users when they are removed from booking.", required = true)
	@XSLVariable(prefix = "java.")
	private String exitUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Removed from booking email subject (users)", description = "The subject of emails sent to the users when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Removed from booking email message (users)", description = "The message of emails sent to the users when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitUserEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Removed from booking email subject (managers)", description = "The subject of emails sent to the managers when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitManagerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Removed from booking email message (managers)", description = "The message of emails sent to the managers when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitManagerEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Rebooking email subject (managers)", description = "The subject of emails sent to the managers when user rebooks", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedManagerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Rebooking email message (managers)", description = "The message of emails sent to the managers when user rebooks", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedManagerEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Removed from booking email subject (global)", description = "The subject of emails sent when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitGlobalEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Removed from booking email message (global)", description = "The message of emails sent when removed from booking", required = true)
	@XSLVariable(prefix = "java.")
	private String exitGlobalEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Rebooked booking email subject (global)", description = "The subject of emails sent when a user rebooks", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedGlobalEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Rebooked booking email message (global)", description = "The message of emails sent when a user rebooks", required = true)
	@XSLVariable(prefix = "java.")
	private String rebookedGlobalEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when a booking slots time is updated", description = "The SMS messages sent to the users when a booking slots time is changed.", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotRebookedUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Rebooked booking slot email subject (global)", description = "The subject of emails sent when a booking slots time is changed", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotRebookedUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Rebooked booking slot email message (global)", description = "The message of emails sent when a booking slots time is changed", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotRebookedUserEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users when a booking slot is deleted", description = "The SMS messages sent to the users when a booking slot is deleted.", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotDeletedUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Deleted booking slot email subject (global)", description = "The subject of emails sent when a booking slot is deleted", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotDeletedUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Deleted booking slot email message (global)", description = "The message of emails sent when a booking slot is deleted", required = true)
	@XSLVariable(prefix = "java.")
	private String bookingSlotDeletedUserEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS to users for reminding of upcoming booking", description = "The SMS messages sent to the users to remind them of an upcoming booking.", required = true)
	@XSLVariable(prefix = "java.")
	private String reminderSMS;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Override contactBySMS", description = "Overrides the contactBySMS field in Contacts to true when sending SMS")
	private boolean overrideContactBySMS = true;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable profile support", description = "Controls of if profile support is enabled")
	private boolean enableProfileSupport = false;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "User booking slot module URL", description = "The full URL of the user booking slot module", required = true)
	private String userBookingSlotModuleAlias = "not set";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Admin booking slot module URL", description = "The full URL of the admin booking slot module", required = true)
	private String bookingSlotAdminModuleAlias = "not set";

	@InstanceManagerDependency(required = true)
	private StandardFlowNotificationHandler notificationHandler;

	@InstanceManagerDependency
	private SMSSender smsSender;

	@InstanceManagerDependency(required = true)
	private CalendarAdminModule calendarAdminModule;

	private SiteProfileHandler siteProfileHandler;

	private AnnotatedDAO<BookingCalendar> calendarDAO;
	private AnnotatedDAO<BookingSlot> bookingSlotDAO;
	private AnnotatedDAO<Booking> bookingDAO;
	private AnnotatedDAOWrapper<Booking, Integer> bookingDAOWrapper;

	private QueryParameterFactory<BookingCalendar, Integer> calendarReminderMinutesParameterFactory;

	private Scheduler scheduler;

	private List<Setting> profileSettings;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		calendarDAO = daoFactory.getDAO(BookingCalendar.class);
		bookingSlotDAO = daoFactory.getDAO(BookingSlot.class);
		bookingDAO = daoFactory.getDAO(Booking.class);

		bookingDAOWrapper = bookingDAO.getWrapper(Integer.class);
		bookingDAOWrapper.addRelations(BOOKING_RELATIONS);
		bookingDAOWrapper.setUseRelationsOnGet(true);

		calendarReminderMinutesParameterFactory = calendarDAO.getParamFactory("reminderMinutes", Integer.class);
	}

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(BookingNotificationHandler.class, this)) {

			throw new RuntimeException("Unable to register module " + this.moduleDescriptor + " in global instance handler using key " + BookingNotificationHandler.class.getSimpleName() + ", another instance is already registered using this key.");
		}

		initScheduler();
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.WARN);

		if (enableProfileSupport) {

			profileSettings = new ArrayList<>(2);

			profileSettings.add(new TextFieldSetting("userBookingModuleAliasSetting", i18nUserBookingModuleAlias, i18nUserBookingModuleAliasDescription, "", false));
			profileSettings.add(new TextFieldSetting("bookingSlotAdminModuleAliasSetting", i18nBookingSlotAdminModuleAlias, i18nBookingSlotAdminModuleAliasDescription, "", false));

		} else {

			profileSettings = null;
		}
	}

	@Override
	public void unload() throws Exception {

		stopScheduler();

		systemInterface.getInstanceHandler().removeInstance(BookingNotificationHandler.class, this);

		if (siteProfileHandler != null) {

			siteProfileHandler.removeSettingProvider(this);
		}

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setSiteProfileHandler(SiteProfileHandler siteProfileHandler) {

		if (siteProfileHandler != null) {

			siteProfileHandler.addSettingProvider(this);

		} else {

			this.siteProfileHandler.removeSettingProvider(this);
		}

		this.siteProfileHandler = siteProfileHandler;
	}

	protected synchronized void initScheduler() {

		scheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());
		scheduler.setDaemon(true);

		scheduler.schedule("* * * * *", this);
		scheduler.start();
	}

	protected void stopScheduler() {

		try {

			if (scheduler != null) {

				scheduler.stop();
				scheduler = null;
			}

		} catch (IllegalStateException e) {

			log.error("Error stopping scheduler", e);
		}
	}

	@Override
	public synchronized void run() {

		sendReminderNotifications();
	}

	private void sendReminderNotifications() {

		log.debug("Checking for reminder notifications..");

		try {

			long now = System.currentTimeMillis();
			List<Booking> bookingsToSendNotificationsFor = new ArrayList<>();

			HighLevelQuery<BookingCalendar> calendarQuery = new HighLevelQuery<>();
			calendarQuery.addParameter(calendarReminderMinutesParameterFactory.getIsNotNullParameter());

			List<BookingCalendar> calendars = calendarDAO.getAll(calendarQuery);

			if (!CollectionUtils.isEmpty(calendars)) {

				Calendar calendar = Calendar.getInstance();

				for (BookingCalendar bookingCalendar : calendars) {

					calendar.setTimeInMillis(now);
					calendar.set(Calendar.MILLISECOND, 0);
					calendar.set(Calendar.SECOND, 0);
					calendar.add(Calendar.MINUTE, bookingCalendar.getReminderMinutes());

					//@formatter:off
					String sql = 
						"SELECT bookingID FROM " + bookingDAO.getTableName() + " b " +
						"INNER JOIN " + bookingSlotDAO.getTableName() + " s ON b.bookingSlotID = s.bookingSlotID " +
						"WHERE s.calendarID = ? " +
						"AND s.start = ? " +
						"AND b.archived = 0";
					//@formatter:on
					ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(dataSource, sql, IntegerPopulator.getPopulator());
					idQuery.setInt(1, bookingCalendar.getCalendarID());
					idQuery.setTimestamp(2, new Timestamp(calendar.getTimeInMillis()));

					ArrayList<Integer> bookingIDs = idQuery.executeQuery();

					if (!CollectionUtils.isEmpty(bookingIDs)) {

						HighLevelQuery<Booking> bookingQuery = new HighLevelQuery<>(Booking.BOOKING_SLOT_RELATION, Booking.CONTACT_RELATION, Booking.MANUAL_CONTACT_RELATION);
						bookingQuery.addParameter(bookingDAOWrapper.getParameterFactory().getWhereInParameter(bookingIDs));

						List<Booking> bookings = bookingDAO.getAll(bookingQuery);

						if (!CollectionUtils.isEmpty(bookings)) {

							bookings.forEach(booking -> booking.getBookingSlot().setCalendar(bookingCalendar));

							bookingsToSendNotificationsFor.addAll(bookings);
						}
					}
				}
			}

			if (!CollectionUtils.isEmpty(bookingsToSendNotificationsFor)) {

				log.info("Sending reminder notifications..");

				int sentNotifications = 0;

				for (Booking booking : bookingsToSendNotificationsFor) {

					Contact contact = BookingUtils.getContact(booking, getUserHandler());

					if (contact != null) {

						String sms = ObjectUtils.getFirstNotNull(booking.getBookingSlot().getCalendar().getUserReminderSMS(), reminderSMS);

						if (sendContactSMS(booking, contact, null, sms, null)) {

							sentNotifications++;
						}
					}
				}

				log.info("Sent " + sentNotifications + " reminder notifications for " + bookingsToSendNotificationsFor.size() + " bookings");
			}

		} catch (Throwable t) {

			log.error("Error sending reminder notifications", t);
		}
	}

	@WebPublic(requireLogin = true)
	public ForegroundModuleResponse manualSendReminderNotifications(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException {

		if (user.isAdmin()) {

			sendReminderNotifications();
			return new SimpleForegroundModuleResponse("See log", getDefaultBreadcrumb());
		}

		throw new URINotFoundException(uriParser);
	}

	@EventListener(channel = Booking.class)
	public void processEvent(ExternalMessageAddedEvent event, EventSource source) throws SQLException {

		Booking booking = bookingDAOWrapper.get(event.getBooking().getBookingID());
		BookingCalendar calendar = booking.getBookingSlot().getCalendar();
		Contact contact = getContact(booking);

		if (calendar.isHideExternalMessages()) {

			return;
		}

		if (event.getSenderType() == SenderType.MANAGER) {

			User manager = event.getEvent() != null ? event.getEvent().getPoster() : null;

			if (contact != null && (calendar.isSendExternalMessageReceivedUserEmail() || calendar.isSendExternalMessageReceivedUserSMS())) {

				if (calendar.isSendExternalMessageReceivedUserEmail()) {

					String subject = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedUserEmailSubject(), externalMessageReceivedUserEmailSubject);
					String message = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedUserEmailMessage(), externalMessageReceivedUserEmailMessage);

					sendContactEmail(booking, contact, manager, subject, message, null);
				}

				if (calendar.isSendExternalMessageReceivedUserSMS()) {

					String sms = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedUserSMS(), externalMessageReceivedUserSMS);

					sendContactSMS(booking, contact, manager, sms, null);
				}
			}

		} else if (event.getSenderType() == SenderType.USER) {

			if (calendar.isSendExternalMessageReceivedManagerEmail()) {

				String subject = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedManagerEmailSubject(), externalMessageReceivedManagerEmailSubject);
				String message = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedManagerEmailMessage(), externalMessageReceivedManagerEmailMessage);

				sendManagerEmails(booking, contact, subject, message, null, null);
			}

			if (calendar.isSendExternalMessageReceivedGlobalEmail() && calendar.getExternalMessageReceivedGlobalEmailAddresses() != null) {

				String subject = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedGlobalEmailSubject(), externalMessageReceivedGlobalEmailSubject);
				String message = ObjectUtils.getFirstNotNull(calendar.getExternalMessageReceivedGlobalEmailMessage(), externalMessageReceivedGlobalEmailMessage);

				for (String email : calendar.getExternalMessageReceivedGlobalEmailAddresses()) {

					sendGlobalEmail(booking, contact, email, subject, message);
				}
			}

		} else {

			log.warn("External message added event received with unsupported sender type: " + event.getSenderType());
		}
	}

	@EventListener(channel = Booking.class)
	public void processEvent(BookingManuallyAddedEvent event, EventSource source) throws SQLException {

		if (!source.isLocal()) {

			return;
		}

		Booking booking = bookingDAOWrapper.get(event.getBooking().getBookingID());
		BookingCalendar calendar = booking.getBookingSlot().getCalendar();
		User poster = event.getPoster();

		if (calendar.isSendEnteredUserEmail() || calendar.isSendEnteredUserSMS() || calendar.isSendEnteredManagerEmail()) {

			Contact contact = getContact(booking);

			if (contact != null) {

				if (calendar.isSendEnteredUserEmail()) {

					String subject = ObjectUtils.getFirstNotNull(calendar.getManualEntranceEmailSubject(), manualEntranceEmailSubject);
					String message = ObjectUtils.getFirstNotNull(calendar.getManualEntranceEmailMessage(), manualEntranceEmailMessage);

					sendContactEmail(booking, contact, poster, subject, message, null);
				}

				if (calendar.isSendEnteredUserSMS()) {

					String sms = ObjectUtils.getFirstNotNull(calendar.getManualEntranceSMS(), manualEntranceSMS);

					sendContactSMS(booking, contact, poster, sms, null);
				}
			}

			if (calendar.isSendEnteredManagerEmail()) {

				String subject = ObjectUtils.getFirstNotNull(calendar.getManualEntranceManagerEmailSubject(), manualEntranceManagerEmailSubject);
				String message = ObjectUtils.getFirstNotNull(calendar.getManualEntranceManagerEmailMessage(), manualEntranceManagerEmailMessage);

				sendManagerEmails(booking, contact, subject, message, null, null);
			}
		}
	}

	@EventListener(channel = Booking.class)
	public void processEvent(BookingArchiveEvent event, EventSource source) throws SQLException {

		if (!source.isLocal()) {

			return;
		}

		Booking booking = bookingDAOWrapper.get(event.getBooking().getBookingID());
		BookingCalendar calendar = booking.getBookingSlot().getCalendar();

		Contact contact = getContact(booking);

		if (contact != null) {

			String subject = ObjectUtils.getFirstNotNull(calendar.getUserExitEmailSubject(), exitUserEmailSubject);
			String message = ObjectUtils.getFirstNotNull(calendar.getUserExitEmailMessage(), exitUserEmailMessage);

			sendContactEmail(booking, contact, null, subject, message, null);

			if (calendar.isSendExitUserSMS()) {

				String sms = ObjectUtils.getFirstNotNull(calendar.getUserExitSMS(), exitUserSMS);

				sendContactSMS(booking, contact, null, sms, null);
			}
		}

		if (calendar.isSendExitManagerEmail()) {

			String subject = ObjectUtils.getFirstNotNull(calendar.getUserExitManagerEmailSubject(), exitManagerEmailSubject);
			String message = ObjectUtils.getFirstNotNull(calendar.getUserExitManagerEmailMessage(), exitManagerEmailMessage);

			sendManagerEmails(booking, contact, subject, message, null, null);
		}

		if (calendar.isSendExitGlobalEmail() && calendar.getExitGlobalEmailAddresses() != null) {

			for (String email : calendar.getExitGlobalEmailAddresses()) {

				String subject = ObjectUtils.getFirstNotNull(calendar.getUserExitGlobalEmailSubject(), exitGlobalEmailSubject);
				String message = ObjectUtils.getFirstNotNull(calendar.getUserExitGlobalEmailMessage(), exitGlobalEmailMessage);

				sendGlobalEmail(booking, contact, email, subject, message);
			}
		}
	}

	@EventListener(channel = BookingSlot.class)
	public void processEvent(BookingSlotRebookedEvent event, EventSource source) throws CloneNotSupportedException {

		if (!source.isLocal()) {

			return;
		}

		BookingSlot bookingSlot = event.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();
		List<Booking> bookings = bookingSlot.getBookings();

		if (bookings == null) {

			return;
		}

		String sms = ObjectUtils.getFirstNotNull(calendar.getBookingSlotRebookedUserSMS(), bookingSlotRebookedUserSMS);
		String subject = ObjectUtils.getFirstNotNull(calendar.getBookingSlotRebookedUserEmailSubject(), bookingSlotRebookedUserEmailSubject);
		String message = ObjectUtils.getFirstNotNull(calendar.getBookingSlotRebookedUserEmailMessage(), bookingSlotRebookedUserEmailMessage);

		List<TagSource> tagSources = new ArrayList<>();

		tagSources.add(new SingleTagSource("$bookingSlotOld.start", DateUtils.DATE_TIME_FORMATTER.format(event.getBookingSlotOldStart())));
		tagSources.add(new SingleTagSource("$bookingSlotOld.end", DateUtils.DATE_TIME_FORMATTER.format(event.getBookingSlotOldEnd())));

		for (Booking booking : bookings) {

			Booking bookingWithBookingSlot = booking.clone();

			bookingWithBookingSlot.setBookingSlot(bookingSlot);

			Contact contact = getContact(bookingWithBookingSlot);

			if (contact != null) {

				if (calendar.isSendBookingSlotRebookedUserSMS()) {

					sendContactSMS(bookingWithBookingSlot, contact, null, sms, tagSources);
				}

				if (calendar.isSendBookingSlotRebookedUserEmail()) {

					sendContactEmail(bookingWithBookingSlot, contact, null, subject, message, tagSources);
				}
			}
		}
	}

	@EventListener(channel = BookingSlot.class)
	public void processEvent(BookingSlotDeletedEvent event, EventSource source) throws CloneNotSupportedException {

		if (!source.isLocal()) {

			return;
		}

		BookingSlot bookingSlot = event.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();
		List<Booking> bookings = bookingSlot.getBookings();

		if (bookings == null) {

			return;
		}

		String sms = ObjectUtils.getFirstNotNull(calendar.getBookingSlotDeletedUserSMS(), bookingSlotDeletedUserSMS);
		String subject = ObjectUtils.getFirstNotNull(calendar.getBookingSlotDeletedUserEmailSubject(), bookingSlotDeletedUserEmailSubject);
		String message = ObjectUtils.getFirstNotNull(calendar.getBookingSlotDeletedUserEmailMessage(), bookingSlotDeletedUserEmailMessage);

		for (Booking booking : bookings) {

			Booking bookingWithBookingSlot = booking.clone();

			bookingWithBookingSlot.setBookingSlot(bookingSlot);

			Contact contact = getContact(bookingWithBookingSlot);

			if (contact != null) {

				if (calendar.isSendBookingSlotDeletedUserSMS()) {

					sendContactSMS(bookingWithBookingSlot, contact, null, sms, null);
				}

				if (calendar.isSendBookingSlotDeletedUserEmail()) {

					sendContactEmail(bookingWithBookingSlot, contact, null, subject, message, null);
				}
			}
		}
	}

	public void sendUserRebookedNotifications(Booking booking, BookingCalendar calendar, BookingSlot oldSlot) {

		if (calendar.isSendUserRebookedManagerEmail()) {

			List<User> managersOld = BookingUtils.getManagers(oldSlot, calendar, getUserHandler(), calendar.isSendUserRebookedManagerEmailOnlyToBookingSlotManagers());
			List<User> managersNew = BookingUtils.getManagers(booking.getBookingSlot(), calendar, getUserHandler(), calendar.isSendUserRebookedManagerEmailOnlyToBookingSlotManagers());

			Contact contact = BookingUtils.getContact(booking, getUserHandler());

			List<TagSource> extraTagSources = new ArrayList<>();
			extraTagSources.add(BOOKING_SLOT_OLD_TAG_SOURCE_FACTORY.getTagSource(oldSlot));
			extraTagSources.add(new SingleTagSource("$managersOld", BookingUtils.getManagerNames(managersOld)));
			extraTagSources.add(new SingleTagSource("$managersNew", BookingUtils.getManagerNames(managersNew)));

			@SuppressWarnings("unchecked")
			HashSet<User> uniqueManagers = CollectionUtils.combineAsSet(managersOld, managersNew);

			if (uniqueManagers != null) {

				String subject = ObjectUtils.getFirstNotNull(calendar.getUserRebookedManagerEmailSubject(), rebookedManagerEmailSubject);
				String message = ObjectUtils.getFirstNotNull(calendar.getUserRebookedManagerEmailMessage(), rebookedManagerEmailMessage);

				booking.getBookingSlot().setManagers(new ArrayList<>(uniqueManagers));

				sendManagerEmails(booking, contact, subject, message, null, extraTagSources);
			}
		}

		if (calendar.isSendRebookedGlobalEmail()) {

			List<User> managersOld = BookingUtils.getManagers(oldSlot, calendar, getUserHandler(), false);
			List<User> managersNew = BookingUtils.getManagers(booking.getBookingSlot(), calendar, getUserHandler(), false);

			Contact contact = BookingUtils.getContact(booking, getUserHandler());

			List<TagSource> extraTagSources = new ArrayList<>();
			extraTagSources.add(BOOKING_SLOT_OLD_TAG_SOURCE_FACTORY.getTagSource(oldSlot));
			extraTagSources.add(new SingleTagSource("$managersOld", BookingUtils.getManagerNames(managersOld)));
			extraTagSources.add(new SingleTagSource("$managersNew", BookingUtils.getManagerNames(managersNew)));

			String subject = ObjectUtils.getFirstNotNull(calendar.getRebookedGlobalEmailSubject(), rebookedGlobalEmailSubject);
			String message = ObjectUtils.getFirstNotNull(calendar.getRebookedGlobalEmailMessage(), rebookedGlobalEmailMessage);

			for (String email : calendar.getRebookedGlobalEmailAddresses()) {

				sendGlobalEmail(booking, contact, email, subject, message, extraTagSources);
			}
		}

		if (calendar.isSendRebookedUserEmail()) {

			List<User> managersOld = BookingUtils.getManagers(oldSlot, calendar, getUserHandler(), false);
			List<User> managersNew = BookingUtils.getManagers(booking.getBookingSlot(), calendar, getUserHandler(), false);

			Contact contact = BookingUtils.getContact(booking, getUserHandler());

			List<TagSource> extraTagSources = new ArrayList<>();
			extraTagSources.add(BOOKING_SLOT_OLD_TAG_SOURCE_FACTORY.getTagSource(oldSlot));
			extraTagSources.add(new SingleTagSource("$managersOld", BookingUtils.getManagerNames(managersOld)));
			extraTagSources.add(new SingleTagSource("$managersNew", BookingUtils.getManagerNames(managersNew)));

			String subject = ObjectUtils.getFirstNotNull(calendar.getRebookedUserEmailSubject(), rebookedUserEmailSubject);
			String message = ObjectUtils.getFirstNotNull(calendar.getRebookedUserEmailMessage(), rebookedUserEmailMessage);

			sendContactEmail(booking, contact, null, subject, message, extraTagSources);
		}

		if (calendar.isSendRebookedUserSMS()) {

			List<User> managersOld = BookingUtils.getManagers(oldSlot, calendar, getUserHandler(), false);
			List<User> managersNew = BookingUtils.getManagers(booking.getBookingSlot(), calendar, getUserHandler(), false);

			Contact contact = BookingUtils.getContact(booking, getUserHandler());

			List<TagSource> extraTagSources = new ArrayList<>();
			extraTagSources.add(BOOKING_SLOT_OLD_TAG_SOURCE_FACTORY.getTagSource(oldSlot));
			extraTagSources.add(new SingleTagSource("$managersOld", BookingUtils.getManagerNames(managersOld)));
			extraTagSources.add(new SingleTagSource("$managersNew", BookingUtils.getManagerNames(managersNew)));

			String message = ObjectUtils.getFirstNotNull(calendar.getRebookedUserSMS(), rebookedUserSMS);

			sendContactSMS(booking, contact, null, message, extraTagSources);
		}
	}

	public void sendManualReminderNotification(User user, Booking booking) {

		Contact contact = BookingUtils.getContact(booking, systemInterface.getUserHandler());

		if (contact != null) {

			log.info("User " + user + " sending booking reminder for booking " + booking + " to contact " + contact);

			BookingCalendar calendar = booking.getBookingSlot().getCalendar();

			String subject = ObjectUtils.getFirstNotNull(calendar.getManualReminderEmailSubject(), manualReminderEmailSubject);
			String message = ObjectUtils.getFirstNotNull(calendar.getManualReminderEmailMessage(), manualReminderEmailMessage);

			sendContactEmail(booking, contact, null, subject, message, null);

			if (calendar.isSendManualReminderSMS()) {

				String sms = ObjectUtils.getFirstNotNull(calendar.getManualReminderSMS(), manualReminderSMS);

				sendContactSMS(booking, contact, null, sms, null);
			}
		}
	}

	public boolean sendContactEmail(Booking booking, Contact contact, User poster, String subject, String message, List<TagSource> extraTagSources) {

		if (!contact.isContactByEmail() || contact.getEmail() == null || subject == null || message == null) {

			return false;
		}

		BookingSlot bookingSlot = booking.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();

		if (bookingSlot.isMuteContactNotifications()) {

			log.info("Contact notifications is muted for booking slot " + bookingSlot + ", not sending email to " + contact);
			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(BOOKING_TAG_SOURCE_FACTORY.getTagSource(booking));
		tagReplacer.addTagSource(BOOKING_SLOT_TAG_SOURCE_FACTORY.getTagSource(bookingSlot));
		tagReplacer.addTagSource(CALENDAR_TAG_SOURCE_FACTORY.getTagSource(calendar));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		tagReplacer.addTagSource(new SingleTagSource("$booking.url", getUserBookingOverviewURL(booking)));
		tagReplacer.addTagSource(new SingleTagSource("$booking.messagesUrl", getUserBookingSlotModuleAlias(booking) + "/messages/" + booking.getBookingID()));

		if (extraTagSources != null) {

			tagReplacer.addTagSources(extraTagSources);
		}

		if (poster != null) {

			tagReplacer.addTagSource(MANAGER_TAG_SOURCE_FACTORY.getTagSource(poster));
		}

		List<User> slotManagers = BookingUtils.getManagers(bookingSlot, calendar, getUserHandler(), false);

		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managers", BookingUtils.getManagerNames(slotManagers)));
		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managerFirstNames", BookingUtils.getManagerFirstNames(slotManagers)));

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {

			email.addRecipient(contact.getEmail());
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(ObjectUtils.getFirstNotNull(calendar.getEmailSenderName(), getEmailSenderName()));
			email.setSenderAddress(ObjectUtils.getFirstNotNull(calendar.getEmailSenderAddress(), getEmailSenderAddress()));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);
			return true;

		} catch (Exception e) {

			log.info("Error generating/sending email " + email, e);
		}

		return false;
	}

	public boolean sendContactSMS(Booking booking, Contact contact, User poster, String message, List<TagSource> extraTagSources) {

		if (!overrideContactBySMS && !contact.isContactBySMS() || contact.getMobilePhone() == null || smsSender == null || message == null) {

			return false;
		}

		BookingSlot bookingSlot = booking.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();

		if (bookingSlot.isMuteContactNotifications()) {

			log.info("Contact notifications is muted for bookingSlot " + bookingSlot + ", not sending SMS to " + contact);
			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(BOOKING_TAG_SOURCE_FACTORY.getTagSource(booking));
		tagReplacer.addTagSource(BOOKING_SLOT_TAG_SOURCE_FACTORY.getTagSource(bookingSlot));
		tagReplacer.addTagSource(CALENDAR_TAG_SOURCE_FACTORY.getTagSource(calendar));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		tagReplacer.addTagSource(new SingleTagSource("$booking.url", getUserBookingOverviewURL(booking)));
		tagReplacer.addTagSource(new SingleTagSource("$booking.messagesUrl", getUserBookingSlotModuleAlias(booking) + "/messages/" + booking.getBookingID()));

		if (extraTagSources != null) {

			tagReplacer.addTagSources(extraTagSources);
		}

		if (poster != null) {

			tagReplacer.addTagSource(MANAGER_TAG_SOURCE_FACTORY.getTagSource(poster));
		}

		List<User> slotManagers = BookingUtils.getManagers(bookingSlot, calendar, getUserHandler(), false);

		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managers", BookingUtils.getManagerNames(slotManagers)));
		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managerFirstNames", BookingUtils.getManagerFirstNames(slotManagers)));

		SimpleSMS sms = new SimpleSMS();

		try {

			sms.setSenderName(getSmsSenderName(calendar));
			sms.setMessage(tagReplacer.replace(message));
			sms.addRecipient(contact.getMobilePhone());

			return smsSender.send(sms);

		} catch (Exception e) {

			log.info("Error generating/sending sms " + sms, e);
		}

		return false;
	}

	public void sendManagerEmails(Booking booking, Contact contact, String subject, String message, List<User> excludedManagers, List<TagSource> extraTagSources) {

		if (subject == null || message == null) {

			return;
		}

		BookingSlot bookingSlot = booking.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();

		List<User> managers = BookingUtils.getManagers(bookingSlot, calendar, getUserHandler(), calendar.isSendUserRebookedManagerEmailOnlyToBookingSlotManagers());

		if (CollectionUtils.isEmpty(managers) || excludedManagers != null && excludedManagers.containsAll(managers)) {

			return;
		}

		if (excludedManagers != null) {

			managers.removeAll(excludedManagers);
		}

		List<TagSource> sharedTagSources = new ArrayList<>(6);

		sharedTagSources.add(BOOKING_TAG_SOURCE_FACTORY.getTagSource(booking));
		sharedTagSources.add(BOOKING_SLOT_TAG_SOURCE_FACTORY.getTagSource(bookingSlot));
		sharedTagSources.add(CALENDAR_TAG_SOURCE_FACTORY.getTagSource(calendar));
		sharedTagSources.add(new SingleTagSource("$calendar.url", getBookingSlotAdminModuleAlias(booking) + "/list/" + calendar.getCalendarID()));
		sharedTagSources.add(new SingleTagSource("$booking.url", getBookingSlotAdminModuleAlias(booking) + "/show/" + booking.getBookingID()));
		sharedTagSources.add(new SingleTagSource("$booking.messagesUrl", getBookingSlotAdminModuleAlias(booking) + "/messages/" + booking.getBookingID()));

		if (contact != null) {

			sharedTagSources.add(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		}

		managers.forEach(manager -> sendManagerEmail(manager, booking, subject, message, extraTagSources, sharedTagSources));
	}

	private void sendManagerEmail(User manager, Booking booking, String subject, String message, List<TagSource> extraTagSources, List<TagSource> sharedTagSources) {

		if (StringUtils.isEmpty(manager.getEmail())) {

			return;
		}

		BookingSlot bookingSlot = booking.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(MANAGER_TAG_SOURCE_FACTORY.getTagSource(manager));
		tagReplacer.addTagSources(sharedTagSources);

		if (extraTagSources != null) {

			tagReplacer.addTagSources(extraTagSources);
		}

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {

			email.addRecipient(manager.getEmail());
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(ObjectUtils.getFirstNotNull(calendar.getEmailSenderName(), getEmailSenderName()));
			email.setSenderAddress(ObjectUtils.getFirstNotNull(calendar.getEmailSenderAddress(), getEmailSenderAddress()));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);

		} catch (Exception e) {

			log.info("Error generating/sending email " + email, e);
		}
	}

	public void sendGlobalEmail(Booking booking, Contact contact, String address, String subject, String message) {

		sendGlobalEmail(booking, contact, address, subject, message, null);
	}

	public void sendGlobalEmail(Booking booking, Contact contact, String address, String subject, String message, List<TagSource> extraTagSources) {

		if (subject == null || message == null) {

			return;
		}

		TagReplacer tagReplacer = new TagReplacer();
		BookingSlot bookingSlot = booking.getBookingSlot();
		BookingCalendar calendar = bookingSlot.getCalendar();

		tagReplacer.addTagSource(BOOKING_TAG_SOURCE_FACTORY.getTagSource(booking));
		tagReplacer.addTagSource(BOOKING_SLOT_TAG_SOURCE_FACTORY.getTagSource(bookingSlot));
		tagReplacer.addTagSource(CALENDAR_TAG_SOURCE_FACTORY.getTagSource(calendar));
		tagReplacer.addTagSource(new SingleTagSource("$calendar.url", getBookingSlotAdminModuleAlias(booking) + "/list/" + calendar.getCalendarID()));
		tagReplacer.addTagSource(new SingleTagSource("$booking.url", getBookingSlotAdminModuleAlias(booking) + "/show/" + booking.getBookingID()));
		tagReplacer.addTagSource(new SingleTagSource("$booking.messagesUrl", getBookingSlotAdminModuleAlias(booking) + "/messages/" + booking.getBookingID()));

		if (contact != null) {

			tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		}

		if (extraTagSources != null) {

			tagReplacer.addTagSources(extraTagSources);
		}

		List<User> slotManagers = BookingUtils.getManagers(bookingSlot, calendar, getUserHandler(), false);

		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managers", BookingUtils.getManagerNames(slotManagers)));
		tagReplacer.addTagSource(new SingleTagSource("$bookingSlot.managerFirstNames", BookingUtils.getManagerFirstNames(slotManagers)));

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {

			email.addRecipient(address);
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(ObjectUtils.getFirstNotNull(calendar.getEmailSenderName(), getEmailSenderName()));
			email.setSenderAddress(ObjectUtils.getFirstNotNull(calendar.getEmailSenderAddress(), getEmailSenderAddress()));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);

		} catch (Exception e) {

			log.info("Error generating/sending email " + email, e);
		}
	}

	public void setCalendarTemplatedFields(BookingCalendar calendar) {

		TemplateUtils.setTemplatedFields(calendar, this);

		setFlowNotificationHandlerTemplatedFields(calendar);
	}

	public void clearUnchangedTemplatedFields(BookingCalendar calendar) {

		TemplateUtils.clearUnchangedTemplatedFields(calendar, this);
	}

	private void setFlowNotificationHandlerTemplatedFields(BookingCalendar calendar) {

		if (calendar.getEmailSenderName() == null) {

			calendar.setEmailSenderName(getEmailSenderName());
		}

		if (calendar.getEmailSenderAddress() == null) {

			calendar.setEmailSenderAddress(getEmailSenderAddress());
		}
	}

	public String getSmsSenderName(BookingCalendar calendar) {

		String alias = getProfileSetting(calendar.getProfileID(), "smsSenderNameSetting");

		return alias != null ? alias : notificationHandler.getSMSSenderName(null);
	}

	private Contact getContact(Booking booking) {

		return BookingUtils.getContact(booking, getUserHandler());
	}

	private UserHandler getUserHandler() {

		return systemInterface.getUserHandler();
	}

	public String getUserBookingSlotModuleAlias(Booking booking) {

		if(booking.getBookingSlot().getCalendar().getProfileID() != null) {
		
			String alias = getProfileSetting(booking.getBookingSlot().getCalendar().getProfileID(), "userBookingModuleAliasSetting");

			return alias != null ? alias : userBookingSlotModuleAlias;
		}
		
		return userBookingSlotModuleAlias;
	}

	public String getBookingSlotAdminModuleAlias() {

		return getBookingSlotAdminModuleAlias(null);
	}

	public String getBookingSlotAdminModuleAlias(Booking booking) {

		if (booking != null) {

			String alias = getProfileSetting(booking.getBookingSlot().getCalendar().getProfileID(), "bookingSlotAdminModuleAliasSetting");

			if (alias != null) {

				return alias;
			}
		}

		return bookingSlotAdminModuleAlias;
	}

	public String getUserBookingOverviewURL(Booking booking) {

		return getUserBookingSlotModuleAlias(booking) + "/overview/" + booking.getBookingID();
	}

	public String getProfileSetting(Integer profileID, String settingID) {

		if (enableProfileSupport && siteProfileHandler != null && profileID != null) {

			ProfileSettingHandler settingHandler = siteProfileHandler.getProfileSettingHandler(profileID);

			if (settingHandler != null) {

				return settingHandler.getString(settingID);
			}
		}

		return null;
	}

	public String getEmailSenderAddress() {

		return notificationHandler.getEmailSenderAddress(null);
	}

	public String getEmailSenderName() {

		return notificationHandler.getEmailSenderName(null);
	}

	public String getFlowInstanceAdminModuleAlias(ImmutableFlowInstance flowInstance) {

		return notificationHandler.getFlowInstanceAdminModuleAlias(flowInstance);
	}

	public SiteProfileHandler getSiteProfileHandler() {

		return siteProfileHandler;
	}

	@Override
	public List<Setting> getSiteProfileSettings() {

		return profileSettings;
	}

	@Override
	public List<Setting> getSiteSubProfileSettings() {

		return null;
	}

	@Override
	public String getModuleName() {

		return moduleDescriptor.getName();
	}

	public boolean isEnableProfileSupport() {

		return enableProfileSupport;
	}

	public boolean hasSMSSender() {

		return smsSender != null;
	}

}
