package com.nordicpeak.flowengine.booking.queries;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.templates.TemplateUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class BookingQueryCRUD extends BaseQueryCRUD<BookingQuery, BookingQueryProviderModule> {

	public BookingQueryCRUD(AnnotatedDAOWrapper<BookingQuery, Integer> queryDAO, BeanRequestPopulator<BookingQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, BookingQueryProviderModule callback) {

		super(BookingQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
	}

	@Override
	protected void appendUpdateFormData(BookingQuery query, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(query, doc, updateTypeElement, user, req, uriParser);

		Element bookingsElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Calendars");

		List<BookingCalendar> calendars = callback.getCalendars();

		XMLUtils.append(doc, bookingsElement, calendars);

		if (query.getCalendar() != null && (CollectionUtils.isEmpty(calendars) || !calendars.contains(query.getCalendar()))) {

			bookingsElement.appendChild(query.getCalendar().toXML(doc));
		}

		XMLUtils.append(doc, updateTypeElement, "Statuses", callback.getFlowStatuses(query.getQueryDescriptor().getStep().getFlow().getFlowID()));
	}

	@Override
	protected void appendBean(BookingQuery bean, Element targetElement, Document doc, User user) {

		TemplateUtils.setTemplatedFields(bean, callback);

		super.appendBean(bean, targetElement, doc, user);
	}

	@Override
	protected BookingQuery populateFromUpdateRequest(BookingQuery oldQuery, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		BookingQuery query = super.populateFromUpdateRequest(oldQuery, req, user, uriParser);

		TemplateUtils.clearUnchangedTemplatedFields(query, callback);

		List<ValidationError> validationErrors = new ArrayList<>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		Integer calendarID = ValidationUtils.validateParameter("calendarID", req, true, IntegerPopulator.getPopulator(), validationErrors);

		if (calendarID != null) {

			BookingCalendar calendar = callback.getCalendar(calendarID);

			if (calendar == null) {

				if (query.getCalendar() == null || !query.getCalendar().getCalendarID().equals(calendarID)) {

					validationErrors.add(new ValidationError("calendarID", ValidationErrorType.InvalidFormat));
				}

			} else {

				query.setCalendar(calendar);
				query.setAcceptText(ValidationUtils.validateParameter("acceptText", req, calendar.isRequireConfirmation(), 0, 255, StringPopulator.getPopulator(), validationErrors));

				if (calendar.getPrice() != null && calendar.getPrice() > 0 && query.getInvoiceLineText() == null) {

					validationErrors.add(new ValidationError("invoiceLineText", ValidationErrorType.RequiredField));
				}
			}
		}

		if (query.getArchiveStatusID() != null) {

			List<? extends ImmutableStatus> statuses = callback.getFlowStatuses(query.getQueryDescriptor().getStep().getFlow().getFlowID());

			ImmutableStatus foundStatus = CollectionUtils.find(statuses, status -> status.getStatusID().equals(query.getArchiveStatusID()));

			if (foundStatus == null) {

				validationErrors.add(new ValidationError("archiveStatusID", ValidationErrorType.InvalidFormat));
			}
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		return query;
	}

	@Override
	protected List<Field> getBeanRelations() {

		return Arrays.asList(BookingQuery.CALENDAR_RELATION);
	}

}
