package com.nordicpeak.flowengine.booking.queries;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.function.Function;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.bool.BooleanUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.enums.DayOfWeek;
import se.unlogic.standardutils.populators.BooleanPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.SimpleImmutableAlternative;
import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.booking.BookingUtils;
import com.nordicpeak.flowengine.booking.CalendarAdminModule;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.cruds.BookingCRUD;
import com.nordicpeak.flowengine.booking.enums.BookingCalendarViewMode;
import com.nordicpeak.flowengine.booking.events.BookingArchiveEvent;
import com.nordicpeak.flowengine.booking.events.BookingPassedEvent;
import com.nordicpeak.flowengine.booking.exceptions.PersonAlreadyInBookingSlotValidationException;
import com.nordicpeak.flowengine.booking.exceptions.PersonAlreadyInCalendarValidationException;
import com.nordicpeak.flowengine.booking.populators.BookingSlotWithCountPopulator;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class BookingQueryProviderModule extends BaseQueryProviderModule<BookingQueryInstance> {

	private static final Field[] EXCLUDED_BOOKING_SLOT_FIELDS = { BookingSlot.FIELD_POSTER, BookingSlot.FIELD_UPDATER };

	@XSLVariable(name = "java.defaultPriceText")
	private String customPriceText;

	@XSLVariable(prefix = "java.")
	private String defaultInvoiceLineDescription;

	@XSLVariable(prefix = "java.")
	private String exportDate;

	@XSLVariable(prefix = "java.")
	private String exportTime;

	@XSLVariable(prefix = "java.")
	private String exportLocation;

	@XSLVariable(prefix = "java.")
	private String exportManagers;

	@XSLVariable(prefix = "java.")
	private String exportComment;

	@XSLVariable(prefix = "java.")
	private String bookingSlotSelectedAlternativeName = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextOpens = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextClosed = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextBetween = "This variable should be set by your stylesheet";

	private AnnotatedDAO<BookingQuery> queryDAO;
	private AnnotatedDAO<BookingQueryInstance> queryInstanceDAO;
	private AnnotatedDAO<Booking> bookingDAO;
	private AnnotatedDAO<BookingCalendar> calendarDAO;
	private AnnotatedDAO<BookingSlot> bookingSlotDAO;
	private AnnotatedDAO<BookingSlot> bookingSlotDAOWithBookingCount;

	private AnnotatedDAOWrapper<BookingCalendar, Integer> calendarDAOWrapper;

	private QueryParameterFactory<BookingQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<BookingQueryInstance, Integer> queryInstanceIDParamFactory;
	private QueryParameterFactory<BookingCalendar, Boolean> calendarEnabledParamFactory;

	private BookingQueryCRUD queryCRUD;

	private ImmutableAlternative bookingSlotSelectedAlternative;

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);

		bookingSlotSelectedAlternative = new SimpleImmutableAlternative(bookingSlotSelectedAlternativeName, 1, 1);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, CalendarAdminModule.class.getName(), new XMLDBScriptProvider(CalendarAdminModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		queryDAO = daoFactory.getDAO(BookingQuery.class);
		queryInstanceDAO = daoFactory.getDAO(BookingQueryInstance.class);
		calendarDAO = daoFactory.getDAO(BookingCalendar.class);
		bookingDAO = daoFactory.getDAO(Booking.class);
		bookingSlotDAO = daoFactory.getDAO(BookingSlot.class);
		bookingSlotDAOWithBookingCount = new AnnotatedDAO<>(dataSource, BookingSlot.class, daoFactory, new BookingSlotWithCountPopulator(daoFactory.getBeanStringPopulators()), daoFactory.getQueryParameterPopulators(), daoFactory.getBeanStringPopulators());

		queryCRUD = new BookingQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(BookingQuery.class), "BookingQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);

		calendarEnabledParamFactory = calendarDAO.getParamFactory("enabled", boolean.class);

		calendarDAOWrapper = calendarDAO.getWrapper(Integer.class);
		calendarDAOWrapper.addRelations(BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION);
		calendarDAOWrapper.setUseRelationsOnGet(true);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		BookingQuery query = new BookingQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setBookingSlotSelectedAlternative(bookingSlotSelectedAlternative);
		query.setInvoiceLineTextFallback(defaultInvoiceLineDescription);

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		BookingQuery query = new BookingQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));

		contentFilter.filterHTML(query);

		setArchiveStatusIDFromConversionMap(query, statusConversionMap);

		setBookingSlotPassedStatusIDFromConversionMap(query, statusConversionMap);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	private void setArchiveStatusIDFromConversionMap(BookingQuery query, Map<Integer, ImmutableStatus> statusConversionMap) {

		if (query.getArchiveStatusID() != null) {

			ImmutableStatus newStatus = statusConversionMap.get(query.getArchiveStatusID());

			if (newStatus != null) {

				query.setArchiveStatusID(newStatus.getStatusID());
			}
		}
	}

	private void setBookingSlotPassedStatusIDFromConversionMap(BookingQuery query, Map<Integer, ImmutableStatus> statusConversionMap) {

		if (query.getBookingSlotPassedStatusID() != null) {

			ImmutableStatus newStatus = statusConversionMap.get(query.getBookingSlotPassedStatusID());

			if (newStatus != null) {

				query.setBookingSlotPassedStatusID(newStatus.getStatusID());
			}
		}
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		BookingQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setBookingSlotSelectedAlternative(bookingSlotSelectedAlternative);
		query.setInvoiceLineTextFallback(defaultInvoiceLineDescription);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		BookingQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setBookingSlotSelectedAlternative(bookingSlotSelectedAlternative);
		query.setInvoiceLineTextFallback(defaultInvoiceLineDescription);

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		BookingQueryInstance queryInstance = descriptor.getQueryInstanceID() != null ? getQueryInstance(descriptor.getQueryInstanceID()) : new BookingQueryInstance();

		if (queryInstance == null) {

			return null;
		}

		BookingQuery query = getQuery(descriptor.getQueryDescriptor().getQueryID());

		if (query == null) {

			return null;
		}

		queryInstance.setQuery(query);

		setBookingSlotsFromDB(queryInstance);

		queryInstance.getQuery().setBookingSlotSelectedAlternative(bookingSlotSelectedAlternative);
		queryInstance.getQuery().setInvoiceLineTextFallback(defaultInvoiceLineDescription);

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private BookingQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<BookingQuery> query = new HighLevelQuery<>(BookingQuery.CALENDAR_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.MANAGER_USERS_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));
		query.addExcludedFields(EXCLUDED_BOOKING_SLOT_FIELDS);

		return queryDAO.get(query);
	}

	private BookingQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<BookingQuery> query = new HighLevelQuery<>(BookingQuery.CALENDAR_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));
		query.addExcludedFields(EXCLUDED_BOOKING_SLOT_FIELDS);

		return queryDAO.get(query, transactionHandler);
	}

	private BookingQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<BookingQueryInstance> query = new HighLevelQuery<>(BookingQueryInstance.BOOKING_SLOT_RELATION, BookingSlot.MANAGERS_RELATION, BookingSlot.CALENDAR_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	public void setBookingSlotsFromDB(BookingQueryInstance queryInstance) throws SQLException {

		BookingCalendar calendar = queryInstance.getQuery().getCalendar();

		if (calendar == null) {

			return;
		}

		Calendar cal = Calendar.getInstance();

		if (queryInstance.getQuery().getMinVisibleDays() == null) {

			cal.add(Calendar.WEEK_OF_YEAR, -1);
			cal.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);

		} else {

			cal.setTimeInMillis(System.currentTimeMillis());
			cal.add(Calendar.DAY_OF_MONTH, queryInstance.getQuery().getMinVisibleDays());
		}

		Timestamp earliestStart = new Timestamp(cal.getTimeInMillis());

		StringBuilder sql = new StringBuilder();
		sql.append("SELECT bs.*, b.bookingsCount FROM " + bookingSlotDAO.getTableName() + " bs");
		sql.append(" LEFT OUTER JOIN (");
		sql.append("  SELECT SUM(size) AS 'bookingsCount', bookingSlotID FROM " + bookingDAO.getTableName() + " WHERE archived = 0 GROUP BY bookingSlotID");
		sql.append(" ) b ON bs.bookingSlotID = b.bookingSlotID");
		sql.append(" WHERE bs.calendarID = ? AND bs.start >= ?");

		if (!calendar.isShowFullyBookedDaysInCalendar() && !calendar.isShowFullyBookedSlotsInList()) {

			sql.append(" AND ((b.bookingsCount IS NULL AND bs.size > 0) OR b.bookingsCount < bs.size)");
		}

		boolean shouldFilterOnBookingSize = queryInstance.isRebooking() && calendar.isRestrictRebookingToCurrentBookingSize();
		
		if (shouldFilterOnBookingSize) {

			sql.append(" AND bs.size - IFNULL(b.bookingsCount, 0) >= ?");
		}

		sql.append(" ORDER BY bs.start, bs.end, bs.added");

		LowLevelQuery<BookingSlot> query = new LowLevelQuery<>(sql.toString());
		query.addRelations(BookingSlot.CALENDAR_RELATION, BookingSlot.MANAGERS_RELATION);

		query.addParameter(calendar.getCalendarID());
		query.addParameter(earliestStart);

		if (shouldFilterOnBookingSize) {

			query.addParameter(queryInstance.getSize());
		}

		List<BookingSlot> bookingSlots = bookingSlotDAOWithBookingCount.getAll(query);

		removeSlotsHiddenByWeekday(calendar, bookingSlots);

		BookingUtils.setNotBookableText(bookingSlots, bookingSlotNotBookableTextBetween, bookingSlotNotBookableTextOpens, bookingSlotNotBookableTextClosed);

		calendar.setBookingSlots(bookingSlots);
	}

	private void removeSlotsHiddenByWeekday(BookingCalendar calendar, List<BookingSlot> bookingSlots) {

		if (calendar.isHideBookingsPerWeekday() && !CollectionUtils.isEmpty(calendar.getWeekdaysWithHiddenBookings()) && !CollectionUtils.isEmpty(bookingSlots)) {

			Calendar cal = Calendar.getInstance();

			for (Iterator<BookingSlot> it = bookingSlots.iterator(); it.hasNext();) {

				BookingSlot bookingSlot = it.next();

				cal.setTime(bookingSlot.getStart());

				DayOfWeek dayOfWeek = DayOfWeek.fromInt(cal.get(Calendar.DAY_OF_WEEK), true);

				if (CollectionUtils.contains(calendar.getWeekdaysWithHiddenBookings(), dayOfWeek)) {

					it.remove();
				}
			}
		}
	}

	@Override
	public void save(BookingQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public QueryResponse getFormHTML(BookingQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (requestMetadata.flowInstanceIsSubmitted() && !queryInstance.isRebooking()) {

			return getShowHTML(queryInstance, req, user, poster, null, queryRequestURL, requestMetadata, attributeHandler);
		}

		if (validationErrors == null && poster != null) {

			String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

			if (citizenIdentifier == null) {

				validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("NoCitizenIdentifierFound"));

			}

		}

		if (queryInstance.getSubmitValidationErrors() != null) {
			
			validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, queryInstance.getSubmitValidationErrors());
			queryInstance.resetSubmitValidationError();
		}

		if (queryInstance.getQuery().getCalendar() == null && CollectionUtils.isEmpty(validationErrors)) {

			validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("QueryNotConfigured"));
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	public boolean isValidBooking(BookingQueryInstance queryInstance, User poster, FlowInstanceManager flowInstanceManager) throws SQLException {

		BookingSlot selectedBookingSlot = queryInstance.getBookingSlot();
		BookingCalendar calendar = queryInstance.getQuery().getCalendar();

		List<BookingQueryInstance> queryInstances = flowInstanceManager.getQueries(BookingQueryInstance.class);

		List<BookingQueryInstance> otherPosterQueryInstances = CollectionUtils.filter(queryInstances, qi -> !qi.getQuery().equals(queryInstance.getQuery()) && qi.getBookingSlot() != null && qi.getBookingSlot().equals(selectedBookingSlot));
		
		int otherQueryInstancesSize = getSize(otherPosterQueryInstances);
		
		if (otherQueryInstancesSize > 0 && !calendar.isAllowDuplicateBookings()) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			queryInstance.setSubmitValidationError(new ValidationError("DuplicateBookings"));
			return false;
		}
		
		if (queryInstance.getSize() + otherQueryInstancesSize > selectedBookingSlot.getFreeSlots()) {
			
			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			queryInstance.setSubmitValidationError(new ValidationError("NotEnoughFreeSlots"));
			return false;
		}

		setBookingSlotsFromDB(queryInstance);

		BookingSlot bookingSlot = CollectionUtils.find(calendar.getBookingSlots(), slot -> slot.equals(selectedBookingSlot));
		
		queryInstance.setBookingSlot(bookingSlot);

		try {

			BookingCRUD.validateBookingSlot(bookingSlot, queryInstance.getSize(), false, calendar);

		} catch (ValidationException e) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			queryInstance.setSubmitValidationErrors(e.getErrors());
			return false;
		}

		String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);

		if (citizenIdentifier != null) {

			try {

				BookingCRUD.validateCitizenIdentifier(citizenIdentifier, calendar, bookingSlot, false, dataSource);

			} catch (PersonAlreadyInCalendarValidationException | PersonAlreadyInBookingSlotValidationException e) {

				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				queryInstance.setSubmitValidationErrors(e.getErrors());
				return false;
			}
		}

		queryInstance.setSubmitValidationErrors(null);
		return true;
	}

	private int getSize(List<BookingQueryInstance> queryInstances) {

		int size = 0;
		
		if (queryInstances != null) {
			
			for (BookingQueryInstance queryInstance : queryInstances) {
				
				size += queryInstance.getSize();
			}
		}
		
		return size;
	}

	@Override
	public void populate(BookingQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if (requestMetadata.flowInstanceIsSubmitted() && !queryInstance.isRebooking()) {

			return;
		}

		String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);

		if (poster != null && citizenIdentifier == null) {

			throw new ValidationException(new ValidationError("NoCitizenIdentifierFound"));
		}

		BookingCalendar calendar = queryInstance.getQuery().getCalendar();

		if (calendar == null) {

			throw new ValidationException(new ValidationError("QueryNotConfigured"));
		}
		
		try {
			
			setBookingSlotsFromDB(queryInstance);
			
		} catch (SQLException e) {
			
			throw new RuntimeException(e);
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		Integer selectedSlotID = ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_bookingSlotID", req, false, IntegerPopulator.getPopulator(), validationErrors);

		BookingSlot selectedBookingSlot = null;

		if (selectedSlotID != null) {

			selectedBookingSlot = CollectionUtils.find(calendar.getBookingSlots(), slot -> slot.getBookingSlotID().equals(selectedSlotID));
		}

		boolean accepted = true;

		if (calendar.isRequireConfirmation()) {

			accepted = BooleanUtils.valueOf(ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_accept", req, false, BooleanPopulator.getPopulator(), validationErrors));
		}

		String comment = null;

		if (queryInstance.getQuery().showCommentField()) {

			comment = ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_comment", req, false, 0, 65534, StringPopulator.getPopulator(), validationErrors);
		}

		Integer size = ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_size", req, false, PositiveStringIntegerPopulator.getPopulator(), validationErrors);

		if (size == null) {

			size = 1;
		}

		if (!allowPartialPopulation) {

			if (selectedSlotID != null) {

				BookingCRUD.validateBookingSlot(selectedBookingSlot, size, false, calendar);
			}

			if (selectedBookingSlot != null && poster != null) {

				validateCitizenIdentfier(citizenIdentifier, calendar, selectedBookingSlot, queryInstance.isRebooking());
			}

			if (selectedBookingSlot != null && !accepted) {

				validationErrors.add(new ValidationError("MustAccept", queryInstance.getQuery().getAcceptText()));

			} else if (!accepted && !StringUtils.isEmpty(comment)) {

				validationErrors.add(new ValidationError("MustAcceptToComment", queryInstance.getQuery().getAcceptText()));

			} else if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && (!accepted || selectedBookingSlot == null)) {

				validationErrors.add(new ValidationError("RequiredQuery"));
			}

			if (!validationErrors.isEmpty()) {

				throw new ValidationException(validationErrors);
			}
		}

		queryInstance.setBookingSlot(selectedBookingSlot);
		queryInstance.setAccepted(accepted);
		queryInstance.setComment(comment);
		queryInstance.setSize(size);
		queryInstance.getQueryInstanceDescriptor().setPopulated(selectedBookingSlot != null && accepted);

		if (queryInstance.getQuery().isSetAsAttribute()) {

			if (queryInstance.getQueryInstanceDescriptor().isPopulated()) {

				queryInstance.setAttribute(attributeHandler, systemInterface.getUserHandler(), calendar);

			} else {

				queryInstance.resetAttribute(attributeHandler);
			}
		}
	}

	private void validateCitizenIdentfier(String citizenIdentifier, BookingCalendar calendar, BookingSlot selectedBookingSlot, boolean rebooking) throws PersonAlreadyInCalendarValidationException, PersonAlreadyInBookingSlotValidationException {

		try {

			BookingCRUD.validateCitizenIdentifier(citizenIdentifier, calendar, selectedBookingSlot, rebooking, dataSource);

		} catch (SQLException e) {

			throw new RuntimeException(e);
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		BookingQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		BookingQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		BookingQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		setArchiveStatusIDFromConversionMap(query, statusConversionMap);

		setBookingSlotPassedStatusIDFromConversionMap(query, statusConversionMap);

		this.queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendQueryInstance(BookingQueryInstance queryInstance, Document doc, Element targetElement, AttributeHandler attributeHandler) {

		super.appendQueryInstance(queryInstance, doc, targetElement, attributeHandler);

		BookingCalendar calendar = queryInstance.getQuery().getCalendar();

		if (calendar == null) {

			return;
		}

		List<BookingSlot> bookingSlots = calendar.getBookingSlots();

		XMLUtils.appendNewElement(doc, targetElement, "BookingSlotsJSON", BookingUtils.getBookingSlotsJSON(calendar, false, null, bookingSlots).toJson());
		XMLUtils.appendNewElement(doc, targetElement, "BookingSlotsTitleJSON", BookingUtils.getBookingSlotsTitleJSON(bookingSlots, false).toJson());

		if (!CollectionUtils.isEmpty(bookingSlots) && calendar.getViewMode() != BookingCalendarViewMode.LIST && queryInstance.getQuery().isShowLocationFilter()) {

			List<String> locations = CollectionUtils.map(bookingSlots, BookingSlot::getLocation);

			if (!CollectionUtils.isEmpty(locations)) {

				Set<String> sortedUniqueLocations = new TreeSet<>(locations);

				XMLUtils.append(doc, targetElement, "Locations", "Location", sortedUniqueLocations);
			}
		}

		boolean noFreeSlots = CollectionUtils.find(bookingSlots, slot -> slot.getFreeSlots() > 0 && !slot.hasPassed() && slot.isBookable()) == null;

		XMLUtils.appendNewElement(doc, targetElement, "NoFreeSlots", noFreeSlots);

		if (calendar.isShowManagers()) {

			BookingUtils.appendCalendarManagers(doc, targetElement, bookingSlots, calendar.isHideDisabledManagersInFilter());
		}
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, BookingQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<BookingQueryInstance> getQueryInstanceClass() {

		return BookingQueryInstance.class;
	}

	public List<BookingCalendar> getCalendars() throws SQLException {

		HighLevelQuery<BookingCalendar> query = new HighLevelQuery<>();

		query.addParameter(calendarEnabledParamFactory.getParameter(true));

		return calendarDAO.getAll(query);
	}

	public BookingCalendar getCalendar(Integer calendarID) throws SQLException {

		BookingCalendar calendar = calendarDAOWrapper.get(calendarID);

		if (calendar != null && !calendar.isEnabled()) {

			return null;
		}

		return calendar;
	}

	public List<? extends ImmutableStatus> getFlowStatuses(Integer flowID) {

		ImmutableFlow flow = flowAdminModule.getFlow(flowID);

		if (flow != null) {

			return flow.getStatuses();
		}

		return null;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(BookingQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws IOException {

		return new BookingQueryRequestProcessor(queryInstance, systemInterface);
	}

	public String getExportDate() {

		return exportDate;
	}

	public String getExportTime() {

		return exportTime;
	}

	public String getExportLocation() {

		return exportLocation;
	}

	public String getExportComment() {

		return exportComment;
	}

	@EventListener(channel = Booking.class)
	public void processEvent(BookingPassedEvent event, EventSource source) {

		if (source.isLocal()) {

			Booking booking = event.getBooking();

			if (booking.getFlowInstanceID() != null) {

				try {

					String info = "Booking updated by the system, changing status of instance ";
					alterFlowInstanceStatus(booking, BookingQuery::getBookingSlotPassedStatusID, info);

				} catch (Exception e) {

					log.error("Error updating status of " + booking + " and its related flow instance " + booking.getFlowInstanceID(), e);
				}
			}
		}
	}

	@EventListener(channel = Booking.class)
	public void processEvent(BookingArchiveEvent event, EventSource source) {

		if (source.isLocal()) {

			Booking booking = event.getBooking();

			if (booking.getFlowInstanceID() != null) {

				try {

					String info = "Booking archived by " + event.getPoster() + ", changing status of instance ";
					alterFlowInstanceStatus(booking, BookingQuery::getArchiveStatusID, info);

				} catch (Exception e) {

					log.error("Error updating status of " + booking + " and its related flow instance " + booking.getFlowInstanceID(), e);
				}
			}
		}
	}

	private void alterFlowInstanceStatus(Booking booking, Function<BookingQuery, Integer> statusIDGetter, String info) throws Exception {

		ImmutableFlowInstanceManager flowInstanceManager = flowAdminModule.getImmutableFlowInstanceManager(booking.getFlowInstanceID());

		if (flowInstanceManager == null) {

			log.warn("Could not get flowInstanceManager for booking " + booking);
			return;
		}

		List<BookingQueryInstance> queryInstances = flowInstanceManager.getQueries(BookingQueryInstance.class);

		if (queryInstances == null) {

			log.warn("Could not get queryInstances for booking " + booking);
			return;
		}

		FlowInstance flowInstance = (FlowInstance) flowInstanceManager.getFlowInstance();
		List<Status> statuses = flowInstance.getFlow().getStatuses();

		if (CollectionUtils.isEmpty(statuses)) {

			log.warn("Could not get statuses for booking " + booking);
			return;
		}

		Status statusID = null;

		for (BookingQueryInstance queryInstance : queryInstances) {

			BookingQuery query = queryInstance.getQuery();

			if (!queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getBookingSlot() == null) {

				continue;
			}

			Integer bookingBookingSlotID = booking.getBookingSlot().getBookingSlotID();
			Integer queryInstanceBookingSlotID = queryInstance.getBookingSlot().getBookingSlotID();

			Integer bookingQueryStatusID = statusIDGetter.apply(query);

			if (bookingBookingSlotID.equals(queryInstanceBookingSlotID) && bookingQueryStatusID != null) {

				statusID = CollectionUtils.find(statuses, status -> status.getStatusID().equals(bookingQueryStatusID));
			}
		}

		if (statusID == null) {

			log.warn("Could not get new status id for booking " + booking);
			return;
		}

		if (statusID.equals(flowInstance.getStatus())) {

			log.warn("Status " + flowInstance.getStatus() + " is already set on flow instance in booking " + booking);
			return;
		}

		Status previousStatus = flowInstance.getStatus();

		log.info(info + flowInstance + " from " + previousStatus + " to " + statusID);

		flowInstance.setStatus(statusID);
		flowInstance.setLastStatusChange(TimeUtils.getCurrentTimestamp());
		flowAdminModule.getDAOFactory().getFlowInstanceDAO().update(flowInstance);

		flowAdminModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstance, EventType.STATUS_UPDATED, null, null);

		systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<>(CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
	}

}
