<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/featherlight/js/featherlight.min.js
	</xsl:variable>

	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/moment.min.js
		/js/moment.sv.js
		/js/handlebars.min.js
		/js/clndr.min.js
		/js/bookingquery.js?v=14
		/js/bookingcommon.js?v=1
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/calendar.css?v=4
		/css/bookingquery.css?v=4
		/css/bookingcommon.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
	
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="BookingQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="BookingQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="BookingQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="BookingQueryInstance/BookingQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="BookingQueryInstance/BookingQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="BookingQueryInstance/BookingQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:apply-templates select="BookingQueryInstance/BookingQuery/Calendar"/>
				
				<p>
					<strong><xsl:value-of select="$i18n.BookingSlot" /></strong>
					<br/>
					
					<xsl:if test="BookingQueryInstance/BookingSlot/location and BookingQueryInstance/BookingQuery/showLocationInList = 'true' and BookingQueryInstance/BookingQuery/Calendar/viewMode = 'LIST'">
						<xsl:value-of select="BookingQueryInstance/BookingSlot/location"/>
						<xsl:text> </xsl:text>
					</xsl:if>
					
					<xsl:value-of select="BookingQueryInstance/BookingSlot/startDate" />
					<xsl:text> </xsl:text>
					<xsl:call-template name="printBookingSlotTime">
						<xsl:with-param name="bookingSlot" select="BookingQueryInstance/BookingSlot"/>
						<xsl:with-param name="calendar" select="BookingQueryInstance/BookingQuery/Calendar"/>
					</xsl:call-template>
				</p>
				
				<xsl:if test="BookingQueryInstance/BookingSlot/location and not(BookingQueryInstance/BookingQuery/showLocationInList = 'true' and BookingQueryInstance/BookingQuery/Calendar/viewMode = 'LIST')">
					<p>
						<strong><xsl:value-of select="$i18n.BookingSlot.Location" /></strong>
						<br/>
						<xsl:value-of select="BookingQueryInstance/BookingSlot/location"/>
					</p>
				</xsl:if>
				
				<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/showManagers = 'true' and BookingQueryInstance/BookingSlot/Managers">
					<p>
						<strong><xsl:value-of select="$i18n.BookingSlot.Managers" /></strong>
						<br/>
						<xsl:apply-templates select="BookingQueryInstance/BookingSlot/Managers/user" mode="manager"/>
					</p>
				</xsl:if>
				
				<xsl:if test="BookingQueryInstance/BookingSlot/description">
					<p>
						<strong><xsl:value-of select="$i18n.BookingSlot.Description" /></strong>
						<br/>
						<xsl:call-template name="replaceLineBreak">
							<xsl:with-param name="string" select="BookingQueryInstance/BookingSlot/description"/>
						</xsl:call-template>
					</p>
				</xsl:if>
				
				<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/allowUserBookingSizeSelection = 'true'">
					<p>
						<strong><xsl:value-of select="$i18n.BookingSlot.Size" /></strong>
						<br/>
						<xsl:value-of select="BookingQueryInstance/size" />
					</p>
				</xsl:if>
				
				<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/requireConfirmation = 'true' and BookingQueryInstance/accepted = 'true'">
				
					<p>
						<xsl:value-of select="BookingQueryInstance/BookingQuery/acceptText" />
					</p>
				
				</xsl:if>
				
				<xsl:if test="BookingQueryInstance/comment">
					<div>
						<strong class="floatleft">
							<xsl:choose>
								<xsl:when test="BookingQueryInstance/BookingQuery/commentName != ''">
									<xsl:value-of select="BookingQueryInstance/BookingQuery/commentName"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$i18n.Comment"/>
								</xsl:otherwise>
							</xsl:choose>
						</strong>
						
						<span class="floatleft clearboth">
							<xsl:call-template name="replaceLineBreak">
								<xsl:with-param name="string" select="BookingQueryInstance/comment"/>
							</xsl:call-template>
						</span>
					</div>
				</xsl:if>
				
			</article>
		
		</div>	
	
	</xsl:template>
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', BookingQueryInstance/BookingQuery/queryID)" />
		<xsl:variable name="qID" select="concat('q', BookingQueryInstance/BookingQuery/queryID)" />
	
		<div id="{$queryID}" data-id="{BookingQueryInstance/BookingQuery/queryID}">
	
			<xsl:attribute name="class">
				<xsl:text>query bookingquery</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="BookingQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
	
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
							<xsl:apply-templates select="ValidationErrors/validationError[messageKey]">
								<xsl:with-param name="id"><xsl:value-of select="$queryID" />_error</xsl:with-param>
							</xsl:apply-templates>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
			
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>

				<xsl:variable name="selectedFieldName">
					<xsl:value-of select="$qID"/>
					<xsl:value-of select="'_bookingSlotID'"/>
				</xsl:variable>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="BookingQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="BookingQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="BookingQueryInstance/BookingQuery/helpText">
						<xsl:apply-templates select="BookingQueryInstance/BookingQuery/helpText" />
					</xsl:if>
					
					<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/viewMode != 'LIST'">

						<select id="view-select" class="floatright clearright" aria-label="{$i18n.CalendarMode}"> 
						
							<xsl:attribute name="style">
								<xsl:if test="(BookingQueryInstance/BookingSlot and not(requestparameters/parameter[name=$selectedFieldName])) or (requestparameters/parameter[name=$selectedFieldName]/value != '')">display: none;</xsl:if>
							</xsl:attribute>
							
							<option value="MONTH"><xsl:value-of select="$i18n.Month" /></option>
							<option value="WEEK"><xsl:value-of select="$i18n.Week" /></option>
							
							<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/viewMode = 'TOGGLE'">
								<option value="LIST"><xsl:value-of select="$i18n.List" /></option>
							</xsl:if>
							
						</select>
				
					</xsl:if>

				</div>
				
				<div class="clearboth"/>
				
				<xsl:if test="BookingQueryInstance/BookingQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="BookingQueryInstance/BookingQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<xsl:if test="not(BookingQueryInstance/BookingQuery/Calendar)">
					<p>
						<xsl:value-of select="$i18n.Error.NoCalendar"/>
					</p>
				</xsl:if>
				
				<xsl:apply-templates select="BookingQueryInstance/BookingQuery/Calendar"/>
				
				<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/viewMode != 'LIST'">
				
					<div class="selecting" style="display: none;">
					
						<xsl:attribute name="style">
							<xsl:if test="BookingQueryInstance/BookingSlot/bookingSlotID or requestparameters/parameter[name=$selectedFieldName]/value != ''">display: none;</xsl:if>
						</xsl:attribute>

						<script type="text/javascript">
							$(function(){
								BookingQueryi18n = {
									OneSlot: "<xsl:value-of select="$js.Calendar.OneSlot" />",
									MultipleSlots: "<xsl:value-of select="$js.Calendar.MultipleSlots" />",
									Free: "<xsl:value-of select="$js.Calendar.Free" />",
									Booked: "<xsl:value-of select="$js.Calendar.Booked" />",
									NotBookable: "<xsl:value-of select="$i18n.BookingSlot.NotBookable" />",
									LocationTitle: "<xsl:value-of select="$i18n.BookingSlot.Location" />",
									DescriptionTitle: "<xsl:value-of select="$i18n.BookingSlot.Description" />",
									ManagersTitle: "<xsl:value-of select="$i18n.BookingSlot.Managers" />",
									SlotPassed: "<xsl:value-of select="$js.Calendar.SlotPassed" />",
									Week: "<xsl:value-of select="$i18n.Week" />",
									Slots: "<xsl:value-of select="$i18n.Slots" />",
									More: "<xsl:value-of select="$i18n.More" />",
									Choose: "<xsl:value-of select="$i18n.Modal.Choose" />",
								}
							});
						</script>
							
						<div class="events-json" style="display: none;">
							<xsl:value-of select="BookingSlotsJSON" />
						</div>
						
						<div class="events-title-json" style="display: none;">
							<xsl:value-of select="BookingSlotsTitleJSON" />
						</div>
						
						<div class="calendar calendar-full block" data-showduration="{BookingQueryInstance/BookingQuery/Calendar/showDuration}" data-showmanagers="{BookingQueryInstance/BookingQuery/Calendar/showManagers}" aria-describedby="{$queryID}_error">
						
							<div class="calendar-template" style="display: none;">
							
								<p class="no-free-slots hidden hugemargintop error">
									<xsl:choose>
										<xsl:when test="BookingQueryInstance/BookingQuery/noFreeSlotsText">
											<xsl:value-of select="BookingQueryInstance/BookingQuery/noFreeSlotsText" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$i18n.NoFreeSlotsDefaultText" />
										</xsl:otherwise>		
									</xsl:choose>								
								</p>
								
								<div class="month clndr-controls bigpaddingtop bigpaddingbottom">
								
									<div class="booking-filter">
									
										<xsl:if test="not(BookingQueryInstance/BookingQuery/Calendar/showManagers = 'true' and CalendarManagers/user)">
											<xsl:attribute name="class">booking-filter hidden-booking-filter</xsl:attribute>
										</xsl:if>
										
										<div><xsl:value-of select="$i18n.FilterManagers"/>: </div>
										<select class="managerfilter" onchange="bookingQueryUpdateFilter(event, this)" data-url="{queryRequestURL}" aria-label="{$i18n.FilterByManager}">
							
											<option value=""><xsl:value-of select="$i18n.FilterManagers.NoFilter"/></option>
											
											<xsl:for-each select="CalendarManagers/user">
											
												<option value="{userID}">
													<xsl:choose>
														<xsl:when test="CustomName"><xsl:value-of select="CustomName"/></xsl:when>
														<xsl:otherwise><xsl:value-of select="firstname"/>&#160;<xsl:value-of select="lastname"/></xsl:otherwise>
													</xsl:choose>
												</option>
											
											</xsl:for-each>
							
										</select>
									</div>
									
									<div class="booking-filter">
										<xsl:if test="not(BookingQueryInstance/BookingQuery/showLocationFilter = 'true' and Locations/Location)">
											<xsl:attribute name="class">booking-filter hidden-booking-filter</xsl:attribute>
										</xsl:if>
								
										<div><xsl:value-of select="$i18n.FilterLocations"/>: </div>
										<select class="locationfilter" onchange="bookingQueryUpdateFilter(event, this)" data-url="{queryRequestURL}" aria-label="{$i18n.FilterByLocation}">
							
											<option value=""><xsl:value-of select="$i18n.FilterLocation.NoFilter"/></option>
											
											<xsl:for-each select="Locations/Location">
											
												<option value="{.}">
													<xsl:value-of select="."/>
												</option>
											
											</xsl:for-each>
							
										</select>
									</div>
									
									<div class="navigation-container bigpaddingtop">
										<a class="btn btn-light btn-inline clndr-previous-button arrow-button" href="#" onclick="return false;" title="{$js.Calendar.PreviousMonth}">
											<i data-icon-before="&lt;"></i>
										</a>
										
										<a class="hidden btn btn-light btn-inline previous-week-button arrow-button" href="#" onclick="return false;" title="{$js.Calendar.PreviousWeek}">
											<i data-icon-before="&lt;"></i>
										</a>
										
										<h2 class="calendar-month">{{month}} {{year}}</h2>
										
										<a class="btn btn-light btn-inline clndr-next-button arrow-button" href="#" onclick="return false;" title="{$js.Calendar.NextMonth}">
											<i data-icon-before="&gt;"></i>
										</a>
										
										<a class="hidden btn btn-light btn-inline next-week-button arrow-button" href="#" onclick="return false;" title="{$js.Calendar.NextWeek}">
											<i data-icon-before="&gt;"></i>
										</a>
									</div>
								</div>
					
								<div class="days inner-padded-rbl">
									<div class="calendar-row calendar-row-header">
										<div><xsl:value-of select="$i18n.Week.Short"/></div>
										{{#each daysOfTheWeek}}
											<div>{{bookingQuerySafe_string this}}</div>
										{{/each}}
									</div>
									
									{{#BookingQuery-Week days}}
										{{this}}
									{{/BookingQuery-Week}}
									<div class="calendar-day"></div>
								</div>
								
							</div>
	
							<div style="display: none;">
						
								<div class="calendar-day-template">
									<div class="modal-content contentitem">
									
										<div class="modal-header">
											<h3>
												<xsl:value-of select="$i18n.Modal.ChooseTime"/>
											</h3>
											<div>
												{{date}}
											</div>
										</div>
										
										<div class="modal-body">
											<div class="event-list-container">
												{{#BookingQuery-Day events}}
													{{this}}
												{{/BookingQuery-Day}}
											</div>
										</div>
										
									</div>
								</div>
							
							</div>							
							
							
						</div>
					</div>
					
				</xsl:if>
				
				<div class="selected" style="display: none;">
					
					<div class="selected-header">
					
						<span class="selected-header-text">
							<xsl:value-of select="$i18n.ChosenSlot" />
						</span>
					
						<div class="selected-header-divider"></div>
						
						<a class="revert-choice" data-icon-before="w">
							<xsl:value-of select="$i18n.Delete"/>
						</a>
					
					</div>
					
					<div class="selecting-header">

						<span class="selecting-header-text">
							<xsl:value-of select="$i18n.AvailableSlots" />
						</span>
					
					</div>
					
					<div class="selected-body">
					
						<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/viewMode != 'CALENDAR'">
										
							<div class="list-view">
							
								<p class="error hidden">
									
									<xsl:if test="count(BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot[(size - bookingsCount) > 0]) = 0">
										<xsl:attribute name="class">error</xsl:attribute>
									</xsl:if>
								
									<xsl:choose>
										<xsl:when test="BookingQueryInstance/BookingQuery/noFreeSlotsText">
											<xsl:value-of select="BookingQueryInstance/BookingQuery/noFreeSlotsText" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$i18n.NoFreeSlotsDefaultText" />
										</xsl:otherwise>		
									</xsl:choose>								
								</p>
					
								<xsl:variable name="options">
								
									<xsl:choose>
										<xsl:when test="BookingQueryInstance/BookingQuery/Calendar/showFullyBookedSlotsInList = 'true'">
											<xsl:apply-templates select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot" mode="list" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:apply-templates select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot[(size - bookingsCount) > 0]" mode="list" />
										</xsl:otherwise>
									</xsl:choose>
								
								</xsl:variable>
								
								<div class="margintop bigmarginbottom">
									<xsl:call-template name="createDropdown">
										<xsl:with-param name="id" select="$selectedFieldName" />
										<xsl:with-param name="name" select="$selectedFieldName" />
										<xsl:with-param name="labelElementName" select="'name'" />
										<xsl:with-param name="valueElementName" select="'value'" />
										<xsl:with-param name="element" select="exsl:node-set($options)/option" />
										<xsl:with-param name="selectedValue" select="BookingQueryInstance/BookingSlot/bookingSlotID" />
										<xsl:with-param name="addEmptyOption" select="$i18n.ChooseSlot"/>
										<xsl:with-param name="width" select="'100%'"/>
									</xsl:call-template>
									
								</div>
							</div>
							
						</xsl:if>
							
						<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/viewMode != 'LIST'">

							<div class="calendar-view">
							
								<div class="selected-name">
								
									<xsl:choose>
										<xsl:when test="requestparameters/parameter[name=$selectedFieldName]/value != ''">
											
											<xsl:variable name="slotID" select="requestparameters/parameter[name=$selectedFieldName]/value"/>
											
											<xsl:value-of select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot[bookingSlotID=$slotID]/localizedStartDate" />
											<br/>
											<xsl:call-template name="printBookingSlotTime">
												<xsl:with-param name="bookingSlot" select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot[bookingSlotID=$slotID]"/>
												<xsl:with-param name="calendar" select="BookingQueryInstance/BookingQuery/Calendar"/>
											</xsl:call-template>
											
										</xsl:when>
										<xsl:when test="BookingQueryInstance/BookingSlot">
											
											<xsl:value-of select="BookingQueryInstance/BookingSlot/localizedStartDate" />
											<br/>
											<xsl:call-template name="printBookingSlotTime">
												<xsl:with-param name="bookingSlot" select="BookingQueryInstance/BookingSlot"/>
												<xsl:with-param name="calendar" select="BookingQueryInstance/BookingQuery/Calendar"/>
											</xsl:call-template>
											
										</xsl:when>
									</xsl:choose>
									
								</div>
								
								<xsl:call-template name="createHiddenField">
									<xsl:with-param name="id" select="$selectedFieldName" />
									<xsl:with-param name="name" select="$selectedFieldName" />
									<xsl:with-param name="class" select="'selected-value'" />
									<xsl:with-param name="value" select="BookingQueryInstance/BookingSlot/bookingSlotID" />
								</xsl:call-template>
								
							</div>
						
						</xsl:if>
								
						<div class="selected-description">
							<xsl:choose>
								<xsl:when test="requestparameters/parameter[name=$selectedFieldName]/value != ''">
									
									<xsl:variable name="slotID" select="requestparameters/parameter[name=$selectedFieldName]/value"/>
									<xsl:variable name="slot" select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot[bookingSlotID=$slotID]"/>
									
									<xsl:call-template name="bookingSlotInfo">
										<xsl:with-param name="qID" select="$qID" />
										<xsl:with-param name="queryInstance" select="BookingQueryInstance" />
										<xsl:with-param name="bookingSlot" select="$slot" />
									</xsl:call-template>	
									
								</xsl:when>
								<xsl:when test="BookingQueryInstance/BookingSlot">

									<xsl:call-template name="bookingSlotInfo">
										<xsl:with-param name="qID" select="$qID" />
										<xsl:with-param name="queryInstance" select="BookingQueryInstance" />
										<xsl:with-param name="bookingSlot" select="BookingQueryInstance/BookingSlot" />
									</xsl:call-template>	
									
								</xsl:when>
							</xsl:choose>
						</div>
						
						<xsl:if test="BookingQueryInstance/BookingQuery/Calendar/requireConfirmation = 'true'">
							<div class="alternative" >
								<xsl:variable name="fieldName">
									<xsl:value-of select="$qID"/>
									<xsl:value-of select="'_accept'"/>
								</xsl:variable>
							
								<xsl:call-template name="createCheckbox">
									<xsl:with-param name="id" select="$fieldName"/>
									<xsl:with-param name="name" select="$fieldName"/>
									<xsl:with-param name="class" select="'vertical-align-bottom'" />
									<xsl:with-param name="checked" select="BookingQueryInstance/accepted"/>
								</xsl:call-template>
								
								<label for="{$fieldName}" class="checkbox">
									<xsl:value-of select="BookingQueryInstance/BookingQuery/acceptText" />
								</label>
							</div>
						</xsl:if>
					
						<xsl:if test="BookingQueryInstance/BookingQuery/showCommentField = 'true'">
							<div>
								<xsl:variable name="fieldName">
									<xsl:value-of select="$qID"/>
									<xsl:value-of select="'_comment'"/>
								</xsl:variable>
							
								<label for="{$fieldName}" >
									<xsl:choose>
										<xsl:when test="BookingQueryInstance/BookingQuery/commentName != ''">
											<xsl:value-of select="BookingQueryInstance/BookingQuery/commentName"/>
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$i18n.Comment"/>
										</xsl:otherwise>
									</xsl:choose>
								</label>
							
								<xsl:call-template name="createTextArea">
									<xsl:with-param name="id" select="$fieldName"/>
									<xsl:with-param name="name" select="$fieldName"/>
									<xsl:with-param name="rows" select="'4'"/>
									<xsl:with-param name="value" select="BookingQueryInstance/comment" />
								</xsl:call-template>
							</div>
						</xsl:if>
					
					</div>
					
				</div>
				
				<div class="slot-description" style="display: none;">
					
					<xsl:variable name="queryInstance" select="BookingQueryInstance" />
				
					<xsl:for-each select="BookingQueryInstance/BookingQuery/Calendar/BookingSlots/BookingSlot">
					
						<div data-id="{bookingSlotID}" >
	
							<xsl:call-template name="bookingSlotInfo">
								<xsl:with-param name="qID" select="$qID" />												
								<xsl:with-param name="queryInstance" select="$queryInstance" />
								<xsl:with-param name="bookingSlot" select="." />
								<xsl:with-param name="disabled" select="true()" />
							</xsl:call-template>											
						
						</div>
					
					</xsl:for-each>
				
				</div>
					
			</article>
	
		</div>
		
		<script type="text/javascript">
		
			<xsl:variable name="fullyBookedCalendarDayText">
				<xsl:choose>
					<xsl:when test="BookingQueryInstance/BookingQuery/Calendar/fullyBookedCalendarDayText"><xsl:value-of select="BookingQueryInstance/BookingQuery/Calendar/fullyBookedCalendarDayText" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="$i18n.FullyBookedCalendarDayDefaultText" /></xsl:otherwise>		
				</xsl:choose>
			</xsl:variable>
		
			$(function(){
			
				initBookingQuery(
					'<xsl:value-of select="BookingQueryInstance/BookingQuery/queryID" />', 
					<xsl:value-of select="BookingQueryInstance/BookingQuery/Calendar/showFullyBookedDaysInCalendar" />, 
					'<xsl:value-of select="$fullyBookedCalendarDayText" />', 
					<xsl:value-of select="BookingQueryInstance/BookingQuery/showFirstFreeSlot" />, 
					<xsl:value-of select="NoFreeSlots" />,
					<xsl:value-of select="BookingQueryInstance/BookingQuery/Calendar/calendarWeekView" />,
					'<xsl:value-of select="BookingQueryInstance/BookingQuery/Calendar/viewMode" />'
				);
			});
		</script>
		
	</xsl:template>
	
	<xsl:template name="bookingSlotInfo">
		<xsl:param name="qID" />
		<xsl:param name="queryInstance" />
		<xsl:param name="bookingSlot" />
		<xsl:param name="disabled" select="false()" />
		
		<xsl:variable name="calendar" select="$queryInstance/BookingQuery/Calendar" />
	
		<xsl:if test="$calendar/showManagers = 'true' and $bookingSlot/Managers">
		
			<p>
				<div>
					<strong><xsl:value-of select="$i18n.BookingSlot.Managers" /></strong>
				</div>
				<div>
					<xsl:apply-templates select="$bookingSlot/Managers" mode="manager"/>
				</div>
			</p>
			
		</xsl:if>
		
		<xsl:if test="$bookingSlot/location">
			
			<p>
				<xsl:if test="$queryInstance/BookingQuery/showLocationInList = 'true'">
					<xsl:attribute name="class">list-view-hidden</xsl:attribute>
				</xsl:if>
			
				<strong><xsl:value-of select="$i18n.BookingSlot.Location" />: </strong>
				<xsl:value-of select="$bookingSlot/location" />
			</p>
			
		</xsl:if>
		
		<xsl:if test="$bookingSlot/description">
		
			<p>
				<div>
					<strong><xsl:value-of select="$i18n.BookingSlot.Description" /></strong>
				</div>
				<div>
					<xsl:call-template name="replaceLineBreak">
						<xsl:with-param name="string" select="$bookingSlot/description"/>
					</xsl:call-template>
				</div>
			</p>
		
		</xsl:if>
		
		<xsl:if test="$calendar/allowUserBookingSizeSelection = 'true'">
		
			<xsl:call-template name="BookingSizeSelector">
				<xsl:with-param name="name" select="concat($qID, '_size')" />
				<xsl:with-param name="value" select="$queryInstance/size" />
				<xsl:with-param name="bookingSlot" select="$bookingSlot" />
				<xsl:with-param name="disabled" select="$disabled" />
				<xsl:with-param name="maxSize" select="$calendar/userBookingMaxSize" />
				<xsl:with-param name="readOnly" select="$calendar/restrictRebookingToCurrentBookingSize = 'true' and $queryInstance/rebooking = 'true'" />
			</xsl:call-template>
			
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="Calendar">

		<xsl:if test="price > 0">
			<p>
				
				<xsl:choose>
					<xsl:when test="../customPriceText">
						<xsl:value-of select="../customPriceText"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.PaymentRequired.1"/>
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text> </xsl:text>
				<xsl:value-of select="price"/>
				<xsl:text> </xsl:text>
				<xsl:value-of select="$i18n.PaymentRequired.2"/>
			</p>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="BookingSlot" mode="list">
	
		<xsl:variable name="calendar" select="../../."/>
		<xsl:variable name="query" select="../../../."/>
	
		<xsl:if test="bookable = 'true' or $calendar/showNotBookableSlots = 'true'">
		
			<option>
				<name>
					<xsl:if test="location and $query/showLocationInList = 'true'">
						<xsl:value-of select="location" />
						<xsl:text> </xsl:text>
					</xsl:if>
					<xsl:value-of select="startDate" />
					<xsl:text> </xsl:text>
					<xsl:call-template name="printBookingSlotTime">
						<xsl:with-param name="bookingSlot" select="."/>
						<xsl:with-param name="calendar" select="$calendar"/>
					</xsl:call-template>
					
					<xsl:variable name="freeSlots"><xsl:value-of select="size - bookingsCount" /></xsl:variable>
					<xsl:text> (</xsl:text>
					
					<xsl:choose>
						<xsl:when test="$freeSlots > 0 and bookable = 'false'">
						
								<xsl:choose>
									<xsl:when test="notBookableText"><xsl:value-of select="notBookableText" /></xsl:when>
									<xsl:otherwise><xsl:value-of select="$i18n.BookingSlot.NotBookable" /></xsl:otherwise>		
								</xsl:choose>
							
						</xsl:when>
						<xsl:when test="0 >= $freeSlots and $calendar/showFullyBookedSlotsInList = 'true'">
						
							<xsl:choose>
								<xsl:when test="$calendar/fullyBookedListSlotText"><xsl:value-of select="$calendar/fullyBookedListSlotText" /></xsl:when>
								<xsl:otherwise><xsl:value-of select="$i18n.FullyBookedListSlotDefaultText" /></xsl:otherwise>		
							</xsl:choose>
							
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:choose>
								<xsl:when test="../../showFreeSlotsAsBoolean = 'true'">
								
									<xsl:choose>
										<xsl:when test="$freeSlots > 0">
											<xsl:value-of select="$js.Calendar.Free" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$js.Calendar.Booked" />
										</xsl:otherwise>
									</xsl:choose>
								
								</xsl:when>
								<xsl:otherwise>
								
									<xsl:value-of select="$freeSlots" />
									<xsl:text> </xsl:text>
									<xsl:choose>
										<xsl:when test="$freeSlots = 1">
											<xsl:value-of select="$js.Calendar.OneSlot" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$js.Calendar.MultipleSlots" />
										</xsl:otherwise>
									</xsl:choose>
									
								</xsl:otherwise>
							</xsl:choose>
							
						</xsl:otherwise>
					</xsl:choose>
					
					<xsl:text>)</xsl:text>
				</name>
				<value>
					<xsl:value-of select="bookingSlotID"/>
				</value>
			</option>
			
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="user" mode="manager">
		
		<xsl:choose>
			<xsl:when test="Attributes/Attribute[Name = 'bookingManagerName']">
				<xsl:value-of select="Attributes/Attribute[Name = 'bookingManagerName']/Value" />
			</xsl:when>
			<xsl:otherwise>
		<xsl:value-of select="firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="lastname" />
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:if test="position() != last()">
			<xsl:text>, </xsl:text>
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="invalidFormatMessage">
						<xsl:value-of select="invalidFormatMessage"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.Error.InvalidFormat"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.RequiredQuery"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PersonAlreadyInCalendar']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.PersonAlreadyInCalendar"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'DuplicateBookings']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.DuplicateBookings"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PersonAlreadyInBookingSlot']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.PersonAlreadyInBookingSlot"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoCitizenIdentifierFound']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NoCitizenIdentifierFound"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'HasPassed']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.HasPassed"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoFreeSlots']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NoFreeSlots"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NotEnoughFreeSlots']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NotEnoughFreeSlots"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NotAllowedToSelectSize']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NotAllowedToSelectSize"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'MustAccept']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.MustAccept"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'MustAcceptToComment']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.MustAcceptToComment"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'QueryNotConfigured']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NoCalendar"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoLongerValidForSubmit']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NoLongerValidForSubmit"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NotBookable']">
		
		<xsl:param name="id" />
		
		<span>
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NotBookable"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<xsl:param name="id" />
	
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<span>
				<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
				<strong data-icon-before="!">
			
					<xsl:choose>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.Error.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.Error.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.Error.UnknownValidationError" />
						</xsl:otherwise>
					</xsl:choose>
					
					<xsl:text>&#x20;</xsl:text>
					
					<xsl:choose>
						<xsl:when test="displayName">
							<xsl:value-of select="displayName"/>
						</xsl:when>
						<xsl:when test="fieldName = 'comment'">
							<xsl:value-of select="$i18n.Comment"/>
						</xsl:when>
						<xsl:when test="fieldName = 'comment'">
							<xsl:value-of select="$i18n.Comment"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName"/>
						</xsl:otherwise>
					</xsl:choose>
				</strong>
			</span>
		</xsl:if>
				
		<xsl:if test="messageKey">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.Error.UnknownValidationError" />
				</strong>
			</span>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>