package com.nordicpeak.flowengine.changeownerhsa;

import java.io.IOException;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleSMS;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.interfaces.sms.SMSSender;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.userproviders.SimpleUser;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.standardutils.arrays.ArrayUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.BaseFlowBrowserModule;
import com.nordicpeak.flowengine.BaseFlowModule;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.UserFlowInstanceModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.changeownerhsa.beans.FlowFamilyChangeOwnerSettings;
import com.nordicpeak.flowengine.changeownerhsa.beans.OwnershipChangeRequest;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.OwnersChangedEvent;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.evaluationprovider.EvaluationProviderException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryinstance.QueryRequestException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderException;
import com.nordicpeak.flowengine.interfaces.ContactQueryInstance;
import com.nordicpeak.flowengine.interfaces.EvaluationHandler;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceAccessController;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowEngineInterface;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.ListFlowInstancesExtensionLinkProvider;
import com.nordicpeak.flowengine.interfaces.ListFlowInstancesViewFragmentExtensionProvider;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.internalnotifications.beans.NotificationMetadata;
import com.nordicpeak.flowengine.internalnotifications.interfaces.Notification;
import com.nordicpeak.flowengine.internalnotifications.interfaces.NotificationHandler;
import com.nordicpeak.flowengine.internalnotifications.interfaces.NotificationSource;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.managers.ManagerResponse;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;

public class ChangeOwnerModule extends AnnotatedForegroundModule implements ViewFragmentModule<ForegroundModuleDescriptor>, ListFlowInstancesExtensionLinkProvider, ListFlowInstancesViewFragmentExtensionProvider, NotificationSource, FlowInstanceAccessController, ImmutableFlowEngineInterface, FlowAdminExtensionViewProvider {

	private static final AnnotatedBeanTagSourceFactory<OwnershipChangeRequest> CHANGE_OWNER_REQUEST_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<OwnershipChangeRequest>(OwnershipChangeRequest.class, "$changeOwnerRequest.");
	private static final AnnotatedBeanTagSourceFactory<Contact> CONTACT_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<Contact>(Contact.class, "$contact.");
	private static final AnnotatedBeanTagSourceFactory<User> POSTER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$poster.");
	private static final AnnotatedBeanTagSourceFactory<Flow> FLOW_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<Flow>(Flow.class, "$flow.");

	private static final AnnotatedRequestPopulator<FlowFamilyChangeOwnerSettings> SETTINGS_POPULATOR = new AnnotatedRequestPopulator<FlowFamilyChangeOwnerSettings>(FlowFamilyChangeOwnerSettings.class);
	private static final AnnotatedRequestPopulator<OwnershipChangeRequest> OWNERSHIP_REQUEST_POPULATOR = new AnnotatedRequestPopulator<OwnershipChangeRequest>(OwnershipChangeRequest.class);

	private static final HighLevelQuery<FlowInstance> UPDATE_QUERY = new HighLevelQuery<FlowInstance>(FlowInstance.OWNERS_RELATION, FlowInstance.ATTRIBUTES_RELATION);

	private static final ContentType[] SUBMITTED_CONTENT_TYPES = new ContentType[] { ContentType.SUBMITTED, ContentType.IN_PROGRESS, ContentType.WAITING_FOR_COMPLETION };

	@XSLVariable(prefix = "java.")
	private String extensionViewTitle = "Change owner";

	@XSLVariable(prefix = "java.")
	private String flowEventDescription = "change owner";

	@XSLVariable(prefix = "java.")
	private String changeOwner = "Change owner";

	@XSLVariable(prefix = "java.")
	private String requestWaiting = "Ownershipchange request waiting";

	@XSLVariable(prefix = "java.")
	private String requestDenied = "Ownershipchange request declined";

	@XSLVariable(prefix = "java.")
	private String notificationNewOwnershipchangeRequest = "New request";

	@XSLVariable(prefix = "java.")
	private String notificationPostedByManager = "Manager";

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS when new request is created", description = "The SMS messages sent to the users when they receive a change owner request.", required = true)
	@XSLVariable(prefix = "java.")
	private String newRequestSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New request received email subject", description = "The subject of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String newRequestEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New request received email message", description = "The message of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String newRequestEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS when new request is created", description = "The SMS messages sent to the users when they receive a change owner request.", required = true)
	@XSLVariable(prefix = "java.")
	private String revokedRequestSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New request received email subject", description = "The subject of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String revokedRequestEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New request received email message", description = "The message of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String revokedRequestEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS when new request is created", description = "The SMS messages sent to the users when they receive a change owner request.", required = true)
	@XSLVariable(prefix = "java.")
	private String acceptedRequestSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New request received email subject", description = "The subject of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String acceptedRequestEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New request received email message", description = "The message of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String acceptedRequestEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "SMS when new request is created", description = "The SMS messages sent to the users when they receive a change owner request.", required = true)
	@XSLVariable(prefix = "java.")
	private String declinedRequestSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "New request received email subject", description = "The subject of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String declinedRequestEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "New request received email message", description = "The message of emails sent to the users when change owner requests are received", required = true)
	@XSLVariable(prefix = "java.")
	private String declinedRequestEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;

	protected FlowEngineDAOFactory daoFactory;

	protected AnnotatedDAO<OwnershipChangeRequest> ownershipChangeRequestDAO;
	protected AnnotatedDAOWrapper<FlowFamilyChangeOwnerSettings, Integer> flowFamilyChangeOwnerSettingsDAOWrapper;

	protected QueryParameterFactory<OwnershipChangeRequest, Integer> flowInstanceIDParamFactory;
	protected QueryParameterFactory<OwnershipChangeRequest, User> posterParamFactory;
	protected QueryParameterFactory<OwnershipChangeRequest, String> hsaIDParamFactory;

	protected ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	protected UserFlowInstanceModule userFlowInstanceModule;
	protected FlowAdminModule flowAdminModule;

	protected NotificationHandler notificationHandler;

	@InstanceManagerDependency(required = true)
	private StandardFlowNotificationHandler flowNotificationHandler;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	protected EvaluationHandler evaluationHandler;

	@InstanceManagerDependency
	protected SMSSender smsSender;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ChangeOwnerModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		daoFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		HierarchyAnnotatedDAOFactory simpleDAOFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		ownershipChangeRequestDAO = simpleDAOFactory.getDAO(OwnershipChangeRequest.class);
		flowFamilyChangeOwnerSettingsDAOWrapper = simpleDAOFactory.getDAO(FlowFamilyChangeOwnerSettings.class).getWrapper(Integer.class);

		flowInstanceIDParamFactory = ownershipChangeRequestDAO.getParamFactory("flowInstanceID", Integer.class);
		posterParamFactory = ownershipChangeRequestDAO.getParamFactory("poster", User.class);
		hsaIDParamFactory = ownershipChangeRequestDAO.getParamFactory("hsaID", String.class);
	}

	@Override
	public void unload() throws Exception {

		if (this.flowAdminModule != null) {

			flowAdminModule.removeExtensionViewProvider(this);
		}

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.removeListFlowInstancesExtensionLinkProvider(this);
			userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		if (notificationHandler != null) {

			notificationHandler.removeNotificationSource(this);
		}

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setUserFlowInstanceModule(UserFlowInstanceModule userFlowInstanceModule) {

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.addListFlowInstancesExtensionLinkProvider(this);
			userFlowInstanceModule.addListFlowInstancesViewFragmentExtensionProvider(this);

		} else {

			this.userFlowInstanceModule.removeListFlowInstancesExtensionLinkProvider(this);
			this.userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		this.userFlowInstanceModule = userFlowInstanceModule;
	}

	@InstanceManagerDependency
	public void setNotificationHandlerModule(NotificationHandler notificationHandler) {

		if (notificationHandler != null) {

			notificationHandler.addNotificationSource(this);

		} else {

			this.notificationHandler.removeNotificationSource(this);
		}

		this.notificationHandler = notificationHandler;
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (flowAdminModule == null && this.flowAdminModule != null) {

			this.flowAdminModule.removeExtensionViewProvider(this);
		}

		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}

	private OwnershipChangeRequest getOwnershipChangeRequest(Integer flowInstanceID, User poster) throws SQLException {

		HighLevelQuery<OwnershipChangeRequest> query = new HighLevelQuery<OwnershipChangeRequest>();

		query.addParameter(flowInstanceIDParamFactory.getParameter(flowInstanceID));
		query.addParameter(posterParamFactory.getParameter(poster));

		return ownershipChangeRequestDAO.get(query);
	}

	private OwnershipChangeRequest getOwnershipChangeRequest(Integer flowInstanceID, String citizenIdentifier) throws SQLException {

		HighLevelQuery<OwnershipChangeRequest> query = new HighLevelQuery<OwnershipChangeRequest>();

		query.addParameter(flowInstanceIDParamFactory.getParameter(flowInstanceID));
		query.addParameter(hsaIDParamFactory.getParameter(citizenIdentifier));

		return ownershipChangeRequestDAO.get(query);
	}

	private List<OwnershipChangeRequest> getOwnershipChangeRequests(String hsaID) throws SQLException {

		HighLevelQuery<OwnershipChangeRequest> query = new HighLevelQuery<OwnershipChangeRequest>();

		query.addParameter(hsaIDParamFactory.getParameter(hsaID));

		return ownershipChangeRequestDAO.getAll(query);
	}

	@WebPublic(alias = "give")
	public ForegroundModuleResponse giveFlowInstance(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer flowInstanceID;
		FlowInstance flowInstance;
		FlowFamilyChangeOwnerSettings settings;

		if (uriParser.size() == 3 && (flowInstanceID = uriParser.getInt(2)) != null && (flowInstance = userFlowInstanceModule.getFlowInstance(flowInstanceID)) != null && !CollectionUtils.isEmpty(flowInstance.getOwners()) && flowInstance.getOwners().contains(user) && (settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(getFlowFamilyID(flowInstance))) != null) {

			if (flowInstance.getStatus().getContentType() != ContentType.NEW && !(ArrayUtils.contains(SUBMITTED_CONTENT_TYPES, flowInstance.getStatus().getContentType()) && settings.isAllowSubmitted())) {
				throw new AccessDeniedException("Change ownership settings does not allow submitted flow instances");
			}

			List<ValidationError> validationErrors = null;

			OwnershipChangeRequest ownershipChangeRequest = getOwnershipChangeRequest(flowInstance.getFlowInstanceID(), user);

			if (req.getMethod().equalsIgnoreCase("POST")) {

				if (ownershipChangeRequest == null) {

					validationErrors = new ArrayList<ValidationError>();

					try {
						ownershipChangeRequest = OWNERSHIP_REQUEST_POPULATOR.populate(req);
						ownershipChangeRequest.setFlowInstanceID(flowInstanceID);
						ownershipChangeRequest.setAdded(new Timestamp(System.currentTimeMillis()));
						ownershipChangeRequest.setPoster(user);

						if (smsSender == null) {

							if (StringUtils.isEmpty(ownershipChangeRequest.getEmail())) {

								validationErrors.add(new ValidationError("email", ValidationErrorType.RequiredField));
							}

						} else if (StringUtils.isEmpty(ownershipChangeRequest.getEmail()) && StringUtils.isEmpty(ownershipChangeRequest.getMobilePhone())) {

							validationErrors.add(new ValidationError("EmailOrPhoneRequired"));
						}

					} catch (ValidationException e) {
						validationErrors.addAll(e.getErrors());
					}

					if (validationErrors.isEmpty()) {

						log.info("User " + user + " adding ownership change request for flow instance " + flowInstance + " to " + ownershipChangeRequest.getHsaID());

						ownershipChangeRequestDAO.add(ownershipChangeRequest);
						systemInterface.getEventHandler().sendEvent(OwnershipChangeRequest.class, new CRUDEvent<OwnershipChangeRequest>(OwnershipChangeRequest.class, CRUDAction.ADD, ownershipChangeRequest), EventTarget.ALL);

						Contact recipient = ownershipChangeRequest.getRecipientContact();
						sendRecipientEmail(flowInstance, ownershipChangeRequest, recipient, newRequestEmailSubject, newRequestEmailMessage, uriParser.getFullContextPath());
						sendRecipientSMS(flowInstance, ownershipChangeRequest, recipient, newRequestSMS, uriParser.getFullContextPath());

					} else {

						ownershipChangeRequest = null;
					}

				} else {

					if ("true".equals(req.getParameter("cancel"))) {

						log.info("User " + user + " removing ownership change request for flow instance " + flowInstance + " to " + ownershipChangeRequest.getHsaID());

						ownershipChangeRequestDAO.delete(ownershipChangeRequest);
						systemInterface.getEventHandler().sendEvent(OwnershipChangeRequest.class, new CRUDEvent<OwnershipChangeRequest>(OwnershipChangeRequest.class, CRUDAction.DELETE, ownershipChangeRequest), EventTarget.ALL);

						if (!ownershipChangeRequest.isDeclined()) {

							Contact recipient = ownershipChangeRequest.getRecipientContact();
							sendRecipientEmail(flowInstance, ownershipChangeRequest, recipient, revokedRequestEmailSubject, revokedRequestEmailMessage, uriParser.getFullContextPath());
							sendRecipientSMS(flowInstance, ownershipChangeRequest, recipient, revokedRequestSMS, uriParser.getFullContextPath());
						}

						ownershipChangeRequest = null;
					}
				}
			}

			log.info("User " + user + " requesting give flow instance form for flow instance " + flowInstance);

			Document doc = createDocument(req, uriParser);

			Element giveElement = doc.createElement("GiveFlowInstance");
			doc.getDocumentElement().appendChild(giveElement);

			XMLUtils.append(doc, giveElement, flowInstance);
			XMLUtils.append(doc, giveElement, ownershipChangeRequest);

			if (smsSender != null) {

				XMLUtils.appendNewElement(doc, giveElement, "HasSMS");
			}

			if (validationErrors != null) {

				XMLUtils.append(doc, giveElement, "ValidationErrors", validationErrors);
				giveElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			}

			return new SimpleForegroundModuleResponse(doc, getDefaultBreadcrumb());
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "receive")
	public ForegroundModuleResponse receiveFlowInstance(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer flowInstanceID;
		FlowInstance flowInstance;
		FlowFamilyChangeOwnerSettings settings;

		if (uriParser.size() == 3 && (flowInstanceID = uriParser.getInt(2)) != null && (flowInstance = userFlowInstanceModule.getFlowInstance(flowInstanceID)) != null && !CollectionUtils.isEmpty(flowInstance.getOwners()) && !flowInstance.getOwners().contains(user) && (settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(getFlowFamilyID(flowInstance))) != null) {

			if (flowInstance.getStatus().getContentType() != ContentType.NEW && !(ArrayUtils.contains(SUBMITTED_CONTENT_TYPES, flowInstance.getStatus().getContentType()) && settings.isAllowSubmitted())) {
				throw new AccessDeniedException("Change ownership settings does not allow submitted flow instances");
			}

			List<ValidationError> validationErrors = new ArrayList<ValidationError>();

			OwnershipChangeRequest ownershipChangeRequest = null;

			String hsaID = user.getAttributeHandler().getString("HSAID");

			if (!StringUtils.isEmpty(hsaID)) {

				ownershipChangeRequest = getOwnershipChangeRequest(flowInstance.getFlowInstanceID(), hsaID);

				if (ownershipChangeRequest != null && !ownershipChangeRequest.isDeclined()) {

					if (req.getMethod().equalsIgnoreCase("POST")) {

						if (req.getParameter("accept") != null) {

							log.info("User " + user + " accepting ownership of flow instance " + flowInstance + " from " + ownershipChangeRequest.getPoster());

							List<User> previousOwners = new ArrayList<User>(flowInstance.getOwners());

							flowInstance.getOwners().remove(ownershipChangeRequest.getPoster());
							flowInstance.getOwners().add(user);
							flowInstance.getAttributeHandler().setAttribute("OwnershipTransfered", true);

							if (flowInstance.getFirstSubmitted() == null) {
								flowInstance.setPoster(user);
							}

							StringBuilder stringBuilder = new StringBuilder();

							for (User owner : flowInstance.getOwners()) {

								if (stringBuilder.length() > 0) {

									stringBuilder.append(", ");
								}

								stringBuilder.append(owner.getFirstname());
								stringBuilder.append(" ");
								stringBuilder.append(owner.getLastname());
							}

							String eventDetailString = stringBuilder.toString();

							daoFactory.getFlowInstanceDAO().update(flowInstance, UPDATE_QUERY);
							
							HttpSession session = req.getSession(false);
							
							if (session != null) {
								synchronized (session) {
									
									MutableFlowInstanceManager oldInstanceManager = userFlowInstanceModule.getMutableFlowInstanceManagerFromSession(flowInstance.getFlow().getFlowID(), flowInstanceID, session);
									
									if (oldInstanceManager != null) {
										
										log.info("Closing old MutableFlowInstanceManager for user " + user + " before accepting ownership of flow instance " + flowInstance);
										userFlowInstanceModule.removeMutableFlowInstanceManagerFromSession(oldInstanceManager, session);
									}
								}
							}

							// Change contact detail query instance to current user
							{
								InstanceMetadata instanceMetadata = new DefaultInstanceMetadata(userFlowInstanceModule.getSiteProfile(flowInstance));
								MutableFlowInstanceManager instanceManager = new MutableFlowInstanceManager(flowInstance, queryHandler, evaluationHandler, "userid-" + user.getUserID() + "-uuid-" + UUID.randomUUID(), req, user, instanceMetadata, BaseFlowBrowserModule.OWNER_REQUEST_METADATA, userFlowInstanceModule.getAbsoluteFileURL(uriParser, flowInstance.getFlow()));

								setQueryPoster(instanceManager, user);

								instanceManager.saveInstance(userFlowInstanceModule, user, user, EventType.UPDATED, instanceManager.getFlowInstance().getLastStatusChange(), BaseFlowModule.OWNER_REQUEST_METADATA);
								instanceManager.close(queryHandler);
							}

							FlowInstanceEvent flowInstanceEvent = null;

							if (flowInstance.getStatus().getContentType() != ContentType.NEW) {

								flowInstanceEvent = userFlowInstanceModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstance, EventType.OWNERS_UPDATED, eventDetailString, user);
							}

							systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(FlowInstance.class, CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
							systemInterface.getEventHandler().sendEvent(FlowInstance.class, new OwnersChangedEvent(flowInstance, flowInstanceEvent, userFlowInstanceModule.getCurrentSiteProfile(req, user, uriParser, flowInstance.getFlow().getFlowFamily()), previousOwners, ownershipChangeRequest.getPoster()), EventTarget.ALL);

							ownershipChangeRequestDAO.delete(ownershipChangeRequest);
							systemInterface.getEventHandler().sendEvent(OwnershipChangeRequest.class, new CRUDEvent<OwnershipChangeRequest>(OwnershipChangeRequest.class, CRUDAction.DELETE, ownershipChangeRequest), EventTarget.ALL);

							if (flowInstance.getStatus().getContentType() == ContentType.NEW) {

								res.sendRedirect(req.getContextPath() + userFlowInstanceModule.getFullAlias() + "/flowinstance/" + flowInstance.getFlow().getFlowID() + "/" + flowInstance.getFlowInstanceID());

							} else {

								res.sendRedirect(req.getContextPath() + userFlowInstanceModule.getFullAlias() + "/overview/" + flowInstance.getFlow().getFlowID() + "/" + flowInstance.getFlowInstanceID());
							}

							sendPosterEmail(flowInstance, ownershipChangeRequest, ownershipChangeRequest.getPosterContact(), acceptedRequestEmailSubject, acceptedRequestEmailMessage, uriParser.getFullContextPath());
							sendPosterSMS(flowInstance, ownershipChangeRequest, ownershipChangeRequest.getPosterContact(), acceptedRequestSMS, uriParser.getFullContextPath());

							return null;

						} else if (req.getParameter("decline") != null) {

							String declineReason = req.getParameter("declineReason");

							log.info("User " + user + " declining ownership of flow instance " + flowInstance + " from " + ownershipChangeRequest.getPoster() + " with reason " + declineReason);

							ownershipChangeRequest.setDeclined(true);
							ownershipChangeRequest.setDeclineReason(declineReason);

							ownershipChangeRequestDAO.update(ownershipChangeRequest);
							systemInterface.getEventHandler().sendEvent(OwnershipChangeRequest.class, new CRUDEvent<OwnershipChangeRequest>(OwnershipChangeRequest.class, CRUDAction.UPDATE, ownershipChangeRequest), EventTarget.ALL);

							res.sendRedirect(req.getContextPath() + userFlowInstanceModule.getFullAlias());

							sendPosterEmail(flowInstance, ownershipChangeRequest, ownershipChangeRequest.getPosterContact(), declinedRequestEmailSubject, declinedRequestEmailMessage, uriParser.getFullContextPath());
							sendPosterSMS(flowInstance, ownershipChangeRequest, ownershipChangeRequest.getPosterContact(), declinedRequestSMS, uriParser.getFullContextPath());

							return null;
						}
					}

					log.info("User " + user + " requesting receive flow instance form for flow instance " + flowInstance);

					Document doc = createDocument(req, uriParser);

					Element receiveElement = doc.createElement("ReceiveFlowInstance");
					doc.getDocumentElement().appendChild(receiveElement);

					if (ownershipChangeRequest != null) {

						receiveElement.appendChild(ownershipChangeRequest.toXML(doc));
					}

					ImmutableFlowInstanceManager instanceManager = userFlowInstanceModule.getImmutableFlowInstanceManager(flowInstanceID, this, user, false, req, uriParser, BaseFlowModule.OWNER_REQUEST_METADATA);

					setQueryPoster(instanceManager, user);

					receiveElement.appendChild(instanceManager.getFlowInstance().toXML(doc));

					List<ManagerResponse> managerResponses = instanceManager.getFullShowHTML(req, user, this, true, null, getImmutableQueryRequestBaseURL(req, instanceManager), new RequestMetadata(false));
					XMLUtils.append(doc, receiveElement, "ManagerResponses", managerResponses);

					if (validationErrors != null) {

						XMLUtils.append(doc, receiveElement, "ValidationErrors", validationErrors);
						receiveElement.appendChild(RequestUtils.getRequestParameters(req, doc));
					}

					return new SimpleForegroundModuleResponse(doc, getDefaultBreadcrumb());
				}
			}
		}

		throw new URINotFoundException(uriParser);
	}

	private void setQueryPoster(FlowInstanceManager instanceManager, User user) {

		List<ContactQueryInstance> contactQueryInstances = instanceManager.getQueries(ContactQueryInstance.class);

		if (contactQueryInstances != null) {

			MutableAttributeHandler flowInstanceAttributeHandler = (MutableAttributeHandler) instanceManager.getFlowInstance().getAttributeHandler();

			for (ContactQueryInstance contactQueryInstance : contactQueryInstances) {

				contactQueryInstance.forcedRepopulate(user, flowInstanceAttributeHandler, queryHandler);
			}
		}
	}

	protected String getImmutableQueryRequestBaseURL(HttpServletRequest req, FlowInstanceManager instanceManager) {

		return req.getContextPath() + getFullAlias() + "/iquery/" + instanceManager.getFlowID() + "/" + instanceManager.getFlowInstanceID() + "/q/";
	}

	@WebPublic(alias = "iquery")
	public ForegroundModuleResponse processImmutableQueryRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, QueryRequestException, QueryProviderException, EvaluationProviderException, InvalidFlowInstanceStepException, MissingQueryInstanceDescriptor, DuplicateFlowInstanceManagerIDException {

		return userFlowInstanceModule.processImmutableQueryRequest(req, res, user, uriParser, this, true, BaseFlowModule.OWNER_REQUEST_METADATA);
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser) {

		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		doc.appendChild(documentElement);
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		return doc;
	}

	@Override
	public ExtensionLink getListFlowInstancesExtensionLink(FlowInstance flowInstance, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		if (flowInstance.isRemote()) {

			return null;
		}

		ContentType contentType = flowInstance.getStatus().getContentType();

		if (contentType == ContentType.NEW || ArrayUtils.contains(SUBMITTED_CONTENT_TYPES, contentType)) {

			FlowFamilyChangeOwnerSettings settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(getFlowFamilyID(flowInstance));

			if (settings != null && (contentType == ContentType.NEW || settings.isAllowSubmitted())) {

				OwnershipChangeRequest ownershipChangeRequest = getOwnershipChangeRequest(flowInstance.getFlowInstanceID(), user);

				if (ownershipChangeRequest == null) {

					return new ExtensionLink(changeOwner, uriParser.getContextPath() + getFullAlias() + "/give/" + flowInstance.getFlowInstanceID(), "light", null);

				} else if (ownershipChangeRequest.isDeclined()) {

					return new ExtensionLink(requestDenied, uriParser.getContextPath() + getFullAlias() + "/give/" + flowInstance.getFlowInstanceID(), "red", null);

				} else {

					return new ExtensionLink(requestWaiting, uriParser.getContextPath() + getFullAlias() + "/give/" + flowInstance.getFlowInstanceID(), "green", null);
				}
			}
		}

		return null;
	}

	@Override
	public ViewFragment getListFlowInstancesViewFragment(List<FlowInstance> userFlowInstances, SiteProfileHandler profileHandler, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		String hsaID = user.getAttributeHandler().getString("HSAID");

		if (!StringUtils.isEmpty(hsaID)) {

			List<OwnershipChangeRequest> ownershipChangeRequests = getOwnershipChangeRequests(hsaID);

			if (!CollectionUtils.isEmpty(ownershipChangeRequests)) {

				List<FlowInstance> flowInstances = new ArrayList<FlowInstance>();

				for (OwnershipChangeRequest ownershipChangeRequest : ownershipChangeRequests) {

					if (!ownershipChangeRequest.isDeclined()) {

						FlowInstance flowInstance = userFlowInstanceModule.getFlowInstance(ownershipChangeRequest.getFlowInstanceID());

						if (flowInstance != null) {

							ContentType contentType = flowInstance.getStatus().getContentType();

							if (contentType == ContentType.NEW || ArrayUtils.contains(SUBMITTED_CONTENT_TYPES, contentType)) {

								FlowFamilyChangeOwnerSettings settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(getFlowFamilyID(flowInstance));

								if (settings != null && (contentType == ContentType.NEW || settings.isAllowSubmitted())) {

									flowInstances.add(flowInstance);
								}
							}
						}
					}
				}

				if (!CollectionUtils.isEmpty(flowInstances)) {

					Document doc = createDocument(req, uriParser);

					Element listFlowInstancesExtensionElement = doc.createElement("ListFlowInstancesExtension");
					doc.getDocumentElement().appendChild(listFlowInstancesExtensionElement);

					if (profileHandler != null) {

						XMLUtils.append(doc, listFlowInstancesExtensionElement, "SiteProfiles", profileHandler.getProfiles());
					}

					XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
					genDoc.addIgnoredFields(UserFlowInstanceModule.LIST_EXCLUDED_FIELDS);

					Element waitingChangeOwnerFlowInstancesElement = XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "WaitingChangeOwnerFlowInstances");

					for (FlowInstance flowInstance : flowInstances) {

						Element flowInstanceElement = (Element) waitingChangeOwnerFlowInstancesElement.appendChild(flowInstance.toXML(genDoc));

						XMLUtils.appendNewElement(doc, flowInstanceElement, "ChangeOwnerURL", uriParser.getContextPath() + getFullAlias() + "/receive/" + flowInstance.getFlowInstanceID());
					}

					return viewFragmentTransformer.createViewFragment(doc);
				}
			}
		}

		return null;
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@WebPublic(alias = "icon")
	public ForegroundModuleResponse getFlowIcon(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, AccessDeniedException, ModuleConfigurationException, SQLException, IOException {

		return userFlowInstanceModule.getFlowIcon(req, res, user, uriParser);
	}

	public boolean sendRecipientEmail(FlowInstance flowInstance, OwnershipChangeRequest ownershipChangeRequest, Contact contact, String subject, String message, String fullContextPath) {

		if (!contact.isContactByEmail() || contact.getEmail() == null || subject == null || message == null || ownershipChangeRequest == null) {

			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(CHANGE_OWNER_REQUEST_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest));
		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getPoster()));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getRecipientContact()));
		tagReplacer.addTagSource(FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
		tagReplacer.addTagSource(new SingleTagSource("$changeOwnerRequest.URL", fullContextPath + getFullAlias() + "/receive/" + ownershipChangeRequest.getFlowInstanceID()));

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {
			email.addRecipient(contact.getEmail());
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(flowNotificationHandler.getEmailSenderName(flowInstance));
			email.setSenderAddress(flowNotificationHandler.getEmailSenderAddress(flowInstance));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);
			return true;

		} catch (Exception e) {

			log.info("Error generating/sending email " + email, e);
		}

		return false;
	}

	public boolean sendRecipientSMS(FlowInstance flowInstance, OwnershipChangeRequest ownershipChangeRequest, Contact contact, String message, String fullContextPath) {

		if (!contact.isContactBySMS() || contact.getMobilePhone() == null || smsSender == null || message == null || ownershipChangeRequest == null) {

			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(CHANGE_OWNER_REQUEST_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest));
		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getPoster()));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getRecipientContact()));
		tagReplacer.addTagSource(FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
		tagReplacer.addTagSource(new SingleTagSource("$changeOwnerRequest.URL", fullContextPath + getFullAlias() + "/receive/" + ownershipChangeRequest.getFlowInstanceID()));

		SimpleSMS sms = new SimpleSMS();

		try {
			sms.setSenderName(flowNotificationHandler.getSMSSenderName(flowInstance));
			sms.setMessage(tagReplacer.replace(message));
			sms.addRecipient(contact.getMobilePhone());

			return smsSender.send(sms);

		} catch (Exception e) {

			log.info("Error generating/sending sms " + sms, e);
		}

		return false;
	}

	public boolean sendPosterEmail(FlowInstance flowInstance, OwnershipChangeRequest ownershipChangeRequest, Contact contact, String subject, String message, String fullContextPath) {

		if (!contact.isContactByEmail() || contact.getEmail() == null || subject == null || message == null) {

			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(CHANGE_OWNER_REQUEST_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest));
		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getPoster()));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getRecipientContact()));
		tagReplacer.addTagSource(FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
		tagReplacer.addTagSource(new SingleTagSource("$changeOwnerRequest.URL", fullContextPath + getFullAlias() + "/give/" + ownershipChangeRequest.getFlowInstanceID()));

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {
			email.addRecipient(contact.getEmail());
			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(flowNotificationHandler.getEmailSenderName(flowInstance));
			email.setSenderAddress(flowNotificationHandler.getEmailSenderAddress(flowInstance));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);
			return true;

		} catch (Exception e) {

			log.info("Error generating/sending email " + email, e);
		}

		return false;
	}

	public boolean sendPosterSMS(FlowInstance flowInstance, OwnershipChangeRequest ownershipChangeRequest, Contact contact, String message, String fullContextPath) {

		if (!contact.isContactBySMS() || contact.getMobilePhone() == null || smsSender == null || message == null) {

			return false;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(CHANGE_OWNER_REQUEST_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest));
		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getPoster()));
		tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(ownershipChangeRequest.getRecipientContact()));
		tagReplacer.addTagSource(FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
		tagReplacer.addTagSource(new SingleTagSource("$changeOwnerRequest.URL", fullContextPath + getFullAlias() + "/give/" + ownershipChangeRequest.getFlowInstanceID()));

		SimpleSMS sms = new SimpleSMS();

		try {
			sms.setSenderName(flowNotificationHandler.getSMSSenderName(flowInstance));
			sms.setMessage(tagReplacer.replace(message));
			sms.addRecipient(contact.getMobilePhone());

			return smsSender.send(sms);

		} catch (Exception e) {

			log.info("Error generating/sending sms " + sms, e);
		}

		return false;
	}

	@se.unlogic.hierarchy.core.annotations.EventListener(channel = OwnershipChangeRequest.class)
	public void processEvent(CRUDEvent<OwnershipChangeRequest> event, EventSource source) {

		log.debug("Received CRUD " + event.getAction() + " event regarding " + event.getBeanClass());

		if (source == EventSource.LOCAL && notificationHandler != null) {

			if (event.getAction() == CRUDAction.ADD) {

				for (OwnershipChangeRequest ownershipChangeRequest : event.getBeans()) {

					try {
						User recipient = ownershipChangeRequest.getRecipient(systemInterface.getUserHandler());

						if (recipient != null) {
							notificationHandler.addNotification(ownershipChangeRequest.getFlowInstanceID(), recipient.getUserID(), moduleDescriptor.getModuleID(), "recipient", ownershipChangeRequest.getPoster().getUserID(), notificationNewOwnershipchangeRequest, null);
						}

					} catch (SQLException e) {

						log.error("Error sending notification for " + ownershipChangeRequest + " regarding flowinstance " + ownershipChangeRequest.getFlowInstanceID(), e);
					}
				}

			} else if (event.getAction() == CRUDAction.DELETE) {

				for (OwnershipChangeRequest ownershipChangeRequest : event.getBeans()) {

					try {
						User recipient = ownershipChangeRequest.getRecipient(systemInterface.getUserHandler());
						notificationHandler.deleteNotifications(moduleDescriptor.getModuleID(), ownershipChangeRequest.getFlowInstanceID(), recipient, null);

					} catch (SQLException e) {

						log.error("Error deleting notificatios for recipient of " + ownershipChangeRequest + " regarding flowinstance " + ownershipChangeRequest.getFlowInstanceID(), e);
					}
				}
			}
		}

	}

	@Override
	public NotificationMetadata getNotificationMetadata(Notification notification, FlowInstance flowInstance, String fullContextPath) throws Exception {

		NotificationMetadata metadata = new NotificationMetadata();
		metadata.setShowURL(fullContextPath + getFullAlias() + "/receive/" + notification.getFlowInstanceID());
		metadata.setUrl(fullContextPath + getFullAlias() + "/receive/" + notification.getFlowInstanceID());

		if (!flowInstance.getFlow().hidesManagerDetails()) {

			metadata.setPoster(systemInterface.getUserHandler().getUser(notification.getExternalNotificationID(), false, true));

		} else {

			SimpleUser hiddenManager = new SimpleUser();
			hiddenManager.setFirstname(notificationPostedByManager);

			metadata.setPoster(hiddenManager);
		}

		return metadata;
	}

	private Integer getFlowFamilyID(FlowInstance flowInstance) throws SQLException {

		Integer flowID = null;

		if (flowInstance.getFlow() != null) {

			if (flowInstance.getFlow().getFlowFamily() != null) {

				return flowInstance.getFlow().getFlowFamily().getFlowFamilyID();
			}

			flowID = flowInstance.getFlow().getFlowID();
		}

		if (flowID == null) {

			ObjectQuery<Integer> flowIDQuery = new ObjectQuery<Integer>(daoFactory.getFlowInstanceDAO().getDataSource(), "SELECT flowID FROM " + daoFactory.getFlowInstanceDAO().getTableName() + " WHERE flowInstanceID = ?", IntegerPopulator.getPopulator());
			flowIDQuery.setInt(1, flowInstance.getFlow().getFlowID());

			flowID = flowIDQuery.executeQuery();
		}

		ObjectQuery<Integer> familyIDQuery = new ObjectQuery<Integer>(daoFactory.getFlowDAO().getDataSource(), "SELECT flowFamilyID FROM " + daoFactory.getFlowDAO().getTableName() + " WHERE flowID = ?", IntegerPopulator.getPopulator());
		familyIDQuery.setInt(1, flowID);

		return familyIDQuery.executeQuery();
	}

	@Override
	public void checkNewFlowInstanceAccess(Flow flow, User user, SiteProfile profile) throws AccessDeniedException {}

	@Override
	public void checkFlowInstanceAccess(ImmutableFlowInstance flowInstance, User user) throws AccessDeniedException {

		String hsaID = user.getAttributeHandler().getString("HSAID");

		if (!StringUtils.isEmpty(hsaID)) {

			try {
				OwnershipChangeRequest ownershipChangeRequest = getOwnershipChangeRequest(flowInstance.getFlowInstanceID(), hsaID);

				if (ownershipChangeRequest != null) {
					return;
				}

			} catch (SQLException e) {
				log.error("Error getting OwnershipChangeRequest for user " + user + " and flow instance " + flowInstance.getFlowInstanceID());
			}
		}

		throw new AccessDeniedException("Access denied to flow instance " + flowInstance + ", the current user has no ownership change request for the requested instance.");
	}

	@Override
	public boolean isMutable(ImmutableFlowInstance flowInstance, User user) {

		return false;
	}

	@Override
	public QueryHandler getQueryHandler() {

		return queryHandler;
	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {

		Document doc = createDocument(req, uriParser);

		Element showViewElement = doc.createElement("ShowSettings");
		doc.getDocumentElement().appendChild(showViewElement);

		XMLUtils.appendNewElement(doc, showViewElement, "FullAlias", getFullAlias());
		showViewElement.appendChild(flow.toXML(doc));

		FlowFamilyChangeOwnerSettings settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(flow.getFlowFamily().getFlowFamilyID());

		boolean enabled = false;

		if (settings != null) {

			XMLUtils.append(doc, showViewElement, settings);
			enabled = true;
		}

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
	}

	@Override
	public String getExtensionViewTitle() {

		return extensionViewTitle;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		FlowFamilyChangeOwnerSettings settings = flowFamilyChangeOwnerSettingsDAOWrapper.get(flow.getFlowFamily().getFlowFamilyID());

		List<ValidationError> validationErrors = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			validationErrors = new ArrayList<ValidationError>();

			if (settings == null) {

				settings = new FlowFamilyChangeOwnerSettings();
			}

			try {
				SETTINGS_POPULATOR.populate(settings, req);

				if (CollectionUtils.isEmpty(validationErrors)) {

					if ("true".equals(req.getParameter("useChangeOwner"))) {

						log.info("User " + user + " updating change ownership settings for flow " + flow);

						if (settings.getFlowFamilyID() == null) {

							settings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
							flowFamilyChangeOwnerSettingsDAOWrapper.add(settings);

						} else {

							flowFamilyChangeOwnerSettingsDAOWrapper.update(settings);
						}

					} else if (settings.getFlowFamilyID() != null) {

						log.info("User " + user + " deleting change ownership settings for flow " + flow);

						flowFamilyChangeOwnerSettingsDAOWrapper.delete(settings);
					}

					flowAdminModule.addFlowFamilyEvent(flowAdminModule.getEventFunctionConfigured() + " " + flowEventDescription, flow.getFlowFamily(), user);
					flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#changeownersettings");
					return null;
				}

			} catch (ValidationException e) {

				validationErrors.addAll(e.getErrors());
			}
		}

		log.info("User " + user + " requesting update change ownership settings form for flow " + flow);

		Document doc = createDocument(req, uriParser);

		Element settingsElement = doc.createElement("UpdateSettings");
		doc.getDocumentElement().appendChild(settingsElement);

		settingsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, settingsElement, settings);

		if (validationErrors != null) {

			XMLUtils.append(doc, settingsElement, "ValidationErrors", validationErrors);
			settingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

}
