<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>
	
	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/featherlight/js/featherlight.min.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>		
	
	<xsl:variable name="scripts">
		/js/UserGroupList.js
		/js/changeowner.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/UserGroupList.css?v=1
	</xsl:variable>	

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowSettings"/>
		<xsl:apply-templates select="UpdateSettings" />
		
		<xsl:apply-templates select="GiveFlowInstance"/>
		<xsl:apply-templates select="ReceiveFlowInstance"/>
		<xsl:apply-templates select="ListFlowInstancesExtension"/>
		
	</xsl:template>
	
	<xsl:template match="ShowSettings">
	
		<a name="changeownersettings" />
		
		<xsl:choose>
			<xsl:when test="FlowFamilyChangeOwnerSettings">
				
				<xsl:value-of select="$i18n.ShowSettings.Enabled" />
				
				<xsl:if test="FlowFamilyChangeOwnerSettings/allowSubmitted = 'true'">
					<br/>
					<xsl:value-of select="$i18n.ShowSettings.AllowsSubmitted" />
				</xsl:if>
				
			</xsl:when>
			<xsl:otherwise>

				<xsl:value-of select="$i18n.ShowSettings.Disabled" />
				
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/requestinfo/contextpath}{FullAlias}/updatesettings/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateSettings" />
			<img class="marginleft" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/pen.png" alt="" />
		</a>
		
	</xsl:template>
	
	<xsl:template match="UpdateSettings">
		
		<div id="changeownersettings" class="contentitem errands-wrapper border-box">
			
			<h1>
				<xsl:value-of select="$i18n.UpdateSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
			
			<xsl:apply-templates select="ValidationErrors/validationError" />
			
			<form method="post" action="{/Document/requestinfo/uri}">
			
				<div class="floatleft full bigmarginbottom">
					
					<div class="floatleft full">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'useChangeOwner'" />
							<xsl:with-param name="name" select="'useChangeOwner'" />
							<xsl:with-param name="value" select="'true'" />
							<xsl:with-param name="checked">
								<xsl:if test="FlowFamilyChangeOwnerSettings">
									<xsl:text>true</xsl:text>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						
						<label for="useChangeOwner">
							<xsl:value-of select="$i18n.Settings.enabled" />
						</label>
					</div>
					
				</div>
				
				<div id="settings">
				
					<div class="floatleft full bigmarginbottom">
						
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'allowSubmitted'" />
								<xsl:with-param name="name" select="'allowSubmitted'" />
								<xsl:with-param name="value" select="'true'" />
								<xsl:with-param name="element" select="FlowFamilyChangeOwnerSettings" />
							</xsl:call-template>
							
							<label for="allowSubmitted">
								<xsl:value-of select="$i18n.Settings.allowSubmitted" />
							</label>
						</div>
						
					</div>
					
				</div>
				
				<script type="text/javascript">
					$(function() {
					
						var checkbox = $("#useChangeOwner");
						var settingsDiv = $("#settings");
						
						settingsDiv.toggle(checkbox[0].checked);
						
						checkbox.on("click", function(){
							settingsDiv.toggle(this.checked);
						});
					});
				</script>
				
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
				
			</form>
			
		</div>

	</xsl:template>
	
	<xsl:template match="GiveFlowInstance">
	
		<div class="contentitem" style="margin-top: 0;">
		
			<div class="section-inside full">
				<div class="heading-wrapper bigpaddingleft">
					<figure>
 						<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{FlowInstance/Flow/flowID}" alt="" />
	 				</figure>
	 				<div class="heading">
						<h1 class="xl"><xsl:value-of select="FlowInstance/Flow/name"/></h1>
						<xsl:if test="FlowInstance/firstSubmitted">
							<span class="errandno"><xsl:value-of select="$i18n.FlowInstanceID" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="FlowInstance/flowInstanceID" /></span>
						</xsl:if>
					</div>
				</div>
			</div>
			
			<div class="clearboth bigmargin">
			
				<xsl:apply-templates select="ValidationErrors/validationError"/>
				
				<xsl:choose>
					<xsl:when test="OwnershipChangeRequest">
						
						<h2 class="h1 nopadding"><xsl:value-of select="$i18n.GiveFlowInstance.Title" /></h2>
						
						<p>
							<xsl:choose>
								<xsl:when test="OwnershipChangeRequest/declined = 'true'">
									<xsl:value-of select="$i18n.GiveFlowInstance.InProgress.Description.Declined"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$i18n.GiveFlowInstance.InProgress.Description"/>
								</xsl:otherwise>
							</xsl:choose>
						</p>
						
						<xsl:if test="OwnershipChangeRequest/declineReason">
							<div class="floatleft full bigmarginbottom">
										
								<label for="alternative-value" class="floatleft clearboth">
									<xsl:value-of select="$i18n.ReceiveFlowInstance.Decline.Reason" />
								</label>
								
								<div class="floatleft full">
									<xsl:value-of select="OwnershipChangeRequest/declineReason" disable-output-escaping="no"/>
							  </div>
							</div>
						</xsl:if>
						
						<table class="full">
							<tr>
								<th>
									<xsl:value-of select="$i18n.Request.Added"/>
								</th>
								<th>
									<xsl:value-of select="$i18n.Firstname"/>
								</th>
								<th>
									<xsl:value-of select="$i18n.Lastname"/>
								</th>
								<th>
									<xsl:value-of select="$i18n.Username"/>
								</th>
								<th>
									<xsl:value-of select="$i18n.Request.Accepted"/>
								</th>
							</tr>
							
							<tr>
								<td>
									<xsl:value-of select="OwnershipChangeRequest/added"/>
								</td>
								<td>
									<xsl:value-of select="OwnershipChangeRequest/Recipient/firstname"/>
								</td>
								<td>
									<xsl:value-of select="OwnershipChangeRequest/Recipient/lastname"/>
								</td>
								<td>
									<xsl:value-of select="OwnershipChangeRequest/Recipient/username"/>
								</td>
								<td>
									
									<xsl:choose>
										<xsl:when test="OwnershipChangeRequest/declined = 'true'">
											<xsl:value-of select="$i18n.Request.Accepted.No"/>
										</xsl:when>
										<xsl:otherwise>
											<xsl:text>-</xsl:text>
										</xsl:otherwise>
									</xsl:choose>
									
								</td>
							</tr>
							
						</table>
						
						<p>
							<xsl:choose>
								<xsl:when test="HasSMS"><xsl:value-of select="$i18n.GiveFlowInstance.InProgress.DescriptionBottom"/></xsl:when>
								<xsl:otherwise><xsl:value-of select="$i18n.GiveFlowInstance.InProgress.DescriptionBottom.NoSMS"/></xsl:otherwise>
							</xsl:choose>
						</p>
						
						<form name="changeOwnerForm" method="post" action="{/Document/requestinfo/uri}">
						
							<input type="hidden" name="cancel" value="true"/>
							
							<div class="clearboth text-align-center" style="margin-top: 30px">
								<input class="btn btn-red xl display-inline" type="submit" value="{$i18n.GiveFlowInstance.Cancel}" />
							</div>
						
						</form>
						
					</xsl:when>
					<xsl:otherwise>
					
						<h2 class="h1 nopadding"><xsl:value-of select="$i18n.GiveFlowInstance.Title" /></h2>
				
						<p>
							<xsl:value-of select="$i18n.GiveFlowInstance.Description"/>
						</p>
					
						<form name="changeOwnerForm" method="post" action="{/Document/requestinfo/uri}">
						
							<div class="floatleft full bigmarginbottom">
								
								<label class="floatleft full">
									<xsl:value-of select="$i18n.GiveFlowInstance.Recipient" />
								</label>
								
								<xsl:call-template name="UserList">
									<xsl:with-param name="connectorURL">
										<xsl:value-of select="/Document/requestinfo/currentURI"/>
										<xsl:text>/</xsl:text>
										<xsl:value-of select="/Document/module/alias"/>
										<xsl:text>/users</xsl:text>
									</xsl:with-param>
									<xsl:with-param name="name" select="'recipient'"/>
									<xsl:with-param name="users" select="OwnershipChangeRequest/Recipient" />
								</xsl:call-template>
							</div>
							
							<div class="text-align-center" style="margin-top: 30px">
								<input class="btn btn-green xl display-inline" type="submit" value="{$i18n.GiveFlowInstance.CreateRequest}" />
							</div>
						
						</form>
						
					</xsl:otherwise>
				</xsl:choose>
			
			</div>
		</div>
	
	</xsl:template>
	
	<xsl:template match="ReceiveFlowInstance">
	
		<div class="contentitem" style="margin-top: 0;">
			<section>
				<div class="section-full">
				
					<div class="section-inside full">
						<div class="heading-wrapper bigpaddingleft">
							<figure>
		 						<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{FlowInstance/Flow/flowID}" alt="" />
			 				</figure>
			 				<div class="heading">
								<h1 class="xl"><xsl:value-of select="FlowInstance/Flow/name"/></h1>
								<xsl:if test="FlowInstance/firstSubmitted">
									<span class="errandno"><xsl:value-of select="$i18n.FlowInstanceID" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="FlowInstance/flowInstanceID" /></span>
								</xsl:if>
							</div>
						</div>
					</div>
					
					<xsl:apply-templates select="ValidationErrors/validationError[not(starts-with(fieldName, 'accept'))]"/>
					
					<div class="clearboth bigmargin">
								
						<h2 class="h1 nopadding"><xsl:value-of select="$i18n.Request" /></h2>
						
						<p>
							<xsl:value-of select="$i18n.ReceiveFlowInstance.Description"/>
						</p>
						
						<div class="full bigmarginbottom">
							<label class="full">
								<xsl:value-of select="$i18n.Request.Poster" />
							</label>
							<div class="full">
								<xsl:value-of select="OwnershipChangeRequest/user/firstname"/>
								<xsl:text>&#160;</xsl:text>
								<xsl:value-of select="OwnershipChangeRequest/user/lastname"/>
							</div>
						</div>
						
						<div class="full bigmarginbottom">
							<label class="full">
								<xsl:value-of select="$i18n.Request.Added" />
							</label>
							<div class="full">
								<xsl:value-of select="OwnershipChangeRequest/added"/>
							</div>
						</div>
						
						<div class="divider"></div>
						
						<div class="header-full">
							<div class="description">
								<h2><xsl:value-of select="$i18n.Description" /></h2>
								<xsl:value-of select="FlowInstance/Flow/shortDescription" disable-output-escaping="yes" />
							</div>
						</div>
					
						<div class="service">
							<div class="queries">
								<xsl:apply-templates select="ManagerResponses/ManagerResponse"/>
							</div>
						</div>
						
						<form id="changeOwnerForm" name="changeOwnerForm" method="post" action="{/Document/requestinfo/uri}">
						
							<xsl:call-template name="createHiddenField">
								<xsl:with-param name="name" select="'accept'"/>
								<xsl:with-param name="class" select="'accept-field'"/>
							</xsl:call-template>
						
							<xsl:call-template name="createHiddenField">
								<xsl:with-param name="name" select="'decline'"/>
								<xsl:with-param name="class" select="'decline-field'"/>
							</xsl:call-template>
							
							<xsl:call-template name="createHiddenField">
								<xsl:with-param name="name" select="'declineReason'"/>
								<xsl:with-param name="class" select="'decline-field'"/>
							</xsl:call-template>
							
							<div style="display: none;">
							
								<div class="decline-modal contentitem">
									<div class="modal-content">
									
										<div class="modal-header bigmarginbottom">
											<h1><xsl:value-of select="$i18n.ReceiveFlowInstance.Decline.Title" /></h1>
										</div>
										
										<div class="modal-body">
										
											<p class="tiny full">
										  		<xsl:value-of select="$i18n.ReceiveFlowInstance.Decline.Description" />
										 	 </p>
											  
											<div class="full bigmarginbottom">
											
												<label for="alternative-value" class="full">
													<xsl:value-of select="$i18n.ReceiveFlowInstance.Decline.Reason" />
												</label>
												
												<div class="full">
													<xsl:call-template name="createTextArea">
														<xsl:with-param name="name" select="'declineReason'"/>
														<xsl:with-param name="class" select="'decline-field'"/>
													</xsl:call-template>
												</div>
											</div>
											
											<div class="full text-align-center bigmargintop">
												<input class="btn btn-red submit display-inline floatright bigmargintop bigmarginleft" type="button" value="{$i18n.ReceiveFlowInstance.Decline}" />
												<input class="btn btn-light close display-inline floatright bigmargintop" type="button" value="{$i18n.ReceiveFlowInstance.Cancel}" />
											</div>
											
										</div>
										
									</div>
								</div>
							
								<div class="accept-modal contentitem">
									<div class="modal-content">
									
										<div class="modal-header bigmarginbottom">
											<h1><xsl:value-of select="$i18n.ReceiveFlowInstance.Accept.Title" /></h1>
										</div>
										
										<div class="modal-body">
										
											<div class="accept-modal-errors bigmarginbottom">
												<xsl:apply-templates select="ValidationErrors/validationError[starts-with(fieldName, 'accept')]"/>
											</div>
										
											<p class="tiny full">
												<xsl:value-of select="$i18n.ReceiveFlowInstance.Accept.Description" />
											</p>
											
											<div class="full text-align-center bigmargintop">
												<input class="btn btn-green submit display-inline floatright bigmargintop bigmarginleft" type="button" value="{$i18n.ReceiveFlowInstance.Accept}" />
												<input class="btn btn-light close display-inline floatright bigmargintop" type="button" value="{$i18n.ReceiveFlowInstance.Cancel}" />
											</div>
											
										</div>
										
									</div>
								</div>
							
							</div>
							
							<div class="text-align-center bigmargintop">
								<input class="btn btn-green xl btn-inline bigmargintop bigmarginright" type="submit" value="{$i18n.ReceiveFlowInstance.Accept}" onclick="acceptOwnershipChange(this, event)" />
								<input class="btn btn-red xl btn-inline bigmargintop" type="submit" value="{$i18n.ReceiveFlowInstance.Decline}" onclick="declineOwnershipChange(this, event)" />
							</div>
						
						</form>
						
					</div>
				
				</div>
			</section>
		</div>
	
	</xsl:template>
	
	<xsl:template match="ManagerResponse">

		<xsl:variable name="stepID" select="currentStepID"/>
	
		<div class="section-full preview" style="padding-left: 0; padding-right: 0;">
			
			<h2 data-icon-before="c" class="h1">
				<xsl:value-of select="currentStepIndex + 1"/>
				<xsl:text>. </xsl:text>
				<xsl:value-of select="../../FlowInstance/Flow/Steps/Step[stepID = $stepID]/name"/>
			</h2>
			
			<xsl:choose>
				<xsl:when test="QueryResponses">
					<xsl:apply-templates select="QueryResponses/QueryResponse"/>
				</xsl:when>
				<xsl:otherwise>
					<p><xsl:value-of select="$i18n.noAnsweredQueriesInThisStep"/></p>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	
		<xsl:if test="position() != last()">
			<div class="divider preview" />
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="QueryResponse">
	
		<xsl:choose>
			<xsl:when test="HTML">
				<xsl:value-of select="HTML" disable-output-escaping="yes"/>
			</xsl:when>
			<xsl:otherwise>
				<div id="query_{QueryDescriptor/queryID}" class="hidden" />
			</xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>
	
	<xsl:template match="ListFlowInstancesExtension">
	
		<xsl:variable name="flowInstanceCount" select="count(WaitingChangeOwnerFlowInstances/FlowInstance)" />

		<div class="errands-wrapper draft">
			<div class="heading-wrapper">
				<h2><xsl:value-of select="$i18n.WaitingChangeOwnerFlowInstancesTitle" /></h2>
				<h3 class="clearboth"><xsl:value-of select="$i18n.WaitingChangeOwnerFlowInstances.Part1" />
				<xsl:text>&#160;</xsl:text>
				<strong>
					<xsl:value-of select="$flowInstanceCount" />
					<xsl:text>&#160;</xsl:text>
					<xsl:choose>
						<xsl:when test="$flowInstanceCount > 1">
							<xsl:value-of select="$i18n.WaitingChangeOwnerFlowInstances.Part2.Plural" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.WaitingChangeOwnerFlowInstances.Part2" />
						</xsl:otherwise>
					</xsl:choose>
				</strong>
				<xsl:text>&#160;</xsl:text><xsl:value-of select="$i18n.WaitingChangeOwnerFlowInstances.Part3" /></h3>
				<xsl:call-template name="createHelpDialog">
					<xsl:with-param name="id" select="'waiting-ChangeOwner'" />
					<xsl:with-param name="text" select="$i18n.WaitingChangeOwnerFlowInstancesHelp" />
				</xsl:call-template>
			</div>
			<table class="oep-table errand-table">
				<thead>
					<tr>
						<th class="icon"></th>
						<th class="service"><span><xsl:value-of select="$i18n.FlowName" /></span></th>
						
						<xsl:if test="SiteProfiles">
							<th class="status"><span><xsl:value-of select="$i18n.SiteProfile" /></span></th>
						</xsl:if>
						
						<th class="status"><span><xsl:value-of select="$i18n.Status" /></span></th>
						<th class="date"><span><xsl:value-of select="$i18n.Updated" /></span></th>
						<th class="link"></th>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="WaitingChangeOwnerFlowInstances/FlowInstance" mode="waiting-ChangeOwner" />
				</tbody>
			</table>
		</div>
	
		<div class="divider errands"></div>
	
	</xsl:template>
	
	<xsl:template match="FlowInstance" mode="waiting-ChangeOwner">
	
		<tr>
			<td class="icon"><i data-icon-before="w"></i></td>
			<td data-title="{$i18n.FlowName}" class="service"><xsl:value-of select="Flow/name" /></td>
			
			<xsl:call-template name="printSiteProfile"/>
			
			<td data-title="{$i18n.Status}" class="status"><xsl:value-of select="Status/name" /></td>
			<td data-title="{$i18n.Updated}" class="date">
				<xsl:choose>
					<xsl:when test="updated"><xsl:value-of select="updated" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="added" /></xsl:otherwise>
				</xsl:choose>
			</td>
			<td class="link">
				<a class="btn btn-green vertical-align-middle" href="{ChangeOwnerURL}"><xsl:value-of select="$i18n.WaitingForYourAcceptance" /></a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="createHelpDialog">
		
		<xsl:param name="id" />
		<xsl:param name="text" />
		<xsl:param name="class" select="''" />
		
		<div class="help {$class}">
			<a class="open-help" href="#" data-icon-after="?" data-help-box="helpdialog_{$id}"><span><xsl:value-of select="$i18n.Help" /></span></a>
			<div class="help-box" data-help-box="helpdialog_{$id}">
				<div>
		  			<div> 
		  				<a class="close" href="#" data-icon-after="x"></a>
		  				<xsl:copy-of select="$text" />
		  			</div> 
				</div>
			</div>
		</div>
		
		<div class="help-backdrop" data-help-box="helpdialog_{$id}" />
		
	</xsl:template>
	
	<xsl:template name="printSiteProfile">
	
		<xsl:if test="../../SiteProfiles">
			
			<td data-title="{$i18n.SiteProfile}" class="siteProfile">
				
				<xsl:choose>
					<xsl:when test="Attributes/Attribute/Name = 'siteProfileName'">
					
						<xsl:value-of select="Attributes/Attribute[Name = 'siteProfileName']/Value"/>
					
					</xsl:when>
					<xsl:otherwise>
					
						<xsl:variable name="profileID" select="profileID"/>
						
						<xsl:value-of select="../../SiteProfiles/Profile[profileID = $profileID]/name" />
					
					</xsl:otherwise>
				</xsl:choose>
			</td>
					
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="fieldName = 'recipient'">
						<xsl:value-of select="$i18n.GiveFlowInstance.Recipient"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>