package com.nordicpeak.flowengine.queries.childinfoquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.childrelationprovider.Child;
import com.nordicpeak.childrelationprovider.ChildRelationProvider;
import com.nordicpeak.childrelationprovider.ChildrenResponse;
import com.nordicpeak.childrelationprovider.Guardian;
import com.nordicpeak.childrelationprovider.exceptions.ChildRelationProviderException;
import com.nordicpeak.childrelationprovider.exceptions.CommunicationException;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.childinfoquery.beans.ChildInfoQuery;
import com.nordicpeak.flowengine.queries.childinfoquery.beans.ChildInfoQueryInstance;
import com.nordicpeak.flowengine.queries.childinfoquery.beans.StoredChild;
import com.nordicpeak.flowengine.queries.childinfoquery.enums.LookupMode;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class ChildInfoQueryProviderModule extends BaseQueryProviderModule<ChildInfoQueryInstance> implements BaseQueryCRUDCallback {

	@InstanceManagerDependency(required = true)
	private ChildRelationProvider childRelationProvider;

	private AnnotatedDAO<ChildInfoQuery> queryDAO;
	private AnnotatedDAO<ChildInfoQueryInstance> queryInstanceDAO;

	private ChildInfoQueryCRUD queryCRUD;

	private QueryParameterFactory<ChildInfoQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<ChildInfoQueryInstance, Integer> queryInstanceIDParamFactory;

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ChildInfoQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(ChildInfoQuery.class);
		queryInstanceDAO = daoFactory.getDAO(ChildInfoQueryInstance.class);

		queryCRUD = new ChildInfoQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<ChildInfoQuery>(ChildInfoQuery.class), "ChildInfoQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ChildInfoQuery query = new ChildInfoQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		ChildInfoQuery query = new ChildInfoQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		ChildInfoQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ChildInfoQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		ChildInfoQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new ChildInfoQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		if (descriptor.getQueryInstanceID() == null && queryInstance.getQuery().getLookupMode() == LookupMode.POSTER) {

			queryInstance.setChildren(getChildren(queryInstance, poster, null));
		}

		return queryInstance;
	}

	private ChildInfoQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<ChildInfoQuery> query = new HighLevelQuery<ChildInfoQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private ChildInfoQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<ChildInfoQuery> query = new HighLevelQuery<ChildInfoQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private ChildInfoQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<ChildInfoQueryInstance> query = new HighLevelQuery<ChildInfoQueryInstance>(ChildInfoQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(ChildInfoQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(ChildInfoQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		queryInstance.getQueryInstanceDescriptor().setPopulated(!queryInstance.getQuery().isDontSetPopulated());
		
		if(queryInstance.getQuery().isSetAsAttribute()) {
			
			queryInstance.setAttribute(attributeHandler);
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ChildInfoQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ChildInfoQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		ChildInfoQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, ChildInfoQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<ChildInfoQueryInstance> getQueryInstanceClass() {

		return ChildInfoQueryInstance.class;
	}

	private List<StoredChild> getChildren(ChildInfoQueryInstance queryInstance, User poster, RequestMetadata requestMetadata) {

		if (poster != null) {

			String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);

			return getChildren(citizenIdentifier, queryInstance);
		}

		return null;
	}

	private List<StoredChild> getChildren(String citizenIdentifier, ChildInfoQueryInstance queryInstance) {

		if (citizenIdentifier != null) {

			if (childRelationProvider == null) {

				log.error("Missing required instance manager dependency: childRelationProvider");
				queryInstance.setFetchChildrenException(new CommunicationException("Missing required instance manager dependency: childRelationProvider"));
				return null;
			}

			log.info("Getting children information for citizenIdentifier " + citizenIdentifier);

			try {
				
				queryInstance.setFetchChildrenException(null);
				
				ChildrenResponse childrenResponse = childRelationProvider.getChildren(citizenIdentifier, true, true);

				if (childrenResponse != null) {

					queryInstance.setHasChildrenUnderSecrecy(childrenResponse.hasChildrenUnderSecrecy());

					List<StoredChild> storedChildren = new ArrayList<StoredChild>();

					Map<String, Child> childMap = childrenResponse.getChildren();

					for (Entry<String, Child> entry : childMap.entrySet()) {

						boolean posterIsGuardian = false;

						for (Guardian guardian : entry.getValue().getGuardians()) {

							if (citizenIdentifier.equals(guardian.getCitizenIdentifier())) {

								posterIsGuardian = true;
								break;
							}
						}

						if (posterIsGuardian) {

							storedChildren.add(new StoredChild(entry.getValue()));
						}
					}

					return storedChildren;

				} else {

					queryInstance.setHasChildrenUnderSecrecy(false);
				}

			} catch (ChildRelationProviderException e) {

				queryInstance.setFetchChildrenException(e);
			}		
		}
		
		return null;
	}

	@Override
	public QueryResponse getFormHTML(ChildInfoQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if(queryInstance.getChildren() == null && queryInstance.getQuery().getLookupMode() == LookupMode.POSTER) {
			
			queryInstance.setChildren(getChildren(queryInstance, poster, null));
		
		}else if(queryInstance.getQuery().getLookupMode() == LookupMode.ATTRIBUTE) {
			
			queryInstance.setChildren(getChildren(attributeHandler.getString(queryInstance.getQuery().getLookupAttributeName()), queryInstance));
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

}
