<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="links">
		/css/childinfoquery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="ChildInfoQueryQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="childinfoquery show-mode">
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ChildInfoQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ChildInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="ChildInfoQueryInstance/ChildInfoQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="ChildInfoQueryInstance/ChildInfoQuery/description">
					<span class="italic">
						<xsl:value-of select="ChildInfoQueryInstance/ChildInfoQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
					
				<div>

					<h3><xsl:value-of select="$i18n.Children" /></h3>
					
					<div class="marginleft">

						<xsl:apply-templates select="ChildInfoQueryInstance/Children/Child"/>
					
					</div>
				
				</div>
				
			</article>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', ChildInfoQueryInstance/ChildInfoQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', ChildInfoQueryInstance/ChildInfoQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="ChildInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
			
			<xsl:if test="validationErrors">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
						<xsl:apply-templates select="validationErrors"/>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
			
			<article class="childinfoquery">
			
				<xsl:if test="validationErrors">
					<xsl:attribute name="class">childinfoquery error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ChildInfoQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ChildInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="ChildInfoQueryInstance/ChildInfoQuery/helpText">		
						<xsl:apply-templates select="ChildInfoQueryInstance/ChildInfoQuery/helpText" />
					</xsl:if>
				
				</div>

				<span class="italic">
					<xsl:if test="ChildInfoQueryInstance/ChildInfoQuery/description">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="ChildInfoQueryInstance/ChildInfoQuery/description" disable-output-escaping="yes" />
					</xsl:if>
						
					<xsl:if test="ChildInfoQueryInstance/hasChildrenUnderSecrecy = 'true'">
						<p class="tiny">
							<xsl:value-of select="$i18n.SecretChildrenInfo"/>
						</p>
					</xsl:if>
				</span>
				
				<fieldset>
				
					<xsl:choose>
						<xsl:when test="ChildInfoQueryInstance/Children/Child">
						
							<div>
								<xsl:apply-templates select="ChildInfoQueryInstance/Children/Child">
									<xsl:with-param name="styling" select="true()"/>
								</xsl:apply-templates>
							</div>
							
						</xsl:when>
						<xsl:when test="ChildInfoQueryInstance/Children">
						
							<xsl:choose>
								<xsl:when test="ChildInfoQueryInstance/ChildInfoQuery/lookupMode = 'ATTRIBUTE'">
									<xsl:value-of select="$i18n.Error.NoChildrenAttribute"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$i18n.Error.NoChildren"/>
								</xsl:otherwise>
							</xsl:choose>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:choose>
								<xsl:when test="ChildInfoQueryInstance/FetchChildrenException/CommunicationException"><xsl:value-of select="$i18n.Error.Provider.CommunicationError"/></xsl:when>
								<xsl:when test="ChildInfoQueryInstance/FetchChildrenException/IncompleteDataException"><xsl:value-of select="$i18n.Error.Provider.IncompleteData"/></xsl:when>
								<xsl:when test="ChildInfoQueryInstance/FetchChildrenException/InvalidCitizenIdentifierException"><xsl:value-of select="$i18n.Error.Provider.InvalidCitizenIdentifier"/></xsl:when>
								<xsl:otherwise><xsl:value-of select="$i18n.Error.Provider.Unknown"/></xsl:otherwise>
							</xsl:choose>
							
						</xsl:otherwise>
					</xsl:choose>
					
				</fieldset>
				
			</article>
				
		</div>
		
	</xsl:template>
	
	<xsl:template match="Child">
		<xsl:param name="styling" select="false()"/>
	
		<div>
		
			<xsl:if test="$styling">
				<xsl:attribute name="class">child</xsl:attribute>
			</xsl:if>
		
			<label>
			
				<xsl:value-of select="firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="lastname" />
				
			</label>

			<p class="tiny">
			
				<xsl:value-of select="$i18n.Column.SocialSecurityNumber" />
				<xsl:text>:&#160;</xsl:text>
				<xsl:value-of select="citizenIdentifier" />
				
				<xsl:if test="../../ChildInfoQuery/showAddress = 'true'">
				
					<br/>
					<xsl:value-of select="$i18n.Column.Address" />
					<xsl:text>:&#160;</xsl:text>
					
					<xsl:if test="address">
						<xsl:value-of select="address" />
						<br/>
					</xsl:if>
					
					<xsl:if test="zipCode or postalAddress">
						<xsl:if test="zipCode">
							<xsl:value-of select="zipCode" />
							<xsl:text>&#160;</xsl:text>
						</xsl:if>
						
						<xsl:value-of select="postalAddress" />
					</xsl:if>
					
				</xsl:if>
				
			</p>
			
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.Error.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<xsl:choose>
			<xsl:when test="fieldName != ''">
				
				<i data-icon-after="!" title="{$i18n.Error.UnknownValidationError}"></i>
				
			</xsl:when>
			<xsl:otherwise>
				
				<span>
					<strong data-icon-before="!">
						<xsl:value-of select="$i18n.Error.UnknownValidationError"/>
					</strong>
				</span>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
		
</xsl:stylesheet>