package com.nordicpeak.flowengine.evaluators.citizenidgroupmembershipquerystateevaluator;

import java.sql.SQLException;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.handlers.GroupHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.usergrouplist.UserGroupListConnector;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.CitizenIdentifierQuery;
import com.nordicpeak.flowengine.interfaces.CitizenIdentifierQueryInstance;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;


public class CitizenIDGroupMembershipQueryStateEvaluationProviderModule extends BaseQueryStateEvaluationProviderModule<CitizenIDGroupMembershipQueryStateEvaluator> {

	protected AnnotatedDAO<CitizenIDGroupMembershipQueryStateEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<CitizenIDGroupMembershipQueryStateEvaluator, Integer> evaluatorIDParamFactory;
	
	protected CitizenIDGroupMembershipQueryStateEvaluatorCRUD evaluatorCRUD;
	
	protected UserGroupListConnector userGroupListConnector;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, CitizenIDGroupMembershipQueryStateEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(CitizenIDGroupMembershipQueryStateEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new CitizenIDGroupMembershipQueryStateEvaluatorCRUD(CitizenIDGroupMembershipQueryStateEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
		
		userGroupListConnector = new UserGroupListConnector(systemInterface);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, CitizenIDGroupMembershipQueryStateEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (!(queryInstance instanceof CitizenIdentifierQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " is not of the type " + CitizenIdentifierQueryInstance.class.getName());
			return null;
		}
		
		CitizenIdentifierQueryInstance citizenIdentifierQueryInstance = (CitizenIdentifierQueryInstance) queryInstance;
		
		String citizenID = citizenIdentifierQueryInstance.getCitizenIdentifier();
		
		boolean hasGroupAccess = false;
		User foundUser = null;
		
		if (citizenID != null) {
			
			foundUser = systemInterface.getUserHandler().getUserByAttribute(Constants.MANAGER_CITIZEN_IDENTIFIER_ATTRIBUTE, citizenID, true, false);
			hasGroupAccess = foundUser != null && AccessUtils.checkGroupAccess(foundUser, evaluator.getGroupIDs());
			
			if (!hasGroupAccess) {
				
				foundUser = systemInterface.getUserHandler().getUserByAttribute(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE, citizenID, true, false);
				hasGroupAccess = foundUser != null && AccessUtils.checkGroupAccess(foundUser, evaluator.getGroupIDs());
			}
		}

		if ((hasGroupAccess || evaluator.isInverted()) && !(hasGroupAccess && evaluator.isInverted())) {

			return applyEvaluatorQueryStates(queryInstance, evaluator, callback, attributeHandler);

		} else {

			return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
		}
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		CitizenIDGroupMembershipQueryStateEvaluator evaluator = new CitizenIDGroupMembershipQueryStateEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return CitizenIdentifierQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		CitizenIDGroupMembershipQueryStateEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<CitizenIDGroupMembershipQueryStateEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	protected Class<CitizenIDGroupMembershipQueryStateEvaluator> getEvaluatorClass() {

		return CitizenIDGroupMembershipQueryStateEvaluator.class;
	}

	@Override
	protected AnnotatedDAO<CitizenIDGroupMembershipQueryStateEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	@Override
	protected QueryParameterFactory<CitizenIDGroupMembershipQueryStateEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return this.evaluatorIDParamFactory;
	}

	@Override
	public CitizenIDGroupMembershipQueryStateEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		CitizenIDGroupMembershipQueryStateEvaluator evaluator = importEvaluatorDescriptor(descriptor, transactionHandler, query, statusConversionMap);
		
		evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
	
	@WebPublic(alias = "groups")
	public ForegroundModuleResponse getGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {
		
		if(this.flowAdminModule == null || !AccessUtils.checkAccess(user, flowAdminModule.getModuleDescriptor())) {
			
			throw new AccessDeniedException("User does not have access to list groups via this module");
		}
		
		return userGroupListConnector.getGroups(req, res, user, uriParser);
	}

	public GroupHandler getGroupHandler() {
		return systemInterface.getGroupHandler();
	}
	
}
