<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="scripts">
		/js/coloredtagsettings.js
		/js/colorpicker/jquery.minicolors.min.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/coloredtagsettings.css
		/js/colorpicker/jquery.minicolors.css
	</xsl:variable>
	
	<xsl:variable name="scripts-buttonextension">
		/js/coloredtagbuttonextension.js
	</xsl:variable>
	
	<xsl:template match="Document">
	
		<xsl:apply-templates select="ShowSettings"/>
		<xsl:apply-templates select="UpdateSettings"/>
		<xsl:apply-templates select="ColorTagFlowInstances"/>

	</xsl:template>
	
	<xsl:template match="ShowSettings">
		
		<xsl:variable name="extensionImgPath"><xsl:value-of select="/Document/requestinfo/contextpath" /><xsl:value-of select="extensionRequestURL" />/static/pics</xsl:variable>
		
		<a name="coloredtags" />
		
		<xsl:choose>
			<xsl:when test="Settings">
				<xsl:value-of select="$i18n.Enabled" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NotEnabled" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/requestinfo/contextpath}{/Document/extensionRequestURL}/updateflowsettings" class="floatright">
			<xsl:value-of select="$i18n.UpdateSettings" />
			<img class="marginleft vertical-align-bottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="Settings">
			<a href="{/Document/requestinfo/contextpath}{/Document/extensionRequestURL}/deleteflowsettings" class="floatright clearboth" onclick="return confirm('{$i18n.DeleteSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteSettings" />
				<img class="marginleft vertical-align-bottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="UpdateSettings">

		<div id="ColoredTagsSettingAdmin" class="contentitem errands-wrapper">
		
			<h1 class="bigmarginbottom">
				<xsl:value-of select="$i18n.UpdateSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError"/>
	
			<form method="post" action="{/Document/requestinfo/contextpath}{/Document/extensionRequestURL}/updateflowsettings">
			
				<div class="floatleft full bigmarginbottom">
				
					<div class="floatleft full">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="name" select="'enabled'" />
							<xsl:with-param name="id" select="'enabled'" />
							<xsl:with-param name="element" select="Settings" />       
						</xsl:call-template>
						
						<label for="enabled">
							<xsl:value-of select="$i18n.Enabled" />
						</label>
					</div>
					
					<div id="tag-container">
					
						<div class="full floatleft bigmargintop">
							<h2><xsl:value-of select="$i18n.Tag.Title" /></h2>
						</div>
					
						<table id="tag-table" class="full coloredtable">
							<thead>
								<tr>
									<th><xsl:value-of select="$i18n.Tag.Name" /></th>
									<th><xsl:value-of select="$i18n.Tag.HexColor" /></th>
									<th width="16px"></th>
								</tr>
							</thead>
							<tbody>
							
								<xsl:choose>
									<xsl:when test="requestparameters">
									
										<xsl:apply-templates select="requestparameters/parameter[name = 'tagID']/value" mode="tag"/>
									
									</xsl:when>
									<xsl:otherwise>
									
										<xsl:apply-templates select="Settings/Tags/Tag"/>
									
									</xsl:otherwise>
								</xsl:choose>					
							
							</tbody>
						</table>
						
						<div class="floatright margintop marginbottom">
							<a onclick="addTag();">
								<xsl:value-of select="$i18n.Tag.Add" />
								<img class="alignbottom marginleft" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
							</a>
						</div>	
												
					</div>
	
				</div>
				
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
			
			<div style="display: none;">
				
				<table>
					<tr id="tag-template">
						<td>
							<input type="text" name="name" class="full"/>
						</td>
						<td>
							<input type="text" name="hexColor" class="color-picker"/>
						</td>
						<td>
							<input type="hidden" name="tagID"/>
							
							<a onclick="deleteTag(this)">
								<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
							</a>				
						</td>
					</tr>
				</table>
				
			</div>
		
		</div>
		
		<script type="text/javascript">
			$(function(){
			
				i18n = {
					ConfirmDeletePart1: "<xsl:value-of select="$i18n.DeleteMapping.Confirm.Part1" />",
					ConfirmDeletePart2Single: "<xsl:value-of select="$i18n.DeleteMapping.Confirm.Part2.Single" />",
					ConfirmDeletePart2Multiple: "<xsl:value-of select="$i18n.DeleteMapping.Confirm.Part2.Multiple" />",
				}
			});
		</script>

	</xsl:template>			
	
	<xsl:template match="Tag">
		
		<tr id="tag-{tagID}">
			<td>
				<input type="text" name="name-{tagID}" value="{name}" class="full"/>
			</td>
			<td>
				<input type="text" name="hexColor-{tagID}" value="{hexColor}" class="color-picker"/>
			</td>
			<td>
				<input type="hidden" name="tagID" value="{tagID}"/>
				
				<a onclick="deleteTag(this)" data-mapping-count="{count(Mappings/Mapping)}">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="value" mode="tag">
		
 		<xsl:variable name="tagID" select="."/>
		
		<xsl:variable name="nameParam" select="concat('name-', $tagID)"/>
		
		<xsl:variable name="hexColorParam" select="concat('hexColor-', $tagID)"/>
		
		<tr id="tag-{$tagID}">
 			<td>
				<input type="text" name="name-{$tagID}" value="{../../parameter[name = $nameParam]/value}" class="full"/>
			</td>
			<td>
				<input type="text" name="hexColor-{$tagID}" value="{../../parameter[name = $hexColorParam]/value}" class="color-picker"/>
			</td>
			<td>
				<input type="hidden" name="tagID" value="{$tagID}"/>
				
				<a onclick="deleteTag(this)" data-mapping-count="{count(../../../Settings/Tags/Tag[tagID = $tagID]/Mappings/Mapping)}">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>	
	
	<xsl:template match="ColorTagFlowInstances">
		
		<div id="FlowBrowser" class="contentitem errands-wrapper">
		
			<div>
			
				<h1>
					<xsl:value-of select="$i18n.ColorTagFlowInstances.Title"/>
					<xsl:text>:&#160;</xsl:text>
					<xsl:value-of select="Flow/name"/>
				</h1>
				
				<p class="nomargin">
					<xsl:value-of select="$i18n.ColorTagFlowInstances.Description.1"/>
					<xsl:text> </xsl:text>
					<strong>
						<xsl:value-of select="count(FlowInstances/flowInstanceID)"/>
					</strong>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$i18n.ColorTagFlowInstances.Description.2"/>
				</p>
					
				<xsl:if test="ValidationErrors/validationError[messageKey != 'NoColoredTagSetting']">
					<div class="bigmargintop">
						<xsl:apply-templates select="ValidationErrors/validationError"/>
					</div>
				</xsl:if>
				
				<form id="ColorTagFlowInstancesForm" method="POST" action="{/Document/requestinfo/uri}">
					
					<xsl:for-each select="FlowInstances/flowInstanceID">
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="name" select="'flowInstanceID'"/>
							<xsl:with-param name="value" select="."/>
						</xsl:call-template>
					</xsl:for-each>
					
					<article class="bigmargintop">
					
						<h2><xsl:value-of select="$i18n.ColorTagFlowInstances.Choose" />:</h2>
						
						<div class="clearfix"/>
						
						<div class="colortags-wrapper">
						
							<xsl:choose>
								<xsl:when test="Tags/Tag">
									
									<div class="colortag">
			
										<input type="radio" value="" id="tag_notag" name="tagID">
											
											<xsl:if test="requestparameters/parameter[name = 'tagID']/value = ''">
												<xsl:attribute name="checked"/>
											</xsl:if>
											
										</input>
										
										<label class="radio" for="tag_notag" data-color="#DDDDDD">
											<span style="background-color: #DDDDDD" />
											<xsl:value-of select="$i18n.ColorTagFlowInstances.NoColorTag"/>
										</label>
										
									</div>
									
									<xsl:apply-templates select="Tags/Tag" mode="radiobutton" >
										<xsl:with-param name="selectedID" select="requestparameters/parameter[name = 'tagID']/value" />
									</xsl:apply-templates>
									
								</xsl:when>
								<xsl:otherwise>
									
									<p class="error"><xsl:value-of select="$i18n.validationError.NoColoredTagSetting"/></p>
									
								</xsl:otherwise>
							</xsl:choose>
						
						</div>
						
					</article>
				
					<div class="floatright bigmargintop">
					
						<a class="btn btn-light btn-inline marginright" href="{/Document/requestinfo/contextpath}{/Document/fullAlias}" title="{$i18n.Back}">
							<xsl:value-of select="$i18n.Back"/>
						</a>
						<xsl:if test="Tags/Tag">
							<input class="btn btn-green btn-inline" type="submit" value="{$i18n.ColorTagFlowInstances.Submit}"/>
						</xsl:if>
					</div>
				
				</form>
			</div>
		
		</div>
		
	</xsl:template>
	
	<xsl:template match="Tag" mode="radiobutton">
	
		<xsl:param name="selectedID" />
	
		<div class="colortag">
			
			<input type="radio" value="{tagID}" id="tag_{tagID}" name="tagID">
				
				<xsl:if test="$selectedID = tagID">
					<xsl:attribute name="checked"/>
				</xsl:if>
				
			</input>
			
			<label class="radio" for="tag_{tagID}" data-color="{hexColor}">
				<span style="background-color: {hexColor}" />
				<xsl:value-of select="name"/>
			</label>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='AtLeastOneTagRequired']">
		<p class="error">
			<xsl:value-of select="$i18n.validationError.AtLeastOneTagRequired"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='NoColoredTagSetting']">
		<p class="error">
			<xsl:value-of select="$i18n.validationError.NoColoredTagSetting"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ColorTagNotFound']">
		<p class="error">
			<xsl:value-of select="$i18n.validationError.ColorTagNotFound"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidColorTag']">
		<p class="error">
			<xsl:value-of select="$i18n.validationError.InvalidColorTag"/>
		</p>
	</xsl:template>
		
	<xsl:template match="validationError">
		
		<xsl:if test="fieldName and validationErrorType">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="validationErrorType='RequiredField'">
							<xsl:value-of select="$i18n.validationError.RequiredField" />
						</xsl:when>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.validationError.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooShort'">
							<xsl:value-of select="$i18n.validationError.TooShort" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.validationError.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.Other" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#x20;</xsl:text>
					<xsl:choose>
						<xsl:when test="substring(fieldName, 1, 4) = 'name'">
							<xsl:value-of select="$i18n.Tag.Name" />
						</xsl:when>
						<xsl:when test="substring(fieldName, 1, 8) = 'hexColor'">
							<xsl:value-of select="$i18n.Tag.HexColor" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>!</xsl:text>
				</span>
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>