package com.nordicpeak.flowengine.coloredtag.daos;

import java.lang.reflect.Field;
import java.sql.SQLException;

import javax.sql.DataSource;

import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;

import com.nordicpeak.flowengine.coloredtag.beans.ColoredTagSettings;

public class ColoredTagSettingsDAO extends AnnotatedDAO<ColoredTagSettings> {

	private final QueryParameterFactory<ColoredTagSettings, Integer> flowFamilyIDParamFactory;

	public ColoredTagSettingsDAO(DataSource dataSource, Class<ColoredTagSettings> beanClass, AnnotatedDAOFactory daoFactory) {

		super(dataSource, beanClass, daoFactory);

		flowFamilyIDParamFactory = this.getParamFactory(ColoredTagSettings.FIELD_FLOW_FAMILY_ID, Integer.class);
	}

	public ColoredTagSettings getByFlowFamilyID(Integer flowFamilyID, Field... relations) throws SQLException {

		HighLevelQuery<ColoredTagSettings> query = new HighLevelQuery<>();

		query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamilyID));

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.get(query);
	}

	public int deleteByFlowFamilyID(Integer flowFamilyID) throws SQLException {

		HighLevelQuery<ColoredTagSettings> query = new HighLevelQuery<>();

		query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamilyID));

		return this.delete(query);
	}

}
