package com.nordicpeak.flowengine.queries.constructionpermit.beans;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.FCKContent;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;
import se.unlogic.webutils.annotations.URLRewrite;

import com.nordicpeak.flowengine.annotations.TextTagReplace;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;

@Table(name = "construction_permit_queries")
@XMLElement
public class ConstructionPermitQuery extends BaseQuery implements FixedAlternativesQuery {

	private static final long serialVersionUID = -5329625101347410733L;

	public static final Field MODEL_RELATION = ReflectionUtils.getField(ConstructionPermitQuery.class, "model");
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged(columnName="modelID")
	@ManyToOne
	@XMLElement
	private Model model;
	
	@FCKContent
	@TextTagReplace
	@URLRewrite
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement(cdata = true)
	protected String selectPermitsDescription;
	
	@FCKContent
	@TextTagReplace
	@URLRewrite
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement(cdata = true)
	protected String permitsSelectedDescription;	
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ConstructionPermitQueryInstance> instances;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	private String importModelID;
	
	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<ConstructionPermitQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<ConstructionPermitQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return getClass().getSimpleName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return getClass().getSimpleName() + queryID;
	}

	@Override
	public void toXSD(Document doc) {
		
		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());
		
		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);
		
		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);
		
		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);
		
		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);
		
		Element actionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		actionElement.setAttribute("name", "Action");
		actionElement.setAttribute("type", getXSDTypeName() + "Action");
		
		actionElement.setAttribute("minOccurs", "1");
		actionElement.setAttribute("maxOccurs", "unbounded");
				
		sequenceElement.appendChild(actionElement);
		
		doc.getDocumentElement().appendChild(complexTypeElement);
		
		// Action type
		Element actionComplexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		actionComplexTypeElement.setAttribute("name", getXSDTypeName() + "Action");
		
		Element userSequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		actionComplexTypeElement.appendChild(userSequenceElement);
		
		addElementType(doc, userSequenceElement, "Name", "xs:string");
		addElementType(doc, userSequenceElement, "Description", "xs:string", false);
		addElementType(doc, userSequenceElement, "From", "xs:string", false);
		addElementType(doc, userSequenceElement, "To", "xs:string", false);
		addElementType(doc, userSequenceElement, "Count", "xs:string", false);
		addElementType(doc, userSequenceElement, "RegisterTitle", "xs:string", false);
		addElementType(doc, userSequenceElement, "Weight", "xs:int", false);
		
		doc.getDocumentElement().appendChild(actionComplexTypeElement);
	}
	
	public void addElementType(Document doc, Element sequenceElement, String name, String type) {
		
		addElementType(doc, sequenceElement, name, type, null, true);
	}
	
	public void addElementType(Document doc, Element sequenceElement, String name, String type, boolean required) {
		
		addElementType(doc, sequenceElement, name, type, null, required);
	}
	
	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed) {
		
		addElementType(doc, sequenceElement, name, type, fixed, true);
	}
	
	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed, boolean required) {
		
		Element element = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		element.setAttribute("name", name);
		element.setAttribute("type", type);
		element.setAttribute("minOccurs", required ? "1" : "0");
		element.setAttribute("maxOccurs", "1");
		
		if (!StringUtils.isEmpty(fixed)) {
			element.setAttribute("fixed", fixed);
		}
		
		sequenceElement.appendChild(element);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		selectPermitsDescription = XMLValidationUtils.validateParameter("selectPermitsDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		permitsSelectedDescription = XMLValidationUtils.validateParameter("permitsSelectedDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		importModelID = xmlParser.getString("Model/modelID");
		
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (attributeName != null) {
			
			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public List<PermitNode> getAlternatives() {
		
		if(model != null){
			
			List<PermitNode> permitNodes = new ArrayList<>(40);
			
			if(model.getRootNodes() != null){
				
				for(PermitNode permitNode : model.getRootNodes()){
					
					getPermitNodes(permitNode, permitNodes);
				}
			}
			
			if(!permitNodes.isEmpty()){
				
				return permitNodes;
			}
		}
		
		return null;
	}

	private void getPermitNodes(PermitNode permitNode, List<PermitNode> permitNodes) {

		if(permitNode.isTitleNode() && permitNode.getChildNodes() != null){
			
			for(PermitNode childNode : permitNode.getChildNodes()){
				
				getPermitNodes(childNode, permitNodes);
			}
			
		}else{
			
			permitNodes.add(permitNode);
		}
	}

	@Override
	public String getFreeTextAlternative() {

		return null;
	}

	
	public Model getModel() {
	
		return model;
	}

	
	public void setModel(Model model) {
	
		this.model = model;
	}

	
	public String getImportModelID() {
	
		return importModelID;
	}
	
	public boolean isSetAsAttribute() {
		
		return setAsAttribute;
	}
	
	public void setSetAsAttribute(boolean setAsAttribute) {
	
		this.setAsAttribute = setAsAttribute;
	}
	
	public String getAttributeName() {
	
		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	@Override
	public Map<Integer, Integer> getAlternativeConversionMap() {

		return null;
	}
}
