package com.nordicpeak.flowengine.queries;

import java.lang.reflect.Field;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Time;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.time.TimeStringyfier;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.interfaces.SubmitCheck;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@Table(name = "date_time_query_instances")
@XMLElement
public class DateTimeQueryInstance extends BaseQueryInstance implements StringValueQueryInstance, SubmitCheck {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(DateTimeQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private DateTimeQuery query;

	@DAOManaged
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date startDate;

	@DAOManaged
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date endDate;

	@DAOManaged
	@XMLElement(valueFormatter = TimeStringyfier.class)
	private Time startTime;

	@DAOManaged
	@XMLElement(valueFormatter = TimeStringyfier.class)
	private Time endTime;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	public Date getStartDate() {

		return startDate;
	}

	public void setStartDate(Date startDate) {

		this.startDate = startDate;
	}

	public Date getEndDate() {

		return endDate;
	}

	public void setEndDate(Date endDate) {

		this.endDate = endDate;
	}

	public Time getStartTime() {

		return startTime;
	}

	public void setStartTime(Time startTime) {

		this.startTime = startTime;
	}

	public Time getEndTime() {

		return endTime;
	}

	public void setEndTime(Time endTime) {

		this.endTime = endTime;
	}

	@Override
	public DateTimeQuery getQuery() {

		return query;
	}

	public void setQuery(DateTimeQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.startDate = null;
		this.endDate = null;
		this.startTime = null;
		this.endTime = null;
		this.resetAttribute(attributeHandler);
		super.reset(attributeHandler);
	}

	public void setAttribute(MutableAttributeHandler attributeHandler){
		
		if(query.getAttributeName() != null){
			
			attributeHandler.setAttribute(query.getAttributeName(), getStringValue());
		}
	}
	
	public void resetAttribute(MutableAttributeHandler attributeHandler){
		
		attributeHandler.removeAttribute(query.getAttributeName());
	}	
	
	public void copyQueryValues() {

	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		if(startDate != null){
			
			XMLUtils.appendNewElement(doc, element, "StartDate", DateUtils.DATE_FORMATTER.format(startDate));
		}
		
		if(endDate != null){
			
			XMLUtils.appendNewElement(doc, element, "EndDate", DateUtils.DATE_FORMATTER.format(endDate));
		}
		
		if(startTime != null){
			
			XMLUtils.appendNewElement(doc, element, "StartTime", TimeUtils.TIME_SECONDS_FORMATTER .format(startTime));
		}
		
		if(endTime != null){
			
			XMLUtils.appendNewElement(doc, element, "EndTime", TimeUtils.TIME_SECONDS_FORMATTER .format(endTime));
		}			

		return element;
	}

	@Override
	public String getStringValue() {

		StringBuilder builder = new StringBuilder();
		
		if(startDate != null){
			
			builder.append(DateUtils.DATE_FORMATTER.format(startDate));
		}
		
		if(startTime != null){
			
			if(startDate != null){
				
				builder.append(" ");
			}
			
			builder.append(TimeUtils.TIME_FORMATTER .format(startTime));
		}
		
		if(endDate != null || endTime != null){
			
			builder.append(" - ");
		}
		
		if(endDate != null){
			
			builder.append(DateUtils.DATE_FORMATTER.format(endDate));
		}
		
		if(endTime != null){
			
			if(endDate != null){
				
				builder.append(" ");
			}
			
			builder.append(TimeUtils.TIME_FORMATTER .format(endTime));
		}		
		
		return builder.toString();
	}

	@Override
	public boolean isValidForSubmit(MutableAttributeHandler attributeHandler, User poster, QueryHandler queryHandler, FlowInstanceManager flowInstanceManager) throws SQLException {

		return queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), DateTimeQueryProviderModule.class).isValidForSubmit(attributeHandler, this, poster);
	}
}
