<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/timepicker/js/jquery.timepicker.js?v=1
	</xsl:variable>

	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
		/timepicker/css/jquery.timepicker.css
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/jquery.ui.datepicker-sv.js
		/js/datetimequery.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/datetimequery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="DateTimeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="datetimequery show-mode">
				
				<div class="heading-wrapper">
					<xsl:if test="DateTimeQueryInstance/DateTimeQuery/hideTitle = 'false'">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="DateTimeQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="DateTimeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
										
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="DateTimeQueryInstance/DateTimeQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="DateTimeQueryInstance/DateTimeQuery/description">
					<span class="italic">
						<xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<fieldset>
				
					<xsl:variable name="useMultiDate"  select="DateTimeQueryInstance/DateTimeQuery/useMultiDate = 'true'" />
					<xsl:variable name="useMultiTime"  select="DateTimeQueryInstance/DateTimeQuery/useMultiTime = 'true'" />
				
					<xsl:if test="DateTimeQueryInstance/startDate or DateTimeQueryInstance/endDate">
						<div class="split">
					
							<xsl:if test="DateTimeQueryInstance/startDate">
							
								<div class="split date-field">
									
									<xsl:variable name="label">
										<xsl:value-of select="$i18n.Date"/>
										<xsl:if test="$useMultiDate">
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="$i18n.from" />
										</xsl:if>
									</xsl:variable>
									
									<label>
										<xsl:value-of select="$label" />
									</label>
							
									<div><xsl:value-of select="DateTimeQueryInstance/startDate" /></div>
																
								</div>
								
							</xsl:if>
							
							<xsl:if test="DateTimeQueryInstance/endDate">
								
									<div class="split date-field">
										
										<xsl:variable name="label">
											<xsl:value-of select="$i18n.Date"/>
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="$i18n.to" />
										</xsl:variable>
										
										<label>
											<xsl:value-of select="$label"/>
										</label>
								
										<div><xsl:value-of select="DateTimeQueryInstance/endDate" /></div>
									
									</div>
								
							</xsl:if>
							
							
						
						</div>
					</xsl:if>
					
					
						<div class="split odd">
							<xsl:if test="DateTimeQueryInstance/startTime">
							
								<div class="split time-field">
									
									<xsl:variable name="label">
										<xsl:value-of select="$i18n.Time"/>
										<xsl:if test="$useMultiTime">
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="$i18n.from" />
										</xsl:if>
									</xsl:variable>
								
									<label>
										<xsl:value-of select="$label" />
									</label>
							
									<div><xsl:value-of select="DateTimeQueryInstance/startTime" /></div>
									
								</div>
								
							</xsl:if>
						
							
							<xsl:if test="DateTimeQueryInstance/endTime">
								
								<div class="split time-field">
									
									<xsl:variable name="label">
										<xsl:value-of select="$i18n.Time"/>
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="$i18n.to" />
									</xsl:variable>
								
									<label>
										<xsl:value-of select="$label"/>
									</label>
							
									<div><xsl:value-of select="DateTimeQueryInstance/endTime" /></div>
								
								</div>
								
							</xsl:if>
					
						</div>
					
				
				</fieldset>
				
			</article>
				
		</div>
		
	</xsl:template>	
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', DateTimeQueryInstance/DateTimeQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', DateTimeQueryInstance/DateTimeQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="DateTimeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						
						<xsl:choose>
							<xsl:when test="ValidationErrors/validationError[messageKey]">
								
								<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
								
							</xsl:when>
							<xsl:otherwise>
							
								<span/>
								
							</xsl:otherwise>
						</xsl:choose>
						
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article class="datetimequery">
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">datetimequery error</xsl:attribute>
				</xsl:if>
			
				<xsl:variable name="useMultiDate"  select="DateTimeQueryInstance/DateTimeQuery/useMultiDate = 'true'" />
				<xsl:variable name="useMultiTime"  select="DateTimeQueryInstance/DateTimeQuery/useMultiTime = 'true'" />
			
				<div class="heading-wrapper">
					<xsl:if test="DateTimeQueryInstance/DateTimeQuery/hideTitle = 'false'">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="DateTimeQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="DateTimeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
										
					<xsl:if test="DateTimeQueryInstance/DateTimeQuery/helpText">		
						<xsl:apply-templates select="DateTimeQueryInstance/DateTimeQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:if test="DateTimeQueryInstance/DateTimeQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
				
				<xsl:variable name="fieldsetclass">
					<xsl:if test="DateTimeQueryInstance/DateTimeQuery/useDateFields = 'true' and $useMultiDate != 'true' and DateTimeQueryInstance/DateTimeQuery/useTimeFields = 'true' and $useMultiTime">
						displayflex
					</xsl:if>
				</xsl:variable>
				
				<fieldset class="{$fieldsetclass}">
				
					<xsl:if test="DateTimeQueryInstance/DateTimeQuery/useDateFields = 'true'">
						<div class="split">
					
							
							
								<xsl:variable name="fieldName" select="concat($shortQueryID, '_startDate')" />
							
								<xsl:variable name="class">
									<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
										<xsl:text>invalid input-error</xsl:text>
									</xsl:if>
								</xsl:variable>
							
								<div class="split date-field {$class}">
									
									<xsl:variable name="label">
										<xsl:value-of select="$i18n.Date"/>
										<xsl:if test="$useMultiDate">
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="$i18n.from" />
										</xsl:if>
									</xsl:variable>
									
									<label>
										<xsl:value-of select="$label" />
									</label>
							
									<xsl:call-template name="createTextField">
										<xsl:with-param name="id" select="$fieldName" />
										<xsl:with-param name="name" select="$fieldName" />
										<xsl:with-param name="title" select="$label"/>
										<xsl:with-param name="type" select="'date'"/>
										<xsl:with-param name="value" select="DateTimeQueryInstance/startDate"/>
										<xsl:with-param name="class" select="$class"/>
										<xsl:with-param name="disabled" select="Locked"/>
									</xsl:call-template>
									
									<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
								
								</div>
								
							
							
							<xsl:if test="DateTimeQueryInstance/DateTimeQuery/useDateFields = 'true' and $useMultiDate">
								
									<xsl:variable name="fieldName" select="concat($shortQueryID, '_endDate')" />
								
									<xsl:variable name="class">
										<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
											<xsl:text>invalid input-error</xsl:text>
										</xsl:if>
									</xsl:variable>
								
									<div class="split date-field {$class}">
										
										<xsl:variable name="label">
											<xsl:value-of select="$i18n.Date"/>
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="$i18n.to" />
										</xsl:variable>
										
										<label>
											<xsl:value-of select="$label"/>
										</label>
								
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="$fieldName" />
											<xsl:with-param name="name" select="$fieldName" />
											<xsl:with-param name="title" select="$label"/>
											<xsl:with-param name="type" select="'date'"/>
											<xsl:with-param name="value" select="DateTimeQueryInstance/endDate"/>
											<xsl:with-param name="class" select="$class"/>
											<xsl:with-param name="disabled" select="Locked"/>
										</xsl:call-template>
										
										<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
									
									</div>
								
								</xsl:if>		
							
						
						</div>
					</xsl:if>
					
						<div class="split odd ">
						
							<xsl:if test="DateTimeQueryInstance/DateTimeQuery/useTimeFields = 'true'">
							
							<xsl:variable name="fieldName" select="concat($shortQueryID, '_startTime')" />
							
							<xsl:variable name="class">
								<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
									<xsl:text>invalid input-error</xsl:text>
								</xsl:if>
							</xsl:variable>
							
							<div class="split time-field {$class}">
								
								<xsl:variable name="label">
									<xsl:value-of select="$i18n.Time"/>
									<xsl:if test="$useMultiTime">
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="$i18n.from" />
									</xsl:if>
								</xsl:variable>
							
								<label>
									<xsl:value-of select="$label" />
								</label>
						
								<xsl:call-template name="createTextField">
									<xsl:with-param name="id" select="$fieldName" />
									<xsl:with-param name="name" select="$fieldName" />
									<xsl:with-param name="title" select="$label"/>
									<xsl:with-param name="type" select="'time'"/>
									<xsl:with-param name="value" select="DateTimeQueryInstance/startTime"/>
									<xsl:with-param name="class" select="$class"/>
									<xsl:with-param name="disabled" select="Locked"/>
								</xsl:call-template>
								
								<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
								
							</div>
							
						</xsl:if>
						
							
							
							<xsl:if test="DateTimeQueryInstance/DateTimeQuery/useTimeFields = 'true' and $useMultiTime">
								
								<xsl:variable name="fieldName" select="concat($shortQueryID, '_endTime')" />
								
								<xsl:variable name="class">
									<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
										<xsl:text>invalid input-error</xsl:text>
									</xsl:if>
								</xsl:variable>
								
								<div class="split time-field {$class}">
									
									<xsl:variable name="label">
										<xsl:value-of select="$i18n.Time"/>
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="$i18n.to" />
									</xsl:variable>
								
									<label>
										<xsl:value-of select="$label"/>
									</label>
							
									<xsl:call-template name="createTextField">
										<xsl:with-param name="id" select="$fieldName" />
										<xsl:with-param name="name" select="$fieldName" />
										<xsl:with-param name="title" select="$label"/>
										<xsl:with-param name="type" select="'time'"/>
										<xsl:with-param name="value" select="DateTimeQueryInstance/endTime"/>
										<xsl:with-param name="class" select="$class"/>
										<xsl:with-param name="disabled" select="Locked"/>
									</xsl:call-template>
									
									<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
								
								</div>
								
							</xsl:if>
					
						</div>
					
				
				</fieldset>
				
				<script type="text/javascript">
				
					<xsl:variable name="timeInterval">
						<xsl:choose>
							<xsl:when test="DateTimeQueryInstance/DateTimeQuery/timeInterval"><xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/timeInterval" /></xsl:when>
							<xsl:otherwise>15</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					
					<xsl:variable name="startTime">
						<xsl:choose>
							<xsl:when test="DateTimeQueryInstance/DateTimeQuery/earliestTime"><xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/earliestTime" /></xsl:when>
							<xsl:otherwise>00:00</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					
					<xsl:variable name="endTime">
						<xsl:choose>
							<xsl:when test="DateTimeQueryInstance/DateTimeQuery/latestTime"><xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/latestTime" /></xsl:when>
							<xsl:otherwise>23:59</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					
					initDateTimeQuery('<xsl:value-of select="DateTimeQueryInstance/DateTimeQuery/queryID" />', <xsl:value-of select="$timeInterval" />, '<xsl:value-of select="$startTime" />', '<xsl:value-of select="$endTime" />');
					
				</script>
				
			</article>			
		
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'StartDateNotBigEnoughPositive']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateNotBigEnoughPositive.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.StartDateNotBigEnoughPositive.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateNotBigEnoughPositive']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateNotBigEnoughPositive.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.EndDateNotBigEnoughPositive.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateNotBigEnoughToday']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateNotBigEnoughToday" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateNotBigEnoughToday']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateNotBigEnoughToday" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateNotBigEnoughNegative']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateNotBigEnoughNegative.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.StartDateNotBigEnoughNegative.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateNotBigEnoughNegative']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateNotBigEnoughNegative.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.EndDateNotBigEnoughNegative.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateTooBigPositive']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateTooBigPositive.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.StartDateTooBigPositive.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateTooBigPositive']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateTooBigPositive.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.EndDateTooBigPositive.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateTooBigToday']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateTooBigToday" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateTooBigToday']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateTooBigToday" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateTooBigNegative']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateTooBigNegative.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.StartDateTooBigNegative.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateTooBigNegative']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateTooBigNegative.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.EndDateTooBigNegative.2" />
			</strong>
		</span>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'StartDateAfterEndDate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateAfterEndDate" />
			</strong>
		</span>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'DateIntervalMonthsToSmall']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalMonthsToSmall.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalMonthsToSmall.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'DateIntervalYearsToSmall']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalMonthsToSmall.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="years" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalYears" />
				<xsl:text>&#160;</xsl:text>
				<xsl:if test="months &gt; 0"> 
					<xsl:value-of select="$i18n.DateIntervalAnd" />
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="months" />
					<xsl:text>&#160;</xsl:text>
					<xsl:choose>
						<xsl:when test="months = 1">
							<xsl:value-of select="$i18n.DateIntervalMonth" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.DateIntervalMonths" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#160;</xsl:text>
				</xsl:if>
				<xsl:value-of select="$i18n.DateIntervalYearsToSmall.2" />
			</strong>
		</span>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'DateIntervalDaysToSmall']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalDaysToSmall.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalDaysToSmall.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'DateIntervalMonthsToBig']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalMonthsToBig.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalMonthsToBig.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'DateIntervalYearsToBig']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalMonthsToBig.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="years" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalYears" />
				<xsl:text>&#160;</xsl:text>
				<xsl:if test="months &gt; 0"> 
					<xsl:value-of select="$i18n.DateIntervalAnd" />
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="months" />
					<xsl:text>&#160;</xsl:text>
					<xsl:choose>
						<xsl:when test="months = 1">
							<xsl:value-of select="$i18n.DateIntervalMonth" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.DateIntervalMonths" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#160;</xsl:text>
				</xsl:if>
				<xsl:value-of select="$i18n.DateIntervalYearsToBig.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'DateIntervalDaysToBig']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.DateIntervalDaysToBig.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.DateIntervalDaysToBig.2" />
			</strong>
		</span>

	</xsl:template>

	
	<xsl:template match="validationError[messageKey = 'StartTimeAfterEndTime']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartTimeAfterEndTime" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'StartDateBeforeMinDate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateBeforeMinDate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDateAfterMaxDate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartDateAfterMaxDate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateBeforeMinDate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateBeforeMinDate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'EndDateAfterMaxDate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndDateAfterMaxDate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'StartTimeTooEarly']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartTimeTooEarly" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'StartTimeTooLate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.StartTimeTooLate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'EndTimeTooEarly']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndTimeTooEarly" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'EndTimeTooLate']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.EndTimeTooLate" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TimeNotMultipleOfInterval']">
	
		<span>
			<strong data-icon-before="!">

				<xsl:value-of select="$i18n.TimeNotMultipleOfInterval.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="interval" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.TimeNotMultipleOfInterval.2" />
				<xsl:text>&#160;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="fieldName = 'startTime'">
						<xsl:value-of select="$i18n.FromTime"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endTime'">
						<xsl:value-of select="$i18n.ToTime"/>
					</xsl:when>
				</xsl:choose>
			
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="originalTime" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.TimeNotMultipleOfInterval.3" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="alteredTime" />
			</strong>
		</span>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'StartDatOnNotAllowedWeekday']">
	
		<span>
			<strong data-icon-before="!">

				<xsl:value-of select="$i18n.StartDatOnNotAllowedWeekday.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
			</strong>
		</span>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RestrictTomorrowOnTimeNeedsStartDateAfterToday']">

		<span>
			<strong data-icon-before="!">

				<xsl:value-of select="$i18n.RestrictTomorrowOnTimeNeedsStartDateAfterToday" />
			</strong>
		</span>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RestrictTomorrowOnTimeNeedsStartDateTodayOrAfter']">

		<span>
			<strong data-icon-before="!">

				<xsl:value-of select="$i18n.RestrictTomorrowOnTimeNeedsStartDateTodayOrAfter" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'TomorrowIsNotAllowedNow']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TomorrowIsNotAllowedNow.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.TomorrowIsNotAllowedNow.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'TodayIsNotAllowedNow']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TodayIsNotAllowedNow.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.TodayIsNotAllowedNow.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentMonthIsNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentMonthIsNotAllowed" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentAndOneUpcomingMonthAreNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentAndOneUpcomingMonthAreNotAllowed" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentAndSomeUpcomingMonthsAreNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentAndSomeUpcomingMonthsAreNotAllowed.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.CurrentAndSomeUpcomingMonthsAreNotAllowed.2" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentEndMonthIsNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentEndMonthIsNotAllowed" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentAndOneUpcomingEndMonthAreNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentAndOneUpcomingEndMonthAreNotAllowed" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'CurrentAndSomeUpcomingEndMonthsAreNotAllowed']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.CurrentAndSomeUpcomingEndMonthsAreNotAllowed.1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.CurrentAndSomeUpcomingEndMonthsAreNotAllowed.2" />
			</strong>
		</span>

	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!" title="{$i18n.InvalidFormat}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
		
</xsl:stylesheet>