<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>
	
	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/jquery.ui.datepicker-sv.js
	</xsl:variable>
	
	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowSettings "/>
		<xsl:apply-templates select="UpdateSettings" />
		
	</xsl:template>
	
	<xsl:template match="ShowSettings">
	
		<a name="deduplicationsettings" />
		
		<xsl:choose>
			<xsl:when test="FlowFamilyDeduplicationSettings">
				
				<div class="full floatleft">
					<xsl:value-of select="$i18n.Settings.Mode" />
					<xsl:text> </xsl:text>
					<xsl:choose>
						<xsl:when test="FlowFamilyDeduplicationSettings/matchPoster = 'true' and not(FlowFamilyDeduplicationSettings/attribute)">
							
							<xsl:value-of select="$i18n.Settings.Mode.Poster" />
							
						</xsl:when>
						<xsl:when test="FlowFamilyDeduplicationSettings/matchPoster = 'true' and FlowFamilyDeduplicationSettings/attribute">
							
							<xsl:value-of select="$i18n.Settings.Mode.PosterAndAttribute" />
							
						</xsl:when>
						<xsl:when test="not(FlowFamilyDeduplicationSettings/matchPoster = 'true') and FlowFamilyDeduplicationSettings/attribute">
							
							<xsl:value-of select="$i18n.Settings.Mode.Attribute" />
							
						</xsl:when>
					</xsl:choose>
				</div>
				
				<xsl:if test="FlowFamilyDeduplicationSettings/attribute">
					<div class="full floatleft">
						<xsl:value-of select="$i18n.Settings.Attribute" />
						<xsl:text>: </xsl:text>
						<xsl:value-of select="FlowFamilyDeduplicationSettings/attribute" />
					</div>
				</xsl:if>
				
				<xsl:if test="FlowFamilyDeduplicationSettings/startDate">
					<div class="full floatleft">
						<xsl:value-of select="$i18n.Settings.StartDate" />
						<xsl:text>: </xsl:text>
						<xsl:value-of select="FlowFamilyDeduplicationSettings/startDate" />
					</div>
				</xsl:if>
				
				<xsl:if test="FlowFamilyDeduplicationSettings/endDate">
					<div class="full floatleft">
						<xsl:value-of select="$i18n.Settings.EndDate" />
						<xsl:text>: </xsl:text>
						<xsl:value-of select="FlowFamilyDeduplicationSettings/endDate" />
					</div>
				</xsl:if>
				
			</xsl:when>
			<xsl:otherwise>
			
				<xsl:value-of select="$i18n.SettingsDisabled" />
				
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/updatesettings/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateSettings" />
			<img class="marginleft" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="FlowFamilyDeduplicationSettings">
			<a href="{/Document/ModuleURI}/deletesettings/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteSettings" />
				<img class="marginleft" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="UpdateSettings">
		
		<div id="deduplicationsettings" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdateSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="ValidationErrors/validationError" />
	
			<form method="post" action="{/Document/ModuleURI}/updatesettings/{Flow/flowID}">
			
				<label>
					<xsl:value-of select="$i18n.Settings.Mode" />
				</label>
			
				<div class="floatleft full bigmarginbottom">

					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'match-poster'" />
							<xsl:with-param name="name" select="'mode'" />
							<xsl:with-param name="value" select="'poster'" />
							<xsl:with-param name="checked" select="FlowFamilyDeduplicationSettings/matchPoster = 'true' and not(FlowFamilyDeduplicationSettings/attribute)" />
						</xsl:call-template>
						
						<label for="match-poster">
							<xsl:value-of select="$i18n.Settings.Mode.Poster" />
						</label>
					</div>
	
				</div>
				
				<div class="floatleft full bigmarginbottom">

					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'match-both'" />
							<xsl:with-param name="name" select="'mode'" />
							<xsl:with-param name="value" select="'both'" />
							<xsl:with-param name="checked" select="FlowFamilyDeduplicationSettings/matchPoster = 'true' and FlowFamilyDeduplicationSettings/attribute" />
						</xsl:call-template>
						
						<label for="match-both">
							<xsl:value-of select="$i18n.Settings.Mode.PosterAndAttribute" />
						</label>
					</div>
	
				</div>
				
				<div class="floatleft full bigmarginbottom">

					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'match-attribute'" />
							<xsl:with-param name="name" select="'mode'" />
							<xsl:with-param name="value" select="'attribute'" />
							<xsl:with-param name="checked" select="not(FlowFamilyDeduplicationSettings/matchPoster = 'true') and FlowFamilyDeduplicationSettings/attribute" />
						</xsl:call-template>
						
						<label for="match-attribute">
							<xsl:value-of select="$i18n.Settings.Mode.Attribute" />
						</label>
					</div>
	
				</div>
				
				<script type="text/javascript">
				
					$(function() {
	
						var showHideAttributeField = function(){
						
							var value = $("input[name='mode']:checked").val();
							var useAttribute = value != "poster";
							
							$("#attribute").attr("disabled", !useAttribute).parent().toggle(useAttribute);
						};
						
						$("input[name='mode']").on("change", function() {
							showHideAttributeField();
						});
						
						showHideAttributeField();
					
					});
				
				</script>
				
				<div class="floatleft full">
				
					<label for="attribute" class="floatleft full">
						<xsl:value-of select="$i18n.Settings.Attribute" />
						<span class="required">*</span>
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'attribute'"/>
						<xsl:with-param name="name" select="'attribute'"/>
						<xsl:with-param name="element" select="FlowFamilyDeduplicationSettings"/>
					</xsl:call-template>
					
				</div>
				
				<div class="floatleft full">
				
					<label for="statusName" class="floatleft full">
						<xsl:value-of select="$i18n.Settings.StatusName" />
						<span class="required">*</span>
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'statusName'"/>
						<xsl:with-param name="name" select="'statusName'"/>
						<xsl:with-param name="element" select="FlowFamilyDeduplicationSettings"/>
					</xsl:call-template>
					
				</div>
				
				<div class="floatleft full">
				
					<label for="startDate" class="floatleft full">
						<xsl:value-of select="$i18n.Settings.StartDate" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'startDate'"/>
						<xsl:with-param name="name" select="'startDate'"/>
						<xsl:with-param name="type" select="'date'"/>
						<xsl:with-param name="element" select="FlowFamilyDeduplicationSettings"/>
					</xsl:call-template>
					
				</div>
				
				<div class="floatleft full">
				
					<label for="attribute" class="floatleft full">
						<xsl:value-of select="$i18n.Settings.EndDate" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'endDate'"/>
						<xsl:with-param name="name" select="'endDate'"/>
						<xsl:with-param name="type" select="'date'"/>
						<xsl:with-param name="element" select="FlowFamilyDeduplicationSettings"/>
					</xsl:call-template>
					
				</div>
					
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
			
			<script type="text/javascript">
			
				if (!Modernizr.inputtypes.date) {

					$("#deduplicationsettings input[type='date']").datepicker({
						changeMonth: true,
						changeYear: true,
						showWeek: true,
						dayNamesShort: true,
						onSelect: function() {
							$(this).trigger("blur");
						}
					});
				}
    
			</script>
		
		</div>

	</xsl:template>	
	
	<xsl:template match="validationError">
	
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="fieldName = 'mode'">
						<xsl:value-of select="$i18n.Settings.Mode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'attribute'">
						<xsl:value-of select="$i18n.Settings.Attribute"/>
					</xsl:when>
					<xsl:when test="fieldName = 'statusName'">
						<xsl:value-of select="$i18n.Settings.StatusName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'startDate'">
						<xsl:value-of select="$i18n.Settings.StartDate"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endDate'">
						<xsl:value-of select="$i18n.Settings.EndDate"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>