<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" >
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/featherlight/js/featherlight.min.js
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>		
	
	<xsl:variable name="scripts">
		/js/UserGroupList.js
		/js/documentsigneradmin.js?v=1
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/flowengine.css
		/css/UserGroupList.css?v=1
		/css/documentsigneradmin.css
	</xsl:variable>
	
	<xsl:template match="Document">
		
		<div id="DocumentSignerModule" class="contentitem errands-wrapper">
			
			<xsl:apply-templates select="ListDocuments" />
			<xsl:apply-templates select="ShowDocument" />
			<xsl:apply-templates select="AddDocument" />
			<xsl:apply-templates select="UpdateDocument" />
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="ListDocuments">
		
		<div class="floatright">
			<a class="btn btn-green display-inline-block" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/adddocument">
				<xsl:value-of select="$i18n.AddDocument"/>
			</a>
		</div>
		
		<h1>
			<xsl:value-of select="$i18n.ListDocuments" />
		</h1>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		
		<xsl:if test="PendingDocuments/Document">
			<fieldset class="bigmarginbottom">
				<legend><xsl:value-of select="$i18n.ListDocuments.PendingDocuments" /></legend>
				
				<table class="oep-table">
					<thead class="sortable">
						<tr>
							<th class="default-sort">
								<span><xsl:value-of select="$i18n.Document.name" /></span>
							</th>
							<th>
								<span><xsl:value-of select="$i18n.Document.poster" /></span>
							</th>
						</tr>
					</thead>
					<tbody>
						<xsl:apply-templates select="PendingDocuments/Document" mode="list-pending" />
					</tbody>
				</table>
			</fieldset>
		</xsl:if>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.ListDocuments.MyDocuments" /></legend>
			
			<xsl:choose>
				<xsl:when test="Documents/Document">
			
					<table class="oep-table">
						<thead class="sortable">
							<tr>
								<th class="default-sort">
									<span><xsl:value-of select="$i18n.Document.name" /></span>
								</th>
								<th>
									<span><xsl:value-of select="$i18n.Document.filename" /></span>
								</th>
								<th>
									<span><xsl:value-of select="$i18n.Document.signatures" /></span>
								</th>
								<th>
									<span><xsl:value-of select="$i18n.Document.status" /></span>
								</th>
								<th>
									<span title="{$i18n.Document.lastDate.title}">
										<xsl:value-of select="$i18n.Document.lastDate" />
									</span>
								</th>
								<th style="width: 40px;">
								</th>
							</tr>
						</thead>
						<tbody>
									<xsl:apply-templates select="Documents/Document" mode="list" />
						</tbody>
					</table>
				</xsl:when>
						<xsl:otherwise>
							<div class="bigmarginbottom marginleft">
								<xsl:value-of select="$i18n.ListDocuments.NoDocuments" />
							</div>
						</xsl:otherwise>
			</xsl:choose>
		</fieldset>
		
	</xsl:template>
	
	<xsl:template match="Document" mode="list-pending">
		
		<tr>
			<td data-title="{$i18n.Document.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sign/{documentID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Document.poster}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sign/{documentID}">
					<xsl:value-of select="poster/firstname" />
					<xsl:text> </xsl:text>
					<xsl:value-of select="poster/lastname" />
				</a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="Document" mode="list">
		
		<tr>
			<td data-title="{$i18n.Document.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showdocument/{documentID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Document.filename}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showdocument/{documentID}">
					<xsl:value-of select="filename" />
				</a>
			</td>
			<td data-title="{$i18n.Document.signatures}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showdocument/{documentID}">
					<xsl:value-of select="count(Signers/DocumentSigner[signedDate])" />
					<xsl:text> / </xsl:text>
					<xsl:value-of select="count(Signers/DocumentSigner)" />
				</a>
			</td>
			<td data-title="{$i18n.Document.status}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showdocument/{documentID}">
					<xsl:call-template name="documentStatus"/>
				</a>
			</td>
			<td data-title="{$i18n.Document.lastDate}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showdocument/{documentID}">
					<xsl:choose>
						<xsl:when test="completedDate">
							<xsl:value-of select="completedDate" />
						</xsl:when>
						<xsl:when test="startDate">
							<xsl:value-of select="startDate" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="added" />
						</xsl:otherwise>
					</xsl:choose>
				</a>
			</td>
			<td>
				<xsl:if test="status = 'SAVED' or status = 'CANCELLED'">
					<a class="marginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatedocument/{documentID}" title="{$i18n.UpdateDocument}: {name}">
						<img src="{$imgPath}/pen.png" alt="" />
					</a>
				</xsl:if>
				
				<xsl:if test="status != 'SIGNING'">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletedocument/{documentID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.Document.Delete}: {Document/name}">
						<img src="{$imgPath}/delete.png" alt="" />
					</a>
				</xsl:if>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="ShowDocument">
		
		<div class="floatright">
			<xsl:if test="Document/status = 'SAVED' or Document/status = 'CANCELLED'">
			
				<a class="marginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatedocument/{Document/documentID}">
					<img src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletedocument/{Document/documentID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.Document.Delete}: {Document/name}">
					<img src="{$imgPath}/delete.png" alt="" />
				</a>
				
			</xsl:if>
		</div>
		
		<h1>
			<xsl:value-of select="$i18n.ShowDocument" />
			<xsl:text>: </xsl:text>
			<xsl:value-of select="Document/name" />
		</h1>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<div>
			<strong>
				<xsl:value-of select="$i18n.DocumentSigner.name" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:value-of select="Document/name" />
		</div>
		
		<div class="marginbottom">
			<strong>
				<xsl:value-of select="$i18n.Document.status" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:call-template name="documentStatus">
				<xsl:with-param name="status" select="Document/status" />
			</xsl:call-template>
		</div>
		
		<div>
			<strong>
				<xsl:value-of select="$i18n.Document.added" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:value-of select="Document/added" />
		</div>
		
		<xsl:if test="Document/startDate">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.startDate" />
					<xsl:text>: </xsl:text>
				</strong>
				
				<xsl:value-of select="Document/startDate" />
			</div>
		</xsl:if>
		
		<xsl:if test="Document/completedDate">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.completedDate" />
					<xsl:text>: </xsl:text>
				</strong>
				
				<xsl:value-of select="Document/completedDate" />
			</div>
		</xsl:if>
		
		<div class="marginbottom margintop">
			
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.ExistingFile" />
				</strong>
			</div>
			
			<div>
				<xsl:value-of select="$i18n.Document.filename" />
				<xsl:text>: </xsl:text>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/originalpdf/{Document/documentID}" title="{$i18n.DownloadUnsignedPDF}">
					<xsl:value-of select="Document/filename" />
				</a>
			</div>
			
			<div>
				<xsl:value-of select="$i18n.Document.filesize" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="Document/FormattedFilesize" />
			</div>
			
		</div>
		
		<xsl:if test="Document/description">

			<div class="marginbottom">
				<strong>
					<xsl:value-of select="$i18n.Document.description" />
					<xsl:text>:</xsl:text>
				</strong>
				
				<div>
					<xsl:call-template name="replaceLineBreak">
						<xsl:with-param name="string" select="Document/description"/>
					</xsl:call-template>
				</div>
			</div>
		
		</xsl:if>
		
		<div>
			<strong>
				<xsl:value-of select="$i18n.Document.cancelInXDays" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:value-of select="Document/cancelInXDays" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="$i18n.Days" />
		</div>
		
		<div>
			<strong>
				<xsl:value-of select="$i18n.Document.sendReminderInXDays" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:value-of select="Document/sendReminderInXDays" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="$i18n.Days" />
		</div>
		
		<div class="marginbottom">
			<strong>
				<xsl:value-of select="$i18n.Document.deleteAfterXDays" />
				<xsl:text>: </xsl:text>
			</strong>
			
			<xsl:value-of select="Document/deleteAfterXDays" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="$i18n.Days" />
		</div>
		
		<xsl:if test="Document/useSequentialSigning = 'true'">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.useSequentialSigning" />
				</strong>
			</div>
		</xsl:if>
		
		<xsl:if test="Document/showPreviousSignaturesToSigners = 'true'">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.showPreviousSignaturesToSigners" />
				</strong>
			</div>
		</xsl:if>
		
		<xsl:if test="Document/showSignedDocumentInUserModule = 'true'">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.showSignedDocumentInUserModule" />
				</strong>
			</div>
		</xsl:if>
		
		<xsl:if test="Document/publishOriginal = 'true'">
			<div>
				<strong>
					<xsl:value-of select="$i18n.Document.publishOriginal" />
				</strong>
			</div>
		</xsl:if>		
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Document.Signers" /></legend>
			
			<table class="oep-table">
				<thead>
					<tr>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.name" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.email" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.citizenID" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.mobilePhone" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.signedDate" /></span>
						</th>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Document/Signers/DocumentSigner" mode="show" />
				</tbody>
			</table>
			
		</fieldset>
		
		<div class="floatright margintop">
			<a class="btn btn-light btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" title="{$i18n.Back}">
				<xsl:value-of select="$i18n.Back"/>
			</a>
			
			<xsl:if test="Document/status = 'SIGNING'">
				<a class="btn btn-red btn-inline bigmarginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/cancel/{Document/documentID}" onclick="return confirmPost(this.href, '{$i18n.Document.CancelConfirm}');">
					<xsl:value-of select="$i18n.Document.Cancel"/>
				</a>
			</xsl:if>
			
			<xsl:if test="Document/completedDate and Document/status = 'SIGNED'">
				<a class="btn btn-green btn-inline bigmarginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/signedpdf/{Document/documentID}">
					<xsl:value-of select="$i18n.DownloadSignedPDF"/>
				</a>
				
				<xsl:if test="Document/publishOriginal = 'false'">
					<a class="btn btn-green btn-inline bigmarginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/signreceiptpdf/{Document/documentID}">
						<xsl:value-of select="$i18n.DownloadSignReceiptPDF"/>
					</a>
				</xsl:if>
			</xsl:if>
		</div>
		
	</xsl:template>
	
	<xsl:template match="DocumentSigner" mode="show">
		
		<tr>
			<td data-title="{$i18n.DocumentSigner.name}">
				<xsl:if test="user and type != 'EXTERNAL_USER'">
					<img class="usericon vertical-align-middle marginright" src="{$imgPath}/user.png" alt="" />
				</xsl:if>
				<span>
					<xsl:value-of select="firstname" />
					<xsl:text> </xsl:text>
					<xsl:value-of select="lastname" />
					
					<xsl:if test="type != 'EXTERNAL_USER'">
						
						<xsl:if test="user/username or title">
							<xsl:text> (</xsl:text>
						</xsl:if>
						
						<xsl:if test="user/username">
							<xsl:value-of select="user/username" />
						</xsl:if>
						
						<xsl:if test="title">
							<xsl:if test="user/username">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:value-of select="title" />
						</xsl:if>
						
						<xsl:if test="user/username or title">
							<xsl:text>)</xsl:text>
						</xsl:if>
						
					</xsl:if>
				</span>
			</td>
			<td data-title="{$i18n.DocumentSigner.email}">
				<xsl:value-of select="email" />
			</td>
			<td data-title="{$i18n.DocumentSigner.citizenID}">
				<xsl:if test="type = 'EXTERNAL_USER'">
					<xsl:value-of select="citizenID" />
				</xsl:if>
			</td>
			<td data-title="{$i18n.DocumentSigner.mobilePhone}">
				<xsl:if test="type = 'EXTERNAL_USER'">
					<xsl:value-of select="mobilePhone" />
				</xsl:if>
			</td>
			<td data-title="{$i18n.DocumentSigner.signedDate}">
				<xsl:value-of select="signedDate" />
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="AddDocument">
		
		<h1>
			<xsl:value-of select="$i18n.AddDocument" />
		</h1>
		
		<xsl:if test="validationException">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationException/validationError"/>
			</div>
		</xsl:if>
		
		<form role="form" method="POST" enctype="multipart/form-data" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="EditDocument">
				<xsl:with-param name="mode" select="'add'"/>
				<xsl:with-param name="poster" select="/Document/user"/>
			</xsl:call-template>
			
			<input id="sendOnSave" type="hidden" name="sendOnSave" value="false" />
			
			<div class="floatright margintop marginbottom">
				<a class="btn btn-red btn-inline marginbottom" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Cancel"/></a>
				<input class="btn btn-green btn-inline marginbottom" type="submit" onclick="$('#sendOnSave').val('false');" value="{$i18n.AddDocument.save}" />
				<input class="btn btn-green btn-inline marginbottom" type="submit" onclick="$('#sendOnSave').val('true');" value="{$i18n.AddDocument.saveAndSend}" />
			</div>
		</form>
		
	</xsl:template>
	
	<xsl:template match="UpdateDocument">
		
		<div class="floatright">
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletedocument/{Document/documentID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.Document.Delete}: {Document/name}">
				<img src="{$imgPath}/delete.png" alt="" />
			</a>
		</div>
		
		<h1>
			<xsl:value-of select="$i18n.UpdateDocument" />
			<xsl:text>: </xsl:text>
			<xsl:value-of select="Document/name"/>
		</h1>
		
		<xsl:if test="validationException">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationException/validationError"/>
			</div>
		</xsl:if>
		
		<form role="form" method="POST" enctype="multipart/form-data" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="EditDocument">
				<xsl:with-param name="mode" select="'update'"/>
				<xsl:with-param name="poster" select="Document/poster"/>
			</xsl:call-template>
			
			<input id="sendOnSave" type="hidden" name="sendOnSave" value="false" />
			
			<div class="floatright margintop">
				<a class="btn btn-red btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Cancel"/></a>
				<input class="btn btn-green btn-inline" type="submit" onclick="$('#sendOnSave').val('false');" value="{$i18n.AddDocument.save}" />
				<input class="btn btn-green btn-inline" type="submit" onclick="$('#sendOnSave').val('true');" value="{$i18n.AddDocument.saveAndSend}" />
			</div>
		</form>
		
	</xsl:template>
	
	<xsl:template name="EditDocument">
		<xsl:param name="mode"/>
		<xsl:param name="poster"/>
		
		<div class="floatleft full marginbottom">
			
			<label for="name" class="floatleft full required">
				<xsl:value-of select="$i18n.Document.name" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'name'"/>
					<xsl:with-param name="name" select="'name'"/>
					<xsl:with-param name="element" select="Document" />
				</xsl:call-template>
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			
			<label for="description" class="floatleft full">
				<xsl:value-of select="$i18n.Document.description" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="id" select="'description'"/>
					<xsl:with-param name="name" select="'description'"/>
					<xsl:with-param name="element" select="Document" />
				</xsl:call-template>
			</div>
		</div>
		
		<xsl:if test="Document/filename and Document/filesize">
			
			<div class="floatleft full bigmarginbottom">
			
				<label class="floatleft full">
					<xsl:value-of select="$i18n.Document.ExistingFile" />
				</label>
				
				<div>
					<xsl:value-of select="$i18n.Document.filename" />
					<xsl:text>: </xsl:text>
					<xsl:value-of select="Document/filename" />
				</div>
				
				<div>
					<xsl:value-of select="$i18n.Document.filesize" />
					<xsl:text>: </xsl:text>
					<xsl:value-of select="Document/FormattedFilesize" />
				</div>
				
			</div>
			
		</xsl:if>
		
		<div class="floatleft full">
		
			<label class="floatleft full">
				<xsl:value-of select="$i18n.Document.NewFile" />
			</label>
			
			<div>
				<xsl:value-of select="$i18n.FileUpload.MaximumFileSize" /><xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="FormatedMaxFileSize" />
			</div>
			
			<div class="floatleft full">
				<input type="file" name="file" accept=".pdf,application/pdf"/>
			</div>
			
		</div>
		
		<div class="floatleft full bigmarginbottom">
			
			<label for="cancelInXDays" class="floatleft full required">
				<xsl:value-of select="$i18n.Document.cancelInXDays" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'cancelInXDays'"/>
					<xsl:with-param name="name" select="'cancelInXDays'"/>
					<xsl:with-param name="width" select="'80px'" />
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="value" select="DefaultCancelInXDays" />
				</xsl:call-template>
				
				<span class="marginleft">
					<xsl:value-of select="$i18n.Days" />
				</span>
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			
			<label for="sendReminderInXDays" class="floatleft full required">
				<xsl:value-of select="$i18n.Document.sendReminderInXDays" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'sendReminderInXDays'"/>
					<xsl:with-param name="name" select="'sendReminderInXDays'"/>
					<xsl:with-param name="width" select="'80px'" />
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="value" select="DefaultSendReminderInXDays" />
				</xsl:call-template>
				
				<span class="marginleft">
					<xsl:value-of select="$i18n.Days" />
				</span>
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			
			<label for="deleteAfterXDays" class="floatleft full required">
				<xsl:value-of select="$i18n.Document.deleteAfterXDays" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'deleteAfterXDays'"/>
					<xsl:with-param name="name" select="'deleteAfterXDays'"/>
					<xsl:with-param name="width" select="'80px'" />
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="value" select="DefaultDeleteAfterXDays" />
				</xsl:call-template>
				
				<span class="marginleft">
					<xsl:value-of select="$i18n.Days" />
				</span>
			</div>
		</div>
		
		<script type="text/javascript">
			DocumentSigning18n = {
				deleteConfirm: "<xsl:value-of select="$i18n.DocumentSigner.DeleteConfirm"/>",
				moduleURI: "<xsl:value-of select="/Document/requestinfo/contextpath" /><xsl:value-of select="/Document/module/fullAlias" />"
			};
		</script>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Document.Signers" /></legend>
			
			<table class="oep-table">
				<thead class="sortable">
					<tr>
						<th style="width: 16px;" />
						<th class="default-sort">
							<span><xsl:value-of select="$i18n.DocumentSigner.name" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.email" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.citizenID" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.DocumentSigner.mobilePhone" /></span>
						</th>
						<th style="width: 37px;" />
					</tr>
				</thead>
				<tbody id="signers" class="sortable">
					
					<xsl:choose>
						<xsl:when test="requestparameters">
							<xsl:apply-templates select="requestparameters/parameter[name = 'signerID']" mode="DocumentSigner" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="Document/Signers/DocumentSigner" mode="update" />
						</xsl:otherwise>
					</xsl:choose>
				</tbody>
			</table>
			
			<div id="poster-info" data-userid="{$poster/userID}" data-firstname="{$poster/firstname}" data-lastname="{$poster/lastname}" data-email="{$poster/email}" data-username="{$poster/username}" style="display: none;" />
			
			<table style="display: none;">
				<tbody id="signerTemplate">
					<xsl:call-template name="documentSigner">
						<xsl:with-param name="ID" select="''" />
						<xsl:with-param name="sortIndex" select="''" />
						<xsl:with-param name="type" select="''" />
						<xsl:with-param name="userID" select="''" />
						<xsl:with-param name="citizenID" select="''" />
						<xsl:with-param name="firstname" select="''" />
						<xsl:with-param name="lastname" select="''" />
						<xsl:with-param name="email" select="''" />
						<xsl:with-param name="mobilePhone" select="''" />
						<xsl:with-param name="disabled" select="'true'" />
					</xsl:call-template>
				</tbody>
			</table>
			
			<div style="display: none;">
			
				<div id="signerModal" class="contentitem">
					<div class="modal-content">
					
						<div class="modal-header bigmarginbottom">
							<h1>
								<xsl:value-of select="$i18n.DocumentSigner.ModalTitle" />
							</h1>
						</div>
						
						<div class="modal-body">
							
							<div class="full marginbottom">
								
								<div class="full">
									<xsl:call-template name="createRadio">
										<xsl:with-param name="id" select="'type_poster'"/>
										<xsl:with-param name="name" select="'type'"/>
										<xsl:with-param name="value" select="'POSTER'"/>
									</xsl:call-template>
									
									<label for="type_poster">
										<xsl:value-of select="$i18n.DocumentSigner.type.poster" />
									</label>
								</div>
							</div>
							
							<div class="full marginbottom">
								
								<div class="full">
									<xsl:call-template name="createRadio">
										<xsl:with-param name="id" select="'type_internal'"/>
										<xsl:with-param name="name" select="'type'"/>
										<xsl:with-param name="value" select="'INTERNAL_USER'"/>
									</xsl:call-template>
									
									<label for="type_internal">
										<xsl:value-of select="$i18n.DocumentSigner.type.internalUser" />
									</label>
								</div>
							</div>
							
							<div class="full marginbottom">
							
								<div class="full">
									<xsl:call-template name="createRadio">
										<xsl:with-param name="id" select="'type_external'"/>
										<xsl:with-param name="name" select="'type'"/>
										<xsl:with-param name="value" select="'EXTERNAL_USER'"/>
									</xsl:call-template>
									
									<label for="type_external">
										<xsl:value-of select="$i18n.DocumentSigner.type.externalUser" />
									</label>
								</div>
							</div>
							
							<div id="internalFields">
								
								<input type="hidden" name="email" disabled="true" value="" />
								<input type="hidden" name="username" disabled="true" value="" />
								<input type="hidden" name="title" disabled="true" value="" />
								
								<div class="full marginbottom">
								
									<label>
										<xsl:value-of select="$i18n.DocumentSigner.User" />
									</label>
									
									<xsl:call-template name="UserList">
										<xsl:with-param name="connectorURL">
											<xsl:value-of select="/Document/requestinfo/currentURI"/>
											<xsl:text>/</xsl:text>
											<xsl:value-of select="/Document/module/alias"/>
											<xsl:text>/users</xsl:text>
										</xsl:with-param>
										<xsl:with-param name="name" select="'internal-user'"/>
										<xsl:with-param name="showEmail" select="true()"/>
									</xsl:call-template>
								</div>
								
							</div>
							
							<div id="externalFields">
								
								<p id="invalidCitizenID" class="error" style="display: none;">
									<xsl:value-of select="$i18n.Validation.InvalidFormat"/>
									<xsl:text> </xsl:text>
									<xsl:value-of select="$i18n.DocumentSigner.citizenID" />
								</p>
								
								<p id="invalidEmail" class="error" style="display: none;">
									<xsl:value-of select="$i18n.Validation.InvalidFormat"/>
									<xsl:text> </xsl:text>
									<xsl:value-of select="$i18n.DocumentSigner.email" />
								</p>
								
								<p id="invalidMobilePhone" class="error" style="display: none;">
									<xsl:value-of select="$i18n.Validation.InvalidFormat"/>
									<xsl:text> </xsl:text>
									<xsl:value-of select="$i18n.DocumentSigner.mobilePhone" />
								</p>
								
								<p id="emailOrPhoneRequired" class="error" style="display: none;">
									<xsl:value-of select="$i18n.Validation.EmailOrPhoneRequired"/>
								</p>
								
								<div class="full marginbottom">
									<label for="firstname" class="required">
										<xsl:value-of select="$i18n.DocumentSigner.firstname" />
									</label>
									
									<div class=" full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="'firstname'"/>
											<xsl:with-param name="name" select="'firstname'"/>
										</xsl:call-template>
									</div>
								</div>
								
								<div class="full marginbottom">
									<label for="lastname">
										<xsl:value-of select="$i18n.DocumentSigner.lastname" />
									</label>
									
									<div class="full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="'lastname'"/>
											<xsl:with-param name="name" select="'lastname'"/>
										</xsl:call-template>
									</div>
								</div>
								
								<div class="full marginbottom">
									<label for="email">
										<xsl:value-of select="$i18n.DocumentSigner.email" />
									</label>
									
									<div class="full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="'email'"/>
											<xsl:with-param name="name" select="'email'"/>
										</xsl:call-template>
									</div>
								</div>
								
								<div class="full marginbottom">
									<label for="mobilePhone">
										<xsl:value-of select="$i18n.DocumentSigner.mobilePhone" />
									</label>
									
									<div class="full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="'mobilePhone'"/>
											<xsl:with-param name="name" select="'mobilePhone'"/>
										</xsl:call-template>
									</div>
								</div>
								
								<div class="full marginbottom">
									<label for="citizenID">
										<xsl:value-of select="$i18n.DocumentSigner.citizenID" />
									</label>
									
									<p class="tiny">
										<xsl:value-of select="$i18n.DocumentSigner.CitizenIDInfo.1" />
										<br/>
										<xsl:value-of select="$i18n.DocumentSigner.CitizenIDInfo.2" />
									</p>
									
									<div class="full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="id" select="'citizenID'"/>
											<xsl:with-param name="name" select="'citizenID'"/>
											<xsl:with-param name="placeholder" select="$i18n.DocumentSigner.citizenID.placeholder"/>
										</xsl:call-template>
									</div>
								</div>
								
							</div>
							
							<input class="close bigmargintop floatright paddingbottom bigmarginleft" id="saveSigner" type="button" value="{$i18n.DocumentSigner.ModalClose}" />
							<input class="close bigmargintop floatright paddingbottom" id="saveAndOpenSigner" type="button" value="{$i18n.DocumentSigner.ModalReOpen}" />
							
						</div>
						
					</div>
				</div>
				
			</div>
			
			<div class="floatright marginright">
				<a href="#" id="addDocumentSignerRef" onclick="addDocumentSigner(this, event)">
					<xsl:value-of select="$i18n.DocumentSigner.Add" />
					<img class="marginleft" src="{$imgPath}/add.png" alt="" />
				</a>
			</div>
			
			<div class="floatleft full marginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'useSequentialSigning'"/>
					<xsl:with-param name="name" select="'useSequentialSigning'"/>
					<xsl:with-param name="element" select="Document" />
				</xsl:call-template>
				
				<label for="useSequentialSigning">
					<xsl:value-of select="$i18n.Document.useSequentialSigning" />
				</label>
			</div>
			
			<div class="floatleft full marginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showPreviousSignaturesToSigners'"/>
					<xsl:with-param name="name" select="'showPreviousSignaturesToSigners'"/>
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="checked" select="'true'" />
				</xsl:call-template>
				
				<label for="showPreviousSignaturesToSigners">
					<xsl:value-of select="$i18n.Document.showPreviousSignaturesToSigners" />
				</label>
			</div>
			
			<div class="floatleft full marginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showSignedDocumentInUserModule'"/>
					<xsl:with-param name="name" select="'showSignedDocumentInUserModule'"/>
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="checked" select="'true'" />
				</xsl:call-template>
				
				<label for="showSignedDocumentInUserModule">
					<xsl:value-of select="$i18n.Document.showSignedDocumentInUserModule" />
				</label>
			</div>
			
			<div class="floatleft full marginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'publishOriginal'"/>
					<xsl:with-param name="name" select="'publishOriginal'"/>
					<xsl:with-param name="element" select="Document" />
				</xsl:call-template>
				
				<label for="publishOriginal">
					<xsl:value-of select="$i18n.Document.publishOriginal" />
				</label>
			</div>
			
			<xsl:variable name="sendEmail" select="SendEmailWhenSigningPartiallyDone"/>
			
			<div class="floatleft full marginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'sendEmailWhenSigningPartiallyDone'"/>
					<xsl:with-param name="name" select="'sendEmailWhenSigningPartiallyDone'"/>
					<xsl:with-param name="element" select="Document" />
					<xsl:with-param name="checked" select="$sendEmail" />
				</xsl:call-template>
				
				<label for="sendEmailWhenSigningPartiallyDone">
					<xsl:value-of select="$i18n.Flow.sendEmailWhenSigningPartiallyDone" />
				</label>
			</div>
			
		</fieldset>
		
	</xsl:template>
	
	<xsl:template match="parameter" mode="DocumentSigner">
	
		<xsl:for-each select="value">
			<xsl:variable name="id" select="." />
			
			<xsl:call-template name="documentSigner">
				<xsl:with-param name="ID" select="$id" />
				<xsl:with-param name="type" select="../../parameter[name = concat('signer-type-', $id)]/value" />
				<xsl:with-param name="sortIndex" select="../../parameter[name = concat('signer-sortIndex-', $id)]/value" />
				<xsl:with-param name="userID" select="../../parameter[name = concat('signer-userID-', $id)]/value" />
				<xsl:with-param name="citizenID" select="../../parameter[name = concat('signer-citizenID-', $id)]/value" />
				<xsl:with-param name="firstname" select="../../parameter[name = concat('signer-firstname-', $id)]/value" />
				<xsl:with-param name="lastname" select="../../parameter[name = concat('signer-lastname-', $id)]/value" />
				<xsl:with-param name="email" select="../../parameter[name = concat('signer-email-', $id)]/value" />
				<xsl:with-param name="mobilePhone" select="../../parameter[name = concat('signer-mobilePhone-', $id)]/value" />
				<xsl:with-param name="username" select="../../parameter[name = concat('signer-username-', $id)]/value" />
				<xsl:with-param name="title" select="../../parameter[name = concat('signer-title-', $id)]/value" />
			</xsl:call-template>
		</xsl:for-each>
		
	</xsl:template>
	
	<xsl:template match="DocumentSigner" mode="update" name="documentSigner">
		<xsl:param name="ID" select="signerID" />
		<xsl:param name="sortIndex" select="sortIndex" />
		<xsl:param name="type" select="type" />
		<xsl:param name="userID" select="user/userID" />
		<xsl:param name="citizenID" select="citizenID" />
		<xsl:param name="firstname" select="firstname" />
		<xsl:param name="lastname" select="lastname" />
		<xsl:param name="email" select="email" />
		<xsl:param name="mobilePhone" select="mobilePhone" />
		<xsl:param name="username" select="user/username" />
		<xsl:param name="title" select="title" />
		<xsl:param name="disabled" select="''" />
		
		<tr>
			<td>
				<img class="vertical-align-middle cursor-move" src="{$imgPath}/move.png" alt="{$i18n.Move}" />
				<input type="hidden" name="signerID" id="signerID" value="{$ID}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-sortIndex-{$ID}" id="signer-sortIndex-{$ID}" value="{$sortIndex}" class="sortorder" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-type-{$ID}" id="signer-type-{$ID}" value="{$type}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-userID-{$ID}" id="signer-userID-{$ID}" value="{$userID}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-citizenID-{$ID}" id="signer-citizenID-{$ID}" value="" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
					<xsl:if test="not($userID) or $userID= ''">
						<xsl:attribute name="value">
							<xsl:value-of select="$citizenID" />
						</xsl:attribute>
					</xsl:if>
				</input>
				<input type="hidden" name="signer-firstname-{$ID}" id="signer-firstname-{$ID}" value="{$firstname}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-lastname-{$ID}" id="signer-lastname-{$ID}" value="{$lastname}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-email-{$ID}" id="signer-email-{$ID}" value="{$email}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-mobilePhone-{$ID}" id="signer-mobilePhone-{$ID}"  value="" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
					<xsl:if test="not($userID) or $userID= ''">
						<xsl:attribute name="value">
							<xsl:value-of select="$mobilePhone" />
						</xsl:attribute>
					</xsl:if>
				</input>
				<input type="hidden" name="signer-username-{$ID}" id="signer-username-{$ID}" value="{$username}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
				<input type="hidden" name="signer-title-{$ID}" id="signer-title-{$ID}" value="{$title}" >
					<xsl:if test="$disabled != ''"><xsl:attribute name="disabled">true</xsl:attribute></xsl:if>
				</input>
			</td>
			<td class="name" data-title="{$i18n.DocumentSigner.name}">
				<img class="usericon vertical-align-middle marginright" src="{$imgPath}/user.png" alt="" >
					<xsl:if test="not($userID) or $userID = ''">
						<xsl:attribute name="style">display: none;</xsl:attribute>
					</xsl:if>
				</img>
				<span>
					<xsl:value-of select="$firstname" />
					<xsl:text> </xsl:text>
					<xsl:value-of select="$lastname" />
					
					<xsl:if test="($username and $username != '') or ($title and $title != '')">
						<xsl:text> (</xsl:text>
					</xsl:if>
					
					<xsl:if test="($username and $username != '')">
						<xsl:value-of select="$username" />
					</xsl:if>
					
					<xsl:if test="$title and $title != ''">
						<xsl:if test="$username">
							<xsl:text>, </xsl:text>
						</xsl:if>
						<xsl:value-of select="$title" />
					</xsl:if>
					
					<xsl:if test="($username and $username != '') or ($title and $title != '')">
						<xsl:text>)</xsl:text>
					</xsl:if>
				</span>
			</td>
			<td class="email" data-title="{$i18n.DocumentSigner.email}">
				<xsl:value-of select="$email" />
			</td>
			<td class="citizenid" data-title="{$i18n.DocumentSigner.citizenID}">
				<xsl:if test="not($userID) or $userID= ''">
					<xsl:value-of select="$citizenID" />
				</xsl:if>
			</td>
			<td class="mobilePhone" data-title="{$i18n.DocumentSigner.mobilePhone}">
				<xsl:if test="not($userID) or $userID= ''">
					<xsl:value-of select="$mobilePhone" />
				</xsl:if>
			</td>
			<td>
				<a class="marginright open-signer-modal" href="#" onclick="openDocumentSignerModal(this, event)" title="{$i18n.DocumentSigner.Edit}">
					<img class="vertical-align-middle" src="{$imgPath}/pen.png" alt="{$i18n.DocumentSigner.Edit}" />
				</a>
				<a href="#" onclick="deleteDocumentSigner(this, event)" title="{$i18n.DocumentSigner.Delete}">
					<img class="vertical-align-middle" src="{$imgPath}/delete.png" alt="{$i18n.DocumentSigner.Delete}" />
				</a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="documentStatus">
		<xsl:param name="status" select="status"/>
		
		<xsl:choose>
			<xsl:when test="$status = 'SAVED'">
				<xsl:value-of select="$i18n.Document.status.saved"/>
			</xsl:when>
			<xsl:when test="$status = 'SIGNING'">
				<xsl:value-of select="$i18n.Document.status.signing"/>
			</xsl:when>
			<xsl:when test="$status = 'CANCELLED'">
				<xsl:value-of select="$i18n.Document.status.cancelled"/>
			</xsl:when>
			<xsl:when test="$status = 'SIGNED'">
				<xsl:value-of select="$i18n.Document.status.signed"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="status"/>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='EmailOrPhoneRequired']">
	
		<p class="error">
			<xsl:value-of select="$i18n.Validation.EmailOrPhoneRequired"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ShowFailedDocumentNotFound' or messageKey='DeleteFailedDocumentNotFound' or messageKey='UpdateFailedDocumentNotFound']">
	
		<p class="error">
			<xsl:value-of select="$i18n.Validation.ShowFailedDocumentNotFound"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='MaxFileCountReached']">
	
		<p class="error">
			<xsl:value-of select="$i18n.MaxFileCountReached.part1"/>
			<xsl:value-of select="maxFileCount"/>
			<xsl:value-of select="$i18n.MaxFileCountReached.part2"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidFileExtension']">
	
		<p class="error">
			<xsl:value-of select="$i18n.InvalidFileExtension.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.InvalidFileExtension.part2"/>
		</p>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='FileSizeLimitExceeded']">
	
		<p class="error">
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part2"/>
			<xsl:value-of select="size"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part3"/>
			<xsl:value-of select="maxFileSize"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part4"/>
		</p>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='DeleteAfterXDaysTooLarge']">
	
		<p class="error">
			<xsl:value-of select="$i18n.DeleteAfterXDaysTooLarge"/>
		</p>
			
	</xsl:template>

	<xsl:template match="validationError[messageKey='FileNameLengthLimitExceeded']">
	
		<p class="error">
			<xsl:value-of select="$i18n.FileNameLengthLimitExceeded.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.FileNameLengthLimitExceeded.part2"/>
			<xsl:value-of select="length"/>
			<xsl:value-of select="$i18n.FileNameLengthLimitExceeded.part3"/>
			<xsl:value-of select="maxFileNameLength"/>
			<xsl:value-of select="$i18n.FileNameLengthLimitExceeded.part4"/>
		</p>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseFile']">
	
		<p class="error">
			<xsl:value-of select="$i18n.Validation.UnableToParseFile.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.Validation.UnableToParseFile.part2"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='NoCitizenIDOnInternalUser']">
	
		<p class="error">
			<xsl:variable name="id" select="substring(fieldName, 15)" />
			
			<xsl:value-of select="$i18n.DocumentSigner" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="$i18n.Validation.NoCitizenIDOnInternalUser.part1"/>
			<xsl:text> </xsl:text>
			<xsl:value-of select="../../requestparameters/parameter[name = concat('signer-firstname-', $id)]/value" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="../../requestparameters/parameter[name = concat('signer-lastname-', $id)]/value" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="$i18n.Validation.NoCitizenIDOnInternalUser.part2"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.Validation.RequiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.Validation.InvalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.Validation.TooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.Validation.TooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.Validation.UnknownError" />
					</xsl:otherwise>
				</xsl:choose>

				<xsl:text>&#x20;</xsl:text>

				<xsl:choose>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.Document.name" />
					</xsl:when>
					<xsl:when test="fieldName = 'cancelInXDays'">
						<xsl:value-of select="$i18n.Document.cancelInXDays" />
					</xsl:when>
					<xsl:when test="fieldName = 'sendReminderInXDays'">
						<xsl:value-of select="$i18n.Document.sendReminderInXDays" />
					</xsl:when>
					<xsl:when test="fieldName = 'deleteAfterXDays'">
						<xsl:value-of select="$i18n.Document.deleteAfterXDays" />
					</xsl:when>
					<xsl:when test="fieldName = 'signerID'">
						<xsl:value-of select="$i18n.Document.Signers" />
					</xsl:when>
					<xsl:when test="fieldName = 'file'">
						<xsl:value-of select="$i18n.Document.NewFile" />
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-firstname-')">
						<xsl:variable name="id" select="substring(fieldName, 18)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.firstname" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-lastname-')">
						<xsl:variable name="id" select="substring(fieldName, 16)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.lastname" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-email-')">
						<xsl:variable name="id" select="substring(fieldName, 14)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.email" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-mobilePhone-')">
						<xsl:variable name="id" select="substring(fieldName, 20)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.mobilePhone" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-citizenID-')">
						<xsl:variable name="id" select="substring(fieldName, 18)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.citizenID" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'signer-userID-')">
						<xsl:variable name="id" select="substring(fieldName, 15)" />
			
						<xsl:value-of select="$i18n.DocumentSigner" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="1 + ../../requestparameters/parameter[name = concat('signer-sortIndex-', $id)]/value" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.DocumentSigner.User" />
						
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>

		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.Validation.UnknownFault" />
			</p>
		</xsl:if>

	</xsl:template>

</xsl:stylesheet>