<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" >
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/flowengine.css
	</xsl:variable>
	
	<xsl:template match="Document">
		
		<xsl:choose>
			<xsl:when test="ListDocuments">
				
				<div id="DocumentSignerModule" class="contentitem">
					
					<xsl:apply-templates select="ListDocuments" />
					
				</div>
				
			</xsl:when>
			<xsl:otherwise>
			
				<xsl:apply-templates select="SignDocument"/>
				<xsl:apply-templates select="Signing"/>
				<xsl:apply-templates select="Message"/>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="ListDocuments">
		
		<section>
			<div class="errands-wrapper">
				
				<div class="heading-wrapper">
					<h1>
						<xsl:value-of select="Title" />
					</h1>
				</div>
				
				<xsl:if test="validationError">
					<div class="bigmarginbottom">
						<xsl:apply-templates select="validationError"/>
					</div>
				</xsl:if>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:value-of select="$i18n.ListDocuments.PendingDocuments" />
					</h2>
				</div>
				
				<table class="oep-table errand-table bigmarginbottom">
					<thead class="sortable">
						<tr>
							<th class="default-sort">
								<span><xsl:value-of select="$i18n.Document.name" /></span>
							</th>
							<th class="default-sort">
								<span><xsl:value-of select="$i18n.Document.poster" /></span>
							</th>
						</tr>
					</thead>
					<tbody>
						
						<xsl:choose>
							<xsl:when test="PendingDocuments/Document">
								<xsl:apply-templates select="PendingDocuments/Document" mode="list" />
							</xsl:when>
							<xsl:otherwise>
								<tr>
									<td colspan="2"><xsl:value-of select="$i18n.ListDocuments.NoPendingDocuments" /></td>
								</tr>
							</xsl:otherwise>
						</xsl:choose>
						
					</tbody>
				</table>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:value-of select="$i18n.ListDocuments.SignedDocuments" />
					</h2>
				</div>
				
				<table class="oep-table errand-table">
					<thead class="sortable">
						<tr>
							<th class="default-sort">
								<span><xsl:value-of select="$i18n.Document.name" /></span>
							</th>
							<th class="default-sort">
								<span><xsl:value-of select="$i18n.Document.poster" /></span>
							</th>
						</tr>
					</thead>
					<tbody>
						
						<xsl:choose>
							<xsl:when test="SignedDocuments/Document">
								<xsl:apply-templates select="SignedDocuments/Document" mode="list" />
							</xsl:when>
							<xsl:otherwise>
								<tr>
									<td colspan="4"><xsl:value-of select="$i18n.ListDocuments.NoSignedDocuments" /></td>
								</tr>
							</xsl:otherwise>
						</xsl:choose>
						
					</tbody>
				</table>
				
			</div>
		</section>
		
	</xsl:template>
	
	<xsl:template match="Document" mode="list">
		
		<tr>
			<td data-title="{$i18n.Document.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sign/{documentID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Document.poster}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sign/{documentID}">
					<xsl:value-of select="poster/firstname" />
					<xsl:text> </xsl:text>
					<xsl:value-of select="poster/lastname" />
				</a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="SignDocument">
	
		<div class="contentitem">
			<section>
				<div class="section-full">
			
					<h1 style="padding-left: 0;">
						<xsl:value-of select="$i18n.SignDocument"/>
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="Document/name"/>
					</h1>
					
					<xsl:value-of select="Message" disable-output-escaping="yes"/>
					
					<xsl:if test="Document/description">
						
						<hr/>
						
						<p>
							<xsl:call-template name="replaceLineBreak">
								<xsl:with-param name="string" select="Document/description"/>
							</xsl:call-template>
						</p>
					</xsl:if>
					
					<hr/>
					
					<xsl:value-of select="Disclaimer" disable-output-escaping="yes"/>
					
					<xsl:if test="SequentialSigningBlock">
						<p>
							<xsl:value-of select="$i18n.SequentialSigningBlock.message"/>
						</p>
					</xsl:if>
					
					<div class="divider" />
						
					<div class="heading-wrapper">
						
						<a class="btn btn-right btn-light xl" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/unsignedpdf/{Document/documentID}/{DocumentSigner/unsecureKey}">
							<xsl:value-of select="$i18n.DownloadUnsignedPDF"/>
						</a>
						
					</div>
					
					<div class="service clearboth">
						
						<div class="bigpaddingbottom">
							
						</div>
						
						<div class="bigmargintop">
							
							<xsl:if test="Document/showPreviousSignaturesToSigners = 'true' and Document/Signers/DocumentSigner/signedDate">
								
								<div class="bigmarginbottom">
									<p>
										<xsl:value-of select="$i18n.PreviousSigners" />
									</p>
									
									<table class="signingstatustable">
										<thead>
											<tr>
												<th class="paddingright">
													<xsl:value-of select="$i18n.DocumentSigner.name"/>
												</th>
												<th>
													<xsl:value-of select="$i18n.DocumentSigner.signedDate"/>
												</th>
											</tr>
										</thead>
										<tbody>
											<xsl:apply-templates select="Document/Signers/DocumentSigner[signedDate]" mode="previousSigners" />
										</tbody>
									</table>
								</div>
								
							</xsl:if>
						
							<xsl:choose>
								<xsl:when test="Document/completedDate and Document/status = 'SIGNED'">
								
									<a class="btn btn-green xl next arrow-mobile" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/signedpdf/{Document/documentID}/{DocumentSigner/unsecureKey}">
										<xsl:value-of select="$i18n.DownloadSignedPDF"/>
									</a>
									
								</xsl:when>
								<xsl:when test="SequentialSigningBlock">
								
									<p>
										<xsl:value-of select="$i18n.SequentialSigningBlock.message"/>
									</p>
									
								</xsl:when>
								<xsl:when test="not(DocumentSigner/signedDate)">
								
									<a class="btn btn-green xl next arrow-mobile" href="{/Document/requestinfo/uri}?sign=true">
										
										<span class="only-mobile">
											<xsl:value-of select="$i18n.SignDocument.Sign"/>
										</span>
										
										<span class="hide-mobile">
											<xsl:value-of select="$i18n.SignDocument.Sign"/>
										</span>
									</a>
									
									<a class="btn btn-red bigmargintop" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/cancel/{Document/documentID}/{DocumentSigner/unsecureKey}" onclick="return confirmPost(this.href, '{$i18n.SignDocument.CancelConfirm}')">
										
										<span class="only-mobile">
											<xsl:value-of select="$i18n.SignDocument.Cancel"/>
										</span>
										
										<span class="hide-mobile">
											<xsl:value-of select="$i18n.SignDocument.Cancel"/>
										</span>
										
									</a>
								
								</xsl:when>
							</xsl:choose>
							
						</div>
						
					</div>
					
				</div>
			</section>
		</div>
	
	</xsl:template>
	
	<xsl:template match="DocumentSigner" mode="previousSigners">
		
		<tr>
			<td class="paddingright" data-name="{$i18n.DocumentSigner.name}">
				<xsl:value-of select="firstname"/>
				<xsl:text> </xsl:text>
				<xsl:value-of select="lastname"/>
			</td>
			<td data-name="{$i18n.DocumentSigner.signedDate}">
				<xsl:value-of select="signedDate"/>
			</td>
		</tr>

	</xsl:template>
	
	<xsl:template match="Signing">
	
		<div class="contentitem">
	
			<h1 class="bigmarginbottom">
				<xsl:value-of select="$i18n.SignDocument"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="Document/name"/>
				<xsl:text>&#160;#</xsl:text>
				<xsl:value-of select="Document/documentID"/>
			</h1>

			<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>
			
			<div class="navigator-buttons">
			
				<a class="btn btn-light prev arrow-mobile" href="{SigningLink}">
					
					<span class="only-mobile">
						<xsl:value-of select="$i18n.SignDocument.Back"/>
					</span>
					
					<span class="hide-mobile">
						<xsl:value-of select="$i18n.SignDocument.Back"/>
					</span>
				</a>
			</div>
	
		</div>
		
	</xsl:template>
	
	<xsl:template match="Message[@messageKey='SignerNotFound']">
	
		<div class="contentitem">
		
			<h1><xsl:value-of select="$i18n.SignerNotFound.title"/></h1>
			
			<p><xsl:value-of select="$i18n.SignerNotFound.message"/></p>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="Message[@messageKey='SequentialSigningBlock']">
	
		<div class="contentitem">
		
			<h1><xsl:value-of select="$i18n.SequentialSigningBlock.title"/></h1>
			
			<p><xsl:value-of select="$i18n.SequentialSigningBlock.message"/></p>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="Message[@messageKey='WrongDocumentStatus']">
	
		<div class="contentitem">
		
			<h1><xsl:value-of select="$i18n.WrongDocumentStatus.title"/></h1>
			
			<p><xsl:value-of select="$i18n.WrongDocumentStatus.message"/></p>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="Message">
	
		<div class="contentitem">
		
			<h1>Oknt fel</h1>
			
			<p>Ett oknt fel har uppsttt</p>
		
		</div>
	
	</xsl:template>
	
</xsl:stylesheet>