package com.nordicpeak.flowengine.queries.dynamictablequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basetablequery.BaseTableQuery;

@Table(name = "dynamic_table_queries")
@XMLElement
public class DynamicTableQuery extends BaseTableQuery<DropDownAlternative, DynamicTableQuery, DynamicTableColumn> {

	private static final long serialVersionUID = 5116885863935428968L;

	public static final Field COLUMNS_RELATION = ReflectionUtils.getField(DynamicTableQuery.class, "columns");

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<DynamicTableColumn> columns;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer maxRows;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer decimalsToShow;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true)
	@XMLElement
	private List<DynamicTableQueryInstance> instances;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideTitle;

	@Override
	public List<DynamicTableColumn> getColumns() {

		return columns;
	}

	public List<DynamicTableQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<DynamicTableQueryInstance> instances) {

		this.instances = instances;
	}

	@Override
	public void setColumns(List<DynamicTableColumn> alternatives) {

		this.columns = alternatives;
	}

	@Override
	public boolean hasRequiredColumn() {

		if (!CollectionUtils.isEmpty(columns)) {

			for (DynamicTableColumn column : columns) {

				if (column.isRequired()) {

					return true;
				}
			}
		}

		return false;
	}

	@Override
	protected int getMinRowOccurance() {

		return 0;
	}

	@Override
	protected int getMaxRowOccurance() {

		if (maxRows != null) {

			return maxRows;
		}
		
		return 255;
	}

	public boolean isHideTitle() {

		return hideTitle;
	}

	public void setHideTitle(boolean hideTitle) {

		this.hideTitle = hideTitle;
	}

	@Override
	protected void populateColumns(XMLParser xmlParser, List<ValidationError> errors) throws ValidationException {

		maxRows = XMLValidationUtils.validateParameter("maxRows", xmlParser, false, 1, 255, IntegerPopulator.getPopulator(), errors);

		hideTitle = xmlParser.getPrimitiveBoolean("hideTitle");

		List<XMLParser> xmlParsers = xmlParser.getNodes("Columns/Column");

		if (xmlParsers != null) {

			columns = new ArrayList<>();

			for (XMLParser parser : xmlParsers) {

				DynamicTableColumn column = new DynamicTableColumn();

				column.populate(parser);

				column.setQuery(this);

				columns.add(column);
			}
		}
	}

	public Integer getMaxRows() {

		return maxRows;
	}

	public void setMaxRows(Integer maxRows) {

		this.maxRows = maxRows;
	}

	@Override
	public Integer getDecimalsToShow() {

		return decimalsToShow;
	}

}
