package com.nordicpeak.flowengine.integration.easit;

import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.ws.BindingProvider;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.bool.BooleanUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.jaxws.WSUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.easit.bpsservice.proxy.BPS;
import com.easit.bpsservice.proxy.BPSService;
import com.easit.bpsservice.proxy.ImportItemsRequest;
import com.easit.bpsservice.proxy.ImportItemsRequest.ItemToImport;
import com.easit.bpsservice.proxy.ImportItemsResponse;
import com.easit.bpsservice.proxy.ImportItemsResponse.ImportItemResult;
import com.easit.bpsservice.proxy.ImportItemsResponse.ImportItemResult.ReturnValues.ReturnValue;
import com.easit.bpsservice.proxy.Item.Attachment;
import com.easit.bpsservice.proxy.Item.Property;
import com.easit.bpsservice.proxy.PingResponse;
import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceAttribute;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.integration.easit.beans.IntegrationEndpoint;
import com.nordicpeak.flowengine.integration.easit.beans.IntegrationParameter;
import com.nordicpeak.flowengine.integration.easit.beans.IntegrationParameterGroup;
import com.nordicpeak.flowengine.integration.easit.beans.IntegrationSettings;
import com.nordicpeak.flowengine.integration.easit.utils.ParameterFormatter;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.utils.PDFByteAttachment;

public class EasitIntegrationProviderModule extends AnnotatedForegroundModule implements FlowAdminExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor> {

	private static final AnnotatedRequestPopulator<IntegrationSettings> SETTINGS_POPULATOR = new AnnotatedRequestPopulator<IntegrationSettings>(IntegrationSettings.class);

	private static final AnnotatedRequestPopulator<IntegrationParameterGroup> GROUP_POPULATOR = new AnnotatedRequestPopulator<IntegrationParameterGroup>(IntegrationParameterGroup.class);

	@XSLVariable(prefix = "java.")
	private String paramNameDisplayName;

	@XSLVariable(prefix = "java.")
	private String paramValueDisplayName;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Provider priority", description = "The priority of this provider", required = true)
	private Integer priority = 100;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Supported actionID's", description = "The action ID's which will trigger export XML to be generated and stored when a submit event is detected", required = true)
	private List<String> supportedActionIDs;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "Connection timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	private Integer connectionTimeout = 5;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Request timeout", description = "Request timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	private Integer requestTimeout = 10;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Debug logging", description = "Toggles debug logging when calling esit web service")
	protected boolean enableDebugLoggning;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Parameter formatters", description = "Parameter names to be formatted by specified formatter. Example someParameter:com.formatter.example")
	private List<String> parameterFormatters;

	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;

	@InstanceManagerDependency(required = true)
	private PDFProvider pdfProvider;

	private FlowAdminModule flowAdminModule;

	private AnnotatedDAO<IntegrationSettings> settingsDAO;
	private AnnotatedDAO<IntegrationParameterGroup> parameterGroupDAO;
	private QueryParameterFactory<IntegrationSettings, Integer> flowFamilyIDParamFactory;
	private QueryParameterFactory<IntegrationParameterGroup, Integer> parameterGroupIDParamFactory;

	private AnnotatedDAO<IntegrationEndpoint> endpointDAO;
	private QueryParameterFactory<IntegrationEndpoint, Integer> endpointIDParamFactory;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	private Map<String, ParameterFormatter> parameterFormattersMap;

	protected boolean configured;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, "com.nordicpeak.flowengine.integration.easit.EasitIntegrationProvider", new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		settingsDAO = daoFactory.getDAO(IntegrationSettings.class);
		parameterGroupDAO = daoFactory.getDAO(IntegrationParameterGroup.class);
		endpointDAO = daoFactory.getDAO(IntegrationEndpoint.class);

		flowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
		parameterGroupIDParamFactory = parameterGroupDAO.getParamFactory("parameterGroupID", Integer.class);
		endpointIDParamFactory = endpointDAO.getParamFactory("endpointID", Integer.class);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		this.configured = ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.WARN);

		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);

		if (parameterFormatters != null) {

			Map<String, ParameterFormatter> parameterFormattersMap = new HashMap<String, ParameterFormatter>(parameterFormatters.size());

			for (String parameterDefinition : parameterFormatters) {

				String[] splitDefinition = parameterDefinition.split(":");

				if (splitDefinition.length != 2 || StringUtils.isEmpty(splitDefinition[0]) || StringUtils.isEmpty(splitDefinition[1])) {

					log.warn("Unable to parse format of parameter definition: " + parameterDefinition + ", ignoring paramter.");
					continue;
				}

				try {

					ParameterFormatter parameterFormatter = (ParameterFormatter) ReflectionUtils.getInstance(splitDefinition[1].trim());

					parameterFormattersMap.put(splitDefinition[0].trim(), parameterFormatter);

				} catch (Throwable t) {

					log.error("Unable to create instance of parameter formatter specified for parameter: " + parameterDefinition + ", ignoring parameter.", t);
				}

			}

			this.parameterFormattersMap = parameterFormattersMap;

		}

	}

	@Override
	public void unload() throws Exception {

		if (flowAdminModule != null) {

			setFlowAdminModule(null);
		}

		super.unload();
	}

	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {

		if (!AccessUtils.checkAccess(user, this.moduleDescriptor)) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showViewElement = doc.createElement("ShowViewIntegrationSettings");
		doc.getDocumentElement().appendChild(showViewElement);

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		boolean enabled = false;

		if (integrationSettings != null) {

			XMLUtils.append(doc, showViewElement, integrationSettings);
			enabled = integrationSettings.isEnabled();
		}

		showViewElement.appendChild(flow.toXML(doc));

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
	}

	private IntegrationSettings getIntegrationSettings(Flow flow) throws SQLException {

		HighLevelQuery<IntegrationSettings> query = new HighLevelQuery<IntegrationSettings>();

		query.addParameter(flowFamilyIDParamFactory.getParameter(flow.getFlowFamily().getFlowFamilyID()));

		return settingsDAO.get(query);
	}

	private IntegrationParameterGroup getIntegrationParameterGroup(Integer parameterGroupID) throws SQLException {

		HighLevelQuery<IntegrationParameterGroup> query = new HighLevelQuery<IntegrationParameterGroup>(IntegrationParameterGroup.INTEGRATIONSETTINGS_RELATION, IntegrationParameterGroup.PARAMETERS_RELATION, IntegrationParameterGroup.ATTACHMENTPREFIXES_RELATION, IntegrationParameterGroup.ATTRIBUTEVALUES_RELATION);

		query.addParameter(parameterGroupIDParamFactory.getParameter(parameterGroupID));

		query.disableAutoRelations(true);

		return parameterGroupDAO.get(query);
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");

		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));

		doc.appendChild(document);

		return doc;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 49)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		if (!configured) {

			log.warn("Module " + moduleDescriptor + " is not properly configured, ignoring event " + event);

			return;
		}

		if (event.getEvent().getEventType() != EventType.SUBMITTED || event.getActionID() == null || !supportedActionIDs.contains(event.getActionID())) {

			return;
		}

		processEvent(event.getFlowInstanceManager().getFlowInstance(), event.getEvent(), false);

	}

	public void processEvent(ImmutableFlowInstance flowInstance, FlowInstanceEvent event, boolean generateXML) throws SQLException {

		IntegrationSettings integrationSettings = getIntegrationSettings((Flow) flowInstance.getFlow());

		if (integrationSettings == null || !integrationSettings.isEnabled()) {

			return;
		}

		if (!integrationSettings.isPopulated()) {

			log.warn("Integration settings for flow " + flowInstance.getFlow() + " enabled but not configured, skipping importing");

			return;
		}

		MutableAttributeHandler attributeHandler = (MutableAttributeHandler) flowInstance.getAttributeHandler();

		IntegrationEndpoint integrationEndpoint = integrationSettings.getIntegrationEndpoint();

		BPS proxy = new BPSService().getBPSSoap11();

		((BindingProvider) proxy).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, integrationEndpoint.getWebServiceAddress());
		((BindingProvider) proxy).getRequestContext().put(BindingProvider.USERNAME_PROPERTY, integrationEndpoint.getApiKey());
		((BindingProvider) proxy).getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, integrationEndpoint.getApiKey());

		WSUtils.setTimeouts((BindingProvider) proxy, connectionTimeout, requestTimeout);

		String previousCaseID = null;

		log.info("Importing parameter groups for flow instance " + flowInstance);

		File pdf = pdfProvider.getPDF(flowInstance.getFlowInstanceID(), event.getEventID());

		if (pdf == null) {

			log.error("No PDF found for event " + event + " belonging to flow instance " + flowInstance);
			return;
		}

		boolean attachmentsExists = false;

		for (IntegrationParameterGroup group : integrationSettings.getParameterGroups()) {

			if (group.getAttachmentPrefixes() != null) {

				attachmentsExists = true;
			}

		}

		List<PDFByteAttachment> pdfAttachments = null;

		if (attachmentsExists) {

			try {

				pdfAttachments = pdfProvider.getPDFAttachments(pdf, true);

			} catch (Exception e) {

				log.error("Error getting attachments from PDF for event " + event + " belonging to flow instance " + flowInstance, e);
			}
		}

		IntegrationParameterGroup previousGroup = null;

		boolean isFirst = false;
		String externalID = null;

		for (IntegrationParameterGroup group : integrationSettings.getParameterGroups()) {

			if (!group.isEnabled()) {

				log.info("Parameter group " + group + " is not enabled, skipping import");

				continue;
			}

			if (previousGroup == null) {
				isFirst = true;
			}

			if (previousGroup != null && group.isImportWithPrevious() && !previousGroup.isEnabled()) {

				log.info("Parameter group " + group + " is marked as import with previous but previous group is disabled, skipping import");

				continue;
			}

			if (group.getAttributeName() != null) {

				String attributeValue = attributeHandler.getString(group.getAttributeName());

				if (attributeValue == null) {

					logDebug("Parameter group " + group + " attribute values not matching configured attribute " + group.getAttributeName());

					continue;
				}

				boolean match = group.getAttributeValues().contains(attributeValue);

				if (!match && attributeValue.contains(",")) {

					String[] values = attributeValue.split(", ?");

					for (String splitValue : values) {

						match = group.getAttributeValues().contains(splitValue);

						if (match) {
							break;
						}
					}
				}

				if (match == group.isInverted()) {

					logDebug("Parameter group " + group + " attribute values not matching configured attribute " + group.getAttributeName());

					continue;
				}

			}

			logDebug("Importing parameter group " + group + "...");

			previousGroup = group;

			ImportItemsRequest request = new ImportItemsRequest();
			request.setImportHandlerIdentifier(group.getImportHandlerIdentifier());

			List<ItemToImport> items = new ArrayList<ItemToImport>();

			List<Attachment> attachments = null;

			if (group.getAttachmentPrefixes() != null) {

				attachments = getEasitAttachments(group, pdfAttachments);

			}

			if (group.isImportParametersSeparately()) {

				logDebug("Parameter group " + group + " is enabled to import parameters separately");

				List<Property> properties = group.getEasitProperties(attributeHandler, parameterFormattersMap);

				if (properties != null) {

					for (Property property : properties) {

						ItemToImport item = new ItemToImport();
						item.setUID(UUID.randomUUID().toString());
						item.getProperty().add(property);

						checkImportWithPrevious(group, previousCaseID, item);

						items.add(item);

					}

				}

				if (!CollectionUtils.isEmpty(attachments)) {

					ItemToImport item = new ItemToImport();
					item.setUID(UUID.randomUUID().toString());

					item.getAttachment().addAll(attachments);

					items.add(item);
				}

			} else {

				ItemToImport item = new ItemToImport();
				item.setUID(UUID.randomUUID().toString());

				List<Property> properties = group.getEasitProperties(attributeHandler, parameterFormattersMap);

				if (properties != null) {
					item.getProperty().addAll(properties);
				}

				if(flowInstance.getPoster() != null) {
					item.getProperty().add(getIntegrationProperty("contact", flowInstance.getPoster().getUsername()));
				}

				item.getProperty().add(getIntegrationProperty("openeid", flowInstance.getFlowInstanceID().toString()));

				if (!CollectionUtils.isEmpty(attachments)) {

					item.getAttachment().addAll(attachments);
				}

				checkImportWithPrevious(group, previousCaseID, item);

				items.add(item);

			}

			request.getItemToImport().addAll(items);

			try {

				ImportItemsResponse response = proxy.importItems(request);

				List<ImportItemResult> results = response.getImportItemResult();

				if (results != null) {

					boolean success = true;
					String caseID = null;
					String objectID = null;

					for (ImportItemResult result : results) {

						if (result.getResult() != null && !result.getResult().equalsIgnoreCase("OK")) {

							log.warn("Unexpected result returned from Easit. Got result " + result.getResult() + " with message " + result.getSkippedOrExceptionMessage());

							success = false;
							break;

						} else {

							if (result.getReturnValues() != null) {

								for (ReturnValue value : result.getReturnValues().getReturnValue()) {

									if (value.getName().equalsIgnoreCase("id")) {

										caseID = value.getContent();
										previousCaseID = caseID;

										if (isFirst) {

											externalID = caseID;
											isFirst = false;
										}

									} else if (value.getName().equalsIgnoreCase("item")) {

										objectID = value.getContent();
									}

								}

							}

						}

					}

					if (success) {

						log.info("Successfully imported parameter group " + group + " registered with objectID " + objectID + (caseID != null ? " and caseID " + caseID : ""));

					} else {

						log.warn("Unable to import parameter group " + group + ", got none ok result in response");
					}

				} else {

					log.warn("Unable to import parameter group " + group + ", got no results in response");

				}

			} catch (Exception e) {

				log.warn("Error sending parameter group " + group + " to Easit for flow instance " + flowInstance + " using endpoint " + integrationEndpoint.getWebServiceAddress(), e);
			}

		}

		if (!StringUtils.isEmpty(externalID)) {

			log.info("Got external ID: " + externalID + " from Easit for flow instance " + flowInstance);

			FlowInstanceAttribute attribute = new FlowInstanceAttribute(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE, externalID);
			attribute.setFlowInstance((FlowInstance) flowInstance);

			flowAdminModule.getDAOFactory().getFlowInstanceAttributeDAO().add(attribute);

			attributeHandler.setAttribute(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE, externalID);

		} else {

			log.warn("Got empty external ID for event " + event + " belonging to flow instance " + flowInstance);
		}

	}

	protected void logDebug(String text) {

		if (enableDebugLoggning) {

			log.info(text);
		}

	}

	protected void checkImportWithPrevious(IntegrationParameterGroup group, String previousCaseID, ItemToImport item) {

		if (group.isImportWithPrevious() && previousCaseID != null) {

			Property property = new Property();
			property.setName("id");
			property.setContent(previousCaseID);

			logDebug("Parameter group " + group + " importing with previous caseID " + previousCaseID);

			item.getProperty().add(property);

		}

	}

	protected Property getIntegrationProperty(String name, String value) {

		Property property = new Property();
		property.setName(name);
		property.setContent(value);

		return property;
	}

	protected List<Attachment> getEasitAttachments(IntegrationParameterGroup group, List<PDFByteAttachment> pdfAttachments) {

		if (group.getAttachmentPrefixes() != null && pdfAttachments != null) {

			List<Attachment> attachments = new ArrayList<Attachment>();

			for (PDFByteAttachment pdfAttachment : pdfAttachments) {

				String name = pdfAttachment.getAttachmentName();

				for (String prefix : group.getAttachmentPrefixes()) {

					if (name.startsWith(prefix)) {

						Attachment attachment = new Attachment();
						attachment.setName(name);

						attachment.setValue(pdfAttachment.getData());

						attachments.add(attachment);

					}

				}

			}

			return attachments;
		}

		return null;
	}

	@WebPublic(alias = "settings")
	public ForegroundModuleResponse showFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		if (integrationSettings == null) {

			this.redirectToMethod(req, res, "/update/" + flow.getFlowID());
		}

		log.info("User " + user + " requesting show integration settings for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element integrationSettingsElement = doc.createElement("ShowIntegrationSettings");

		doc.getDocumentElement().appendChild(integrationSettingsElement);

		integrationSettingsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, integrationSettingsElement, integrationSettings);
		XMLUtils.append(doc, integrationSettingsElement, "Endpoints", endpointDAO.getAll());

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	@WebPublic(alias = "update")
	public ForegroundModuleResponse updateFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		ValidationException validationException = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			if (integrationSettings == null) {

				integrationSettings = new IntegrationSettings();
				integrationSettings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
			}

			try {

				SETTINGS_POPULATOR.populate(integrationSettings, req);

				List<ValidationError> validationErrors = new ArrayList<>();

				IntegrationEndpoint endpoint = getEndpoint(req);

				if (endpoint == null) {

					validationErrors.add(new ValidationError("NoEndpointSelected"));

				} else {

					integrationSettings.setIntegrationEndpoint(endpoint);
				}

				log.info("User " + user + " updating base integration settings for flow " + flow);

				if (integrationSettings.isPopulated()) {

					settingsDAO.addOrUpdate(integrationSettings, null);

					redirectToMethod(req, res, "/settings/" + flow.getFlowID());

				} else {

					settingsDAO.delete(integrationSettings);

					flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#easitintegrationsettings");
				}

				return null;

			} catch (ValidationException e) {

				validationException = e;
			}
		}

		log.info("User " + user + " requesting update base integration settings form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element integrationSettingsElement = doc.createElement("UpdateIntegrationSettings");
		doc.getDocumentElement().appendChild(integrationSettingsElement);

		integrationSettingsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, integrationSettingsElement, integrationSettings);
		XMLUtils.append(doc, integrationSettingsElement, "Endpoints", endpointDAO.getAll());

		if (validationException != null) {

			integrationSettingsElement.appendChild(validationException.toXML(doc));
			integrationSettingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

	}

	private IntegrationEndpoint getEndpoint(HttpServletRequest req) throws SQLException {

		Integer endpointID = NumberUtils.toInt(req.getParameter("endpointID"));

		if (endpointID != null) {

			HighLevelQuery<IntegrationEndpoint> query = new HighLevelQuery<>();
			query.addParameter(endpointIDParamFactory.getParameter(endpointID));

			return endpointDAO.get(query);
		}

		return null;
	}

	@WebPublic(alias = "delete")
	public ForegroundModuleResponse deleteFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		if(!HTTPUtils.isPost(req)) {
			
			throw new AccessDeniedException("Delete requests using method " + req.getMethod() + " are not allowed.");
		}
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		if (integrationSettings != null) {

			log.info("User " + user + " deleting integration settings for flow " + flow);

			settingsDAO.delete(integrationSettings);

		} else {

			log.warn("User " + user + " trying to deleting integration settings for flow " + flow + " which has no integration settings");
		}

		flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#easitintegrationsettings");

		return null;
	}

	@WebPublic(alias = "addparametergroup")
	public ForegroundModuleResponse addParameterGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		if (integrationSettings == null) {

			throw new URINotFoundException(uriParser);
		}

		ValidationException validationException = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			try {

				IntegrationParameterGroup parameterGroup = new IntegrationParameterGroup();

				parameterGroup.setIntegrationSettings(integrationSettings);

				populateIntegrationParameterGroup(parameterGroup, req, res, user, uriParser);

				parameterGroupDAO.add(parameterGroup);

				redirectToMethod(req, res, "/settings/" + flow.getFlowID());

			} catch (ValidationException e) {

				validationException = e;
			}
		}

		log.info("User " + user + " requesting add integration parameter group form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element integrationGroupsElement = doc.createElement("AddIntegrationParameterGroup");
		doc.getDocumentElement().appendChild(integrationGroupsElement);

		integrationGroupsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, integrationGroupsElement, integrationSettings);

		if (validationException != null) {

			integrationGroupsElement.appendChild(validationException.toXML(doc));
			integrationGroupsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

	}

	@WebPublic(alias = "updateparametergroup")
	public ForegroundModuleResponse updateParameterGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		Integer parameterGroupID = uriParser.getInt(3);

		IntegrationParameterGroup parameterGroup = null;

		if (parameterGroupID != null && (parameterGroup = getIntegrationParameterGroup(parameterGroupID)) != null) {

			ValidationException validationException = null;

			if (req.getMethod().equalsIgnoreCase("POST")) {

				try {

					populateIntegrationParameterGroup(parameterGroup, req, res, user, uriParser);

					parameterGroupDAO.update(parameterGroup);

					redirectToMethod(req, res, "/settings/" + flow.getFlowID());

				} catch (ValidationException e) {

					validationException = e;
				}
			}

			log.info("User " + user + " requesting update integration parameter group form for parameter group " + parameterGroup);

			Document doc = createDocument(req, uriParser, user);

			Element integrationGroupsElement = doc.createElement("UpdateIntegrationParameterGroup");
			doc.getDocumentElement().appendChild(integrationGroupsElement);

			integrationGroupsElement.appendChild(flow.toXML(doc));

			XMLUtils.append(doc, integrationGroupsElement, parameterGroup);

			if (validationException != null) {

				integrationGroupsElement.appendChild(validationException.toXML(doc));
				integrationGroupsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			}

			return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "deleteparametergroup")
	public ForegroundModuleResponse deleteParameterGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		if(!HTTPUtils.isPost(req)) {
			
			throw new AccessDeniedException("Delete requests using method " + req.getMethod() + " are not allowed.");
		}
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		Integer parameterGroupID = uriParser.getInt(3);

		IntegrationParameterGroup parameterGroup = null;

		if (parameterGroupID != null && (parameterGroup = getIntegrationParameterGroup(parameterGroupID)) != null) {

			parameterGroupDAO.delete(parameterGroup);

			redirectToMethod(req, res, "/settings/" + flow.getFlowID());

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "sortparametergroups")
	public ForegroundModuleResponse sortParameterGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		IntegrationSettings integrationSettings = getIntegrationSettings(flow);

		if (integrationSettings == null) {

			throw new URINotFoundException(uriParser);
		}

		if (req.getMethod().equalsIgnoreCase("POST")) {

			List<IntegrationParameterGroup> parameterGroups = integrationSettings.getParameterGroups();

			if (!CollectionUtils.isEmpty(parameterGroups)) {

				for (IntegrationParameterGroup parameterGroup : parameterGroups) {

					String sortIndex = req.getParameter("sortorder_" + parameterGroup.getParameterGroupID());

					if (NumberUtils.isInt(sortIndex)) {

						parameterGroup.setSortIndex(NumberUtils.toInt(sortIndex));

					}

				}

				settingsDAO.update(integrationSettings, new RelationQuery(IntegrationSettings.GROUPS_RELATION));

				redirectToMethod(req, res, "/settings/" + flow.getFlowID());

				return null;

			}

		}

		log.info("User " + user + " requesting sort integration parameter group form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element integrationGroupsElement = doc.createElement("SortIntegrationParameterGroup");
		doc.getDocumentElement().appendChild(integrationGroupsElement);

		integrationGroupsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, integrationGroupsElement, integrationSettings);

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

	}

	@WebPublic(alias = "testping")
	public ForegroundModuleResponse testPing(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		List<IntegrationEndpoint> endpoints = endpointDAO.getAll();

		if (endpoints == null) {

			return new SimpleForegroundModuleResponse("No endpoints found", moduleDescriptor.getName(), this.getDefaultBreadcrumb());
		}

		StringBuilder responseBuilder = new StringBuilder();

		for (IntegrationEndpoint endpoint : endpoints) {

			try {

				BPS proxy = new BPSService().getBPSSoap11();

				((BindingProvider) proxy).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpoint.getWebServiceAddress());
				((BindingProvider) proxy).getRequestContext().put(BindingProvider.USERNAME_PROPERTY, endpoint.getApiKey());
				((BindingProvider) proxy).getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, endpoint.getApiKey());

				WSUtils.setTimeouts((BindingProvider) proxy, connectionTimeout, requestTimeout);

				log.info("Pinging web service endpoint " + endpoint);

				PingResponse pingResponse = proxy.ping("?");

				String response = null;

				if (pingResponse != null) {

					response = pingResponse.getMessage();

				} else {

					response = "No response";

				}

				responseBuilder.append("<p>Ping response for endoint " + endpoint + ": " + response + "</p>");

			} catch (Exception e) {

				responseBuilder.append("<p>Ping response for endoint " + endpoint + ": " + e.getMessage() + "</p>");
			}

		}

		return new SimpleForegroundModuleResponse(responseBuilder.toString(), moduleDescriptor.getName(), this.getDefaultBreadcrumb());

	}

	private IntegrationParameterGroup populateIntegrationParameterGroup(IntegrationParameterGroup parameterGroup, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ValidationException, SQLException {

		GROUP_POPULATOR.populate(parameterGroup, req);

		if (parameterGroup.getSortIndex() == null) {

			parameterGroup.setSortIndex(this.getCurrentMaxSortIndex(parameterGroup) + 1);
		}

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		String[] parameterIDs = req.getParameterValues("parameterID");

		if (parameterIDs != null) {

			List<IntegrationParameter> parameters = new ArrayList<>(parameterIDs.length);

			for (String parameterID : parameterIDs) {

				String name = ValidationUtils.validateParameter("name-" + parameterID, paramNameDisplayName, req, true, 1, 255, StringPopulator.getPopulator(), validationErrors);
				String value = ValidationUtils.validateParameter("value-" + parameterID, paramValueDisplayName, req, true, 1, 65535, StringPopulator.getPopulator(), validationErrors);

				if (validationErrors.isEmpty()) {

					IntegrationParameter parameter = new IntegrationParameter();
					parameter.setName(name);
					parameter.setValue(value);
					parameter.setSplitValue(BooleanUtils.toBoolean(req.getParameter("split-" + parameterID)));
					parameters.add(parameter);
				}
			}

			parameterGroup.setParameters(parameters);

		} else {

			parameterGroup.setParameters(null);
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		return parameterGroup;

	}

	private int getCurrentMaxSortIndex(IntegrationParameterGroup parameterGroup) throws SQLException {

		ObjectQuery<Integer> query = new ObjectQuery<Integer>(parameterGroupDAO.getDataSource(), "SELECT MAX(sortIndex) FROM " + parameterGroupDAO.getTableName() + " WHERE flowFamilyID = ?", IntegerPopulator.getPopulator());

		query.setInt(1, parameterGroup.getIntegrationSettings().getFlowFamilyID());

		Integer sortIndex = query.executeQuery();

		if (sortIndex == null) {

			return 0;
		}

		return sortIndex;
	}

	@Override
	public String getExtensionViewTitle() {

		return moduleDescriptor.getName();
	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (flowAdminModule == null && this.flowAdminModule != null) {

			this.flowAdminModule.removeExtensionViewProvider(this);
		}

		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}

	@EventListener(channel = FlowFamily.class)
	public void processEvent(CRUDEvent<FlowFamily> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.DELETE) {

			for (FlowFamily flowFamily : event.getBeans()) {

				log.info("Deleting integration settings for flow family ID: " + flowFamily.getFlowFamilyID());

				try {
					HighLevelQuery<IntegrationSettings> query = new HighLevelQuery<IntegrationSettings>();

					query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));

					settingsDAO.delete(query);

				} catch (SQLException e) {

					log.error("Error deleting integration settings for flow family ID: " + flowFamily.getFlowFamilyID(), e);
				}
			}
		}
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

}
