package com.nordicpeak.flowengine.integration.easit.beans;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CaseInsensitiveStringComparator;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.easit.bpsservice.proxy.Item.Property;
import com.nordicpeak.flowengine.integration.easit.utils.ParameterFormatter;

@Table(name = "easit_integration_parametergroups")
@XMLElement
public class IntegrationParameterGroup extends GeneratedElementable {

	public static final Field INTEGRATIONSETTINGS_RELATION = ReflectionUtils.getField(IntegrationParameterGroup.class, "integrationSettings");
	public static final Field PARAMETERS_RELATION = ReflectionUtils.getField(IntegrationParameterGroup.class, "parameters");
	public static final Field ATTACHMENTPREFIXES_RELATION = ReflectionUtils.getField(IntegrationParameterGroup.class, "attachmentPrefixes");
	public static final Field ATTRIBUTEVALUES_RELATION = ReflectionUtils.getField(IntegrationParameterGroup.class, "attributeValues");

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement
	private Integer parameterGroupID;

	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String importHandlerIdentifier;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean enabled;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean importWithPrevious;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean importParametersSeparately;

	@DAOManaged
	@RequiredIfSet(paramNames = { "attributeValues" })
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String attributeName;

	@DAOManaged
	@RequiredIfSet(paramNames = { "attributeName" })
	@WebPopulate(maxLength = 255)
	@OneToMany(autoGet = true, autoAdd = true, autoUpdate = true)
	@SimplifiedRelation(table = "easit_integration_parametergroup_attributevalues", remoteValueColumnName = "value")
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@SplitOnLineBreak
	@XMLElement(fixCase = true)
	private List<String> attributeValues;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean invert;

	@OrderBy
	@DAOManaged
	@XMLElement
	private Integer sortIndex;

	@DAOManaged(columnName = "flowFamilyID")
	@ManyToOne
	@XMLElement
	private IntegrationSettings integrationSettings;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<IntegrationParameter> parameters;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@OneToMany(autoGet = true, autoAdd = true, autoUpdate = true)
	@SimplifiedRelation(table = "easit_integration_parametergroup_attachmentprefixes", remoteValueColumnName = "prefix")
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@SplitOnLineBreak
	@XMLElement(fixCase = true)
	private List<String> attachmentPrefixes;

	public Integer getParameterGroupID() {

		return parameterGroupID;
	}

	public void setParameterGroupID(Integer parameterGroupID) {

		this.parameterGroupID = parameterGroupID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public boolean isEnabled() {

		return enabled;
	}

	public void setEnabled(boolean enabled) {

		this.enabled = enabled;
	}

	public boolean isImportWithPrevious() {

		return importWithPrevious;
	}

	public void setImportWithPrevious(boolean importWithPrevious) {

		this.importWithPrevious = importWithPrevious;
	}

	public boolean isImportParametersSeparately() {

		return importParametersSeparately;
	}

	public void setImportParametersSeparately(boolean importParametersSeparately) {

		this.importParametersSeparately = importParametersSeparately;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public List<String> getAttributeValues() {

		return attributeValues;
	}

	public void setAttributeValues(List<String> values) {

		this.attributeValues = values;
	}

	public boolean isInverted() {

		return invert;
	}

	public void setInvert(boolean invert) {

		this.invert = invert;
	}

	public String getImportHandlerIdentifier() {

		return importHandlerIdentifier;
	}

	public void setImportHandlerIdentifier(String importHandlerIdentifier) {

		this.importHandlerIdentifier = importHandlerIdentifier;
	}

	public List<IntegrationParameter> getParameters() {

		return parameters;
	}

	public void setParameters(List<IntegrationParameter> parameters) {

		this.parameters = parameters;
	}

	public List<String> getAttachmentPrefixes() {

		return attachmentPrefixes;
	}

	public void setAttachmentPrefixes(List<String> attachmentPrefixes) {

		this.attachmentPrefixes = attachmentPrefixes;
	}

	public Integer getSortIndex() {

		return sortIndex;
	}

	public void setSortIndex(Integer sortIndex) {

		this.sortIndex = sortIndex;
	}

	public IntegrationSettings getIntegrationSettings() {

		return integrationSettings;
	}

	public void setIntegrationSettings(IntegrationSettings integrationSettings) {

		this.integrationSettings = integrationSettings;
	}

	public List<Property> getEasitProperties() {

		return getEasitProperties(null, null);
	}

	public List<Property> getEasitProperties(AttributeHandler attributeHandler, Map<String, ParameterFormatter> parameterFormatters) {

		if (parameters != null) {

			List<Property> properties = new ArrayList<Property>();

			for (IntegrationParameter parameter : parameters) {

				properties.addAll(parameter.getEasitProperties(attributeHandler, parameterFormatters));
			}

			return properties;
		}

		return null;
	}

	@Override
	public String toString() {

		return name;
	}
}
