<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/efecteintegration.js
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">
	
		<xsl:apply-templates select="ShowIntegrationSettings" />
		<xsl:apply-templates select="UpdateIntegrationSettings" />
			
	</xsl:template>
	
	<xsl:template match="ShowIntegrationSettings">
	
		<a name="efecteintegrationsettings" />
		
		<xsl:choose>
			<xsl:when test="IntegrationSettings">
					
				<xsl:value-of select="$i18n.IntegrationEnabled"/>
				<xsl:text>:&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="IntegrationSettings/enabled = 'true'">
						<xsl:value-of select="$i18n.Yes"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No"/>
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:if test="IntegrationSettings/url">
					<br/>
								
					<xsl:value-of select="$i18n.Url"/>
					<xsl:text>:&#x20;</xsl:text>
					<xsl:value-of select="IntegrationSettings/url"/>
				</xsl:if>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoIntegrationSettings" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/update/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateIntegrationSettings" />
			<img class="alignbottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="IntegrationSettings">
			<a href="{/Document/ModuleURI}/delete/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteIntegrationSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteIntegrationSettings" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdateIntegrationSettings">
		
		<div id="EfecteIntegrationProvider" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdateIntegrationSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError"/>
	
			<form method="post" action="{/Document/ModuleURI}/update/{Flow/flowID}">
			
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'enabled'" />
						<xsl:with-param name="id" select="'enabled'" />
						<xsl:with-param name="element" select="IntegrationSettings" />       
					</xsl:call-template>
					
					<label for="enabled">
						<xsl:value-of select="$i18n.Enabled" />
					</label>
				</div>
			
				<label for="endpointID" class="floatleft full">
					<xsl:value-of select="$i18n.Endpoint" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'endpointID'"/>
						<xsl:with-param name="name" select="'endpointID'"/>
						<xsl:with-param name="element" select="Endpoints/Endpoint" /> 
						<xsl:with-param name="labelElementName" select="'name'" />        
						<xsl:with-param name="valueElementName" select="'endpointID'" />
						<xsl:with-param name="selectedValue" select="IntegrationSettings/Endpoint/endpointID"/>
					</xsl:call-template>
				</div>
				
				<label for="username" class="floatleft full">
					<xsl:value-of select="$i18n.Username" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'username'"/>
						<xsl:with-param name="name" select="'username'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>
				
				<label for="password" class="floatleft full">
					<xsl:value-of select="$i18n.Password" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'password'"/>
						<xsl:with-param name="name" select="'password'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>	
			
				<label for="folderID" class="floatleft full">
					<xsl:value-of select="$i18n.folderID" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'folderID'"/>
						<xsl:with-param name="name" select="'folderID'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>		
			
				<label for="templateCode" class="floatleft full">
					<xsl:value-of select="$i18n.templateCode" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'templateCode'"/>
						<xsl:with-param name="name" select="'templateCode'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>
				
				<label for="groupCode" class="floatleft full">
					<xsl:value-of select="$i18n.groupCode" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'groupCode'"/>
						<xsl:with-param name="name" select="'groupCode'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>		
			
				<label for="subject" class="floatleft full">
					<xsl:value-of select="$i18n.subject" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'subject'"/>
						<xsl:with-param name="name" select="'subject'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>	
			
				<label for="description" class="floatleft full">
					<xsl:value-of select="$i18n.description" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'description'"/>
						<xsl:with-param name="name" select="'description'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>
				
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.Attachments" /></h2>
				</div>
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'none'" />
							<xsl:with-param name="name" select="'attachmentMode'" />
							<xsl:with-param name="value" select="''" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="none">
							<xsl:value-of select="$i18n.attachmentMode.none" />
						</label>
					</div>
				</div>
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'PDF'" />
							<xsl:with-param name="name" select="'attachmentMode'" />
							<xsl:with-param name="value" select="'PDF'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="PDF">
							<xsl:value-of select="$i18n.attachmentMode.PDF" />
						</label>
					</div>
				</div>
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'PDF_AND_ATTACHMENTS'" />
							<xsl:with-param name="name" select="'attachmentMode'" />
							<xsl:with-param name="value" select="'PDF_AND_ATTACHMENTS'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="PDF_AND_ATTACHMENTS">
							<xsl:value-of select="$i18n.attachmentMode.PDF_AND_ATTACHMENTS" />
						</label>
					</div>
				</div>
				
				<label for="attachmentAttributeID" class="floatleft full">
					<xsl:value-of select="$i18n.attachmentAttributeID" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'attachmentAttributeID'"/>
						<xsl:with-param name="name" select="'attachmentAttributeID'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>			
			
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.ExternalMessages" /></h2>
				</div>
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'externalMessageSupportEnabled.disabled'" />
							<xsl:with-param name="name" select="'externalMessageSupportEnabled'" />
							<xsl:with-param name="value" select="'false'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="externalMessageSupportEnabled.disabled">
							<xsl:value-of select="$i18n.externalMessageSupportEnabled.disabled" />
						</label>
					</div>
				</div>				
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'externalMessageSupportEnabled.enabled'" />
							<xsl:with-param name="name" select="'externalMessageSupportEnabled'" />
							<xsl:with-param name="value" select="'true'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="externalMessageSupportEnabled.enabled">
							<xsl:value-of select="$i18n.externalMessageSupportEnabled.enabled" />
						</label>
					</div>
				</div>			
			
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.LocalStatusChanges" /></h2>
				</div>
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'managerStatusChangedSupportEnabled.disabled'" />
							<xsl:with-param name="name" select="'managerStatusChangedSupportEnabled'" />
							<xsl:with-param name="value" select="'false'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="managerStatusChangedSupportEnabled.disabled">
							<xsl:value-of select="$i18n.managerStatusChangedSupportEnabled.disabled" />
						</label>
					</div>
				</div>				
				
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'managerStatusChangedSupportEnabled.enabled'" />
							<xsl:with-param name="name" select="'managerStatusChangedSupportEnabled'" />
							<xsl:with-param name="value" select="'true'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="managerStatusChangedSupportEnabled.enabled">
							<xsl:value-of select="$i18n.managerStatusChangedSupportEnabled.enabled" />
						</label>
					</div>
				</div>			
			
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.SolutionSupport" /></h2>
				</div>			
			
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'solutionSupportEnabled'" />
						<xsl:with-param name="id" select="'solutionSupportEnabled'" />
						<xsl:with-param name="element" select="IntegrationSettings" />       
					</xsl:call-template>
					
					<label for="solutionSupportEnabled">
						<xsl:value-of select="$i18n.SolutionSupportEnabled" />
					</label>
				</div>			
			
				<label for="solutionStatusName" class="floatleft full">
					<xsl:value-of select="$i18n.SolutionStatusName" />
				</label>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'solutionStatusName'"/>
						<xsl:with-param name="name" select="'solutionStatusName'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>	
			
				<label for="solutionMessage" class="floatleft full">
					<xsl:value-of select="$i18n.SolutionMessage" />
				</label>
				
				<p><xsl:value-of select="$i18n.SolutionMessageDescription" /></p>				
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'solutionMessage'"/>
						<xsl:with-param name="name" select="'solutionMessage'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>			
			
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.Parameters" /></h2>
				</div>
			
				<div class="floatleft full marginbottom">
	
					<div class="floatleft full">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'skipEmptyParameters'" />
							<xsl:with-param name="name" select="'skipEmptyParameters'" />
							<xsl:with-param name="element" select="IntegrationSettings" />
						</xsl:call-template>
	
						<label for="skipEmptyParameters">
							<xsl:value-of select="$i18n.skipEmptyParameters" />
						</label>
					</div>
				</div>			
			
				<table id="param-table" class="full coloredtable ">
					<thead>
						<tr>
							<th><xsl:value-of select="$i18n.Name" /></th>
							<th><xsl:value-of select="$i18n.Value" /></th>
							<th><img class="alignbottom" src="{/Document/StaticContentURL}/pics/split.png" alt="{$i18n.SplitValues}" title="{$i18n.SplitValues}"/></th>
							<th width="16px"></th>
						</tr>
					</thead>
					<tbody>
					
						<xsl:choose>
							<xsl:when test="requestparameters">
							
								<xsl:apply-templates select="requestparameters/parameter[name = 'parameterID']/value" mode="parameter"/>
							
							</xsl:when>
							<xsl:otherwise>
							
								<xsl:apply-templates select="IntegrationSettings/Parameters/IntegrationParameter"/>
							
							</xsl:otherwise>
						</xsl:choose>					
					
					</tbody>
				</table>
				
				<div class="floatright">
					<a onclick="addParameter();">
						<xsl:value-of select="$i18n.AddParameter" />
						<img class="alignbottom" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
					</a>
				</div>			
								
				<div class="clearboth marginbottom">
					<br/>
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
		
					<h2><xsl:value-of select="$i18n.Notifications"/></h2>
				
					<xsl:call-template name="notificationEmail">
						<xsl:with-param name="toggleField" select="'sendErrorEmail'" />
						<xsl:with-param name="addressesField" select="'errorEmailAddresses'" />
						<xsl:with-param name="toggleLabel" select="$i18n.Notifications.ErrorEmail" />
						<xsl:with-param name="addressesLabel" select="$i18n.Notifications.ErrorEmail.EmailAddresses" />
						<xsl:with-param name="addresses" select="IntegrationSettings/ErrorEmailAddresses/Address" />
					</xsl:call-template>
					
				</div>
			
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
		
			<div style="display: none;">
				
				<table>
					<tr id="parameter-template">
						<td>
							<input type="text" name="name" class="full"/>
						</td>
						<td>
							<input type="text" name="value" class="full"/>
						</td>
						<td>
							<input type="checkbox" name="split"/>
						</td>						
						<td>
							<input type="hidden" name="parameterID"/>
							
							<a onclick="deleteParameter(this)">
								<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
							</a>				
						</td>
					</tr>
				</table>
				
			</div>		
		
		</div>

	</xsl:template>
	
	<xsl:template match="IntegrationParameter">
		
		<tr id="parameter-{parameterID}">
			<td>
				<input type="text" name="name-{parameterID}" value="{name}" class="full"/>
			</td>
			<td>
				<input type="text" name="value-{parameterID}" value="{value}" class="full"/>
			</td>
			<td>
				<input type="checkbox" name="split-{parameterID}" value="true">
					
					<xsl:if test="splitValue = 'true'">
						<xsl:attribute name="checked">true</xsl:attribute>
					</xsl:if>
					
				</input>
			</td>			
			<td>
				<input type="hidden" name="parameterID" value="{parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="value" mode="parameter">
		
 		<xsl:variable name="parameterID" select="."/>
		
		<xsl:variable name="nameParam" select="concat('name-', $parameterID)"/>
		
		<xsl:variable name="valueParam" select="concat('value-', $parameterID)"/>
		
		<xsl:variable name="splitParam" select="concat('split-', $parameterID)"/>
		
		<tr id="parameter-{$parameterID}">
 			<td>
				<input type="text" name="name-{$parameterID}" value="{../../parameter[name = $nameParam]/value}" class="full"/>
			</td>
			<td>
				<input type="text" name="value-{$parameterID}" value="{../../parameter[name = $valueParam]/value}" class="full"/>
			</td>
			<td>
				<input type="checkbox" name="split-{$parameterID}" value="true">
				
					<xsl:if test="../../parameter[name = $splitParam]/value = 'true'">
						<xsl:attribute name="checked">true</xsl:attribute>
					</xsl:if>
				
				</input>
			</td>				
			<td>
				<input type="hidden" name="parameterID" value="{$parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>	
	
	<xsl:template name="notificationEmail">
		<xsl:param name="toggleField" />
		<xsl:param name="addressesField" />
		<xsl:param name="toggleLabel" />
		<xsl:param name="addressesLabel" />
		<xsl:param name="addresses" />
	
		<div class="notification-email">
		
			<div>
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="$toggleField" />
					<xsl:with-param name="id" select="$toggleField" />
					<xsl:with-param name="element" select="IntegrationSettings" />
					<xsl:with-param name="class" select="'notification-email-toggle'" />
				</xsl:call-template>
				
				<label class="marginright" for="{$toggleField}">
					<xsl:value-of select="$toggleLabel" />
				</label>
			</div>
			
			<div class="notification-email-text">
			
				<xsl:if test="not(validationException/validationError/fieldName = $addressesField)">
					<xsl:attribute name="class">notification-email-text hidden</xsl:attribute>
				</xsl:if>
			
				<div>
				
					<label for="{$addressesField}">
						<xsl:value-of select="$addressesLabel" />
					</label>
					
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="$addressesField"/>
						<xsl:with-param name="name" select="$addressesField"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="$addresses" />
					</xsl:call-template>
				</div>
			</div>
		</div>
			
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endpointID'">
						<xsl:value-of select="$i18n.Endpoint"/>
					</xsl:when>
					<xsl:when test="fieldName = 'username'">
						<xsl:value-of select="$i18n.Username"/>
					</xsl:when>
					<xsl:when test="fieldName = 'password'">
						<xsl:value-of select="$i18n.Password"/>
					</xsl:when>
					<xsl:when test="fieldName = 'folderID'">
						<xsl:value-of select="$i18n.folderID"/>
					</xsl:when>
					<xsl:when test="fieldName = 'templateCode'">
						<xsl:value-of select="$i18n.templateCode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'groupCode'">
						<xsl:value-of select="$i18n.groupCode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'subject'">
						<xsl:value-of select="$i18n.subject"/>
					</xsl:when>
					<xsl:when test="fieldName = 'description'">
						<xsl:value-of select="$i18n.description"/>
					</xsl:when>
					<xsl:when test="fieldName = 'solutionStatusName'">
						<xsl:value-of select="$i18n.SolutionStatusName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'solutionMessage'">
						<xsl:value-of select="$i18n.SolutionMessage"/>
					</xsl:when>					
					<xsl:when test="fieldName = 'attachmentAttributeID'">
						<xsl:value-of select="$i18n.attachmentAttributeID"/>
					</xsl:when>		
					<xsl:when test="fieldName = 'errorEmailAddresses'">
						<xsl:value-of select="$i18n.Notifications.ErrorEmail.EmailAddresses"/>
					</xsl:when>													
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>