package com.nordicpeak.flowengine.integration.evolution.beans;

import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;

@Table(name = "evolution_integration_settings")
@XMLElement
public class IntegrationSettings extends BaseIntegrationSettings {

	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private IntegrationEndpoint endpoint;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String unitCode;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String description;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String notes;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String firstname;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String lastname;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String address;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String coAddress;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String zipCode;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String region;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String email;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String phone;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String mobilePhone;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean addAttachmentsSeparately;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean createCaseOnSpecificStatus;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "createCaseOnSpecificStatus")
	@XMLElement
	private String caseStatusName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "evolution_integration_email_recipients_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "Address")
	private List<String> errorEmailAddresses;

	@Override
	public boolean isPopulated() {

		return isEnabled() || endpoint != null || unitCode != null;
	}

	public IntegrationEndpoint getEndpoint() {

		return endpoint;
	}

	public void setEndpoint(IntegrationEndpoint endpoint) {

		this.endpoint = endpoint;
	}

	public boolean isSendErrorEmail() {

		return sendErrorEmail;
	}

	public void setSendErrorEmail(boolean sendErrorEmail) {

		this.sendErrorEmail = sendErrorEmail;
	}

	public List<String> getErrorEmailAddresses() {

		return errorEmailAddresses;
	}

	public void setErrorEmailAddresses(List<String> errorEmailAddresses) {

		this.errorEmailAddresses = errorEmailAddresses;
	}

	public boolean isAddAttachmentsSeparately() {

		return addAttachmentsSeparately;
	}

	public void setAddAttachmentsSeparately(boolean addAttachmentsSeparately) {

		this.addAttachmentsSeparately = addAttachmentsSeparately;
	}

	public boolean isCreateCaseOnSpecificStatus() {

		return createCaseOnSpecificStatus;
	}

	public void setCreateCaseOnSpecificStatus(boolean createCaseOnSpecificStatus) {

		this.createCaseOnSpecificStatus = createCaseOnSpecificStatus;
	}

	public String getCaseStatusName() {

		return caseStatusName;
	}

	public void setCaseStatusName(String caseStatusName) {

		this.caseStatusName = caseStatusName;
	}

	public String getUnitCode() {

		return unitCode;
	}

	public void setUnitCode(String unitCode) {

		this.unitCode = unitCode;
	}

	public String getDescription() {

		return description;
	}

	public void setDescription(String description) {

		this.description = description;
	}

	public String getNotes() {

		return notes;
	}

	public void setNotes(String notes) {

		this.notes = notes;
	}

	public String getFirstname() {

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getCoAddress() {

		return coAddress;
	}

	public void setCoAddress(String coAddress) {

		this.coAddress = coAddress;
	}

	public String getZipCode() {

		return zipCode;
	}

	public void setZipCode(String zipCode) {

		this.zipCode = zipCode;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getMobilePhone() {

		return mobilePhone;
	}

	public void setMobilePhone(String mobilePhone) {

		this.mobilePhone = mobilePhone;
	}

	
	public String getRegion() {
	
		return region;
	}

	
	public void setRegion(String region) {
	
		this.region = region;
	}

	
	public String getPhone() {
	
		return phone;
	}

	
	public void setPhone(String phone) {
	
		this.phone = phone;
	}

}
