package com.nordicpeak.flowengine.externalalternativelist;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeListCRUD;

public class ListSourceCRUD extends BaseAlternativeListCRUD<ListSource, ExternalAlternativeListInstance, ListSourceCRUD, ExternalAlternativeListProviderModule, ListSourceEvent> {

	public ListSourceCRUD(AnnotatedDAOWrapper<ListSource, Integer> queryDAO, BeanRequestPopulator<ListSource> populator, ExternalAlternativeListProviderModule callback) {

		super(IntegerBeanIDParser.getInstance(), queryDAO, populator, "AlternativeList", "list source", "/", callback);
	}

	@Override
	protected void appendShowFormData(ListSource source, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {

		super.appendShowFormData(source, doc, showTypeElement, user, req, res, uriParser);

		ExternalAlternativeListInstance instance = callback.getAlternativeInstance(source, null, null);

		if (instance != null && !CollectionUtils.isEmpty(instance.getAlternatives())) {

			XMLUtils.append(doc, showTypeElement, "Alternatives", instance.getAlternatives());
		}
	}

	@Override
	protected void appendListFormData(Document doc, Element listTypeElement, User user, HttpServletRequest req, URIParser uriParser, List<ValidationError> validationError) throws SQLException {

		super.appendListFormData(doc, listTypeElement, user, req, uriParser, validationError);

		Collection<ExternalAlternativeListInstance> alternativeInstances = callback.getAlternativeInstances();

		if (!CollectionUtils.isEmpty(alternativeInstances)) {
			for (AlternativeListInstance alternativeList : alternativeInstances) {

				if (alternativeList.getAlternatives() != null) {

					Element sizeElement = XMLUtils.appendNewElement(doc, listTypeElement, "ListSize");

					XMLUtils.appendNewElement(doc, sizeElement, "Name", alternativeList.getDescriptor().getName());
					XMLUtils.appendNewElement(doc, sizeElement, "Value", alternativeList.getAlternatives().size());
				}
			}
		}
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.append(doc, addTypeElement, "AllowedEncodings", "Encoding", callback.getAllowedEncodings());
	}

	@Override
	protected void appendUpdateFormData(ListSource source, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.append(doc, updateTypeElement, "AllowedEncodings", "Encoding", callback.getAllowedEncodings());
	}

	@Override
	protected void validateAddPopulation(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateAddPopulation(source, req, user, uriParser);
		validate(source, req, user, uriParser);
	}

	@Override
	protected void validateUpdatePopulation(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateUpdatePopulation(source, req, user, uriParser);
		validate(source, req, user, uriParser);
	}

	protected void validate(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		List<ValidationError> validationErrors = new ArrayList<>();

		super.validate(source, req, user, uriParser, validationErrors);

		if (!HTTPUtils.isValidURL(source.getUrl())) {
			validationErrors.add(new ValidationError("InvalidURL"));
		}

		checkEncoding(source);

		if (!callback.testSource(source)) {
			validationErrors.add(new ValidationError("InvalidSource", "", "url"));
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
	}

	private void checkEncoding(ListSource bean) throws ValidationException {

		if (!callback.getAllowedEncodings().contains(bean.getEncoding())) {
			throw new ValidationException(new ValidationError("encoding", ValidationErrorType.InvalidFormat));
		}
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		if(callback.getAPISourceHandler() != null) {
			
			res.sendRedirect(uriParser.getContextPath() + callback.getAPISourceHandler().getFullAlias());
			return null;
			
		}else {
			
			return super.list(req, res, user, uriParser, validationErrors);
		}
	}
}
