package com.nordicpeak.flowengine.queries.fixedtablequery;

import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.IntegerBasedCRUD;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;

public class FixedTableRowCRUD extends IntegerBasedCRUD<FixedTableRow, FixedTableQueryProviderModule> {

	protected AnnotatedDAOWrapper<FixedTableRow, Integer> tableRowDAO;

	public FixedTableRowCRUD(AnnotatedDAOWrapper<FixedTableRow, Integer> tableRowDAO, BeanRequestPopulator<FixedTableRow> populator, String typeElementName, String typeLogName, String listMethodAlias, FixedTableQueryProviderModule callback) {

		super(tableRowDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.tableRowDAO = tableRowDAO;
	}

	@Override
	protected ForegroundModuleResponse beanAdded(FixedTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventRowAddedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(FixedTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventRowUpdatedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanDeleted(FixedTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQuery().getQueryDescriptor().getName() + "\", " + callback.getEventRowDeletedMessage() + " \"" + bean.getLabel() + "\"", bean.getQuery().getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	public FixedTableRow getBean(Integer beanID) throws SQLException, AccessDeniedException {

		HighLevelQuery<FixedTableRow> query = new HighLevelQuery<>(FixedTableRow.QUERY_RELATION);

		query.addParameter(tableRowDAO.getParameterFactory().getParameter(beanID));

		FixedTableRow tableRow = tableRowDAO.getAnnotatedDAO().get(query);

		if (tableRow != null && tableRow.getQuery() != null) {

			tableRow.getQuery().init(callback.getFlowAdminModule().getQueryDescriptor(tableRow.getQuery().getQueryID()), null);

		}

		return tableRow;
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, addTypeElement);
	}

	@Override
	protected void appendUpdateFormData(FixedTableRow bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, updateTypeElement);
	}

	protected void appendFormatValidators(Document doc, Element element) {

		XMLUtils.append(doc, element, callback.getFormatValidators());
	}

	@Override
	protected FixedTableRow populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		FixedTableRow tableRow = super.populateFromAddRequest(req, user, uriParser);

		tableRow.setQuery((FixedTableQuery) req.getAttribute("FixedTableQuery"));

		tableRow.setSortIndex(this.getCurrentMaxSortIndex(tableRow.getQuery()) + 1);

		return tableRow;
	}

	@Override
	protected void checkUpdateAccess(FixedTableRow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	@Override
	protected void checkDeleteAccess(FixedTableRow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	private int getCurrentMaxSortIndex(FixedTableQuery fixedTableQuery) throws SQLException {

		ObjectQuery<Integer> query = new ObjectQuery<>(tableRowDAO.getAnnotatedDAO().getDataSource(), "SELECT MAX(sortIndex) FROM " + tableRowDAO.getAnnotatedDAO().getTableName() + " WHERE queryID = ?", IntegerPopulator.getPopulator());

		query.setInt(1, fixedTableQuery.getQueryID());

		Integer sortIndex = query.executeQuery();

		if (sortIndex == null) {

			return 0;
		}

		return sortIndex;
	}

}
