package com.nordicpeak.flowengine.flowinstanceadminremoteinfo.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.APISource;

@Table(name = "flow_instance_admin_remote_info_endpoints")
@XMLElement(name = "Endpoint")
public class FlowInstanceAdminRemoteInfoEndpoint extends GeneratedElementable implements XMLParserPopulateable, Serializable, APISource {

	private static final long serialVersionUID = -8348666528294482053L;

	public static final Field SETTINGS_RELATION = ReflectionUtils.getField(FlowInstanceAdminRemoteInfoEndpoint.class, "settings");

	@DAOManaged
	@Key
	@XMLElement
	private Integer endpointID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 1024)
	@XMLElement
	private String address;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String username;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String password;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private String encoding;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<FlowInstanceAdminRemoteInfoSetting> settings;

	@Override
	public String toString() {

		return name + " (ID: " + endpointID + ")";
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		this.endpointID = XMLValidationUtils.validateParameter("endpointID", xmlParser, true, PositiveStringIntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public Integer getEndpointID() {

		return endpointID;
	}

	public void setEndpointID(Integer endpointID) {

		this.endpointID = endpointID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getUsername() {

		return username;
	}

	public void setUsername(String username) {

		this.username = username;
	}

	public String getPassword() {

		return password;
	}

	public void setPassword(String password) {

		this.password = password;
	}

	public String getEncoding() {

		return encoding;
	}

	public void setEncoding(String encoding) {

		this.encoding = encoding;
	}

	public Charset getCharset() {

		return Charset.forName(encoding);
	}

	public List<FlowInstanceAdminRemoteInfoSetting> getSettings() {

		return settings;
	}

	public void setSettings(List<FlowInstanceAdminRemoteInfoSetting> settings) {

		this.settings = settings;
	}

}
