package com.nordicpeak.flowengine.flowinstanceadminremoteinfo.cruds;

import java.sql.SQLException;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.hierarchy.core.utils.crud.ModularCRUD;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.CRUDDAO;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.flowinstanceadminremoteinfo.FlowInstanceAdminRemoteInfoModule;
import com.nordicpeak.flowengine.flowinstanceadminremoteinfo.beans.FlowInstanceAdminRemoteInfoEndpoint;

public class FlowInstanceAdminRemoteInfoEndpointCRUD extends ModularCRUD<FlowInstanceAdminRemoteInfoEndpoint, Integer, User, FlowInstanceAdminRemoteInfoModule> {

	public FlowInstanceAdminRemoteInfoEndpointCRUD(CRUDDAO<FlowInstanceAdminRemoteInfoEndpoint, Integer> crudDAO, FlowInstanceAdminRemoteInfoModule callback) {

		super(IntegerBeanIDParser.getInstance(), crudDAO, new AnnotatedRequestPopulator<FlowInstanceAdminRemoteInfoEndpoint>(FlowInstanceAdminRemoteInfoEndpoint.class), "Endpoint", "endpoint", "/", callback);
	
		setRequirePostForDelete(true);
	}

	@Override
	protected void checkDeleteAccess(FlowInstanceAdminRemoteInfoEndpoint endpoint, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		super.checkDeleteAccess(endpoint, user, req, uriParser);

		if (!CollectionUtils.isEmpty(callback.getFlowFamilies(endpoint))) {

			throw new AccessDeniedException("User " + user + " not allowed to delete endpoint " + endpoint + " whilst being used in flow families");
		}
	}

	@Override
	protected void appendAllBeans(Document doc, Element listTypeElement, User user, HttpServletRequest req, URIParser uriParser, List<ValidationError> validationError) throws SQLException  {
		
		List<FlowInstanceAdminRemoteInfoEndpoint> endpoints = getAllBeans(user, req, uriParser);
		
		if (endpoints != null) {

			Element listElement = XMLUtils.appendNewElement(doc, listTypeElement, typeElementPluralName);

			for (FlowInstanceAdminRemoteInfoEndpoint endpoint : endpoints) {

				Element element = endpoint.toXML(doc);

				if (callback.isInUse(endpoint)) {
					
					XMLUtils.appendNewElement(doc, element, "InUse");
				}

				listElement.appendChild(element);
			}
		}
	}	
	
	@Override
	protected void appendShowFormData(FlowInstanceAdminRemoteInfoEndpoint endpoint, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws Exception {

		List<FlowFamily> flowFamilies = callback.getFlowFamilies(endpoint);

		if (!CollectionUtils.isEmpty(flowFamilies)) {

			XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);

			genDoc.addRootElementableListener(FlowFamily.class, (document, element, flowFamily) -> {

				Flow latestFlow = callback.getLatestFlowVersion(flowFamily);

				XMLUtils.append(document, element, latestFlow);
			});

			XMLUtils.append(genDoc, showTypeElement, "FlowFamilies", flowFamilies);
		}
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.append(doc, addTypeElement, "AllowedEncodings", "Encoding", callback.getAllowedEncodings());
	}

	@Override
	protected void appendUpdateFormData(FlowInstanceAdminRemoteInfoEndpoint bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.append(doc, updateTypeElement, "AllowedEncodings", "Encoding", callback.getAllowedEncodings());
	}

	@Override
	protected void validateAddPopulation(FlowInstanceAdminRemoteInfoEndpoint bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		checkEncoding(bean);
	}

	@Override
	protected void validateUpdatePopulation(FlowInstanceAdminRemoteInfoEndpoint bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		checkEncoding(bean);
	}

	private void checkEncoding(FlowInstanceAdminRemoteInfoEndpoint bean) throws ValidationException {

		if (!callback.getAllowedEncodings().contains(bean.getEncoding())) {

			throw new ValidationException(new ValidationError("encoding", ValidationErrorType.InvalidFormat));
		}
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		if(callback.getAPISourceHandler() != null) {
			
			res.sendRedirect(uriParser.getContextPath() + callback.getAPISourceHandler().getFullAlias());
			return null;
			
		}else {
			
			return super.list(req, res, user, uriParser, validationErrors);
		}
	}
}