package com.nordicpeak.flowengine.flowinstanceadmin2.sendmessages;

import java.sql.Timestamp;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.Status;

@XMLElement
public class FlowSelection extends GeneratedElementable {

	@XMLElement(fixCase = true)
	private final FlowFamily flowFamily;
	
	@XMLElement(fixCase = true)
	private final List<Flow> flows;

	private final List<FlowInstance> flowInstances;

	@XMLElement(valueFormatter = DateStringyfier.class)
	private final Timestamp fromDate;

	@XMLElement(valueFormatter = DateStringyfier.class)
	private final Timestamp toDate;

	public FlowSelection(Flow flow, List<FlowInstance> flowInstances) {
		super();

		this.flows = Collections.singletonList(flow);
		flowFamily = flow.getFlowFamily();
		this.flowInstances = flowInstances;
		fromDate = null;
		toDate = null;
	}

	public FlowSelection(FlowFamily flowFamily, List<Flow> flows, Timestamp fromDate, Timestamp toDate, List<Status> statuses) {
		super();

		this.flowFamily = flowFamily;
		this.flows = flows;
		this.fromDate = fromDate;
		this.toDate = toDate;
		flowInstances = null;
	}
	
	public FlowFamily getFlowFamily() {
		
		return flowFamily;
	}

	public List<Flow> getFlows() {
		
		return flows;
	}
	
	public List<FlowInstance> getFlowInstances() {
		return flowInstances;
	}
	
	public boolean hasDirectFlowInstances() {
		return flowInstances != null;
	}

	public Timestamp getFromDate() {
		return fromDate;
	}

	public Timestamp getToDate() {
		return toDate;
	}

	@Override
	public String toString() {

		StringBuilder builder = new StringBuilder();

		builder.append("flowFamily=" + flowFamily.getFlowFamilyID());
		
		if (fromDate != null) {
			builder.append(", fromDate=" + fromDate);
		}
		
		if (toDate != null) {
			builder.append(", toDate=" + toDate);
		}

		if (!CollectionUtils.isEmpty(flows)) {
			
			builder.append(", flows=[ ");
			
			boolean first = true;
			for (Flow flow : flows) {
				
				if (!first) {
					builder.append(", ");
				} else {
					first = false;
				}
				
				builder.append(flow);
			}
			
			builder.append(" ]");
		}
		
		if (!CollectionUtils.isEmpty(flowInstances)) {

			builder.append(", flowInstances=[ ");

			boolean first = true;
			for (FlowInstance flowInstance : flowInstances) {

				if (!first) {
					builder.append(", ");
				} else {
					first = false;
				}
				
				builder.append(flowInstance.getFlowInstanceID());
			}

			builder.append(" ]");
		}

		return builder.toString();
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (hasDirectFlowInstances()) {
			XMLUtils.appendNewElement(doc, element, "DirectFlowInstances");
		}
		
		if (flowInstances != null) {

			Element flowInstanceIDsElement = XMLUtils.appendNewElement(doc, element, "FlowInstanceIDs");
			
			for (FlowInstance flowInstance : flowInstances) {
				XMLUtils.appendNewElement(doc, flowInstanceIDsElement, "FlowInstanceID", flowInstance.getFlowInstanceID());
			}
		}

		return element;
	}

}
