<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/flowinstanceextraparameters.js
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">
	
		<xsl:apply-templates select="ShowExtraParameters" />
		<xsl:apply-templates select="UpdateExtraParameters" />
			
	</xsl:template>
	
	<xsl:template match="ShowExtraParameters">
	
		<a name="extraparameters" />
		
		<xsl:choose>
			<xsl:when test="FlowInstanceExtraParameters">
				<span><xsl:value-of select="$i18n.NumberOfExtraParameters"/>: <xsl:value-of select="count(FlowInstanceExtraParameters/FlowInstanceExtraParameter)"/></span>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoFlowFamilyExtraParameters" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/update/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateFlowFamilyExtraParameters" />
			<img class="marginleft" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="FlowInstanceExtraParameters/FlowInstanceExtraParameter">
			<a href="{/Document/ModuleURI}/delete/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteFlowFamilyExtraParameters.Confirm}');">
				<xsl:value-of select="$i18n.DeleteExtraParameters" />
				<img class="marginleft" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdateExtraParameters">
		
		<div id="ExtraParametersProvider" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdateExtraParametersTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
				<xsl:text>&#x20;(</xsl:text>
				<xsl:value-of select="$i18n.flowVersion"/>
				<xsl:text>&#x20;</xsl:text>
				<xsl:value-of select="Flow/version"/>
				<xsl:text>)</xsl:text>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError"/>
			
	
			<form method="post" action="{/Document/ModuleURI}/update/{Flow/flowID}">
			
				<div class="full floatleft bigmargintop">
					<h2><xsl:value-of select="$i18n.Parameters" /></h2>
				</div>
			
				<table id="param-table" class="full coloredtable ">
					<thead>
						<tr>
							<th><xsl:value-of select="$i18n.Name" /></th>
							<th><xsl:value-of select="$i18n.Value" /></th>
							<th width="16px"></th>
						</tr>
					</thead>
					<tbody>
					
						<xsl:choose>
							<xsl:when test="requestparameters">
							
								<xsl:apply-templates select="requestparameters/parameter[name = 'parameterID']/value" mode="parameter"/>
							
							</xsl:when>
							<xsl:otherwise>
							
								<xsl:apply-templates select="FlowInstanceExtraParameters/FlowInstanceExtraParameter"/>
							
							</xsl:otherwise>
						</xsl:choose>					
					
					</tbody>
				</table>
				
				<div class="floatright">
					<a onclick="addParameter();">
						<xsl:value-of select="$i18n.AddParameter" />
						<img class="alignbottom" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
					</a>
				</div>			
				
				<div>
			<table class="full border">
				<tr>
					<th>
						<xsl:value-of select="$i18n.Tag"/>
					</th>
					<th>
						<xsl:value-of select="$i18n.Description"/>
					</th>
				</tr>
				<tr>
					<td>
						<xsl:text>$flow.name</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.FlowNameTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$flowInstance.url</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.FlowInstanceUrlTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$flowInstance.flowInstanceID</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.FlowInstanceIDTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$flowInstance.firstSubmitted</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.FlowInstanceFirstSubmittedTag"/>
					</td>
				</tr>
			</table>
			</div>
			<br/>
			
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>			
			
			<div style="display: none;">
			
				<table>
					<tr id="parameter-template">
						<td class="vertical-align-top">
							<input type="text" name="name" class="full"/>
						</td>
						<td>
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="name" select="'value'"/>
							<xsl:with-param name="rows" select="'3'" />
							<xsl:with-param name="class" select="'full'" />
						</xsl:call-template>
						</td>
						<td>
							<input type="hidden" name="parameterID"/>
							
							<a onclick="deleteParameter(this)">
								<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
							</a>				
						</td>
					</tr>
				</table>
				
			</div>		
		
		</div>

	</xsl:template>
	
	<xsl:template match="FlowInstanceExtraParameter">
	
		<xsl:variable name="parameterID" select="parameterID"/>
		<xsl:variable name="name" select="concat('value-', parameterID)"/>
		
		<tr id="parameter-{parameterID}">
			<td class="vertical-align-top">
				<input type="text" name="name-{parameterID}" value="{name}" class="full"/>
			</td>
			<td>
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="name" select="$name"/>
					<xsl:with-param name="value" select="value"/>
					<xsl:with-param name="rows" select="'3'" />
					<xsl:with-param name="class" select="'full'" />
				</xsl:call-template>
			</td>
			<td>
				<input type="hidden" name="parameterID" value="{parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="value" mode="parameter">
		
 		<xsl:variable name="parameterID" select="."/>
		
		<xsl:variable name="nameParam" select="concat('name-', $parameterID)"/>
		
		<xsl:variable name="valueParam" select="concat('value-', $parameterID)"/>
		
		<xsl:variable name="valueTextParam" select="../../parameter[name = $valueParam]/value"/>
		
		<tr id="parameter-{$parameterID}">
 			<td class="vertical-align-top">
				<input type="text" name="name-{$parameterID}" value="{../../parameter[name = $nameParam]/value}" class="full"/>
			</td>
			<td>
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="name" select="$valueParam"/>
					<xsl:with-param name="value" select="$valueTextParam"/>
					<xsl:with-param name="rows" select="'3'" />
					<xsl:with-param name="class" select="'full'" />
				</xsl:call-template>
			</td>
			<td>
				<input type="hidden" name="parameterID" value="{$parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>	
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'url'">
						<xsl:value-of select="$i18n.Url"/>
					</xsl:when>
					<xsl:when test="fieldName = 'username'">
						<xsl:value-of select="$i18n.Username"/>
					</xsl:when>
					<xsl:when test="fieldName = 'password'">
						<xsl:value-of select="$i18n.Password"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>