<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js
		/jquery/jquery-migrate.js
		/jquery/plugins/jquery.qloader.js
		/js/confirmpost.js
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/flowlistextension.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/flowlistextension.css
	</xsl:variable>

	<xsl:template match="Document">
		<xsl:apply-templates select="MyListedFlows" />
		<xsl:apply-templates select="ShowSettings"/>
		<xsl:apply-templates select="ShowUpdateSettings"/>
		<xsl:apply-templates select="ValidationErrors" />
	
	</xsl:template>
	
	<xsl:template match="MyListedFlows">
	
		<div id="FlowListExtensionProviderModule" class="contentitem">
			<section class="flowlistextension-content">
				<xsl:value-of  select="../listCurrentFlows" disable-output-escaping="yes" />
			</section>
			<xsl:choose>  
				<xsl:when test="count(FlowType) > 0">
					<xsl:for-each select="FlowType">
						<p>
							 <section class="border">
					          <h2 class="floatleft flowlistextension-content bigmargintop bigpaddingtop bigmarginbottom bigpaddingbottom full"><xsl:value-of  select="FlowTypeName" /></h2>
					          <br/>
					          
					          <xsl:for-each select="Flows/Flow">
					          		<p class="flowlistextension-content bigmargintop bigmarginright">
					          			<xsl:call-template name="FlowTypeAndFlow"/>
					          		</p>
					          		<xsl:if test="not(position() = last())">
						          		<div class="flowlistextension-separator bigmargintop paddingbottom bigmarginright"/>
						          	</xsl:if>
					          </xsl:for-each>
					         </section>
						</p>        
				 	</xsl:for-each>
		 		</xsl:when>
		 		<xsl:otherwise>
			 		<br/>
			 		<section class="flowlistextension-padding paddingtop">
			 			<h3 style="margin-left:25px">
			 				<xsl:value-of select="$i18n.NoFlowsFound"/>
			 			</h3>
			 		</section>
		 		</xsl:otherwise>
			</xsl:choose>
			
		</div>
	
	</xsl:template>

	<xsl:template name="FlowTypeAndFlow">
		<xsl:variable name="flowID" select="FlowID"></xsl:variable>
    	<xsl:variable name="flowIconFileName" select="FlowIconFileName"></xsl:variable>
     	<xsl:variable name="href" select="FlowHref"></xsl:variable>
    		
    		<xsl:choose>
    			<xsl:when test="FlowEnabled='true'">
    				<a href="{$href}" class="full " target="_blank" rel="noopener noreferrer">
			      			<img class="bigmarginright" alt="{$flowIconFileName}" src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/flowicon/{$flowID}" width="25" />
			       			
			       		<xsl:value-of select="FlowName"/>
			       	</a>
    			</xsl:when>
    			<xsl:otherwise>
    				<img class="bigmarginright" alt="{$flowIconFileName}" src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/flowicon/{$flowID}" width="25" />
			       	<xsl:value-of select="FlowName"/>
		       		
		       		<xsl:value-of select="$i18n.FlowDisabled"/>
    			</xsl:otherwise>
    		</xsl:choose>
					          		
					            
	</xsl:template>
	
	<xsl:template match="ShowSettings">
		<xsl:choose>
			<xsl:when test="//FlowListExtensionSettings/moduleEnabled">
				<xsl:value-of select="$i18n.Enabled" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NotEnabled" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/requestinfo/contextpath}{extensionRequestURL}/showupdatesettings" class="floatright">
			<xsl:value-of select="$i18n.UpdateSettings" />
			<img class="marginleft vertical-align-bottom" src="{/Document/requestinfo/contextpath}{extensionRequestURL}/static/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="//FlowListExtensionSettings/flowFamilyID">
			<a href="{/Document/requestinfo/contextpath}{extensionRequestURL}/deletesettings" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteSettings" />
				<img class="marginleft vertical-align-bottom" src="{/Document/requestinfo/contextpath}{extensionRequestURL}/static/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
		
	</xsl:template>
	
	
	<xsl:template match="ShowUpdateSettings">
		
		<div id="Settings" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.adminExtensionViewTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
			
			<xsl:apply-templates select="ValidationErrors/validationError"/>
			
			<xsl:value-of select="$i18n.UpdateSettings.description" disable-output-escaping="yes" />
				
			<form method="post" action="{/Document/requestinfo/contextpath}{extensionRequestURL}/updatesettings">
				
					<div class="floatleft full">
	
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'moduleEnabled'" />
								<xsl:with-param name="name" select="'moduleEnabled'" />
								<xsl:with-param name="value" select="'true'" />
								<xsl:with-param name="element" select="FlowListExtensionSettings" />
							</xsl:call-template>
							
							<label for="moduleEnabled">
								<xsl:value-of select="$i18n.Settings.Active" />
							</label>
						</div>
		
					</div>
				
					<div class="floatleft full hidden" id="formWrapper">
				
						<div class="floatleft full bigmarginbottom" id="linkUrl">
						
							<label class="floatleft" for="externalLink">
								<xsl:value-of select="$i18n.Settings.ActivateLink" />
							</label>
		
							<div class="floatleft full">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="id" select="'externalLink'" />
									<xsl:with-param name="name" select="'externalLink'" />
									<xsl:with-param name="title" select="$i18n.Settings.ActivateLink"/>
									<xsl:with-param name="element" select="FlowListExtensionSettings" />
								</xsl:call-template>
								
								
							</div>
			
						</div>
						
						<div class="floatleft full bigmarginbottom" id="customNameWrapper">
						
							<label class="floatleft" for="customName">
								<xsl:value-of select="$i18n.Settings.CustomName" />
							</label>
		
							<div class="floatleft full">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="id" select="'customName'" />
									<xsl:with-param name="name" select="'customName'" />
									<xsl:with-param name="title" select="$i18n.Settings.CustomName"/>
									<xsl:with-param name="element" select="FlowListExtensionSettings" />
								</xsl:call-template>
								
								
							</div>
			
						</div>
					
					</div>
					
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
		
		</div>

	</xsl:template>
	
	<xsl:template match="ValidationErrors">
		
		<div class="bigmarginbottom">
			<xsl:apply-templates select="validationError" />
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
		
		<p class="error">
			<xsl:value-of select="$i18n.UnableToParseRequest"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="validationErrorType='RequiredField'">
							<xsl:value-of select="$i18n.validationError.RequiredField" />
						</xsl:when>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.validationError.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooShort'">
							<xsl:value-of select="$i18n.validationError.TooShort" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.validationError.TooLong" />
						</xsl:when>
						<xsl:when test="validationErrorType='Other'">
							<xsl:value-of select="$i18n.validationError.Other" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.unknownValidationErrorType" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#x20;</xsl:text>
					
					<xsl:choose>
						<xsl:when test="fieldName = 'externalLink'">
							<xsl:value-of select="$i18n.Settings.ActivateLink" />
						</xsl:when>
						<xsl:when test="fieldName = 'customName'">
							<xsl:value-of select="$i18n.Settings.CustomName" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName"/>
						</xsl:otherwise>
					</xsl:choose>
					
					<xsl:text>!</xsl:text>
				</span>
			</p>
		</xsl:if>
		<xsl:if test="messageKey">
			<p class="error">
				<span data-icon-before="!">
					<xsl:value-of select="$i18n.validationError.unknownMessageKey" />!
				</span>
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>