<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/genericpaymentprovider.js
	</xsl:variable>

	<xsl:template match="Document">
	
		<xsl:apply-templates select="NoPaymentSettingsError" />
		<xsl:apply-templates select="ShowPaymentSettings" />
		<xsl:apply-templates select="UpdatePaymentSettings" />
		<xsl:apply-templates select="PaymentForm" />
			
	</xsl:template>
	
	<xsl:template match="NoPaymentSettingsError">
			
		<div id="GenericPaymentProvider">
			
			<div class="errands-wrapper nomargin nopadding error">
				
				<p class="error">
					<xsl:value-of select="$i18n.NoPaymentSettingsFound"/>
				</p>
				
			</div>
		
		</div>
		
	</xsl:template>	
	
	<xsl:template match="ShowPaymentSettings">
	
		<a name="paymentsettings" />
		
		<xsl:choose>
			<xsl:when test="FlowFamilyPaymentSettings">
				
				<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoFlowPaymentSettings" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/updateflowpaymentsettings/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdatePaymentSettings" />
			<img class="marginleft" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="FlowFamilyPaymentSettings">
			<a href="{/Document/ModuleURI}/deleteflowpaymentsettings/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeletePaymentSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeletePaymentSettings" />
				<img class="marginleft" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdatePaymentSettings">
		
		<div id="GenericPaymentProvider" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdatePaymentSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError" mode="admin"/>
	
			<form method="post" action="{/Document/ModuleURI}/updateflowpaymentsettings/{Flow/flowID}">
			
				<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>
			
				<div id="invoicePaymentEventDescriptionContainer">
					<label for="invoicePaymentAlternativeDescription" class="floatleft full">
						<xsl:value-of select="$i18n.invoicePaymentEventDescription" />
						<span class="required">*</span>
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'invoicePaymentEventDescription'"/>
						<xsl:with-param name="name" select="'invoicePaymentEventDescription'"/>
						<xsl:with-param name="value">
							
							<xsl:choose>
								<xsl:when test="FlowFamilyPaymentSettings/invoicePaymentEventDescription">
									<xsl:value-of select="FlowFamilyPaymentSettings/invoicePaymentEventDescription"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="DefaultInvoicePaymentEventDescription"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:with-param>
					</xsl:call-template>
				</div>
				
				<div id="invoiceDescriptionSuffixContainer">
					<label for="invoiceDescriptionSuffix" class="floatleft full">
						<xsl:value-of select="$i18n.invoiceDescriptionSuffix" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'invoiceDescriptionSuffix'" />
						<xsl:with-param name="name" select="'invoiceDescriptionSuffix'" />
						<xsl:with-param name="value" select="FlowFamilyPaymentSettings/invoiceDescriptionSuffix" />
					</xsl:call-template>
				</div>
				
				<div id="directPaymentNotificationAddresses">
					<label for="directPaymentAlternativeDescription" class="floatleft full">
						<xsl:value-of select="$i18n.directPaymentNotificationAddresses" />
					</label>
					
					<div class="floatleft full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="'directPaymentNotificationAddresses'"/>
							<xsl:with-param name="name" select="'directPaymentNotificationAddresses'"/>
							<xsl:with-param name="separateListValues" select="'true'"/>
							<xsl:with-param name="rows" select="5"/>
							<xsl:with-param name="element" select="FlowFamilyPaymentSettings/DirectPaymentNotificationAddresses/address" />      
						</xsl:call-template>
					</div>
				</div>
				
				<div id="invoicePaymentNotificationAddresses">
					<label for="invoicePaymentAlternativeDescription" class="floatleft full">
						<xsl:value-of select="$i18n.invoicePaymentNotificationAddresses" />
					</label>
					
					<div class="floatleft full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="'invoicePaymentNotificationAddresses'"/>
							<xsl:with-param name="name" select="'invoicePaymentNotificationAddresses'"/>
							<xsl:with-param name="separateListValues" select="'true'"/>
							<xsl:with-param name="rows" select="5"/>
							<xsl:with-param name="element" select="FlowFamilyPaymentSettings/InvoicePaymentNotificationAddresses/address" />      
						</xsl:call-template>
					</div>
				</div>
				
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
		
		</div>

	</xsl:template>	
	
	<xsl:template match="PaymentForm">
		
		<div id="GenericPaymentProvider">
		
			<xsl:apply-templates select="validationError" />
			
			<div class="errands-wrapper nomargin nopadding">
				
				<xsl:if test="validationError">
					<xsl:attribute name="class">errands-wrapper nomargin nopadding error</xsl:attribute>
				</xsl:if>
				
				<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>
				
			</div>
		
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError">

		<xsl:call-template name="printValidationError">
			<xsl:with-param name="message" select="$i18n.UnknownValidationError"></xsl:with-param>
		</xsl:call-template>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PaymentFailed']">

		<xsl:call-template name="printValidationError">
			<xsl:with-param name="message" select="$i18n.PaymentFailed"></xsl:with-param>
		</xsl:call-template>
		
	</xsl:template>
	
	<xsl:template name="printValidationError">
		
		<xsl:param name="message" />
		
		<div class="info-box first error">
			<span>
				<strong data-icon-before="!"><xsl:value-of select="$i18n.PaymentFailedTitle" />.</strong>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$message" />
			</span>
			<div class="marker" />
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError" mode="admin">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="fieldName = 'directPaymentNotificationAddresses'">
						<xsl:value-of select="$i18n.directPaymentNotificationAddresses"/>
					</xsl:when>
					<xsl:when test="fieldName = 'invoicePaymentNotificationAddresses'">
						<xsl:value-of select="$i18n.invoicePaymentNotificationAddresses"/>
					</xsl:when>
					<xsl:when test="fieldName = 'invoicePaymentEventDescription'">
						<xsl:value-of select="$i18n.invoicePaymentEventDescription"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>