package com.nordicpeak.flowengine.internalalternativelist;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.hierarchy.core.utils.crud.ModularCRUD;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.serialization.SerializationUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.exceptions.UnableToDeleteListInUseException;
import com.nordicpeak.flowengine.alternativelist.validationerrors.ListNameAlreadyInUseValidationError;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternative;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeList;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeListInstance;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalCombinedList;

public class InternalCombinedListCRUD extends ModularCRUD<InternalCombinedList, Integer, User, InternalAlternativeListProviderModule> {

	public InternalCombinedListCRUD(AnnotatedDAOWrapper<InternalCombinedList, Integer> queryDAO, BeanRequestPopulator<InternalCombinedList> populator, InternalAlternativeListProviderModule callback) {

		super(IntegerBeanIDParser.getInstance(), queryDAO, populator, "CombinedList", "internal combined list", "/", callback);
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void appendBean(InternalCombinedList combinedList, Element targetElement, Document doc, User user) {

		if (!CollectionUtils.isEmpty(combinedList.getLists())) {

			combinedList.setAlternatives(callback.getCombinedInstance(combinedList, null).getAlternatives());

			for (InternalAlternativeList list : combinedList.getLists()) {

				InternalAlternativeListInstance instance = callback.getAlternativeInstance(list, null, null);
				list.setAlternatives((List<InternalAlternative>) instance.getAlternatives());
			}
		}

		Element listElement = combinedList.toXML(doc);

		if (!callback.hasListAccess(user, combinedList)) {

			XMLUtils.appendNewElement(doc, listElement, "HideAdminLinks");
		}

		targetElement.appendChild(listElement);
	}

	@Override
	protected void appendShowFormData(InternalCombinedList combinedList, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {

		super.appendShowFormData(combinedList, doc, showTypeElement, user, req, res, uriParser);

		XMLUtils.append(doc, showTypeElement, "Events", callback.getRecentCombinedListEvents(combinedList));
	}

	@Override
	protected void appendUpdateFormData(InternalCombinedList list, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(list, doc, updateTypeElement, user, req, uriParser);

		Element adminsElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Admins");

		AccessUtils.appendAllowedGroupsAndUsers(doc, adminsElement, list, callback.getUserHandler(), callback.getGroupHandler());
	}

	@Override
	protected void checkUpdateAccess(InternalCombinedList list, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		super.checkUpdateAccess(list, user, req, uriParser);

		checkAdminAccess(user, list);
	}

	@Override
	protected void checkDeleteAccess(InternalCombinedList list, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		super.checkDeleteAccess(list, user, req, uriParser);

		if (!CollectionUtils.isEmpty(callback.getAlternativeListHandlerModule().getQueriesUsingAlternativeList(callback.getNewCombinedDescriptor(list)))) {

			throw new UnableToDeleteListInUseException();
		}

		checkAdminAccess(user, list);
	}

	private void checkAdminAccess(User user, InternalCombinedList list) throws AccessDeniedException {

		if (!callback.hasListAccess(user, list)) {

			throw new AccessDeniedException("User " + user + " is not an administrator for list " + list);
		}
	}

	@Override
	protected InternalCombinedList populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		InternalCombinedList list = super.populateFromAddRequest(req, user, uriParser);

		populate(list, null, req, user, uriParser);

		return list;
	}

	@Override
	protected InternalCombinedList populateFromUpdateRequest(InternalCombinedList oldList, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		req.setAttribute("oldList", SerializationUtils.cloneSerializable(oldList));

		InternalCombinedList list = super.populateFromUpdateRequest(oldList, req, user, uriParser);

		populate(list, oldList, req, user, uriParser);

		return list;
	}

	@Override
	protected void validateAddPopulation(InternalCombinedList list, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateAddPopulation(list, req, user, uriParser);

		validatePopulation(list, req, user, uriParser);
	}

	@Override
	protected void validateUpdatePopulation(InternalCombinedList list, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateUpdatePopulation(list, req, user, uriParser);

		validatePopulation(list, req, user, uriParser);
	}

	private void validatePopulation(InternalCombinedList list, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException {

		if (!callback.hasListAccess(user, list)) {

			throw new ValidationException(new ValidationError("CantRemoveOwnAccess"));
		}
	}

	protected void populate(InternalCombinedList combinedList, InternalCombinedList oldList, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		List<Integer> listIDs = ValidationUtils.validateParameters("listID", req, false, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
		List<InternalAlternativeList> lists = null;

		if (!CollectionUtils.isEmpty(listIDs)) {

			lists = new ArrayList<InternalAlternativeList>();

			for (Integer listID : listIDs) {

				InternalAlternativeList list = callback.getList(listID);

				if (list != null) {

					lists.add(list);

				} else {

					validationErrors.add(new ValidationError("listID", ValidationErrorType.InvalidFormat));
				}
			}
		}

		validate(combinedList, req, user, uriParser, validationErrors);

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		combinedList.setLists(lists);
	}

	protected void validate(InternalCombinedList list, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws ValidationException, Exception {

		InternalCombinedList oldList = (InternalCombinedList) req.getAttribute("oldList");

		if (oldList != null && oldList == list) {
			throw new RuntimeException("Old list should be a separate object from new list");
		}

		AlternativeListDescriptor existingList = callback.getAlternativeListHandlerModule().getAlternativeList(list.getListName());

		if (existingList != null) {

			if (!existingList.getProviderID().equals(callback.getProviderID())) {

				validationErrors.add(new ListNameAlreadyInUseValidationError(existingList, callback.getAlternativeListHandlerModule().getProvider(existingList.getProviderID())));

			} else if (oldList == null || !oldList.getListName().equals(list.getListName())) {

				validationErrors.add(new ListNameAlreadyInUseValidationError(existingList, callback));
			}
		}
	}

	@Override
	protected ForegroundModuleResponse filteredBeanAdded(InternalCombinedList list, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.addCombinedListEvent(callback.getEventListAddedMessage(), list, user);

		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/showcombined/" + list.getListID());
		return null;
	}

	@Override
	protected ForegroundModuleResponse filteredBeanUpdated(InternalCombinedList list, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.addCombinedListEvent(callback.getEventListUpdatedMessage(), list, user);

		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/showcombined/" + list.getListID());
		return null;
	}

	@Override
	protected ForegroundModuleResponse filteredBeanDeleted(InternalCombinedList list, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return super.filteredBeanDeleted(list, req, res, user, uriParser);
	}

}
