package com.nordicpeak.flowengine.internalalternativelist.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToMany;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeList;
import com.nordicpeak.flowengine.internalalternativelist.InternalListAccessInterface;

@Table(name = "internal_combined_lists")
@XMLElement(name = "AlternativeList")
public class InternalCombinedList extends BaseAlternativeList<InternalCombinedList, InternalCombinedListEvent> implements Serializable, InternalListAccessInterface {

	private static final long serialVersionUID = -842191226937409416L;

	public static final Field LISTS_RELATION = ReflectionUtils.getField(InternalCombinedList.class, "lists");
	public static final Field EVENTS_RELATION = ReflectionUtils.getField(InternalCombinedList.class, "events");

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement(name = "listID")
	private Integer combinedListID;

	@DAOManaged
	@OrderBy
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@OrderBy
	@WebPopulate(required = true)
	@XMLElement
	private InternalCombindedListOrdering ordering;

	@DAOManaged
	@ManyToMany(linkTable = "internal_combined_list_alternatives", indexColumn = "sortIndex")
	@XMLElement(fixCase = true)
	private List<InternalAlternativeList> lists;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<InternalCombinedListEvent> events;

	@XMLElement(fixCase = true)
	private List<InternalAlternative> alternatives;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean adminAccessControlEnabled;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "internal_combined_list_admin_groups", remoteValueColumnName = "groupID")
	@WebPopulate(paramName = "adminGroup")
	@NoDuplicates
	@XMLElement
	private List<Integer> adminGroups;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "internal_combined_list_admin_users", remoteValueColumnName = "userID")
	@WebPopulate(paramName = "adminUser")
	@NoDuplicates
	@XMLElement
	private List<Integer> adminUsers;

	public void setAlternatives(List<InternalAlternative> alternatives) {

		this.alternatives = alternatives;
	}

	@Override
	public Integer getListID() {

		return combinedListID;
	}

	public void setListID(Integer combinedListID) {

		this.combinedListID = combinedListID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public List<InternalAlternativeList> getLists() {

		return lists;
	}

	public void setLists(List<InternalAlternativeList> lists) {

		this.lists = lists;
	}

	@Override
	public String getListName() {

		return name;
	}

	@Override
	public List<InternalCombinedListEvent> getEvents() {

		return events;
	}

	@Override
	public void setEvents(List<InternalCombinedListEvent> events) {

		this.events = events;
	}

	public InternalCombindedListOrdering getOrdering() {

		return ordering;
	}

	public void setOrdering(InternalCombindedListOrdering ordering) {

		this.ordering = ordering;
	}

	@Override
	public boolean allowsAdminAccess() {

		return false;
	}

	@Override
	public boolean allowsUserAccess() {

		return false;
	}

	@Override
	public boolean allowsAnonymousAccess() {

		return false;
	}

	@Override
	public Collection<Integer> getAllowedGroupIDs() {

		return adminGroups;
	}

	@Override
	public Collection<Integer> getAllowedUserIDs() {

		return adminUsers;
	}

	@Override
	public boolean isAdminAccessControlEnabled() {

		return adminAccessControlEnabled;
	}

}
