package com.nordicpeak.flowengine.queries.internalcontactdetails.beans;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.SigningParty;
import com.nordicpeak.flowengine.interfaces.ColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.ContactFieldQueryInstance;
import com.nordicpeak.flowengine.interfaces.ContactQueryInstance;
import com.nordicpeak.flowengine.interfaces.MultiSignQueryinstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.UsersClosestManagerQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.internalcontactdetails.InternalContactDetailQueryProviderModule;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesUtils;
import com.nordicpeak.flowengine.queries.internaluserqueries.interfaces.CommonInternalUserQueryInstance;

@Table(name = "internal_contact_detail_query_instances")
@XMLElement
public class InternalContactDetailQueryInstance extends BaseQueryInstance implements ColumnExportableQueryInstance, MultiSignQueryinstance, UsersClosestManagerQueryInstance, CommonInternalUserQueryInstance, ContactFieldQueryInstance, ContactQueryInstance {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(InternalContactDetailQueryInstance.class, "query");
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private InternalContactDetailQuery query;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String username;

	@DAOManaged
	@XMLElement
	private String email;

	@DAOManaged
	@XMLElement
	private String managerCitizenID;

	@DAOManaged
	@XMLElement
	private String managerUsername;

	@DAOManaged
	@XMLElement
	private String managerFirstname;

	@DAOManaged
	@XMLElement
	private String managerLastname;

	@DAOManaged
	@XMLElement
	private String managerEmail;

	@DAOManaged
	@XMLElement
	private String managerMobilePhone;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<UserAttributeValue> userAttributeValues;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<ManagerAttributeValue> managerAttributeValues;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public InternalContactDetailQuery getQuery() {

		return query;
	}

	public void setQuery(InternalContactDetailQuery query) {

		this.query = query;
	}

	@Override
	public String getFirstname() {
		return firstname;
	}

	public void setFirstname(String firstname) {
		this.firstname = firstname;
	}

	@Override
	public String getLastname() {
		return lastname;
	}

	public void setLastname(String lastname) {
		this.lastname = lastname;
	}

	@Override
	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	@Override
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	@Override
	public List<UserAttributeValue> getUserAttributeValues() {
		return userAttributeValues;
	}

	public void setUserAttributeValues(List<UserAttributeValue> userAttributeValues) {
		this.userAttributeValues = userAttributeValues;
	}

	@Override
	public boolean isPopulated() {

		return !ObjectUtils.isNull(firstname, lastname, username, email, userAttributeValues);
	}
	
	public boolean isManagerPopulated() {

		return !ObjectUtils.isNull(managerFirstname, managerLastname, managerUsername, managerEmail, managerAttributeValues);
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		firstname = null;
		lastname = null;
		username = null;
		email = null;

		managerCitizenID = null;
		managerFirstname = null;
		managerLastname = null;
		managerUsername = null;
		managerEmail = null;
		managerMobilePhone = null;

		userAttributeValues = null;
		managerAttributeValues = null;

		if (query.isSetAsAttribute()) {

			resetFlowInstanceAttributes(attributeHandler);
		}

		super.reset(attributeHandler);
	}

	public void resetFlowInstanceAttributes(MutableAttributeHandler attributeHandler) {

		InternalUserQueriesUtils.resetFlowInstanceAttributes(query, this, attributeHandler);
	}

	public void setFlowInstanceAttributes(MutableAttributeHandler attributeHandler) {

		InternalUserQueriesUtils.setFlowInstanceAttributes(query, this, attributeHandler);
	}
	
	@Override
	public void forcedRepopulate(User poster, MutableAttributeHandler attributeHandler, QueryHandler queryHandler) {
		
		if (isPopulated()) {

			reset(attributeHandler);
			
			InternalContactDetailQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryInstanceDescriptor().getQueryDescriptor().getQueryTypeID(), InternalContactDetailQueryProviderModule.class);
			queryProvider.getUserDetails(this, poster);

			getQueryInstanceDescriptor().setPopulated(isPopulated());
			
			if (isPopulated() && getQuery().isSetAsAttribute()) {
				
				resetFlowInstanceAttributes(attributeHandler);
				setFlowInstanceAttributes(attributeHandler);
			}
		}
	}

	@Override
	public String toString() {

		return this.getClass().getName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		InternalUserQueriesUtils.toExportXML(query, this, element, doc);

		return element;
	}

	@Override
	public List<String> getColumnLabels(QueryHandler queryHandler) {

		if (!query.hasFields()) {
			
			return Collections.emptyList();
		}

		InternalContactDetailQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryInstanceDescriptor().getQueryDescriptor().getQueryTypeID(), InternalContactDetailQueryProviderModule.class);

		return InternalUserQueriesUtils.getColumnLabels(query, this, queryProvider);
	}

	@Override
	public List<String> getColumnValues() {

		if (!query.hasFields()) {

			return Collections.emptyList();
		}

		return InternalUserQueriesUtils.getColumnValues(query, this);
	}

	@Override
	public Element toXML(Document doc) {

		Element queryInstanceElement = super.toXML(doc);

		XMLUtils.appendNewElement(doc, queryInstanceElement, "Populated", isPopulated());

		return queryInstanceElement;
	}

	@Override
	public String getManagerCitizenID() {
		return managerCitizenID;
	}

	public void setManagerCitizenID(String managerCitizenID) {
		this.managerCitizenID = managerCitizenID;
	}

	@Override
	public String getManagerFirstname() {
		return managerFirstname;
	}

	public void setManagerFirstname(String managerFirstname) {
		this.managerFirstname = managerFirstname;
	}

	@Override
	public String getManagerLastname() {
		return managerLastname;
	}

	public void setManagerLastname(String managerLastname) {
		this.managerLastname = managerLastname;
	}

	@Override
	public String getManagerUsername() {
		return managerUsername;
	}

	public void setManagerUsername(String managerUsername) {
		this.managerUsername = managerUsername;
	}

	@Override
	public String getManagerEmail() {
		return managerEmail;
	}

	public void setManagerEmail(String managerEmail) {
		this.managerEmail = managerEmail;
	}

	@Override
	public String getManagerMobilePhone() {
		return managerMobilePhone;
	}

	public void setManagerMobilePhone(String managerMobilePhone) {
		this.managerMobilePhone = managerMobilePhone;
	}

	@Override
	public List<ManagerAttributeValue> getManagerAttributeValues() {
		return managerAttributeValues;
	}

	public void setManagerAttributeValues(List<ManagerAttributeValue> managerAttributeValues) {
		this.managerAttributeValues = managerAttributeValues;
	}

	@Override
	public List<? extends SigningParty> getSigningParties() {

		if (query.isRequireManagerSigning()) {
			return Collections.singletonList(new SigningParty(managerFirstname, managerLastname, managerEmail, managerMobilePhone, managerCitizenID, false));
		}

		return null;
	}

	@Override
	public String getCitizenIdentifier() {
		return null;
	}

	@Override
	public User getClosestManager(QueryHandler queryHandler) {

		InternalContactDetailQueryProviderModule queryProviderModule = queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), InternalContactDetailQueryProviderModule.class);
		
		return queryProviderModule.getUserHandler().getUserByUsername(managerUsername, false, true);
	}

	@Override
	public String getContactFieldValue(String contactFieldName) throws IllegalArgumentException, IllegalAccessException {

		if (contactFieldName.equals("firstname")) {

			return firstname;
		}

		if (contactFieldName.equals("lastname")) {

			return lastname;
		}

		if (contactFieldName.equals("username")) {

			return username;
		}

		if (contactFieldName.equals("email")) {

			return email;
		}

		UserAttributeValue userAttributeValue = CollectionUtils.find(userAttributeValues, v -> v.getName().equals(contactFieldName));

		if (userAttributeValue != null) {

			return userAttributeValue.getValue();
		}

		ManagerAttributeValue managerAttributeValue = CollectionUtils.find(managerAttributeValues, v -> v.getName().equals(contactFieldName));

		if (managerAttributeValue != null) {

			return managerAttributeValue.getValue();
		}

		return null;
	}

	@Override
	public Contact getContact() {
		
		if (isPopulated()) {
			
			Contact contact = new Contact();
			
			contact.setFirstname(getFirstname());
			contact.setLastname(getLastname());
			contact.setEmail(getEmail());
			
			contact.setContactBySMS(false);
			contact.setContactByEmail(true);
			
			return contact;
		}
		
		return null;
	}

	@Override
	public boolean updatedPoster() {
		
		return true;
	}
}
