package com.nordicpeak.flowengine.queries.internaluserqueries;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

public class InternalUserQueriesSelectedAttribute extends GeneratedElementable implements XMLParserPopulateable, Serializable {

	private static final long serialVersionUID = -5407922522706409574L;

	@DAOManaged
	@Key
	@XMLElement
	private String name;

	@DAOManaged
	@XMLElement
	private String displayName;
	
	@DAOManaged
	@OrderBy
	@XMLElement
	private Integer sortIndex;
	
	
	public InternalUserQueriesSelectedAttribute() {}

	public InternalUserQueriesSelectedAttribute(String name, String displayName, Integer sortOrder) {

		if (name == null) {

			throw new NullPointerException("name cannot be null");
		}

		this.name = name;

		if (displayName == null) {

			this.displayName = name;

		} else {

			this.displayName = displayName;
		}
		
		this.sortIndex = sortOrder;
	}

	public InternalUserQueriesSelectedAttribute(InternalUserQueriesAttributeDescriptor attributeDescriptor, Integer sortOrder) {

		this(attributeDescriptor.getName(), attributeDescriptor.getDisplayName(), sortOrder);
	}
	
	public String getDisplayName() {

		return displayName;
	}

	public String getName() {

		return name;
	}

	public Integer getSortIndex() {
		
		return sortIndex;
	}
	
	public void setSortIndex(Integer sortIndex) {
	
		this.sortIndex = sortIndex;
	}

	@Override
	public String toString() {

		return name + " (display name: " + displayName + ", sort order: " + sortIndex + ")";
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		name = XMLValidationUtils.validateParameter("name", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		displayName = XMLValidationUtils.validateParameter("displayName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		sortIndex = XMLValidationUtils.validateParameter("sortIndex", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}
}
