package com.nordicpeak.flowengine.queries.internaluserquery;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.templates.TemplateUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.InternalUserQuery;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.SelectedManagerAttribute;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.SelectedUserAttribute;

public class InternalUserQueryCRUD extends BaseQueryCRUD<InternalUserQuery, InternalUserQueryProviderModule> {

	public InternalUserQueryCRUD(AnnotatedDAOWrapper<InternalUserQuery, Integer> queryDAO, BeanRequestPopulator<InternalUserQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, InternalUserQueryProviderModule callback) {

		super(InternalUserQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
	}

	@Override
	protected void appendBean(InternalUserQuery bean, Element targetElement, Document doc, User user) {

		TemplateUtils.setTemplatedFields(bean, callback);

		super.appendBean(bean, targetElement, doc, user);
	}

	@Override
	protected void appendUpdateFormData(InternalUserQuery query, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(query, doc, updateTypeElement, user, req, uriParser);

		if (callback.getAttributeDescriptors() != null) {

			XMLUtils.append(doc, updateTypeElement, "AvailiableAttributes", callback.getAttributeDescriptors());
		}

		if (query.getGroupIDs() != null) {
			XMLUtils.append(doc, updateTypeElement, "Groups", callback.getGroupHandler().getGroups(query.getGroupIDs(), true));
		}

		if (callback.isEnableClosestManager()) {
			XMLUtils.appendNewElement(doc, updateTypeElement, "ClosestManagerEnabled");
		}
	}

	@Override
	protected InternalUserQuery populateFromUpdateRequest(InternalUserQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		InternalUserQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		TemplateUtils.clearUnchangedTemplatedFields(query, callback);

		List<ValidationError> validationErrors = new ArrayList<>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		if (!callback.isEnableClosestManager()) {

			query.setShowManagerInfo(false);
			query.setRequireManagerSigning(false);
			query.setSelectedManagerAttributes(null);
			query.setLimitSearchToMyEmployees(false);
		}

		if (callback.getAttributeDescriptors() != null) {

			List<SelectedUserAttribute> selectedAttributes = new ArrayList<>();

			Integer sortIndex = 1;

			for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {

				if ("true".equals(req.getParameter("attribute-" + attributeDescriptor.getName()))) {

					selectedAttributes.add(new SelectedUserAttribute(attributeDescriptor, sortIndex++));
				}
			}

			query.setSelectedUserAttributes(selectedAttributes);

			if (callback.isEnableClosestManager()) {

				sortIndex = 1;

				List<SelectedManagerAttribute> selectedManagerAttributes = new ArrayList<>();

				for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {

					if ("true".equals(req.getParameter("managerattribute-" + attributeDescriptor.getName()))) {

						selectedManagerAttributes.add(new SelectedManagerAttribute(attributeDescriptor, sortIndex++));
					}
				}

				query.setSelectedManagerAttributes(selectedManagerAttributes);
			}
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

}
