<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	
	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
		
		<div>
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="InternalUserQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="query{ContactDetailQueryInstance/QueryInstanceDescriptor/QueryDescriptor/queryID}"/>
			
			<h2>
				<xsl:value-of select="InternalUserQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
			</h2>
			
			<xsl:if test="Description">
				
				<div class="query-description">
					<xsl:choose>
						<xsl:when test="isHTMLDescription = 'true'">
							<xsl:value-of select="Description" disable-output-escaping="yes"/>
						</xsl:when>
						<xsl:otherwise>
							<p>
								<xsl:value-of select="Description" disable-output-escaping="yes"/>
							</p>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</xsl:if>
			
			<xsl:variable name="query" select="InternalUserQueryInstance/InternalUserQuery" />
			
			<div class="full display-table bigmarginbottom clearboth">
				
				<xsl:if test="$query/showName = 'true'">
					
					<div class="floatleft fifty bigmarginbottom">
						<strong><xsl:value-of select="$i18n.FirstnameAndLastname" /></strong>
						<br/>
						
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="InternalUserQueryInstance/firstname" />
						</xsl:call-template>
						
						<xsl:text>&#160;</xsl:text>
						
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="InternalUserQueryInstance/lastname" />
						</xsl:call-template>
					</div>
					
				</xsl:if>
				
				<xsl:if test="$query/showUsername = 'true'">
				
					<div class="floatleft fifty bigmarginbottom">
						<strong><xsl:value-of select="$i18n.Username" /></strong>
						<br/>
						
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="InternalUserQueryInstance/username" />
						</xsl:call-template>
					</div>
				
				</xsl:if>
				
				<xsl:if test="$query/showEmail = 'true'">
				
					<div class="floatleft fifty bigmarginbottom">
						<strong><xsl:value-of select="$i18n.Email" /></strong>
						<br/>
						
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="InternalUserQueryInstance/email" />
						</xsl:call-template>
					</div>
					
				</xsl:if>
				
				<xsl:for-each select="$query/SelectedUserAttributes/SelectedUserAttribute">
					
					<div class="floatleft fifty bigmarginbottom">
						<strong><xsl:value-of select="displayName" /></strong>
						<br/>
						
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="../../../UserAttributeValues/UserAttributeValue[Name = current()/name]/Value" />
						</xsl:call-template>
					</div>
					
				</xsl:for-each>
				
			</div>
			
			<xsl:if test="$query/showManagerInfo = 'true'">
				
				<div class="full display-table bigmarginbottom">
				
				<div class="floatleft full bigmarginbottom">
					<strong><xsl:value-of select="$i18n.ManagerInfo" /></strong>
				</div>
					
					<xsl:if test="$query/showManagerName = 'true'">
						
						<div class="floatleft fifty bigmarginbottom">
							<strong><xsl:value-of select="$i18n.FirstnameAndLastname" /></strong>
							<br/>
							
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="InternalUserQueryInstance/managerFirstname" />
							</xsl:call-template>
							
							<xsl:text>&#160;</xsl:text>
							
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="InternalUserQueryInstance/managerLastname" />
							</xsl:call-template>
						</div>
						
					</xsl:if>
					
					<xsl:if test="$query/showManagerUsername = 'true'">
					
						<div class="floatleft fifty bigmarginbottom">
							<strong><xsl:value-of select="$i18n.Username" /></strong>
							<br/>
							
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="InternalUserQueryInstance/managerUsername" />
							</xsl:call-template>
						</div>
					
					</xsl:if>
					
					<xsl:if test="$query/showManagerEmail = 'true'">
					
						<div class="floatleft fifty bigmarginbottom">
							<strong><xsl:value-of select="$i18n.Email" /></strong>
							<br/>
							
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="InternalUserQueryInstance/managerEmail" />
							</xsl:call-template>
						</div>
						
					</xsl:if>
					
					<xsl:for-each select="$query/SelectedManagerAttributes/SelectedManagerAttribute">
						
						<div class="floatleft fifty bigmarginbottom">
							<strong><xsl:value-of select="displayName" /></strong>
							<br/>
							
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="../../../ManagerAttributeValues/ManagerAttributeValue[Name = current()/name]/Value" />
							</xsl:call-template>
						</div>
						
					</xsl:for-each>
					
				</div>
				
			</xsl:if>
			
		</div>
		
	</xsl:template>

	<xsl:template name="printValue">
		
		<xsl:param name="value" />
		
		<xsl:choose>
			<xsl:when test="$value">
				<xsl:value-of select="$value"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>-</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>