package com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.interfaces.ExportAlternativeID;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "list_search_queries")
@XMLElement
public class ListSearchQuery extends FixedAlternativesBaseQuery implements AlternativeListQuery, ExportAlternativeID {

	private static final long serialVersionUID = 577463825622564529L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer minChecked;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer maxChecked;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer maxHits;

	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String alternativeListName;

	@WebPopulate(maxLength = 255)
	@DAOManaged
	@XMLElement
	private String placeholderText;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showAlternativesOnEmptySearch;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean exportAlternativeID;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ListSearchQueryInstance> instances;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showExactMatchesFirst;

	private AlternativeListInstance alternativeList;

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		if (alternativeList != null) {

			return alternativeList.getAlternatives();
		}

		return null;
	}

	@Override
	public String getXSDTypeName() {

		return "ListSearchQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		if (maxChecked != null) {

			if (minChecked != null) {

				toXSD(doc, minChecked, maxChecked);

			} else {

				toXSD(doc, maxChecked);
			}

		} else {

			if (alternativeList != null) {

				if (minChecked != null) {

					toXSD(doc, minChecked, alternativeList.getAlternatives().size());

				} else {

					toXSD(doc, alternativeList.getAlternatives().size());
				}

			} else {

				toXSD(doc, 1);

			}
		}
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		placeholderText = XMLValidationUtils.validateParameter("placeholderText", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		minChecked = XMLValidationUtils.validateParameter("minChecked", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		maxChecked = XMLValidationUtils.validateParameter("maxChecked", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		maxHits = XMLValidationUtils.validateParameter("maxHits", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		
		alternativeListName = XMLValidationUtils.validateParameter("alternativeListName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		showAlternativesOnEmptySearch = xmlParser.getPrimitiveBoolean("showAlternativesOnEmptySearch");
		
		showExactMatchesFirst = xmlParser.getPrimitiveBoolean("showExactMatchesFirst");

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		exportAlternativeID  = xmlParser.getPrimitiveBoolean("exportAlternativeID");

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (alternativeList != null) {

			//TODO don't append alternatives on export
			XMLUtils.append(doc, element, "Alternatives", alternativeList.getAlternatives());
		}

		return element;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public Integer getMinChecked() {

		return minChecked;
	}

	public void setMinChecked(Integer minChecked) {

		this.minChecked = minChecked;
	}

	public Integer getMaxChecked() {

		return maxChecked;
	}

	public void setMaxChecked(Integer maxChecked) {

		this.maxChecked = maxChecked;
	}

	public List<ListSearchQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<ListSearchQueryInstance> instances) {

		this.instances = instances;
	}

	public AlternativeListInstance getAlternativeList() {

		return alternativeList;
	}

	public void setAlternativeList(AlternativeListInstance alternativeList) {

		this.alternativeList = alternativeList;
	}

	@Override
	public String getAlternativeListName() {

		return alternativeListName;
	}

	@Override
	public String getFreeTextAlternative() {

		return null;
	}

	public String getPlaceholderText() {

		return placeholderText;
	}

	public void setPlaceholderText(String placeholderText) {

		this.placeholderText = placeholderText;
	}

	public boolean isShowAlternativesOnEmptySearch() {

		return showAlternativesOnEmptySearch;
	}

	public void setShowAlternativesOnEmptySearch(boolean showAlternativesOnEmptySearch) {

		this.showAlternativesOnEmptySearch = showAlternativesOnEmptySearch;
	}

	
	public boolean isShowExactMatchesFirst() {
	
		return showExactMatchesFirst;
	}

	
	public void setShowExactMatchesFirst(boolean showExactMatchesFirst) {
	
		this.showExactMatchesFirst = showExactMatchesFirst;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	
	public Integer getMaxHits() {
	
		return maxHits;
	}

	
	public void setMaxHits(Integer maxHits) {
	
		this.maxHits = maxHits;
	}

	@Override
	public boolean isExportAlternativeID() {
		
		return exportAlternativeID;
	}

	
	public void setExportAlternativeID(boolean exportAlternativeID) {
	
		this.exportAlternativeID = exportAlternativeID;
	}


}
