<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/common/js/queryadmin.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<div id="ManualSignQueryProvider" class="contentitem">
		
			<xsl:apply-templates select="validationError"/>
			<xsl:apply-templates select="ShowManualSignQuery"/>
			<xsl:apply-templates select="UpdateManualSignQuery"/>
			<xsl:apply-templates select="AddManualSignField"/>
			<xsl:apply-templates select="UpdateManualSignField"/>
			<xsl:apply-templates select="SortManualSignFields"/>
		
		</div>
		
	</xsl:template>
		
	<xsl:template match="ShowManualSignQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateQuery" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="ManualSignQuery/QueryDescriptor/name" /></h1>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.BaseInfo"/></legend>
			
			<div class="floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatequery/{ManualSignQuery/queryID}" title="{$i18n.UpdateBaseInformation}">
					<img class="alignbottom" src="{$imagePath}/pen.png"/>
				</a>
			</div>
			
			<xsl:call-template name="createCommonShowFields">
				<xsl:with-param name="element" select="ManualSignQuery" />
			</xsl:call-template>
			
			<xsl:variable name="layout" select="ManualSignQuery/layout" />
			
			<div class="floatleft full bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.Layout" /></label>
				<div class="floatleft full">
					<xsl:value-of select="FieldLayout[value = $layout]/name" />
			    </div>
			</div>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.ManualSignFields"/></legend>
			
			<xsl:if test="ManualSignQuery/Fields/ManualSignField">
				
				<div class="floatleft full">
				
					<xsl:apply-templates select="ManualSignQuery/Fields/ManualSignField" mode="list" />
			
				</div>
			
			</xsl:if>
			
			<xsl:if test="ManualSignQuery/Fields/ManualSignField">
			
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sortmanualsignfields/{ManualSignQuery/queryID}" title="{$i18n.SortManualSignFields.Title}">
						<xsl:value-of select="$i18n.SortManualSignFields.Title"/>
						<img class="alignbottom marginleft" src="{$commonImagePath}/move.png"/>
					</a>
				</div>
				
			</xsl:if>
			
			<div class="clearboth floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addmanualsignfield/{ManualSignQuery/queryID}" title="{$i18n.AddManualSignField}">
					<xsl:value-of select="$i18n.AddManualSignField"/>
					<img class="alignbottom marginleft" src="{$commonImagePath}/add.png"/>
				</a>
			</div>
			
		</fieldset>
		
		<div class="floatright margintop clearboth">
			<input type="button" value="{$i18n.Done}" onclick="window.location = '{showFlowURL}'" />
		</div>
		
	</xsl:template>
		
	<xsl:template match="ManualSignField" mode="list">
				
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<div class="padding floatleft">
				<div class="marginleft"><b><xsl:value-of select="label" /></b></div>
			</div>
			<div class="padding floatright">
				<div class="floatright marginleft marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletemanualsignfield/{manualSignFieldID}" onclick="return confirm('{$i18n.DeleteManualSignField.Confirm}: {label}?');" title="{$i18n.DeleteManualSignField}: {label}">
						<img src="{$imagePath}/delete.png"/>
					</a>
				</div>
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatemanualsignfield/{manualSignFieldID}" title="{$i18n.UpdateManualSignField}: {label}">
						<img src="{$imagePath}/pen.png"/>
					</a>
				</div>
			</div>
			
		</div>
				
	</xsl:template>
	
	<xsl:template match="AddManualSignField">
	
		<h1><xsl:value-of select="$i18n.AddManualSignField" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
	
		<form id="addManualSignFieldForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
			
			<xsl:call-template name="showManualSignFieldForm" />
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template match="UpdateManualSignField">
	
		<h1><xsl:value-of select="$i18n.UpdateManualSignField" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="ManualSignField/label" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="udpateManualSignFieldForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="showManualSignFieldForm">
				<xsl:with-param name="element" select="ManualSignField" />
			</xsl:call-template>
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template name="showManualSignFieldForm">
		
		<xsl:param name="element" select="null" />
		
		<div class="floatleft full bigmarginbottom">
			<label for="label" class="floatleft clearboth"><xsl:value-of select="$i18n.Label" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'label'"/>
					<xsl:with-param name="name" select="'label'"/>
					<xsl:with-param name="title" select="$i18n.Label"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		    </div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="FormatValidator">
		
		<xsl:call-template name="createHiddenField">
			<xsl:with-param name="id" select="concat('validatorMessage-', formatValidatorID)" />
			<xsl:with-param name="name" select="concat('validatorMessage-', formatValidatorID)" />
			<xsl:with-param name="value" select="validationMessage" />
		</xsl:call-template>
		
	</xsl:template>
	
	<xsl:template match="UpdateManualSignQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateBaseInformation" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="updateManualSignQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="ManualSignQuery" />
				<xsl:with-param name="flowTypeID" select="FlowType/flowTypeID" />
			</xsl:call-template>
			
			<div class="floatleft full bigmarginbottom">
				<label for="layout" class="floatleft clearboth"><xsl:value-of select="$i18n.Layout" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'layout'"/>
						<xsl:with-param name="name" select="'layout'"/>
						<xsl:with-param name="valueElementName" select="'value'" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="element" select="FieldLayout" />
						<xsl:with-param name="selectedValue" select="ManualSignQuery/layout" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hidePrintInfoText'"/>
					<xsl:with-param name="name" select="'hidePrintInfoText'"/>
					<xsl:with-param name="title" select="$i18n.HidePrintInfoText"/>
					<xsl:with-param name="element" select="ManualSignQuery" />
					</xsl:call-template>
					
					<label for="hidePrintInfoText"><xsl:value-of select="$i18n.HidePrintInfoText" /></label>
				</div>
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
		
	</xsl:template>

	<xsl:template match="SortManualSignFields">
		
		<h1><xsl:value-of select="$i18n.SortManualSignFields" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="ManualSignQuery/QueryDescriptor/name" /></h1>
		
		<form id="sortManualSignFieldsForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<div class="floatleft full sortable">
							
				<xsl:apply-templates select="ManualSignQuery/Fields/ManualSignField" mode="sort" />
							
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>
		
	</xsl:template>
	
	<xsl:template match="ManualSignField" mode="sort">
	
		<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
	
		<div id="manualsignfield_{manualSignFieldID}" class="floatleft hover border full marginbottom lightbackground cursor-move border-radius">
			<div class="padding">
				<img class="vertical-align-middle marginright" src="{$commonImagePath}/move.png" title="{$i18n.MoveAlternative}"/>
				<xsl:value-of select="label" />
				<xsl:call-template name="createHiddenField">
					<xsl:with-param name="id" select="concat('sortorder_', manualSignFieldID)" />
					<xsl:with-param name="name" select="concat('sortorder_', manualSignFieldID)" />
					<xsl:with-param name="value" select="sortIndex" />
					<xsl:with-param name="class" select="'sortorder'" />
					<xsl:with-param name="requestparameters" select="//requestparameters" />
				</xsl:call-template>
			</div>
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'UpdateFailedManualSignQueryNotFound']">
		
		<p class="error">
			<xsl:value-of select="$i18n.ManualSignQueryNotFound" />
		</p>
		
	</xsl:template>

	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />
	
		<xsl:choose>
			<xsl:when test="$fieldName = 'label'">
				<xsl:value-of select="$i18n.label" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>