<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="ManualSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ManualSignQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ManualSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
				</div>
				
				<xsl:call-template name="form"/>
				
			</article>
			
		</div>
		
	</xsl:template>		
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', ManualSignQueryInstance/ManualSignQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="ManualSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerror" class="validationerrors" />
			</xsl:if>
	
			<article>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ManualSignQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ManualSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="ManualSignQueryInstance/ManualSignQuery/helpText">
						<xsl:apply-templates select="ManualSignQueryInstance/ManualSignQuery/helpText" />
					</xsl:if>
				
				</div>
				
				<xsl:call-template name="form"/>
				
			</article>
			
		</div>
		
	</xsl:template>
	
	<xsl:template name="form">
	
		<xsl:if test="ManualSignQueryInstance/ManualSignQuery/description">
			<span class="italic">
				<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
				<xsl:value-of select="ManualSignQueryInstance/ManualSignQuery/description" disable-output-escaping="yes" />
			</span>
		</xsl:if>
		
		<div class="clearboth"/>
		
		<xsl:if test="ManualSignQueryInstance/ManualSignQuery/hidePrintInfoText = 'false'">
			<p class="tiny">
				<xsl:value-of select="$i18n.FilledAfterPrint"/>
			</p>
		</xsl:if>
		
		<fieldset>
			<xsl:apply-templates select="ManualSignQueryInstance/ManualSignQuery/Fields/ManualSignField" />
		</fieldset>
	
	</xsl:template>
	
	<xsl:template match="ManualSignField">

		<xsl:variable name="odd" select="(position() mod 2) = 0" />

		<div class="split break">
			
			<xsl:if test="../../layout = 'FLOAT'">
				<xsl:attribute name="class">
					<xsl:text>split </xsl:text>
					<xsl:if test="$odd">odd </xsl:if>
				</xsl:attribute>
			</xsl:if>
			
			<label>
				<xsl:if test="required = 'true'">
					<xsl:attribute name="class">required</xsl:attribute>
				</xsl:if>
				<xsl:value-of select="label"/>
			</label>
			
			<input type="text" size="50" title="{label}" disabled="true"/>
	
		</div>	
	
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<i data-icon-after="!" title="{$i18n.UnknownValidationError}"></i>
		
	</xsl:template>
	
</xsl:stylesheet>