package com.nordicpeak.flowengine.multisigninghandlers;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.security.InvalidParameterException;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.MutableUser;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.exceptions.UnableToUpdateUserException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.core.utils.ViewFragmentUtils;
import se.unlogic.hierarchy.core.utils.crud.FragmentLinkScriptFilter;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.standardutils.bool.BooleanUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.collections.ReverseListIterator;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.DBUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.hash.HashAlgorithms;
import se.unlogic.standardutils.hash.HashUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.http.enums.ContentDisposition;

import com.nordicpeak.flowengine.BaseFlowModule;
import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.UserFlowInstanceModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.SigningParty;
import com.nordicpeak.flowengine.beans.SimpleSigningRequest;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.MultiSigningCanceledEvent;
import com.nordicpeak.flowengine.events.MultiSigningInitiatedEvent;
import com.nordicpeak.flowengine.events.MultiSigningPartySignedEvent;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.evaluationprovider.EvaluationProviderException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryinstance.QueryRequestException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryInstanceNotFoundInQueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderErrorException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderNotFoundException;
import com.nordicpeak.flowengine.interfaces.GenericSigningProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowEngineInterface;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstanceEvent;
import com.nordicpeak.flowengine.interfaces.ListFlowInstancesViewFragmentExtensionProvider;
import com.nordicpeak.flowengine.interfaces.MultiSigningHandler;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.SigningResponse;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.managers.ManagerResponse;
import com.nordicpeak.flowengine.multisigninghandlers.exceptions.UnableToCreateSigningSessionException;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;
import com.nordicpeak.flowengine.utils.MultiSignUtils;

public class MultiSigningHandler2Module extends AnnotatedForegroundModule implements MultiSigningHandler, ViewFragmentModule<ForegroundModuleDescriptor>, ListFlowInstancesViewFragmentExtensionProvider, ImmutableFlowEngineInterface {

	private static final String MESSAGE_SEQUENTIAL_SIGNING_BLOCK = "SequentialSigningBlock";
	private static final String MESSAGE_SIGNING_PARTY_NOT_FOUND_OR_INVALID = "SigningPartyNotFound";
	private static final String MESSAGE_WRONG_STATUS_CONTENT_TYPE = "WrongStatusContentType";

	public static final Field[] FLOW_INSTANCE_RELATIONS = { FlowInstance.OWNERS_RELATION, FlowInstance.ATTRIBUTES_RELATION, FlowInstance.EVENTS_RELATION, FlowInstanceEvent.ATTRIBUTES_RELATION, FlowInstance.FLOW_RELATION, FlowInstance.STATUS_RELATION, Flow.FLOW_TYPE_RELATION, Flow.FLOW_FAMILY_RELATION };

	private static final String GET_SIGNING_SESSIONS_BY_CITIZEN_IDENTIFIER = "SELECT s.* FROM multi_signing_sessions s INNER JOIN multi_signing_parties p ON s.sessionID = p.sessionID WHERE p.signedDate IS NULL AND p.identifier = ?";

	private static final List<EventType> SIGNING_EVENT_TYPES = CollectionUtils.getList(EventType.SIGNED, EventType.SIGNING_SKIPPED);
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "The server URL", description = "The URL to this server including protocol")
	protected String serverURL;

	@XSLVariable(prefix = "java.")
	protected String signingMessage = "You will sign $flow.name with application number $flowInstance.flowInstanceID. The application has the following unique key: $hash";

	@XSLVariable(prefix = "java.")
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Sign message", description = "The message displayed above the sign form (available tags: $poster, $flowInstanceID, $flow.name)", required = true)
	protected String signMessage = "Not set";

	@XSLVariable(prefix = "java.")
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signed message", description = "The message displayed above the signed form (available tags: $poster, $flowInstanceID, $flow.name, $signed)", required = true)
	protected String signedMessage = "Not set";

	@XSLVariable(prefix = "java.")
	protected String anonymousUserName = "Anonymous user";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Hide citizen indentifier", description = "If toggled the citizen identifier will not be displayed in the sign form for any flow types.")
	protected boolean hideCitizenIdentifier = false;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Flow types with hidden citizen indentifier", description = "Citizen identifier will not be displayed in the sign form for these flow types.", formatValidator = NonNegativeStringIntegerValidator.class)
	protected List<Integer> flowTypesWithHiddenCitizenIdentifier;

	@InstanceManagerDependency
	protected PDFProvider pdfProvider;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	protected FlowBrowserModule browserModule;

	@InstanceManagerDependency(required = true)
	protected GenericSigningProvider genericSigningProvider;

	@InstanceManagerDependency
	protected StandardFlowNotificationHandler standardFlowNotificationHandler;

	protected UserFlowInstanceModule userFlowInstanceModule;

	protected AnnotatedDAO<FlowInstanceEvent> flowInstanceEventDAO;
	protected AnnotatedDAO<FlowInstance> flowInstanceDAO;
	protected AnnotatedDAO<SigningSession> signingSessionDAO;
	protected AnnotatedDAO<SigningSessionParty> signingPartyDAO;

	protected QueryParameterFactory<FlowInstance, Integer> flowInstanceIDParamFactory;
	protected QueryParameterFactory<SigningSession, Integer> signingSessionIDParamFactory;
	protected QueryParameterFactory<SigningSession, Integer> signingSessionFlowInstanceIDParamFactory;
	protected QueryParameterFactory<FlowInstanceEvent, FlowInstance> flowInstanceEventFlowInstanceParamFactory;
	protected QueryParameterFactory<FlowInstanceEvent, EventType> flowInstanceEventTypeParamFactory;

	protected ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		if (!systemInterface.getInstanceHandler().addInstance(MultiSigningHandler.class, this)) {

			throw new RuntimeException("Unable to register module " + moduleDescriptor + " in global instance handler using key " + MultiSigningHandler.class.getSimpleName() + ", another instance is already registered using this key.");
		}

		if (!systemInterface.getInstanceHandler().addInstance(MultiSigningHandler2Module.class, this)) {

			throw new RuntimeException("Unable to register module " + moduleDescriptor + " in global instance handler using key " + MultiSigningHandler2Module.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, MultiSigningHandler2Module.class.getName(), new XMLDBScriptProvider(MultiSigningHandler2Module.class.getResourceAsStream("DB script2.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		signingSessionDAO = daoFactory.getDAO(SigningSession.class);
		signingPartyDAO = daoFactory.getDAO(SigningSessionParty.class);

		signingSessionIDParamFactory = signingSessionDAO.getParamFactory("sessionID", Integer.class);
		signingSessionFlowInstanceIDParamFactory = signingSessionDAO.getParamFactory("flowInstanceID", Integer.class);

		FlowEngineDAOFactory flowEngineDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		flowInstanceDAO = flowEngineDAOFactory.getFlowInstanceDAO();
		flowInstanceEventDAO = flowEngineDAOFactory.getFlowInstanceEventDAO();

		flowInstanceIDParamFactory = flowInstanceDAO.getParamFactory("flowInstanceID", Integer.class);
		flowInstanceEventFlowInstanceParamFactory = flowInstanceEventDAO.getParamFactory("flowInstance", FlowInstance.class);
		flowInstanceEventTypeParamFactory = flowInstanceEventDAO.getParamFactory("eventType", EventType.class);
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(MultiSigningHandler.class, this);
		systemInterface.getInstanceHandler().removeInstance(MultiSigningHandler2Module.class, this);

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		super.unload();
	}

	@EventListener(channel = FlowInstance.class, priority = 10)
	public void processEvent(MultiSigningInitiatedEvent event, EventSource source) throws SQLException, UnableToCreateSigningSessionException {

		FlowInstanceManager instanceManager = event.getFlowInstanceManager();
		FlowInstanceEvent saveEvent = event.getEvent();

		if (instanceManager.getFlowInstance().getFlow().isSkipPosterSigning()) {

			try {
				Map<String, String> extraElements = new HashMap<>();
				extraElements.put("Signing", "true");

				pdfProvider.createTemporaryPDF(instanceManager, browserModule.getSiteProfile(instanceManager), saveEvent.getPoster(), extraElements);

				if (!pdfProvider.saveTemporaryPDF(instanceManager, saveEvent)) {

					log.error("Unable to find temporary PDF for flow instance " + instanceManager);
				}

			} catch (Exception e) {

				log.error("Error generating temporary PDF for flow instance " + instanceManager, e);
			}
		}

		createSigningSession(instanceManager, saveEvent, instanceManager.getFlowInstance().getFlow().usesSequentialSigning(), null);
	}

	public SigningSession createSigningSession(FlowInstanceManager instanceManager, FlowInstanceEvent pdfEvent, boolean sequentialSigning, TransactionHandler transactionHandler) throws SQLException, UnableToCreateSigningSessionException {

		Set<SigningParty> signingParties = MultiSignUtils.getSigningParties(instanceManager);

		if (signingParties == null) {

			throw new UnableToCreateSigningSessionException("No signing parties found for flow instance " + instanceManager);
		}

		List<SigningSessionParty> sessionParties = new ArrayList<>(signingParties.size());

		for (SigningParty signingParty : signingParties) {

			if (!signingParty.isUnsecure() && signingParty.getSocialSecurityNumber() == null) {

				log.error("Signing party is secure but has no citizenID: " + signingParty + ", " + instanceManager);
			}

			sessionParties.add(new SigningSessionParty(signingParty));
		}

		SigningSession oldSigningSession = getSigningSession(instanceManager.getFlowInstanceID());

		if (oldSigningSession != null) {

			if (!oldSigningSession.isFullySigned()) {

				systemInterface.getEventHandler().sendEvent(FlowInstance.class, new MultiSigningCanceledEvent(instanceManager.getFlowInstance(), oldSigningSession.getSigningParties(), null, pdfEvent.getPoster()), EventTarget.ALL);
			}

			signingSessionDAO.delete(oldSigningSession);
		}

		SigningSession signingSession = new SigningSession();
		signingSession.setFlowInstanceID(instanceManager.getFlowInstanceID());
		signingSession.setSigningSessionParties(sessionParties);
		signingSession.setAdded(TimeUtils.getCurrentTimestamp());
		signingSession.setSequential(sequentialSigning);
		signingSession.setPdfEventID(pdfEvent.getEventID());

		if (transactionHandler != null) {

			signingSessionDAO.add(signingSession, transactionHandler, null);

		} else {

			signingSessionDAO.add(signingSession);
		}

		try {
			pdfEvent.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION, signingSession.getSessionID());
			pdfEvent.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT, Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT_SIGNING_PDF);
			pdfEvent.setFlowInstance((FlowInstance) instanceManager.getFlowInstance());

			flowInstanceEventDAO.update(pdfEvent, new RelationQuery(FlowInstanceEvent.ATTRIBUTES_RELATION), 10);

		} finally {
			pdfEvent.setFlowInstance(null);
		}

		log.info("Created signing session " + signingSession + " for flow instance " + instanceManager.getFlowInstance());

		if (sequentialSigning && standardFlowNotificationHandler != null) {

			SigningSessionParty nextParty = signingSession.getSigningSessionParties().get(0);
			standardFlowNotificationHandler.sendSigningPartyNotifications(instanceManager.getFlowInstance(), nextParty.getSigningParty());
		}

		return signingSession;
	}

	@Override
	public ViewFragment getSigningStatusViewFragment(HttpServletRequest req, User user, URIParser uriParser, ImmutableFlowInstance flowInstance) throws SQLException, TransformerException {

		SigningSession signingSession = getSigningSession(flowInstance.getFlowInstanceID());

		if (signingSession == null) {

			if (flowInstance.getStatus().getContentType() == ContentType.WAITING_FOR_MULTISIGN) {

				log.error("No signing session found for flow instance " + flowInstance + " but status is " + ContentType.WAITING_FOR_MULTISIGN + ". User " + user);

				Document doc = createDocument(req, uriParser);

				Element misconfiguredMultisigningElement = doc.createElement("MisconfiguredMultisigning");
				doc.getDocumentElement().appendChild(misconfiguredMultisigningElement);

				return viewFragmentTransformer.createViewFragment(doc);
			}

			return null;
		}

		Document doc = createDocument(req, uriParser);

		Element signingStatusElement = doc.createElement("SigningStatus");
		doc.getDocumentElement().appendChild(signingStatusElement);

		XMLUtils.append(doc, signingStatusElement, flowInstance);
		XMLUtils.append(doc, signingStatusElement, signingSession);
		
		if (hideCitizenIdentifier || CollectionUtils.contains(flowTypesWithHiddenCitizenIdentifier, flowInstance.getFlow().getFlowType().getFlowTypeID())) {
			
			XMLUtils.appendNewElement(doc, signingStatusElement, "HideCitizenIdentifier", "true");
		}

		XMLUtils.appendNewElement(doc, signingStatusElement, "SigningLink", getSigningURL(flowInstance, signingSession, null, uriParser));

		if (user != null) {
			XMLUtils.appendNewElement(doc, signingStatusElement, "LoggedIn");
		}

		return viewFragmentTransformer.createViewFragment(doc);
	}

	public String getSigningURL(ImmutableFlowInstance flowInstance, SigningSession signingSession, SigningSessionParty signingSessionParty, URIParser uriParser) {

		return getSigningURL(flowInstance, signingSession, signingSessionParty, uriParser.getFullContextPath());
	}

	public String getSigningURL(ImmutableFlowInstance flowInstance, SigningSession signingSession, SigningSessionParty signingSessionParty, String fullContextPath) {

		String urlEnd = flowInstance.getFlowInstanceID() + "/" + signingSession.getSessionID();

		if (signingSessionParty != null && signingSessionParty.isUnsecure()) {

			urlEnd += "/" + signingSessionParty.getUnsecureKey();
		}

		return fullContextPath + getFullAlias() + "/sign/" + urlEnd;
	}

	@Override
	public String getSigningURL(ImmutableFlowInstance flowInstance, SigningParty signingParty) {

		if (signingParty == null) {
			log.error("Null signing party with flow instance " + flowInstance);
			return null;
		}

		try {
			SigningSessionParty signingSessionParty = getSigningSessionParty(flowInstance.getFlowInstanceID(), signingParty);

			if (signingSessionParty == null) {

				log.error("Found no signing session for flow instance " + flowInstance + " and signing party " + signingParty);
				return null;
			}

			return getSigningURL(flowInstance, signingSessionParty.getSigningSession(), signingSessionParty, serverURL + systemInterface.getContextPath());

		} catch (SQLException e) {

			log.error("Error getting signing URL", e);
			return null;
		}
	}

	@WebPublic(alias = "sign", requireLogin = true)
	public ForegroundModuleResponse signFlowInstance(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		RequestData requestData = getRequestData(false, true, user, req, uriParser);

		if (requestData.getModuleResponse() != null) {
			return requestData.getModuleResponse();
		}

		SigningSession signingSession = requestData.getSigningSession();
		SigningSessionParty signingSessionParty = requestData.getSigningParty();
		ImmutableFlowInstance flowInstance = requestData.getFlowInstance();

		boolean sequentialSigningBlock = signingSession.isSequential() && isSequentialSigningBlocked(signingSession, signingSessionParty);

		if (signingSessionParty.getSignedDate() == null) {

			if (flowInstance.getStatus().getContentType() != ContentType.WAITING_FOR_MULTISIGN) {

				//User does not have access to sign this flow instance
				log.info("User does not have access to flow instance " + flowInstance + ", wrong status content type: " + flowInstance.getStatus().getContentType());
				return showMessage(MESSAGE_WRONG_STATUS_CONTENT_TYPE, req, uriParser);
			}

			if (!flowInstance.getFlow().isEnabled() || browserModule.isOperatingStatusDisabled(flowInstance.getFlow(), false, false)) {

				//User does not have access to sign this flow instance
				log.info("User does not have access to flow instance " + flowInstance + ", flow is disabled");
				return showMessage("FlowDisabled", req, uriParser);
			}

			if (signingSessionParty.isUnsecure()) {

				MultiSigning2FlowInstanceAccessController.setSessionAttribute(requestData.getFlowInstance().getFlowInstanceID(), req.getSession());
			}

			if (BooleanUtils.toBoolean(req.getParameter("sign"))) {

				if (sequentialSigningBlock) {

					log.info("User does not have access to sign flow instance " + flowInstance + " yet, as sequential signing is active and not all preceeding parties have signed.");
					return showMessage(MESSAGE_SEQUENTIAL_SIGNING_BLOCK, req, uriParser);
				}

				log.info("User " + user + " preparing signing of flow instance " + flowInstance);

				return showSignForm(req, res, user, uriParser, flowInstance, signingSession, signingSessionParty, null);
			}
		}

		log.info("User " + user + " viewing sign form for flow instance " + flowInstance);

		ImmutableFlowInstanceManager instanceManager = browserModule.getImmutableFlowInstanceManager(flowInstance.getFlowInstanceID(), null, user, false, req, uriParser, BaseFlowModule.OWNER_REQUEST_METADATA);

		Document doc = createDocument(req, uriParser);

		Element signFlowInstanceElement = doc.createElement("SignFlowInstance");
		doc.getDocumentElement().appendChild(signFlowInstanceElement);

		if (signingSessionParty.getSignedDate() == null) {

			XMLUtils.appendNewCDATAElement(doc, signFlowInstanceElement, "Message", getMessage(user, flowInstance, signMessage, null));

		} else {

			XMLUtils.appendNewCDATAElement(doc, signFlowInstanceElement, "Message", getMessage(user, flowInstance, signedMessage, signingSessionParty));

			if (browserModule.requiresPayment(instanceManager)) {

				XMLUtils.appendNewElement(doc, signFlowInstanceElement, "RequiresPayment");
			}
		}

		signFlowInstanceElement.appendChild(flowInstance.toXML(doc));
		XMLUtils.append(doc, signFlowInstanceElement, signingSession);
		XMLUtils.append(doc, signFlowInstanceElement, signingSessionParty);

		if (sequentialSigningBlock) {

			XMLUtils.appendNewElement(doc, signFlowInstanceElement, MESSAGE_SEQUENTIAL_SIGNING_BLOCK);
		}

		List<ManagerResponse> managerResponses = instanceManager.getFullShowHTML(req, user, this, true, null, getImmutableQueryRequestBaseURL(req, instanceManager), BaseFlowModule.OWNER_REQUEST_METADATA);
		XMLUtils.append(doc, signFlowInstanceElement, "ManagerResponses", managerResponses);

		SimpleForegroundModuleResponse moduleResponse = new SimpleForegroundModuleResponse(doc, getDefaultBreadcrumb());

		BaseFlowModule.appendLinksAndScripts(moduleResponse, managerResponses);

		return moduleResponse;
	}

	private SimpleSigningRequest getSigningRequest(ImmutableFlowInstance flowInstance, SigningSession signingSession, SigningSessionParty signingParty, URIParser uriParser) throws IOException {

		File signingPDF = getSigningPDF(flowInstance, signingSession);

		if (signingPDF == null) {
			return null;
		}

		String dataToSign = HashUtils.hash(signingPDF, HashAlgorithms.SHA1);

		String description = signingMessage.replace("$flowInstance.flowInstanceID", flowInstance.getFlowInstanceID() + "");
		description = description.replace("$flow.name", flowInstance.getFlow().getName());
		description = description.replace("$hash", dataToSign);

		String urlEnd = flowInstance.getFlowInstanceID() + "/" + signingSession.getSessionID();

		if (signingParty.isUnsecure()) {

			urlEnd += "/" + signingParty.getUnsecureKey();
		}

		String signingFormURL = uriParser.getFullContextPath() + getFullAlias() + "/sign/" + urlEnd;
		String processSigningURL = uriParser.getFullContextPath() + getFullAlias() + "/processsign/" + urlEnd + "?signreq=1";

		return new SimpleSigningRequest(description, signingPDF, signingFormURL, processSigningURL);
	}

	private ForegroundModuleResponse showSignForm(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, ImmutableFlowInstance flowInstance, SigningSession signingSession, SigningSessionParty signingSessionParty, List<ValidationError> validationErrors) throws Exception {

		Document doc = createDocument(req, uriParser);

		Element signingFormElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "Signing");

		SimpleSigningRequest signingRequest = getSigningRequest(flowInstance, signingSession, signingSessionParty, uriParser);

		if (signingRequest == null) {
			throw new RuntimeException("Unable to find signing PDF for " + signingSession + ", " + flowInstance);
		}

		ViewFragment viewFragment = genericSigningProvider.showSignForm(req, res, user, signingRequest, validationErrors);

		signingFormElement.appendChild(flowInstance.toXML(doc));
		signingFormElement.appendChild(viewFragment.toXML(doc));

		XMLUtils.appendNewElement(doc, signingFormElement, "SigningLink", getSigningURL(flowInstance, signingSession, signingSessionParty, uriParser));

		FragmentLinkScriptFilter.addViewFragment(viewFragment, req);

		SimpleForegroundModuleResponse response = new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
		ViewFragmentUtils.appendLinksAndScripts(response, viewFragment);

		return response;
	}

	@WebPublic(alias = "processsign", requireLogin = true)
	public ForegroundModuleResponse processSigning(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		RequestData requestData = getRequestData(false, false, user, req, uriParser);

		if (requestData.getModuleResponse() != null) {
			return requestData.getModuleResponse();
		}

		SigningSession signingSession = requestData.getSigningSession();
		SigningSessionParty signingSessionParty = requestData.getSigningParty();
		ImmutableFlowInstance flowInstance = requestData.getFlowInstance();

		if (signingSessionParty.getSignedDate() != null) {

			return signFlowInstance(req, res, user, uriParser);
		}

		if (flowInstance.getStatus().getContentType() != ContentType.WAITING_FOR_MULTISIGN) {

			//User does not have access to sign this flow instance
			log.info("User does not have access to flow instance " + flowInstance + ", wrong status content type: " + flowInstance.getStatus().getContentType());
			return showMessage(MESSAGE_WRONG_STATUS_CONTENT_TYPE, req, uriParser);
		}

		if (!flowInstance.getFlow().isEnabled() || browserModule.isOperatingStatusDisabled(flowInstance.getFlow(), false, false)) {

			//User does not have access to sign this flow instance
			log.info("User does not have access to flow instance " + flowInstance + ", flow is disabled");
			return showMessage("FlowDisabled", req, uriParser);
		}

		if (signingSession.isSequential() && isSequentialSigningBlocked(signingSession, signingSessionParty)) {

			log.info("User does not have access to sign flow instance " + flowInstance + " yet, as sequential signing is active and not all preceeding parties have signed.");
			return showMessage(MESSAGE_SEQUENTIAL_SIGNING_BLOCK, req, uriParser);
		}

		log.info("User " + user + " processing singing for flow instance " + flowInstance + ", signingSession " + signingSession);

		List<ValidationError> validationErrors = null;

		try {
			SimpleSigningRequest signingRequest = getSigningRequest(flowInstance, signingSession, signingSessionParty, uriParser);

			if (signingRequest == null) {
				throw new RuntimeException("Unable to find signing PDF for " + signingSession + ", " + flowInstance);
			}

			SigningResponse signingResponse = genericSigningProvider.processSigning(req, res, user, signingRequest);

			if (res.isCommitted()) {
				return null;
			}

			if (signingResponse != null) {

				if (user == null) {

					user = signingResponse.getUser();
				}

				log.info("User " + user + " signed flow instance " + flowInstance + ", signingSession " + signingSession);

				Timestamp now = TimeUtils.getCurrentTimestamp();

				TransactionHandler transactionHandler = signingPartyDAO.createTransaction();

				try {
					FlowInstanceEvent signingEvent = new FlowInstanceEvent();
					signingEvent.setFlowInstance((FlowInstance) flowInstance);
					signingEvent.setEventType(EventType.SIGNED);
					signingEvent.setPoster(user);
					signingEvent.setStatus(flowInstance.getStatus().getName());
					signingEvent.setStatusDescription(flowInstance.getStatus().getDescription());
					signingEvent.setAdded(now);

					if (signingResponse.getSigningAttributes() != null) {

						MutableAttributeHandler attributeHandler = signingEvent.getAttributeHandler();

						for (Entry<String, String> entry : signingResponse.getSigningAttributes().entrySet()) {

							attributeHandler.setAttribute(entry.getKey(), entry.getValue());
						}
					}

					signingEvent.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION, signingSession.getSessionID());
					signingEvent.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT, Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT_SIGNING);

					flowInstanceEventDAO.add(signingEvent, transactionHandler, new RelationQuery(FlowInstanceEvent.ATTRIBUTES_RELATION));

					signingSessionParty.setSigningSession(signingSession);
					signingSessionParty.setSignedDate(now);
					signingSessionParty.setSignedEventID(signingEvent.getEventID());

					if (user != null) {

						signingSessionParty.setSignerUserID(user.getUserID());

						if (signingSessionParty.isUnsecure()) {

							String citizenID = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

							if (citizenID != null) {

								signingSessionParty.setIdentifier(citizenID);
								signingSessionParty.setUnsecureKey(null);
							}
						}
					}

					signingPartyDAO.update(signingSessionParty, transactionHandler, null);

					transactionHandler.commit();

				} finally {
					TransactionHandler.autoClose(transactionHandler);
				}

				if (user != null && user instanceof MutableUser) {

					MutableUser mutableUser = (MutableUser) user;
					boolean changed = false;

					if (!StringUtils.isEmpty(signingSessionParty.getEmail()) && StringUtils.isEmpty(mutableUser.getEmail())) {

						changed = true;
						mutableUser.setEmail(signingSessionParty.getEmail());
					}

					MutableAttributeHandler attributeHandler = mutableUser.getAttributeHandler();

					if (!StringUtils.isEmpty(signingSessionParty.getMobilePhone()) && StringUtils.isEmpty(attributeHandler.getString("mobilePhone"))) {

						changed = true;
						attributeHandler.setAttribute("mobilePhone", signingSessionParty.getMobilePhone());
						attributeHandler.setAttribute("contactBySMS", true);
					}

					if (changed) {

						try {
							systemInterface.getUserHandler().updateUser(mutableUser, false, false, true);

						} catch (UnableToUpdateUserException e) {

							log.error("Unable to update user " + user, e);
						}
					}
				}

				if (signingSession.isFullySigned()) {

					log.info("Multi-party signing of flowinstance " + flowInstance + " complete");

					signingSession.setCompletedDate(now);
					RelationQuery signingSessionQuery = new RelationQuery();
					signingSessionQuery.disableAutoRelations(true);

					signingSessionDAO.update(signingSession, signingSessionQuery);

					Map<String, String> eventAttributes = new HashMap<>();
					eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION, signingSession.getSessionID().toString());

					ImmutableFlowInstanceManager instanceManager = browserModule.getImmutableFlowInstanceManager(flowInstance.getFlowInstanceID(), null, user, false, req, uriParser, BaseFlowModule.OWNER_REQUEST_METADATA);

					if (!browserModule.requiresPayment(instanceManager)) {

						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT, Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT_SIGNED_PDF);
					}

					browserModule.multiSigningComplete(instanceManager, user, browserModule.getSiteProfile(flowInstance), signingSession.getSigningParties(), eventAttributes);

				} else if (signingSession.isSequential() && standardFlowNotificationHandler != null) {

					SigningSessionParty nextParty = signingSession.getSigningSessionParties().get(signingSession.getSigningSessionParties().indexOf(signingSessionParty) + 1);
					standardFlowNotificationHandler.sendSigningPartyNotifications(flowInstance, nextParty.getSigningParty());
				}

				systemInterface.getEventHandler().sendEvent(FlowInstance.class, new MultiSigningPartySignedEvent(flowInstance, signingSessionParty.getSigningParty(), user, signingSession.getSigningParties()), EventTarget.ALL);

				res.sendRedirect(getSigningURL(flowInstance, signingSession, signingSessionParty, uriParser));
				return null;

			} else {

				log.warn("Signing failed");
			}

		} catch (ValidationException e) {

			validationErrors = new ArrayList<>();
			validationErrors.addAll(e.getErrors());
		}

		return showSignForm(req, res, user, uriParser, flowInstance, signingSession, signingSessionParty, validationErrors);
	}

	@WebPublic(alias = "cancel", requireLogin = true)
	public ForegroundModuleResponse cancelSigning(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if (!HTTPUtils.isPost(req)) {

			throw new AccessDeniedException("Cancel requests using method " + req.getMethod() + " are not allowed.");
		}

		RequestData requestData = getRequestData(true, false, user, req, uriParser);

		if (requestData.getModuleResponse() != null) {
			return requestData.getModuleResponse();
		}

		SigningSession signingSession = requestData.getSigningSession();
		SigningSessionParty signingSessionParty = requestData.getSigningParty();
		ImmutableFlowInstance flowInstance = requestData.getFlowInstance();

		if (signingSession.isFullySigned()) {

			if (signingSessionParty != null) {

				return signFlowInstance(req, res, user, uriParser);

			} else {

				res.sendRedirect(uriParser.getContextPath() + userFlowInstanceModule.getFullAlias() + "/overview/" + flowInstance.getFlow().getFlowID() + "/" + flowInstance.getFlowInstanceID());
				return null;
			}
		}

		if (signingSessionParty == null) {

			log.info("Owning user " + user + " cancelling multi part signing of flow instance " + flowInstance);

		} else {

			log.info("Signing party " + signingSessionParty + " (user " + user + ") cancelling multi part signing of flow instance " + flowInstance);
		}
		
		if (flowInstance.getStatus().getContentType() != ContentType.WAITING_FOR_MULTISIGN) {
			
			return showMessage(MESSAGE_WRONG_STATUS_CONTENT_TYPE, req, uriParser);
		}
		
		if (flowInstance.getFirstSubmitted() != null) {
			
			log.info("User " + user + " attempted to cancel multipart signing of already submitted flow instance " + flowInstance);

			return showMessage("MayNotCancelSubmittedFlowInstance", req, uriParser);
		}

		TransactionHandler transactionHandler = signingSessionDAO.createTransaction();

		try {

			FlowInstance mutableflowInstance = userFlowInstanceModule.getFlowInstance(flowInstance.getFlowInstanceID());

			mutableflowInstance.setStatus(mutableflowInstance.getFlow().getDefaultState(FlowBrowserModule.SAVE_ACTION_ID));

			signingSessionDAO.delete(signingSession, transactionHandler);
			
			deleteSigningEvents(flowInstance, transactionHandler);

			mutableflowInstance.setLastStatusChange(TimeUtils.getCurrentTimestamp());

			flowInstanceDAO.update(mutableflowInstance, transactionHandler, null);

			transactionHandler.commit();
			
			Integer eventID = signingSession.getPdfEventID();
			
			if (eventID != null) {
			
				pdfProvider.deletePDF(flowInstance.getFlowInstanceID(), eventID);
			}

			if (signingSession.isSequential() && signingSession.getSigningParties() != null) {

				Iterator<SigningSessionParty> iterator = signingSession.getSigningSessionParties().iterator();

				while (iterator.hasNext()) { // Remove not yet notified signers, we are the current signer in line

					SigningSessionParty party = iterator.next();

					//When owner cancels, the first signing party in the sequence need to have the cancel mail
					if (signingSessionParty == null && party.equals(signingSession.getSigningSessionParties().get(0))) {

						continue;
					}

					if (party.getSignedDate() == null) {

						iterator.remove();
					}
				}
			}

			systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(CRUDAction.UPDATE, mutableflowInstance), EventTarget.ALL);
			systemInterface.getEventHandler().sendEvent(FlowInstance.class, new MultiSigningCanceledEvent(mutableflowInstance, signingSession.getSigningParties(), signingSessionParty == null ? null : signingSessionParty.getSigningParty(), user), EventTarget.ALL);

		} finally {
			TransactionHandler.autoClose(transactionHandler);
		}

		if (signingSessionParty == null) { // Owner cancelled

			String preview = flowInstance.getFlow().usesPreview() ? "?preview=1" : "";
			res.sendRedirect(uriParser.getContextPath() + userFlowInstanceModule.getFullAlias() + "/flowinstance/" + flowInstance.getFlow().getFlowID() + "/" + flowInstance.getFlowInstanceID() + preview);
			return null;

		} else if (user != null) {

			res.sendRedirect(uriParser.getContextPath() + userFlowInstanceModule.getFullAlias());
			return null;

		} else {

			res.sendRedirect(uriParser.getContextPath() + browserModule.getFullAlias());
			return null;
		}
	}

	private void deleteSigningEvents(ImmutableFlowInstance flowInstance, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<FlowInstanceEvent> query = new HighLevelQuery<>();
		
		query.addParameter(flowInstanceEventFlowInstanceParamFactory.getParameter((FlowInstance)flowInstance));
		query.addParameter(flowInstanceEventTypeParamFactory.getWhereInParameter(SIGNING_EVENT_TYPES));
		
		flowInstanceEventDAO.delete(query, transactionHandler, 10);
	}

	//TODO this method should be removed later on and merged with the normal method as it duplicates a lot of code
	@Override
	public void cancelSigning(FlowInstance flowInstance) throws SQLException {

		if (flowInstance.getStatus().getContentType() != ContentType.WAITING_FOR_MULTISIGN) {
			throw new InvalidParameterException();
		}

		SigningSession signingSession = getSigningSession(flowInstance.getFlowInstanceID());

		if (signingSession == null) {
			return;
		}

		log.info("Cancelling multipart signing of flow instance " + flowInstance);

		if (flowInstance.getFirstSubmitted() == null) {

			flowInstance.setStatus(flowInstance.getFlow().getDefaultState(FlowBrowserModule.SAVE_ACTION_ID));

		} else {

			throw new UnsupportedOperationException("Currently unable to cancel signing of submitted flow instances");
		}
		
		TransactionHandler transactionHandler = signingSessionDAO.createTransaction();
		
		try {
			signingSessionDAO.delete(signingSession, transactionHandler);
			
			flowInstance.setLastStatusChange(TimeUtils.getCurrentTimestamp());

			flowInstanceDAO.update(flowInstance, transactionHandler, null);

			if (flowInstance.getFirstSubmitted() == null) {
				
				deleteSigningEvents(flowInstance, transactionHandler);
			}
			
			transactionHandler.commit();
			
			Integer eventID = signingSession.getPdfEventID();
			
			if (eventID != null) {
			
				pdfProvider.deletePDF(flowInstance.getFlowInstanceID(), eventID);
			}
			
		}finally {
			
			TransactionHandler.autoClose(transactionHandler);
		}

		systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(CRUDAction.UPDATE, flowInstance), EventTarget.ALL);

		//TODO currently shows as owner cancelling, should show system
		systemInterface.getEventHandler().sendEvent(FlowInstance.class, new MultiSigningCanceledEvent(flowInstance, signingSession.getSigningParties(), null, null), EventTarget.ALL);
	}

	private String getMessage(User user, ImmutableFlowInstance flowInstance, String messageTemplate, SigningSessionParty signingParty) {

		StringBuilder userTag = new StringBuilder();

		String posterName = FlowInstanceUtils.getSubmitterName(flowInstance, anonymousUserName);

		if (posterName != null) {

			userTag.append(posterName);

			if (!hideCitizenIdentifier && !CollectionUtils.contains(flowTypesWithHiddenCitizenIdentifier, flowInstance.getFlow().getFlowType().getFlowTypeID())) {

				String posterCitizenIdentifier = FlowInstanceUtils.getSubmitterCitizenID(flowInstance);

				if (posterCitizenIdentifier != null) {

					userTag.append(" (");
					userTag.append(posterCitizenIdentifier);
					userTag.append(")");
				}
			}
		}

		String message = messageTemplate.replace("$poster", userTag.toString());

		message = message.replace("$flowInstanceID", flowInstance.getFlowInstanceID().toString());
		message = message.replace("$flow.name", flowInstance.getFlow().getName());

		if (signingParty != null) {

			message = message.replace("$signed", DateUtils.DATE_TIME_SECONDS_FORMATTER.format(signingParty.getSignedDate()));
		}

		return message;
	}

	private ForegroundModuleResponse showMessage(String messageKey, HttpServletRequest req, URIParser uriParser) {

		Document doc = createDocument(req, uriParser);

		Element messageElement = doc.createElement("Message");
		messageElement.setAttribute("messageKey", messageKey);

		doc.getDocumentElement().appendChild(messageElement);

		return new SimpleForegroundModuleResponse(doc, getDefaultBreadcrumb());
	}

	private RequestData getRequestData(boolean allowOwner, boolean optionalSigningSessionID, User user, HttpServletRequest req, URIParser uriParser) throws URINotFoundException, AccessDeniedException, SQLException {

		Integer flowInstanceID = uriParser.getInt(2);
		Integer signingSessionID = uriParser.getInt(3);
		String unsecureKey = uriParser.get(4);

		SigningSession signingSession;

		if (signingSessionID == null && optionalSigningSessionID) {
			
			signingSession = getSigningSession(flowInstanceID);

		} else {
			
			signingSession = getSigningSession(flowInstanceID, signingSessionID);
		}

		if (signingSession == null) {

			return new RequestData(showMessage(MESSAGE_SIGNING_PARTY_NOT_FOUND_OR_INVALID, req, uriParser));
		}

		ImmutableFlowInstance flowInstance = getFlowInstance(flowInstanceID);

		if (flowInstance == null) {
			
			log.error("Flow instance not found for " + signingSession);
			return new RequestData(showMessage(MESSAGE_SIGNING_PARTY_NOT_FOUND_OR_INVALID, req, uriParser));
		}

		SigningSessionParty signingSessionParty = getSigningSessionParty(signingSession, user, unsecureKey);

		if (signingSessionParty == null) {

			boolean hasOwnerAccess = false;

			if (allowOwner) {
				try {
					UserFlowInstanceModule.PREVIEW_ACCESS_CONTROLLER.checkFlowInstanceAccess(flowInstance, user);
					hasOwnerAccess = true;
				} catch (AccessDeniedException e) {}
			}

			if (!hasOwnerAccess) {

				if (unsecureKey != null) {

					log.info("User " + user + " is not a signing party and unsecure key is invalid for " + signingSession);

				} else {

					log.info("User " + user + " is not a signing party for " + signingSession);
				}

				return new RequestData(showMessage(MESSAGE_SIGNING_PARTY_NOT_FOUND_OR_INVALID, req, uriParser));
			}
		}

		return new RequestData(signingSession, flowInstance, signingSessionParty);
	}

	public static boolean isSequentialSigningBlocked(SigningSession signingSession, SigningSessionParty signingSessionParty) {

		for (SigningSessionParty otherParty : signingSession.getSigningSessionParties()) {

			if (signingSessionParty == otherParty) {
				break;
			}

			if (otherParty.getSignedDate() == null) {
				return true;
			}
		}

		return false;
	}

	private File getSigningPDF(ImmutableFlowInstance flowInstance, SigningSession signingSession) {

		if (!CollectionUtils.isEmpty(flowInstance.getEvents())) {

			for (ImmutableFlowInstanceEvent event : new ReverseListIterator<ImmutableFlowInstanceEvent>(flowInstance.getEvents())) {

				if (event.getEventType() == EventType.SIGNED || event.getEventType() == EventType.SIGNING_SKIPPED) {

					AttributeHandler attributeHandler = event.getAttributeHandler();

					if (signingSession.getSessionID().equals(attributeHandler.getInt(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION)) && Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT_SIGNING_PDF.equals(attributeHandler.getString(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT))) {

						File pdfFile = pdfProvider.getPDF(flowInstance.getFlowInstanceID(), event.getEventID());

						if (pdfFile != null) {

							return pdfFile;
						}

						break;
					}
				}
			}
		}

		return null;
	}

	private File getSignedPDF(ImmutableFlowInstance flowInstance, SigningSession signingSession) {

		if (!CollectionUtils.isEmpty(flowInstance.getEvents())) {

			for (ImmutableFlowInstanceEvent event : new ReverseListIterator<ImmutableFlowInstanceEvent>(flowInstance.getEvents())) {

				if (event.getEventType() == EventType.SUBMITTED) {

					AttributeHandler attributeHandler = event.getAttributeHandler();

					if (signingSession.getSessionID().equals(attributeHandler.getInt(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION)) && Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT_SIGNED_PDF.equals(attributeHandler.getString(Constants.FLOW_INSTANCE_EVENT_SIGNING_SESSION_EVENT))) {

						File pdfFile = pdfProvider.getPDF(flowInstance.getFlowInstanceID(), event.getEventID());

						if (pdfFile != null) {

							return pdfFile;
						}

						break;
					}
				}
			}
		}

		return null;
	}

	@WebPublic(alias = "signingpdf", requireLogin = true)
	public ForegroundModuleResponse getSigningPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException, SQLException, AccessDeniedException {

		RequestData requestData = getRequestData(true, false, user, req, uriParser);

		if (requestData.getModuleResponse() != null) {
			return requestData.getModuleResponse();
		}

		SigningSession signingSession = requestData.getSigningSession();
		ImmutableFlowInstance flowInstance = requestData.getFlowInstance();

		File pdfFile = getSigningPDF(flowInstance, signingSession);

		if (pdfFile != null) {

			log.info("Sending PDF for flow instance " + flowInstance + ", signingSession " + signingSession + " to user " + user);

			try {
				HTTPUtils.sendFile(pdfFile, flowInstance.getFlow().getName() + " - " + flowInstance.getFlowInstanceID() + ".pdf", req, res, ContentDisposition.ATTACHMENT);
				return null;

			} catch (IOException e) {
				log.info("Error sending PDF for flow instance " + flowInstance + ", signingSession " + signingSession + " to user " + user + ", " + e);
			}
		}

		log.warn("Unable to find signing PDF for flow instance " + flowInstance + " requested by user " + user);
		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "signedpdf", requireLogin = true)
	public ForegroundModuleResponse getSignedPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException, SQLException, AccessDeniedException {

		RequestData requestData = getRequestData(true, false, user, req, uriParser);

		if (requestData.getModuleResponse() != null) {
			return requestData.getModuleResponse();
		}

		SigningSession signingSession = requestData.getSigningSession();
		ImmutableFlowInstance flowInstance = requestData.getFlowInstance();

		File pdfFile = getSignedPDF(flowInstance, signingSession);

		if (pdfFile != null) {

			log.info("Sending PDF for flow instance " + flowInstance + ", signingSession " + signingSession + " to user " + user);

			try {
				HTTPUtils.sendFile(pdfFile, flowInstance.getFlow().getName() + " - " + flowInstance.getFlowInstanceID() + ".pdf", req, res, ContentDisposition.ATTACHMENT);
				return null;

			} catch (IOException e) {
				log.info("Error sending PDF for flow instance " + flowInstance + ", signingSession " + signingSession + " to user " + user + ", " + e);
			}
		}

		log.warn("Unable to find signed PDF for flow instance " + flowInstance + " requested by user " + user);
		throw new URINotFoundException(uriParser);
	}

	@Override
	public ViewFragment getListFlowInstancesViewFragment(List<FlowInstance> flowInstances, SiteProfileHandler profileHandler, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		if (!StringUtils.isEmpty(citizenIdentifier)) {

			LowLevelQuery<SigningSession> query = new LowLevelQuery<>(GET_SIGNING_SESSIONS_BY_CITIZEN_IDENTIFIER);
			query.addParameter(citizenIdentifier);

			List<SigningSession> signingSessions = signingSessionDAO.getAll(query);

			if (signingSessions != null) {

				Map<Integer, SigningSession> signingSessionMap = new HashMap<>();

				Iterator<SigningSession> it = signingSessions.iterator();
				while (it.hasNext()) {

					SigningSession signingSession = it.next();

					if (signingSession.isSequential()) {

						SigningSessionParty signingSessionParty = getSigningSessionParty(signingSession, user, null);

						if (isSequentialSigningBlocked(signingSession, signingSessionParty)) {

							it.remove();
							continue;
						}
					}

					signingSessionMap.put(signingSession.getFlowInstanceID(), signingSession);
				}

				if (!signingSessions.isEmpty()) {

					List<FlowInstance> multiSignFlowInstances = getMultiSignFlowInstances(signingSessions);

					if (multiSignFlowInstances != null) {

						Document doc = createDocument(req, uriParser);

						Element listFlowInstancesExtensionElement = doc.createElement("ListFlowInstancesExtension");
						doc.getDocumentElement().appendChild(listFlowInstancesExtensionElement);

						if (profileHandler != null) {

							XMLUtils.append(doc, listFlowInstancesExtensionElement, "SiteProfiles", profileHandler.getProfiles());
						}

						XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
						genDoc.addIgnoredFields(UserFlowInstanceModule.LIST_EXCLUDED_FIELDS);

						Element waitingMultiSignFlowInstancesElement = XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "WaitingMultiSignFlowInstances");

						for (FlowInstance flowInstance : multiSignFlowInstances) {

							SigningSession signingSession = signingSessionMap.get(flowInstance.getFlowInstanceID());

							Element flowInstanceElement = (Element) waitingMultiSignFlowInstancesElement.appendChild(flowInstance.toXML(genDoc));

							XMLUtils.appendNewElement(doc, flowInstanceElement, "MultiSignURL", getSigningURL(flowInstance, signingSession, null, uriParser));
						}

						return viewFragmentTransformer.createViewFragment(doc);
					}
				}
			}
		}

		return null;
	}

	protected List<FlowInstance> getMultiSignFlowInstances(List<SigningSession> signingSessions) throws SQLException {

		LowLevelQuery<FlowInstance> query = new LowLevelQuery<>("SELECT i.* FROM flowengine_flow_instances i INNER JOIN flowengine_flow_statuses s ON i.statusID = s.statusID WHERE s.contentType = '" + ContentType.WAITING_FOR_MULTISIGN + "' AND i.flowInstanceID IN (" + DBUtils.getPreparedStatementQuestionMarks(signingSessions) + ")");

		query.addRelations(FlowInstance.FLOW_RELATION, FlowInstance.STATUS_RELATION, Flow.FLOW_TYPE_RELATION);
		query.addExcludedFields(UserFlowInstanceModule.LIST_EXCLUDED_FIELDS);

		for (SigningSession signingSession : signingSessions) {

			query.addParameter(signingSession.getFlowInstanceID());
		}

		return browserModule.getDAOFactory().getFlowInstanceDAO().getAll(query);
	}

	public List<SigningSession> getSigningSessions() throws SQLException {

		return signingSessionDAO.getAll();
	}

	public List<SigningSession> getSigningSessions(List<Integer> flowInstanceIDs) throws SQLException {

		if (CollectionUtils.isEmpty(flowInstanceIDs)) {
			return null;
		}

		HighLevelQuery<SigningSession> query = new HighLevelQuery<>();
		query.addParameter(signingSessionFlowInstanceIDParamFactory.getWhereInParameter(flowInstanceIDs));

		return signingSessionDAO.getAll(query);
	}

	private SigningSession getSigningSession(Integer flowInstanceID, Integer signingSessionID) throws SQLException {

		if (flowInstanceID == null || signingSessionID == null) {
			return null;
		}

		HighLevelQuery<SigningSession> query = new HighLevelQuery<>();

		query.addParameter(signingSessionFlowInstanceIDParamFactory.getParameter(flowInstanceID));
		query.addParameter(signingSessionIDParamFactory.getParameter(signingSessionID));

		return signingSessionDAO.get(query);
	}

	public SigningSession getSigningSession(Integer flowInstanceID) throws SQLException {

		HighLevelQuery<SigningSession> query = new HighLevelQuery<>();

		query.addParameter(signingSessionFlowInstanceIDParamFactory.getParameter(flowInstanceID));

		return signingSessionDAO.get(query);
	}

	protected SigningSessionParty getSigningSessionParty(Integer flowInstanceID, User user) throws SQLException {

		String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		if (StringUtils.isEmpty(citizenIdentifier)) {
			return null;
		}

		//@formatter:off
		LowLevelQuery<SigningSessionParty> query = new LowLevelQuery<>(
			"SELECT * FROM " + signingPartyDAO.getTableName()
			+ " WHERE sessionID IN"
			+ "  (SELECT sessionID FROM " + signingSessionDAO.getTableName() + " WHERE flowInstanceID = ?)"
			+ " AND identifier = ?"
		);
		//@formatter:on

		query.addParameter(flowInstanceID);
		query.addParameter(citizenIdentifier);

		return signingPartyDAO.get(query);
	}

	public FlowInstance getFlowInstance(int flowInstanceID) throws SQLException {

		HighLevelQuery<FlowInstance> query = new HighLevelQuery<>(FLOW_INSTANCE_RELATIONS);
		query.addParameter(flowInstanceIDParamFactory.getParameter(flowInstanceID));

		return flowInstanceDAO.get(query);
	}

	public SigningSessionParty getSigningSessionParty(SigningSession signingSession, User user, String unsecureKey) {

		for (SigningSessionParty party : signingSession.getSigningSessionParties()) {

			if (party.isUnsecure()) {

				if (party.getUnsecureKey().equals(unsecureKey)) {
					return party;
				}

			} else if (party.getIdentifier().equals(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user))) {

				return party;
			}
		}

		return null;
	}

	public SigningSessionParty getSigningSessionParty(Integer flowInstanceID, SigningParty signingParty) throws SQLException {

		SigningSession signingSession = getSigningSession(flowInstanceID);

		if (signingSession == null) {
			return null;
		}

		for (SigningSessionParty sessionParty : signingSession.getSigningSessionParties()) {

			if (sessionParty.isUnsecure()) {

				if (signingParty.isUnsecure()) {

					if (ObjectUtils.compare(signingParty.getFirstname(), sessionParty.getFirstname()) && ObjectUtils.compare(signingParty.getLastname(), sessionParty.getLastname()) && ObjectUtils.compare(signingParty.getEmail(), sessionParty.getEmail()) && ObjectUtils.compare(signingParty.getMobilePhone(), sessionParty.getMobilePhone())) {

						sessionParty.setSigningSession(signingSession);
						return sessionParty;
					}
				}

			} else if (sessionParty.getIdentifier().equals(signingParty.getSocialSecurityNumber())) {

				sessionParty.setSigningSession(signingSession);
				return sessionParty;
			}
		}

		return null;
	}

	@Override
	public boolean partyHasSigned(Integer flowInstanceID, SigningParty signingParty, Timestamp latestSubmitTimestamp) throws Exception {

		SigningSessionParty sessionParty = getSigningSessionParty(flowInstanceID, signingParty);

		return sessionParty != null && sessionParty.getSignedDate() != null;
	}

	@EventListener(channel = FlowInstance.class)
	public void processEvent(CRUDEvent<FlowInstance> event, EventSource source) throws SQLException {

		if (source.isLocal() && event.getAction() == CRUDAction.DELETE) {

			//TODO wrap loop in transaction

			for (FlowInstance flowInstance : event.getBeans()) {

				log.info("Deleting signing sessions for flow instance " + flowInstance);

				HighLevelQuery<SigningSession> query = new HighLevelQuery<>();
				query.addParameter(signingSessionFlowInstanceIDParamFactory.getParameter(flowInstance.getFlowInstanceID()));

				signingSessionDAO.delete(query);
			}
		}
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser) {

		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		doc.appendChild(documentElement);
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		return doc;
	}

	@InstanceManagerDependency(required = true)
	public void setUserFlowInstanceModule(UserFlowInstanceModule userFlowInstanceModule) {

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.addListFlowInstancesViewFragmentExtensionProvider(this);

		} else {

			this.userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		this.userFlowInstanceModule = userFlowInstanceModule;
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

	@Override
	public boolean supportsSequentialSigning() {

		return true;
	}

	protected String getImmutableQueryRequestBaseURL(HttpServletRequest req, FlowInstanceManager instanceManager) {

		return req.getContextPath() + getFullAlias() + "/iquery/" + instanceManager.getFlowID() + "/" + instanceManager.getFlowInstanceID() + "/q/";
	}

	@WebPublic(alias = "iquery")
	public ForegroundModuleResponse processImmutableQueryRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, QueryRequestException, QueryProviderException, EvaluationProviderException, InvalidFlowInstanceStepException, MissingQueryInstanceDescriptor, DuplicateFlowInstanceManagerIDException {

		return browserModule.processImmutableQueryRequest(req, res, user, uriParser, new MultiSigning2FlowInstanceAccessController(this, req.getSession()), true, BaseFlowModule.OWNER_REQUEST_METADATA);
	}

	@Override
	public QueryHandler getQueryHandler() {

		return queryHandler;
	}

}
