package com.nordicpeak.flowengine.multisigninghandlers;

import java.io.Serializable;
import java.sql.Timestamp;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.random.RandomUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.beans.SigningParty;

@Table(name = "multi_signing_parties")
@XMLElement(name = "SigningSessionParty")
public final class SigningSessionParty extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = -7320825095300722075L;

	@DAOManaged(autoGenerated = true)
	@Key
	@OrderBy
	@XMLElement
	private Integer signingPartyID;

	@DAOManaged(columnName = "sessionID")
	@ManyToOne
	private SigningSession signingSession;

	@DAOManaged
	@XMLElement
	private String identifier;

	@DAOManaged
	@XMLElement
	private String unsecureKey;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String email;

	@DAOManaged
	@XMLElement
	private String mobilePhone;

	@DAOManaged
	@XMLElement
	private boolean addAsOwner;

	@DAOManaged
	@XMLElement
	private Timestamp signedDate;

	@DAOManaged
	@XMLElement(fixCase = true)
	private Integer signedEventID;

	@DAOManaged
	@XMLElement
	private Integer signerUserID;

	public SigningSessionParty() {}

	public SigningSessionParty(SigningParty signingParty) {

		firstname = signingParty.getFirstname();
		lastname = signingParty.getLastname();
		email = signingParty.getEmail();
		mobilePhone = signingParty.getMobilePhone();
		identifier = signingParty.getSocialSecurityNumber();

		addAsOwner = signingParty.isAddAsOwner();

		if (signingParty.isUnsecure()) {

			unsecureKey = RandomUtils.getRandomString(16, RandomUtils.MIXED_CASE_CHARACTERS_AND_DIGITS);
		}
	}

	public Integer getSigningPartyID() {

		return signingPartyID;
	}

	public void setSigningPartyID(Integer signingPartyID) {

		this.signingPartyID = signingPartyID;
	}

	public SigningSession getSigningSession() {

		return signingSession;
	}

	public void setSigningSession(SigningSession signingSession) {

		this.signingSession = signingSession;
	}

	public String getIdentifier() {

		return identifier;
	}

	public void setIdentifier(String identifier) {

		this.identifier = identifier;
	}

	public String getFirstname() {

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getMobilePhone() {

		return mobilePhone;
	}

	public void setMobilePhone(String mobilePhone) {

		this.mobilePhone = mobilePhone;
	}

	public boolean isUnsecure() {

		return unsecureKey != null;
	}

	public boolean isAddAsOwner() {

		return addAsOwner;
	}

	public void setAddAsOwner(boolean addAsOwner) {

		this.addAsOwner = addAsOwner;
	}

	public Timestamp getSignedDate() {

		return signedDate;
	}

	public void setSignedDate(Timestamp signed) {

		this.signedDate = signed;
	}

	public Integer getSignedEventID() {

		return signedEventID;
	}

	public void setSignedEventID(Integer signedEventID) {

		this.signedEventID = signedEventID;
	}

	public Integer getSignerUserID() {

		return signerUserID;
	}

	public void setSignerUserID(Integer signerUserID) {

		this.signerUserID = signerUserID;
	}

	public String getUnsecureKey() {

		return unsecureKey;
	}

	public void setUnsecureKey(String unsafeKey) {

		this.unsecureKey = unsafeKey;
	}

	@Override
	public String toString() {

		if (identifier != null) {

			return firstname + " " + lastname + " (" + identifier + ")";

		} else {

			return firstname + " " + lastname;
		}
	}

	@Override
	public int hashCode() {

		final int prime = 31;
		int result = 1;
		result = prime * result + ((signingPartyID == null) ? 0 : signingPartyID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		SigningSessionParty other = (SigningSessionParty) obj;
		if (signingPartyID == null) {
			if (other.signingPartyID != null) {
				return false;
			}
		} else if (!signingPartyID.equals(other.signingPartyID)) {
			return false;
		}
		return true;
	}

	public SigningParty getSigningParty() {

		SigningParty party = new SigningParty(firstname, lastname, email, mobilePhone, identifier, addAsOwner, isUnsecure());
		party.setSignerUserID(signerUserID);
		return party;
	}

}
