package com.nordicpeak.flowengine.evaluators.multipartsigningmailevaluator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.SigningParty;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.MultiSigningPartySignedEvent;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryInstanceNotFoundInQueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderErrorException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderNotFoundException;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MultiSignQuery;
import com.nordicpeak.flowengine.interfaces.MultiSignQueryinstance;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.notifications.NotificationOptions;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;

public class MultipartSigningMailEvaluatorEvaluationProviderModule extends BaseEvaluationProviderModule<MultipartSigningMailEvaluatorEvaluator> {

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency
	protected PDFProvider pdfProvider;

	@InstanceManagerDependency
	protected FlowInstanceAdminModule flowInstanceAdminModule;

	@InstanceManagerDependency(required = true)
	private StandardFlowNotificationHandler notificationHandler;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Allow signer contact as mail sender", description = "Controls if signer contact should be allowed to use as sender when sending emails")
	private boolean allowSignerContactAsSender;

	protected AnnotatedDAO<MultipartSigningMailEvaluatorEvaluator> evaluatorDAO;

	protected QueryParameterFactory<MultipartSigningMailEvaluatorEvaluator, Integer> evaluatorIDParamFactory;

	protected MultipartSigningMailEvaluatorEvaluatorCRUD evaluatorCRUD;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, MultipartSigningMailEvaluatorEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(MultipartSigningMailEvaluatorEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new MultipartSigningMailEvaluatorEvaluatorCRUD(MultipartSigningMailEvaluatorEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, MultipartSigningMailEvaluatorEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		return null;
	}

	@EventListener(channel = FlowInstance.class, priority = 50)
	public void processEvent(MultiSigningPartySignedEvent event, EventSource eventSource) throws SQLException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException {

		FlowInstanceManager flowInstanceManager = flowInstanceAdminModule.getImmutableFlowInstanceManager(event.getFlowInstance().getFlowInstanceID());

		sendMail(flowInstanceManager, event.getSigningParty(), event.getUser());
	}

	protected void sendMail(FlowInstanceManager flowInstanceManager, SigningParty signingParty, User eventUser) throws SQLException {

		List<MultiSignQueryinstance> queryInstances = flowInstanceManager.getQueries(MultiSignQueryinstance.class);

		if (queryInstances != null) {

			for (MultiSignQueryinstance queryInstance : queryInstances) {

				if (queryInstance == null || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {
					continue;
				}
				
				for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {

					if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {

						if (notificationHandler == null) {

							log.error("Notification handler not set, unable to send emails regarding flow instance " + flowInstanceManager.getFlowInstance());
							return;
						}
						
						MultipartSigningMailEvaluatorEvaluator evaluator = getEvaluator(evaluatorDescriptor);

						List<? extends SigningParty> querySigningParties = queryInstance.getSigningParties();

						if (querySigningParties != null && querySigningParties.contains(signingParty)) {
							
							SiteProfile siteProfile = flowInstanceAdminModule.getSiteProfile(flowInstanceManager);
							SimpleEmail email = null;

							try {
								ImmutableFlowInstance flowInstance = flowInstanceManager.getFlowInstance();

								List<String> recipients = new ArrayList<String>(evaluator.getEmailRecipients().size());

								for (String recipient : evaluator.getEmailRecipients()) {

									if (EmailUtils.isValidEmailAddress(recipient)) {

										recipients.add(recipient);

									} else {

										recipient = AttributeTagUtils.replaceTags(recipient, flowInstance.getAttributeHandler());

										if (EmailUtils.isValidEmailAddress(recipient)) {

											recipients.add(recipient);
										}
									}
								}

								if (!recipients.isEmpty()) {

									log.info("Sending email about partial signing of flow instance " + flowInstance + " to recipients " + StringUtils.toCommaSeparatedString(recipients) + " trigged by evaluator " + evaluatorDescriptor);

									Contact contact = new Contact();
									contact.setFirstname(signingParty.getFirstname());
									contact.setLastname(signingParty.getLastname());
									contact.setEmail(signingParty.getEmail());
									contact.setMobilePhone(signingParty.getMobilePhone());
									contact.setContactByEmail(true);

									NotificationOptions options = null;

									if (allowSignerContactAsSender && evaluator.isUsePosterContactAsSender() && !StringUtils.isEmpty(contact.getEmail()) && !StringUtils.isEmpty(contact.getFirstname()) && !StringUtils.isEmpty(contact.getLastname())) {

										options = new NotificationOptions(contact.getFirstname() + " " + contact.getLastname(), contact.getEmail());
									}

									for (String recipient : recipients) {

										notificationHandler.sendGlobalEmail(siteProfile, flowInstance, contact, recipient, evaluator.getEmailSubject(), evaluator.getEmailMessage(), null, false, options);
									}
								}

							} catch (Exception e) {

								log.error("Unable to send email " + email + " about partial signing of flow instance " + flowInstanceManager.getFlowInstance() + " triggered by evaluator " + evaluatorDescriptor, e);
							}
						}
					}
				}
			}
		}
	}

	public ImmutableFlowInstanceManager getImmutableFlowInstanceManager(FlowInstance flowInstance, SiteProfile siteProfile) throws DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException {

		return new ImmutableFlowInstanceManager(flowInstance, queryHandler, null, new DefaultInstanceMetadata(siteProfile), null);
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		MultipartSigningMailEvaluatorEvaluator evaluator = new MultipartSigningMailEvaluatorEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	private MultipartSigningMailEvaluatorEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {

		return evaluatorDAO.get(new HighLevelQuery<MultipartSigningMailEvaluatorEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		MultipartSigningMailEvaluatorEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<MultipartSigningMailEvaluatorEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<MultipartSigningMailEvaluatorEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		if (queryClass.getName().equals("com.nordicpeak.flowengine.queries.unsecuremultisignquery.UnsecureMultiSignQuery")) {
			return false;
		}
		
		return MultiSignQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		MultipartSigningMailEvaluatorEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<MultipartSigningMailEvaluatorEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	public MultipartSigningMailEvaluatorEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		MultipartSigningMailEvaluatorEvaluator evaluator = new MultipartSigningMailEvaluatorEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		if (!allowSignerContactAsSender) {

			evaluator.setUsePosterContactAsSender(false);
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	public String getAvailableTags() {

		return StringUtils.toCommaSeparatedString(StandardFlowNotificationHandler.getGlobalTags());
	}

	public boolean isAllowSignerContactAsSender() {

		return allowSignerContactAsSender;
	}
}
