package com.nordicpeak.flowengine.queries.pudquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.BeanStringPopulator;
import se.unlogic.standardutils.populators.QueryParameterPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.TextTagReplacer;
import com.nordicpeak.lmsearch.LMSearchProvider;

public class PUDQueryProviderModule extends BaseQueryProviderModule<PUDQueryInstance> implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	protected String lmUserSettingName = "LM user";

	@XSLVariable(prefix = "java.")
	protected String lmUserSettingDescription = "User to use for LM Search";

	@XSLVariable(prefix = "java.")
	protected String searchPrefixSettingName = "LM search prefix";

	@XSLVariable(prefix = "java.")
	protected String searchPrefixSettingDescription = "Search prefix used when searching for pud or address using LM Search";

	@XSLVariable(prefix = "java.")
	protected String isPopulatedAlternativeName = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String isNotPopulatedAlternativeName = "This variable should be set by your stylesheet";

	private AnnotatedDAO<PUDQuery> queryDAO;
	private AnnotatedDAO<PUDQueryInstance> queryInstanceDAO;

	private PUDQueryCRUD queryCRUD;

	private QueryParameterFactory<PUDQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<PUDQueryInstance, Integer> queryInstanceIDParamFactory;

	private PUDAlternative isPopulatedAlternative;
	private PUDAlternative isNotPopulatedAlternative;

	@InstanceManagerDependency(required = true)
	private LMSearchProvider lmSearchProvider;

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		isPopulatedAlternative = new PUDAlternative(isPopulatedAlternativeName, 1, 1);
		isNotPopulatedAlternative = new PUDAlternative(isNotPopulatedAlternativeName, 2, 2);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		// Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PUDQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		List<QueryParameterPopulator<?>> queryParameterPopulators = new ArrayList<QueryParameterPopulator<?>>();

		queryParameterPopulators.add(PUDQueryCRUD.SEARCH_SERVICE_POPULATOR);

		List<BeanStringPopulator<?>> typePopulators = new ArrayList<BeanStringPopulator<?>>();
		typePopulators.add(PUDQueryCRUD.SEARCH_SERVICE_POPULATOR);

		queryDAO = daoFactory.getDAO(PUDQuery.class, queryParameterPopulators, typePopulators);
		queryInstanceDAO = daoFactory.getDAO(PUDQueryInstance.class);

		queryCRUD = new PUDQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<PUDQuery>(PUDQuery.class), "PUDQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PUDQuery query = new PUDQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setIsPopulatedAlternative(isPopulatedAlternative);
		query.setIsNotPopulatedAlternative(isNotPopulatedAlternative);

		return query;

	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PUDQuery query = new PUDQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws Throwable {

		PUDQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setIsPopulatedAlternative(isPopulatedAlternative);
		query.setIsNotPopulatedAlternative(isNotPopulatedAlternative);

		return query;

	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PUDQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setIsPopulatedAlternative(isPopulatedAlternative);
		query.setIsNotPopulatedAlternative(isNotPopulatedAlternative);

		return query;

	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws Throwable {

		PUDQueryInstance queryInstance = null;

		// Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PUDQueryInstance();

			queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

			if (queryInstance.getQuery() == null) {

				return null;
			}

			queryInstance.set(descriptor);
			queryInstance.copyQueryValues();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}

			queryInstance.set(descriptor);

		}

		queryInstance.getQuery().setIsPopulatedAlternative(isPopulatedAlternative);
		queryInstance.getQuery().setIsNotPopulatedAlternative(isNotPopulatedAlternative);

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		return queryInstance;
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PUDQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;

	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PUDQueryInstance queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;

	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PUDQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);

	}

	@Override
	public void save(PUDQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}

	}

	@Override
	public void populate(PUDQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		PUDQuery query = queryInstance.getQuery();
		Integer queryID = query.getQueryID();

		String pud = req.getParameter("q" + queryID + "_propertyUnitDesignation");
		String pun = req.getParameter("q" + queryID + "_propertyUnitNumber");
		String poid = req.getParameter("q" + queryID + "_propertyObjectIdentity");
		String address = req.getParameter("q" + queryID + "_address");

		boolean empty = false;
		
		
		if (query.getUseAsResult() == null || UseAsResult.USE_ALL_AS_RESULT.equals(query.getUseAsResult())) {
			if (StringUtils.isEmpty(address) && StringUtils.isEmpty(pud)) {
				empty = true;
			}
		} 
		
		else if (UseAsResult.USE_ADDRESS_AS_RESULT.equals(query.getUseAsResult())) {
			if (StringUtils.isEmpty(address)) {
				empty = true;
			}
		} 
		
		else {
			if (StringUtils.isEmpty(pud)) {
				empty = true;
			}
		} 
		
		if (empty) {

			if (!allowPartialPopulation && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED) {

				queryInstance.reset(attributeHandler);

				throw new ValidationException(new ValidationError("RequiredQuery"));
			}

			queryInstance.reset(attributeHandler);

			return;
		}

		//TODO Validate submitted values against search

		queryInstance.setAddress(address);
		queryInstance.setPropertyUnitDesignation(pud);
		queryInstance.setPropertyUnitNumber(NumberUtils.toInt(pun));
		queryInstance.setPropertyObjectIdentity(poid);

		queryInstance.getQueryInstanceDescriptor().setPopulated(true);

		if (queryInstance.getQuery().isSetAsAttribute()) {

			queryInstance.resetAttribute(attributeHandler);
			queryInstance.setAttribute(attributeHandler);
		}
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	private PUDQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PUDQueryInstance> query = new HighLevelQuery<>(PUDQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);

	}

	private PUDQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<PUDQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private PUDQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PUDQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	@Override
	protected Class<PUDQueryInstance> getQueryInstanceClass() {

		return PUDQueryInstance.class;
	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		if (lmSearchProvider != null) {

			XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "lmSearchProviderURL", req.getContextPath() + lmSearchProvider.getFullAlias());
		}

		return doc;
	}
}
