package com.nordicpeak.flowengine.evaluators.weightcalculatedparagraphevaluator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.paragraphinfoquery.Paragraph;
import com.nordicpeak.flowengine.queries.paragraphinfoquery.ParagraphInfoQuery;

public class WeightCalculatedParagraphEvaluatorCRUD extends BaseEvaluatorCRUD<WeightCalculatedParagraphEvaluator, WeightCalculatedParagraphEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<WeightCalculatedParagraphEvaluator> POPULATOR = new AnnotatedRequestPopulator<WeightCalculatedParagraphEvaluator>(WeightCalculatedParagraphEvaluator.class);

	public WeightCalculatedParagraphEvaluatorCRUD(Class<WeightCalculatedParagraphEvaluator> beanClass, AnnotatedDAOWrapper<WeightCalculatedParagraphEvaluator, Integer> evaluatorDAO, WeightCalculatedParagraphEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "WeightCalculatedParagraphEvaluator", "weight calculated paragraph evaluator", null, callback);
	}

	@Override
	protected void validateUpdatePopulation(WeightCalculatedParagraphEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		if (bean.getFromValue() != null && bean.getToValue() != null && bean.getFromValue() > bean.getToValue()) {

			errors.add(new ValidationError("FromValueBiggerThanToValue"));
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
		
		bean.getEvaluatorDescriptor().setTargetQueryIDs(null);

	}

	@Override
	protected void appendUpdateFormData(WeightCalculatedParagraphEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		ParagraphInfoQuery query = (ParagraphInfoQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<Paragraph> paragraphs = query.getParagraphs();

		if (paragraphs != null) {

			Element paragraphsElement = doc.createElement("Paragraphs");
			updateTypeElement.appendChild(paragraphsElement);

			for (Paragraph paragraph : paragraphs) {

				paragraphsElement.appendChild(paragraph.toXML(doc));
			}
		}
	}

}
