package com.nordicpeak.flowengine.queries.paragraphinfoquery;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class ParagraphInfoQueryCRUD extends BaseQueryCRUD<ParagraphInfoQuery, ParagraphInfoQueryProviderModule> {

	private final RelationQuery UPDATE_SORT_RELATIONS = new RelationQuery(ParagraphInfoQuery.PARAGRAPHS_RELATION);
	
	protected AnnotatedDAOWrapper<ParagraphInfoQuery, Integer> queryDAO;
	
	public ParagraphInfoQueryCRUD(AnnotatedDAOWrapper<ParagraphInfoQuery, Integer> queryDAO, BeanRequestPopulator<ParagraphInfoQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, ParagraphInfoQueryProviderModule callback) {
		
		super(ParagraphInfoQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.queryDAO = queryDAO;
	}

	public ForegroundModuleResponse sort(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		ParagraphInfoQuery query = this.getRequestedBean(req, res, user, uriParser, UPDATE);

		if (query != null) {

			this.checkUpdateAccess(query, user, req, uriParser);

			if (req.getMethod().equalsIgnoreCase("POST")) {

				List<Paragraph> paragraphs = query.getParagraphs();

				if(!CollectionUtils.isEmpty(paragraphs)) {

					for(Paragraph paragraph : paragraphs) {

						String sortIndex = req.getParameter("sortorder_" + paragraph.getParagraphID());

						if(NumberUtils.isInt(sortIndex)) {

							paragraph.setSortIndex(NumberUtils.toInt(sortIndex));
						}
					}

					queryDAO.getAnnotatedDAO().update(query, UPDATE_SORT_RELATIONS);

					callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) query.getQueryDescriptor()), EventTarget.ALL);

					callback.redirectToQueryConfig(query, req, res);

					return null;
				}
			}

			Document doc = this.callback.createDocument(req, uriParser, user);
			Element updateTypeElement = doc.createElement("SortParagraphs");
			doc.getFirstChild().appendChild(updateTypeElement);

			updateTypeElement.appendChild(query.toXML(doc));

			return new SimpleForegroundModuleResponse(doc);
		}

		throw new URINotFoundException(uriParser);
	}
	
	@Override
	protected ParagraphInfoQuery populateFromUpdateRequest(ParagraphInfoQuery oldQuery, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		ParagraphInfoQuery query = super.populateFromUpdateRequest(oldQuery, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		if(query.isHideTitle() && query.getDescription() == null && !callback.isAllowEmptyDescriptionWhenHiddenTitle()) {
			
			throw new ValidationException(new ValidationError("DescriptionRequiredWhenTitleHidden"));
		}
		
		if(!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}
	
	@Override
	protected void appendShowFormData(ParagraphInfoQuery bean, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {

		XMLUtils.appendNewElement(doc, showTypeElement, "showFlowURL", callback.getFlowAdminModule().getFlowQueryRedirectURL(req, bean.getQueryDescriptor().getStep().getFlow().getFlowID()));
		
	}
	
	@Override
	protected ForegroundModuleResponse beanUpdated(ParagraphInfoQuery bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQueryDescriptor()), EventTarget.ALL);
		
		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQueryDescriptor().getName() + "\"", bean.getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean, req, res);

		return null;

	}	
	
	@Override
	protected List<Field> getBeanRelations() {

		return Arrays.asList(ParagraphInfoQuery.PARAGRAPHS_RELATION);
	}
}
