package com.nordicpeak.flowengine.queries.paragraphinfoquery;

import java.lang.reflect.Field;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToMany;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.NonColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@Table(name = "paragraph_info_query_instances")
@XMLElement
public class ParagraphInfoQueryInstance extends BaseQueryInstance implements NonColumnExportableQueryInstance{

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(ParagraphInfoQueryInstance.class, "query");
	
	public static final Field VISIBLE_PARAGRAPHS_RELATION = ReflectionUtils.getField(ParagraphInfoQueryInstance.class, "visibleParagraphs");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private ParagraphInfoQuery query;

	@DAOManaged
	@ManyToMany(linkTable="paragraph_info_query_instance_paragraphs")
	@XMLElement(fixCase=true)
	private List<Paragraph> visibleParagraphs;
	
	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		return element;
	}

	@Override
	public ParagraphInfoQuery getQuery() {

		return query;
	}

	public void setQuery(ParagraphInfoQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		if (query.isSetAsAttribute() && attributeHandler.isSet(query.getAttributeName()) && attributeHandler.getString(query.getAttributeName()).equals(query.getDescription())) {

			attributeHandler.removeAttribute(query.getAttributeName());
		}

		super.reset(attributeHandler);
	}

	
	public List<Paragraph> getVisibleParagraphs() {
	
		return visibleParagraphs;
	}

	
	public void setVisibleParagraphs(List<Paragraph> visibleParagraphs) {
	
		this.visibleParagraphs = visibleParagraphs;
	}

	public void init() {

		if(query.getParagraphs() != null){
			
			for(Paragraph paragraph : query.getParagraphs()){
				
				if(paragraph.isVisible()){
					
					this.visibleParagraphs = CollectionUtils.addAndInstantiateIfNeeded(this.visibleParagraphs, paragraph);
				}
			}
		}
	}

}
