package com.nordicpeak.flowengine.paymentlog;

import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.ss.util.WorkbookUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryOperators;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberStringyfier;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.threads.ExeptionHandlingRunnable;
import se.unlogic.standardutils.threads.ThreadUtils;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.paymentlog.beans.Account;
import com.nordicpeak.flowengine.paymentlog.beans.AccountNumberPart;
import com.nordicpeak.flowengine.paymentlog.beans.ChartOfAccounts;
import com.nordicpeak.flowengine.paymentlog.beans.Transaction;
import com.nordicpeak.flowengine.paymentlog.enums.AccountType;
import com.nordicpeak.flowengine.paymentlog.interfaces.TransactionFileExporter;

public class TransactionModule extends AnnotatedForegroundModule {

	public static final NumberStringyfier NUMBER_FORMATTER = new NumberStringyfier(2, 2, false);

	private static final Pattern TRANSACTION_DESCRIPTION_PATTERN = Pattern.compile("\n", Pattern.LITERAL);

	@XSLVariable(name = "i18n.Column.transactionID")
	private String columnTransactionID;

	@XSLVariable(name = "i18n.Column.added")
	private String columnAdded;

	@XSLVariable(name = "i18n.Column.sourceType")
	private String columnSourceType;

	@XSLVariable(name = "i18n.Column.sourceID")
	private String columnSourceID;

	@XSLVariable(name = "i18n.Column.amount")
	private String columnAmount;

	@XSLVariable(name = "i18n.Column.description")
	private String columnDescription;

	@XSLVariable(name = "i18n.Column.user")
	private String columnUser;

	@XSLVariable(name = "i18n.Column.sourceType.flowInstance")
	private String columnSourceTypeFlowInstance;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default instance list page size", description = "Default instance list page size", formatValidator = PositiveStringIntegerValidator.class, required = true)
	protected Integer defaultPageSize = 15;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default from date in days from today", description = "Default from date in days from today", formatValidator = PositiveStringIntegerValidator.class, required = true)
	protected Integer defaultFromDateInDays = 90;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Transaction exporter class", description = "The class to use when exporting transaction files")
	protected String transactionFileExporterClass;

	protected TransactionFileExporter transactionFileExporter;

	private AnnotatedDAO<Transaction> transactionDAO;
	private AnnotatedDAOWrapper<Account, Integer> accountDAOWrapper;
	private AnnotatedDAOWrapper<ChartOfAccounts, Integer> chartDAOWrapper;

	private QueryParameterFactory<Transaction, Account> transactionAccountParameterFactory;
	private QueryParameterFactory<Transaction, Timestamp> transactionAddedParameterFactory;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(TransactionModule.class, this)) {

			throw new RuntimeException("Unable to register module " + this.moduleDescriptor + " in global instance handler using key " + AccountModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AccountModule.class.getName(), new XMLDBScriptProvider(AccountModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		transactionDAO = daoFactory.getDAO(Transaction.class);
		transactionAccountParameterFactory = transactionDAO.getParamFactory("account", Account.class);
		transactionAddedParameterFactory = transactionDAO.getParamFactory("added", Timestamp.class);

		accountDAOWrapper = daoFactory.getDAO(Account.class).getWrapper(Integer.class);
		chartDAOWrapper = daoFactory.getDAO(ChartOfAccounts.class).getWrapper(Integer.class);
	}

	@Override
	protected void moduleConfigured() {

		if (transactionFileExporterClass != null) {

			try {
				this.transactionFileExporter = (TransactionFileExporter) ReflectionUtils.getInstance(transactionFileExporterClass);

			} catch (Exception e) {

				transactionFileExporter = null;
				log.error("Error creating instance of TransactionFileExporter " + transactionFileExporterClass, e);
			}

		} else {

			transactionFileExporter = null;
		}
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(TransactionModule.class, this);

		super.unload();
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(moduleDescriptor.toXML(doc));

		doc.appendChild(documentElement);
		return doc;
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return listCharts(req, res, user, uriParser, null);
	}

	public Account getAccount(Integer accountID, Field... relations) throws SQLException {

		HighLevelQuery<Account> query = new HighLevelQuery<>();
		query.addParameter(accountDAOWrapper.getParameterFactory().getParameter(accountID));

		if (relations != null) {
			query.addRelations(relations);
		}

		return accountDAOWrapper.getAnnotatedDAO().get(query);
	}

	public ChartOfAccounts getChart(Integer chartID, Field... relations) throws SQLException {

		HighLevelQuery<ChartOfAccounts> query = new HighLevelQuery<>();
		query.addParameter(chartDAOWrapper.getParameterFactory().getParameter(chartID));

		if (relations != null) {
			query.addRelations(relations);
		}

		return chartDAOWrapper.getAnnotatedDAO().get(query);
	}
	
	public ChartOfAccounts getChartForTransactionFile(Integer chartID) throws SQLException {

		HighLevelQuery<ChartOfAccounts> query = new HighLevelQuery<>();
		query.addParameter(chartDAOWrapper.getParameterFactory().getParameter(chartID));

		query.addRelations(ChartOfAccounts.ACCOUNT_NUMBER_PART_DEFINITIONS_RELATION, ChartOfAccounts.ACCOUNTS_RELATION, Account.ACCOUNT_NUMBER_PARTS_RELATION, AccountNumberPart.ACCOUNT_NUMBER_PART_DEFINITION_RELATION);
		query.addCachedRelation(AccountNumberPart.ACCOUNT_NUMBER_PART_DEFINITION_RELATION);

		return chartDAOWrapper.getAnnotatedDAO().get(query);
	}

	public ForegroundModuleResponse listCharts(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception, Throwable {

		log.info("User " + user + " listing charts");

		Document doc = createDocument(req, uriParser, user);
		Element listTypeElement = doc.createElement("ListCharts");
		doc.getFirstChild().appendChild(listTypeElement);

		XMLUtils.append(doc, listTypeElement, "Charts", chartDAOWrapper.getAll());

		if (!CollectionUtils.isEmpty(validationErrors)) {

			XMLUtils.append(doc, listTypeElement, validationErrors);
			listTypeElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	@WebPublic(alias = "accounts")
	public ForegroundModuleResponse listAccounts(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer chartID;
		ChartOfAccounts chart;

		if (uriParser.size() == 3 && (chartID = uriParser.getInt(2)) != null && (chart = getChart(chartID, ChartOfAccounts.ACCOUNTS_RELATION, Account.ACCOUNT_NUMBER_PARTS_RELATION, AccountNumberPart.ACCOUNT_NUMBER_PART_DEFINITION_RELATION)) != null) {

			return listAccounts(chart, req, res, user, uriParser, null);
		}

		return listCharts(req, res, user, uriParser, Collections.singletonList(new ValidationError("ChartNotFound")));
	}

	public ForegroundModuleResponse listAccounts(ChartOfAccounts chart, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception, Throwable {

		log.info("User " + user + " listing accounts for chart " + chart);

		Document doc = createDocument(req, uriParser, user);
		Element listTypeElement = doc.createElement("ListAccounts");
		doc.getFirstChild().appendChild(listTypeElement);

		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
		genDoc.addIgnoredField(Account.CHART_RELATION);

		if (!CollectionUtils.isEmpty(chart.getAccounts())) {
			for (Account account : chart.getAccounts()) {
				account.setChart(chart);
			}
		}

		listTypeElement.appendChild(chart.toXML(genDoc));

		if (transactionFileExporter != null) {

			XMLUtils.appendNewElement(doc, listTypeElement, "TransactionFileExporter");
		}

		XMLUtils.appendNewElement(doc, listTypeElement, "PageSize", defaultPageSize);
		XMLUtils.appendNewElement(doc, listTypeElement, "DefaultStartDate", DateUtils.getTodaysDatePlusMinusDays(-defaultFromDateInDays));

		if (!CollectionUtils.isEmpty(validationErrors)) {

			XMLUtils.append(doc, listTypeElement, validationErrors);
			listTypeElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	@WebPublic(alias = "transactions")
	public ForegroundModuleResponse listTransactions(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer accountID;
		Account account;

		if (uriParser.size() == 3 && (accountID = uriParser.getInt(2)) != null && (account = getAccount(accountID, Account.CHART_RELATION, Account.ACCOUNT_NUMBER_PARTS_RELATION, AccountNumberPart.ACCOUNT_NUMBER_PART_DEFINITION_RELATION)) != null) {

			return listTransactions(account, req, res, user, uriParser, null);
		}

		return listCharts(req, res, user, uriParser, Collections.singletonList(new ValidationError("AccountNotFound")));
	}

	public ForegroundModuleResponse listTransactions(Account account, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception, Throwable {

		log.info("User " + user + " listing transactions for account " + account);

		Document doc = createDocument(req, uriParser, user);
		Element listTypeElement = doc.createElement("ListTransactions");
		doc.getFirstChild().appendChild(listTypeElement);

		listTypeElement.appendChild(account.toXML(doc));

		XMLUtils.appendNewElement(doc, listTypeElement, "ServerURL", RequestUtils.getServerURL(req));
		XMLUtils.appendNewElement(doc, listTypeElement, "PageSize", defaultPageSize);
		XMLUtils.appendNewElement(doc, listTypeElement, "DefaultStartDate", DateUtils.getTodaysDatePlusMinusDays(-defaultFromDateInDays));

		if (!CollectionUtils.isEmpty(validationErrors)) {

			XMLUtils.append(doc, listTypeElement, validationErrors);
			listTypeElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	@WebPublic(alias = "charttransactions")
	public ForegroundModuleResponse getChartTransactionData(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());

		PrintWriter writer = res.getWriter();

		Integer chartID;
		ChartOfAccounts chart;

		if (uriParser.size() == 3 && (chartID = uriParser.getInt(2)) != null && (chart = getChart(chartID, ChartOfAccounts.ACCOUNTS_RELATION)) != null) {

			Date from = null;
			Date to = null;

			try {
				List<ValidationError> validationErrors = new ArrayList<>();

				from = ValidationUtils.validateParameter("from", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
				to = ValidationUtils.validateParameter("to", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);

				if (!validationErrors.isEmpty()) {

					writer.append("{\"rows\": [], \"error\": \"Invalid date parameters\"}");
					writer.flush();
					return null;
				}

				log.info("Sending transactions for " + chart + " from " + from + " to " + to + " to user " + user);

				List<Transaction> transactions = getTransactions(chart, from, to);
				sendTransactionData(writer, transactions);

				return null;

			} catch (Exception e) {

				writer.append("{\"rows\": [], \"error\": \"Internal error\"}");
				writer.flush();
				throw e;
			}

		} else {

			writer.append("{\"rows\": [], \"error\": \"No account parameter\"}");
			writer.flush();
			return null;
		}
	}

	@WebPublic(alias = "accounttransactions")
	public ForegroundModuleResponse getAccountTransactionData(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());

		PrintWriter writer = res.getWriter();

		Integer accountID;
		Account account;

		if (uriParser.size() == 3 && (accountID = uriParser.getInt(2)) != null && (account = getAccount(accountID)) != null) {

			Date from = null;
			Date to = null;

			try {
				List<ValidationError> validationErrors = new ArrayList<>();

				from = ValidationUtils.validateParameter("from", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
				to = ValidationUtils.validateParameter("to", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);

				if (!validationErrors.isEmpty()) {

					writer.append("{\"rows\": [], \"error\": \"Invalid date parameters\"}");
					writer.flush();
					return null;
				}

				log.info("Sending transactions for " + account + " from " + from + " to " + to + " to user " + user);

				List<Transaction> transactions = getTransactions(account, from, to);
				sendTransactionData(writer, transactions);

				return null;

			} catch (Exception e) {

				writer.append("{\"rows\": [], \"error\": \"Internal error\"}");
				writer.flush();
				throw e;
			}

		} else {

			writer.append("{\"rows\": [], \"error\": \"No account parameter\"}");
			writer.flush();
			return null;
		}
	}

	public void sendTransactionData(PrintWriter writer, List<Transaction> transactions) throws Throwable {

		writer.append("{\"rows\":[");

		if (transactions != null) {

			JsonArray row = new JsonArray(8);

			for (Iterator<Transaction> iterator = transactions.iterator(); iterator.hasNext();) {

				Transaction transaction = iterator.next();

				row.addNode(transaction.getTransactionID());

				if (transaction.getAccount() != null) {

					row.addNode(transaction.getAccount().getName());

				} else {

					row.addNode("");
				}

				row.addNode(DateUtils.DATE_TIME_FORMATTER.format(transaction.getAdded()));
				row.addNode(transaction.getSourceType());
				row.addNode(transaction.getSourceID());
				row.addNode(NUMBER_FORMATTER.format(transaction.getAmount()));

				if (transaction.getUser() != null) {

					String userString = transaction.getUser().getFirstname() + " " + transaction.getUser().getLastname();

					String citizenID;

					if (transaction.getUser().getAttributeHandler() != null && (citizenID = transaction.getUser().getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE)) != null) {

						userString += " (";
						userString += citizenID;
						userString += ")";
					}

					row.addNode(userString);

				} else {

					row.addNode(transaction.getUserString());
				}

				String description = transaction.getDescription();

				if (description != null) {

					description = TRANSACTION_DESCRIPTION_PATTERN.matcher(description).replaceAll(". ");
				}

				row.addNode(description);

				writer.append(row.toJson());

				row.clearNodes();

				if (iterator.hasNext()) {

					writer.append(",");
				}
			}
		}

		writer.append("]}");
		writer.flush();
	}

	protected List<Transaction> getTransactions(Account account, Date from, Date to) throws SQLException {

		HighLevelQuery<Transaction> query = new HighLevelQuery<>();
		query.addParameter(transactionAccountParameterFactory.getParameter(account));

		if (from != null) {
			query.addParameter(transactionAddedParameterFactory.getParameter(DateUtils.setTimeToMidnight(new Timestamp(from.getTime())), QueryOperators.BIGGER_THAN_OR_EQUALS));
		}

		if (to != null) {
			query.addParameter(transactionAddedParameterFactory.getParameter(DateUtils.setTimeToMaximum(new Timestamp(to.getTime())), QueryOperators.SMALLER_THAN_OR_EQUALS));
		}
		
		return transactionDAO.getAll(query);
	}

	protected List<Transaction> getTransactions(ChartOfAccounts chart, Date from, Date to) throws SQLException {

		if (chart.getAccounts() == null) {
			return null;
		}

		HighLevelQuery<Transaction> query = new HighLevelQuery<>();
		query.addParameter(transactionAccountParameterFactory.getWhereInParameter(chart.getAccounts()));

		if (from != null) {
			query.addParameter(transactionAddedParameterFactory.getParameter(DateUtils.setTimeToMidnight(new Timestamp(from.getTime())), QueryOperators.BIGGER_THAN_OR_EQUALS));
		}

		if (to != null) {
			query.addParameter(transactionAddedParameterFactory.getParameter(DateUtils.setTimeToMaximum(new Timestamp(to.getTime())), QueryOperators.SMALLER_THAN_OR_EQUALS));
		}

		query.addRelations(Transaction.ACCOUNT_RELATION);

		return transactionDAO.getAll(query);
	}

	@WebPublic(alias = "exportchartexcel")
	public ForegroundModuleResponse exportChartTransactionsToExcel(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer chartID;
		ChartOfAccounts chart;
		Date from = null;
		Date to = null;

		if (uriParser.size() == 3 && (chartID = uriParser.getInt(2)) != null && (chart = getChart(chartID, ChartOfAccounts.ACCOUNTS_RELATION)) != null) {

			List<ValidationError> validationErrors = new ArrayList<>();

			from = ValidationUtils.validateParameter("from", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
			to = ValidationUtils.validateParameter("to", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);

			if (!validationErrors.isEmpty()) {
				
				return listAccounts(chart, req, res, user, uriParser, validationErrors);
			}
			
			log.info("User " + user + " exporting transactions to Excel for " + chart + " from " + from + " to " + to);

			if (!CollectionUtils.isEmpty(chart.getAccounts())) {

				for (Account account : chart.getAccounts()) {
					
					if(account.getAccountType() != AccountType.BALANCE) {
						
						account.setTransactions(getTransactions(account, from, to));
					}
				}
			}

			return exportTransactionsToExcel(chart, res, user);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "exportaccountexcel")
	public ForegroundModuleResponse exportAccountTransactionsToExcel(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer accountID;
		Account account;
		Date from = null;
		Date to = null;

		if (uriParser.size() == 3 && (accountID = uriParser.getInt(2)) != null && (account = getAccount(accountID, Account.CHART_RELATION)) != null) {

			List<ValidationError> validationErrors = new ArrayList<>();

			from = ValidationUtils.validateParameter("from", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
			to = ValidationUtils.validateParameter("to", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);

			if (!validationErrors.isEmpty()) {
				
				return listTransactions(account, req, res, user, uriParser, validationErrors);
			}
			
			log.info("User " + user + " exporting transactions to Excel for " + account + " from " + from + " to " + to);

			account.setTransactions(getTransactions(account, from, to));

			ChartOfAccounts chart = account.getChart();
			chart.setAccounts(Collections.singletonList(account));

			return exportTransactionsToExcel(chart, res, user);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "exporttransactionfile")
	public ForegroundModuleResponse exportTransactionFile(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if (transactionFileExporter == null) {

			throw new URINotFoundException(uriParser);
		}

		Integer chartID;
		ChartOfAccounts chart;
		Date from = null;
		Date to = null;

		if (uriParser.size() == 3 && (chartID = uriParser.getInt(2)) != null && (chart = getChartForTransactionFile(chartID)) != null) {

			List<ValidationError> validationErrors = new ArrayList<>();

			from = ValidationUtils.validateParameter("from", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
			to = ValidationUtils.validateParameter("to", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);

			if (!validationErrors.isEmpty()) {
				
				return listAccounts(chart, req, res, user, uriParser, validationErrors);
			}
			
			log.info("User " + user + " exporting transaction file for " + chart + " from " + from + " to " + to);

			if (!CollectionUtils.isEmpty(chart.getAccounts())) {

				for (Account account : chart.getAccounts()) {
					
					if(account.getAccountType() != AccountType.BALANCE) {
						
						account.setTransactions(getTransactions(account, from, to));
					}
				}
			}

			return transactionFileExporter.exportTransactions(chart, res, user);
		}

		throw new URINotFoundException(uriParser);
	}

	public ForegroundModuleResponse exportTransactionsToExcel(final ChartOfAccounts chart, final HttpServletResponse res, final User user) throws Throwable {

		//Workaround for thread local usage in XMLBeans used by XSSF
		ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

			private Exception exception;

			@Override
			public void run() {

				Workbook workbook = null;

				try {
					workbook = new XSSFWorkbook();

					for (Account account : chart.getAccounts()) {

						int allowedNameLength = 31;

						String fullSheetName = chart.getName() + ": " + account.getName();
						String sheetName = StringUtils.substring(account.getName(), allowedNameLength, "...");

						sheetName = WorkbookUtil.createSafeSheetName(sheetName);

						Sheet sheet = workbook.createSheet(sheetName);

						sheet.setColumnWidth(0, 1500);

						Row row = null;

						int rowIndex = 0;
						int cellIndex = 0;

						row = sheet.createRow(rowIndex++);

						Font titleFont = workbook.createFont();
						titleFont.setBold(true);
						titleFont.setFontHeightInPoints((short) 16);

						CellStyle titleStyle = workbook.createCellStyle();
						titleStyle.setFont(titleFont);

						Font subTitleFont = workbook.createFont();
						subTitleFont.setBold(true);
						CellStyle subTitleStyle = workbook.createCellStyle();
						subTitleStyle.setFont(subTitleFont);

						CellStyle rightNumberStyle = workbook.createCellStyle();
						rightNumberStyle.setAlignment(HorizontalAlignment.RIGHT);

						createCell(row, 0, fullSheetName, titleStyle);

						row = sheet.createRow(rowIndex++);
						sheet.setColumnWidth(cellIndex, 2500);
						createCell(row, cellIndex++, columnTransactionID, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 5000);
						createCell(row, cellIndex++, columnAdded, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 4000);
						createCell(row, cellIndex++, columnSourceType, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 2500);
						createCell(row, cellIndex++, columnSourceID, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 3000);
						createCell(row, cellIndex++, columnAmount, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 10000);
						createCell(row, cellIndex++, columnUser, subTitleStyle);
						sheet.setColumnWidth(cellIndex, 15000);
						createCell(row, cellIndex++, columnDescription, subTitleStyle);

						sheet.setAutoFilter(CellRangeAddress.valueOf("A2:" + CellReference.convertNumToColString(cellIndex - 1) + "2"));

						if (account.getTransactions() != null) {

							// Create data rows
							for (Transaction transaction : account.getTransactions()) {

								row = sheet.createRow(rowIndex++);
								cellIndex = 0;

								row.createCell(cellIndex++).setCellValue(transaction.getTransactionID());
								row.createCell(cellIndex++).setCellValue(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(transaction.getAdded()));

								String sourceType = transaction.getSourceType();

								if ("flowinstance".equals(sourceType)) {
									sourceType = columnSourceTypeFlowInstance;
								}

								row.createCell(cellIndex++).setCellValue(sourceType);
								row.createCell(cellIndex++).setCellValue(transaction.getSourceID());

								Cell amountCell = row.createCell(cellIndex++);
								amountCell.setCellValue(transaction.getAmount().toPlainString());
								amountCell.setCellStyle(rightNumberStyle);

								if (transaction.getUser() != null) {

									String userString = transaction.getUser().getFirstname() + " " + transaction.getUser().getLastname();

									String citizenID;

									if (transaction.getUser().getAttributeHandler() != null && (citizenID = transaction.getUser().getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE)) != null) {

										userString += " (";
										userString += citizenID;
										userString += ")";
									}

									row.createCell(cellIndex++).setCellValue(userString);

								} else {

									row.createCell(cellIndex++).setCellValue(transaction.getUserString());
								}

								String description = transaction.getDescription();

								if (description != null) {

									row.createCell(cellIndex++).setCellValue(description);

								} else {

									cellIndex++;
								}
							}
						}
					}

					try {
						res.setHeader("Content-Disposition", "attachment;filename=\"" + FileUtils.toValidHttpFilename(chart.getName() + " " + DateUtils.DATE_FORMATTER.format(new Date(System.currentTimeMillis())) + ".xlsx") + "\"");
						res.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

						workbook.write(res.getOutputStream());

						res.getOutputStream().flush();

					} catch (Exception e) {

						log.info("Unable to send excel file to user " + user + ", " + e);
					}

				} catch (Exception e) {

					exception = e;

				} finally {

					CloseUtils.close(workbook);
				}
			}

			@Override
			public Exception getException() {

				return exception;
			}
		};

		ThreadUtils.runAndWait(runnable, "POI thread for user " + user, true);

		if (runnable.getException() != null) {

			throw runnable.getException();
		}

		return null;
	}

	protected static Cell createCell(Row row, int cellPos, Object value, CellStyle style) {

		Cell cell = row.createCell(cellPos);
		cell.setCellValue(value.toString());
		cell.setCellStyle(style);

		return cell;
	}

	public TransactionFileExporter getTransactionFileExporter() {

		return transactionFileExporter;
	}
}
