package com.nordicpeak.flowengine.queries.personquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurity12DigitsWithoutMinusPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;
import com.nordicpeak.personprovider.Person;
import com.nordicpeak.personprovider.PersonProvider;

public class PersonQueryProviderModule extends BaseQueryProviderModule<PersonQueryInstance> implements BaseQueryCRUDCallback {

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max lookups per flow session", description = "", required = true, formatValidator = PositiveStringIntegerPopulator.class)
	private Integer maxLookups = 3;

	@XSLVariable(prefix = "java.")
	private String exportFirstName;

	@XSLVariable(prefix = "java.")
	private String exportLastName;

	@XSLVariable(prefix = "java.")
	private String exportCareOf;

	@XSLVariable(prefix = "java.")
	private String exportAddress;

	@XSLVariable(prefix = "java.")
	private String exportZipCode;

	@XSLVariable(prefix = "java.")
	private String exportPostalAddress;

	@XSLVariable(prefix = "java.")
	private String exportCitizenID;

	@XSLVariable(prefix = "java.")
	private String exportEmail;
	
	@XSLVariable(prefix = "java.")
	private String exportPhone;
	
	@XSLVariable(prefix = "java.")
	protected String personSelectedAlternativeName = "This variable should be set by your stylesheet";
	
	@XSLVariable(prefix = "java.")
	protected String personNotFoundAlternativeName = "This variable should be set by your stylesheet";
	
	private AnnotatedDAO<PersonQuery> queryDAO;
	private AnnotatedDAO<PersonQueryInstance> queryInstanceDAO;

	private PersonQueryQueryCRUD queryCRUD;

	private QueryParameterFactory<PersonQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<PersonQueryInstance, Integer> queryInstanceIDParamFactory;

	@InstanceManagerDependency
	protected PersonProvider personProvider;
	
	private PersonAlternative personSelectedAlternative;
	private PersonAlternative personNotFoundAlternative;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		// Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PersonQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(PersonQuery.class);
		queryInstanceDAO = daoFactory.getDAO(PersonQueryInstance.class);

		queryCRUD = new PersonQueryQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(PersonQuery.class), "PersonQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}
	
	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);

		personSelectedAlternative = new PersonAlternative(personSelectedAlternativeName, 1, 1);
		personNotFoundAlternative = new PersonAlternative(personNotFoundAlternativeName, 2, 2);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PersonQuery query = new PersonQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		query.setPersonSelectedAlternative(personSelectedAlternative);
		query.setPersonNotFoundAlternative(personNotFoundAlternative);

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PersonQuery query = new PersonQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		PersonQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		query.setPersonSelectedAlternative(personSelectedAlternative);
		query.setPersonNotFoundAlternative(personNotFoundAlternative);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PersonQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		query.setPersonSelectedAlternative(personSelectedAlternative);
		query.setPersonNotFoundAlternative(personNotFoundAlternative);

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		PersonQueryInstance queryInstance;

		// Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PersonQueryInstance();

			queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

			if (queryInstance.getQuery() == null) {

				return null;
			}

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}

			queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

			if (queryInstance.getQuery() == null) {

				return null;
			}
		}
		
		queryInstance.getQuery().setPersonSelectedAlternative(personSelectedAlternative);
		queryInstance.getQuery().setPersonNotFoundAlternative(personNotFoundAlternative);

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private PersonQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<PersonQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private PersonQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PersonQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private PersonQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PersonQueryInstance> query = new HighLevelQuery<>(PersonQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(PersonQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(PersonQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		PersonQuery query = queryInstance.getQuery();

		boolean queryRequired = queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED;
		boolean fieldsRequired = queryRequired && !allowPartialPopulation;

		List<ValidationError> validationErrors = new ArrayList<>();

		String qParam = "q" + query.getQueryID() + "_citizenID";

		String citizenID = ValidationUtils.validateParameter(qParam, req, fieldsRequired, SwedishSocialSecurity12DigitsWithoutMinusPopulator.getPopulator(), validationErrors);

		if (citizenID != null) {
			
			String posterCitizenID = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);

			if (query.isBlockPosterSelection() && citizenID.equals(posterCitizenID)) {

				validationErrors.add(new ValidationError("BlockPosterSelection"));

			} else {

				if (!citizenID.equals(queryInstance.getCitizenID())) { // Not saved previously
	
					queryInstance.reset(attributeHandler);
	
					if (queryInstance.getCache().containsKey(citizenID)) {
	
						Person person = queryInstance.getCache().get(citizenID);
						queryInstance.setPerson(person);
	
					} else {
	
						try {
							Person person = personProvider.getPerson(citizenID);
	
							if (person != null) {
	
								queryInstance.setPerson(person);
	
							} else {
	
								queryInstance.setPersonNotFound(true);
								
								if (query.isAllowNoPersonFound()) {
									
									queryInstance.setCitizenID(citizenID);
									
								} else {
									
									validationErrors.add(new ValidationError("PersonNotFound"));
								}
							}
	
						} catch (Exception e) {
	
							log.error("Error looking up citizen ID " + citizenID, e);
							validationErrors.add(new ValidationError("ProviderError"));
						}
					}
				}
	
				if (query.isUseForMultipartSigning()) {
					
					String emailID = "q" + query.getQueryID() + "_email";
					String phoneID = "q" + query.getQueryID() + "_phone";
	
					if (StringUtils.isEmpty(req.getParameter(emailID)) && StringUtils.isEmpty(req.getParameter(phoneID))) {
						
						if (!allowPartialPopulation) {
							validationErrors.add(new ValidationError("EmailOrPhoneRequired"));
						}
						
					} else {
						
						String email = ValidationUtils.validateParameter(emailID, req, false, EmailPopulator.getPopulator(), validationErrors);
						String phone = ValidationUtils.validateParameter(phoneID, req, false, StringSwedishPhoneNumberPopulator.getPopulator(), validationErrors);
						
						queryInstance.setEmail(email);
						queryInstance.setPhone(phone);
					}
				}
			}
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		if (citizenID == null) {

			queryInstance.reset(attributeHandler);
			queryInstance.getQueryInstanceDescriptor().setPopulated(false);

		} else {

			queryInstance.getQueryInstanceDescriptor().setPopulated(true);
			
			if (query.isSetAsAttribute()) {
				
				queryInstance.resetAttribute(attributeHandler);
				queryInstance.setAttribute(attributeHandler);
			}
		}
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(PersonQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws Exception {

		return new PersonQueryRequestProcessor(personProvider, queryInstance, maxLookups, poster);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PersonQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PersonQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "MaxLookUps", maxLookups);

		return doc;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PersonQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, PersonQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<PersonQueryInstance> getQueryInstanceClass() {
		return PersonQueryInstance.class;
	}

	public String getExportFirstName() {
		return exportFirstName;
	}

	public String getExportLastName() {
		return exportLastName;
	}

	public String getExportCareOf() {
		return exportCareOf;
	}

	public String getExportAddress() {
		return exportAddress;
	}

	public String getExportZipCode() {
		return exportZipCode;
	}

	public String getExportPostalAddress() {
		return exportPostalAddress;
	}

	public String getExportCitizenID() {
		return exportCitizenID;
	}

	public String getExportEmail() {
		return exportEmail;
	}

	public String getExportPhone() {
		return exportPhone;
	}

}
