package com.nordicpeak.flowengine.queries.plotquery;

import java.io.IOException;
import java.net.URL;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.MultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.PasswordSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.threads.MutexKeyProvider;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xsl.XSLVariableReader;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SessionUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.plotquery.beans.PlotQuery;
import com.nordicpeak.flowengine.queries.plotquery.beans.PlotQueryInstance;
import com.nordicpeak.flowengine.queries.plotquery.clients.PlotQueryClient;
import com.nordicpeak.flowengine.queries.plotquery.clients.tokenhandler.ArcGisTokenHandler;
import com.nordicpeak.flowengine.queries.plotquery.eventmodule.beans.PlotEventType;
import com.nordicpeak.flowengine.queries.plotquery.eventmodule.events.PlotHistoryEvent;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.InvalidPlotParametersException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.InvalidResponseException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotCurrentlyNotBookableException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotLockException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotLockExpiredException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotLockedException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotNotBookableException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotSoldException;
import com.nordicpeak.flowengine.queries.plotquery.exceptions.PlotUnavailableException;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class PlotQueryProviderModule extends BaseQueryProviderModule<PlotQueryInstance> {

	@XSLVariable(prefix = "java.")
	private String connectionFeeNotIncludedText = "connectionFeeNotIncludedText";

	@XSLVariable(prefix = "java.")
	private String lockKeyText = "lockKeyText";

	private String plotSoldErrorText = "plotSoldErrorText";

	private String plotNotBookableErrorText = "plotNotBookableErrorText";

	private String plotCurrentlyNotBookableErrorText = "plotCurrentlyNotBookableErrorText";

	private String plotLockExpiredErrorText = "plotLockExpiredErrorText";

	private String plotLockedErrorText = "plotLockedErrorText";

	private String plotLockErrorText = "plotLockErrorText";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Search plot URL", description = "Search plot URL", required = true)
	private String searchPlotURL;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Update plot URL", description = "Update plot URL", required = true)
	private String updatePlotURL;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Referer header", description = "Referer header for integration requests")
	private String refererHeader;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Lock time", description = "The number of minutes a plot is locked", required = true)
	private Integer lockTime = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection", required = true)
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response", required = true)
	private Integer readTimeout = 10;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Ignore bookable field", description = "If checked, the bookable field will always be regarded as true")
	private boolean ignoreBookableField;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Sold field string mode", description = "If checked, handle sold field as string (Bokad/Ledig) instead of integer (1/0) when checking availability and marking plot as sold. Inverted check ('!Ledig' instead of '1')")
	private boolean soldFieldStringMode;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable access token", description = "Enable access token")
	private boolean tokenEnabled;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Token URL", description = "Token URL")
	private String tokenURL;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Token username", description = "Token username")
	private String tokenUsername;

	@ModuleSetting
	@PasswordSettingDescriptor(name = "Token password", description = "Token password")
	private String tokenPassword;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Token source IP", description = "Token source IP")
	private String tokenSourceIP;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable token handler logging", description = "Enable token handler logging")
	private boolean tokenLoggingEnabled;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable verbose API logging", description = "Enables verbose logging of requests and responses")
	private boolean verboseAPILoggingEnabled;

	@ModuleSetting(allowsNull = true)
	@MultiListSettingDescriptor(name = "Hidden fields", description = "Fields to hide in ShowQueryForm and ShowQueryValues", valueDescriptions = { "objectID", "connectionFee", "location", "comment" }, values = { "objectID", "connectionFee", "location", "comment" })
	List<String> hiddenFields;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Generate plot GUID", description = "If enabled, GUID will be generated in system instead of required as parameter")
	private boolean generatePlotGUID;

	private AnnotatedDAO<PlotQuery> queryDAO;
	private AnnotatedDAO<PlotQueryInstance> queryInstanceDAO;

	private PlotQueryCRUD queryCRUD;

	private QueryParameterFactory<PlotQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<PlotQueryInstance, Integer> queryInstanceIDParamFactory;

	private final MutexKeyProvider<Integer> mutexKeyProvider = new MutexKeyProvider<>();

	private PlotQueryClient client;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(PlotQueryProviderModule.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + PlotQueryProviderModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(PlotQueryProviderModule.class, this);

		super.unload();
	}

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		client = null;

		super.moduleConfigured();
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PlotQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(PlotQuery.class);
		queryInstanceDAO = daoFactory.getDAO(PlotQueryInstance.class);

		queryCRUD = new PlotQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(PlotQuery.class), "PlotQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PlotQuery query = new PlotQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PlotQuery query = new PlotQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));

		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		PlotQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PlotQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		PlotQueryInstance queryInstance;

		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PlotQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private PlotQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<PlotQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private PlotQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PlotQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private PlotQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PlotQueryInstance> query = new HighLevelQuery<>(PlotQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(PlotQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PlotQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PlotQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PlotQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, PlotQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}

	}

	@Override
	protected Class<PlotQueryInstance> getQueryInstanceClass() {

		return PlotQueryInstance.class;
	}

	public PlotQueryInstance validateAndLockPlot(User poster, PlotQueryInstance queryInstance) throws ValidationException {

		List<ValidationError> validationErrors = null;

		String loggingPrefix = getLoggingPrefix(queryInstance.getObjectID(), poster, queryInstance);

		try {

			if (!queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getObjectID() == null || queryInstance.getGUID() == null) {

				throw new InvalidPlotParametersException();
			}

			PlotEventType eventType = PlotEventType.LOCKED;

			synchronized (mutexKeyProvider.getKey(queryInstance.getObjectID())) {

				queryInstance = getClient().getPlotDataRequest(queryInstance, loggingPrefix);

				if (queryInstance.getRemoteLockKey() != null && queryInstance.getRemoteLockKey().equals(queryInstance.getLockKey())) {

					eventType = PlotEventType.LOCK_REFRESHED;
				}

				queryInstance = getClient().lockPlot(poster, queryInstance, loggingPrefix);
			}

			sendPlotHistoryEvent(queryInstance, poster, eventType, lockKeyText + ": " + queryInstance.getLockKey());

		} catch (PlotLockedException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotLockedError"));

			log.warn(loggingPrefix + "Requested a locked plot for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotLockedErrorText);

		} catch (PlotLockExpiredException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotLockExpiredError"));

			log.warn(loggingPrefix + "Requested a locked plot with an expired lock for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotLockExpiredErrorText);

		} catch (PlotSoldException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotSoldError"));

			log.warn(loggingPrefix + "Requested a sold plot for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotSoldErrorText);

		} catch (PlotNotBookableException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotNotBookableError"));

			log.warn(loggingPrefix + "Requested a not bookable plot for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotNotBookableErrorText);

		} catch (PlotCurrentlyNotBookableException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotCurrentlyNotBookableError"));

			log.warn(loggingPrefix + "Requested a currently not bookable plot for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotCurrentlyNotBookableErrorText);

		} catch (PlotUnavailableException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotUnavailableError"));

			log.warn(loggingPrefix + "Requested an unavailable plot for query (" + queryInstance + ")");

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED);

		} catch (InvalidPlotParametersException e) {

			validationErrors = Collections.singletonList(new ValidationError("UnableToGetDataError"));

			log.warn(loggingPrefix + "Requested plot query without valid parameters (" + queryInstance + ")");

		} catch (InvalidResponseException e) {

			validationErrors = Collections.singletonList(new ValidationError("UnableToGetDataError"));

			log.warn(loggingPrefix + "Invalid response getting data from endpoint for query " + queryInstance + "\n" + e.getMessage());

		} catch (IOException e) {

			validationErrors = Collections.singletonList(new ValidationError("UnableToGetDataError"));

			log.warn(loggingPrefix + "Error getting data from endpoint for query " + queryInstance, e);

		} catch (PlotLockException e) {

			validationErrors = Collections.singletonList(new ValidationError("PlotLockError"));

			log.warn(loggingPrefix + "Error locking plot for " + queryInstance + ")\n", e);

			sendPlotHistoryEvent(queryInstance, poster, PlotEventType.LOCK_FAILED, plotLockErrorText);
		}

		if (!CollectionUtils.isEmpty(validationErrors)) {

			throw new ValidationException(validationErrors);
		}

		return queryInstance;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 50)
	public void processEvent(SubmitEvent event, EventSource eventSource) {

		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();

		User poster = flowInstanceManager.getFlowInstance().getPoster();

		List<PlotQueryInstance> queryInstances = flowInstanceManager.getQueries(PlotQueryInstance.class);

		if (queryInstances != null) {

			for (PlotQueryInstance queryInstance : queryInstances) {

				String loggingPrefix = getLoggingPrefix(queryInstance.getObjectID(), flowInstanceManager.getFlowInstance().getPoster(), queryInstance);

				getClient().setPlotAsSold(queryInstance, loggingPrefix);

				sendPlotHistoryEvent(queryInstance, poster, PlotEventType.SUBMITTED);
			}
		}
	}

	@Override
	public QueryResponse getFormHTML(PlotQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (validationErrors == null) {

			if (searchPlotURL == null || updatePlotURL == null) {

				validationErrors = CollectionUtils.getList(new ValidationError("ModuleConfigurationError"));

			} else if (!requestMetadata.isManager()) {

				Integer objectID = (Integer) SessionUtils.getAttribute(PlotIntegrationConstants.QUERY_PARAM_OBJECT_ID, req);
				String guid = generatePlotGUID ? UUID.randomUUID().toString() : (String) SessionUtils.getAttribute(PlotIntegrationConstants.QUERY_PARAM_GUID, req);

				if (objectID != null) {

					queryInstance.setObjectID(objectID);
					queryInstance.setGUID(guid);

					SessionUtils.setAttribute(PlotIntegrationConstants.QUERY_PARAM_OBJECT_ID, null, req);
					SessionUtils.setAttribute(PlotIntegrationConstants.QUERY_PARAM_GUID, null, req);

					queryInstance.getQueryInstanceDescriptor().setPopulated(true);

					queryInstance.resetLockKey();
				}

				try {

					validateAndLockPlot(poster, queryInstance);

				} catch (ValidationException e) {

					validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, e.getErrors());
				}
			}

			queryInstance.setValidationErrors(validationErrors);
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	@Override
	protected void appendQueryInstance(PlotQueryInstance queryInstance, Document doc, Element targetElement, AttributeHandler attributeHandler) {

		super.appendQueryInstance(queryInstance, doc, targetElement, attributeHandler);

		if (hiddenFields != null) {

			Element hiddenFieldsElement = XMLUtils.appendNewElement(doc, targetElement, "HiddenFields");

			for (String field : hiddenFields) {

				XMLUtils.appendNewElement(doc, hiddenFieldsElement, "HiddenField", field);
			}
		}
	}

	@Override
	public void populate(PlotQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		validateAndLockPlot(poster, queryInstance);

		if (!CollectionUtils.isEmpty(queryInstance.getValidationErrors())) {

			throw new ValidationException(queryInstance.getValidationErrors());
		}

		if (queryInstance.getQuery().isSetAsAttribute()) {

			if (queryInstance.getQueryInstanceDescriptor().isPopulated()) {

				queryInstance.setAttributes(attributeHandler, connectionFeeNotIncludedText);

			} else {

				queryInstance.resetAttributes(attributeHandler);
			}
		}

		queryInstance.setValidationErrors(null);
	}

	public PlotQueryClient getClient() {

		if (client == null) {

			ArcGisTokenHandler tokenHandler = null;
			if (tokenEnabled && !StringUtils.isEmpty(tokenURL) && !StringUtils.isEmpty(tokenUsername) && !StringUtils.isEmpty(tokenPassword) && !StringUtils.isEmpty(tokenSourceIP)) {

				tokenHandler = new ArcGisTokenHandler(tokenURL, tokenUsername, tokenPassword, tokenSourceIP, connectionTimeout, readTimeout, tokenLoggingEnabled);
			}

			client = new PlotQueryClient(searchPlotURL, updatePlotURL, connectionTimeout, readTimeout, lockTime, refererHeader, soldFieldStringMode, ignoreBookableField, tokenHandler, verboseAPILoggingEnabled);
			//			client = new PlotQueryDummyClient(soldFieldStringMode, ignoreBookableField, new ArcGisTokenDummyHandler());
		}

		return client;
	}

	public void abortPlotBooking(Integer objectID) {

		getClient().abortPlotBooking(objectID, getLoggingPrefix(objectID));
	}

	private String getLoggingPrefix(Integer objectID) {

		return getLoggingPrefix(objectID, null, null);
	}

	private String getLoggingPrefix(Integer objectID, User user, QueryInstance queryInstance) {

		StringBuilder prefixSB = new StringBuilder();

		prefixSB.append("(objectID: " + objectID);

		if (user != null) {

			prefixSB.append(", user: " + user);
		}

		if (queryInstance != null) {

			Integer flowInstanceID = queryInstance.getQueryInstanceDescriptor().getFlowInstanceID();

			if (flowInstanceID != null) {

				prefixSB.append(", flowInstanceID: " + flowInstanceID);
			}
		}

		prefixSB.append(") ");

		return prefixSB.toString();
	}

	private void sendPlotHistoryEvent(PlotQueryInstance queryInstance, User poster, PlotEventType eventType) {

		sendPlotHistoryEvent(queryInstance, poster, eventType, null);
	}

	private void sendPlotHistoryEvent(PlotQueryInstance queryInstance, User poster, PlotEventType eventType, String details) {

		systemInterface.getEventHandler().sendEvent(PlotQueryInstance.class, new PlotHistoryEvent(queryInstance, poster, eventType, details), EventTarget.ALL);
	}

	@Override
	protected XSLVariableReader parseQueryXSLStyleSheet(URL styleSheetURL) {

		XSLVariableReader variableReader = super.parseQueryXSLStyleSheet(styleSheetURL);

		if (variableReader != null) {

			try {

				plotSoldErrorText = variableReader.getValue("i18n.PlotSoldErrorText");
				plotNotBookableErrorText = variableReader.getValue("i18n.PlotNotBookableErrorText");
				plotNotBookableErrorText = variableReader.getValue("i18n.PlotCurrentlyNotBookableErrorText");
				plotLockExpiredErrorText = variableReader.getValue("i18n.PlotLockExpiredErrorText");
				plotLockedErrorText = variableReader.getValue("i18n.PlotLockedErrorText");
				plotLockErrorText = variableReader.getValue("i18n.PlotLockErrorText");

			} catch (Exception e) {

				log.error("Unable to get values from query style sheet " + queryStyleSheet, e);
			}
		}

		return variableReader;
	}

}
