package com.nordicpeak.flowengine.queries.plotquery.beans;

import java.lang.reflect.Field;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.SubmitCheck;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.plotquery.PlotQueryProviderModule;

@Table(name = "plot_query_instances")
@XMLElement
public class PlotQueryInstance extends BaseQueryInstance implements SubmitCheck {

	private static final long serialVersionUID = -9021036086225846509L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(PlotQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private PlotQuery query;

	@DAOManaged
	@XMLElement
	private Integer objectID;

	@DAOManaged
	@XMLElement
	private Integer plotID;

	@DAOManaged
	@XMLElement
	private String guid;

	@DAOManaged
	@XMLElement
	private String timestamp;

	@DAOManaged
	@XMLElement
	private String propertyUnitDesignation;

	@DAOManaged
	@XMLElement
	private String address;

	@DAOManaged
	@XMLElement
	private Integer area;

	@DAOManaged
	@XMLElement
	private Integer connectionFee;

	@DAOManaged
	@XMLElement
	private Integer price;

	@DAOManaged
	@XMLElement
	private String location;

	@DAOManaged
	@XMLElement
	private String comment;

	private List<ValidationError> validationErrors;

	private String remoteLockKey;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ", objectID=" + objectID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewElement(doc, element, "ObjectID", objectID);
		XMLUtils.appendNewElement(doc, element, "PlotID", plotID);
		XMLUtils.appendNewElement(doc, element, "GUID", guid);
		XMLUtils.appendNewElement(doc, element, "Timestamp", timestamp);
		XMLUtils.appendNewElement(doc, element, "PropertyUnitDesignation", propertyUnitDesignation);
		XMLUtils.appendNewElement(doc, element, "Address", address);
		XMLUtils.appendNewElement(doc, element, "Area", area);
		XMLUtils.appendNewElement(doc, element, "ConnectionFee", connectionFee);
		XMLUtils.appendNewElement(doc, element, "Price", price);
		XMLUtils.appendNewElement(doc, element, "Location", location);
		XMLUtils.appendNewElement(doc, element, "Comment", comment);

		return element;
	}

	@Override
	public PlotQuery getQuery() {

		return query;
	}

	public void setQuery(PlotQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.plotID = null;
		this.propertyUnitDesignation = null;
		this.address = null;
		this.area = null;
		this.connectionFee = null;
		this.price = null;
		this.location = null;
		this.comment = null;

		super.reset(attributeHandler);
	}

	@Override
	public boolean isValidForSubmit(MutableAttributeHandler attributeHandler, User poster, QueryHandler queryHandler, FlowInstanceManager flowInstanceManager) throws Exception {

		PlotQueryProviderModule plotQueryProviderModule = queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), PlotQueryProviderModule.class);

		try {

			plotQueryProviderModule.validateAndLockPlot(poster, this);

		} catch (ValidationException e) {

			return false;
		}

		return true;
	}

	public Integer getObjectID() {

		return objectID;
	}

	public void setObjectID(Integer objectID) {

		this.objectID = objectID;
	}

	public Integer getPlotID() {

		return plotID;
	}

	public void setPlotID(Integer plotID) {

		this.plotID = plotID;
	}

	public String getGUID() {

		return guid;
	}

	public void setGUID(String guid) {

		this.guid = guid;
	}

	public String getTimestamp() {

		return timestamp;
	}

	public void setTimestamp(String timestamp) {

		this.timestamp = timestamp;
	}

	public String getPropertyUnitDesignation() {

		return propertyUnitDesignation;
	}

	public void setPropertyUnitDesignation(String propertyUnitDesignation) {

		this.propertyUnitDesignation = propertyUnitDesignation;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public Integer getArea() {

		return area;
	}

	public void setArea(Integer area) {

		this.area = area;
	}

	public Integer getConnectionFee() {

		return connectionFee;
	}

	public void setConnectionFee(Integer connectionFee) {

		this.connectionFee = connectionFee;
	}

	public Integer getPrice() {

		return price;
	}

	public void setPrice(Integer price) {

		this.price = price;
	}

	public String getLocation() {

		return location;
	}

	public void setLocation(String location) {

		this.location = location;
	}

	public String getComment() {

		return comment;
	}

	public void setComment(String comment) {

		this.comment = comment;
	}

	public List<ValidationError> getValidationErrors() {

		return validationErrors;
	}

	public void setValidationErrors(List<ValidationError> validationErrors) {

		this.validationErrors = validationErrors;
	}

	public String getRemoteLockKey() {

		return remoteLockKey;
	}

	public void setRemoteLockKey(String remoteLockKey) {

		this.remoteLockKey = remoteLockKey;
	}

	public void resetLockKey() {

		timestamp = null;
	}

	public String getLockKey() {

		if (timestamp == null || guid == null) {

			return null;
		}

		return timestamp + "|" + guid;
	}

	public void setAttributes(MutableAttributeHandler attributeHandler, String connectionFeeNotIncludedText) {

		if (query.getAttributeName() != null) {

			attributeHandler.setAttribute(query.getAttributeName() + ".objectid", objectID);
			attributeHandler.setAttribute(query.getAttributeName() + ".timestamp", timestamp);
			attributeHandler.setAttribute(query.getAttributeName() + ".propertyUnitDesignation", propertyUnitDesignation);
			attributeHandler.setAttribute(query.getAttributeName() + ".address", address);
			attributeHandler.setAttribute(query.getAttributeName() + ".area", area);
			attributeHandler.setAttribute(query.getAttributeName() + ".connectionFee", connectionFee != null ? connectionFee : connectionFeeNotIncludedText);
			attributeHandler.setAttribute(query.getAttributeName() + ".price", price);
			attributeHandler.setAttribute(query.getAttributeName() + ".location", location);
			attributeHandler.setAttribute(query.getAttributeName() + ".comment", comment);
		}
	}

	public void resetAttributes(MutableAttributeHandler attributeHandler) {

		if (query.getAttributeName() != null) {

			attributeHandler.removeAttribute(query.getAttributeName() + ".objectid");
			attributeHandler.removeAttribute(query.getAttributeName() + ".timestamp");
			attributeHandler.removeAttribute(query.getAttributeName() + ".propertyUnitDesignation");
			attributeHandler.removeAttribute(query.getAttributeName() + ".address");
			attributeHandler.removeAttribute(query.getAttributeName() + ".area");
			attributeHandler.removeAttribute(query.getAttributeName() + ".connectionFee");
			attributeHandler.removeAttribute(query.getAttributeName() + ".price");
			attributeHandler.removeAttribute(query.getAttributeName() + ".location");
			attributeHandler.removeAttribute(query.getAttributeName() + ".comment");
		}
	}
}
