package com.nordicpeak.flowengine.queries.plotquery.daos;

import java.sql.SQLException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.List;

import javax.sql.DataSource;

import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryOperators;
import se.unlogic.standardutils.dao.QueryParameterFactory;

import com.nordicpeak.flowengine.queries.plotquery.eventmodule.beans.PlotEvent;

public class PlotEventDAO extends AnnotatedDAO<PlotEvent> {

	private QueryParameterFactory<PlotEvent, Long> addedParamFactory;

	public PlotEventDAO(DataSource dataSource, Class<PlotEvent> beanClass, AnnotatedDAOFactory daoFactory) {

		super(dataSource, beanClass, daoFactory);

		addedParamFactory = getParamFactory(PlotEvent.FIELD_ADDED, Long.class);
	}

	public List<PlotEvent> getInDateRange(LocalDateTime fromDate, LocalDateTime toDate) throws SQLException {

		HighLevelQuery<PlotEvent> query = new HighLevelQuery<>();

		if (fromDate != null) {

			Long fromDateMillis = fromDate.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();

			query.addParameter(addedParamFactory.getParameter(fromDateMillis, QueryOperators.BIGGER_THAN_OR_EQUALS));
		}

		if (toDate != null) {

			Long toDateMillis = toDate.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();

			query.addParameter(addedParamFactory.getParameter(toDateMillis, QueryOperators.SMALLER_THAN_OR_EQUALS));
		}

		return getAll(query);
	}
}
