package com.nordicpeak.flowengine.queries.predefineduserquery.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;

@XMLElement
@Table(name = "predefined_user_query_selected_dropdown_attributes")
public class SelectedDropDownAttribute extends GeneratedElementable implements XMLParserPopulateable, Serializable {
	
	private static final long serialVersionUID = -5407922522706409574L;

	@DAOManaged(columnName = "queryID")
	@Key
	@ManyToOne
	private PredefinedUserQuery internalContactDetailQuery;
	
	@DAOManaged
	@Key
	@XMLElement
	private String name;
	
	@DAOManaged
	@XMLElement
	private String displayName;
	
	public SelectedDropDownAttribute() {}
	
	public SelectedDropDownAttribute(String name, String displayName) {
		
		if (name == null) {
			
			throw new NullPointerException("name cannot be null");
		}
		
		this.name = name;
		
		if (displayName == null) {
			
			this.displayName = name;
			
		} else {
			
			this.displayName = displayName;
		}
	}
	
	public SelectedDropDownAttribute(InternalUserQueriesAttributeDescriptor attributeDescriptor) {
		
		this(attributeDescriptor.getName(), attributeDescriptor.getDisplayName());
	}
	
	public String getDisplayName() {
		
		return displayName;
	}
	
	public String getName() {
		
		return name;
	}
	
	@Override
	public String toString() {
		
		return name + " (display name: " + displayName + ")";
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		name = XMLValidationUtils.validateParameter("name", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		displayName = XMLValidationUtils.validateParameter("displayName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
}
