<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>
	
	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/common/js/queryadmin.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<div id="PriceTableQueryProvider" class="contentitem">
		
			<xsl:apply-templates select="validationError"/>
			<xsl:apply-templates select="ShowPriceTableQuery"/>
			<xsl:apply-templates select="UpdatePriceTableQuery"/>
			<xsl:apply-templates select="AddPriceTableRow"/>
			<xsl:apply-templates select="UpdatePriceTableRow"/>
			<xsl:apply-templates select="SortRows"/>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="ShowPriceTableQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateQuery" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="PriceTableQuery/QueryDescriptor/name" /></h1>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.BaseInfo"/></legend>
			
			<div class="floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatequery/{PriceTableQuery/queryID}" title="{$i18n.UpdateBaseInformation}">
					<img class="alignbottom" src="{$imagePath}/pen.png"/>
				</a>
			</div>
			
			<xsl:call-template name="createCommonShowFields">
				<xsl:with-param name="element" select="PriceTableQuery" />
			</xsl:call-template>
			
			<xsl:if test="not(PriceTableQuery/hideRowLabels = 'true')">
				<div class="clearboth floatleft bigmarginbottom">
					<label class="floatleft clearboth"><xsl:value-of select="$i18n.RowsLabel" /></label>
					<div class="floatleft full">
						<xsl:value-of select="PriceTableQuery/rowsLabel" disable-output-escaping="yes" />
				  </div>
				</div>
			</xsl:if>
			
			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.Query.ShowComments" /></label>
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="PriceTableQuery/showComments = 'true'">
							<xsl:value-of select="$i18n.Yes" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.No" />
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
			<div class="clearboth floatleft bigmarginbottom">
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="PriceTableQuery/decimalSetting = 'ALLOW'">
							<xsl:value-of select="$i18n.DecimalSetting.Allow" />
						</xsl:when>
						<xsl:when test="PriceTableQuery/decimalSetting = 'DISALLOW'">
							<xsl:value-of select="$i18n.DecimalSetting.Disallow" />
						</xsl:when>
						<xsl:when test="PriceTableQuery/decimalSetting = 'FORCE'">
							<xsl:value-of select="$i18n.DecimalSetting.Force" />
						</xsl:when>
					</xsl:choose>
				</div>
			</div>
			
			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.AllowNegative" /></label>
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="PriceTableQuery/allowNegative = 'true'">
							<xsl:value-of select="$i18n.Yes" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.No" />
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>

			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.HideEmptyRows" /></label>
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="PriceTableQuery/hideEmptyRows = 'true'">
							<xsl:value-of select="$i18n.Yes" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.No" />
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Rows"/></legend>
			
			<xsl:if test="PriceTableQuery/Rows/Row">
				
				<div class="floatleft full">
				
					<xsl:apply-templates select="PriceTableQuery/Rows/Row" mode="list" />
			
				</div>
			
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sortrows/{PriceTableQuery/queryID}" title="{$i18n.SortRows.Title}">
						<xsl:value-of select="$i18n.SortRows.Title"/>
						<img class="alignbottom marginleft" src="{$commonImagePath}/move.png"/>
					</a>
				</div>
				
			</xsl:if>
			
			<div class="clearboth floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addrow/{PriceTableQuery/queryID}" title="{$i18n.AddRow}">
					<xsl:value-of select="$i18n.AddRow"/>
					<img class="alignbottom marginleft" src="{$commonImagePath}/add.png"/>
				</a>
			</div>
			
		</fieldset>
		
		<div class="floatright margintop clearboth">
			<input type="button" value="{$i18n.Done}" onclick="window.location = '{showFlowURL}'" />
		</div>
		
	</xsl:template>
		
	<xsl:template match="Row" mode="list">
		
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<div class="padding floatleft thirty">
				<div><b><xsl:value-of select="label" /></b></div>
			</div>
			
			<div class="padding floatleft text-align-center" style="width: 80px;">
				<xsl:if test="required='true'">
					<div><b><xsl:value-of select="$i18n.Required" /></b></div>
				</xsl:if>
			</div>
			
			<div class="padding floatleft" style="width: 100px;">
				<div><b><xsl:value-of select="formattedPrice" />&#160;kr</b></div>
			</div>
			
			<div class="padding floatright">
				<div class="floatright marginleft marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deleterow/{rowID}" onclick="return confirm('{$i18n.DeleteRow.Confirm}: {label}?');" title="{$i18n.DeleteRow}: {label}">
						<img src="{$imagePath}/delete.png"/>
					</a>
				</div>
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updaterow/{rowID}" title="{$i18n.UpdateRow}: {label}">
						<img src="{$imagePath}/pen.png"/>
					</a>
				</div>
			</div>
			
		</div>
				
	</xsl:template>
	
	<xsl:template match="AddPriceTableRow">
	
		<h1><xsl:value-of select="$i18n.AddRow" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
	
		<form id="addRowForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
			
			<xsl:call-template name="showRowForm" />
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template match="UpdatePriceTableRow">
	
		<h1><xsl:value-of select="$i18n.UpdateRow" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="Row/label" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="udpateRowForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="showRowForm">
				<xsl:with-param name="element" select="Row" />
			</xsl:call-template>
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template name="showRowForm">
		
		<xsl:param name="element" select="null" />
		
		<div class="floatleft full bigmarginbottom">
			<label for="label" class="floatleft clearboth"><xsl:value-of select="$i18n.Label" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'label'"/>
					<xsl:with-param name="name" select="'label'"/>
					<xsl:with-param name="title" select="$i18n.Label"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'required'" />
					<xsl:with-param name="name" select="'required'" />
					<xsl:with-param name="element" select="$element" /> 
					<xsl:with-param name="class" select="'vertical-align-middle'" />
				</xsl:call-template>
					
				<label for="required">
					<xsl:value-of select="$i18n.Required" />
				</label>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="price" class="floatleft clearboth"><xsl:value-of select="$i18n.Price" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'price'"/>
					<xsl:with-param name="name" select="'price'"/>
					<xsl:with-param name="title" select="$i18n.Price"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="UpdatePriceTableQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateBaseInformation" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="updatePriceTableQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="PriceTableQuery" />
				<xsl:with-param name="flowTypeID" select="FlowType/flowTypeID" />
			</xsl:call-template>
			
			<div class="floatleft full bigmarginbottom">
				<label for="helpText" class="floatleft clearboth"><xsl:value-of select="$i18n.RowsLabel" /></label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'rowsLabel'"/>
						<xsl:with-param name="name" select="'rowsLabel'"/>
						<xsl:with-param name="title" select="$i18n.RowsLabel"/>
						<xsl:with-param name="element" select="PriceTableQuery" />
					</xsl:call-template>
			  </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showComments'"/>
					<xsl:with-param name="name" select="'showComments'"/>
					<xsl:with-param name="title" select="$i18n.Query.ShowComments"/>
					<xsl:with-param name="element" select="PriceTableQuery" />
				</xsl:call-template>
				
				<label for="showComments"><xsl:value-of select="$i18n.Query.ShowComments" /></label>
			</div>
			
			<xsl:variable name="decimalSettingOptions">
				<option>
					<name><xsl:value-of select="$i18n.DecimalSetting.Allow"/></name>
					<value>ALLOW</value>
				</option>
				<option>
					<name><xsl:value-of select="$i18n.DecimalSetting.Force"/></name>
					<value>FORCE</value>
				</option>
				<option>
					<name><xsl:value-of select="$i18n.DecimalSetting.Disallow"/></name>
					<value>DISALLOW</value>
				</option>
			</xsl:variable>			
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'decimalSetting'"/>
						<xsl:with-param name="name" select="'decimalSetting'"/>
						<xsl:with-param name="labelElementName" select="'name'" />					
						<xsl:with-param name="valueElementName" select="'value'" />
						<xsl:with-param name="element" select="exsl:node-set($decimalSettingOptions)/option" />
						<xsl:with-param name="selectedValue" select="PriceTableQuery/decimalSetting" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowNegative'"/>
					<xsl:with-param name="name" select="'allowNegative'"/>
					<xsl:with-param name="title" select="$i18n.AllowNegative"/>
					<xsl:with-param name="element" select="PriceTableQuery" />
				</xsl:call-template>
				
				<label for="allowNegative"><xsl:value-of select="$i18n.AllowNegative" /></label>
			</div>

			<div class="floatleft full bigmarginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hideEmptyRows'"/>
					<xsl:with-param name="name" select="'hideEmptyRows'"/>
					<xsl:with-param name="title" select="$i18n.HideEmptyRows"/>
					<xsl:with-param name="element" select="PriceTableQuery" />
				</xsl:call-template>
				
				<label for="hideEmptyRows"><xsl:value-of select="$i18n.HideEmptyRows" /></label>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<h2><xsl:value-of select="$i18n.AdvancedSettings" /></h2>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setAsAttribute'" />
						<xsl:with-param name="name" select="'setAsAttribute'" />
						<xsl:with-param name="element" select="PriceTableQuery" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="setAsAttribute">
						<xsl:value-of select="$i18n.setAsAttribute" />
					</label>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="amountSumAttributeName" class="floatleft clearboth"><xsl:value-of select="$i18n.amountSumAttributeName" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'amountSumAttributeName'"/>
						<xsl:with-param name="name" select="'amountSumAttributeName'"/>
						<xsl:with-param name="title" select="$i18n.amountSumAttributeName"/>
						<xsl:with-param name="element" select="PriceTableQuery" />
						<xsl:with-param name="maxlength" select="'255'"/>
					</xsl:call-template>
			    </div>
			</div>
			
			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
		
	</xsl:template>

	<xsl:template match="SortRows">
		
		<h1><xsl:value-of select="$i18n.SortRows" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="PriceTableQuery/QueryDescriptor/name" /></h1>
		
		<form id="sortRowsForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<div class="floatleft full sortable">
							
				<xsl:apply-templates select="PriceTableQuery/Rows/Row" mode="sort" />
							
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>
		
	</xsl:template>
	
	<xsl:template match="Row" mode="sort">
	
		<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
	
		<div class="floatleft hover border full marginbottom lightbackground cursor-move border-radius">
			<div class="padding">
				<img class="vertical-align-middle marginright" src="{$commonImagePath}/move.png" title="{$i18n.MoveAlternative}"/>
				<xsl:value-of select="label" />
				<xsl:call-template name="createHiddenField">
					<xsl:with-param name="id" select="concat('sortorder_', rowID)" />
					<xsl:with-param name="name" select="concat('sortorder_', rowID)" />
					<xsl:with-param name="value" select="sortIndex" />
					<xsl:with-param name="class" select="'sortorder'" />
					<xsl:with-param name="requestparameters" select="//requestparameters" />
				</xsl:call-template>
			</div>
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'UpdateFailedPriceTableQueryNotFound']">
		
		<p class="error">
			<xsl:value-of select="$i18n.PriceTableQueryNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PriceMayNotBeNegative']">
		
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.PriceMayNotBeNegative" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />
	
		<xsl:choose>
			<xsl:when test="$fieldName = 'label'">
				<xsl:value-of select="$i18n.label" />
			</xsl:when>
			<xsl:when test="$fieldName = 'helpText'">
				<xsl:value-of select="$i18n.helpText" />
			</xsl:when>
			<xsl:when test="$fieldName = 'rowsLabel'">
				<xsl:value-of select="$i18n.RowsLabel" />
			</xsl:when>
			<xsl:when test="$fieldName = 'price'">
				<xsl:value-of select="$i18n.Price" />
			</xsl:when>
			<xsl:when test="$fieldName = 'amountSumAttributeName'">
				<xsl:value-of select="$i18n.amountSumAttributeName" />
			</xsl:when>		
			<xsl:when test="$fieldName = 'decimalSetting'">
				<xsl:value-of select="$i18n.DecimalSetting" />
			</xsl:when>		
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>