<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/pricetablequery.js?v=3
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/pricetablequery.css?v=1
	</xsl:variable>
	
	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
		
		<div class="query pricetablequery">
			<xsl:attribute name="class">
				<xsl:text>query pricetablequery</xsl:text>
				<xsl:if test="PriceTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper tablequery-heading">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="PriceTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="PriceTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="PriceTableQueryInstance/PriceTableQuery/queryID" />
					</xsl:call-template>
				</div>

				<xsl:if test="PriceTableQueryInstance/PriceTableQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="PriceTableQueryInstance/PriceTableQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<table class="pricetable full">
				
					<thead>
						<tr>
							<th class="paddingright">
								<xsl:value-of select="PriceTableQueryInstance/PriceTableQuery/rowsLabel"/>
							</th>
							<th class="paddingright">
								<xsl:value-of select="$i18n.Column.Count"/>
							</th>
							<th class="paddingright">
								<xsl:value-of select="$i18n.Column.Sum"/>
							</th>
							<th/>
							<xsl:if test="PriceTableQueryInstance/PriceTableQuery/showComments = 'true'">
								<th class="paddingright">
									<xsl:value-of select="$i18n.Column.Comment"/>
								</th>
							</xsl:if>
						</tr>
					</thead>
					
					<tbody>
					
						<xsl:apply-templates select="PriceTableQueryInstance/PriceTableQuery/Rows/Row" mode="tbody-show" />
						
					</tbody>
					
					<tfoot>
					
						<tr data-title="{$i18n.Summary}">
							
							<td class="paddingright"><strong><xsl:value-of select="$i18n.Summary"/></strong></td>
							
							<td class="padding bigpaddingright text-align-right summary" data-count-summarise="true" data-title="{$i18n.Column.Count}">
								<xsl:value-of select="PriceTableQueryInstance/count"/>
							</td>
							
							<td class="padding bigpaddingright text-align-right summary" data-sum-summarise="true" data-title="{$i18n.Column.Sum}">
								<xsl:value-of select="PriceTableQueryInstance/sum"/>
							</td>
							
							<td class="padding" />
							
							<xsl:if test="PriceTableQueryInstance/PriceTableQuery/showComments = 'true'">
								<td class="padding" />
							</xsl:if>
							
						</tr>
					
					</tfoot>
					
				</table>
				
			</article>
			
		</div>
		
	</xsl:template>		
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', PriceTableQueryInstance/PriceTableQuery/queryID)" />
	
		<div class="query pricetablequery" id="{$queryID}" data-summary-url="{../ModuleURL}/getsummary/{PriceTableQueryInstance/PriceTableQuery/queryID}">
			<xsl:attribute name="class">
				<xsl:text>query pricetablequery</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="PriceTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
			
			<xsl:if test="ValidationErrors/validationError">
			
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<span/>
						<div class="marker"></div>
					</div>
				</div>
			
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper tablequery-heading">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="PriceTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="PriceTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="PriceTableQueryInstance/PriceTableQuery/helpText">
						<xsl:apply-templates select="PriceTableQueryInstance/PriceTableQuery/helpText" />
					</xsl:if>
					
				</div>

				<xsl:if test="PriceTableQueryInstance/PriceTableQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="PriceTableQueryInstance/PriceTableQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<table>
				
					<xsl:attribute name="class">
						<xsl:text>pricetable full</xsl:text>
						
						<xsl:if test="PriceTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
							<xsl:text> required</xsl:text>
						</xsl:if>
					</xsl:attribute>
				
					<thead>
						<tr>
							<th class="paddingright">
								<xsl:value-of select="PriceTableQueryInstance/PriceTableQuery/rowsLabel"/>
							</th>
							<th class="paddingright">
								<xsl:value-of select="$i18n.Column.Count"/>
							</th>
							<th class="paddingright">
								<xsl:value-of select="$i18n.Column.Sum"/>
							</th>
							<xsl:if test="PriceTableQueryInstance/PriceTableQuery/showComments = 'true'">
								<th class="paddingright">
									<xsl:value-of select="$i18n.Column.Comment"/>
								</th>
							</xsl:if>
						</tr>
					</thead>
					
					<tbody>
					
						<xsl:apply-templates select="PriceTableQueryInstance/PriceTableQuery/Rows/Row" mode="tbody" />
						
					</tbody>
					
					<tfoot>
					
						<tr data-title="{$i18n.Summary}">
							
							<td class="paddingright">
								<strong>
									<xsl:value-of select="$i18n.Summary"/>
								</strong>
							</td>
							
							<td class="padding bigpaddingright text-align-right summary" data-count-summarise="true" data-title="{$i18n.Column.Count}">
								<span>
									<xsl:value-of select="PriceTableQueryInstance/count"/>
								</span>
							</td>
							
							<td class="padding bigpaddingright text-align-right summary" data-sum-summarise="true" data-title="{$i18n.Column.Sum}">
								<span>
									<xsl:value-of select="PriceTableQueryInstance/sum"/>
								</span>
							</td>
							
							<xsl:if test="PriceTableQueryInstance/PriceTableQuery/showComments = 'true'">
								<td class="padding bigpaddingright"/>
							</xsl:if>
							
						</tr>
							
					</tfoot>
					
				</table>
		
			</article>
			
		</div>
		
		<script type="text/javascript">$(function(){initPriceTableQuery('<xsl:value-of select="PriceTableQueryInstance/PriceTableQuery/queryID" />');});</script>
		
	</xsl:template>
	
	<xsl:template match="Row" mode="tbody">
	
		<xsl:variable name="rowID" select="rowID"/>
	
		<xsl:variable name="tdClass">
			<xsl:value-of select="'paddingright'"/>
			
			<xsl:if test="required = 'true'">
				<xsl:value-of select="' required'"/>
			</xsl:if>
		</xsl:variable>
		
		<tr data-price="{price}">
		
			<td class="{$tdClass}">
				<strong>
					<xsl:value-of select="label"/>
				</strong>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="formattedPrice"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:text>kr</xsl:text>
			</td>
			
			<xsl:variable name="name">
				<xsl:text>q</xsl:text>
				<xsl:value-of select="../../queryID"/>
				<xsl:text>_</xsl:text>
				<xsl:value-of select="rowID"/>
			</xsl:variable>
			
			<td class="summary" data-title="{$i18n.Column.Count}" data-sum-summarise="true">
			
				<xsl:variable name="class">
			
					<xsl:if test="../../../../ValidationErrors/validationError[fieldName = $name]">
						<xsl:text> invalid input-error</xsl:text>
					</xsl:if>
					
					<xsl:text> required</xsl:text>
					
				</xsl:variable>
			
				<xsl:attribute name="class">
					<xsl:text>paddingright</xsl:text>
					<xsl:value-of select="$class"/>
				</xsl:attribute>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="$name"/>
					<xsl:with-param name="name" select="$name"/>
					<xsl:with-param name="value" select="../../../Values/RowValue[Row/rowID = $rowID]/formattedValue"/>
					<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
					<xsl:with-param name="class">
						<xsl:value-of select="$class"/>
						<xsl:text> text-align-right</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="maxlength" select="'255'"/>
					<xsl:with-param name="aria-describedby">
						<xsl:value-of select="$rowID" />
						<xsl:value-of select="'_error'" />
					</xsl:with-param>
					<xsl:with-param name="aria-required"><xsl:if test="required = 'true'">true</xsl:if></xsl:with-param>
				</xsl:call-template>
				
				<xsl:apply-templates select="../../../../ValidationErrors/validationError[fieldName = $name]">
					<xsl:with-param name="id"><xsl:value-of select="$rowID" />_error</xsl:with-param>
				</xsl:apply-templates>
				
			</td>
			
			<td class="padding bigpaddingright text-align-right" data-title="{$i18n.Column.Sum}" data-sum-summariseto="true">
				
				<span><xsl:value-of select="sum"/></span>
					
			</td>
			
			<xsl:if test="../../../PriceTableQuery/showComments = 'true'">
			
				<xsl:variable name="commentName">
					<xsl:value-of select="$name"/>
					<xsl:text>_comment</xsl:text>
				</xsl:variable>
				
				<td data-title="{$i18n.Column.Comment}">
				
					<xsl:variable name="class">
				
						<xsl:if test="../../../../ValidationErrors/validationError[fieldName = $commentName]">
							<xsl:text> invalid input-error</xsl:text>
						</xsl:if>
						
					</xsl:variable>
				
					<xsl:attribute name="class">
						<xsl:text>paddingright</xsl:text>
						<xsl:value-of select="$class"/>
					</xsl:attribute>
					
					<span></span>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="$commentName"/>
						<xsl:with-param name="name" select="$commentName"/>
						<xsl:with-param name="value" select="../../../Values/RowValue[Row/rowID = $rowID]/comment"/>
						<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
						<xsl:with-param name="class">
							<xsl:value-of select="$class"/>
						</xsl:with-param>
						<xsl:with-param name="maxlength" select="'255'"/>
						<xsl:with-param name="aria-describedby">
							<xsl:value-of select="$rowID" />
							<xsl:value-of select="'_commenterror'" />
						</xsl:with-param>
					</xsl:call-template>
					
					<xsl:apply-templates select="../../../../ValidationErrors/validationError[fieldName = $commentName]">
						<xsl:with-param name="id"><xsl:value-of select="$rowID" />_commenterror</xsl:with-param>
					</xsl:apply-templates>
					
				</td>
				
			</xsl:if>
			
		</tr>
	
	</xsl:template>
	
	<xsl:template match="Row" mode="tbody-show">
		
		<xsl:variable name="rowID" select="rowID"/>

		<xsl:variable name="value" select="../../../Values/RowValue[Row/rowID = $rowID]/formattedValue"/>

		<xsl:if test="(../../../PriceTableQuery/hideEmptyRows = 'true' and $value != 0) or ../../../PriceTableQuery/hideEmptyRows = 'false'">
			<tr>
			
				<td class="paddingright">
					<strong>
						<xsl:value-of select="label"/>
					</strong>
					<span class="marginleft">
						<xsl:value-of select="formattedPrice"/>
						<xsl:text>&#160;</xsl:text>
						<xsl:text>kr</xsl:text>
					</span>
				</td>
				
				<td class="padding bigpaddingright text-align-right" data-title="{$i18n.Column.Count}" data-sum-summariseto="true">
					
					
					<xsl:choose>
						<xsl:when test="$value">
							<xsl:value-of select="$value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>-</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</td>
				
				<td class="padding bigpaddingright text-align-right" data-title="{$i18n.Column.Sum}" data-sum-summariseto="true">
					<xsl:variable name="value" select="sum"/>
					
					<xsl:choose>
						<xsl:when test="$value">
							<xsl:value-of select="$value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>0</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</td>
				
				<td/>
				
				<xsl:if test="../../../PriceTableQuery/showComments = 'true'">
				
					<td class="padding" data-title="{$i18n.Column.Comment}">
						<xsl:variable name="value" select="../../../Values/RowValue[Row/rowID = $rowID]/comment"/>
						
						<xsl:choose>
							<xsl:when test="$value">
								<xsl:value-of select="$value"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>-</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</td>
					
				</xsl:if>
				
			</tr>
		</xsl:if>

	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!" title="{$i18n.RequiredField}">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
		</i>
		
	</xsl:template>	
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.TooLongFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part2"/>
				<xsl:value-of select="maxLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part3"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey]">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
			
				<xsl:value-of select="messageKey"/>
				
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'AmountMayNotBeNegative']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.ValidationError.AmountMayNotBeNegative"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.InvalidFormat"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
		
	<xsl:template match="validationError">
		
		<xsl:param name="id" />
		
		<i data-icon-after="!" title="{$i18n.UnknownValidationError}">
			<xsl:if test="$id"><xsl:attribute name="id"><xsl:value-of select="$id" /></xsl:attribute></xsl:if>
		</i>
		
	</xsl:template>
	
</xsl:stylesheet>