package com.nordicpeak.flowengine.queries.pricetablequery;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.IntegerBasedCRUD;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableQuery;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableRow;

public class PriceTableRowCRUD extends IntegerBasedCRUD<PriceTableRow, PriceTableQueryProviderModule> {
	
	
	protected AnnotatedDAOWrapper<PriceTableRow, Integer> tableRowDAO;
	
	public PriceTableRowCRUD(AnnotatedDAOWrapper<PriceTableRow, Integer> tableRowDAO, BeanRequestPopulator<PriceTableRow> populator, String typeElementName, String typeLogName, String listMethodAlias, PriceTableQueryProviderModule callback) {
		
		super(tableRowDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.tableRowDAO = tableRowDAO;
	}
	
	@Override
	protected ForegroundModuleResponse beanAdded(PriceTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);
		
		callback.redirectToQueryConfig(bean.getQuery(), req, res);
		
		return null;
	}
	
	@Override
	protected ForegroundModuleResponse beanUpdated(PriceTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);
		
		callback.redirectToQueryConfig(bean.getQuery(), req, res);
		
		return null;
	}
	
	@Override
	protected ForegroundModuleResponse beanDeleted(PriceTableRow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);
		
		callback.redirectToQueryConfig(bean.getQuery(), req, res);
		
		return null;
	}
	
	@Override
	public PriceTableRow getBean(Integer beanID) throws SQLException, AccessDeniedException {
		
		HighLevelQuery<PriceTableRow> query = new HighLevelQuery<>(PriceTableRow.QUERY_RELATION);
		
		query.addParameter(tableRowDAO.getParameterFactory().getParameter(beanID));
		
		PriceTableRow tableRow = tableRowDAO.getAnnotatedDAO().get(query);
		
		if (tableRow != null) {
			
			tableRow.getQuery().init(callback.getFlowAdminModule().getQueryDescriptor(tableRow.getQuery().getQueryID()), null);
		}
		
		return tableRow;
	}
	
	@Override
	protected PriceTableRow populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		PriceTableRow tableRow = super.populateFromAddRequest(req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<>();
		
		tableRow.setQuery((PriceTableQuery) req.getAttribute("PriceTableQuery"));
		tableRow.setSortIndex(this.getCurrentMaxSortIndex(tableRow.getQuery()) + 1);
		
		validatePopulation(tableRow, req, user, uriParser, validationErrors);
		
		if (!CollectionUtils.isEmpty(validationErrors)) {
			throw new ValidationException(validationErrors);
		}
		
		return tableRow;
	}
	
	@Override
	protected void validateUpdatePopulation(PriceTableRow tableRow, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {
		
		super.validateUpdatePopulation(tableRow, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<>();
		
		validatePopulation(tableRow, req, user, uriParser, validationErrors);
		
		if (!CollectionUtils.isEmpty(validationErrors)) {
			throw new ValidationException(validationErrors);
		}
	}
	
	protected void validatePopulation(PriceTableRow tableRow, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws ValidationException, SQLException, Exception {
		
		ValidationUtils.validateBigDecimalSize("price", tableRow.getPrice(), 12, 2, validationErrors);
		
		if (tableRow.getPrice().compareTo(BigDecimal.ZERO) == -1) {
			
			validationErrors.add(new ValidationError("PriceMayNotBeNegative"));
		}
	}
	
	@Override
	protected void checkUpdateAccess(PriceTableRow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}
	
	@Override
	protected void checkDeleteAccess(PriceTableRow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}
	
	private int getCurrentMaxSortIndex(PriceTableQuery fixedTableQuery) throws SQLException {
		
		ObjectQuery<Integer> query = new ObjectQuery<>(tableRowDAO.getAnnotatedDAO().getDataSource(), "SELECT MAX(sortIndex) FROM " + tableRowDAO.getAnnotatedDAO().getTableName() + " WHERE queryID = ?", IntegerPopulator.getPopulator());
		
		query.setInt(1, fixedTableQuery.getQueryID());
		
		Integer sortIndex = query.executeQuery();
		
		if (sortIndex == null) {
			
			return 0;
		}
		
		return sortIndex;
	}
	
}
