$(function() {
	
	setQueryRequiredFunctions["PriceTableQueryInstance"] = makePriceTableQueryRequired;
	setQueryValidationErrorFunctions["PriceTableQueryInstance"] = setPriceTableTextFieldValidationError;
});

function initPriceTableQuery(queryID) {
	
	var query = $("#query_" + queryID);
	var table = query.find("table.pricetable");
	
	initPriceQueryValidationErrors(query);
	
	var inputs = table.find("input");
	
	inputs.on("change", function () {
		
		updatePriceTableSummary(query);
		
		if (query.hasClass("enableAjaxPosting")) {
			
			var parameters = [];
			
			inputs.each(function(){
				var input = $(this);
				parameters.push({name: input.attr("name"), value: input.val()});
			});
			
			parameters.push({name: "queryID", value: queryID});
			
			runQueryEvaluators(queryID, parameters);
		}
	});
}

function updatePriceTableSummary(query){
	
	var table = query.find("table");
	var parameters = [];
	
	table.children("tbody").children("tr").each(function(index, row) {
		var row = $(row);
		var input = row.children("td:eq(1)").find("input");
		var id = input.attr("id");
		
		parameters.push({name: "row", value: id});
		parameters.push({name: id + "-price", value: row.data("price")});
		parameters.push({name: id + "-amount", value: input.val()});
	});
	
	$.ajax({
		type: "POST",
		cache: false,
		url: query.data("summary-url"),
		data: parameters,
		dataType: "json",
		contentType: "application/x-www-form-urlencoded;charset=UTF-8",
		
	}).done(function(result) {
	
		table.children("tfoot").children("tr").children("td[data-sum-summarise='true']").find("span").text(result.sum);
		table.children("tfoot").children("tr").children("td[data-count-summarise='true']").find("span").text(result.count);
		
		table.children("tbody").children("tr").each(function(index, row) {
			var row = $(row);
			var input = row.children("td[data-sum-summariseto='true']").find("span").text(result.rows[index]);
		});
		
	}).fail(function(jqXHR) {
		
		table.children("tbody").children("tr").children("td[data-sum-summariseto='true']").find("span").text("?");
		table.children("tfoot").children("tr").children("td[data-sum-summarise='true']").find("span").text("?");
		table.children("tfoot").children("tr").children("td[data-count-summarise='true']").find("span").text("?");
	});
}


function initPriceQueryValidationErrors($query) {
	
	$query.find(".input-error input").tooltip({
		position : {
			my : "right top-38",
			at : "right+3 top",
			collision : "none"
		},
		track : false,
		content : function() {
			return $(this).next().prop("title") + "<span class=\"marker\"></span>";
		},
		show : {
			effect : 'none'
		},
		hide : {
			effect : 'none'
		}
	}).off("mouseover mouseout");
	
} 

function makePriceTableQueryRequired(queryID) {
	
	var query = $("#query_" + queryID); 
	
	query.find(".heading-wrapper h2").addClass("required");
	query.find("table").addClass("required");
}

function setPriceTableTextFieldValidationError(queryID, validationErrors) {
	
	var $query = $("#query_" + queryID);
	
	$.each(validationErrors, function(i, error) {
		
		var $input = $query.find("input[name='" + error.field + "']");
		
		$input.parent().addClass("invalid input-error");
		
		var message;
		
		if(error.displayName) {
		
			message = error.displayName;
		
		} else if(error.messageKey) {
		
			message = error.messageKey;
	
		} else {
			
			message = "Unkown validation error";
			
		}
		
		$input.parent().append($("<i data-icon-after='!'' title='" + message + "' />"));
		
	});
	
	initPriceQueryValidationErrors($query);
}
