package com.nordicpeak.flowengine.queries.priorityquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.populators.AlternativesPopulator;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class PriorityQueryCRUD extends BaseQueryCRUD<PriorityQuery, PriorityQueryProviderModule> {

	protected AnnotatedDAOWrapper<PriorityQuery, Integer> queryDAO;

	protected static AlternativesPopulator<PriorityAlternative> ALTERNATIVES_POPLATOR = new PriorityAlternativesPopulator(PriorityAlternative.class);

	public PriorityQueryCRUD(AnnotatedDAOWrapper<PriorityQuery, Integer> queryDAO, BeanRequestPopulator<PriorityQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, PriorityQueryProviderModule callback) {

		super(PriorityQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.queryDAO = queryDAO;
	}

	@Override
	protected PriorityQuery populateFromUpdateRequest(PriorityQuery oldQuery, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		PriorityQuery query = super.populateFromUpdateRequest(oldQuery, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		List<PriorityAlternative> alternatives = ALTERNATIVES_POPLATOR.populate(oldQuery.getAlternatives(), req, validationErrors);

		if (CollectionUtils.isEmpty(alternatives) || alternatives.size() < 2) {

			validationErrors.add(new ValidationError("TooFewAlternatives"));

		} else {

			validateMinAndMax(query.getMinChoices(), query.getMaxChoices(), alternatives, validationErrors);
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		query.setAlternatives(alternatives);

		return query;
	}

	public static void validateMinAndMax(Integer minChoices, Integer maxChoices, List<PriorityAlternative> alternatives, List<ValidationError> validationErrors) {

		if (minChoices != null) {

			if (minChoices < 1) {
				validationErrors.add(new ValidationError("MinChoicesTooSmall"));
			}

			if (minChoices > alternatives.size()) {
				validationErrors.add(new ValidationError("MinChoicesTooBig"));
			}

			if (maxChoices != null && (minChoices > maxChoices || maxChoices < minChoices)) {
				validationErrors.add(new ValidationError("MinChoicesBiggerThanMaxChoices"));
			}
		}

		if (maxChoices != null) {

			if (maxChoices > alternatives.size()) {
				validationErrors.add(new ValidationError("MaxChoicesTooBig"));
			}
		}
	}

	@Override
	protected List<Field> getBeanRelations() {

		return Arrays.asList(PriorityQuery.ALTERNATIVES_RELATION);
	}

}
