package com.nordicpeak.flowengine.queries.priorityquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.checkboxquery.validationerrors.TooFewAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.checkboxquery.validationerrors.TooManyAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class PriorityQueryProviderModule extends BaseQueryProviderModule<PriorityQueryInstance> implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	private String countText = "Count";

	@XSLVariable(prefix = "java.")
	private String alternativesText = "Alternative";

	private AnnotatedDAO<PriorityQuery> queryDAO;
	private AnnotatedDAO<PriorityQueryInstance> queryInstanceDAO;

	private PriorityQueryCRUD queryCRUD;

	private QueryParameterFactory<PriorityQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<PriorityQueryInstance, Integer> queryInstanceIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PriorityQueryProviderModule.class.getName(), new XMLDBScriptProvider(PriorityQueryProviderModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(PriorityQuery.class);
		queryInstanceDAO = daoFactory.getDAO(PriorityQueryInstance.class);

		queryCRUD = new PriorityQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<PriorityQuery>(PriorityQuery.class), PriorityQuery.class.getSimpleName(), "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PriorityQuery query = new PriorityQuery();

		query.setQueryID(descriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PriorityQuery query = new PriorityQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		List<Integer> oldAlternativeIDs = FixedAlternativeQueryUtils.getAlternativeIDs(query);

		FixedAlternativeQueryUtils.clearAlternativeIDs(query.getAlternatives());

		queryDAO.add(query, transactionHandler, null);

		query.setAlternativeConversionMap(FixedAlternativeQueryUtils.getAlternativeConversionMap(query.getAlternatives(), oldAlternativeIDs));

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		PriorityQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PriorityQuery query = getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		PriorityQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PriorityQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance.copyQueryValues();
		}

		return queryInstance;
	}

	public PriorityQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<PriorityQuery> query = new HighLevelQuery<PriorityQuery>(PriorityQuery.ALTERNATIVES_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	public PriorityQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PriorityQuery> query = new HighLevelQuery<PriorityQuery>(PriorityQuery.ALTERNATIVES_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private PriorityQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PriorityQueryInstance> query = new HighLevelQuery<PriorityQueryInstance>(PriorityQueryInstance.ALTERNATIVES_RELATION, PriorityQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(PriorityQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		HighLevelQuery<PriorityQueryInstance> query = new HighLevelQuery<PriorityQueryInstance>(PriorityQueryInstance.ALTERNATIVES_RELATION);

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			queryInstanceDAO.add(queryInstance, transactionHandler, query);

		} else {

			queryInstanceDAO.update(queryInstance, transactionHandler, query);
		}
	}

	@Override
	public void populate(PriorityQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		List<PriorityAlternative> availableAlternatives = queryInstance.getQuery().getAlternatives();

		if (CollectionUtils.isEmpty(availableAlternatives)) {

			//If the parent query doesn't have any alternatives then there is no population to do
			queryInstance.reset(attributeHandler);
			return;
		}

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		int submittedAlternativesCount = 0;
		List<PriorityAlternative> selectedAlternatives = new ArrayList<PriorityAlternative>();

		for (int i = 0; i < Math.min(availableAlternatives.size(), queryInstance.getQuery().getMaxChoices()); i++) {

			String alternativeName = "q" + queryInstance.getQuery().getQueryID() + "_alternative_" + i;

			Integer alternativeID = NumberUtils.toInt(req.getParameter(alternativeName));

			if (alternativeID != null) {

				for (PriorityAlternative alternative : availableAlternatives) {

					if (alternative.getAlternativeID().equals(alternativeID)) {

						submittedAlternativesCount++;

						if (!alternative.isAllowMultiple() && selectedAlternatives.contains(alternative)) {
							validationErrors.add(new ValidationError("CanOnlyChooseAlternativeOnce", "", alternativeName));
							break;
						}

						selectedAlternatives.add(alternative);
						break;
					}
				}

			} else {
				
				String nextAlternativeName = "q" + queryInstance.getQuery().getQueryID() + "_alternative_" + (i + 1);
				
				if (NumberUtils.toInt(req.getParameter(nextAlternativeName)) != null) {
					
					validationErrors.add(new ValidationError("q" + queryInstance.getQuery().getQueryID() + "_alternative_" + i, ValidationErrorType.RequiredField));
				}
			}
		}

		//If partial population is allowed, skip validation
		if (allowPartialPopulation && validationErrors.isEmpty()) {

			queryInstance.setAlternatives(selectedAlternatives);
			queryInstance.getQueryInstanceDescriptor().setPopulated(!selectedAlternatives.isEmpty());
			queryInstance.setAttributes(attributeHandler);
			return;
		}

		Integer minChoices = getMinChoices(queryInstance, attributeHandler);
		
		//Check if this query is required or if the user has selected any alternatives anyway
		if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED || selectedAlternatives.size() > 0) {

			if (minChoices != null && submittedAlternativesCount < minChoices) {

				validationErrors.add(new TooFewAlternativesSelectedValidationError(submittedAlternativesCount, minChoices));

			} else if (queryInstance.getQuery().getMaxChoices() != null && submittedAlternativesCount > queryInstance.getQuery().getMaxChoices()) {

				validationErrors.add(new TooManyAlternativesSelectedValidationError(submittedAlternativesCount, queryInstance.getQuery().getMaxChoices()));

			} else if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && selectedAlternatives.size() == 0) {

				validationErrors.add(new ValidationError("RequiredQuery"));
			}
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		queryInstance.setAlternatives(selectedAlternatives);
		queryInstance.getQueryInstanceDescriptor().setPopulated(!selectedAlternatives.isEmpty());
		queryInstance.setAttributes(attributeHandler);
	}
	
	protected Integer getMinChoices(PriorityQueryInstance queryInstance, AttributeHandler attributeHandler){
		
		return queryInstance.getQuery().getMinChoices();
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PriorityQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PriorityQueryInstance queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PriorityQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		if (query.getAlternatives() != null) {

			for (PriorityAlternative alternative : query.getAlternatives()) {

				alternative.setAlternativeID(null);
			}
		}

		queryDAO.add(query, transactionHandler, new RelationQuery(PriorityQuery.ALTERNATIVES_RELATION));
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, PriorityQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<PriorityQueryInstance> getQueryInstanceClass() {

		return PriorityQueryInstance.class;
	}

}
