<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/js/confirmpost.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/flowengine.tablesorter.js
		/js/flowengine.tablefilter.js
		/js/UserGroupList.js
		/js/queueadmin.js?v=1
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/UserGroupList.css?v=1
		/css/flowengine.css
		/uitheme/jquery-ui-1.8.7.custom.css
	</xsl:variable>

	<xsl:template match="Document">
	
		<xsl:choose>
			<xsl:when test="QueueUserAdminExtension">
			
				<xsl:apply-templates select="QueueUserAdminExtension" />
			
			</xsl:when>			
			<xsl:otherwise>
						
				<div id="QueueAdminModule" class="contentitem errands-wrapper">
						
					<xsl:apply-templates select="ListQueues" />
					<xsl:apply-templates select="AddQueue" />
					<xsl:apply-templates select="UpdateQueue" />
						
				</div>
							
			</xsl:otherwise>			
		</xsl:choose>	
					
	</xsl:template>
	
	<xsl:template match="ListQueues">
		
		<div class="floatright marginbottom">
			<label for="name" class="bigmarginleft marginright margintop"><xsl:value-of select="$i18n.Filter" />:</label>
			<input type="text" size="20" class="filter-input" data-tableid="queues" />
		</div>
		
		<h1>
			<xsl:value-of select="/Document/module/name" />
		</h1>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<table id="queues" class="oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Queue.Name" /></span>
					</th>
					<xsl:if test="../EnableProfileSupport = 'true'">
						<th>
							<span><xsl:value-of select="$i18n.Queue.Profile" /></span>
						</th>
					</xsl:if>
					<th>
						<span><xsl:value-of select="$i18n.Queue.Enabled" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Queue.ActiveSlots" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Queue.ArchivedSlots" /></span>
					</th>
					<th class="no-sort"></th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Queues/Queue">
						<xsl:apply-templates select="Queues/Queue" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoQueues" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<article class="buttons floatright">

			<a class="btn btn-green btn-right" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/add">
				<xsl:value-of select="$i18n.AddQueue" />
			</a>
	
		</article>
		
		<!-- Seems unused? -->
		<div style="display: none;">
			<div id="profiles">
				<xsl:for-each select="Profiles/Profile">
					<div data-profileid="{profileID}">
						<xsl:value-of select="name"/>
					</div>
				</xsl:for-each>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="Queue" mode="list">
	
		<tr>
			<td data-title="{$i18n.Queue.Name}"><xsl:value-of select="name" /></td>
			<xsl:if test="../../../EnableProfileSupport = 'true'">
				<xsl:variable name="profileID" select="profileID" />
				<td data-title="{$i18n.Queue.Profile}"><xsl:value-of select="../../Profiles/Profile[profileID=$profileID]/name" /></td>
			</xsl:if>
			<td data-title="{$i18n.Queue.Enabled}">
				<xsl:choose>
					<xsl:when test="enabled = 'true'">
						<xsl:value-of select="$i18n.Yes" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No" />
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td data-title="{$i18n.Queue.ActiveSlots}"><xsl:value-of select="activeSlots" /></td>
			<td data-title="{$i18n.Queue.ArchivedSlots}"><xsl:value-of select="archivedSlots" /></td>
			<td class="link">
				<a class="btn btn-green vertical-align-middle" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/update/{queueID}"><xsl:value-of select="$i18n.Update" /></a>
				<a class="btn btn-red vertical-align-middle marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/delete/{queueID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.DeleteConfirm}: {name}"><xsl:value-of select="$i18n.Delete" /></a>
				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="AddQueue">
	
		<h1>
			<xsl:value-of select="$i18n.AddQueue" />
		</h1>
		
		<form method="POST" action="{/Document/requestinfo/uri}" name="queueForm">
			
			<xsl:call-template name="editQueue" />
		
			<article class="buttons floatright">
				<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline" />
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
			</article>
		
		</form>
		
	</xsl:template>
	
	<xsl:template match="UpdateQueue">
		
		<h1>
			<xsl:value-of select="$i18n.UpdateQueue" />
		</h1>
		
		<form method="POST" action="{/Document/requestinfo/uri}" name="queueForm">
			
			<xsl:call-template name="editQueue"/>
		
			<article class="buttons floatright">
				<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline" />
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
			</article>
		
		</form>
		
	</xsl:template>
	
	<xsl:template name="editQueue">
		
		<article class="clearfix">
		
			<xsl:if test="validationException/validationError">
				<div class="bigmarginbottom">
					<xsl:apply-templates select="validationException/validationError"/>
				</div>
			</xsl:if>
			
			<div class="floatleft full bigmarginbottom">
				<label for="name" class="required full nomargin">
					<xsl:value-of select="$i18n.Queue.Name" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'name'" />
					<xsl:with-param name="id" select="'name'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="value" select="Queue/name" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="price" class="required full nomargin">
					<xsl:value-of select="$i18n.Queue.Price" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'price'" />
					<xsl:with-param name="id" select="'price'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="value" select="Queue/price" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="description" class="required full nomargin">
					<xsl:value-of select="$i18n.Queue.Description" />
				</label>
				
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="name" select="'description'" />
					<xsl:with-param name="id" select="'description'" />
					<xsl:with-param name="rows" select="4" />
					<xsl:with-param name="value" select="Queue/description" />
				</xsl:call-template>
			</div>

			<xsl:if test="../EnableProfileSupport = 'true'">
				<div class="floatleft full bigmarginbottom">
					<label for="endpointID" class="required full nomargin">
						<xsl:value-of select="$i18n.Queue.ProfileTitle" />
					</label>

					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'profileID'" />
						<xsl:with-param name="name" select="'profileID'" />
						<xsl:with-param name="title" select="$i18n.Queue.ProfileTitle"/>
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'profileID'" />
						<xsl:with-param name="element" select="Profiles/Profile" />
			 			<xsl:with-param name="class" select="'marginright'" />
			 			<xsl:with-param name="addEmptyOption" select="$i18n.ProfileEmptyOption" />
			 			<xsl:with-param name="selectedValue" select="Queue/profileID" />
					</xsl:call-template>
				</div>
			</xsl:if>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'enabled'" />
					<xsl:with-param name="name" select="'enabled'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="enabled">
					<xsl:value-of select="$i18n.Queue.Enabled" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showQueuePosition'" />
					<xsl:with-param name="name" select="'showQueuePosition'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="showQueuePosition">
					<xsl:value-of select="$i18n.Queue.ShowQueuePosition" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'leavingRequiresManagerConfirmation'" />
					<xsl:with-param name="name" select="'leavingRequiresManagerConfirmation'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="leavingRequiresManagerConfirmation">
					<xsl:value-of select="$i18n.Queue.LeavingRequiresManagerConfirmation" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hideAddedDateFromUser'" />
					<xsl:with-param name="name" select="'hideAddedDateFromUser'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="hideAddedDateFromUser">
					<xsl:value-of select="$i18n.Queue.HideAddedDateFromUser" />
				</label>
			</div>
						
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowDuplicateQueueSlots'" />
					<xsl:with-param name="name" select="'allowDuplicateQueueSlots'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="allowDuplicateQueueSlots">
					<xsl:value-of select="$i18n.Queue.AllowDuplicateQueueSlots" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'useWaitingPeriod'" />
					<xsl:with-param name="name" select="'useWaitingPeriod'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Queue" />
				</xsl:call-template>
				
				<label class="nomargin" for="useWaitingPeriod">
					<xsl:value-of select="$i18n.Queue.WaitingPeriodActive" />
				</label>
			</div>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			
			
			<h2><xsl:value-of select="$i18n.Queue.Manager.Users"/></h2>
			
			<xsl:call-template name="UserList">
				<xsl:with-param name="connectorURL">
					<xsl:value-of select="/Document/requestinfo/currentURI"/>
					<xsl:text>/</xsl:text>
					<xsl:value-of select="/Document/module/alias"/>
					<xsl:text>/users</xsl:text>
				</xsl:with-param>
				<xsl:with-param name="name" select="'user'"/>
				<xsl:with-param name="users" select="users" />
				<xsl:with-param name="document" select="/Document" />
				<xsl:with-param name="showUsername" select="true()" />
			</xsl:call-template>
			
			<br/>
			
			<h2><xsl:value-of select="$i18n.Queue.Manager.Groups"/></h2>
			
			<xsl:call-template name="GroupList">
				<xsl:with-param name="connectorURL">
					<xsl:value-of select="/Document/requestinfo/currentURI"/>
					<xsl:text>/</xsl:text>
					<xsl:value-of select="/Document/module/alias"/>
					<xsl:text>/groups</xsl:text>
				</xsl:with-param>
				<xsl:with-param name="name" select="'group'"/>
				<xsl:with-param name="groups" select="groups" />
				<xsl:with-param name="document" select="/Document" />
			</xsl:call-template>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<xsl:if test="ViewFragment">
			
				<div id="paymentsettings">
			
					<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>
				
					<div class="floatleft full bigmarginbottom">
						<label for="price" class="required full nomargin">
							<xsl:value-of select="$i18n.Queue.eventPaymentInvoiceMessage" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'eventPaymentInvoiceMessage'" />
							<xsl:with-param name="id" select="'eventPaymentInvoiceMessage'" />
							<xsl:with-param name="size" select="40" />
							<xsl:with-param name="value">
								<xsl:choose>
									<xsl:when test="Queue/eventPaymentInvoiceMessage">
										<xsl:value-of select="Queue/eventPaymentInvoiceMessage"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="DefaultEventPaymentInvoiceMessage"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:with-param>
						</xsl:call-template>
					</div>
				
				</div>
				
				<script type="text/javascript">
					$(function() {
					
						var paymentSettings = $("#paymentsettings");
						var priceField = $("#price");
						
						var showHidePaymentSettings = function() {
							
							var price = priceField.val();
							var emptyPrice = price == "" || price == 0;
							
							paymentSettings.toggle(!emptyPrice);
						};
						
						showHidePaymentSettings();
						
						priceField.on("change", function(){
							showHidePaymentSettings();
						});
					});
				</script>
			
			</xsl:if>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.UserNotifications"/></h2>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendEnteredUserSMS'" />
						<xsl:with-param name="id" select="'sendEnteredUserSMS'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendEnteredUserSMS">
						<xsl:value-of select="$i18n.Queue.SendEnteredUserSMS" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExternalMessageReceivedUserSMS'" />
						<xsl:with-param name="id" select="'sendExternalMessageReceivedUserSMS'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExternalMessageReceivedUserSMS">
						<xsl:value-of select="$i18n.Queue.SendExternalMessageReceivedUserSMS" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitRequestUserSMS'" />
						<xsl:with-param name="id" select="'sendExitRequestUserSMS'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitRequestUserSMS">
						<xsl:value-of select="$i18n.Queue.SendExitRequestUserSMS" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitUserSMS'" />
						<xsl:with-param name="id" select="'sendExitUserSMS'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitUserSMS">
						<xsl:value-of select="$i18n.Queue.SendExitUserSMS" />
					</label>				
				</div>
			</div>			
		
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendEnteredUserEmail'" />
						<xsl:with-param name="id" select="'sendEnteredUserEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendEnteredUserEmail">
						<xsl:value-of select="$i18n.Queue.SendEnteredUserEmail" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExternalMessageReceivedUserEmail'" />
						<xsl:with-param name="id" select="'sendExternalMessageReceivedUserEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExternalMessageReceivedUserEmail">
						<xsl:value-of select="$i18n.Queue.SendExternalMessageReceivedUserEmail" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitRequestUserEmail'" />
						<xsl:with-param name="id" select="'sendExitRequestUserEmail'" />
						<xsl:with-param name="element" select="Queue" />
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitRequestUserEmail">
						<xsl:value-of select="$i18n.Queue.SendExitRequestUserEmail" />
					</label>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendPaymentCompleteUserEmail'" />
						<xsl:with-param name="id" select="'sendPaymentCompleteUserEmail'" />
						<xsl:with-param name="element" select="Queue" />
					</xsl:call-template>
					
					<label class="nomargin" for="sendPaymentCompleteUserEmail">
						<xsl:value-of select="$i18n.Queue.SendPaymentCompleteUserEmail" />
					</label>
				</div>
			</div>
			
			<xsl:if test="/Document/ShowSenderEmailAddress='true'">
			
				<div class="floatleft full bigmarginbottom">
					<label for="emailSenderName" class="full nomargin">
						<xsl:value-of select="$i18n.Queue.EmailSenderName" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="name" select="'emailSenderName'" />
						<xsl:with-param name="id" select="'emailSenderName'" />
						<xsl:with-param name="size" select="40" />
						<xsl:with-param name="value" select="Queue/emailSenderName" />
					</xsl:call-template>
				</div>
			
				<div class="floatleft full bigmarginbottom">
					<label for="emailSenderAddress" class="full nomargin">
						<xsl:value-of select="$i18n.Queue.EmailSenderAddress" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="name" select="'emailSenderAddress'" />
						<xsl:with-param name="id" select="'emailSenderAddress'" />
						<xsl:with-param name="size" select="40" />
						<xsl:with-param name="value" select="Queue/emailSenderAddress" />
					</xsl:call-template>
				</div>
				
			</xsl:if>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.ManagerNotifications"/></h2>
		
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExternalMessageReceivedManagerEmail'" />
						<xsl:with-param name="id" select="'sendExternalMessageReceivedManagerEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExternalMessageReceivedManagerEmail">
						<xsl:value-of select="$i18n.Queue.SendExternalMessageReceivedManagerEmail" />
					</label>				
				</div>
			</div>	
		
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitRequestManagerEmail'" />
						<xsl:with-param name="id" select="'sendExitRequestManagerEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitRequestManagerEmail">
						<xsl:value-of select="$i18n.Queue.sendExitRequestManagerEmail" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitManagerEmail'" />
						<xsl:with-param name="id" select="'sendExitManagerEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitManagerEmail">
						<xsl:value-of select="$i18n.Queue.sendExitManagerEmail" />
					</label>				
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendPaymentCompleteManagerEmail'" />
						<xsl:with-param name="id" select="'sendPaymentCompleteManagerEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendPaymentCompleteManagerEmail">
						<xsl:value-of select="$i18n.Queue.sendPaymentCompleteManagerEmail" />
					</label>				
				</div>
			</div>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
		
			<h2><xsl:value-of select="$i18n.GlobalNotifications"/></h2>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExternalMessageReceivedGlobalEmail'" />
						<xsl:with-param name="id" select="'sendExternalMessageReceivedGlobalEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExternalMessageReceivedGlobalEmail">
						<xsl:value-of select="$i18n.Queue.SendExternalMessageReceivedGlobalEmail" />
					</label>
				</div>
			</div>	
		
			<div class="floatleft full bigmarginbottom">
			
				<label for="externalMessageReceivedGlobalEmailAddresses" class="floatleft full nomargin">
					<xsl:value-of select="$i18n.Queue.ExternalMessageReceivedGlobalEmailAddresses" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'externalMessageReceivedGlobalEmailAddresses'"/>
						<xsl:with-param name="name" select="'externalMessageReceivedGlobalEmailAddresses'"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="Queue/ExternalMessageReceivedGlobalEmailAddresses/address" />          
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitRequestGlobalEmail'" />
						<xsl:with-param name="id" select="'sendExitRequestGlobalEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitRequestGlobalEmail">
						<xsl:value-of select="$i18n.Queue.SendExitRequestGlobalEmail" />
					</label>
				</div>
			</div>	
		
			<div class="floatleft full bigmarginbottom">
			
				<label for="exitRequestGlobalEmailAddresses" class="floatleft full nomargin">
					<xsl:value-of select="$i18n.Queue.ExitRequestGlobalEmailAddresses" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'exitRequestGlobalEmailAddresses'"/>
						<xsl:with-param name="name" select="'exitRequestGlobalEmailAddresses'"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="Queue/ExitRequestGlobalEmailAddresses/address" />          
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendExitGlobalEmail'" />
						<xsl:with-param name="id" select="'sendExitGlobalEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendExitGlobalEmail">
						<xsl:value-of select="$i18n.Queue.SendExitGlobalEmail" />
					</label>
				</div>
			</div>
		
			<div class="floatleft full bigmarginbottom">
			
				<label for="exitGlobalEmailAddresses" class="floatleft full nomargin">
					<xsl:value-of select="$i18n.Queue.ExitGlobalEmailAddresses" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'exitGlobalEmailAddresses'"/>
						<xsl:with-param name="name" select="'exitGlobalEmailAddresses'"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="Queue/ExitGlobalEmailAddresses/address" />          
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'sendPaymentCompleteGlobalEmail'" />
						<xsl:with-param name="id" select="'sendPaymentCompleteGlobalEmail'" />
						<xsl:with-param name="element" select="Queue" />       
					</xsl:call-template>
					
					<label class="nomargin" for="sendPaymentCompleteGlobalEmail">
						<xsl:value-of select="$i18n.Queue.SendPaymentCompleteGlobalEmail" />
					</label>
				</div>
			</div>	
		
			<div class="floatleft full bigmarginbottom">
			
				<label for="exitGlobalEmailAddresses" class="floatleft full nomargin">
					<xsl:value-of select="$i18n.Queue.PaymentCompleteGlobalEmailAddresses" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'paymentCompleteGlobalEmailAddresses'"/>
						<xsl:with-param name="name" select="'paymentCompleteGlobalEmailAddresses'"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="Queue/PaymentCompleteGlobalEmailAddresses/address" />          
					</xsl:call-template>
				</div>
			</div>
			
			<h2 class="floatleft full bigmargintop"><xsl:value-of select="$i18n.AdvancedSettings" /></h2>

			<div class="floatleft full bigmarginbottom">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'useQueueForInternalUsers'" />
					<xsl:with-param name="id" select="'useQueueForInternalUsers'" />
					<xsl:with-param name="class" select="'bigmarginbottom'" />
					<xsl:with-param name="element" select="Queue" />       
				</xsl:call-template>			
				
				<label class="nomargin" for="useQueueForInternalUsers">
					<xsl:value-of select="$i18n.Queue.UseQueueForInternalUsers" />
				</label>			
	
				<div id="internalGroups">
				
					<div class="marginbottom">
						<xsl:value-of select="$i18n.Queue.InternalGroupDescription"/>
					</div>
				
					<xsl:call-template name="GroupList">
						<xsl:with-param name="connectorURL">
							<xsl:value-of select="/Document/requestinfo/currentURI"/>
							<xsl:text>/</xsl:text>
							<xsl:value-of select="/Document/module/alias"/>
							<xsl:text>/groups</xsl:text>
						</xsl:with-param>
						<xsl:with-param name="name" select="'internalGroup'"/>
						<xsl:with-param name="groups" select="internalGroups" />
					</xsl:call-template>
	
				</div>
				
			</div>
		
		</article>
		
		<script type="text/javascript">
			$(function() {
			
				$('#useQueueForInternalUsers').on("change", function(){
				
					$('#internalGroups').toggle(this.checked);
					
				}).trigger("change");
				
			});
		</script>
		
	</xsl:template>
	
	<xsl:template match="QueueUserAdminExtension">
	
		<xsl:if test="Queues">
		
			<div class="full floatleft bigmargintop">
	
				<h2><xsl:value-of select="$i18n.Queues" /></h2>
				
				<div>
				
					<xsl:for-each select="Queues/Queue">
				
						<div class="floatleft full border marginbottom">
						
							<div class="floatleft">
								<span data-icon-before="L" style="margin-left:2px;"></span>
								<a href="{../../QueueAdminURL}/update/{queueID}" title="{name}"><xsl:value-of select="name" /></a>
							</div>
						
						</div>
					
					</xsl:for-each>
				
				</div>
			</div>
					
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PaymentProviderConfigurationError']">
	
		<p class="error">
			<span data-icon-before="!">
				<xsl:value-of select="$i18n.validationError.PaymentProviderConfigurationError" />
			</span>
		</p>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UpdateFailedQueueNotFound']">
	
		<p class="error">
			<xsl:value-of select="$i18n.validationError.UpdateFailedQueueNotFound" />
		</p>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey='NoProfileChosen']">
	
		<p class="error">
			<xsl:value-of select="$i18n.validationError.NoProfileChosen" />
		</p>
	
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<xsl:if test="fieldName and validationErrorType">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="validationErrorType='RequiredField'">
							<xsl:value-of select="$i18n.validationError.RequiredField" />
						</xsl:when>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.validationError.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooShort'">
							<xsl:value-of select="$i18n.validationError.TooShort" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.validationError.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.Other" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#x20;</xsl:text>
					<xsl:choose>
						<xsl:when test="fieldName = 'name'">
							<xsl:value-of select="$i18n.Queue.Name" />
						</xsl:when>
						<xsl:when test="fieldName = 'description'">
							<xsl:value-of select="$i18n.Queue.Description" />
						</xsl:when>
						<xsl:when test="fieldName = 'enabled'">
							<xsl:value-of select="$i18n.Queue.Enabled" />
						</xsl:when>
						<xsl:when test="fieldName = 'showQueuePosition'">
							<xsl:value-of select="$i18n.Queue.ShowQueuePosition" />
						</xsl:when>
						<xsl:when test="fieldName = 'price'">
							<xsl:value-of select="$i18n.Queue.Price" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailSenderName'">
							<xsl:value-of select="$i18n.Queue.EmailSenderName" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailSenderAddress'">
							<xsl:value-of select="$i18n.Queue.EmailSenderAddress" />
						</xsl:when>
						<xsl:when test="fieldName = 'exitRequestGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Queue.ExitRequestGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'exitGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Queue.ExitGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Queue.ExternalMessageReceivedGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'paymentCompleteGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Queue.PaymentCompleteGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'eventPaymentInvoiceMessage'">
							<xsl:value-of select="$i18n.Queue.eventPaymentInvoiceMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'internalGroup'">
							<xsl:value-of select="$i18n.Queue.InternalGroup"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>!</xsl:text>
				</span>
			</p>
		</xsl:if>
		<xsl:if test="messageKey">
			<p class="error">
				<span data-icon-before="!">
					<xsl:value-of select="$i18n.validationError.unknownMessageKey" />!
				</span>
			</p>
		</xsl:if>
		<xsl:apply-templates select="message" />
		
	</xsl:template>
	
</xsl:stylesheet>