<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:template match="QueueSlotEvent">
		
		<xsl:variable name="odd" select="(position() mod 2) != 0" />
		
		<tr>
			<xsl:if test="$odd">
				<xsl:attribute name="class">odd</xsl:attribute>
			</xsl:if>

			<td data-title="{$i18n.Action}" class="service">
				<xsl:call-template name="getSlotEventTypeText" />
			</td>
			
			<td data-title="{$i18n.Details}">
				<xsl:choose>
					<xsl:when test="details"><xsl:value-of select="details" /></xsl:when>
					<xsl:when test="eventType = 'CUSTOMER_MESSAGE_SENT'"><xsl:value-of select="$i18n.eventCustomerMessageSent"/></xsl:when>
					<xsl:when test="eventType = 'MANAGER_MESSAGE_SENT'"><xsl:value-of select="$i18n.eventManagerMessageSent"/></xsl:when>
					<xsl:otherwise>-</xsl:otherwise>
				</xsl:choose>
			</td>
			
			<td data-title="{$i18n.Date}" class="date"><xsl:value-of select="added" /></td>
			
			<td data-title="{$i18n.Person}" class="person">
				<xsl:call-template name="printUser">
					<xsl:with-param name="user" select="poster" />
				</xsl:call-template>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="getSlotEventTypeText">
		
		<xsl:choose>
			<xsl:when test="eventType = 'SUBMITTED'">
				<xsl:value-of select="$i18n.SubmittedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'PAYED'">
				<xsl:value-of select="$i18n.PayedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'MOVE'">
				<xsl:value-of select="$i18n.MoveEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'CUSTOMER_MESSAGE_SENT'">
				<xsl:value-of select="$i18n.MessageEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'MANAGER_MESSAGE_SENT'">
				<xsl:value-of select="$i18n.MessageEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'ARCHIVED'">
				<xsl:value-of select="$i18n.ArchivedEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'WAITING_PERIOD'">
				<xsl:value-of select="$i18n.WaitingPeriodEvent" />
			</xsl:when>			
			<xsl:when test="eventType = 'ARCHIVE_REQUEST'">
				<xsl:value-of select="$i18n.ArchiveRequestEvent" />
			</xsl:when>
			<xsl:when test="eventType = 'PAYMENT_REQUEST'">
				<xsl:value-of select="$i18n.PaymentRequestEvent" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.OtherEvent" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template name="getExternalMessageTypeText">
	
		<xsl:if test="type != ''">
					
			<strong>
				<xsl:choose>
					<xsl:when test="type = 'ENTRANCE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ENTRANCE"/>
					</xsl:when>
					<xsl:when test="type = 'MANUAL_ENTRANCE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.MANUAL_ENTRANCE"/>
					</xsl:when>
					<xsl:when test="type = 'RESTORE'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.RESTORE"/>
					</xsl:when>
					<xsl:when test="type = 'CHANGED_POSITION'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.CHANGED_POSITION"/>
					</xsl:when>
					<xsl:when test="type = 'PAYMENT_REQUEST'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.PAYMENT_REQUEST"/>
					</xsl:when>
					<xsl:when test="type = 'PAYMENT_REQUEST_REMOVAL'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.PAYMENT_REQUEST_REMOVAL"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_REQUEST'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ARCHIVE_REQUEST"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_BY_USER'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ARCHIVE_BY_USER"/>
					</xsl:when>
					<xsl:when test="type = 'ARCHIVE_BY_MANAGER'">
						<xsl:value-of select="$i18n.ExternalMessage.Type.ARCHIVE_BY_MANAGER"/>
					</xsl:when>
				</xsl:choose>
				
				<xsl:text>:&#160;</xsl:text>
			</strong>
		
			<br/>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template name="printUser">
		<xsl:param name="user" />
		
		<xsl:value-of select="$user/firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="$user/lastname" />
		
	</xsl:template>
	
	<xsl:template name="printQueueSlotUser">
		<xsl:param name="queueSlot" select="." />
		
		<xsl:choose>
			<xsl:when test="$queueSlot/Poster">
				<xsl:value-of select="$queueSlot/Poster/user/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$queueSlot/Poster/user/lastname" />
			</xsl:when>
			<xsl:when test="$queueSlot/ManualContact">
				<xsl:value-of select="$queueSlot/ManualContact/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$queueSlot/ManualContact/lastname" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$queueSlot/PrimaryContact/firstname" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$queueSlot/PrimaryContact/lastname" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
</xsl:stylesheet>