package com.nordicpeak.flowengine.queue;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.sql.rowset.serial.SerialBlob;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadBase.FileSizeLimitExceededException;
import org.apache.commons.fileupload.FileUploadBase.SizeLimitExceededException;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.io.FilenameUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.Attachment;
import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.fileuploadutils.MultipartRequest;
import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.UserMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.sms.SMSSender;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.FileItemAttachment;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.crud.CRUDEventFilter;
import se.unlogic.hierarchy.core.utils.crud.TransactionRequestFilter;
import se.unlogic.hierarchy.core.validationerrors.FileSizeLimitExceededValidationError;
import se.unlogic.hierarchy.core.validationerrors.InvalidFileExtensionValidationError;
import se.unlogic.hierarchy.core.validationerrors.RequestSizeLimitExceededValidationError;
import se.unlogic.hierarchy.core.validationerrors.UnableToParseFileValidationError;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.io.BinarySizeFormater;
import se.unlogic.standardutils.io.BinarySizes;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurityPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.threads.ExeptionHandlingRunnable;
import se.unlogic.standardutils.threads.ThreadUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.SwedishPhoneNumberValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.http.enums.ContentDisposition;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.enums.SenderType;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.interfaces.MessageCRUDCallback;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.queue.beans.ExternalMessage;
import com.nordicpeak.flowengine.queue.beans.ExternalMessageAttachment;
import com.nordicpeak.flowengine.queue.beans.InternalMessage;
import com.nordicpeak.flowengine.queue.beans.InternalMessageAttachment;
import com.nordicpeak.flowengine.queue.beans.ManualContact;
import com.nordicpeak.flowengine.queue.beans.PrimaryContact;
import com.nordicpeak.flowengine.queue.beans.Queue;
import com.nordicpeak.flowengine.queue.beans.QueueSlot;
import com.nordicpeak.flowengine.queue.beans.QueueSlotEvent;
import com.nordicpeak.flowengine.queue.beans.SendingStatus;
import com.nordicpeak.flowengine.queue.cruds.InternalMessageCRUD;
import com.nordicpeak.flowengine.queue.cruds.QueueSlotCRUD;
import com.nordicpeak.flowengine.queue.enums.ExternalMessageType;
import com.nordicpeak.flowengine.queue.enums.QueueSlotEventType;
import com.nordicpeak.flowengine.queue.events.ExternalMessageAddedEvent;
import com.nordicpeak.flowengine.queue.events.QueueSlotArchiveEvent;
import com.nordicpeak.flowengine.queue.events.QueueSlotManuallyAddedEvent;
import com.nordicpeak.flowengine.queue.validationerrors.DontContactRecipientBySMSValidationError;
import com.nordicpeak.flowengine.queue.validationerrors.ErrorSendingToRecipientViaEmailValidationError;
import com.nordicpeak.flowengine.queue.validationerrors.ErrorSendingToRecipientViaSMSValidationError;
import com.nordicpeak.flowengine.queue.validationerrors.InvalidRecipientEmailValidationError;
import com.nordicpeak.flowengine.queue.validationerrors.InvalidRecipientPhoneNumberValidationError;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentResponse;
import com.nordicpeak.personprovider.Person;
import com.nordicpeak.personprovider.PersonProvider;

public class QueueSlotAdminModule extends BaseQueueSlotModule implements CRUDCallback<User>, MessageCRUDCallback, AccessInterface {

	private static final java.lang.reflect.Field[] LIST_QUEUE_SLOT_RELATIONS = { QueueSlot.CONTACT_RELATION, QueueSlot.MANUAL_CONTACT_RELATION, QueueSlot.INTERNAL_MESSAGES_RELATION, QueueSlot.EXTERNAL_MESSAGES_RELATION, QueueSlot.ATTRIBUTES_RELATION };

	protected static final ValidationError NO_RECIPIENT_TYPES_VALIDATION_ERROR = new ValidationError("NoRecipientTypesSelected");
	protected static final ValidationError EMAIL_SENDER_NOT_SET_VALIDATION_ERROR = new ValidationError("EmailSenderNotSet");
	protected static final ValidationError NO_MESSAGE_PATH_VALIDATION_ERROR = new ValidationError("NoMessagePathSelected");

	private static final SwedishSocialSecurityPopulator SWEDISH_SOCIAL_SECURITY_POPULATOR = new SwedishSocialSecurityPopulator(false, true, true, true);

	@XSLVariable(name = "i18n.QueueSlot.Position")
	protected String columnQueueSlotPosition;

	@XSLVariable(name = "i18n.QueueSlot.Added")
	protected String columnQueueSlotAdded;

	@XSLVariable(name = "i18n.QueueSlot.ArchivedDate")
	protected String columnQueueSlotArchivedDate;

	@XSLVariable(name = "i18n.QueueSlot.LatestPayment")
	protected String columnQueueSlotLatestPayment;

	@XSLVariable(name = "i18n.QueueSlot.PaymentRequested")
	protected String columnQueueSlotLatestPaymentRequested;

	@XSLVariable(name = "i18n.QueueSlot.LatestPaymentType")
	protected String columnQueueSlotLatestPaymentType;

	@XSLVariable(name = "i18n.QueueSlot.LatestPaymentType.Direct")
	protected String columnQueueSlotLatestPaymentTypeDirect;

	@XSLVariable(name = "i18n.QueueSlot.Comment")
	protected String columnQueueSlotComment;

	@XSLVariable(name = "i18n.QueueSlot.Digital")
	protected String columnQueueSlotDigital;

	@XSLVariable(name = "i18n.QueueSlot.WaitingPeriod")
	protected String columnQueueSlotWaitingPeriod;

	@XSLVariable(name = "i18n.Yes")
	protected String columnQueueSlotYes;

	@XSLVariable(name = "i18n.No")
	protected String columnQueueSlotNo;

	private static BeanRequestPopulator<ManualContact> MANUAL_CONTACT_POPULATOR = new AnnotatedRequestPopulator<>(ManualContact.class);

	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Admin groups", description = "Groups allowed to administrate global parts of this module such as deleting queue slots")
	protected List<Integer> adminGroupIDs;

	@ModuleSetting(allowsNull = true)
	@UserMultiListSettingDescriptor(name = "Admin users", description = "Users allowed to administrate global parts of this module such as deleting queue slots")
	protected List<Integer> adminUserIDs;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default instance list page size", description = "", formatValidator = PositiveStringIntegerValidator.class, required = true)
	protected Integer defaultPageSize = 15;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max e-mail file size", description = "Maxmium allowed e-mail file size in megabytes", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer maxEmailFileSize = 10;

	protected AnnotatedDAO<Queue> queueDAO;
	protected AnnotatedDAO<ManualContact> manualContactDAO;

	protected AnnotatedDAOWrapper<Queue, Integer> queueDAOWrapper;
	protected AdvancedAnnotatedDAOWrapper<QueueSlot, Integer> queueSlotDAOWrapper;

	protected QueryParameterFactory<QueueSlot, Queue> queueSlotQueueParamFactory;
	protected QueryParameterFactory<QueueSlot, Boolean> queueSlotArchivedParamFactory;

	protected QueueSlotCRUD queueSlotCRUD;

	@InstanceManagerDependency(required = true)
	protected FlowAdminModule flowAdminModule;

	@InstanceManagerDependency(required = true)
	protected FlowInstanceAdminModule flowInstanceAdminModule;

	@InstanceManagerDependency
	protected PersonProvider personProvider;

	@InstanceManagerDependency(required = true)
	private PDFProvider pdfProvider;

	@InstanceManagerDependency(required = true)
	protected QueueAdminModule queueAdminModule;

	@InstanceManagerDependency
	protected SMSSender smsSender;

	protected InternalMessageCRUD internalMessageCRUD;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, QueueAdminModule.class.getName(), new XMLDBScriptProvider(QueueAdminModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		queueDAO = daoFactory.getDAO(Queue.class);
		manualContactDAO = daoFactory.getDAO(ManualContact.class);

		queueSlotQueueParamFactory = queueSlotDAO.getParamFactory("queue", Queue.class);
		queueSlotArchivedParamFactory = queueSlotDAO.getParamFactory("archived", boolean.class);

		queueDAOWrapper = queueDAO.getWrapper(Integer.class);
		queueDAOWrapper.addRelations(Queue.MANAGER_USERS_RELATION, Queue.MANAGER_GROUPS_RELATION, Queue.INTERNAL_GROUPS_RELATION);
		queueDAOWrapper.setUseRelationsOnGet(true);

		queueSlotDAOWrapper = queueSlotDAO.getAdvancedWrapper(Integer.class);
		queueSlotDAOWrapper.getGetQuery().addRelations(QueueSlot.EVENTS_RELATION, QueueSlot.CONTACT_RELATION, QueueSlot.MANUAL_CONTACT_RELATION, QueueSlot.QUEUE_RELATION, Queue.MANAGER_USERS_RELATION, Queue.MANAGER_GROUPS_RELATION, Queue.INTERNAL_GROUPS_RELATION, QueueSlot.INTERNAL_MESSAGES_RELATION, QueueSlot.EXTERNAL_MESSAGES_RELATION, InternalMessage.ATTACHMENTS_RELATION, ExternalMessage.ATTACHMENTS_RELATION, ExternalMessage.QUOTED_MESSAGE_RELATION, QueueSlotEvent.ATTRIBUTES_RELATION, QueueSlot.ATTRIBUTES_RELATION);
		queueSlotDAOWrapper.getAddQuery().addRelations(QueueSlot.EVENTS_RELATION, QueueSlot.CONTACT_RELATION, QueueSlot.MANUAL_CONTACT_RELATION);
		queueSlotDAOWrapper.getUpdateQuery().addRelations(QueueSlot.EVENTS_RELATION, QueueSlot.CONTACT_RELATION, QueueSlot.MANUAL_CONTACT_RELATION);

		internalMessageCRUD = new InternalMessageCRUD(daoFactory.getDAO(InternalMessage.class), daoFactory.getDAO(InternalMessageAttachment.class), this, true);

		queueSlotCRUD = new QueueSlotCRUD(queueSlotDAOWrapper, internalMessageCRUD, externalMessageCRUD, this);
		queueSlotCRUD.addRequestFilter(new TransactionRequestFilter(dataSource));
		queueSlotCRUD.addBeanFilter(new CRUDEventFilter<>(QueueSlot.class, systemInterface.getEventHandler()));
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Document doc = createDocument(req, uriParser, user);

		Element listQueuesElement = XMLUtils.appendNewElement(doc, (Element) doc.getFirstChild(), "ListQueues");

		List<Queue> queues = getQueues(user);

		if (!CollectionUtils.isEmpty(queues)) {
			for (Queue queue : queues) {

				queue.setActiveSlots(QueueUtils.getActiveQueueSlotCount(queue, queueDAO));
				queue.setArchivedSlots(QueueUtils.getArchivedQueueSlotCount(queue, queueDAO));
			}
		}

		XMLUtils.append(doc, listQueuesElement, "Queues", queues);

		return new SimpleForegroundModuleResponse(doc);
	}

	@WebPublic(alias = "list")
	public ForegroundModuleResponse listQueueSlots(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueID;
		Queue queue;

		if (uriParser.size() >= 3 && (queueID = uriParser.getInt(2)) != null && (queue = queueDAOWrapper.get(queueID)) != null) {

			req.setAttribute("queue", queue);

			return queueSlotCRUD.list(req, res, user, uriParser, null);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "add")
	public ForegroundModuleResponse addQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueID;
		Queue queue;

		if (uriParser.size() >= 3 && (queueID = uriParser.getInt(2)) != null && (queue = queueDAOWrapper.get(queueID)) != null) {

			req.setAttribute("queue", queue);

			return queueSlotCRUD.add(req, res, user, uriParser);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "show")
	public ForegroundModuleResponse showQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return queueSlotCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "messages")
	public ForegroundModuleResponse showQueueSlotMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, URINotFoundException {

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2))) {

			redirectToMethod(req, res, "/show/" + uriParser.get(2) + "#messages");

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "update")
	public ForegroundModuleResponse updateQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return queueSlotCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "updatedescription")
	public ForegroundModuleResponse updateQueueSlotDescription(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueSlotID;
		QueueSlot queueSlot;

		if (uriParser.size() >= 3 && (queueSlotID = uriParser.getInt(2)) != null && (queueSlot = queueSlotDAOWrapper.get(queueSlotID)) != null) {

			if (!hasQueueManagerAccess(queueSlot.getQueue(), user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			List<ValidationError> validationErrors = null;

			if (req.getMethod().equalsIgnoreCase("POST")) {

				validationErrors = new ArrayList<>();

				String description = ValidationUtils.validateParameter("description", req, false, 0, 255, validationErrors);

				if (CollectionUtils.isEmpty(validationErrors)) {

					if (StringUtils.isEmpty(description)) {

						queueSlot.getAttributeHandler().removeAttribute(Constants.FLOW_INSTANCE_DESCRIPTION_ATTRIBUTE);

					} else {

						queueSlot.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_DESCRIPTION_ATTRIBUTE, description);
					}

					queueSlot.addEvent(new QueueSlotEvent(TimeUtils.getCurrentTimestamp(), user, QueueSlotEventType.OTHER, getEventDescriptionUpdated(), null));

					HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.EVENTS_RELATION, QueueSlot.ATTRIBUTES_RELATION);
					queueSlotDAO.update(queueSlot, query);

					getEventHandler().sendEvent(QueueSlot.class, new CRUDEvent<Serializable>(CRUDAction.UPDATE, queueSlot), EventTarget.ALL);

					redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());
					return null;
				}
			}

			log.info("User " + user + " requesting update description form for queue slot " + queueSlot);

			Document doc = createDocument(req, uriParser, user);

			Element updateElement = doc.createElement("UpdateQueueSlotDescription");
			doc.getDocumentElement().appendChild(updateElement);

			updateElement.appendChild(queueSlot.toXML(doc));

			if (validationErrors != null) {

				updateElement.appendChild(RequestUtils.getRequestParameters(req, doc));
				XMLUtils.append(doc, updateElement, "ValidationErrors", validationErrors);
			}

			return new SimpleForegroundModuleResponse(doc);
		}

		throw new URINotFoundException(uriParser);
	}

	@RESTMethod(alias = "archive/{queueSlotID}", method = { "get", "post" }, requireLogin = true)
	public ForegroundModuleResponse archiveQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "queueSlotID") Integer queueSlotID) throws Exception {

		QueueSlot queueSlot = queueSlotDAOWrapper.get(queueSlotID);

		if (queueSlot == null) {

			throw new URINotFoundException(uriParser);
		}

		archiveQueueSlot(req, user, queueSlot);

		if (req.getParameter("show") != null) {

			redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());

		} else {

			redirectToMethod(req, res, "/list/" + queueSlot.getQueue().getQueueID());
		}

		return null;
	}

	@RESTMethod(alias = "archive", method = { "post" }, requireLogin = true)
	public ForegroundModuleResponse archiveQueueSlots(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String queueID = req.getParameter("queueID");
		String[] queueSlotIDs;

		if (queueID == null || (queueSlotIDs = req.getParameterValues("queueSlotID")) == null) {

			throw new URINotFoundException(uriParser);
		}

		for (String queueSlotID : queueSlotIDs) {

			QueueSlot queueSlot = queueSlotDAOWrapper.get(NumberUtils.toInt(queueSlotID));

			if (queueSlot != null) {

				archiveQueueSlot(req, user, queueSlot);
			}
		}

		redirectToMethod(req, res, "/list/" + queueID);

		return null;
	}

	private void archiveQueueSlot(HttpServletRequest req, User user, QueueSlot queueSlot) throws SQLException {

		if (queueSlot != null && !queueSlot.isArchived()) {

			String comment = req.getParameter("comment");

			String waitingPeriod = req.getParameter("selectedWaitingPeriod");

			log.info("User " + user + " archiving queue slot " + queueSlot + " with reason: " + StringUtils.toLogFormat(comment, 50));

			addQueueSlotEvent(queueSlot, QueueSlotEventType.ARCHIVED, eventArchiveMessage + (StringUtils.isEmpty(comment) ? "" : eventSeeExternalMessage), user, null, null);

			QueueUtils.getCalculatedQueuePosition(dataSource, queueSlot);

			queueSlot.setArchived(true);

			if (!StringUtils.isEmpty(waitingPeriod)) {
				Date currentDate = DateUtils.getDate(new SimpleDateFormat("yyyy-MM-dd"), waitingPeriod);
				DateUtils.setTimeToMidnight(currentDate);
				queueSlot.setWaitingPeriodTime(new Timestamp(currentDate.getTime()));

			}

			queueSlotDAO.update(queueSlot);

			if (!StringUtils.isEmpty(comment)) {

				String waitingPeriodInformation = "";
				if (!StringUtils.isEmpty(waitingPeriod)) {

					waitingPeriodInformation += System.lineSeparator() + eventWaitingPeriodMessage + waitingPeriod;
				}
				addQueueSlotExternalMessage(queueSlot, comment + waitingPeriodInformation, true, user, ExternalMessageType.ARCHIVE_BY_MANAGER, null);

			} else if (!StringUtils.isEmpty(waitingPeriod)) {
				addQueueSlotExternalMessage(queueSlot, eventWaitingPeriodMessage + waitingPeriod, true, user, ExternalMessageType.ARCHIVE_BY_MANAGER, null);

			}

			getEventHandler().sendEvent(QueueSlot.class, new QueueSlotArchiveEvent(queueSlot, user), EventTarget.ALL);
		}
	}

	@RESTMethod(alias = "unarchive/{queueSlotID}", method = "post", requireLogin = true)
	public ForegroundModuleResponse unarchiveQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "queueSlotID") Integer queueSlotID) throws Exception {

		QueueSlot queueSlot = queueSlotDAOWrapper.get(queueSlotID);

		if (queueSlot == null) {

			throw new URINotFoundException(uriParser);
		}

		Queue queue = queueSlot.getQueue();

		if (queueSlot.isArchived()) {

			if (!queue.isAllowDuplicateQueueSlots() && isActiveMemberOfQueue(queue, queueSlot.getPrimaryContact().getCitizenIdentifier())) {

				return queueSlotCRUD.show(req, res, user, uriParser, Collections.singletonList(new ValidationError("PersonAlreadyInQueue")));

			} else {

				String comment = req.getParameter("comment");

				log.info("User " + user + " restoring queue slot " + queueSlot + " with reason: " + StringUtils.toLogFormat(comment, 50));

				TransactionHandler transactionHandler = queueSlotDAO.createTransaction();

				try {
					queueSlot.setArchived(false);
					queueSlot.setWaitingPeriodTime(null);

					addQueueSlotEvent(queueSlot, QueueSlotEventType.SUBMITTED, eventAdminRestoredMessage + (StringUtils.isEmpty(comment) ? "" : eventSeeExternalMessage), user, null, null);

					ObjectQuery<Integer> newQueueIndexQuery = transactionHandler.getObjectQuery("SELECT MAX(queueIndex) FROM " + queueSlotDAOWrapper.getAnnotatedDAO().getTableName() + " WHERE queueID = ?", IntegerPopulator.getPopulator());
					newQueueIndexQuery.setInt(1, queue.getQueueID());

					queueSlot.setQueueIndex(1 + newQueueIndexQuery.executeQuery());

					resetWaitingPeriods(queueSlot, transactionHandler);

					queueSlotDAO.update(queueSlot, transactionHandler, null);

					transactionHandler.commit();

					if (!StringUtils.isEmpty(comment)) {
						addQueueSlotExternalMessage(queueSlot, comment, true, user, ExternalMessageType.RESTORE, null);
					}

					QueueUtils.getCalculatedQueuePosition(dataSource, queueSlot);

					getEventHandler().sendEvent(QueueSlot.class, new QueueSlotManuallyAddedEvent(queueSlot, user), EventTarget.ALL);

				} finally {
					TransactionHandler.autoClose(transactionHandler);
				}
			}
		}

		if (req.getParameter("show") != null) {

			redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());

		} else {

			redirectToMethod(req, res, "/list/" + queue.getQueueID());
		}

		return null;
	}

	private void resetWaitingPeriods(QueueSlot queueSlot, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.EVENTS_RELATION, QueueSlot.CONTACT_RELATION, QueueSlot.MANUAL_CONTACT_RELATION);
		query.addParameter(queueSlotQueueParamFactory.getParameter(queueSlot.getQueue()));

		List<QueueSlot> list = queueSlotDAO.getAll(query);

		for (QueueSlot slot : list) {
			//reset all waiting periods for this queue and user
			if (!queueSlot.getQueueSlotID().equals(slot.getQueueSlotID()) && queueSlot.getPrimaryContact().getCitizenIdentifier().equals(slot.getPrimaryContact().getCitizenIdentifier())) {
				slot.setQueue(queueSlot.getQueue());
				slot.setWaitingPeriodTime(null);
				queueSlotDAO.update(slot, transactionHandler, null);
			}

		}
	}

	@WebPublic(alias = "delete")
	public ForegroundModuleResponse deleteQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return queueSlotCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "download")
	public ForegroundModuleResponse downloadQueueSlotPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueSlotID;
		QueueSlot queueSlot;

		if (uriParser.size() >= 3 && (queueSlotID = uriParser.getInt(2)) != null && (queueSlot = queueSlotDAOWrapper.get(queueSlotID)) != null) {

			if (!hasQueueManagerAccess(queueSlot.getQueue(), user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			sendFlowInstancePDF(req, res, user, uriParser, queueSlot.getFlowInstanceID());

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "preview")
	public ForegroundModuleResponse showFlowInstancePreview(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueSlotID;
		QueueSlot queueSlot;

		if (uriParser.size() >= 3 && (queueSlotID = uriParser.getInt(2)) != null && (queueSlot = queueSlotDAOWrapper.get(queueSlotID)) != null) {

			if (!hasQueueManagerAccess(queueSlot.getQueue(), user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			res.sendRedirect(req.getContextPath() + flowInstanceAdminModule.getFlowInstancePreviewURL(queueSlot.getFlowInstanceID()));

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "payment")
	public ForegroundModuleResponse addPayment(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueSlotID;
		QueueSlot queueSlot;

		if (uriParser.size() >= 3 && (queueSlotID = uriParser.getInt(2)) != null && (queueSlot = queueSlotDAOWrapper.get(queueSlotID)) != null) {

			if (!hasQueueManagerAccess(queueSlot.getQueue(), user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			List<ValidationError> validationErrors = null;

			if (req.getMethod().equalsIgnoreCase("POST")) {

				validationErrors = new ArrayList<>();

				Date paymentDate = ValidationUtils.validateParameter("paymentDate", req, false, DatePopulator.getPopulator(), validationErrors);
				Integer paymentAmount = ValidationUtils.validateParameter("paymentAmount", req, true, IntegerPopulator.getPopulator(), validationErrors);

				if (paymentDate != null && new Date().before(paymentDate)) {

					validationErrors.add(new ValidationError("PaymentDateInFuture"));
				}

				if (CollectionUtils.isEmpty(validationErrors)) {

					Timestamp timestamp;

					if (paymentDate != null) {

						timestamp = new Timestamp(paymentDate.getTime());

					} else {

						timestamp = TimeUtils.getCurrentTimestamp();
					}

					queueSlot.setPendingPaymentRequestDate(null);
					queueSlot.setLastPaymentDate(timestamp);
					queueSlot.addEvent(new QueueSlotEvent(TimeUtils.getCurrentTimestamp(), user, QueueSlotEventType.PAYED, getEventPaymentManagerMessage() + " " + DateUtils.DATE_FORMATTER.format(timestamp) + " (" + paymentAmount + " SEK)", null));

					HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.EVENTS_RELATION);
					queueSlotDAO.update(queueSlot, query);

					getEventHandler().sendEvent(QueueSlot.class, new CRUDEvent<Serializable>(CRUDAction.UPDATE, queueSlot), EventTarget.ALL);

					if (QueueUtils.isDigitalMember(queueSlot, getUserHandler())) {

						Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());

						log.info("Sending payment received message to queue slot " + queueSlot);

						if (queueSlot.getQueue().isSendPaymentCompleteUserEmail() && contact.isContactByEmail() && !StringUtils.isEmpty(contact.getEmail())) {

							if (!EmailUtils.isValidEmailAddress(contact.getEmail())) {

								validationErrors.add(new InvalidRecipientEmailValidationError(queueSlot, contact.getEmail()));

							} else {

								if (!queueAdminModule.sendContactEmail(queueSlot, contact, queueAdminModule.getPaymentCompleteUserEmailSubject(), queueAdminModule.getPaymentCompleteUserEmailMessage(), null)) {

									validationErrors.add(new ErrorSendingToRecipientViaEmailValidationError(queueSlot, contact.getEmail()));
								}
							}
						}

						if (queueSlot.getQueue().isSendPaymentCompleteUserSMS() && contact.isContactBySMS() && !StringUtils.isEmpty(contact.getMobilePhone())) {

							if (!queueAdminModule.sendContactSMS(queueSlot, contact, queueAdminModule.getPaymentCompleteUserSMS())) {

								validationErrors.add(new ErrorSendingToRecipientViaSMSValidationError(queueSlot, contact.getMobilePhone()));
							}
						}
					}

					if (!CollectionUtils.isEmpty(validationErrors)) {

						return queueSlotCRUD.show(req, res, user, uriParser, validationErrors);

					} else {

						redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());
						return null;
					}
				}
			}

			log.info("User " + user + " requesting add payment form for queue slot " + queueSlot);

			Document doc = createDocument(req, uriParser, user);

			Element addPaymentElement = doc.createElement("AddPayment");
			doc.getDocumentElement().appendChild(addPaymentElement);

			addPaymentElement.appendChild(queueSlot.toXML(doc));

			if (validationErrors != null) {

				addPaymentElement.appendChild(RequestUtils.getRequestParameters(req, doc));
				XMLUtils.append(doc, addPaymentElement, "ValidationErrors", validationErrors);
			}

			return new SimpleForegroundModuleResponse(doc);
		}

		throw new URINotFoundException(uriParser);
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public QueueSlot getQueueSlot(Integer queueSlotID, Field... relations) throws SQLException {

		HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.CONTACT_RELATION, QueueSlot.QUEUE_RELATION, Queue.INTERNAL_GROUPS_RELATION);
		query.addParameter(queueSlotDAOWrapper.getParameterFactory().getParameter(queueSlotID));

		if (relations != null) {
			query.addRelations(relations);
		}

		return queueSlotDAO.get(query);
	}
	
	public List<QueueSlot> getQueueSlots(Collection<Integer> queueSlotIDs, Field... relations) throws SQLException {

		HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.CONTACT_RELATION, QueueSlot.QUEUE_RELATION, Queue.INTERNAL_GROUPS_RELATION);
		query.addParameter(queueSlotDAOWrapper.getParameterFactory().getWhereInParameter(queueSlotIDs));

		if (relations != null) {
			query.addRelations(relations);
		}

		return queueSlotDAO.getAll(query);
	}

	public List<QueueSlot> getQueueSlots(Queue queue, Field... relations) throws SQLException {

		HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.CONTACT_RELATION, QueueSlot.QUEUE_RELATION, Queue.INTERNAL_GROUPS_RELATION);

		if (relations != null) {
			query.addRelations(relations);
		}

		query.addParameter(queueSlotQueueParamFactory.getParameter(queue));
		query.addParameter(queueSlotArchivedParamFactory.getParameter(false));

		List<QueueSlot> slots = queueSlotDAO.getAll(query);

		if (!CollectionUtils.isEmpty(slots)) {

			int i = 1;
			for (QueueSlot slot : slots) {

				slot.setCalculatedPosition(i++);
			}
		}

		return slots;
	}

	public List<QueueSlot> getArchivedQueueSlots(Queue queue, Field... relations) throws SQLException {

		HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.CONTACT_RELATION, QueueSlot.QUEUE_RELATION, Queue.INTERNAL_GROUPS_RELATION);

		if (relations != null) {
			query.addRelations(relations);
		}

		query.addParameter(queueSlotQueueParamFactory.getParameter(queue));
		query.addParameter(queueSlotArchivedParamFactory.getParameter(true));

		return queueSlotDAO.getAll(query);
	}

	public List<Queue> getQueues(User user) throws SQLException {

		return queueAdminModule.getQueues(user);
	}

	public List<Queue> getAllQueues() throws SQLException {

		return queueDAO.getAll();
	}

	public FlowInstance getFlowInstance(Integer flowInstanceID) throws SQLException {

		return flowAdminModule.getFlowInstance(flowInstanceID);
	}

	// Copied from com.nordicpeak.flowengine.infomodule.FlowInstanceInfoModule
	public void sendFlowInstancePDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, Integer flowInstanceID) throws Throwable {

		FlowInstance flowInstance = getFlowInstance(flowInstanceID);

		if (flowInstance == null) {

			throw new URINotFoundException(uriParser);
		}

		FlowInstanceAdminModule.GENERAL_ACCESS_CONTROLLER.checkFlowInstanceAccess(flowInstance, user);

		log.info("User " + user + " requesting PDF for flow instance " + flowInstance);

		FlowInstanceEvent event = getLatestPDFEvent(flowInstance);

		if (event == null) {

			log.info("Unable to find any PDF events for flow instance " + flowInstance + " requested by user " + user);

			throw new URINotFoundException(uriParser);
		}

		File pdfFile = pdfProvider.getPDF(flowInstanceID, event.getEventID());

		if (pdfFile == null) {

			log.warn("Unable to find PDF file for event " + event + " for flow instance " + flowInstance + " requested by user " + user);

			throw new URINotFoundException(uriParser);
		}

		HTTPUtils.sendFile(pdfFile, "flowinstance-" + flowInstanceID + "-event-" + event.getEventID() + ".pdf", req, res, ContentDisposition.ATTACHMENT);
	}

	// Copied from com.nordicpeak.flowengine.infomodule.FlowInstanceInfoModule
	private FlowInstanceEvent getLatestPDFEvent(FlowInstance flowInstance) {

		if (CollectionUtils.isEmpty(flowInstance.getEvents())) {

			return null;
		}

		for (int i = flowInstance.getEvents().size() - 1; i > -1; i--) {

			FlowInstanceEvent event = flowInstance.getEvents().get(i);

			if (Boolean.TRUE.equals(event.getAttributeHandler().getBoolean("pdf"))) {

				return event;
			}
		}

		return null;
	}

	public Person getPerson(String citizenIdentifier) {

		if (personProvider != null) {

			return personProvider.getPerson(citizenIdentifier);
		}

		return null;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateContact(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		QueueSlot queueSlot;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (queueSlot = queueSlotDAOWrapper.get(Integer.valueOf(uriParser.get(2)))) != null) {

			queueSlotCRUD.checkUpdateAccess(queueSlot, user, req, uriParser);

			User poster = getUserHandler().getUserByAttribute("citizenIdentifier", queueSlot.getPrimaryContact().getCitizenIdentifier(), false, true);

			if (poster != null) {

				redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());
				return null;
			}

			ManualContact contact = queueSlot.getManualContact();

			log.info("User " + user + " requesting update contact form for queue slot " + queueSlot);

			Document doc = this.createDocument(req, uriParser, user);

			Element updateContactElement = doc.createElement("UpdateContact");

			doc.getDocumentElement().appendChild(updateContactElement);

			List<ValidationError> validationErrors = null;

			if (req.getMethod().equalsIgnoreCase("POST")) {

				validationErrors = new ArrayList<>();

				try {
					contact = MANUAL_CONTACT_POPULATOR.populate(contact, req);

				} catch (ValidationException e) {

					validationErrors.addAll(e.getErrors());
				}

				if (CollectionUtils.isEmpty(validationErrors)) {

					log.info("User " + user + " updating contact for queue slot " + queueSlot);

					if (contact.getQueueSlotID() == null) {

						contact.setQueueSlotID(queueSlot.getQueueSlotID());
						contact.setCitizenIdentifier(queueSlot.getPrimaryContact().getCitizenIdentifier());

						manualContactDAO.add(contact);

					} else {

						manualContactDAO.update(contact);
					}

					redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());
					return null;
				}
			}

			updateContactElement.appendChild(queueSlot.toXML(doc));

			if (validationErrors != null) {

				for (ValidationError validationError : validationErrors) {
					updateContactElement.appendChild(validationError.toXML(doc));
				}

				updateContactElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			}

			return new SimpleForegroundModuleResponse(doc, queueSlot.getQueue().getName(), this.getDefaultBreadcrumb());
		}

		return defaultMethod(req, res, user, uriParser);
	}

	@WebPublic(alias = "internalattachment")
	public ForegroundModuleResponse getInternalMessageAttachment(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException {

		return internalMessageCRUD.getRequestedMessageAttachment(req, res, user, uriParser);
	}

	@Override
	public Collection<Integer> getAllowedGroupIDs() {

		return adminGroupIDs;
	}

	@Override
	public Collection<Integer> getAllowedUserIDs() {

		return adminUserIDs;
	}

	@Override
	public boolean allowsAdminAccess() {

		return false;
	}

	@Override
	public boolean allowsUserAccess() {

		return false;
	}

	@Override
	public boolean allowsAnonymousAccess() {

		return false;
	}

	public boolean hasAdminAccess(User user) {

		return AccessUtils.checkAccess(user, this);
	}

	@RESTMethod(alias = "export/{queueID}", method = { "get", "post" }, requireLogin = true)
	public ForegroundModuleResponse exportQueueToExcel(HttpServletRequest req, final HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "queueID") Integer queueID) throws InterruptedException, Throwable {

		final Queue queue = queueDAOWrapper.get(queueID);

		if (queue == null) {

			throw new URINotFoundException(uriParser);
		}

		if (!hasQueueManagerAccess(queue, user) && !hasAdminAccess(user)) {

			throw new AccessDeniedException(getSectionDescriptor());
		}

		final TreeSet<Integer> queueSlotIDs = new TreeSet<>();
		String[] queueSlotIDStrings = req.getParameterValues("queueSlotID");

		if (queueSlotIDStrings != null) {

			for (String queueSlotIDString : queueSlotIDStrings) {

				Integer id = NumberUtils.toInt(queueSlotIDString);

				if (id != null) {
					queueSlotIDs.add(id);
				}
			}
		}

		if (queueSlotIDs.isEmpty()) {

			log.info("User " + user + " exporting queue " + queue + " to excel");

		} else {

			log.info("User " + user + " exporting queue slots " + StringUtils.toCommaSeparatedString(queueSlotIDs) + " from queue " + queue + " to excel");
		}

		//TODO use QueryResultsStreamer
		final List<QueueSlot> queueSlots = getQueueSlots(queue, QueueSlot.MANUAL_CONTACT_RELATION, QueueSlot.EXTERNAL_MESSAGES_RELATION, QueueSlot.EVENTS_RELATION, QueueSlotEvent.ATTRIBUTES_RELATION);
		final List<QueueSlot> archivedQueueSlots = getArchivedQueueSlots(queue, QueueSlot.MANUAL_CONTACT_RELATION, QueueSlot.EXTERNAL_MESSAGES_RELATION, QueueSlot.EVENTS_RELATION, QueueSlotEvent.ATTRIBUTES_RELATION);

		final Date now = new Date();
		final String nowString = DateUtils.DATE_FORMATTER.format(now);

		res.setHeader("Content-Disposition", "attachment;filename=\"" + FileUtils.toValidHttpFilename(queue.getName()) + " " + nowString + ".xlsx\"");
		res.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		//Workaround for thread local usage in XMLBeans used by XSSF
		ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

			private Exception exception;

			@Override
			public void run() {

				try {
					Workbook workbook = new XSSFWorkbook();

					try {
						Sheet sheet = workbook.createSheet("Aktiva kplatser");

						int rowIndex = 0;
						int columnIndex = 0;

						Row titleRow = sheet.createRow(rowIndex++);
						columnIndex = 0;

						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotPosition);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotAdded);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPayment);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPaymentType);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPaymentRequested);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotComment);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotDigital);

						columnIndex = SpreadSheetUtils.addContactTitleCells(titleRow, columnIndex);

						if (queueSlots != null) {

							for (QueueSlot queueSlot : queueSlots) {

								if (!queueSlotIDs.isEmpty() && !queueSlotIDs.contains(queueSlot.getQueueSlotID())) {
									continue;
								}

								Row slotRow = sheet.createRow(rowIndex++);
								columnIndex = 0;

								SpreadSheetUtils.addCell(slotRow, columnIndex++, queueSlot.getCalculatedPosition());
								SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getAdded()));

								if (queueSlot.getLastPaymentDate() != null) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getLastPaymentDate()));

									QueueSlotEvent paymentEvent = QueueUtils.getLatestPaymentEvent(queueSlot);
									String paymentType = paymentEvent.getAttributeHandler().getString(PaymentResponse.ATTRIBUTE_PAYMENT_TYPE);

									if ("DIRECT".equalsIgnoreCase(paymentType)) {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, columnQueueSlotLatestPaymentTypeDirect);

									} else if ("INVOICE".equalsIgnoreCase(paymentType)) {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, queue.getEventPaymentInvoiceMessage());

									} else {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, paymentType);
									}

								} else {

									columnIndex++;
									columnIndex++;
								}

								if (queueSlot.getPendingPaymentRequestDate() != null) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getPendingPaymentRequestDate()));

								} else {

									columnIndex++;
								}

								SpreadSheetUtils.addCell(slotRow, columnIndex++, getFirstCommentExternalMessage(queueSlot));

								if (QueueUtils.isDigitalMember(queueSlot, systemInterface.getUserHandler())) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, columnQueueSlotYes);

								} else {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, columnQueueSlotNo);
								}

								Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());
								columnIndex = SpreadSheetUtils.addContactCells(contact, slotRow, columnIndex);
							}
						}

						Sheet sheet2 = workbook.createSheet("Arkiverade kplatser");

						rowIndex = 0;
						columnIndex = 0;

						titleRow = sheet2.createRow(rowIndex++);
						columnIndex = 0;

						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotAdded);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotArchivedDate);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPayment);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPaymentType);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotLatestPaymentRequested);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotComment);
						SpreadSheetUtils.addCell(titleRow, columnIndex++, columnQueueSlotWaitingPeriod);

						columnIndex = SpreadSheetUtils.addContactTitleCells(titleRow, columnIndex);

						if (archivedQueueSlots != null) {

							for (QueueSlot queueSlot : archivedQueueSlots) {

								if (!queueSlotIDs.isEmpty() && !queueSlotIDs.contains(queueSlot.getQueueSlotID())) {
									continue;
								}

								Row slotRow = sheet2.createRow(rowIndex++);
								columnIndex = 0;

								SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getAdded()));

								if (queueSlot.getArchivedDate() != null) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getArchivedDate()));
								}

								if (queueSlot.getLastPaymentDate() != null) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getLastPaymentDate()));

									QueueSlotEvent paymentEvent = QueueUtils.getLatestPaymentEvent(queueSlot);
									String paymentType = paymentEvent.getAttributeHandler().getString(PaymentResponse.ATTRIBUTE_PAYMENT_TYPE);

									if ("DIRECT".equalsIgnoreCase(paymentType)) {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, columnQueueSlotLatestPaymentTypeDirect);

									} else if ("INVOICE".equalsIgnoreCase(paymentType)) {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, queue.getEventPaymentInvoiceMessage());

									} else {

										SpreadSheetUtils.addCell(slotRow, columnIndex++, paymentType);
									}

								} else {

									columnIndex++;
									columnIndex++;
								}

								if (queueSlot.getPendingPaymentRequestDate() != null) {

									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(queueSlot.getPendingPaymentRequestDate()));

								} else {

									columnIndex++;
								}

								SpreadSheetUtils.addCell(slotRow, columnIndex++, getFirstCommentExternalMessage(queueSlot));

								if (queueSlot.getWaitingPeriodTime() != null) {
									SpreadSheetUtils.addCell(slotRow, columnIndex++, DateUtils.DATE_FORMATTER.format(queueSlot.getWaitingPeriodTime()));
								} else {
									SpreadSheetUtils.addCell(slotRow, columnIndex++, "");

								}

								Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());
								columnIndex = SpreadSheetUtils.addContactCells(contact, slotRow, columnIndex);
							}
						}

						workbook.write(res.getOutputStream());

					} finally {

						CloseUtils.close(workbook);
					}

				} catch (Exception e) {

					exception = e;
				}
			}

			@Override
			public Exception getException() {

				return exception;
			}
		};

		ThreadUtils.runAndWait(runnable, "Excel export of queue " + queue + " started by user " + user, true);

		if (runnable.getException() != null) {

			throw runnable.getException();
		}

		return null;
	}

	private String getFirstCommentExternalMessage(QueueSlot queueSlot) {

		if (!CollectionUtils.isEmpty(queueSlot.getExternalMessages())) {

			for (ExternalMessage externalMessage : queueSlot.getExternalMessages()) {

				if (externalMessage.getType() == ExternalMessageType.ENTRANCE) {

					return externalMessage.getMessage();
				}
			}
		}

		return "";
	}

	@WebPublic(alias = "import")
	public ForegroundModuleResponse importQueueSlotsFromExcelUpload(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Queue queue;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (queue = queueDAOWrapper.get(Integer.valueOf(uriParser.get(2)))) != null) {

			if (!hasQueueManagerAccess(queue, user) && !hasAdminAccess(user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			ValidationException validationException = null;

			if (req.getMethod().equalsIgnoreCase("POST") && MultipartRequest.isMultipartRequest(req)) {

				log.info("User " + user + " importing queue slots to queue " + queue);

				MultipartRequest multipartRequest = null;

				try {
					multipartRequest = MultipartRequest.getMultipartRequest(ramThreshold * BinarySizes.KiloByte, maxRequestSize * BinarySizes.MegaByte, tempDir, req);
					req = multipartRequest;

					if (multipartRequest.getFileCount() == 0 || multipartRequest.getFileCount() == 1 && multipartRequest.getFile(0).getName().equals("")) {

						throw new ValidationException(new ValidationError("NoAttachedFile"));
					}

					FileItem fileItem = multipartRequest.getFile(0);

					if (!fileItem.getName().endsWith(".xlsx")) {

						throw new ValidationException(new InvalidFileExtensionValidationError(FilenameUtils.getName(fileItem.getName()), "xlsx"));
					}

					InputStream inputStream = null;

					try {
						inputStream = fileItem.getInputStream();

						validationException = importQueueSlotsFromExcel(inputStream, FilenameUtils.getName(fileItem.getName()), queue, multipartRequest, res, user, uriParser);

					} finally {

						CloseUtils.close(inputStream);
					}

				} catch (ValidationException e) {

					validationException = e;

				} catch (SizeLimitExceededException e) {

					validationException = new ValidationException(new RequestSizeLimitExceededValidationError(e.getActualSize(), e.getPermittedSize()));

				} catch (FileSizeLimitExceededException e) {

					validationException = new ValidationException(new FileSizeLimitExceededValidationError(e.getFileName(), e.getActualSize(), e.getPermittedSize()));

				} catch (FileUploadException e) {

					validationException = new ValidationException(new ValidationError("UnableToParseRequest"));

				} finally {

					if (validationException != null) {

						log.info("Import of queue slots to " + queue + " by user " + user + " failed due to validation error(s) " + validationException);
					}

					if (multipartRequest != null) {

						multipartRequest.deleteFiles();
					}
				}
			}

			log.info("User " + user + " requesting export import form for " + queue);

			Document doc = this.createDocument(req, uriParser, user);
			Element importFlowElement = doc.createElement("ImportExcel");
			doc.getFirstChild().appendChild(importFlowElement);

			if (validationException != null) {
				importFlowElement.appendChild(validationException.toXML(doc));
				importFlowElement.appendChild(RequestUtils.getRequestParameters(req, doc, "categoryID"));
			}

			return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), this.getDefaultBreadcrumb());
		}

		return defaultMethod(req, res, user, uriParser);
	}

	private synchronized ValidationException importQueueSlotsFromExcel(final InputStream inputStream, final String filename, final Queue queue, final HttpServletRequest req, final HttpServletResponse res, final User user, final URIParser uriParser) throws Throwable {

		final AnnotatedDAO<QueueSlot> queueSlotDAO = this.queueSlotDAO;

		//Workaround for thread local usage in XMLBeans used by XSSF
		ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

			private Exception exception;

			@Override
			public void run() {

				Workbook workbook = null;

				try {
					try {
						workbook = new XSSFWorkbook(inputStream);

					} catch (Exception e) {

						log.info("Unable to parse file " + filename, e);
						throw new ValidationException(new UnableToParseFileValidationError(filename));
					}

					Sheet sheet = null;

					if (workbook.getNumberOfSheets() > 0) {

						sheet = workbook.getSheetAt(0);
					}

					if (sheet == null) {

						log.info("Error parsing file " + filename + ", unable to find sheet");
						throw new ValidationException(new ValidationError("ErrorImportingExcel", "Hittar inget blad att importera rader ifrn"));
					}

					Row headerRow = sheet.getRow(sheet.getFirstRowNum());

					if (headerRow == null) {

						log.info("Error parsing file " + filename + ", unable to find header row");
						throw new ValidationException(new ValidationError("ErrorImportingExcel", "Hittar inte sidhuvudet (frsta raden med kolumnnamn)"));
					}

					int columnCount = 0;
					int rowCount = sheet.getLastRowNum() - sheet.getFirstRowNum();

					// Fix for header styling counting as column count
					for (int i = 0; i < headerRow.getLastCellNum(); i++) {

						Cell cell = headerRow.getCell(i);

						if (!StringUtils.isEmpty(getCellValue(cell))) {
							columnCount++;
						}
					}

					if (columnCount < 12) {

						log.info("Error parsing file " + filename + ", wrong column count " + columnCount + ", expected at least 12");
						throw new ValidationException(new ValidationError("ErrorImportingExcel", "Hittar endast " + columnCount + " kolumner. Frvntas minst 12 fr att kunna importera."));
					}

					if (rowCount < 1) {

						log.info("Error parsing file " + filename + ", too few rows " + rowCount);
						throw new ValidationException(new ValidationError("ErrorImportingExcel", "Hittar inga rader att importera"));
					}

					boolean useDescriptionColumn = columnCount > 12 && "Beskrivning".equalsIgnoreCase(getCellValue(headerRow.getCell(12)));

					List<ValidationError> validationErrors = new ArrayList<>();

					//Create transaction
					TransactionHandler transactionHandler = null;

					try {
						transactionHandler = queueSlotDAO.createTransaction();

						ObjectQuery<Integer> indexQuery = transactionHandler.getObjectQuery("SELECT MAX(queueIndex) FROM " + queueSlotDAOWrapper.getAnnotatedDAO().getTableName() + " WHERE queueID = ?", IntegerPopulator.getPopulator());
						indexQuery.setInt(1, queue.getQueueID());

						int queueIndex = indexQuery.executeQuery();

						List<QueueSlot> slots = new ArrayList<>(rowCount);

						for (int rowIndex = headerRow.getRowNum() + 1; rowIndex <= sheet.getLastRowNum(); rowIndex++) {

							Row row = sheet.getRow(rowIndex);

							if (row != null) {

								String firstname = fixCase(getCellValue(row.getCell(0)));
								String lastname = fixCase(getCellValue(row.getCell(1)));
								String citizenIdentifier = getCellValue(row.getCell(2));
								String email = getCellValue(row.getCell(3));
								String phone = cleanPhone(getCellValue(row.getCell(4)), rowIndex, validationErrors);
								String phoneMobile = cleanPhone(getCellValue(row.getCell(5)), rowIndex, validationErrors);
								String address = fixCase(getCellValue(row.getCell(6)));
								String zipcode = getCellValue(row.getCell(7));
								String postalAddress = fixCase(getCellValue(row.getCell(8)));
								Date entryDate = getCellDateValue(row.getCell(9));
								Date latestPaymentDate = getCellDateValue(row.getCell(10));
								String latestPaymentAmount = getCellValue(row.getCell(11));

								if (ObjectUtils.isNull(firstname, lastname, citizenIdentifier, email, phone, phoneMobile, address, zipcode, postalAddress, entryDate, latestPaymentDate, latestPaymentAmount)) {
									continue;
								}

								int rowColumnCount = row.getLastCellNum();

								List<String> comments = new ArrayList<>();

								for (int i = useDescriptionColumn ? 13 : 12; i < rowColumnCount; i++) {

									String comment = getCellValue(row.getCell(i));

									if (!StringUtils.isEmpty(comment)) {
										comments.add(comment);
									}
								}

								if (StringUtils.isEmpty(citizenIdentifier)) {

									validationErrors.add(new ValidationError("ErrorImportingExcel", "Personnummer saknas p rad " + (rowIndex + 1)));

								} else if (!SWEDISH_SOCIAL_SECURITY_POPULATOR.validateFormat(citizenIdentifier)) {

									validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt personnummer \"" + citizenIdentifier + "\" p rad " + (rowIndex + 1)));
								}

								if (!StringUtils.isEmpty(email) && !EmailPopulator.getPopulator().validateFormat(email)) {

									validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt epost \"" + email + "\" p rad " + (rowIndex + 1)));
								}

								if (!StringUtils.isEmpty(zipcode) && !NonNegativeStringIntegerPopulator.getPopulator().validateFormat(zipcode)) {

									validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt postnummer \"" + zipcode + "\" p rad " + (rowIndex + 1)));
								}

								QueueSlot queueSlot = new QueueSlot();

								queueSlot.setQueue(queue);

								if (entryDate == null) {

									validationErrors.add(new ValidationError("ErrorImportingExcel", "Intrdesdatum saknas p rad " + (rowIndex + 1)));

								} else {

									try {
										queueSlot.setAdded(new Timestamp(entryDate.getTime()));

										if (queueSlot.getAdded().after(new Date())) {

											validationErrors.add(new ValidationError("ErrorImportingExcel", "Registreringsdatumet \"" + entryDate + "\" r i framtiden p rad " + (rowIndex + 1)));
										}

									} catch (Exception e) {

										validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt registreringsdatum \"" + entryDate + "\" p rad " + (rowIndex + 1)));
									}
								}

								PrimaryContact primaryContact = new PrimaryContact();

								primaryContact.setCitizenIdentifier(citizenIdentifier);
								primaryContact.setFirstname(firstname);
								primaryContact.setLastname(lastname);
								primaryContact.setEmail(email);
								primaryContact.setPhone(phone);
								primaryContact.setMobilePhone(phoneMobile);
								primaryContact.setAddress(address);
								primaryContact.setZipCode(zipcode);
								primaryContact.setPostalAddress(postalAddress);
								queueSlot.setPrimaryContact(primaryContact);

								if (latestPaymentDate != null) {

									if (StringUtils.isEmpty(latestPaymentAmount)) {

										validationErrors.add(new ValidationError("ErrorImportingExcel", "Betalnings belopp saknas p rad " + (rowIndex + 1)));

									} else if (!NonNegativeStringIntegerPopulator.getPopulator().validateFormat(latestPaymentAmount)) {

										validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt betalnings belopp \"" + latestPaymentAmount + "\" p rad " + (rowIndex + 1)));
									}

									try {
										Timestamp timestamp = new Timestamp(latestPaymentDate.getTime());

										if (timestamp.after(new Date())) {

											validationErrors.add(new ValidationError("ErrorImportingExcel", "Betalningsdatumet \"" + entryDate + "\" r i framtiden p rad " + (rowIndex + 1)));
										}

										queueSlot.setLastPaymentDate(timestamp);
										queueSlot.addEvent(new QueueSlotEvent(timestamp, null, QueueSlotEventType.PAYED, getEventPaymentManagerMessage() + " (" + latestPaymentAmount + " SEK)", null));

									} catch (Exception e) {

										validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt betalningsdatum \"" + latestPaymentDate + "\" p rad " + (rowIndex + 1)));
									}
								}

								if (useDescriptionColumn) {

									String description = getCellValue(row.getCell(12));

									if (!StringUtils.isEmpty(description)) {
										queueSlot.getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_DESCRIPTION_ATTRIBUTE, description);
									}
								}

								if (!CollectionUtils.isEmpty(comments)) {

									queueSlot.setInternalMessages(new ArrayList<InternalMessage>(comments.size()));

									for (String comment : comments) {

										queueSlot.getInternalMessages().add(new InternalMessage(queueSlot, comment, null));
									}
								}

								queueSlot.setQueueIndex(++queueIndex);

								slots.add(queueSlot);
							}
						}

						if (!validationErrors.isEmpty()) {

							throw new ValidationException(validationErrors);
						}

						HighLevelQuery<QueueSlot> query = new HighLevelQuery<>(QueueSlot.CONTACT_RELATION, QueueSlot.EVENTS_RELATION, QueueSlot.INTERNAL_MESSAGES_RELATION, QueueSlot.ATTRIBUTES_RELATION);
						queueSlotDAO.addAll(slots, transactionHandler, query);

						transactionHandler.commit();

						log.info("User " + user + " successfully imported " + slots.size() + " queue slots to " + queue);

						getEventHandler().sendEvent(QueueSlot.class, new CRUDEvent<>(QueueSlot.class, CRUDAction.ADD, slots), EventTarget.ALL);

						redirectToMethod(req, res, "/list/" + queue.getQueueID());

					} finally {

						TransactionHandler.autoClose(transactionHandler);
					}

				} catch (Exception e) {

					exception = e;

				} finally {

					CloseUtils.close(workbook);
				}
			}

			private String fixCase(String text) {

				if (!StringUtils.isEmpty(text)) {

					text = text.toLowerCase();

					if (text.length() > 1) {

						text = text.substring(0, 1).toUpperCase() + text.substring(1);
					}

					return text;
				}

				return null;
			}

			private String cleanPhone(String text, int rowIndex, List<ValidationError> validationErrors) {

				if (!StringUtils.isEmpty(text)) {

					if (text.equals("0")) {
						return null;
					}

					if (!SwedishPhoneNumberValidator.getValidator().validateFormat(text)) {

						validationErrors.add(new ValidationError("ErrorImportingExcel", "Ogiltigt telefonummer: \"" + text + "\" p rad " + (rowIndex + 1)));
					}

					return text;
				}

				return null;
			}

			private String getCellValue(Cell cell) {

				if (cell != null) {

					CellType type = cell.getCellType();

					if (type == CellType.NUMERIC) {

						return Integer.toString((int) cell.getNumericCellValue());

					} else if (type == CellType.STRING) {

						String value = cell.getStringCellValue();

						if (StringUtils.isEmpty(value) || "NULL".equalsIgnoreCase(value) || "-".equals(value)) {
							return null;
						}

						return value.trim();
					}

					return cell.toString();
				}

				return null;
			}

			private Date getCellDateValue(Cell cell) throws ParseException {

				if (cell != null) {

					CellType type = cell.getCellType();

					if (type == CellType.NUMERIC && DateUtil.isCellDateFormatted(cell)) {

						return cell.getDateCellValue();

					} else if (type == CellType.STRING) {

						String value = cell.getStringCellValue();

						if (StringUtils.isEmpty(value) || "NULL".equalsIgnoreCase(value) || "-".equals(value)) {
							return null;
						}

						return DateUtils.DATE_FORMATTER.parse(value.trim());
					}
				}

				return null;
			}

			@Override
			public Exception getException() {

				return exception;
			}
		};

		ThreadUtils.runAndWait(runnable, "Queue slot importing for user " + user, true);

		if (runnable.getException() != null) {

			if (runnable.getException() instanceof ValidationException) {

				return (ValidationException) runnable.getException();
			}

			log.error("Error importing queue slots to " + queue + " for user " + user + "", runnable.getException());

			return new ValidationException(new UnableToParseFileValidationError(filename));
		}

		return null;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse sendMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Queue queue;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (queue = queueDAOWrapper.get(Integer.valueOf(uriParser.get(2)))) != null) {

			if (!hasQueueManagerAccess(queue, user) && !hasAdminAccess(user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}
			
			if(req.getMethod().equals("GET")) {
				redirectToMethod(req, res, "/list/"+uriParser.get(2));
				return null;
			}
			
			Document doc = createDocument(req, uriParser, user);
			Element sendMessageElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "SendMessageForm");

			XMLUtils.appendNewElement(doc, sendMessageElement, "FormatedMaxEmailFileSize", BinarySizeFormater.getFormatedSize(maxEmailFileSize * BinarySizes.MegaByte));
			XMLUtils.appendNewElement(doc, sendMessageElement, "FormatedMaxMessageFileSize", BinarySizeFormater.getFormatedSize(maxMessageFileSize * BinarySizes.MegaByte));

			List<QueueSlot> queueSlots = getQueueSlots(queue, QueueSlot.MANUAL_CONTACT_RELATION);
			
			String[] queueSlotIDs = req.getParameterValues("queueSlotID");
			
			List<ValidationError> validationErrors = new ArrayList<>();

			if (req.getMethod().equals("POST") && StringUtils.isEmpty(req.getParameter("loadMessageForm"))) {

				try {
					
					if (MultipartRequest.isMultipartRequest(req)) {

						req = MultipartRequest.getMultipartRequest(this.ramThreshold * BinarySizes.KiloByte, this.maxRequestSize * BinarySizes.MegaByte, tempDir, req);
					}

					boolean toDigitalUsers = !StringUtils.isEmpty(req.getParameter("toDigital"));
					boolean toAnalogWithEmail = !StringUtils.isEmpty(req.getParameter("toAnalogWithEmail"));
					boolean toDigitalPaymentPending = !StringUtils.isEmpty(req.getParameter("toDigitalPaymentPending"));
					boolean toAnalogPaymentPendingWithEmail = !StringUtils.isEmpty(req.getParameter("toAnalogPaymentPendingWithEmail"));

					boolean sendEmail = !StringUtils.isEmpty(req.getParameter("send-email"));
					boolean sendSMS = !StringUtils.isEmpty(req.getParameter("send-sms"));
					boolean sendExternalMessage = !StringUtils.isEmpty(req.getParameter("send-external-message"));
					boolean overrideUserSettingSMS = !StringUtils.isEmpty(req.getParameter("override-user-sms-settings"));
					
					queueSlotIDs = req.getParameterValues("queueSlotID");
					boolean selectedQueueSlots = false;
					
					if(queueSlotIDs != null) {
						Collection<Integer> selectedQueueSlotIDs = new ArrayList<>();
						for (String queueSlotID : queueSlotIDs) {
							selectedQueueSlotIDs.add(NumberUtils.toInt(queueSlotID));
						}
						queueSlots = getQueueSlots(selectedQueueSlotIDs);
						
						selectedQueueSlots = true;
						
					}

					if (queueSlotIDs == null && (!toDigitalUsers && !toAnalogWithEmail && !toDigitalPaymentPending && !toAnalogPaymentPendingWithEmail)) {
						validationErrors.add(NO_RECIPIENT_TYPES_VALIDATION_ERROR);
					}

					if (!sendEmail && !sendSMS && !sendExternalMessage) {

						validationErrors.add(NO_MESSAGE_PATH_VALIDATION_ERROR);
					}

					String emailSubject = ValidationUtils.validateParameter("emailSubject", req, sendEmail, 1, 255, validationErrors);
					String emailMessage = ValidationUtils.validateParameter("emailMessage", req, sendEmail, 1, 65535, validationErrors);
					String smsMessage = ValidationUtils.validateParameter("smsMessage", req, sendSMS, 1, null, validationErrors);
					String externalMessageString = ValidationUtils.validateParameter("externalMessage", req, sendExternalMessage, 1, null, validationErrors);
					Integer minimumDaysDigitalPaymentRequestHasBeenActive = ValidationUtils.validateParameter("minimumDaysDigitalPaymentRequestHasBeenActive", req, toDigitalPaymentPending, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
					Integer minimumDaysAnalogPaymentRequestHasBeenActive = ValidationUtils.validateParameter("minimumDaysAnalogPaymentRequestHasBeenActive", req, toAnalogPaymentPendingWithEmail, PositiveStringIntegerPopulator.getPopulator(), validationErrors);

					Collection<FileItem> attachedEmailFileItems = null;
					Collection<FileItem> attachedExternalMessageFileItems = null;

					if (req instanceof MultipartRequest && ((MultipartRequest) req).getFileCount() > 0) {

						attachedEmailFileItems = ((MultipartRequest) req).getFiles("email-attachments");
						attachedExternalMessageFileItems = ((MultipartRequest) req).getFiles("external-message-attachments");

						Iterator<FileItem> emailFileIterator = attachedEmailFileItems.iterator();

						while (emailFileIterator.hasNext()) {

							FileItem fileItem = emailFileIterator.next();

							if (StringUtils.isEmpty(fileItem.getName()) || fileItem.getSize() == 0) {

								emailFileIterator.remove();
								continue;
							}

							if (fileItem.getSize() > maxEmailFileSize * BinarySizes.MegaByte) {

								validationErrors.add(new FileSizeLimitExceededValidationError(FilenameUtils.getName(fileItem.getName()), fileItem.getSize(), maxEmailFileSize * BinarySizes.MegaByte));

								continue;
							}
						}

						Iterator<FileItem> externalMessageFileIterator = attachedExternalMessageFileItems.iterator();

						while (externalMessageFileIterator.hasNext()) {

							FileItem fileItem = externalMessageFileIterator.next();

							if (StringUtils.isEmpty(fileItem.getName()) || fileItem.getSize() == 0) {

								externalMessageFileIterator.remove();
								continue;
							}

							if (fileItem.getSize() > maxMessageFileSize * BinarySizes.MegaByte) {

								validationErrors.add(new FileSizeLimitExceededValidationError(FilenameUtils.getName(fileItem.getName()), fileItem.getSize(), maxMessageFileSize * BinarySizes.MegaByte));

								continue;
							}
						}
					}

					if (validationErrors.isEmpty()) {

						List<Attachment> emailAttachments = new ArrayList<>(CollectionUtils.getSize(attachedEmailFileItems));
						List<ExternalMessageAttachment> externalMessageattachments = new ArrayList<>(CollectionUtils.getSize(attachedExternalMessageFileItems));

						if (sendEmail && attachedEmailFileItems != null) {

							for (FileItem file : attachedEmailFileItems) {

								FileItemAttachment attachment = new FileItemAttachment(file);

								emailAttachments.add(attachment);
							}
						}

						if (sendExternalMessage && attachedExternalMessageFileItems != null) {

							for (FileItem file : attachedExternalMessageFileItems) {

								ExternalMessageAttachment externalMessageAttachment = new ExternalMessageAttachment();

								String fileName = FilenameUtils.getName(file.getName());

								externalMessageAttachment.setFilename(fileName);
								externalMessageAttachment.setSize(file.getSize());
								externalMessageAttachment.setData(new SerialBlob(file.get()));

								externalMessageattachments.add(externalMessageAttachment);
							}
						}

						int sentEmails = 0;
						int sentSMSs = 0;
						int sentExternalMessages = 0;
						int sentExternalMessageEmailNotifications = 0;
						int sentExternalMessagesSMSNotifications = 0;

						ExternalMessage externalMessage = null;

						Timestamp now = TimeUtils.getCurrentTimestamp();

						if (!CollectionUtils.isEmpty(queueSlots)) {

							for (QueueSlot queueSlot : queueSlots) {

								queueSlot.setQueue(queue);

								boolean digital = QueueUtils.isDigitalMember(queueSlot, getUserHandler());
								Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());

								boolean validEmail = false;

								if (!StringUtils.isEmpty(contact.getEmail()) && EmailUtils.isValidEmailAddress(contact.getEmail())) {

									validEmail = true;
								}
								
								if(!selectedQueueSlots) {

									if (digital) {
	
										if (!toDigitalUsers && !toDigitalPaymentPending) {
											continue;
										}
	
										if (toDigitalPaymentPending && (queueSlot.getPendingPaymentRequestDate() == null || DateUtils.getDaysBetween(queueSlot.getPendingPaymentRequestDate(), now) < minimumDaysDigitalPaymentRequestHasBeenActive)) {
											continue;
										}
	
									} else {
	
										if (!toAnalogWithEmail && !toAnalogPaymentPendingWithEmail || !validEmail) {
											continue;
										}
	
										if (toAnalogPaymentPendingWithEmail && (queueSlot.getPendingPaymentRequestDate() == null || DateUtils.getDaysBetween(queueSlot.getPendingPaymentRequestDate(), now) < minimumDaysAnalogPaymentRequestHasBeenActive)) {
											continue;
										}
									}
								}

								boolean externalMessageEmailSent = false;
								boolean externalMessageSMSSent = false;

								if (sendExternalMessage && digital) {

									log.info("User " + user + " sending external message to queue slot " + queueSlot);

									externalMessage = queueAdminModule.sendContactExternalMessage(queueSlot, contact, true, user, externalMessageString, externalMessageattachments);

									if (externalMessage != null) {

										QueueSlotEvent queueSlotEvent = addQueueSlotEvent(queueSlot, QueueSlotEventType.MANAGER_MESSAGE_SENT, null, user, null, null);

										ExternalMessageAddedEvent externalMessageAddedEvent = new ExternalMessageAddedEvent(queueSlot, queueSlotEvent, externalMessage, SenderType.MANAGER);
										getEventHandler().sendEvent(QueueSlot.class, externalMessageAddedEvent, EventTarget.ALL);

										sentExternalMessages++;

										externalMessageEmailSent = externalMessageAddedEvent.isEmailNotificationSent();
										externalMessageSMSSent = externalMessageAddedEvent.isSmsNotificationSent();

										if (externalMessageEmailSent) {
											sentExternalMessageEmailNotifications++;
										}

										if (externalMessageSMSSent) {
											sentExternalMessagesSMSNotifications++;
										}
									}
								}

								if (sendEmail && validEmail && !externalMessageEmailSent) {

									log.info("User " + user + " sending message '" + emailSubject + "' to queue slot " + queueSlot);

									if (!EmailUtils.isValidEmailAddress(contact.getEmail())) {

										validationErrors.add(new InvalidRecipientEmailValidationError(queueSlot, contact.getEmail()));

									} else {

										if (queueAdminModule.sendContactEmail(queueSlot, contact, emailSubject, emailMessage, emailAttachments)) {

											sentEmails++;

										} else {

											validationErrors.add(new ErrorSendingToRecipientViaEmailValidationError(queueSlot, contact.getEmail()));
										}
									}
								}

								if (sendSMS && !externalMessageSMSSent && smsSender != null && !StringUtils.isEmpty(contact.getMobilePhone()) && SwedishPhoneNumberValidator.getValidator().validateFormat(contact.getMobilePhone())) {

									log.info("User " + user + " sending sms to " + contact.getMobilePhone() + " in queue slot " + queueSlot+" "+(overrideUserSettingSMS ? " overriding user SMS settings" : ""));

									if (!overrideUserSettingSMS && !contact.isContactBySMS()) {

										validationErrors.add(new DontContactRecipientBySMSValidationError(queueSlot, contact.getMobilePhone()));

									} else if (!queueAdminModule.sendContactSMS(queueSlot, contact, smsMessage, overrideUserSettingSMS)) {

										validationErrors.add(new ErrorSendingToRecipientViaSMSValidationError(queueSlot, contact.getMobilePhone()));

									} else {

										sentSMSs++;
									}
								}
							}
						}

						if (sendEmail) {
							XMLUtils.appendNewElement(doc, sendMessageElement, "SentEmails", sentEmails);
						}

						if (sendSMS) {
							XMLUtils.appendNewElement(doc, sendMessageElement, "SentSMSs", sentSMSs);
						}

						if (sendExternalMessage) {
							XMLUtils.appendNewElement(doc, sendMessageElement, "SentExternalMessages", sentExternalMessages);
							XMLUtils.appendNewElement(doc, sendMessageElement, "SentExternalMessagesEmailNotifications", sentExternalMessageEmailNotifications);
							XMLUtils.appendNewElement(doc, sendMessageElement, "SentExternalMessagesSMSNotifications", sentExternalMessagesSMSNotifications);
						}

						if (!CollectionUtils.isEmpty(queueSlots)) {
							XMLUtils.appendNewElement(doc, sendMessageElement, "QueueSlotCount", queueSlots.size());

						} else {
							XMLUtils.appendNewElement(doc, sendMessageElement, "QueueSlotCount", 0);
						}
					}

				} catch (SizeLimitExceededException e) {

					validationErrors.add(new RequestSizeLimitExceededValidationError(e.getActualSize(), e.getPermittedSize()));

				} catch (FileUploadException e) {

					validationErrors.add(new ValidationError("UnableToParseRequest"));

				} finally {

					if (req instanceof MultipartRequest) {

						((MultipartRequest) req).deleteFiles();
					}
				}

			}

			if (!validationErrors.isEmpty()) {
				XMLUtils.append(doc, sendMessageElement, "ValidationErrors", validationErrors);
			}

			sendMessageElement.appendChild(queue.toXML(doc));

			{
				Element queueSlotCounts = XMLUtils.appendNewElement(doc, sendMessageElement, "QueueSlotCounts");

				int digitalCount = 0;
				int analogWithEmailCount = 0;
				int analogNoEmailOrPhoneCount = 0;
				int digitalPaymentPending = 0;
				int analogPaymentPending = 0;

				if (!CollectionUtils.isEmpty(queueSlots)) {

					QueueUtils.setQueueSlotUsers(queue, queueSlots, getUserHandler());

					for (QueueSlot queueSlot : queueSlots) {

						Contact contact = QueueAdminModule.getContact(queueSlot, null);

						if (QueueUtils.isDigitalMember(queueSlot, getUserHandler())) {

							digitalCount++;

							if (queueSlot.getPendingPaymentRequestDate() != null) {

								digitalPaymentPending++;
							}

						} else if (!StringUtils.isEmpty(contact.getEmail())) {

							analogWithEmailCount++;

							if (queueSlot.getPendingPaymentRequestDate() != null) {

								analogPaymentPending++;
							}

						} else if (!StringUtils.isEmpty(contact.getMobilePhone())) {

							analogNoEmailOrPhoneCount++;
						}
					}
				}

				XMLUtils.appendNewElement(doc, queueSlotCounts, "Digital", digitalCount);
				XMLUtils.appendNewElement(doc, queueSlotCounts, "AnalogWithEmail", analogWithEmailCount);
				XMLUtils.appendNewElement(doc, queueSlotCounts, "AnalogNoEmailOrPhone", analogNoEmailOrPhoneCount);
				XMLUtils.appendNewElement(doc, queueSlotCounts, "DigitalPaymentPending", digitalPaymentPending);
				XMLUtils.appendNewElement(doc, queueSlotCounts, "AnalogPaymentPendingWithEmail", analogPaymentPending);
			}

			sendMessageElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			
			
			if(queueSlotIDs != null) {
				Element queueSlotIDElement = doc.createElement("SelectedQueueSlots");
				for (String queueSlotID : queueSlotIDs) {
					XMLUtils.appendNewElement(doc, queueSlotIDElement, "queueSlotID", queueSlotID);
				}
				sendMessageElement.appendChild(queueSlotIDElement);
			}
			
			log.info("User " + user + " requesting send messages form");
			
			return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse requestPayment(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Queue queue;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (queue = queueDAOWrapper.get(Integer.valueOf(uriParser.get(2)))) != null) {

			if (!hasQueueManagerAccess(queue, user) && !hasAdminAccess(user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}
			
			if(req.getMethod().equals("GET")) {
				redirectToMethod(req, res, "/list/"+uriParser.get(2));
				return null;
			}

			Document doc = createDocument(req, uriParser, user);
			Element requestPaymentElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "RequestPayment");

			List<ValidationError> validationErrors = new ArrayList<>(1);

			if (req.getMethod().equals("POST") && StringUtils.isEmpty(req.getParameter("loadMessageForm"))) {

				Date olderThanDate = ValidationUtils.validateParameter("olderThanDate", req, true, DatePopulator.getPopulator(), validationErrors);
				
				

				if (validationErrors.isEmpty()) {

					int newPaymentRequests = 0;
					SendingStatus sendingStatus = new SendingStatus();

					Timestamp now = new Timestamp(System.currentTimeMillis());
					
					List<QueueSlot> queueSlots = null;
					
					String[] queueSlotIDs = req.getParameterValues("queueSlotID");
					
					if(queueSlotIDs != null) {
						
						Collection<Integer> selectedQueueSlotIDs = new ArrayList<>();
						for (String queueSlotID : queueSlotIDs) {
							selectedQueueSlotIDs.add(NumberUtils.toInt(queueSlotID));
						}
						queueSlots = getQueueSlots(selectedQueueSlotIDs);
							
						
					} else {
						queueSlots = getQueueSlots(queue, QueueSlot.MANUAL_CONTACT_RELATION, QueueSlot.EVENTS_RELATION);
					}

					
					boolean overrideUserSettingSMS = !StringUtils.isEmpty(req.getParameter("override-user-sms-payment-settings"));

					if (!CollectionUtils.isEmpty(queueSlots)) {

						for (QueueSlot queueSlot : queueSlots) {

							if (queueSlot.getLastPaymentDate() == null || queueSlot.getLastPaymentDate().before(olderThanDate)) {

								if (queueSlot.getPendingPaymentRequestDate() == null) {

									newPaymentRequests++;
									queueSlot.setQueue(queue);
									setPaymentPending(now, null, queueSlot, user, validationErrors, sendingStatus, overrideUserSettingSMS);
								}
							}
						}
					}

					XMLUtils.appendNewElement(doc, requestPaymentElement, "SentEmails", sendingStatus.sentEmails);
					XMLUtils.appendNewElement(doc, requestPaymentElement, "SentSMSs", sendingStatus.sentSMSs);
					XMLUtils.appendNewElement(doc, requestPaymentElement, "NewPaymentRequests", newPaymentRequests);
					XMLUtils.appendNewElement(doc, requestPaymentElement, "QueueSlotCount", CollectionUtils.getSize(queueSlots));
				}
			}

			if (!validationErrors.isEmpty()) {
				XMLUtils.append(doc, requestPaymentElement, "ValidationErrors", validationErrors);
			}

			requestPaymentElement.appendChild(queue.toXML(doc));

			requestPaymentElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			
			String[] queueSlotIDs = req.getParameterValues("queueSlotID");
			
			
			if(queueSlotIDs != null) {
				Element queueSlotIDElement = doc.createElement("SelectedQueueSlots");
				for (String queueSlotID : queueSlotIDs) {
					XMLUtils.appendNewElement(doc, queueSlotIDElement, "queueSlotID", queueSlotID);
				}
				requestPaymentElement.appendChild(queueSlotIDElement);
			}

			log.info("User " + user + " requesting request payment form");

			return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse requestPaymentForQueueSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		QueueSlot queueSlot;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (queueSlot = getQueueSlot(Integer.valueOf(uriParser.get(2)), QueueSlot.EVENTS_RELATION, QueueSlot.QUEUE_RELATION, Queue.MANAGER_GROUPS_RELATION, Queue.MANAGER_USERS_RELATION)) != null) {

			if (!hasQueueManagerAccess(queueSlot.getQueue(), user) && !hasAdminAccess(user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			List<ValidationError> validationErrors = new ArrayList<>();

			setPaymentPending(new Timestamp(System.currentTimeMillis()), req.getParameter("comment"), queueSlot, user, validationErrors, null, false);

			if (!CollectionUtils.isEmpty(validationErrors)) {

				return queueSlotCRUD.show(req, res, user, uriParser, validationErrors);

			} else {

				redirectToMethod(req, res, "/show/" + queueSlot.getQueueSlotID());
				return null;
			}
		}

		throw new URINotFoundException(uriParser);
	}

	private void setPaymentPending(Timestamp now, String comment, QueueSlot queueSlot, User user, List<ValidationError> validationErrors, SendingStatus sendingStatus, boolean overrideUserSettingSMS) throws SQLException {

		if (queueSlot.getPendingPaymentRequestDate() == null) {

			if (queueSlot.getQueue() == null) {

				throw new NullPointerException("Queue needs to be set on queue slot");
			}

			queueSlot.setPendingPaymentRequestDate(now);
			queueSlot.addEvent(new QueueSlotEvent(now, user, QueueSlotEventType.PAYMENT_REQUEST, getEventPaymentRequestAddedMessage() + (StringUtils.isEmpty(comment) ? "" : eventSeeExternalMessage), null));

			HighLevelQuery<QueueSlot> updateQuery = new HighLevelQuery<>(QueueSlot.EVENTS_RELATION);
			queueSlotDAO.update(queueSlot, updateQuery, 20);

			if (!StringUtils.isEmpty(comment)) {
				addQueueSlotExternalMessage(queueSlot, comment, true, user, ExternalMessageType.PAYMENT_REQUEST, null);
			}

			Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());

			if (contact.isContactByEmail() && !StringUtils.isEmpty(contact.getEmail())) {

				log.info("User " + user + " sending payment request email for queue slot " + queueSlot);

				if (!EmailUtils.isValidEmailAddress(contact.getEmail())) {

					validationErrors.add(new InvalidRecipientEmailValidationError(queueSlot, contact.getEmail()));

				} else {

					if (queueAdminModule.sendContactEmail(queueSlot, contact, queueAdminModule.getPaymentRequestEmailSubject(), queueAdminModule.getPaymentRequestEmailMessage(), null)) {

						if (sendingStatus != null) {
							sendingStatus.sentEmails++;
						}

					} else {

						validationErrors.add(new ErrorSendingToRecipientViaEmailValidationError(queueSlot, contact.getEmail()));
					}
				}
			}

			if ((contact.isContactBySMS() || overrideUserSettingSMS) && !StringUtils.isEmpty(contact.getMobilePhone())) {

				log.info("User " + user + " sending payment request SMS for queue slot " + queueSlot+" "+(overrideUserSettingSMS ? " overriding user SMS settings" : ""));

				if (!SwedishPhoneNumberValidator.getValidator().validateFormat(contact.getMobilePhone())) {

					validationErrors.add(new InvalidRecipientPhoneNumberValidationError(queueSlot, contact.getMobilePhone()));

				} else {

					if (queueAdminModule.sendContactSMS(queueSlot, contact, queueAdminModule.getPaymentRequestSMS(), overrideUserSettingSMS)) {

						if (sendingStatus != null) {
							sendingStatus.sentSMSs++;
						}

					} else {

						validationErrors.add(new ErrorSendingToRecipientViaSMSValidationError(queueSlot, contact.getMobilePhone()));
					}
				}
			}
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse revokePaymentRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer queueSlotID;

		if (uriParser.size() == 3 && (queueSlotID = NumberUtils.toInt(uriParser.get(2))) != null) {

			QueueSlot queueSlot = getQueueSlot(queueSlotID, QueueSlot.QUEUE_RELATION, QueueSlot.EVENTS_RELATION);

			if (queueSlot != null) {

				if (queueSlot.getPendingPaymentRequestDate() != null) {

					String comment = req.getParameter("comment");

					log.info("User " + user + " revoking payment request for queue slot " + queueSlot + " with reason: " + StringUtils.toLogFormat(comment, 50));

					queueSlot.setPendingPaymentRequestDate(null);
					queueSlot.addEvent(new QueueSlotEvent(new Timestamp(System.currentTimeMillis()), user, QueueSlotEventType.PAYMENT_REQUEST, getEventPaymentRequestRemovedMessage() + (StringUtils.isEmpty(comment) ? "" : eventSeeExternalMessage), null));

					HighLevelQuery<QueueSlot> updateQuery = new HighLevelQuery<>(QueueSlot.EVENTS_RELATION);
					queueSlotDAO.update(queueSlot, updateQuery);

					if (!StringUtils.isEmpty(comment)) {
						addQueueSlotExternalMessage(queueSlot, comment, true, user, ExternalMessageType.PAYMENT_REQUEST_REMOVAL, null);
					}
				}

				redirectToMethod(req, res, "/show/" + queueSlotID);
				return null;
			}
		}

		throw new URINotFoundException(uriParser);
	}

	public Integer getDefaultPageSize() {

		return defaultPageSize;
	}

	@WebPublic(alias = "slotdata")
	public ForegroundModuleResponse getQueueSlotData(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());
		PrintWriter writer = res.getWriter();

		Integer queueID;
		Queue queue;

		if (uriParser.size() == 3 && (queueID = NumberUtils.toInt(uriParser.get(2))) != null && (queue = queueDAOWrapper.get(queueID)) != null && hasQueueManagerAccess(queue, user)) {

			log.info("Sending queue slots in queue " + queue + " to user " + user);

			writer.append("{\"rows\":[");

			boolean archived = req.getParameter("archived") != null;

			List<QueueSlot> queueSlots;

			//TODO use QueryResultsStreamer
			if (!archived) {

				queueSlots = getQueueSlots(queue, LIST_QUEUE_SLOT_RELATIONS);

			} else {

				queueSlots = getArchivedQueueSlots(queue, LIST_QUEUE_SLOT_RELATIONS);
			}

			if (!CollectionUtils.isEmpty(queueSlots)) {

				QueueUtils.setQueueSlotUsers(queue, queueSlots, getUserHandler());

				JsonArray row = new JsonArray(9);

				for (Iterator<QueueSlot> instanceIterator = queueSlots.iterator(); instanceIterator.hasNext();) {

					QueueSlot queueSlot = instanceIterator.next();

					if (!archived) {

						row.addNode(""); //Select row checkbox
					}

					row.addNode(queueSlot.getQueueSlotID());

					if (!archived) {

						row.addNode(queueSlot.getCalculatedPosition());
						row.addNode(queueSlot.getPendingPaymentRequestDate() != null ? 1 : 0);
					}

					Contact contact = QueueAdminModule.getContact(queueSlot, getUserHandler());
					row.addNode(contact.getFirstname() + " " + contact.getLastname());
					row.addNode(queueSlot.getAttributeHandler().getString(Constants.FLOW_INSTANCE_DESCRIPTION_ATTRIBUTE));

					row.addNode(queueSlot.getAdded() != null ? DateUtils.DATE_TIME_FORMATTER.format(queueSlot.getAdded()) : "");
					row.addNode(queueSlot.getArchivedDate() != null ? DateUtils.DATE_TIME_FORMATTER.format(queueSlot.getArchivedDate()) : "");
					if(queueSlot.getQueue().isUseWaitingPeriod() && archived) {
						row.addNode(queueSlot.getWaitingPeriodTime() != null ? DateUtils.DATE_FORMATTER.format(queueSlot.getWaitingPeriodTime()) : "");
					}
					row.addNode(queueSlot.getLastPaymentDate() != null ? DateUtils.DATE_TIME_FORMATTER.format(queueSlot.getLastPaymentDate()) : "");
					row.addNode(QueueUtils.isDigitalMember(queueSlot, getUserHandler()) ? 1 : 0);

					StringBuilder searchExtraBuilder = new StringBuilder();

					String citizenIdentifier = contact.getCitizenIdentifier();

					if (citizenIdentifier != null) {

						searchExtraBuilder.append(citizenIdentifier);
						searchExtraBuilder.append(" ");
					}

					if (queueSlot.getManualContact() != null) {

						searchExtraBuilder.append(queueSlot.getPrimaryContact().getFirstname() + " " + queueSlot.getPrimaryContact().getLastname());
						searchExtraBuilder.append(" ");
					}

					if (queueSlot.getInternalMessages() != null) {

						for (InternalMessage message : queueSlot.getInternalMessages()) {

							searchExtraBuilder.append(message.getMessage());
							searchExtraBuilder.append(" ");
						}
					}

					if (queueSlot.getExternalMessages() != null) {

						for (ExternalMessage message : queueSlot.getExternalMessages()) {

							searchExtraBuilder.append(message.getMessage());
							searchExtraBuilder.append(" ");
						}
					}

					row.addNode(searchExtraBuilder.toString());

					writer.append(row.toJson());

					row.clearNodes();

					if (instanceIterator.hasNext()) {

						writer.append(",");
					}
				}
			}

			writer.append("]}");

			writer.flush();
			return null;
		}

		writer.append("{\"error\": true");
		writer.flush();
		return null;
	}
}
