<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/plugins/jquery.qloader.js
		/featherlight/js/featherlight.min.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>

	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/flowengine.js?v=1
		/js/flowengine.helpdialog.js
		/js/flowengine.tablesorter.js
		/js/userflowinstancemodule.js
		/js/jquery.qloader-init.js
		/js/userqueueslot.js
		/js/js.cookie-2.0.4.min.js
		/js/messages.js?v=3
	</xsl:variable>

	<xsl:variable name="links">
		/css/flowengine.css
		/css/overrides.css
		/css/messages.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<div id="UserQueueSlotModule" class="contentitem">
			<xsl:apply-templates select="ListUserQueueSlots" />
			<xsl:apply-templates select="ShowQueueSlotOverview" />
			<xsl:apply-templates select="Payment" />
		</div>
		
	</xsl:template>
		
	<xsl:template match="ListUserQueueSlots">
	
		<xsl:apply-templates select="validationError"/>
		
		<section class="my-errands">
			
			<div class="errands-wrapper">
				<div class="heading-wrapper">
					<h2><xsl:value-of select="$i18n.UserQueueSlots" /></h2>
					<xsl:call-template name="createHelpDialog">
						<xsl:with-param name="id" select="'queueslot-list'" />
						<xsl:with-param name="text" select="$i18n.UserQueueSlotsHelp" />
					</xsl:call-template>
				</div>
				<table class="oep-table errand-table">
					<thead class="sortable">
						<tr>
							<th class="icon no-sort"></th>
							<th class="service"><span data-icon-after="_"><xsl:value-of select="$i18n.Queue.Name" /></span></th>
							<th class="errando"><span data-icon-after="_"><xsl:value-of select="$i18n.Position" /></span></th>
							<th class="status"><span data-icon-after="_"><xsl:value-of select="$i18n.QueueSlot.Added" /></span></th>
							<th class="date default-sort"><span data-icon-after="_"><xsl:value-of select="$i18n.LastEvent" /></span></th>
							<th class="link no-sort"></th>
						</tr>
					</thead>
					<tbody>
						<xsl:choose>
							<xsl:when test="QueueSlot[not(archived = 'true')]">
								<xsl:apply-templates select="QueueSlot[not(archived = 'true')]" mode="list" />
							</xsl:when>
							<xsl:otherwise>
								<tr><td class="icon" /><td colspan="6"><xsl:value-of select="$i18n.NoQueueSlots" /></td></tr>
							</xsl:otherwise>
						</xsl:choose>
					</tbody>
				</table>
			</div>
			
			<xsl:if test="QueueSlot[archived = 'true']">
				<div class="errands-wrapper">
					<div class="heading-wrapper">
						<h2><xsl:value-of select="$i18n.UserArchivedQueueSlots" /></h2>
						<xsl:call-template name="createHelpDialog">
							<xsl:with-param name="id" select="'archivedqueueslot-list'" />
							<xsl:with-param name="text" select="$i18n.UserArchivedQueueSlotsHelp" />
						</xsl:call-template>
					</div>
					<table class="oep-table errand-table">
						<thead class="sortable">
							<tr>
								<th class="icon no-sort"></th>
								<th class="service"><span data-icon-after="_"><xsl:value-of select="$i18n.Queue.Name" /></span></th>
								<th class="status"><span data-icon-after="_"><xsl:value-of select="$i18n.QueueSlot.Added" /></span></th>
								<th class="date default-sort"><span data-icon-after="_"><xsl:value-of select="$i18n.LastEvent" /></span></th>
								<th class="link no-sort"></th>
							</tr>
						</thead>
						<tbody>
							<xsl:apply-templates select="QueueSlot[archived = 'true']" mode="archived-list" />
						</tbody>
					</table>
				</div>
			</xsl:if>
					
		</section>
		
	</xsl:template>
	
	<xsl:template match="QueueSlot" mode="list">
		
		<tr>
			<td class="icon"/>
			<td data-title="{$i18n.Queue.Name}" class="service"><xsl:value-of select="Queue/name" /></td>
			<td data-title="{$i18n.Position}" class="errando">
				<xsl:choose>
					<xsl:when test="Queue/showQueuePosition = 'true'">
						<xsl:value-of select="position" />
					</xsl:when>
					<xsl:otherwise>
						
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td data-title="{$i18n.Payed}" class="status">
				<xsl:choose>
					<xsl:when test="Queue/hideAddedDateFromUser = 'true'">
						<xsl:text>-</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="added" />
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td data-title="{$i18n.LastEvent}" class="date">
				<xsl:value-of select="Events/QueueSlotEvent[position() = last()]/added" />
			</td>
			<td class="link">
				<a class="btn btn-green" href="{/Document/ModuleURL}/overview/{queueSlotID}"><xsl:value-of select="$i18n.ShowQueueSlot" /></a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="QueueSlot" mode="archived-list">
		
		<tr>
			<td class="icon"/>
			<td data-title="{$i18n.Queue.Name}" class="service"><xsl:value-of select="Queue/name" /></td>
			<td data-title="{$i18n.Payed}" class="status">
				<xsl:choose>
					<xsl:when test="Queue/hideAddedDateFromUser = 'true'">
						<xsl:text>-</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="added" />
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td data-title="{$i18n.LastEvent}" class="date">
				<xsl:value-of select="Events/QueueSlotEvent[position() = last()]/added" />
			</td>
			<td class="link">
				<a class="btn btn-green" href="{/Document/ModuleURL}/overview/{queueSlotID}"><xsl:value-of select="$i18n.ShowQueueSlot" /></a>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="ShowQueueSlotOverview">
	
		<xsl:if test="RequestedLeave">
			<section class="modal success bigmarginbottom">
				<span data-icon-before="c"><xsl:value-of select="$i18n.RequestLeaveConfirmation" /></span>
				<i class="icon close">x</i>
		</section>
		</xsl:if>
	
		<section class="child">
			<div class="section-inside step">
  				<div class="heading-wrapper">
  					<div class="inner">
	  					<!--
	  					<figure>
		  					<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{Flow/flowID}" alt="" />
		  				</figure>
		  				-->
		  				<div class="heading">
	  						<h1 class="xl">
	  							<xsl:value-of select="QueueSlot/Queue/name" />
	  						</h1>
						</div>
					</div>
  				</div>
  			</div>
  			<div class="section-inside header-full no-pad-top">
  				<div class="description">
  					
  					<xsl:if test="QueueSlot/archived = 'true' or QueueSlot/Queue/showQueuePosition = 'true'">
	  					<p>
	  						<strong>
	  							<xsl:value-of select="$i18n.Position" />
	  							<xsl:text>:&#160;</xsl:text>
	  						</strong>
	  						
	  						<xsl:choose>
									<xsl:when test="QueueSlot/archived = 'true'">
									
										<xsl:value-of select="$i18n.Archived" />
										
									</xsl:when>
									<xsl:otherwise>
									
										<xsl:value-of select="QueueSlot/position" />
										
									</xsl:otherwise>
								</xsl:choose>
	  					</p>
  					</xsl:if>
  					
					<xsl:if test="QueueSlot/Queue/hideAddedDateFromUser = 'false'">
	  					<p>	
							<strong>
								<xsl:value-of select="$i18n.QueueSlot.Added" />
								<xsl:text>:&#160;</xsl:text>
							</strong>
							<xsl:value-of select="QueueSlot/added" />
						</p>
					</xsl:if>
  					
					<p>
						<strong>
							<xsl:value-of select="$i18n.Payed" />
							<xsl:text>:&#160;</xsl:text>
						</strong>
						<xsl:value-of select="QueueSlot/lastPaymentDate" />
					</p>

  				</div>
  			</div>

			<div id="tabs">
 				<ul class="tabs">
 					<li class="active">
 						<a data-icon-before="o" href="#history"><xsl:value-of select="$i18n.QueueSlotEvents" /></a>
 					</li>
 					
 					<li data-tabid="#messages">
 						<a data-icon-before="m" href="#messages">
 							<xsl:value-of select="$i18n.ExternalMessages" />
 							<!-- TODO count how many unread messages since last login -->
 							<xsl:if test="false()">
 								<span class="count">0</span>
 							</xsl:if>
 						</a>
 					</li>
 				</ul>
 				
 				<div id="messages">
  					
  					<div id="new-message" class="tabs-content">
  						
  						<div class="heading-wrapper">
  							<h2><xsl:value-of select="$i18n.NewMessage" /></h2>
  							<a href="#" class="btn btn-light btn-right close_message"><xsl:value-of select="$i18n.Close" /><i data-icon-after="x"></i></a>
  						</div>
  						
  						<form action="{/Document/requestinfo/uri}#messages" method="post" enctype="multipart/form-data">
  						
  							<div id="message-quote" style="display: none;">
  								<a href="#" class="btn btn-light btn-right stop_quote">
  									<xsl:value-of select="$i18n.Message.StopQuote" />
  									<i data-icon-after="x"/>
  								</a>
  								
  								<label><xsl:value-of select="$i18n.Message.Quote" /></label>
  								<input type="hidden" name="quotedMessageID" disabled="true"/>
  								<div class="quote clearboth bigmarginbottom"/>
  							</div>
  						
	  						<label class="required" for="message"><xsl:value-of select="$i18n.Message" /></label>
	  						<xsl:apply-templates select="validationError[fieldName = 'externalmessage']" />
	  						<textarea id="message" name="externalmessage" class="full"></textarea>
	  						<div class="heading-wrapper">
	  							<label class="required"><xsl:value-of select="$i18n.AttachFiles" /></label>
	  						</div>
	  						
	  						<script>
								imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
								deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
							</script>				
							
							<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />
							
							<div class="full">
								
								<div class="upload clearboth">
									<span class="btn btn-upload btn-blue">
										<xsl:value-of select="$i18n.ChooseFiles" />
										<input id="external-message" type="file" name="attachments" multiple="multiple" size="55" class="qloader externalmessages bigmarginbottom" />
									</span>
									<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxExternalMessageFileSize" /></span>
								</div>
								
								<ul id="external-message-qloader-filelist" class="files" />
								
							</div>
							
	  						<input type="submit" value="{$i18n.SubmitMessage}" name="addmessage" class="btn btn-green btn-inline" />
	  						<a href="#" class="btn btn-light btn-inline close_message"><xsl:value-of select="$i18n.Cancel" /></a>
	  						
  						</form>
  						
  					</div>
  					
  					<div class="tabs-content">
	  					
	  					<div class="heading-wrapper">
	  						<h2><xsl:value-of select="$i18n.ExternalMessages" /></h2>
	  						<a href="#" class="btn btn-blue btn-right open_message"><i data-icon-before="+"></i><xsl:value-of select="$i18n.NewMessage" /></a>
	  					</div>
	  					
	  					<xsl:choose>
	  						<xsl:when test="QueueSlot/externalMessages/ExternalMessage">
								
								<xsl:call-template name="sortMessagesButton">
									<xsl:with-param name="defaultOrder" select="DefaultMessageOrder" />
								</xsl:call-template>
																
	  							<ul class="messages">
	  								<xsl:apply-templates select="QueueSlot/externalMessages/ExternalMessage" mode="user" />
	  							</ul>
	  						</xsl:when>
	  						<xsl:otherwise>
	  							<xsl:value-of select="$i18n.NoExternalMessages" />
	  						</xsl:otherwise>	  					
	  					</xsl:choose>
	  					
	  				</div>
	  				
  				</div>
 				
 				<div id="history" class="tabs-content nopadding" >
 					
 					<div class="errands-wrapper">
  					
	  					<div class="heading-wrapper">
		  					<h2><xsl:value-of select="$i18n.QueueSlotEvents" /></h2>
	  					</div>
	  					
	  					<table class="oep-table">
	  						<thead class="errand">
	  							<tr>
	  								<th class="service active"><span><xsl:value-of select="$i18n.Action" /></span></th>
		  							<th class="details"><span><xsl:value-of select="$i18n.Details" /></span></th>
	  								<th class="date"><span><xsl:value-of select="$i18n.Date" /></span></th>
	  								<th class="person"><span><xsl:value-of select="$i18n.Person" /></span></th>
	  							</tr>
	  						</thead>
	  						<tbody>
	  							<xsl:choose>
	  								<xsl:when test="QueueSlot/Events/QueueSlotEvent">
	  									<xsl:apply-templates select="QueueSlot/Events/QueueSlotEvent" />
	  								</xsl:when>
	  								<xsl:otherwise>
	  									<tr><td class="icon" /><td colspan="5"><xsl:value-of select="$i18n.NoEvents" /></td></tr>
	  								</xsl:otherwise>
	  							</xsl:choose>
	  						</tbody>
	  					</table>
  					
  					</div>
  				
  				</div>
  			
 			</div>
			
			<div class="navigator-buttons centered">
			
				<xsl:if test="not(QueueSlot/archived = 'true')">
				
					<form action="{/Document/ModuleURL}/leavequeue/{QueueSlot/queueSlotID}" method="post">
				
						<input type="hidden" name="comment" />
				
						<div style="display: none;">
						
							<div class="comment-modal contentitem">
								<div class="modal-content">
								
									<div class="modal-header bigmarginbottom">
										<h1>
											<xsl:choose>
												<xsl:when test="QueueSlot/Queue/leavingRequiresManagerConfirmation = 'true'">
													<xsl:value-of select="$i18n.RequestLeaveQueue" />
												</xsl:when>
												<xsl:otherwise>
													<xsl:value-of select="$i18n.LeaveQueue" />
												</xsl:otherwise>
											</xsl:choose>
										</h1>
									</div>
									
									<div class="modal-body">
									
										<div class="floatleft full bigmarginbottom">
										
											<p class="tiny floatleft full">
										  	<xsl:choose>
													<xsl:when test="QueueSlot/Queue/leavingRequiresManagerConfirmation = 'true'">
														<xsl:value-of select="$i18n.RequestLeaveQueueDescription" />
													</xsl:when>
													<xsl:otherwise>
														<xsl:value-of select="$i18n.LeaveQueueDescription" />
													</xsl:otherwise>
												</xsl:choose>
										  </p>
										  
											<label for="save-email" class="floatleft clearboth">
												<xsl:value-of select="$i18n.LeaveReason" />
											</label>
											
											<div class="floatleft full">
												<xsl:call-template name="createTextArea">
													<xsl:with-param name="name" select="'message'"/>
													<xsl:with-param name="maxlength" select="'1024'"/>
												</xsl:call-template>
										  </div>
										</div>
										
										<input class="btn btn-red close bigmargintop floatright" type="button" >
											<xsl:attribute name="value">
												<xsl:choose>
													<xsl:when test="QueueSlot/Queue/leavingRequiresManagerConfirmation = 'true'">
														<xsl:value-of select="$i18n.RequestLeaveQueue" />
													</xsl:when>
													<xsl:otherwise>
														<xsl:value-of select="$i18n.LeaveQueue" />
													</xsl:otherwise>
												</xsl:choose>
											</xsl:attribute>
										</input>
										
									</div>
									
								</div>
							</div>
						
						</div>
						
						<xsl:if test="QueueSlot/pendingPaymentRequestDate">
							
							<a href="{/Document/ModuleURL}/pay/{QueueSlot/queueSlotID}" class="btn btn-green xl prev bigmarginright">
								<xsl:value-of select="$i18n.PayForQueue" />
							</a>
							
						</xsl:if>
						
						<xsl:choose>
							<xsl:when test="QueueSlot/Queue/leavingRequiresManagerConfirmation = 'true'">
							
								<a href="#" class="btn btn-red xl prev" onclick="commentModal(event, this)">
									<i data-icon-before="t" class="xl"/>
									<xsl:value-of select="$i18n.RequestLeaveQueue" />
								</a>
							
							</xsl:when>
							<xsl:otherwise>
							
								<a href="#" class="btn btn-red xl prev" onclick="commentModal(event, this)">
									<i data-icon-before="t" class="xl"/>
									<xsl:value-of select="$i18n.LeaveQueue" />
								</a>
							
							</xsl:otherwise>
						</xsl:choose>
						
					</form>
				
				</xsl:if>

 			</div>
 			
		</section>
		
	</xsl:template>
	
	<xsl:template name="createHelpDialog">
		
		<xsl:param name="id" />
		<xsl:param name="text" />
		<xsl:param name="class" select="''" />
		
		<div class="help {$class}">
			<a class="open-help" href="#" data-icon-after="?" data-help-box="helpdialog_{$id}"><span><xsl:value-of select="$i18n.Help" /></span></a>
			<div class="help-box" data-help-box="helpdialog_{$id}">
				<div>
		  			<div> 
		  				<a class="close" href="#" data-icon-after="x"></a>
		  				<xsl:copy-of select="$text" />
		  			</div> 
				</div>
			</div>
		</div>
		
		<div class="help-backdrop" data-help-box="helpdialog_{$id}" />
		
	</xsl:template>
	
	<xsl:template match="ExternalMessage" mode="user">

		<xsl:call-template name="createUserMessage">
			<xsl:with-param name="message" select="." />
			<xsl:with-param name="hideManagerDetails" select="../../Queue/hideManagerDetails = 'true'" />
			<xsl:with-param name="repliesEnabled" select="true()" />
		</xsl:call-template>
		
	</xsl:template>
	
	<xsl:template match="Payment">
	
		<section class="service">

			<form id="paymentForm" method="POST" action="{/Document/ModuleURL}/processPayment/{QueueSlot/queueSlotID}">

				<div class="section-full push">

					<h2 class="h1 hide-tablet"><xsl:value-of select="Invoice/title"/></h2>

					<xsl:value-of select="ViewFragment/HTML" disable-output-escaping="yes"/>

					<div class="navigator-buttons clearboth">
						
						<a href="#" class="btn btn-green xl next" onclick="$(this).closest('form').trigger('submit'); return false;"><span><xsl:value-of select="$i18n.Pay" /></span></a>
					
					</div>

				</div>

			</form>

		</section>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='FileSizeLimitExceeded']">
	
		<div class="info-box error">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part1"/>
					<xsl:value-of select="filename"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part2"/>
					<xsl:value-of select="size"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part3"/>
					<xsl:value-of select="maxFileSize"/>
					<xsl:value-of select="$i18n.FileSizeLimitExceeded.part4"/>
				</strong>
			</span>
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
	
		<div class="info-box error">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.ValidationError.UnableToParseRequest" />
				</strong>
			</span>
		</div>
		
	</xsl:template>						
	
</xsl:stylesheet>