package com.nordicpeak.flowengine.queue.query;

import java.sql.SQLException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.BooleanPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queue.QueueAdminModule;
import com.nordicpeak.flowengine.queue.beans.Queue;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class QueueQueryProviderModule extends BaseQueryProviderModule<QueueQueryInstance> implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	protected String alternativeName = "This variable should be set by your stylesheet";

	private AnnotatedDAO<QueueQuery> queryDAO;
	private AnnotatedDAO<QueueQueryInstance> queryInstanceDAO;

	private QueueQueryCRUD queryCRUD;

	private QueryParameterFactory<QueueQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<QueueQueryInstance, Integer> queryInstanceIDParamFactory;

	private QueueAlternative alternative;

	@InstanceManagerDependency(required = true)
	protected QueueAdminModule queueAdminModule;

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);

		alternative = new QueueAlternative(alternativeName);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, QueueAdminModule.class.getName(), new XMLDBScriptProvider(QueueAdminModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(QueueQuery.class);
		queryInstanceDAO = daoFactory.getDAO(QueueQueryInstance.class);

		queryCRUD = new QueueQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(QueueQuery.class), "QueueQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		QueueQuery query = new QueueQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setAlternative(alternative);

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		QueueQuery query = new QueueQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		QueueQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setAlternative(alternative);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		QueueQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setAlternative(alternative);

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		QueueQueryInstance queryInstance = null;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new QueueQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		queryInstance.getQuery().setAlternative(alternative);

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance.defaultQueryValues();
		}

		return queryInstance;
	}

	private QueueQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<QueueQuery> query = new HighLevelQuery<>(QueueQuery.QUEUE_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private QueueQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<QueueQuery> query = new HighLevelQuery<>(QueueQuery.QUEUE_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private QueueQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<QueueQueryInstance> query = new HighLevelQuery<>();

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(QueueQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	private boolean isActiveMemberOfQueue(QueueQueryInstance queryInstance, String citizenIdentifier) {

		try {
			return queueAdminModule.isActiveMemberOfQueue(queryInstance.getQuery().getQueue(), citizenIdentifier);

		} catch (SQLException e) {

			throw new RuntimeException(e);
		}
	}
	
	private LocalDate getWaitingPeriod(QueueQueryInstance queryInstance, String citizenIdentifier) {
		
		try {
			return queueAdminModule.getWaitingPeriodDate(queryInstance.getQuery().getQueue(), citizenIdentifier);
			
		} catch (SQLException e) {

			throw new RuntimeException(e);
		}
	}

	@Override
	public QueryResponse getFormHTML(QueueQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws TransformerConfigurationException, TransformerException {

		Document doc = createDocument(req, poster);

		Element showQueryFormElement = doc.createElement("ShowQueryForm");
		doc.getDocumentElement().appendChild(showQueryFormElement);

		if (validationErrors == null) {

			if (queryInstance.getQuery().getQueue() == null) {

				validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("NotConfigured-NoQueue"));
			}

			if (poster == null) {

				validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("RequiresLogin"));

			} else {

				String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);
				
				LocalDate waitingPeriod = getWaitingPeriod(queryInstance, citizenIdentifier);

				if (citizenIdentifier == null) {

					validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("NoCitizenIdentifierFound"));

				} else if(queryInstance.getQuery().getQueue() != null && isInWaitingPeriod(waitingPeriod)) {
					
					validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("PersonIsInWaitingPeriod", waitingPeriod.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))));
				}
				else if (queryInstance.getQuery().getQueue() != null && !queryInstance.getQuery().getQueue().isAllowDuplicateQueueSlots() && isActiveMemberOfQueue(queryInstance, citizenIdentifier)) {

					if (queryInstance.getQuery().isAllowNextStepIfAlreadyMember()) {

						XMLUtils.appendNewElement(doc, showQueryFormElement, "AlreadyMemberWarning");

					} else {

						validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("AlreadyMember"));
					}
				}
			}
		}

		if (queryInstance.getSubmitValidationError() != null) {

			validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, queryInstance.getSubmitValidationError());
			queryInstance.setSubmitValidationError(null);
		}

		XMLUtils.appendNewCDATAElement(doc, showQueryFormElement, "queryRequestURL", queryRequestURL);

		appendQueryInstance(queryInstance, doc, showQueryFormElement, attributeHandler);

		if (!CollectionUtils.isEmpty(validationErrors) && req.getMethod().equalsIgnoreCase("POST")) {

			showQueryFormElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		if (enableAjaxPosting) {

			showQueryFormElement.appendChild(doc.createElement("EnableAjaxPosting"));
		}

		XMLUtils.append(doc, showQueryFormElement, "ValidationErrors", validationErrors);

		showQueryFormElement.appendChild(requestMetadata.toXML(doc));

		return createQueryResponse(doc, queryInstance.getQueryInstanceDescriptor().getQueryDescriptor());
	}

	private boolean isInWaitingPeriod(LocalDate waitingDate) {

		LocalDate currentDate = LocalDate.now();
		
		return waitingDate != null && (waitingDate.equals(currentDate) || waitingDate.isAfter(currentDate));
	}

	public boolean isValidQueueQueryInstance(QueueQueryInstance queryInstance, User poster) throws SQLException {

		if (queryInstance.getQuery().getQueue() == null) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			queryInstance.setSubmitValidationError(new ValidationError("NotConfigured-NoQueue"));
			return false;
		}

		// Reload queue
		queryInstance.getQuery().setQueue(getQueue(queryInstance.getQuery().getQueue().getQueueID()));
		Queue queue = queryInstance.getQuery().getQueue();

		if (!queue.isEnabled()) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			queryInstance.setSubmitValidationError(new ValidationError("QueueDisabled"));
			return false;
		}

		return true;
	}

	@Override
	public void populate(QueueQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if (queryInstance.getQuery().getQueue() == null) {

			throw new ValidationException(new ValidationError("NotConfigured-NoQueue"));
		}

		if (!queryInstance.getQuery().getQueue().isEnabled()) {

			throw new ValidationException(new ValidationError("QueueDisabled"));
		}

		if (poster == null) {

			throw new ValidationException(new ValidationError("RequiresLogin"));

		} else {

			String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);
			
			LocalDate waitingPeriod = getWaitingPeriod(queryInstance, citizenIdentifier);

			if (citizenIdentifier == null) {

				throw new ValidationException(new ValidationError("NoCitizenIdentifierFound"));

			}else if(queryInstance.getQuery().getQueue() != null && isInWaitingPeriod(waitingPeriod)) {
				
				throw new ValidationException(new ValidationError("PersonIsInWaitingPeriod", waitingPeriod.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))));
			} 
			else if (!queryInstance.getQuery().getQueue().isAllowDuplicateQueueSlots() && isActiveMemberOfQueue(queryInstance, citizenIdentifier) && !queryInstance.getQuery().isAllowNextStepIfAlreadyMember()) {

				throw new ValidationException(new ValidationError("AlreadyMember"));
			} 
			
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		Boolean accepted = ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_accept", req, false, BooleanPopulator.getPopulator(), validationErrors);

		if (accepted == null) {
			accepted = false;
		}

		String comment;

		if (queryInstance.getQuery().showCommentField()) {

			comment = ValidationUtils.validateParameter("q" + queryInstance.getQuery().getQueryID() + "_comment", req, false, 0, 65534, StringPopulator.getPopulator(), validationErrors);

		} else {

			comment = null;
		}

		//If partial population is allowed and the user has not selected any alternatives, skip validation
		if (allowPartialPopulation) {

			queryInstance.setAccepted(accepted);
			queryInstance.setComment(comment);
			queryInstance.getQueryInstanceDescriptor().setPopulated(accepted);

			return;
		}

		//Check if this query is required or if the user has selected any alternatives anyway
		if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED) {

			if (!accepted) {
				validationErrors.add(new ValidationError("RequiredQuery"));
			}

		} else if (!accepted && !StringUtils.isEmpty(comment)) {

			validationErrors.add(new ValidationError("MustAcceptToComment", queryInstance.getQuery().getAcceptText()));
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		queryInstance.setComment(comment);
		queryInstance.setAccepted(accepted);
		queryInstance.getQueryInstanceDescriptor().setPopulated(accepted);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		QueueQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		QueueQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		QueueQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, QueueQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<QueueQueryInstance> getQueryInstanceClass() {

		return QueueQueryInstance.class;
	}

	public List<Queue> getQueues() throws SQLException {

		return queueAdminModule.getQueues();
	}

	public Queue getQueue(Integer queueID) throws SQLException {

		return queueAdminModule.getQueue(queueID);
	}

}
