<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/queuequery.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
	
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="QueueQueryInstance/QueueQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="QueueQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<article>
				
				<div class="heading-wrapper">
				
					<xsl:if test="not(QueueQueryInstance/QueueQuery/hideTitle = 'true')">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="QueueQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="QueueQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="QueueQueryInstance/QueueQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="QueueQueryInstance/QueueQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="QueueQueryInstance/QueueQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:apply-templates select="QueueQueryInstance/QueueQuery/Queue" />
				
				<xsl:if test="QueueQueryInstance/accepted = 'true'">
				
					<p>
						<xsl:value-of select="QueueQueryInstance/QueueQuery/acceptText" />
					</p>
				
				</xsl:if>
				
				<xsl:if test="QueueQueryInstance/comment">
					<div>
						<strong class="floatleft"><xsl:value-of select="$i18n.Comment"/></strong>
						
						<span class="floatleft clearboth">
							<xsl:call-template name="replaceLineBreak">
								<xsl:with-param name="string" select="QueueQueryInstance/comment"/>
							</xsl:call-template>
						</span>
					</div>
				</xsl:if>
				
			</article>
		
		</div>	
	
	</xsl:template>
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', QueueQueryInstance/QueueQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
	
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="QueueQueryInstance/QueueQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="QueueQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			
			<a name="{$queryID}" />
	
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
							<xsl:apply-templates select="ValidationErrors/validationError"/>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<xsl:if test="not(QueueQueryInstance/QueueQuery/hideTitle = 'true')">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="QueueQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="QueueQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
					
					<xsl:if test="QueueQueryInstance/QueueQuery/helpText">		
						<xsl:apply-templates select="QueueQueryInstance/QueueQuery/helpText" />
					</xsl:if>

				</div>
				
				<xsl:if test="QueueQueryInstance/QueueQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="QueueQueryInstance/QueueQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<xsl:apply-templates select="QueueQueryInstance/QueueQuery/Queue"/>
				
				<xsl:variable name="AlreadyMember">
					<xsl:if test="ValidationErrors/validationError[messageKey = 'AlreadyMember'] or AlreadyMemberWarning">
						<xsl:text>true</xsl:text>
					</xsl:if>
				</xsl:variable>
				
				<div class="alternative" >
					<xsl:variable name="fieldName">
						<xsl:value-of select="'q'"/>
						<xsl:value-of select="QueueQueryInstance/QueueQuery/queryID"/>
						<xsl:value-of select="'_accept'"/>
					</xsl:variable>
				
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="$fieldName"/>
						<xsl:with-param name="name" select="$fieldName"/>
						<xsl:with-param name="class" select="'vertical-align-bottom'" />
						<xsl:with-param name="checked" select="QueueQueryInstance/accepted"/>
						<xsl:with-param name="disabled" select="$AlreadyMember"/>
					</xsl:call-template>
					
					<label for="{$fieldName}" class="checkbox">
					
						<xsl:choose>
							<xsl:when test="$AlreadyMember = 'true'">

								<xsl:attribute name="class">checkbox disabled</xsl:attribute>

								<xsl:choose>
									<xsl:when test="QueueQueryInstance/QueueQuery/alreadyMemberText">
										<xsl:value-of select="QueueQueryInstance/QueueQuery/alreadyMemberText" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="$i18n.Error.AlreadyMember"/>
									</xsl:otherwise>
								</xsl:choose>								
							
							</xsl:when>
							<xsl:otherwise>
							
								<xsl:value-of select="QueueQueryInstance/QueueQuery/acceptText" />
	
							</xsl:otherwise>
						</xsl:choose>
						
					</label>
				</div>
			
				<xsl:if test="QueueQueryInstance/QueueQuery/showCommentField = 'true'">
					<div>
						<xsl:variable name="fieldName">
							<xsl:value-of select="'q'"/>
							<xsl:value-of select="QueueQueryInstance/QueueQuery/queryID"/>
							<xsl:value-of select="'_comment'"/>
						</xsl:variable>
					
						<label for="{$fieldName}" ><xsl:value-of select="$i18n.Comment" /></label>
					
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="$fieldName"/>
							<xsl:with-param name="name" select="$fieldName"/>
							<xsl:with-param name="rows" select="'4'"/>
							<xsl:with-param name="value" select="QueueQueryInstance/comment" />
						</xsl:call-template>
					</div>
				</xsl:if>
				
			</article>
	
		</div>
		
		<script type="text/javascript">$(function(){initQueueQuery('<xsl:value-of select="QueueQueryInstance/QueueQuery/queryID" />');});</script>
		
	</xsl:template>
	
	<xsl:template match="Queue">

		<xsl:if test="price > 0">
			<p class="tiny">
				<xsl:choose>
					<xsl:when test="../paymentRequiredText">
						<xsl:value-of select="../paymentRequiredText"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.PaymentRequired.DefaultText"/>
					</xsl:otherwise>
				</xsl:choose>			
				<xsl:text> </xsl:text>
				<xsl:value-of select="price"/>
				<xsl:value-of select="$i18n.PaymentRequired.Suffix"/>
			</p>
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="invalidFormatMessage">
						<xsl:value-of select="invalidFormatMessage"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.Error.InvalidFormat"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.RequiredQuery"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiresLogin']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.RequiresLogin"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'AlreadyMember']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:choose>
					<xsl:when test="../../QueueQueryInstance/QueueQuery/alreadyMemberText">
						<xsl:value-of select="../../QueueQueryInstance/QueueQuery/alreadyMemberText" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.Error.AlreadyMember"/>
					</xsl:otherwise>
				</xsl:choose>	
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PersonIsInWaitingPeriod']">
	<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.PersonIsInWaitingPeriod" /> <xsl:value-of select="displayName"/>!
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoCitizenIdentifierFound']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NoCitizenIdentifierFound"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'MustAcceptToComment']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.MustAcceptToComment"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="displayName"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NotConfigured-NoQueue']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.NotConfigured.NoQueue"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'QueueDisabled']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.QueueDisabled"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<span>
				<strong data-icon-before="!">
			
					<xsl:choose>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.Error.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.Error.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.Error.UnknownValidationError" />
						</xsl:otherwise>
					</xsl:choose>
					
					<xsl:text>&#x20;</xsl:text>
					
					<xsl:choose>
						<xsl:when test="displayName">
							<xsl:value-of select="displayName"/>
						</xsl:when>
						<xsl:when test="fieldName = 'comment'">
							<xsl:value-of select="$i18n.Comment"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName"/>
						</xsl:otherwise>
					</xsl:choose>
				</strong>
			</span>
		</xsl:if>
				
		<xsl:if test="messageKey">
			<span>
				<strong data-icon-before="!">
					<xsl:value-of select="$i18n.Error.UnknownValidationError" />
				</strong>
			</span>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>