package com.nordicpeak.flowengine.randombookingnumber;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.random.RandomUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.booking.BookingSlotAdminModule;
import com.nordicpeak.flowengine.booking.CalendarAdminModule;
import com.nordicpeak.flowengine.booking.UserBookingModule;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.beans.Repeating;
import com.nordicpeak.flowengine.booking.enums.BookingEventType;
import com.nordicpeak.flowengine.booking.interfaces.BookingExtensionProvider;
import com.nordicpeak.flowengine.booking.interfaces.BookingSlotFormExtensionProvider;
import com.nordicpeak.flowengine.booking.queries.BookingQuery;
import com.nordicpeak.flowengine.booking.queries.BookingQueryInstance;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingInterval;
import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingIntervalMapping;
import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingNumber;
import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingNumberMapping;
import com.nordicpeak.flowengine.randombookingnumber.cruds.RandomBookingIntervalCRUD;
import com.nordicpeak.flowengine.randombookingnumber.daos.RandomBookingIntervalDAO;
import com.nordicpeak.flowengine.randombookingnumber.daos.RandomBookingIntervalMappingDAO;
import com.nordicpeak.flowengine.randombookingnumber.daos.RandomBookingNumberMappingDAO;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;

public class RandomBookingNumberGeneratorModule extends AnnotatedForegroundModule implements CRUDCallback<User>, ViewFragmentModule<ForegroundModuleDescriptor>, BookingSlotFormExtensionProvider, BookingExtensionProvider {

	private static final RelationQuery FLOW_INSTANCE_RELATION_QUERY = new RelationQuery(FlowInstance.ATTRIBUTES_RELATION);

	@XSLVariable(prefix = "java.")
	protected String eventRandomlyAssignedNumber = "eventRandomlyAssignedNumber";

	@XSLVariable(prefix = "java.")
	protected String eventManagerSelectedNumber = "eventManagerChangedNumber";

	@XSLVariable(prefix = "java.")
	protected String eventManagerDeselectedNumber = "eventManagerDeselectedNumber";

	@XSLVariable(prefix = "java.")
	protected String eventSystemDeselectedNumber = "eventSystemDeselectedNumber";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmentedXML;

	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;

	private RandomBookingIntervalDAO intervalDAO;
	private RandomBookingIntervalMappingDAO intervalMappingDAO;

	private RandomBookingNumberMappingDAO numberMappingDAO;

	private RandomBookingIntervalCRUD intervalCRUD;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	@InstanceManagerDependency(required = true)
	private FlowAdminModule flowAdminModule;

	@InstanceManagerDependency(required = true)
	private CalendarAdminModule calendarAdminModule;

	private BookingSlotAdminModule bookingSlotAdminModule;

	private UserBookingModule userBookingModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RandomBookingNumberGeneratorModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		intervalDAO = new RandomBookingIntervalDAO(dataSource, RandomBookingInterval.class, daoFactory);
		intervalMappingDAO = new RandomBookingIntervalMappingDAO(dataSource, RandomBookingIntervalMapping.class, daoFactory);
		numberMappingDAO = new RandomBookingNumberMappingDAO(dataSource, RandomBookingNumberMapping.class, daoFactory);

		AnnotatedDAOWrapper<RandomBookingInterval, Integer> intervalCRUDDAO = daoFactory.getDAO(RandomBookingInterval.class).getWrapper(Integer.class);
		intervalCRUDDAO.addRelations(RandomBookingInterval.FIELD_NUMBERS, RandomBookingInterval.FIELD_MAPPINGS);
		intervalCRUDDAO.setUseRelationsOnGet(true);
		intervalCRUDDAO.setUseRelationsOnGetAll(true);
		intervalCRUDDAO.setUseRelationsOnAdd(true);
		intervalCRUDDAO.setUseRelationsOnUpdate(true);

		intervalCRUD = new RandomBookingIntervalCRUD(intervalCRUDDAO, this);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		viewFragmentTransformer.setDebugXML(debugFragmentedXML);
	}

	@Override
	public void unload() throws Exception {

		if (bookingSlotAdminModule != null) {

			bookingSlotAdminModule.removeBookingSlotExtensionProvider(this);
			bookingSlotAdminModule.removeBookingExtensionProvider(this);
		}

		if (userBookingModule != null) {

			userBookingModule.removeUserBookingExtensionProvider(this);
		}

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setBookingSlotAdminModule(BookingSlotAdminModule bookingSlotAdminModule) {

		if (bookingSlotAdminModule != null) {

			bookingSlotAdminModule.addBookingSlotExtensionProvider(this);
			bookingSlotAdminModule.addBookingExtensionProvider(this);

		} else if (this.bookingSlotAdminModule != null) {

			this.bookingSlotAdminModule.removeBookingSlotExtensionProvider(this);
			this.bookingSlotAdminModule.removeBookingExtensionProvider(this);
		}

		this.bookingSlotAdminModule = bookingSlotAdminModule;
	}

	@InstanceManagerDependency(required = true)
	public void setUserBookingModule(UserBookingModule userBookingModule) {

		if (userBookingModule != null) {

			userBookingModule.addUserBookingExtensionProvider(this);

		} else if (this.userBookingModule != null) {

			this.userBookingModule.removeUserBookingExtensionProvider(this);
		}

		this.userBookingModule = userBookingModule;
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");

		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.moduleDescriptor.toXML(doc));
		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));

		doc.appendChild(document);

		return doc;
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return listIntervals(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse listIntervals(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return intervalCRUD.list(req, res, user, uriParser, null);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse addInterval(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return intervalCRUD.add(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateInterval(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return intervalCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteInterval(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return intervalCRUD.delete(req, res, user, uriParser);
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 49)
	public void processEvent(SubmitEvent event, EventSource eventSource) {

		try {

			if (eventSource.isLocal()) {

				if (event.getEvent().getEventType() != EventType.SUBMITTED || event.getActionID() == null || !calendarAdminModule.getSupportedActionIDs().contains(event.getActionID())) {

					return;
				}

				FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();
				List<BookingQueryInstance> bookingQueryInstances = flowInstanceManager.getQueries(BookingQueryInstance.class);

				if (!CollectionUtils.isEmpty(bookingQueryInstances)) {

					FlowInstance flowInstance = (FlowInstance) flowInstanceManager.getFlowInstance();

					for (BookingQueryInstance queryInstance : bookingQueryInstances) {

						if (queryInstance.getQueryInstanceDescriptor().isPopulated() && queryInstance.isAccepted()) {

							Booking booking = calendarAdminModule.getBooking(flowInstance, queryInstance, Booking.BOOKING_SLOT_RELATION);

							if (booking != null) {

								RandomBookingNumber randomNumber = randomizeNumber(booking, event);

								if (randomNumber != null) {

									updateNumber(event, booking, randomNumber);

									setAttribute(randomNumber, flowInstance, queryInstance);
								}
							}
						}
					}
				}
			}

		} catch (Exception e) {

			log.error("Error processing submit event " + event, e);
		}
	}

	private RandomBookingNumber randomizeNumber(Booking booking, SubmitEvent event) throws SQLException {

		BookingSlot bookingSlot = booking.getBookingSlot();

		RandomBookingInterval interval = intervalMappingDAO.getIntervalByBookingSlotID(bookingSlot.getBookingSlotID(), RandomBookingInterval.FIELD_NUMBERS);

		if (interval == null || CollectionUtils.isEmpty(interval.getNumbers())) {

			return null;
		}

		List<RandomBookingNumber> availableNumbers = getAvailableNumbers(booking, interval.getNumbers());

		if (CollectionUtils.isEmpty(availableNumbers)) {

			log.warn("No available numbers for booking " + booking + " and event " + event);
			return null;
		}

		int randomValue = RandomUtils.getRandomInt(0, availableNumbers.size() - 1);

		return availableNumbers.get(randomValue);
	}

	private void updateNumber(SubmitEvent event, Booking booking, RandomBookingNumber randomNumber) throws SQLException {

		RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID());

		if (numberMapping != null) {

			numberMapping.setNumber(randomNumber);

		} else {

			numberMapping = new RandomBookingNumberMapping(booking.getBookingID(), randomNumber);
		}

		log.info("Randomly selected number " + randomNumber + " for booking " + booking + " and event " + event);

		numberMappingDAO.addOrUpdate(numberMapping, null);

		bookingSlotAdminModule.addBookingEvent(booking, BookingEventType.OTHER, eventRandomlyAssignedNumber + numberMapping.getIndex(), null, null, null);
	}

	private void setAttribute(RandomBookingNumber randomNumber, ImmutableFlowInstance flowInstance, BookingQueryInstance queryInstance) throws SQLException {

		BookingQuery query = queryInstance.getQuery();

		if (query.isSetAsAttribute()) {

			String attributeName = query.getAttributeName();
			MutableAttributeHandler attributeHandler = (MutableAttributeHandler) flowInstance.getAttributeHandler();

			attributeHandler.setAttribute(attributeName + ".randomBookingNumberID", randomNumber.getIndex());
			attributeHandler.setAttribute(attributeName + ".randomBookingNumberDescription", randomNumber.getDescription());

			FlowInstanceUtils.setDescriptions((FlowInstance) flowInstance);

			flowAdminModule.getDAOFactory().getFlowInstanceDAO().update((FlowInstance) flowInstance, FLOW_INSTANCE_RELATION_QUERY);
		}
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@Override
	public ViewFragment getRepeatingFormViewFragment(Repeating repeating, HttpServletRequest req, User user, URIParser uriParser, ValidationException validationException) throws Exception {

		if (repeating == null || repeating.getCalendar() == null || CollectionUtils.isEmpty(repeating.getBookingSlots())) {

			return null;
		}

		return getBookingSlotFormViewFragment(repeating.getBookingSlots().get(0), req, user, uriParser, validationException);
	}

	@Override
	public ViewFragment getBookingSlotFormViewFragment(BookingCalendar calendar, BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, ValidationException validationException) throws Exception {

		return getBookingSlotFormViewFragment(bookingSlot, req, user, uriParser, validationException);
	}

	private ViewFragment getBookingSlotFormViewFragment(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, ValidationException validationException) throws SQLException, TransformerException {

		List<RandomBookingInterval> intervals = intervalDAO.getAll();

		if (intervals == null) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element updateElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "SelectBookingSlotInterval");

		if (validationException != null) {

			XMLUtils.append(doc, updateElement, validationException);
		}

		Element requestParameters = RequestUtils.getRequestParameters(req, doc);

		if (requestParameters.hasChildNodes()) {

			updateElement.appendChild(requestParameters);
		}

		RandomBookingInterval currentInterval = bookingSlot == null ? null : intervalMappingDAO.getIntervalByBookingSlotID(bookingSlot.getBookingSlotID());

		List<RandomBookingInterval> availableIntervals = CollectionUtils.filter(intervals, interval -> {

			if (interval.equals(currentInterval)) {

				return true;
			}

			return interval.isEnabled();
		});

		XMLUtils.append(doc, updateElement, "Intervals", availableIntervals);

		if (bookingSlot != null) {

			RandomBookingInterval selectedInterval = intervalMappingDAO.getIntervalByBookingSlotID(bookingSlot.getBookingSlotID());

			if (selectedInterval != null) {

				XMLUtils.appendNewElement(doc, updateElement, "SelectedIntervalID", selectedInterval.getIntervalID());
			}
		}

		return viewFragmentTransformer.createViewFragment(doc);
	}

	@Override
	public void processBookingSlotFormRequest(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		Integer intervalID = NumberUtils.toInt(req.getParameter("randomBookingInterval"));

		if (intervalID == null) {

			checkForExistingNumberMappings(bookingSlot);

		} else {

			RandomBookingInterval interval = intervalDAO.get(intervalID);
			RandomBookingIntervalMapping intervalMapping = intervalMappingDAO.getByBookingSlotID(bookingSlot.getBookingSlotID());

			validateInterval(interval, intervalMapping, bookingSlot);
		}
	}

	@Override
	public void processBookingSlotAddOrUpdate(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, boolean processBookings) throws Exception {

		Integer bookingSlotID = bookingSlot.getBookingSlotID();
		Integer intervalID = NumberUtils.toInt(req.getParameter("randomBookingInterval"));
		RandomBookingInterval interval = null;
		RandomBookingIntervalMapping intervalMapping = null;

		try {

			if (intervalID == null) {

				checkForExistingNumberMappings(bookingSlot);

				log.info("User " + user + " deselecting interval for bookingSlot " + bookingSlot);

				try (TransactionHandler transactionHandler = new TransactionHandler(dataSource)) {

					intervalMappingDAO.deleteByBookingSlotID(bookingSlotID, transactionHandler);

					transactionHandler.commit();
				}

				return;
			}

			interval = intervalDAO.get(intervalID);
			intervalMapping = intervalMappingDAO.getByBookingSlotID(bookingSlotID);

			validateInterval(interval, intervalMapping, bookingSlot);

		} catch (ValidationException e) {

			log.warn("User " + user + " tried selecting interval " + intervalID + " for booking slot " + bookingSlot + " but it is no longer available for selection");
			return;
		}

		if (intervalMapping != null) {

			if (interval.equals(intervalMapping.getInterval())) {

				return;
			}

			intervalMapping.setInterval(interval);

		} else {

			intervalMapping = new RandomBookingIntervalMapping(bookingSlotID, interval);

		}

		log.info("User " + user + " selecting interval " + interval + " for bookingSlot " + bookingSlot);

		try (TransactionHandler transactionHandler = new TransactionHandler(dataSource)) {

			intervalMappingDAO.addOrUpdate(intervalMapping, transactionHandler, null);

			transactionHandler.commit();
		}

	}

	private void validateInterval(RandomBookingInterval interval, RandomBookingIntervalMapping intervalMapping, BookingSlot bookingSlot) throws ValidationException, SQLException {

		if (intervalMapping != null && intervalMapping.getInterval().equals(interval)) {

			return;
		}

		if (interval == null || !interval.isEnabled()) {

			throw new ValidationException(new ValidationError("randomBookingInterval", ValidationErrorType.InvalidFormat));
		}

		checkForExistingNumberMappings(bookingSlot);
	}

	private void checkForExistingNumberMappings(BookingSlot bookingSlot) throws SQLException, ValidationException {

		if (bookingSlot != null) {

			List<Integer> bookingIDs = CollectionUtils.map(bookingSlot.getBookings(), Booking::getBookingID);

			if (bookingIDs != null) {

				List<RandomBookingNumberMapping> numberMappings = numberMappingDAO.getByBookingIDs(bookingIDs);

				if (numberMappings != null) {

					throw new ValidationException(new ValidationError("ExistingNumberMappings"));
				}
			}
		}
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	@Override
	public ViewFragment getAdminBookingViewFragment(Booking booking, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, IOException, TransformerException {

		RandomBookingInterval interval = intervalMappingDAO.getIntervalByBookingSlotID(booking.getBookingSlot().getBookingSlotID(), RandomBookingInterval.FIELD_NUMBERS);
		RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID());

		if (interval == null && numberMapping == null) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ShowAdminBookingNumber");

		if (numberMapping != null) {

			XMLUtils.append(doc, showElement, numberMapping);
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		List<RandomBookingNumber> availableNumbers = null;

		if (interval != null && !CollectionUtils.isEmpty(interval.getNumbers())) {

			availableNumbers = getAvailableNumbers(booking, interval.getNumbers(), numberMapping);

			XMLUtils.append(doc, showElement, "Numbers", availableNumbers);
		}

		String numberIDString = req.getParameter("randomBookingNumber");

		if (req.getMethod().equalsIgnoreCase("POST") && numberIDString != null) {

			updateNumber(booking, req, res, user, availableNumbers, validationErrors, numberIDString);

			if (res.isCommitted()) {

				return null;
			}
		}

		if (!validationErrors.isEmpty()) {

			XMLUtils.append(doc, showElement, validationErrors);
		}

		return viewFragmentTransformer.createViewFragment(doc);
	}

	private List<RandomBookingNumber> getAvailableNumbers(Booking currentBooking, RandomBookingNumberMapping currentNumberMapping) throws SQLException {

		BookingSlot bookingSlot = currentBooking.getBookingSlot();
		RandomBookingInterval interval = intervalMappingDAO.getIntervalByBookingSlotID(bookingSlot.getBookingSlotID(), RandomBookingInterval.FIELD_NUMBERS);

		if (interval == null || CollectionUtils.isEmpty(interval.getNumbers())) {

			return Collections.emptyList();
		}

		return getAvailableNumbers(currentBooking, interval.getNumbers(), currentNumberMapping);
	}

	private List<RandomBookingNumber> getAvailableNumbers(Booking currentBooking, List<RandomBookingNumber> numbers) throws SQLException {

		return getAvailableNumbers(currentBooking, numbers, null);
	}

	private List<RandomBookingNumber> getAvailableNumbers(Booking currentBooking, List<RandomBookingNumber> numbers, RandomBookingNumberMapping currentNumberMapping) throws SQLException {

		List<Booking> bookings = bookingSlotAdminModule.getBookingsByBookingSlot(currentBooking.getBookingSlot());

		if (bookings == null) {

			return Collections.emptyList();
		}

		List<Integer> assignedNumberIDs = getAssignedNumberIDs(bookings, currentNumberMapping);

		List<RandomBookingNumber> availableNumbers = CollectionUtils.filter(numbers, number -> {

			if (!number.isEnabled()) {

				return false;
			}

			return !CollectionUtils.contains(assignedNumberIDs, number.getNumberID());
		});

		return availableNumbers != null ? availableNumbers : Collections.emptyList();
	}

	private List<Integer> getAssignedNumberIDs(List<Booking> bookings, RandomBookingNumberMapping currentNumberMapping) throws SQLException {

		List<Integer> bookingIDs = CollectionUtils.map(bookings, Booking::getBookingID);

		if (bookingIDs != null) {

			List<RandomBookingNumberMapping> assignedNumberMappings = numberMappingDAO.getByBookingIDs(bookingIDs);

			if (assignedNumberMappings != null) {

				assignedNumberMappings.removeIf(mapping -> mapping.equals(currentNumberMapping));

				List<RandomBookingNumber> assignedNumbers = CollectionUtils.map(assignedNumberMappings, RandomBookingNumberMapping::getNumber);

				return CollectionUtils.map(assignedNumbers, RandomBookingNumber::getNumberID);
			}
		}

		return Collections.emptyList();
	}

	private void updateNumber(Booking booking, HttpServletRequest req, HttpServletResponse res, User user, List<RandomBookingNumber> availableNumbers, List<ValidationError> validationErrors, String numberIDString) throws SQLException, IOException {

		Integer numberID = NumberUtils.toInt(numberIDString);

		if (numberID == null) {

			log.info("User " + user + " deselecting number for booking " + booking);

			numberMappingDAO.deleteByBookingID(booking.getBookingID());

			bookingSlotAdminModule.addBookingEvent(booking, BookingEventType.OTHER, eventManagerDeselectedNumber, user, null, null);

			redirectToShowBooking(req, res, booking.getBookingID());
			return;
		}

		if (availableNumbers == null) {

			return;
		}

		RandomBookingNumber number = CollectionUtils.find(availableNumbers, n -> n.getNumberID().equals(numberID));

		if (number == null) {

			validationErrors.add(new ValidationError("randomBookingNumber", ValidationErrorType.InvalidFormat));
			return;
		}

		if (!validationErrors.isEmpty()) {

			return;
		}

		RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID());

		if (numberMapping != null) {

			RandomBookingNumber oldNumber = numberMapping.getNumber();

			numberMapping.setNumber(number);

			log.info("User " + user + " changing booking number of booking " + booking + " from number " + oldNumber + " to number " + number);

			numberMappingDAO.update(numberMapping);

		} else {

			numberMapping = new RandomBookingNumberMapping(booking.getBookingID(), number);

			log.info("User " + user + " selecting booking number " + number + " for booking " + booking);

			numberMappingDAO.add(numberMapping);
		}

		bookingSlotAdminModule.addBookingEvent(booking, BookingEventType.OTHER, eventManagerSelectedNumber + numberMapping.getIndex(), user, null, null);

		redirectToShowBooking(req, res, booking.getBookingID());
	}

	private void redirectToShowBooking(HttpServletRequest req, HttpServletResponse res, Integer bookingID) throws IOException {

		bookingSlotAdminModule.redirectToMethod(req, res, "/show/" + bookingID);
	}

	@Override
	public String getProviderID() {

		return this.moduleDescriptor.getModuleID().toString();
	}

	@Override
	public ViewFragment getUserBookingViewFragment(Booking booking, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, TransformerException {

		RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID(), RandomBookingNumber.FIELD_INTERVAL);

		if (numberMapping == null) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ShowUserBookingNumber");

		XMLUtils.append(doc, showElement, numberMapping);

		return viewFragmentTransformer.createViewFragment(doc);
	}

	@EventListener(channel = Booking.class)
	public void processEvent(CRUDEvent<Booking> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.UPDATE) {

			for (Booking booking : event.getBeans()) {

				processBookingUpdate(booking, event);
			}

		} else if (event.getAction() == CRUDAction.DELETE) {

			for (Booking booking : event.getBeans()) {

				processBookingDelete(booking, event);
			}
		}
	}

	private void processBookingUpdate(Booking booking, CRUDEvent<Booking> event) {

		try {

			RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID());

			if (numberMapping == null) {

				return;
			}

			RandomBookingNumber currentNumber = numberMapping.getNumber();

			List<RandomBookingNumber> availableNumbers = getAvailableNumbers(booking, numberMapping);

			boolean currentNumberIsAvailable = CollectionUtils.find(availableNumbers, number -> number.equals(currentNumber)) != null;

			if (currentNumberIsAvailable) {

				log.info("Not updating mapping " + numberMapping + " due to event " + event + ", the current number is still available");
				return;
			}

			RandomBookingNumber availableNumberWithSameIndex = CollectionUtils.find(availableNumbers, number -> number.getIndex().equals(currentNumber.getIndex()));

			if (availableNumberWithSameIndex != null) {

				log.info("Changing number from " + currentNumber + " to " + availableNumberWithSameIndex + " on mapping " + numberMapping + " due to event " + event);

				numberMapping.setNumber(availableNumberWithSameIndex);

				numberMappingDAO.update(numberMapping);

			} else {

				log.info("Removing mapping " + numberMapping + " due to event " + event + ", couldn't find an available number with same index");

				numberMappingDAO.delete(numberMapping);

				bookingSlotAdminModule.addBookingEvent(booking, BookingEventType.OTHER, eventSystemDeselectedNumber, null, null, null);
			}

		} catch (SQLException e) {

			log.error("Error while processing event " + event + " for booking " + booking);
		}
	}

	private void processBookingDelete(Booking booking, CRUDEvent<Booking> event) {

		try {

			RandomBookingNumberMapping numberMapping = numberMappingDAO.getByBookingID(booking.getBookingID());

			if (numberMapping == null) {

				return;
			}

			log.info("Removing mapping " + numberMapping + " due to event " + event + ", booking has been deleted");

			numberMappingDAO.delete(numberMapping);

		} catch (SQLException e) {

			log.error("Error while processing event " + event + " for booking " + booking);
		}
	}

}
