package com.nordicpeak.flowengine.randombookingnumber.daos;

import java.lang.reflect.Field;
import java.sql.SQLException;

import javax.sql.DataSource;

import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;

import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingInterval;
import com.nordicpeak.flowengine.randombookingnumber.beans.RandomBookingIntervalMapping;

public class RandomBookingIntervalMappingDAO extends AnnotatedDAO<RandomBookingIntervalMapping> {

	private final QueryParameterFactory<RandomBookingIntervalMapping, Integer> bookingSlotIDParamFactory;

	public RandomBookingIntervalMappingDAO(DataSource dataSource, Class<RandomBookingIntervalMapping> beanClass, AnnotatedDAOFactory daoFactory) {

		super(dataSource, beanClass, daoFactory);

		bookingSlotIDParamFactory = this.getParamFactory(RandomBookingIntervalMapping.FIELD_BOOKING_SLOT_ID, Integer.class);
	}

	public RandomBookingInterval getIntervalByBookingSlotID(Integer bookingSlotID, Field... relations) throws SQLException {

		RandomBookingIntervalMapping mapping = getByBookingSlotID(bookingSlotID, relations);

		if (mapping == null) {

			return null;
		}

		return mapping.getInterval();
	}

	public RandomBookingIntervalMapping getByBookingSlotID(Integer bookingSlotID, Field... relations) throws SQLException {

		HighLevelQuery<RandomBookingIntervalMapping> query = new HighLevelQuery<>();

		query.addParameter(bookingSlotIDParamFactory.getParameter(bookingSlotID));

		query.addRelations(RandomBookingIntervalMapping.FIELD_INTERVAL);

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.get(query);
	}

	public void deleteByBookingSlotID(Integer bookingSlotID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<RandomBookingIntervalMapping> query = new HighLevelQuery<>();

		query.addParameter(bookingSlotIDParamFactory.getParameter(bookingSlotID));

		this.delete(query, transactionHandler);
	}

}
