package com.nordicpeak.flowengine.queries.ratingscalequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.populators.AlternativesPopulator;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class RatingScaleQueryCRUD extends BaseQueryCRUD<RatingScaleQuery, RatingScaleQueryProviderModule> {

	protected AnnotatedDAOWrapper<RatingScaleQuery, Integer> queryDAO;
	
	protected static AlternativesPopulator<RatingScaleAlternative> ALTERNATIVES_POPLATOR = new AlternativesPopulator<RatingScaleAlternative>(RatingScaleAlternative.class);
	
	public RatingScaleQueryCRUD(AnnotatedDAOWrapper<RatingScaleQuery, Integer> queryDAO, BeanRequestPopulator<RatingScaleQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, RatingScaleQueryProviderModule callback) {
		
		super(RatingScaleQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.queryDAO = queryDAO;
	}

	@Override
	protected RatingScaleQuery populateFromUpdateRequest(RatingScaleQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		RatingScaleQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		List<RatingScaleAlternative> alternatives = ALTERNATIVES_POPLATOR.populate(bean.getAlternatives(), req, 1024, 255, validationErrors);
		
		if(CollectionUtils.isEmpty(alternatives) || alternatives.size() < 2) {
			validationErrors.add(new ValidationError("TooFewAlternatives"));
		}
		
		if(!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		query.setAlternatives(alternatives);
		
		return query;
		
	}
	
	@Override
	protected List<Field> getBeanRelations() {
		
		return Arrays.asList(RatingScaleQuery.ALTERNATIVES_RELATION);
	}
	
}
