package com.nordicpeak.flowengine.remotegeometryendpointprovider;

import java.io.Serializable;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.APISource;

@Table(name = "remote_geometry_endpoints")
@XMLElement
public class RemoteGeometryEndpoint extends GeneratedElementable implements XMLParserPopulateable, Serializable, RemoteGeometryEndpointDescriptor, APISource {

	private static final long serialVersionUID = -8348666528294482053L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer endpointID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 4096)
	@XMLElement
	private String url;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String username;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String password;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private String encoding;
	
	public Integer getEndpointID() {

		return endpointID;
	}

	public void setEndpointID(Integer endpointID) {

		this.endpointID = endpointID;
	}

	@Override
	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getUrl() {

		return url;
	}

	public void setUrl(String url) {

		this.url = url;
	}

	public String getUsername() {

		return username;
	}

	public void setUsername(String username) {

		this.username = username;
	}

	public String getPassword() {

		return password;
	}

	public void setPassword(String password) {

		this.password = password;
	}

	public String getEncoding() {

		return encoding;
	}

	public void setEncoding(String encoding) {

		this.encoding = encoding;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		this.endpointID = XMLValidationUtils.validateParameter("endpointID", xmlParser, true, PositiveStringIntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public Charset getCharset() {

		return Charset.forName(encoding);
	}
	
	@Override
	public String toString() {

		return name + " (ID: " + endpointID + ")";
	}
	
	//TODO rename the getURL method and field to getAddress and address 
	@Override
	public String getAddress() {

		return url;
	}

}
