<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<xsl:if test="not(RemoteInfoQueryInstance/RemoteInfoQuery/hideOnEmptyResponse = 'true' and not(RemoteInfoQueryInstance/infoText))">
		
			<div class="query">
				<xsl:attribute name="class">
					<xsl:text>query</xsl:text>
					<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/hideTitle = 'true'"> notitle</xsl:if>
					<xsl:if test="RemoteInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
				</xsl:attribute>
				
				<article class="remoteinfoquery show-mode">
					
					<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/hideBackground = 'true'">
						<xsl:attribute name="class">remoteinfoquery show-mode nobackground</xsl:attribute>
					</xsl:if>
					
					<div class="heading-wrapper">
					
						<xsl:if test="not(RemoteInfoQueryInstance/RemoteInfoQuery/hideTitle = 'true')">
							<h2>
								<xsl:value-of select="RemoteInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
							</h2>
						</xsl:if>
					</div>
					
					<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/description">
						<span class="italic">
							<xsl:value-of select="RemoteInfoQueryInstance/RemoteInfoQuery/description" disable-output-escaping="yes" />
						</span>
					</xsl:if>
					
					<div class="clearboth"/>
	
					<xsl:if test="RemoteInfoQueryInstance/infoText">
						<span class="italic">
							<xsl:value-of select="RemoteInfoQueryInstance/infoText" disable-output-escaping="yes" />
						</span>		
					</xsl:if>				
	
	
				</article>
					
			</div>
			
		</xsl:if>
		
	</xsl:template>	
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:if test="not(RemoteInfoQueryInstance/RemoteInfoQuery/hideOnEmptyResponse = 'true' and not(RemoteInfoQueryInstance/infoText))">
		
			<xsl:variable name="shortQueryID" select="concat('q', RemoteInfoQueryInstance/RemoteInfoQuery/queryID)" />
		
			<xsl:variable name="queryID" select="concat('query_', RemoteInfoQueryInstance/RemoteInfoQuery/queryID)" />
		
			<div class="query remoteinfoquery" id="{$queryID}">
				<xsl:attribute name="class">
					<xsl:text>query remoteinfoquery</xsl:text>
					<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/hideTitle = 'true'"> notitle</xsl:if>
					<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
					<xsl:if test="RemoteInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
				</xsl:attribute>
				
				<a name="{$queryID}" />
			
				<xsl:if test="ValidationErrors/validationError">
					<div id="{$queryID}-validationerrors" class="validationerrors">
						<xsl:if test="ValidationErrors/validationError[messageKey]">
							<div class="info-box error">
								<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
								<div class="marker"></div>
							</div>
						</xsl:if>
					</div>
				</xsl:if>
		
				<article>
				
					<xsl:attribute name="class">
						<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/hideBackground = 'true'">
							<xsl:text> nobackground</xsl:text>
						</xsl:if>
						
						<xsl:if test="ValidationErrors/validationError[messageKey]">
							<xsl:text> error</xsl:text>
						</xsl:if>
						
					</xsl:attribute>
				
					<div class="heading-wrapper">
						
						<xsl:if test="not(RemoteInfoQueryInstance/RemoteInfoQuery/hideTitle = 'true')">
							<h2>
								<xsl:value-of select="RemoteInfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
							</h2>
						</xsl:if>
						
						<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/helpText">		
							<xsl:apply-templates select="RemoteInfoQueryInstance/RemoteInfoQuery/helpText" />
						</xsl:if>
					</div>
					
					<xsl:if test="RemoteInfoQueryInstance/RemoteInfoQuery/description">
						<span class="italic">
							<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
							<xsl:value-of select="RemoteInfoQueryInstance/RemoteInfoQuery/description" disable-output-escaping="yes" />
						</span>		
					</xsl:if>				
	
					<xsl:if test="RemoteInfoQueryInstance/infoText">
						<span class="italic">
							<xsl:value-of select="RemoteInfoQueryInstance/infoText" disable-output-escaping="yes" />
						</span>		
					</xsl:if>				
					
				</article>			
			
			</div>		
		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'UnableToGetDataError']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:choose>
					<xsl:when test="../../RemoteInfoQueryInstance/RemoteInfoQuery/endpointErrorText">
						<xsl:value-of select="../../RemoteInfoQueryInstance/RemoteInfoQuery/endpointErrorText" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.StandardEndpointErrorText"></xsl:value-of>
					</xsl:otherwise>
				</xsl:choose>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
		
</xsl:stylesheet>