package com.nordicpeak.flowengine.queries.remotesinglesignquery;

import java.sql.SQLException;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MultiSigningQueryProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.remotesinglesignquery.api.RemoteSinglesignQueryEndpointAdminModule;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class RemoteSinglesignQueryProviderModule extends BaseQueryProviderModule<RemoteSinglesignQueryInstance> implements BaseQueryCRUDCallback, MultiSigningQueryProvider {

	@XSLVariable(prefix = "java.")
	private String exportFirstName;

	@XSLVariable(prefix = "java.")
	private String exportLastName;

	@XSLVariable(prefix = "java.")
	private String exportCitizenID;
	
	@XSLVariable(prefix = "java.")
	private String exportEmail;
	
	@XSLVariable(prefix = "java.")
	private String exportMobilePhone;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	private AnnotatedDAO<RemoteSinglesignQuery> queryDAO;
	private AnnotatedDAO<RemoteSinglesignQueryInstance> queryInstanceDAO;

	private RemoteSinglesignQueryCRUD queryCRUD;

	private QueryParameterFactory<RemoteSinglesignQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<RemoteSinglesignQueryInstance, Integer> queryInstanceIDParamFactory;
	private QueryParameterFactory<RemoteSinglesignQueryInstance, String> queryInstanceCitizenIDParamFactory;
	
	@InstanceManagerDependency
	private RemoteSinglesignQueryEndpointAdminModule apiModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RemoteSinglesignQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(RemoteSinglesignQuery.class);
		queryInstanceDAO = daoFactory.getDAO(RemoteSinglesignQueryInstance.class);

		queryCRUD = new RemoteSinglesignQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<RemoteSinglesignQuery>(RemoteSinglesignQuery.class), "RemoteSinglesignQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		queryInstanceCitizenIDParamFactory = queryInstanceDAO.getParamFactory("citizenID", String.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		RemoteSinglesignQuery query = new RemoteSinglesignQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		RemoteSinglesignQuery query = new RemoteSinglesignQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		RemoteSinglesignQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		RemoteSinglesignQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		RemoteSinglesignQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new RemoteSinglesignQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private RemoteSinglesignQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<RemoteSinglesignQuery> query = new HighLevelQuery<RemoteSinglesignQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private RemoteSinglesignQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<RemoteSinglesignQuery> query = new HighLevelQuery<RemoteSinglesignQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private RemoteSinglesignQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<RemoteSinglesignQueryInstance> query = new HighLevelQuery<RemoteSinglesignQueryInstance>(RemoteSinglesignQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(RemoteSinglesignQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		RemoteSinglesignQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		RemoteSinglesignQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		RemoteSinglesignQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, RemoteSinglesignQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<RemoteSinglesignQueryInstance> getQueryInstanceClass() {

		return RemoteSinglesignQueryInstance.class;
	}

	public RemoteSinglesignQueryEndpointAdminModule getAPIModule() {
		
		return apiModule;
	}

	@Override
	public QueryResponse getFormHTML(RemoteSinglesignQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		RemoteSinglesignQuery query = queryInstance.getQuery();
		
		if (query.getEndpoint() == null && validationErrors == null) {
		
			validationErrors = CollectionUtils.getList(new ValidationError("QueryConfigurationError"));
			
		} else if (query.getEndpoint() != null) {
			
			List<ValidationError> apiErrors = getFromAPI(queryInstance, poster, attributeHandler);

			if (validationErrors instanceof AbstractList) { // Fix for unmodifiable list
				validationErrors = new ArrayList<>(validationErrors);
			}

			if (apiErrors != null) {

				for (ValidationError error : apiErrors) {

					if (validationErrors == null || !validationErrors.contains(error)) {
						
						validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, error);
					}
				}

			} else if (apiErrors == null && validationErrors != null) { // Remove populate validation error if it worked just now

				Iterator<ValidationError> it = validationErrors.iterator();

				while (it.hasNext()) {

					ValidationError error = it.next();
					
					if ("EndpointError".equals(error.getMessageKey()) && "Empty".equals(error.getDisplayName())) {

						it.remove();
						break;
					}
				}
			}
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}
	
	private List<ValidationError> getFromAPI(RemoteSinglesignQueryInstance queryInstance, User poster, AttributeHandler attributeHandler) {

		RemoteSinglesignQuery query = queryInstance.getQuery();

		if (apiModule == null) {

			log.error("Module configuration error apiModule is null");
			return Collections.singletonList(new ValidationError("EndpointError", "Module configuration error"));

		} else {

			try {
				RemoteSigningParty signingParty = apiModule.getRemoteSigningPartyFromEndpoint(query.getEndpoint(), poster, attributeHandler);

				if (signingParty != null) {

					if(!signingParty.getCitizenID().equals(queryInstance.getCitizenID())) {
						
						queryInstance.setCitizenID(signingParty.getCitizenID());
						queryInstance.setFirstname(signingParty.getFirstname());
						queryInstance.setLastname(signingParty.getLastname());
						queryInstance.setEmail(signingParty.getEmail());
						queryInstance.setMobilePhone(signingParty.getMobilePhone());
					}
					
				} else {
					
					return Collections.singletonList(new ValidationError("EndpointError", "Empty"));
				}

			} catch (ValidationException e) {

				return e.getErrors();
			}
		}

		return null;
	}

	@Override
	public void populate(RemoteSinglesignQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		RemoteSinglesignQuery query = queryInstance.getQuery();
		
		if (query.getEndpoint() == null && !allowPartialPopulation) {

				throw new ValidationException(new ValidationError("QueryConfigurationError"));
		}
		
		if (query.getEndpoint() != null && queryInstance.getCitizenID() == null) {
			
			if (allowPartialPopulation) {
				
				return;
				
			} else {
				
				throw new ValidationException(new ValidationError("EndpointError", "Empty"));
			}
		}
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		Integer queryID = queryInstance.getQuery().getQueryID();
		
		String emailID = "q" + queryID + "_email";
		String phoneID = "q" + queryID + "_mobilephone";

		String email = ValidationUtils.validateParameter(emailID, req, false, EmailPopulator.getPopulator(), validationErrors);
		String phone = ValidationUtils.validateParameter(phoneID, req, false, StringSwedishPhoneNumberPopulator.getPopulator(), validationErrors);

		if (email == null && phone == null && validationErrors.isEmpty()) {
			
			validationErrors.add(new ValidationError("ContactWayRequired"));
		}
		
		queryInstance.setEmail(email);
		queryInstance.setMobilePhone(phone);

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		queryInstance.getQueryInstanceDescriptor().setPopulated(true);
		
		if (query.isSetAsAttribute()) {
			
			queryInstance.setAttribute(attributeHandler);
		}

	}

	@Override
	public List<Integer> getQueryInstanceIDs(String citizenIdentifier) throws SQLException {

		ArrayListQuery<Integer> query = new ArrayListQuery<Integer>(queryInstanceDAO.getDataSource(), "SELECT queryInstanceID FROM " + queryInstanceDAO.getTableName() + " WHERE " + queryInstanceCitizenIDParamFactory.getColumnName() + " = ?", IntegerPopulator.getPopulator());
		query.setString(1, citizenIdentifier);

		return query.executeQuery();
	}
	
	public String getExportFirstName() {
		return exportFirstName;
	}

	public String getExportLastName() {
		return exportLastName;
	}

	public String getExportCitizenID() {
		return exportCitizenID;
	}

	public String getExportEmail() {
		return exportEmail;
	}

	public String getExportMobilePhone() {
		return exportMobilePhone;
	}
}
