package com.nordicpeak.flowengine.reservation;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.foregroundmodules.rest.AnnotatedRESTModule;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.hierarchy.foregroundmodules.rest.responsehandlers.XMLResponseHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.reservation.beans.Reservation;
import com.nordicpeak.flowengine.reservation.daos.ReservationDAO;


public class ReservationAPIModule extends AnnotatedRESTModule {

	@ModuleSetting
	@CheckboxSettingDescriptor(name="Force UTF-8 encoding", description="Forces UTF-8 encoding to be used instead of the system default encoding.")
	protected boolean forceUTF8;
	
	@InstanceManagerDependency(required = true)
	private FlowAdminModule flowAdminModule;

	private ReservationDAO reservationDAO;
	
	@Override
	protected void moduleConfigured() throws Exception {

		String encoding;
		
		if(forceUTF8) {
			
			encoding = "UTF-8";
			
		}else {
			
			encoding = sectionInterface.getSystemInterface().getEncoding();
		}
		
		setResponseHandlers(CollectionUtils.getGenericSingletonList(new XMLResponseHandler(encoding)));
	}	
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ReservationModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("daos/DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.warn(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface);

		reservationDAO = new ReservationDAO(dataSource, Reservation.class, daoFactory);
	}
	
	@RESTMethod(alias = "getreservations", method = "get")
	public Document getMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {
		
		log.info("User " + user + " requesting reservations");
		
		List<Reservation> reservations = reservationDAO.getAll();
		
		Document doc = XMLUtils.createDomDocument();
		
		Element reservationsElement = doc.createElement("Reservations");
		doc.appendChild(reservationsElement);
		
		XMLUtils.append(doc, reservationsElement, reservations);
		
		return doc;
	}
	
	@RESTMethod(alias = "deletereservation/{reservationID}", method = {"post"}, requireLogin = true)
	public Document deleteReservation(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "reservationID") String reservationID) throws Throwable {

		Reservation reservation = reservationDAO.getByReservationID(reservationID);

		if (reservation == null) {

			throw new URINotFoundException(uriParser);
		}

		Integer flowFamilyID = reservation.getFlowFamilyID();

		synchronized (flowFamilyID) {

			reservation = reservationDAO.getByReservationID(reservationID);

			if (reservation != null) {

				log.info("User " + user + " deleting reservation " + reservation);

				reservationDAO.delete(reservation);
				
				Document doc = XMLUtils.createDomDocument();
				
				Element deletedReservationsElement = doc.createElement("DeletedReservations");
				doc.appendChild(deletedReservationsElement);
				
				deletedReservationsElement.appendChild(reservation.toXML(doc));

				return doc;
			}
		}

		throw new URINotFoundException(uriParser);
	}	
}
