package com.nordicpeak.flowengine.reservation.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.util.UUID;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.beans.Flow;

@Table(name = "reservation_module_reservations")
@XMLElement
public class Reservation extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = -2881372784113729471L;

	public static final Field FIELD_RESERVATION_ID = ReflectionUtils.getField(Reservation.class, "reservationID");
	public static final Field FIELD_OBJECT_ID = ReflectionUtils.getField(Reservation.class, "objectID");
	public static final Field FIELD_FLOW_FAMILY_ID = ReflectionUtils.getField(Reservation.class, "flowFamilyID");
	public static final Field FIELD_FLOW_INSTANCE_MANAGER_ID = ReflectionUtils.getField(Reservation.class, "flowInstanceManagerID");
	public static final Field FIELD_FLOW_INSTANCE_ID = ReflectionUtils.getField(Reservation.class, "flowInstanceID");
	public static final Field FIELD_ADDED = ReflectionUtils.getField(Reservation.class, "added");
	public static final Field FIELD_EXPIRES = ReflectionUtils.getField(Reservation.class, "expires");

	@Key
	@DAOManaged
	@XMLElement
	private String reservationID;

	@DAOManaged
	@XMLElement
	private String objectID;

	@DAOManaged
	@XMLElement
	private Integer flowFamilyID;

	@DAOManaged
	@XMLElement
	private String flowInstanceManagerID;

	@DAOManaged
	@XMLElement
	private Integer flowInstanceID;

	@DAOManaged
	@XMLElement
	private Timestamp added;

	@DAOManaged
	@XMLElement
	private Timestamp expires;

	public Reservation() {}

	public Reservation(String objectID, Integer flowFamilyID, String flowInstanceManagerID, Integer flowInstanceID, Timestamp added, Timestamp expires) {

		this.reservationID = UUID.randomUUID().toString();
		this.objectID = objectID;
		this.flowFamilyID = flowFamilyID;
		this.flowInstanceManagerID = flowInstanceManagerID;
		this.flowInstanceID = flowInstanceID;
		this.added = added;
		this.expires = expires;
	}

	@Override
	public String toString() {

		return "(reservationID=" + reservationID + ", objectID=" + objectID + ", flowFamilyID=" + flowFamilyID + ", flowInstanceManagerID=" + flowInstanceManagerID + ", flowInstanceID=" + flowInstanceID + ")";
	}

	public JsonNode toJSON(Flow flow) {

		JsonObject json = new JsonObject();

		json.putField("reservationID", reservationID);
		json.putField("objectID", objectID);
		json.putField("flowFamilyID", getFlowFamilyIDToJSON(flow));
		json.putField("flowInstanceManagerID", flowInstanceManagerID);
		json.putField("flowInstanceID", flowInstanceID);
		json.putField("added", DateUtils.DATE_TIME_FORMATTER.format(added));
		json.putField("expires", expires != null ? DateUtils.DATE_TIME_FORMATTER.format(expires) : "");

		return json;
	}

	private String getFlowFamilyIDToJSON(Flow flow) {

		if (flow == null) {

			return flowFamilyID.toString();
		}

		return flowFamilyID + " (" + flow.getName() + ")";
	}

	public String getReservationID() {

		return reservationID;
	}

	public void setReservationID(String reservationID) {

		this.reservationID = reservationID;
	}

	public String getObjectID() {

		return objectID;
	}

	public void setObjectID(String objectID) {

		this.objectID = objectID;
	}

	public Integer getFlowFamilyID() {

		return flowFamilyID;
	}

	public void setFlowFamilyID(Integer flowFamilyID) {

		this.flowFamilyID = flowFamilyID;
	}

	public String getFlowInstanceManagerID() {

		return flowInstanceManagerID;
	}

	public void setFlowInstanceManagerID(String flowInstanceManagerID) {

		this.flowInstanceManagerID = flowInstanceManagerID;
	}

	public Integer getFlowInstanceID() {

		return flowInstanceID;
	}

	public void setFlowInstanceID(Integer flowInstanceID) {

		this.flowInstanceID = flowInstanceID;
	}

	public Timestamp getAdded() {

		return added;
	}

	public void setAdded(Timestamp added) {

		this.added = added;
	}

	public Timestamp getExpires() {

		return expires;
	}

	public void setExpires(Timestamp expires) {

		this.expires = expires;
	}

}