package com.nordicpeak.flowengine.restrictedflowadmin;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.Breadcrumb;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleViewFragment;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.utils.AdvancedIntegerBasedCRUD;
import se.unlogic.hierarchy.core.utils.ViewFragmentUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.CRUDDAO;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.serialization.SerializationUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.templates.TemplateUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.populators.annotated.FieldRequestMapping;
import se.unlogic.webutils.populators.annotated.RequestMapping;

import com.nordicpeak.flowengine.beans.Category;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowForm;
import com.nordicpeak.flowengine.interfaces.FlowAdminFragmentExtensionViewProvider;
import com.nordicpeak.flowengine.listeners.FlowFormElementableListener;
import com.nordicpeak.flowengine.validationerrors.FlowFamilyAliasCollisionValidationError;

public class FlowRestrictedCRUD extends AdvancedIntegerBasedCRUD<Flow, RestrictedFlowAdminModule> {
	
	private static AnnotatedRequestPopulator<FlowFamily> FLOW_FAMILY_POULATOR = new AnnotatedRequestPopulator<FlowFamily>(FlowFamily.class);
	
	protected static final List<Field> FLOW_FIELDS = Arrays.asList(Flow.DESCRIPTION_SHORT_FIELD, Flow.DESCRIPTION_LONG_FIELD, Flow.SUBMITTED_MESSAGE_FIELD, Flow.TAGS_RELATION, Flow.CHECKS_RELATION, Flow.EXTERNAL_LINK_FIELD);
	protected static final List<Field> FLOW_FAMILY_FIELDS = Arrays.asList(FlowFamily.USE_LOGIN_HELP_LINK_FIELD, FlowFamily.LOGIN_HELP_LINK_NAME_FIELD, FlowFamily.LOGIN_HELP_LINK_URL_FIELD, FlowFamily.CONTACT_NAME_FIELD, FlowFamily.CONTACT_EMAIL_FIELD, FlowFamily.CONTACT_PHONE_FIELD, FlowFamily.CONTACT_WEB_ADDRESS_FIELD, FlowFamily.OWNER_NAME_FIELD, FlowFamily.OWNER_EMAIL_FIELD);
	
	static {
		for (Iterator<RequestMapping> it = FLOW_FAMILY_POULATOR.getRequestMappings().iterator(); it.hasNext();) {
			
			RequestMapping requestMapping = it.next();
			
			if (requestMapping instanceof FieldRequestMapping) {
				
				FieldRequestMapping fieldRequestMapping = (FieldRequestMapping) requestMapping;
				
				if (FLOW_FAMILY_FIELDS.contains(fieldRequestMapping.getField())) {
					continue;
				}
			}
			
			it.remove();
		}
	}
	
	protected final FlowFormElementableListener flowFormElementableListener;
	
	public FlowRestrictedCRUD(CRUDDAO<Flow, Integer> crudDAO, RestrictedFlowAdminModule callback) {
		
		super(Flow.class, crudDAO, new AnnotatedRequestPopulator<Flow>(Flow.class), "Flow", "flow", "", callback);
		
		flowFormElementableListener = new FlowFormElementableListener(callback);
		
		for (Iterator<RequestMapping> it = ((AnnotatedRequestPopulator<Flow>) populator).getRequestMappings().iterator(); it.hasNext();) {
			
			RequestMapping requestMapping = it.next();
			
			if (requestMapping instanceof FieldRequestMapping) {
				
				FieldRequestMapping fieldRequestMapping = (FieldRequestMapping) requestMapping;
				
				if (FLOW_FIELDS.contains(fieldRequestMapping.getField())) {
					continue;
				}
			}
			
			it.remove();
		}
	}
	
	@Override
	public Flow getBean(Integer beanID, String getMode) throws SQLException, AccessDeniedException {
		
		Flow flow = callback.getCachedFlow(beanID);
		
		if (flow == null) {
			
			return null;
		}
		
		Blob icon = flow.getIcon();
		
		flow = SerializationUtils.cloneSerializable(flow);
		
		flow.setIcon(icon);
		
		return flow;
	}
	
	@Override
	protected void appendBean(Flow flow, Element targetElement, Document doc, User user) {
		
		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
		genDoc.addFieldElementableListener(FlowForm.class, flowFormElementableListener);
		
		TemplateUtils.setTemplatedFields(flow.getFlowFamily(), callback.getFlowAdminModule());
		
		super.appendBean(flow, targetElement, genDoc, user);
	}
	
	@Override
	protected void appendUpdateFormData(Flow bean, Document doc, Element typeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		
		XMLUtils.appendNewElement(doc, typeElement, "ckConnectorModuleAlias", callback.getCkConnectorModuleAlias());
		XMLUtils.appendNewElement(doc, typeElement, "cssPath", callback.getCssPath());
		
		XMLUtils.appendNewElement(doc, typeElement, "RequiresTags", callback.requiresTags());
	}
	
	@Override
	protected ForegroundModuleResponse beanUpdated(Flow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		callback.getEventHandler().sendEvent(Flow.class, new CRUDEvent<Flow>(CRUDAction.UPDATE, bean), EventTarget.ALL);
		
		callback.addFlowFamilyEvent(callback.getEventFlowUpdatedMessage(), bean, user);
		
		callback.redirectToMethod(req, res, "/showflow/" + bean.getFlowID());
		
		return null;
	}
	
	@Override
	protected void checkAddAccess(User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		throw new UnsupportedOperationException();
	}
	
	@Override
	protected void checkUpdateAccess(Flow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		checkAccess(user, bean);
	}
	
	@Override
	protected void checkDeleteAccess(Flow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		throw new UnsupportedOperationException();
	}
	
	@Override
	protected void checkShowAccess(Flow bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		
		checkAccess(user, bean);
	}
	
	public void checkAccess(User user, Flow bean) throws AccessDeniedException {
		
		if (!callback.hasFlowAccess(user, bean)) {
			
			throw new AccessDeniedException("User does not have access to flow type " + bean.getFlowType());
		}
	}
	
	@Override
	protected void validateUpdatePopulation(Flow bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {
		
		FLOW_FAMILY_POULATOR.populate(bean.getFlowFamily(), req);
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		validateFlowCategory(bean, req, errors);
		
		validateContactFields(bean.getFlowFamily(), errors);
		
		validateAliases(bean.getFlowFamily(), errors);
		
		validateTags(bean, errors);
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
	
	private void validateTags(Flow bean, List<ValidationError> errors) {
		
		if (bean.getTags() != null) {
			
			TreeSet<String> tagSet = new TreeSet<String>(String.CASE_INSENSITIVE_ORDER);
			tagSet.addAll(bean.getTags());
			bean.setTags(new ArrayList<String>(tagSet));
		}
		
		if (callback.requiresTags() && CollectionUtils.isEmpty(bean.getTags())) {
			
			errors.add(new ValidationError("tags", ValidationErrorType.RequiredField));
		}
	}
	
	private void validateContactFields(FlowFamily flowFamily, List<ValidationError> errors) {
		
		if ((!StringUtils.isEmpty(flowFamily.getContactEmail()) || !StringUtils.isEmpty(flowFamily.getContactPhone())) && StringUtils.isEmpty(flowFamily.getContactName())) {
			
			errors.add(new ValidationError("contactName", ValidationErrorType.RequiredField));
		}
		
		if (!StringUtils.isEmpty(flowFamily.getOwnerEmail()) && StringUtils.isEmpty(flowFamily.getOwnerName())) {
			
			errors.add(new ValidationError("ownerName", ValidationErrorType.RequiredField));
		}
	}
	
	private void validateAliases(FlowFamily flowFamily, List<ValidationError> errors) {
		
		if (!CollectionUtils.isEmpty(flowFamily.getAliases())) {
			
			TreeSet<String> tagSet = new TreeSet<String>(String.CASE_INSENSITIVE_ORDER);
			tagSet.addAll(flowFamily.getAliases());
			flowFamily.setAliases(new ArrayList<String>(tagSet));
			
			for (String alias : flowFamily.getAliases()) {
				
				FlowFamily existing = callback.getFlowFamilyByAlias(alias);
				
				if (existing != null && (flowFamily.getFlowFamilyID() == null || !flowFamily.getFlowFamilyID().equals(existing.getFlowFamilyID()))) {
					
					errors.add(new FlowFamilyAliasCollisionValidationError(alias, callback.getLatestFlowVersion(existing).getName()));
				}
				
				if (callback.checkAliasForSystemCollision(alias)) {
					
					errors.add(new ValidationError(alias, ValidationErrorType.Other, "FlowFamilyAliasAlreadyInUseBySystem"));
				}
			}
		}
	}
	
	private void validateFlowCategory(Flow flow, HttpServletRequest req, List<ValidationError> errors) throws ValidationException {
		
		Integer categoryID = NumberUtils.toInt(req.getParameter("categoryID"));
		
		if (categoryID != null) {
			
			List<Category> categories = flow.getFlowType().getCategories();
			
			if (categories != null) {
				
				Category choosenCategory = null;
				
				for (Category category : categories) {
					
					if (category.getCategoryID().equals(categoryID)) {
						choosenCategory = category;
					}
					
				}
				
				if (choosenCategory == null) {
					
					errors.add(new ValidationError("CategoryNotFound"));
					
					return;
				}
				
				flow.setCategory(choosenCategory);
				
			}
			
		} else {
			
			flow.setCategory(null);
		}
	}
	
	@Override
	protected void appendShowFormData(Flow flow, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {
		
		XMLUtils.append(doc, showTypeElement, "FlowVersions", callback.getAllowedFlowVersions(flow.getFlowFamily()));
		
		List<FlowAdminFragmentExtensionViewProvider> fragmentExtensionProviders = callback.getFragmentExtensionViewProviders();
		
		if (fragmentExtensionProviders != null) {
			
			Map<FlowAdminFragmentExtensionViewProvider, FlowAdminExtensionShowView> showViews = new HashMap<>(fragmentExtensionProviders.size());
			
			for (FlowAdminFragmentExtensionViewProvider fragmentExtensionProvider : fragmentExtensionProviders) {
				
				try {
					FlowAdminExtensionShowView showView = fragmentExtensionProvider.getShowView(callback.getFragmentExtensionViewProviderURL(fragmentExtensionProvider, flow), flow, req, user, uriParser);
					
					if (showView != null && showView.getViewFragment() != null) {
						
						ViewFragment viewFragment = showView.getViewFragment();
						
						Element extensionProviderElement = XMLUtils.appendNewElement(doc, showTypeElement, "ExtensionProvider");
						XMLUtils.appendNewElement(doc, extensionProviderElement, "HTML", viewFragment.getHTML());
						XMLUtils.appendNewElement(doc, extensionProviderElement, "Title", fragmentExtensionProvider.getExtensionViewTitle());
						XMLUtils.appendNewElement(doc, extensionProviderElement, "ID", fragmentExtensionProvider.getModuleID());
						XMLUtils.appendNewElement(doc, extensionProviderElement, "Enabled", showView.isEnabled());
						
						showViews.put(fragmentExtensionProvider, showView);
						
						if (viewFragment instanceof SimpleViewFragment) {
							
							SimpleViewFragment simpleViewFragment = (SimpleViewFragment) viewFragment;
							
							if (simpleViewFragment.getDebugXML() != null) {
								
								Element debugXMLElement = XMLUtils.appendNewElement(doc, extensionProviderElement, "DebugXML");
								debugXMLElement.appendChild(doc.adoptNode(simpleViewFragment.getDebugXML().getDocumentElement()));
							}
						}
					}
					
				} catch (Exception e) {
					
					log.error("Error while getting show view fragment for fragment extension provider " + fragmentExtensionProvider, e);
				}
			}
			
			if (!showViews.isEmpty()) {
				
				req.setAttribute("FragmentExtensionProviderShowViews", showViews);
			}
		}
		
		if (flow.isEnabled() && flow.getSteps() == null && !CollectionUtils.isEmpty(flow.getFlowForms()) && flow.isInternal()) {
			
			XMLUtils.appendNewElement(doc, showTypeElement, "MayNotRemoveFlowFormIfNoSteps");
		}
		
		if (callback.allowSkipOverviewForFlowForms()) {
			
			XMLUtils.appendNewElement(doc, showTypeElement, "AllowSkipOverviewForFlowForms", "true");
		}
		
		if (callback.isBlockForeignIDs()) {
			
			XMLUtils.appendNewElement(doc, showTypeElement, "ForeignIDsBlocked");
		}
		
		if (callback.getRestrictedFlowAdminFlowTest() != null) {
			
			XMLUtils.appendNewElement(doc, showTypeElement, "FlowTestModule", uriParser.getFullContextPath() + callback.getRestrictedFlowAdminFlowTest().getFullAlias());
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	protected SimpleForegroundModuleResponse createShowBeanModuleResponse(Flow flow, Document doc, HttpServletRequest req, User user, URIParser uriParser) {
		
		SimpleForegroundModuleResponse moduleResponse = super.createShowBeanModuleResponse(flow, doc, req, user, uriParser);
		
		ViewFragment viewFragment = (ViewFragment) req.getAttribute("ShowFlowSurveysFragment");
		
		if (viewFragment != null) {
			
			ViewFragmentUtils.appendLinksAndScripts(moduleResponse, viewFragment);
		}
		
		Map<FlowAdminFragmentExtensionViewProvider, FlowAdminExtensionShowView> fragmentExtensionViewShowViews = (Map<FlowAdminFragmentExtensionViewProvider, FlowAdminExtensionShowView>) req.getAttribute("FragmentExtensionProviderShowViews");
		
		if (fragmentExtensionViewShowViews != null) {

			for (Entry<FlowAdminFragmentExtensionViewProvider, FlowAdminExtensionShowView> entry : fragmentExtensionViewShowViews.entrySet()) {

				FlowAdminFragmentExtensionViewProvider fragmentExtensionProvider = entry.getKey();
				FlowAdminExtensionShowView showView = entry.getValue();
				
				String extensionRequestURL = callback.getFragmentExtensionViewProviderURL(fragmentExtensionProvider, flow);
				
				ViewFragmentUtils.appendLinksAndScripts(moduleResponse, showView.getViewFragment(), extensionRequestURL);
			}
		}
		
		return moduleResponse;
	}
	
	@Override
	public ForegroundModuleResponse showBean(Flow bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {
		
		this.checkShowAccess(bean, user, req, uriParser);
		
		log.info("User " + user + " viewing " + this.typeLogName + " " + bean);
		
		Document doc = this.callback.createDocument(req, uriParser, user);
		Element showTypeElement = doc.createElement("Show" + typeElementName);
		doc.getFirstChild().appendChild(showTypeElement);
		
		this.appendBean(bean, showTypeElement, doc, user);
		
		this.appendShowFormData(bean, doc, showTypeElement, user, req, res, uriParser);
		
		if (res.isCommitted()) {
			
			return null;
		}
		
		if (validationErrors != null) {
			XMLUtils.append(doc, showTypeElement, validationErrors);
			showTypeElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}
		
		SimpleForegroundModuleResponse moduleResponse = createShowBeanModuleResponse(bean, doc, req, user, uriParser);
		
		List<Breadcrumb> breadcrumbs = getShowBreadcrumbs(bean, req, user, uriParser);
		
		if (breadcrumbs != null) {
			
			moduleResponse.addBreadcrumbsLast(breadcrumbs);
		}
		
		return moduleResponse;
	}
	
	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {
		
		return callback.list(req, res, user, uriParser, validationErrors);
	}
	
}
