package com.nordicpeak.flowengine.evaluators.sendmailevaluator;

import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "send_mail_evaluators")
@XMLElement
public class SendMailEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = 5501896243489595373L;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "send_mail_evaluator_alternatives", remoteValueColumnName = "alternativeID")
	@WebPopulate(paramName = "alternativeID")
	@XMLElement(fixCase = true, childName = "alternativeID")
	private List<Integer> alternativeIDs;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean enableStatusFiltering;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "send_mail_evaluator_statuses", remoteValueColumnName = "statusID")
	@WebPopulate(paramName = "statusID")
	@XMLElement(fixCase = true, childName = "statusID")
	private List<Integer> statusIDs;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "send_mail_evaluator_recipients", remoteValueColumnName = "emailAddress")
	@XMLElement
	private List<String> emailRecipients;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String emailSubject;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 65536)
	@XMLElement
	private String emailMessage;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean attachPDF;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean attachPDFAttachmentsSeparately;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean usePosterContactAsSender;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean freeTextAlternative;

	public List<Integer> getAlternativeIDs() {

		return alternativeIDs;
	}

	public void setAlternativeIDs(List<Integer> alternativeIDs) {

		this.alternativeIDs = alternativeIDs;
	}

	public String getEmailSubject() {

		return emailSubject;
	}

	public void setEmailSubject(String emailSubject) {

		this.emailSubject = emailSubject;
	}

	public String getEmailMessage() {

		return emailMessage;
	}

	public void setEmailMessage(String emailMessage) {

		this.emailMessage = emailMessage;
	}

	public List<String> getEmailRecipients() {

		return emailRecipients;
	}

	public void setEmailRecipients(List<String> emailRecipients) {

		this.emailRecipients = emailRecipients;
	}

	public boolean attachPDF() {

		return attachPDF;
	}

	public void setAttachPDF(boolean attachPDF) {

		this.attachPDF = attachPDF;
	}

	public boolean isAttachPDFAttachmentsSeparately() {
		
		return attachPDFAttachmentsSeparately;
	}

	public void setAttachPDFAttachmentsSeparately(boolean attachPDFAttachmentsSeparately) {
		
		this.attachPDFAttachmentsSeparately = attachPDFAttachmentsSeparately;
	}

	public boolean isUsePosterContactAsSender() {

		return usePosterContactAsSender;
	}

	public void setUsePosterContactAsSender(boolean usePosterContactAsSender) {

		this.usePosterContactAsSender = usePosterContactAsSender;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		alternativeIDs = xmlParser.getIntegers("AlternativeIDs/alternativeID");

		statusIDs = xmlParser.getIntegers("StatusIDs/statusID");

		emailRecipients = xmlParser.getStrings("emailRecipients/value");

		emailSubject = xmlParser.getString("emailSubject");

		emailMessage = xmlParser.getString("emailMessage");

		attachPDF = xmlParser.getPrimitiveBoolean("attachPDF");
		attachPDFAttachmentsSeparately = xmlParser.getPrimitiveBoolean("attachPDFAttachmentsSeparately");

		enableStatusFiltering = xmlParser.getPrimitiveBoolean("enableStatusFiltering");
		freeTextAlternative = xmlParser.getPrimitiveBoolean("freeTextAlternative");
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	public boolean enableStatusFiltering() {

		return enableStatusFiltering;
	}

	public void setEnableStatusFiltering(boolean enableStatusFiltering) {

		this.enableStatusFiltering = enableStatusFiltering;
	}

	public List<Integer> getStatusIDs() {

		return statusIDs;
	}

	public void setStatusIDs(List<Integer> statusIDs) {

		this.statusIDs = statusIDs;
	}

	public boolean isFreeTextAlternative() {

		return freeTextAlternative;
	}

	public void setFreeTextAlternative(boolean freeTextAlternative) {

		this.freeTextAlternative = freeTextAlternative;
	}
}
