package com.nordicpeak.flowengine.evaluators.sendmailevaluator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;

public class SendMailEvaluatorCRUD extends BaseEvaluatorCRUD<SendMailEvaluator, SendMailEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SendMailEvaluator> POPULATOR = new AnnotatedRequestPopulator<SendMailEvaluator>(SendMailEvaluator.class);

	public SendMailEvaluatorCRUD(Class<SendMailEvaluator> beanClass, AnnotatedDAOWrapper<SendMailEvaluator, Integer> evaluatorDAO, SendMailEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SendMailEvaluator", "send mail evaluator", null, callback);
	}

	@Override
	protected SendMailEvaluator populateFromUpdateRequest(SendMailEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		SendMailEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		String emailRecipientsParam = req.getParameter("emailRecipients");

		if (!StringUtils.isEmpty(emailRecipientsParam)) {

			List<ValidationError> errors = new ArrayList<ValidationError>();

			String[] addresses = emailRecipientsParam.split("[,;:]");

			List<String> recipients = Arrays.asList(addresses);

			List<String> trimmedRecipients = new ArrayList<String>(recipients.size());

			for (String recipient : recipients) {

				recipient = recipient.trim();

				if (!EmailUtils.isValidEmailAddress(recipient) && AttributeTagUtils.getAttributeTags(recipient) == null) {

					errors.add(new ValidationError("emailRecipients", recipient, ValidationErrorType.InvalidFormat));
					
				} else if(!trimmedRecipients.contains(recipient)){

					trimmedRecipients.add(recipient);
				}
			}

			if (!errors.isEmpty()) {

				throw new ValidationException(errors);
			}

			if(!callback.isAllowPosterContactAsSender()) {
				
				evaluator.setUsePosterContactAsSender(false);
			}
			
			evaluator.setEmailRecipients(trimmedRecipients);

		} else {

			throw new ValidationException(new ValidationError("emailRecipients", ValidationErrorType.RequiredField));
		}

		return evaluator;
	}

	@Override
	protected void appendUpdateFormData(SendMailEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		FixedAlternativesQuery query = (FixedAlternativesQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), true);

		List<? extends ImmutableAlternative> alternatives = query.getAlternatives();

		if (alternatives != null) {

			Element alternativesElement = doc.createElement("Alternatives");
			updateTypeElement.appendChild(alternativesElement);

			for (ImmutableAlternative alternative : alternatives) {

				Element alternativeElement = doc.createElement("Alternative");
				alternativesElement.appendChild(alternativeElement);

				XMLUtils.appendNewElement(doc, alternativeElement, "alternativeID", alternative.getAlternativeID());
				XMLUtils.appendNewCDATAElement(doc, alternativeElement, "name", alternative.getName());
			}

			XMLUtils.appendNewCDATAElement(doc, alternativesElement, "FreeTextAlternative", query.getFreeTextAlternative());
		}
		
		XMLUtils.append(doc, updateTypeElement, "Statuses", callback.getFlowAdminModule().getCachedFlow(bean.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID()).getStatuses());

		if (bean.getEmailRecipients() != null) {
			XMLUtils.appendNewElement(doc, updateTypeElement, "EmailRecipients", StringUtils.toCommaSeparatedString(bean.getEmailRecipients()));
		}

		XMLUtils.appendNewElement(doc, updateTypeElement, "AvailableTags", callback.getAvailableTags());
		
		if(callback.isAllowPosterContactAsSender()) {
			
			XMLUtils.appendNewElement(doc, updateTypeElement, "allowPosterContactAsSender", true);
		}
		
	}

}
