<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/CKEditor.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js			
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/sendsmsevaluator.js		
	</xsl:variable>	
	
	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateSendTextFieldSMSEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateSendTextFieldSMSEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="SendTextFieldSMSEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="SendTextFieldSMSEvaluator" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<xsl:variable name="fieldsCount" select="count(TextFields/TextField)" />
	
				<label class="floatleft full">
					<xsl:value-of select="$i18n.fields.title" />
				</label>
		
				<xsl:choose>
					<xsl:when test="$fieldsCount > 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.description" />
						</div>
					
						<xsl:apply-templates select="TextFields/TextField">
							<xsl:with-param name="queryStateEvaluator" select="SendTextFieldSMSEvaluator" />
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$fieldsCount = 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.single.description" />
						</div>
						
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="id" select="'textFieldID'" />
							<xsl:with-param name="name" select="'textFieldID'" />
							<xsl:with-param name="value" select="TextFields/TextField/fieldID" />
						</xsl:call-template>
						
						<p class="floatleft full bold nomargin"><xsl:value-of select="TextFields/TextField/name" /></p>
						
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.QueryHasNoTextFields" />
						</p>
					</xsl:otherwise>
				</xsl:choose>
			
			</div>

			<h2 class="floatleft">
				<xsl:value-of select="$i18n.statusfilter.title" />
			</h2>

			<div class="clearboth floatleft marginbottom">
				<xsl:value-of select="$i18n.statusfilter.description" />
			</div>
			
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'enableStatusFiltering'" />
						<xsl:with-param name="name" select="'enableStatusFiltering'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="onclick" select="'showHideStatusList();'"/>
						<xsl:with-param name="element" select="SendTextFieldSMSEvaluator" />
					</xsl:call-template>
	
					<label for="enableStatusFiltering">
						<xsl:value-of select="$i18n.enableStatusFiltering" />
					</label>
				</div>
			</div>		

			<div id="statuslist">
			
				<div class="clearboth floatleft marginbottom">
					<xsl:value-of select="$i18n.statuslist.description" />
				</div>
	
				<xsl:choose>
					<xsl:when test="Statuses/Status">
						<xsl:apply-templates select="Statuses/Status" />
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.FlowHasNoStatuses" />
						</p>
					</xsl:otherwise>
				</xsl:choose>			
			
			</div>

			<h2 class="floatleft full bigmargintop">
				<xsl:value-of select="$i18n.SMSHeader" />
			</h2>

			<div class="clearboth floatleft">
				<xsl:value-of select="$i18n.SMSHeaderDescription" />
				<xsl:text>&#x20;</xsl:text>
				<xsl:value-of select="AvailableTags" />
			</div>
			
			<label class="floatleft" for="smsMessage">
				<xsl:value-of select="$i18n.SMSMessage" />
			</label>

			<div class="floatleft full marginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'smsMessage'" />
						<xsl:with-param name="name" select="'smsMessage'" />
						<xsl:with-param name="title" select="$i18n.SMSMessage"/>
						<xsl:with-param name="element" select="SendTextFieldSMSEvaluator" />
					</xsl:call-template>
				</div>
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.Submit}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="TextField">

		<xsl:param name="queryStateEvaluator" />

		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id">
						<xsl:value-of select="'textfield'"/>
						<xsl:value-of select="fieldID"/>
					</xsl:with-param>				
					<xsl:with-param name="name" select="'textFieldID'" />
					<xsl:with-param name="value" select="fieldID" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>

				<label for="textfield{fieldID}">
					<xsl:value-of select="name" />
				</label>
			</div>
			
		</div>

	</xsl:template>

	<xsl:template match="Status">

		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id">
						<xsl:value-of select="'status'"/>
						<xsl:value-of select="statusID"/>
					</xsl:with-param>				
					<xsl:with-param name="name" select="'statusID'" />
					<xsl:with-param name="value" select="statusID" />
					<xsl:with-param name="element" select="../../SendTextFieldSMSEvaluator/StatusIDs" />
					<xsl:with-param name="requestparameters" select="../../requestparameters"/>
				</xsl:call-template>

				<label for="status{statusID}">
					<xsl:value-of select="name" />
				</label>
			</div>
		</div>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TextFieldNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.TextFieldNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'smsMessage'">
				<xsl:value-of select="$i18n.smsMessage" />
			</xsl:when>
			<xsl:when test="$fieldName = 'smsRecipients'">
				<xsl:value-of select="$i18n.smsRecipients" />
				<xsl:text>&#x20;(</xsl:text>
				<xsl:value-of select="../displayName" />
				<xsl:text>)</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>