package com.nordicpeak.flowengine.evaluators.setcontactdetailsevaluator;

import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.MutableUser;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.exceptions.UnableToUpdateUserException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.foregroundmodules.userprofile.events.UserUpdatedEvent;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.BeanStringPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;

public class SetContactDetailsEvaluationProviderModule extends BaseEvaluationProviderModule<SetContactDetailsEvaluator> {

	private static final EmailPopulator EMAIL_POPULATOR = new EmailPopulator();
	private static final StringSwedishPhoneNumberPopulator PHONE_POPULATOR = new StringSwedishPhoneNumberPopulator();

	private static final String USER_PHONE_ATTRIBUTE = "phone";
	private static final String USER_MOBILE_PHONE_ATTRIBUTE = "mobilePhone";

	protected SetContactDetailsEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetContactDetailsEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetContactDetailsEvaluator, Integer> evaluatorIDParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetContactDetailsEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetContactDetailsEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetContactDetailsEvaluatorCRUD(SetContactDetailsEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetContactDetailsEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		return null;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 1)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();
		MutableUser user = (MutableUser) flowInstanceManager.getFlowInstance().getPoster();
		List<TextFieldQueryInstance> queryInstances = flowInstanceManager.getQueries(TextFieldQueryInstance.class);

		if (user == null || queryInstances == null) {

			return;
		}

		String newUserEmail = null;
		String newUserPhone = null;
		String newUserMobilePhone = null;

		for (TextFieldQueryInstance queryInstance : queryInstances) {

			if (queryInstance == null || !queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {

				continue;
			}

			ImmutableEvaluatorDescriptor thisEvaluatorDescriptor = null;

			for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {

				if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {

					thisEvaluatorDescriptor = evaluatorDescriptor;
					break;
				}
			}

			if (thisEvaluatorDescriptor == null) {

				continue;
			}

			SetContactDetailsEvaluator evaluator = getEvaluator(thisEvaluatorDescriptor);

			String email = getNewContactDetail(queryInstance, evaluator, EMAIL_POPULATOR, evaluator.getEmailTextFieldID(), user.getEmail());

			if (email != null) {

				newUserEmail = email;
			}

			String phone = getNewContactDetail(queryInstance, evaluator, PHONE_POPULATOR, evaluator.getPhoneTextFieldID(), user.getAttributeHandler().getString(USER_PHONE_ATTRIBUTE));

			if (phone != null) {

				newUserPhone = phone;
			}

			String mobilePhone = getNewContactDetail(queryInstance, evaluator, PHONE_POPULATOR, evaluator.getMobilePhoneTextFieldID(), user.getAttributeHandler().getString(USER_MOBILE_PHONE_ATTRIBUTE));

			if (mobilePhone != null) {

				newUserMobilePhone = mobilePhone;
			}
		}

		if (!ObjectUtils.isNull(newUserEmail, newUserPhone, newUserMobilePhone)) {

			if (newUserEmail != null) {

				user.setEmail(newUserEmail);
			}

			if (newUserPhone != null) {

				user.getAttributeHandler().setAttribute(USER_PHONE_ATTRIBUTE, newUserPhone);
			}

			if (newUserMobilePhone != null) {

				user.getAttributeHandler().setAttribute(USER_MOBILE_PHONE_ATTRIBUTE, newUserMobilePhone);
			}

			try {

				systemInterface.getUserHandler().updateUser(user, false, false, true);

				systemInterface.getEventHandler().sendEvent(User.class, new UserUpdatedEvent(user), EventTarget.ALL);

			} catch (UnableToUpdateUserException e) {

				log.error("Error updating user " + user, e);
			}
		}

	}

	private String getNewContactDetail(TextFieldQueryInstance queryInstance, SetContactDetailsEvaluator evaluator, BeanStringPopulator<String> populator, Integer textFieldID, String oldContactDetail) {

		if (StringUtils.isEmpty(oldContactDetail) || evaluator.isOverwriteExistingContactDetails()) {

			String newContactDetail = queryInstance.getFieldValue(textFieldID);

			if (!StringUtils.isEmpty(newContactDetail) && populator.validateFormat(newContactDetail)) {

				return populator.getValue(newContactDetail);
			}
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetContactDetailsEvaluator evaluator = new SetContactDetailsEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return TextFieldQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetContactDetailsEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetContactDetailsEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		if (sourceQuery instanceof TextFieldQuery) {

			List<TextField> sourceTextFields = ((TextFieldQuery) sourceQuery).getFields();
			List<TextField> copyTextFields = ((TextFieldQuery) copyQuery).getFields();

			if (sourceTextFields != null && copyTextFields != null) {

				int textFieldIndex = 0;
				int fieldsFound = 0;

				while (textFieldIndex < sourceTextFields.size() && fieldsFound < 3) {

					Integer sourceTextFieldID = sourceTextFields.get(textFieldIndex).getTextFieldID();
					Integer copyTextFieldID = copyTextFields.get(textFieldIndex).getTextFieldID();

					if (sourceTextFieldID.equals(evaluator.getEmailTextFieldID())) {

						evaluator.setEmailTextFieldID(copyTextFieldID);
						fieldsFound++;

					} else if (sourceTextFieldID.equals(evaluator.getPhoneTextFieldID())) {

						evaluator.setPhoneTextFieldID(copyTextFieldID);
						fieldsFound++;

					} else if (sourceTextFieldID.equals(evaluator.getMobilePhoneTextFieldID())) {

						evaluator.setMobilePhoneTextFieldID(copyTextFieldID);
						fieldsFound++;
					}

					textFieldIndex++;
				}
			}
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	public SetContactDetailsEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetContactDetailsEvaluator evaluator = new SetContactDetailsEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		if (query instanceof TextFieldQuery && ((TextFieldQuery) query).getTextFieldConversionMap() != null) {

			Map<Integer, Integer> conversionMap = ((TextFieldQuery) query).getTextFieldConversionMap();

			if (evaluator.getEmailTextFieldID() != null) {

				evaluator.setEmailTextFieldID(conversionMap.get(evaluator.getEmailTextFieldID()));
			}

			if (evaluator.getPhoneTextFieldID() != null) {

				evaluator.setPhoneTextFieldID(conversionMap.get(evaluator.getPhoneTextFieldID()));
			}

			if (evaluator.getMobilePhoneTextFieldID() != null) {

				evaluator.setMobilePhoneTextFieldID(conversionMap.get(evaluator.getMobilePhoneTextFieldID()));
			}
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	protected Class<SetContactDetailsEvaluator> getEvaluatorClass() {

		return SetContactDetailsEvaluator.class;
	}

	protected AnnotatedDAO<SetContactDetailsEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetContactDetailsEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	protected SetContactDetailsEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {

		return evaluatorDAO.get(new HighLevelQuery<SetContactDetailsEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetContactDetailsEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetContactDetailsEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<SetContactDetailsEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

}
