package com.nordicpeak.flowengine.evaluators.settableweightevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.queries.basetablequery.interfaces.TableColumn;
import com.nordicpeak.flowengine.queries.basetablequery.interfaces.TableQuery;

public class SetTableWeightEvaluatorCRUD extends BaseEvaluatorCRUD<SetTableWeightEvaluator, SetTableWeightEvaluationProviderModule> {
	
	
	private static final AnnotatedRequestPopulator<SetTableWeightEvaluator> POPULATOR = new AnnotatedRequestPopulator<SetTableWeightEvaluator>(SetTableWeightEvaluator.class);
	
	public SetTableWeightEvaluatorCRUD(Class<SetTableWeightEvaluator> beanClass, AnnotatedDAOWrapper<SetTableWeightEvaluator, Integer> evaluatorDAO, SetTableWeightEvaluationProviderModule callback) {
		
		super(beanClass, evaluatorDAO, POPULATOR, "SetTableWeightEvaluator", "set text field weight evaluator", null, callback);
	}
	
	@Override
	protected void appendUpdateFormData(SetTableWeightEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		
		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		Query query = callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);
		
		List<? extends TableColumn> columns = ((TableQuery) query).getColumns();
		
		if (columns != null) {
			
			Element fieldsElement = doc.createElement("Columns");
			updateTypeElement.appendChild(fieldsElement);
			
			for (TableColumn column : columns) {
				
				if (column.isSummarise()) {
					
					Element fieldElement = doc.createElement("Column");
					fieldsElement.appendChild(fieldElement);
					
					XMLUtils.appendNewElement(doc, fieldElement, "columnID", column.getColumnID());
					XMLUtils.appendNewCDATAElement(doc, fieldElement, "name", column.getLabel());
				}
			}
		}
	}
	
	@Override
	protected SetTableWeightEvaluator populateFromUpdateRequest(SetTableWeightEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		SetTableWeightEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		if (evaluator.getEvaluationMode() == EvaluationMode.USE_INTERVALL) {
			
			if (bean.getFromValue() != null && bean.getToValue() != null && bean.getFromValue() > bean.getToValue()) {
				
				errors.add(new ValidationError("FromValueBiggerThanToValue"));
			}
		}
		
		Integer columnID = bean.getColumnID();
		
		boolean found = false;

		Query query = callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<? extends TableColumn> columns = ((TableQuery) query).getColumns();

		if (columns != null) {
			for (TableColumn column : columns) {

				if (column.getColumnID().equals(columnID)) {
					found = true;
					break;
				}
			}
		}
		
		if (!found) {
			
			errors.add(new ValidationError("ColumnNotFound"));
		}
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
		
		return evaluator;
	}
	
}
