package com.nordicpeak.flowengine.targetgroupflowbrowser;

import java.sql.SQLException;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.IntegerBasedCRUD;
import se.unlogic.standardutils.dao.CRUDDAO;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.FlowFamily;

public class TargetGroupCRUD extends IntegerBasedCRUD<TargetGroup, TargetGroupAdminModule> {

	public TargetGroupCRUD(CRUDDAO<TargetGroup, Integer> crudDAO, TargetGroupAdminModule callback) {

		super(crudDAO, new AnnotatedRequestPopulator<TargetGroup>(TargetGroup.class), "TargetGroup", "target group", "/", callback);
	
		setRequirePostForDelete(true);
	}

	@Override
	protected void checkAddAccess(User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		if (!AccessUtils.checkAccess(user, callback.getAccessInterface())) {

			throw new AccessDeniedException("Add target group access denied, user does not have administrative access");
		}
	}

	@Override
	protected void checkUpdateAccess(TargetGroup bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		if (!AccessUtils.checkAccess(user, callback.getAccessInterface())) {

			throw new AccessDeniedException("Update target group access denied, user does not have administrative access");
		}
	}

	@Override
	protected void checkDeleteAccess(TargetGroup bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		if (!AccessUtils.checkAccess(user, callback.getAccessInterface())) {

			throw new AccessDeniedException("Delete target group access denied, user does not have access administrative access");
		}
	}

	@Override
	protected void checkListAccess(User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		if (!AccessUtils.checkAccess(user, callback.getAccessInterface())) {

			throw new AccessDeniedException("List target group access denied, user does not have access administrative access");
		}
	}

	@Override
	protected void validateAddPopulation(TargetGroup bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		validatePopulation(bean, req, user, uriParser);
	}

	@Override
	protected void validateUpdatePopulation(TargetGroup bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		validatePopulation(bean, req, user, uriParser);
	}

	protected void validatePopulation(TargetGroup bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		if (bean.getFlowFamilyIDs() != null) {

			for (Iterator<Integer> iterator = bean.getFlowFamilyIDs().iterator(); iterator.hasNext();) {

				Integer flowFamilyID = iterator.next();

				if (callback.getFlowAdminModule().getFlowFamily(flowFamilyID) == null) {

					iterator.remove();
				}

			}

		}

	}

	@Override
	protected void appendUpdateFormData(TargetGroup bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.appendNewCDATAElement(doc, updateTypeElement, "FlowAdminModuleAlias", callback.getFlowAdminModule().getFullAlias());

		if(bean.getFlowFamilyIDs() != null) {
			
			FlowAdminModule flowAdminModule = callback.getFlowAdminModule();
			
			for(Integer flowFamilyID : bean.getFlowFamilyIDs()) {
				
				FlowFamily flowFamily = flowAdminModule.getFlowFamily(flowFamilyID);
				
				if(flowFamily != null) {
				
					XMLUtils.append(doc, updateTypeElement, flowAdminModule.getLatestFlowVersion(flowFamily));
				}
			}
			
		}
		
	}
	
	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.appendNewCDATAElement(doc, addTypeElement, "FlowAdminModuleAlias", callback.getFlowAdminModule().getFullAlias());
	}

	@Override
	protected ForegroundModuleResponse beanAdded(TargetGroup bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.cacheTargetGroups();
		
		return super.beanAdded(bean, req, res, user, uriParser);
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(TargetGroup bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.cacheTargetGroups();
		
		return super.beanUpdated(bean, req, res, user, uriParser);
	}

	@Override
	protected ForegroundModuleResponse beanDeleted(TargetGroup bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.cacheTargetGroups();
		
		return super.beanDeleted(bean, req, res, user, uriParser);
	}

}
