package com.nordicpeak.flowengine.targetgroupflowbrowser;

import java.io.IOException;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EnumDropDownSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.beans.FlowType;

public class TargetGroupFlowBrowserModule extends FlowBrowserModule {

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Clear filter function", description = "Specifies which clear filter function should be used", required = true)
	protected ClearFilterFunction clearFilterFunction = ClearFilterFunction.BOTH;
	
	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Default view", description = "Specifies which view should be shown as default", required = true)
	protected View defaultView = View.GRID;
	
	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Prioritized filter", description = "Specifies which filter should be shown first", required = true)
	protected Filter prioritizedFilter = Filter.TARGETGROUP;

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Default sorting", description = "Specifies default sorting", required = true)
	protected Sorting defaultSorting = Sorting.FLOWTYPE;
	
	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Expandable filter state", description = "Specifies which visibility state filters should have in desktop", required = true)
	protected ExpandableState expandableFilterState = ExpandableState.DISABLED;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Show flowtype icon", description = "Controls whether icon should shown on flowtype when listing flows")
	protected boolean showFlowTypeIcon = false;
	
	@InstanceManagerDependency
	private TargetGroupAdminModule targetGroupAdminModule;

	
	@Override
	protected void appendAdditionalListFlowsXML(Document doc, Element element) {

		if (targetGroupAdminModule != null) {

			XMLUtils.append(doc, element, targetGroupAdminModule.getCachedTargetGroups());

		}

		XMLUtils.appendNewElement(doc, element, "DefaultView", defaultView);
		XMLUtils.appendNewElement(doc, element, "PrioritizedFilter", prioritizedFilter);
		XMLUtils.appendNewElement(doc, element, "DefaultSorting", defaultSorting);
		XMLUtils.appendNewElement(doc, element, "ExpandableFilterState", expandableFilterState);
		XMLUtils.appendNewElement(doc, element, "ClearFilterFunction", clearFilterFunction);
		
		if(showFlowTypeIcon) {
			XMLUtils.appendNewElement(doc, element, "ShowFlowTypeIcon", true);
		}
		
	}
	
	@WebPublic(alias = "category")
	public ForegroundModuleResponse redirectToCategory(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, IOException, AccessDeniedException  {
		
		FlowType flowType = null;
		
		if(uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (flowType = flowAdminModule.getFlowType(uriParser.getInt(2))) != null) {
			
			if(AccessUtils.checkAccess(user, flowType.getUserAccessInterface())){
				
				redirectToMethod(req, res, "#category-" + flowType.getFlowTypeID());
				
			}else {
				
				throw new AccessDeniedException("User " + user + " does not have access to flow type " + flowType);
			}
			
			return null;
		}
		
		throw new URINotFoundException(uriParser);
	}
}
