package com.nordicpeak.flowengine.evaluators.setweightbyexistingtreservacaseevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;
import com.nordicpeak.flowengine.integration.treserva.beans.CaseData;

@Table(name = "set_weight_by_existing_treserva_case_evaluators")
@XMLElement
public class SetWeightByExistingTreservaCaseEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer caseFoundWeight;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer caseNotFoundWeight;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer apiErrorWeight;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getCaseFoundWeight() {

		return caseFoundWeight;
	}

	public void setCaseFoundWeight(Integer caseFoundWeight) {

		this.caseFoundWeight = caseFoundWeight;
	}

	public Integer getCaseNotFoundWeight() {

		return caseNotFoundWeight;
	}

	public void setCaseNotFoundWeight(Integer caseNotFoundWeight) {

		this.caseNotFoundWeight = caseNotFoundWeight;
	}

	public Integer getApiErrorWeight() {

		return apiErrorWeight;
	}

	public void setApiErrorWeight(Integer apiErrorWeight) {

		this.apiErrorWeight = apiErrorWeight;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		caseFoundWeight = XMLValidationUtils.validateParameter("caseFoundWeight", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		caseNotFoundWeight = XMLValidationUtils.validateParameter("caseNotFoundWeight", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		apiErrorWeight = XMLValidationUtils.validateParameter("apiErrorWeight", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public void setAttribute(MutableAttributeHandler attributeHandler, CaseData caseData) {

		attributeHandler.setAttribute(attributeName + ".number", caseData.getNumber());
		attributeHandler.setAttribute(attributeName + ".manager", caseData.getManager());
		attributeHandler.setAttribute(attributeName + ".fromDate", caseData.getFormattedFromDate());
	}

}
