package com.nordicpeak.flowengine.integration.treserva.beans;

import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;

@Table(name = "treserva_integration_settings")
@XMLElement
public class IntegrationSettings extends BaseIntegrationSettings {

	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private IntegrationEndpoint endpoint;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<IntegrationParameter> parameters;

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String personNr;

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String stadsDelId;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean commentEnabled;

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "commentEnabled")
	@XMLElement
	private String commentName;

	@DAOManaged
	@WebPopulate(maxLength = 5000)
	@RequiredIfSet(paramNames = "commentEnabled")
	@XMLElement
	private String commentValue;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setStadsDelIdFromAttribute;

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "setStadsDelIdFromAttribute")
	@XMLElement
	private String stadsDelIdAttributeName;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_valid_case_data_types", remoteValueColumnName = "type")
	@WebPopulate
	@SplitOnLineBreak
	@XMLElement(childName = "validCaseDataType")
	private List<String> validCaseDataTypes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean autoAssignManager;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean duplicateCheckEnabled;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean decisionModuleEnabled;

	@DAOManaged
	@WebPopulate(maxLength = 1024)
	@RequiredIfSet(paramNames = "decisionModuleEnabled")
	@XMLElement
	private String decisionMessageTemplate;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExistingApplicationErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_email_recipients_existing_application_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendExistingApplicationErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> existingApplicationErrorEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendProtectedPersonErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_email_recipients_protected_person_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendProtectedPersonErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> protectedPersonErrorEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendUnknownErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_email_recipients_unknown_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendUnknownErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> unknownErrorEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendMaxRetriesExceededEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_email_recipients_max_retries_exceeded", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendMaxRetriesExceededEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> maxRetriesExceededEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendNoManagerToAutoAssignEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "treserva_integration_email_recipients_no_manager_to_auto_assign", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendNoManagerToAutoAssignEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> noManagerToAutoAssignEmailAddresses;

	@Override
	public boolean isPopulated() {

		return isEnabled() || endpoint != null || parameters != null || personNr != null || stadsDelId != null;
	}

	public IntegrationEndpoint getEndpoint() {

		return endpoint;
	}

	public void setEndpoint(IntegrationEndpoint endpoint) {

		this.endpoint = endpoint;
	}

	public List<IntegrationParameter> getParameters() {

		return parameters;
	}

	public void setParameters(List<IntegrationParameter> parameters) {

		this.parameters = parameters;
	}

	public String getPersonNr() {

		return personNr;
	}

	public void setPersonNr(String personNr) {

		this.personNr = personNr;
	}

	public String getStadsDelId() {

		return stadsDelId;
	}

	public void setStadsDelId(String stadsDelId) {

		this.stadsDelId = stadsDelId;
	}

	public boolean isCommentEnabled() {

		return commentEnabled;
	}

	public void setCommentEnabled(boolean commentEnabled) {

		this.commentEnabled = commentEnabled;
	}

	public String getCommentName() {

		return commentName;
	}

	public void setCommentName(String commentName) {

		this.commentName = commentName;
	}

	public String getCommentValue() {

		return commentValue;
	}

	public void setCommentValue(String commentValue) {

		this.commentValue = commentValue;
	}

	public boolean isSetStadsDelIdFromAttribute() {

		return setStadsDelIdFromAttribute;
	}

	public void setSetStadsDelIdFromAttribute(boolean setStadsDelIdFromAttribute) {

		this.setStadsDelIdFromAttribute = setStadsDelIdFromAttribute;
	}

	public String getStadsDelIdAttributeName() {

		return stadsDelIdAttributeName;
	}

	public void setStadsDelIdAttributeName(String stadsDelIdAttributeName) {

		this.stadsDelIdAttributeName = stadsDelIdAttributeName;
	}

	public List<String> getValidCaseDataTypes() {

		return validCaseDataTypes;
	}

	public void setValidCaseDataTypes(List<String> validCaseDataTypes) {

		this.validCaseDataTypes = validCaseDataTypes;
	}

	public boolean isAutoAssignManager() {

		return autoAssignManager;
	}

	public void setAutoAssignManager(boolean autoAssignManager) {

		this.autoAssignManager = autoAssignManager;
	}

	public boolean isDuplicateCheckEnabled() {

		return duplicateCheckEnabled;
	}

	public void setDuplicateCheckEnabled(boolean duplicateCheckEnabled) {

		this.duplicateCheckEnabled = duplicateCheckEnabled;
	}

	public boolean isDecisionModuleEnabled() {

		return decisionModuleEnabled;
	}

	public void setDecisionModuleEnabled(boolean decisionModuleEnabled) {

		this.decisionModuleEnabled = decisionModuleEnabled;
	}

	public String getDecisionMessageTemplate() {

		return decisionMessageTemplate;
	}

	public void setDecisionMessageTemplate(String decisionMessageTemplate) {

		this.decisionMessageTemplate = decisionMessageTemplate;
	}

	public boolean isSendExistingApplicationErrorEmail() {

		return sendExistingApplicationErrorEmail;
	}

	public void setSendExistingApplicationErrorEmail(boolean sendExistingApplicationErrorEmail) {

		this.sendExistingApplicationErrorEmail = sendExistingApplicationErrorEmail;
	}

	public List<String> getExistingApplicationErrorEmailAddresses() {

		return existingApplicationErrorEmailAddresses;
	}

	public void setExistingApplicationErrorEmailAddresses(List<String> existingApplicationErrorEmailAddresses) {

		this.existingApplicationErrorEmailAddresses = existingApplicationErrorEmailAddresses;
	}

	public boolean isSendProtectedPersonErrorEmail() {

		return sendProtectedPersonErrorEmail;
	}

	public void setSendProtectedPersonErrorEmail(boolean sendProtectedPersonErrorEmail) {

		this.sendProtectedPersonErrorEmail = sendProtectedPersonErrorEmail;
	}

	public List<String> getProtectedPersonErrorEmailAddresses() {

		return protectedPersonErrorEmailAddresses;
	}

	public void setProtectedPersonErrorEmailAddresses(List<String> protectedPersonErrorEmailAddresses) {

		this.protectedPersonErrorEmailAddresses = protectedPersonErrorEmailAddresses;
	}

	public boolean isSendUnknownErrorEmail() {

		return sendUnknownErrorEmail;
	}

	public void setSendUnknownErrorEmail(boolean sendUnknownErrorEmail) {

		this.sendUnknownErrorEmail = sendUnknownErrorEmail;
	}

	public List<String> getUnknownErrorEmailAddresses() {

		return unknownErrorEmailAddresses;
	}

	public void setUnknownErrorEmailAddresses(List<String> unknownErrorEmailAddresses) {

		this.unknownErrorEmailAddresses = unknownErrorEmailAddresses;
	}

	public boolean isSendMaxRetriesExceededEmail() {

		return sendMaxRetriesExceededEmail;
	}

	public void setSendMaxRetriesExceededEmail(boolean sendMaxRetriesExceededEmail) {

		this.sendMaxRetriesExceededEmail = sendMaxRetriesExceededEmail;
	}

	public List<String> getMaxRetriesExceededEmailAddresses() {

		return maxRetriesExceededEmailAddresses;
	}

	public void setMaxRetriesExceededEmailAddresses(List<String> maxRetriesExceededEmailAddresses) {

		this.maxRetriesExceededEmailAddresses = maxRetriesExceededEmailAddresses;
	}

	public boolean isSendNoManagerToAutoAssignEmail() {

		return sendNoManagerToAutoAssignEmail;
	}

	public void setSendNoManagerToAutoAssignEmail(boolean sendNoManagerToAutoAssignEmail) {

		this.sendNoManagerToAutoAssignEmail = sendNoManagerToAutoAssignEmail;
	}

	public List<String> getNoManagerToAutoAssignEmailAddresses() {

		return noManagerToAutoAssignEmailAddresses;
	}

	public void setNoManagerToAutoAssignEmailAddresses(List<String> noManagerToAutoAssignEmailAddresses) {

		this.noManagerToAutoAssignEmailAddresses = noManagerToAutoAssignEmailAddresses;
	}
}
