package com.nordicpeak.flowengine.evaluators.settextfieldvalueparameter;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.enums.ModificationAction;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.urlparameterfilter.URLParameterFilterModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;

public class SetTextFieldValueParameterEvaluationProviderModule extends BaseEvaluationProviderModule<SetTextFieldValueParameterEvaluator> {
	
	protected SetTextFieldValueParameterEvaluatorCRUD evaluatorCRUD;
	
	protected AnnotatedDAO<SetTextFieldValueParameterEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<SetTextFieldValueParameterEvaluator, Integer> evaluatorIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetTextFieldValueParameterEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetTextFieldValueParameterEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new SetTextFieldValueParameterEvaluatorCRUD(SetTextFieldValueParameterEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetTextFieldValueParameterEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (evaluator.isInitialized() || evaluator.getParamName() == null || callback.getRequestMetadata().isManager()) {
			
			return null;
		}
		
		evaluator.setInitialized(true);
		
		if (!(queryInstance instanceof TextFieldQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " is not of the type " + TextFieldQueryInstance.class.getName());
			
			return null;
		}
		
		TextFieldQueryInstance textFieldQueryInstance = ((TextFieldQueryInstance) queryInstance);
		
		String oldValue = textFieldQueryInstance.getFieldValue(evaluator.getTextFieldID());
		
		String paramValue = getParamValue(evaluator, callback);
		
		if(paramValue != null){
			
			((TextFieldQueryInstance) queryInstance).setFieldValue(evaluator.getTextFieldID(), paramValue, attributeHandler);
			
		}else if(evaluator.getNoParamValueFoundMessage() != null){
			
			((TextFieldQueryInstance) queryInstance).setFieldValue(evaluator.getTextFieldID(), evaluator.getNoParamValueFoundMessage(), attributeHandler);
		}
		
		if (!ObjectUtils.compare(oldValue, textFieldQueryInstance.getFieldValue(evaluator.getTextFieldID())) && queryInstance.getQueryInstanceDescriptor().getQueryState() != QueryState.HIDDEN) {
			
			return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
		}
		
		return null;
	}
	
	private String getParamValue(SetTextFieldValueParameterEvaluator evaluator, EvaluationCallback callback) {

		HttpSession session = callback.getRequest().getSession(false);
		
		if(session != null){
			
			Object object = session.getAttribute(URLParameterFilterModule.PARAM_MAP_SESSION_ATTRIBUTE);
			
			if(object != null && object instanceof Map){
				
				@SuppressWarnings("unchecked")
				Map<String,String> paramMap = (Map<String, String>) object;
				
				return paramMap.get(evaluator.getParamName());
			}
		}
		
		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		SetTextFieldValueParameterEvaluator evaluator = new SetTextFieldValueParameterEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return TextFieldQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		SetTextFieldValueParameterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetTextFieldValueParameterEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		List<TextField> sourceTextFields = ((TextFieldQuery) sourceQuery).getFields();
		List<TextField> copyTextFields = ((TextFieldQuery) copyQuery).getFields();
		
		if (sourceTextFields != null && copyTextFields != null) {
			
			int textFieldIndex = 0;
			
			while (textFieldIndex < sourceTextFields.size()) {
				
				if (sourceTextFields.get(textFieldIndex).getTextFieldID().equals(evaluator.getTextFieldID())) {
					
					evaluator.setTextFieldID(copyTextFields.get(textFieldIndex).getTextFieldID());
					
					break;
				}
				
				textFieldIndex++;
			}
			
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	protected Class<SetTextFieldValueParameterEvaluator> getEvaluatorClass() {
		
		return SetTextFieldValueParameterEvaluator.class;
	}
	
	protected AnnotatedDAO<SetTextFieldValueParameterEvaluator> getEvaluatorDAO() {
		
		return evaluatorDAO;
	}
	
	protected QueryParameterFactory<SetTextFieldValueParameterEvaluator, Integer> getEvaluatorIDParameterFactory() {
		
		return evaluatorIDParamFactory;
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		SetTextFieldValueParameterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetTextFieldValueParameterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<SetTextFieldValueParameterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	public QueryHandler getQueryHandler() {
		
		return this.queryHandler;
	}
	
	@Override
	public SetTextFieldValueParameterEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		SetTextFieldValueParameterEvaluator evaluator = getEvaluatorClass().newInstance();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		if (evaluator.getTextFieldID() != null && query instanceof TextFieldQuery && ((TextFieldQuery) query).getTextFieldConversionMap() != null) {
			
			Map<Integer, Integer> conversionMap = ((TextFieldQuery) query).getTextFieldConversionMap();
			
			Integer newTextFieldID = conversionMap.get(evaluator.getTextFieldID());
			
			evaluator.setTextFieldID(newTextFieldID);
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
