<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

 	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/unsecuremultisignquery.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/queryID" />
					</xsl:call-template>
					
				</div>
				
				<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/description">
					<span class="italic">
						<xsl:value-of select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<fieldset>
					
					<div class="split">
						<strong class="block"><xsl:value-of select="$i18n.Firstname" /></strong>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/firstname" />
					</div>
					
					<div class="split odd">
						<strong class="block"><xsl:value-of select="$i18n.Lastname" /></strong>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/lastname" />
					</div>					
					
					<div class="split">
						<strong class="block"><xsl:value-of select="$i18n.Email" /></strong>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/email" />
					</div>
					
					<div class="split odd">
						<strong class="block"><xsl:value-of select="$i18n.MobilePhone" /></strong>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/mobilePhone" />
					</div>								
					
				</fieldset>
				
				
			</article>
			
		</div>
		
	</xsl:template>	
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
			
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
						<xsl:variable name="validationErrors" select="ValidationErrors/validationError[messageKey and messageKey != 'EmailVerificationMismatch'  and messageKey != 'PhoneVerificationMismatch']" />
						
						<xsl:choose>
							<xsl:when test="$validationErrors">
							
								<xsl:apply-templates select="$validationErrors" />
								
							</xsl:when>
							<xsl:otherwise>
							
								<span/>
								
							</xsl:otherwise>
						</xsl:choose>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
		
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/helpText">		
						<xsl:apply-templates select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/helpText" />
					</xsl:if>
				
				</div>
				
				<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/description" disable-output-escaping="yes" />
					</span>	
				</xsl:if>
				
				<fieldset>
					
					<div class="split">
					
						<xsl:variable name="fieldName" select="concat($shortQueryID, '_firstname')" />
					
						<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
							<xsl:attribute name="class">
								<xsl:text>split invalid input-error</xsl:text>
							</xsl:attribute>
						</xsl:if>
					
						<label for="{$fieldName}" class="required"><xsl:value-of select="$i18n.Firstname" /></label>
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$fieldName" />
							<xsl:with-param name="name" select="$fieldName" />
							<xsl:with-param name="title" select="$i18n.Firstname"/>
							<xsl:with-param name="size" select="50"/>
							<xsl:with-param name="value">
								<xsl:value-of select="UnsecureMultiSignQueryInstance/firstname" />
							</xsl:with-param>
							<xsl:with-param name="disabled">
								<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/disabled = 'true' or Locked">
									<xsl:text>true</xsl:text>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						
						<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
						
					</div>
					
					<div class="split odd">
					
						<xsl:variable name="fieldName" select="concat($shortQueryID, '_lastname')" />
						
						<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
							<xsl:attribute name="class">
								<xsl:text>split odd invalid input-error</xsl:text>
							</xsl:attribute>
						</xsl:if>							
					
						<label for="{$fieldName}" class="required"><xsl:value-of select="$i18n.Lastname" /></label>
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$fieldName" />
							<xsl:with-param name="name" select="$fieldName" />
							<xsl:with-param name="title" select="$i18n.Lastname"/>
							<xsl:with-param name="size" select="50"/>
							<xsl:with-param name="value">
								<xsl:value-of select="UnsecureMultiSignQueryInstance/lastname" />
							</xsl:with-param>
							<xsl:with-param name="disabled">
								<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/disabled = 'true' or Locked">
									<xsl:text>true</xsl:text>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						
						<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
						
					</div>
					
					<div class="split">
						
						<xsl:variable name="fieldName" select="concat($shortQueryID, '_email')" />
					
						<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
							<xsl:attribute name="class">
								<xsl:text>split invalid input-error</xsl:text>
							</xsl:attribute>
						</xsl:if>
					
						<label for="{$fieldName}"><xsl:value-of select="$i18n.Email" /></label>
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$fieldName" />
							<xsl:with-param name="name" select="$fieldName" />
							<xsl:with-param name="title" select="$i18n.Email"/>
							<xsl:with-param name="size" select="50"/>
							<xsl:with-param name="value">
								<xsl:value-of select="UnsecureMultiSignQueryInstance/email" />
							</xsl:with-param>
							<xsl:with-param name="disabled">
								<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/disabled = 'true' or Locked">
									<xsl:text>true</xsl:text>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						
						<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
						
					</div>
					
					<xsl:variable name="confirm" select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/requireContactInfoVerification = 'true' and not(UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/disabled = 'true' or Locked)" />
					
					<xsl:if test="$confirm">
						
						<div class="split odd">
							
							<xsl:variable name="fieldName" select="concat($shortQueryID, '_email2')" />
							
							<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
								<xsl:attribute name="class">
									<xsl:text>split odd invalid input-error</xsl:text>
								</xsl:attribute>
							</xsl:if>
						
							<label for="{$fieldName}">
								<xsl:value-of select="$i18n.Email" />
								<xsl:text>&#160;</xsl:text>
								<xsl:value-of select="$i18n.Confirmation"/>
							</label>
							<xsl:call-template name="createTextField">
								<xsl:with-param name="id" select="$fieldName" />
								<xsl:with-param name="name" select="$fieldName" />
								<xsl:with-param name="class" select="'disablepaste'" />
								<xsl:with-param name="title" select="$i18n.Email"/>
								<xsl:with-param name="size" select="50"/>
								<xsl:with-param name="value">
									<xsl:value-of select="UnsecureMultiSignQueryInstance/email" />
								</xsl:with-param>
							</xsl:call-template>
							
							<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
							
						</div>
						
					</xsl:if>
					
					<div>
						
						<xsl:variable name="fieldName" select="concat($shortQueryID, '_mobilePhone')" />
					
						<xsl:attribute name="class">
							<xsl:text>split</xsl:text>
							
							<xsl:if test="not($confirm)">
								<xsl:text> odd</xsl:text>
							</xsl:if>
							
							<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
								<xsl:text> invalid input-error</xsl:text>
							</xsl:if>
						</xsl:attribute>
					
						<label for="{$fieldName}"><xsl:value-of select="$i18n.MobilePhone" /></label>
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$fieldName" />
							<xsl:with-param name="name" select="$fieldName" />
							<xsl:with-param name="title" select="$i18n.MobilePhone"/>
							<xsl:with-param name="size" select="50"/>
							<xsl:with-param name="value">
								<xsl:value-of select="UnsecureMultiSignQueryInstance/mobilePhone" />
							</xsl:with-param>
							<xsl:with-param name="disabled">
								<xsl:if test="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/disabled = 'true' or Locked">
									<xsl:text>true</xsl:text>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						
						<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
						
					</div>
					
					<xsl:if test="$confirm">
						
						<div class="split odd">
							
							<xsl:variable name="fieldName" select="concat($shortQueryID, '_mobilePhone2')" />
							
							<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
								<xsl:attribute name="class">
									<xsl:text>split odd invalid input-error</xsl:text>
								</xsl:attribute>
							</xsl:if>
						
							<label for="{$fieldName}">
								<xsl:value-of select="$i18n.MobilePhone" />
								<xsl:text>&#160;</xsl:text>
								<xsl:value-of select="$i18n.Confirmation"/>
							</label>
							<xsl:call-template name="createTextField">
								<xsl:with-param name="id" select="$fieldName" />
								<xsl:with-param name="name" select="$fieldName" />
								<xsl:with-param name="title" select="$i18n.MobilePhone"/>
								<xsl:with-param name="class" select="'disablepaste'" />
								<xsl:with-param name="size" select="50"/>
								<xsl:with-param name="value">
									<xsl:value-of select="UnsecureMultiSignQueryInstance/mobilePhone" />
								</xsl:with-param>
							</xsl:call-template>
							
							<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
							
						</div>
					
					</xsl:if>
					
				</fieldset>
				
			</article>
		
		</div>
		
		<script type="text/javascript">$(function(){initUnsecureMultiSignQuery('<xsl:value-of select="UnsecureMultiSignQueryInstance/UnsecureMultiSignQuery/queryID" />');});</script>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'EmailVerificationMismatch']">
		
		<i data-icon-after="!" title="{$i18n.Error.EmailVerificationMismatch}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'PhoneVerificationMismatch']">
		
		<i data-icon-after="!" title="{$i18n.Error.PhoneVerificationMismatch}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'Required']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.Required"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TooLongFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part2"/>
				<xsl:value-of select="maxLength"/>
				<xsl:value-of select="$i18n.TooLongFieldContent.part3"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!" title="{$i18n.InvalidFormat}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoContactChannelSpecified']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.NoContactChannelSpecified"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>
		
</xsl:stylesheet>