package com.nordicpeak.flowengine.evaluators.useraddressquerystateevaluator;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

@Table(name = "user_address_query_state_evaluator_addresses")
@XMLElement
public class Address extends GeneratedElementable implements Serializable, XMLParserPopulateable {

	private static final long serialVersionUID = -9138314881826903923L;

	@DAOManaged(columnName = "evaluatorID")
	@ManyToOne
	@Key
	private UserAddressQueryStateEvaluator evaluator;

	@DAOManaged(autoGenerated = true)
	@XMLElement
	@Key
	private Integer addressID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private String street;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private String postalAddress;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		street = XMLValidationUtils.validateParameter("street", xmlParser, true, StringPopulator.getPopulator(), errors);
		postalAddress = XMLValidationUtils.validateParameter("postalAddress", xmlParser, true, StringPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public UserAddressQueryStateEvaluator getEvaluator() {

		return evaluator;
	}

	public void setEvaluator(UserAddressQueryStateEvaluator evaluator) {

		this.evaluator = evaluator;
	}

	public Integer getAddressID() {

		return addressID;
	}

	public void setAddressID(Integer addressID) {

		this.addressID = addressID;
	}

	public String getStreet() {

		return street;
	}

	public void setStreet(String street) {

		this.street = street;
	}

	public String getPostalAddress() {

		return postalAddress;
	}

	public void setPostalAddress(String postalAddress) {

		this.postalAddress = postalAddress;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Address other = (Address) obj;
		if (postalAddress == null) {
			if (other.postalAddress != null)
				return false;
		} else if (!postalAddress.equalsIgnoreCase(other.postalAddress))
			return false;
		if (street == null) {
			if (other.street != null)
				return false;
		} else if (!street.equalsIgnoreCase(other.street))
			return false;
		return true;
	}

	@Override
	public String toString() {

		return street + ", " + postalAddress;
	}

}
