<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>
	
	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/useraddressquerystateevaluator.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/useraddressquerystateevaluator.css?v=1
	</xsl:variable>

	<xsl:template match="Document">

		<div class="contentitem">
			<xsl:apply-templates select="UpdateUserAddressQueryStateEvaluator" />
		</div>

	</xsl:template>

	<xsl:template name="createCustomFormFields">
		
		<xsl:param name="queryStateEvaluator" />
		
		<div class="floatleft full">
	
			<label>
				<xsl:value-of select="$i18n.Evaluator.Invert.title" />
			</label>
			
			<br/>
			
			<xsl:value-of select="$i18n.Evaluator.Invert.description" />
		</div>
		
		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id" select="'not-inverted'" />
					<xsl:with-param name="name" select="'invert'" />
					<xsl:with-param name="value" select="'false'" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>

				<label for="not-inverted">
					<xsl:value-of select="$i18n.Evaluator.NotInverted" />
				</label>
			</div>
		</div>
		
		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id" select="'invert'" />
					<xsl:with-param name="name" select="'invert'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>

				<label for="invert">
					<xsl:value-of select="$i18n.Evaluator.Inverted" />
				</label>
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			
			<label class="floatleft full" for="geometry">
				<xsl:value-of select="$i18n.Evaluator.addresses" />
			</label>
			
			<div id="address-list" class="floatleft full margintop marginbottom" data-url="{/Document/requestinfo/currentURI}/{/Document/module/alias}/searchforaddress" data-select-by-street-number="{SelectByStreetNumber}">
	
				<xsl:choose>
					<xsl:when test="requestparameters">
					
						<xsl:variable name="requestparameters" select="requestparameters"/>
					
						<xsl:for-each select="requestparameters/parameter[name='addressID']/value">
							
							<xsl:variable name="street">
								<xsl:text>street-</xsl:text>
								<xsl:value-of select="."/>
							</xsl:variable>

							<xsl:variable name="postalAddress">
								<xsl:text>postalAddress-</xsl:text>
								<xsl:value-of select="."/>
							</xsl:variable>

							<xsl:call-template name="AddressSearch">
								<xsl:with-param name="addressID" select="."/>
								<xsl:with-param name="street" select="$requestparameters/parameter[name = $street]/value"/>
								<xsl:with-param name="postalAddress" select="$requestparameters/parameter[name = $postalAddress]/value"/>
							</xsl:call-template>
							
						</xsl:for-each>
						
					</xsl:when>
					<xsl:otherwise>
						
						<xsl:apply-templates select="$queryStateEvaluator/Addresses/Address" mode="search" />
						
					</xsl:otherwise>
				</xsl:choose>
			</div>
			
			<div id="address-template" style="display: none;">
				<xsl:call-template name="AddressSearch">
					<xsl:with-param name="addressID" />
					<xsl:with-param name="street" />
					<xsl:with-param name="postalAddress" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full ui-widget">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'address-search'" />
					<xsl:with-param name="placeholder" select="$i18n.Address.Add" />
				</xsl:call-template>
			</div>
		
		</div>
		
	</xsl:template>
	
	<xsl:template match="Address" mode="search" name="AddressSearch">
		<xsl:param name="addressID" select="addressID"/>
		<xsl:param name="street" select="street"/>
		<xsl:param name="postalAddress" select="postalAddress"/>
		
		<xsl:variable name="fullAddress">
			<xsl:value-of select="$street" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="$postalAddress" />
		</xsl:variable>
		
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<h3 class="floatleft marginleft" style="margin-top: 3px">
				<xsl:value-of select="$fullAddress" />
			</h3>
			
			<input name="addressID" hidden="true" value="{$addressID}"/>
			<input name="street-{$addressID}" hidden="true" value="{$street}"/>
			<input name="postalAddress-{$addressID}" hidden="true" value="{$postalAddress}"/>
			
			<div class="floatright padding">
				<a class="floatright marginright" href="#" onclick="if(confirm('{$i18n.Address.Delete.Confirm}?'))$(this).closest('div.border-radius').remove(); return false;" title="{$i18n.Address.Delete}">
					<img src="{$imgPath}/delete.png"/>
				</a>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="UpdateUserAddressQueryStateEvaluator">

		<xsl:call-template name="createQueryStateEvaluatorForm">
			<xsl:with-param name="queryStateEvaluator" select="UserAddressQueryStateEvaluator" />
		</xsl:call-template>

	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'queryState'">
				<xsl:value-of select="$i18n.queryState" />
			</xsl:when>
			<xsl:when test="$fieldName = 'address'">
				<xsl:value-of select="$i18n.Evaluator.addresses" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>
	
</xsl:stylesheet>