<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="staticcontent"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/></xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/UserGroupList.js
		/js/userlistquery.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/UserGroupList.css?v=1
		/css/userlistquery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="UserListQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="userlistquery show-mode">
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UserListQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UserListQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="UserListQueryInstance/UserListQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="UserListQueryInstance/UserListQuery/description">
					<span class="italic">
						<xsl:value-of select="UserListQueryInstance/UserListQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<fieldset>
				
						<table class="full">
							<thead>
								<tr>
									<th>
										<xsl:value-of select="$i18n.Column.Firstname"/>
									</th>
									<th>
										<xsl:value-of select="$i18n.Column.Lastname"/>
									</th>
	
									<xsl:if test="UserListQueryInstance/UserListQuery/showUsername = 'true'">
										<th>
											<xsl:value-of select="$i18n.Column.Username"/>
										</th>
									</xsl:if>	
									
									<xsl:if test="UserListQueryInstance/UserListQuery/showCitizenIdentifier = 'true'">
										<th>
											<xsl:value-of select="$i18n.Column.SocialSecurityNumber"/>
										</th>
									</xsl:if>
									
									<xsl:if test="UserListQueryInstance/UserListQuery/showContactInformation = 'true'">
										<th>
											<xsl:value-of select="$i18n.Column.Email"/>
										</th>
										<th>
											<xsl:value-of select="$i18n.Column.Phone"/>
										</th>
									</xsl:if>
								</tr>
								
							</thead>
							<tbody>
							
								<xsl:apply-templates select="UserListQueryInstance/Users/user" mode="show">
									<xsl:with-param name="showUsername" select="UserListQueryInstance/UserListQuery/showUsername"/>
									<xsl:with-param name="showCitizenIdentifier" select="UserListQueryInstance/UserListQuery/showCitizenIdentifier"/>
									<xsl:with-param name="showContactInformation" select="UserListQueryInstance/UserListQuery/showContactInformation"/>
								</xsl:apply-templates>
							
							</tbody>
						</table>
							
				</fieldset>
				
			</article>
				
		</div>
		
	</xsl:template>
	
	<xsl:template match="user" mode="show">
		<xsl:param name="showUsername" select="false()"/>
		<xsl:param name="showCitizenIdentifier" select="false()"/>
		<xsl:param name="showContactInformation" select="false()"/>
	
		<tr>
			<td>
				<xsl:value-of select="firstname" />
			</td>
			<td>
				<xsl:value-of select="lastname" />
			</td>
			
			<xsl:if test="$showUsername = 'true'">
				<td>
					<xsl:value-of select="username" />	
				</td>
			</xsl:if>			
			
			<xsl:if test="$showCitizenIdentifier = 'true'">
				<td>
					<xsl:value-of select="citizenIdentifier" />	
				</td>
			</xsl:if>
			
			<xsl:if test="$showContactInformation = 'true'">
				<td>
					<xsl:value-of select="email" />
				</td>
				<td>
					<xsl:value-of select="phone" />
				</td>	
			</xsl:if>
		</tr>
	</xsl:template>
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', UserListQueryInstance/UserListQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', UserListQueryInstance/UserListQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="UserListQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
						<xsl:choose>
							<xsl:when test="ValidationErrors/validationError[messageKey]">
								<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
							</xsl:when>
							<xsl:otherwise><span/></xsl:otherwise>
						</xsl:choose>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
			
			<article class="userlistquery">
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">userlistquery error</xsl:attribute>
				</xsl:if>
				
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UserListQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UserListQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="UserListQueryInstance/UserListQuery/helpText">		
						<xsl:apply-templates select="UserListQueryInstance/UserListQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:if test="UserListQueryInstance/UserListQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="UserListQueryInstance/UserListQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<fieldset>
				
					<xsl:call-template name="UserList">
						<xsl:with-param name="connectorURL" select="queryRequestURL"/>
						<xsl:with-param name="name">
							<xsl:text>q</xsl:text>
							<xsl:value-of select="UserListQueryInstance/UserListQuery/queryID"/>
							<xsl:text>_user</xsl:text>
						</xsl:with-param>
						<xsl:with-param name="users" select="UserListQueryInstance/Users" />
						<xsl:with-param name="showEmail" select="false()"/>
						<xsl:with-param name="showUsername" select="true()"/>
					</xsl:call-template>
				
				</fieldset>
				
				<script type="text/javascript">initUserListQuery('<xsl:value-of select="UserListQueryInstance/UserListQuery/queryID" />');</script>
				
			</article>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.Required"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'InvalidUser']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.InvalidUser"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TooManyAlternativesSelected']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part1"/>
				<xsl:value-of select="maxChecked"/>
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part2"/>
				<xsl:value-of select="checked"/>
				<xsl:value-of select="$i18n.TooManyAlternativesSelected.part3"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TooFewAlternativesSelected']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.TooFewAlternativesSelected.part1"/>
				<xsl:value-of select="minChecked"/>
				<xsl:value-of select="$i18n.TooFewAlternativesSelected.part2"/>
				<xsl:value-of select="checked"/>
				<xsl:value-of select="$i18n.TooFewAlternativesSelected.part3"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoCitizenID']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="user/firstname"/>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="user/lastname"/>
				<xsl:text> </xsl:text>
				<xsl:value-of select="$i18n.ValidationError.NoCitizenID"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<xsl:choose>
			<xsl:when test="fieldName != ''">
				
				<i data-icon-after="!" title="{$i18n.Error.UnknownValidationError}"></i>
				
			</xsl:when>
			<xsl:otherwise>
				
				<span>
					<strong data-icon-before="!">
						<xsl:value-of select="$i18n.Error.UnknownValidationError"/>
					</strong>
				</span>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
		
</xsl:stylesheet>