<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="staticcontent"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/></xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/fancytree/jquery.fancytree.min.js
		/js/fancytree/src/jquery.fancytree.persist.js
		/js/fancytree/src/jquery.fancytree.filter.js
		/js/usertreequery.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/fancytree/skin-win7/ui.fancytree.min.css
		/css/usertreequery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="UserTreeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="usertreequery show-mode">
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UserTreeQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UserTreeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="UserTreeQueryInstance/UserTreeQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="UserTreeQueryInstance/UserTreeQuery/description">
					<span class="italic">
						<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<fieldset>
				
						<xsl:if test="UserTreeQueryInstance/selectedNodeKey">
						
							<div>
								<h3><xsl:value-of select="$i18n.ChosenTree" /></h3>
								
								<div class="marginleft">
								
									<xsl:call-template name="FancyTreeShow">
										<xsl:with-param name="QueryID" select="UserTreeQueryInstance/UserTreeQuery/queryID"/>
										<xsl:with-param name="TreeNodes" select="UserTreeQueryInstance/SelectedTreeNodes/TreeNode[not(parentNodeKey)]"/>
									</xsl:call-template>
									
									<span class="floatleft bigmargintop">
										<xsl:value-of select="UserTreeQueryInstance/firstname" />
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="UserTreeQueryInstance/lastname" />
										
										<xsl:if test="UserTreeQueryInstance/UserTreeQuery/showCitizenIdentifier = 'true'">
											<xsl:text>&#160;(</xsl:text>
											<xsl:value-of select="UserTreeQueryInstance/citizenIdentifier" />	
											<xsl:text>)</xsl:text>
										</xsl:if>
										
										<xsl:if test="UserTreeQueryInstance/UserTreeQuery/showContactInformation = 'true'">
											<br/>
											<xsl:value-of select="UserTreeQueryInstance/email" />
											<br/>
											<xsl:value-of select="UserTreeQueryInstance/phone" />	
										</xsl:if>
									</span>
									
								</div>
								
								<script type="text/javascript">userTreeQueryCreateShowTree('<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/queryID" />');</script>
							</div>
							
						</xsl:if>
					
				</fieldset>
				
			</article>
				
		</div>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', UserTreeQueryInstance/UserTreeQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', UserTreeQueryInstance/UserTreeQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="UserTreeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
					
						<xsl:choose>
							<xsl:when test="ValidationErrors/validationError[messageKey]">
								<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
							</xsl:when>
							<xsl:otherwise><span/></xsl:otherwise>
						</xsl:choose>
						
						<div class="marker"/>
					</div>
				</div>
			</xsl:if>
			
			<article class="usertreequery">
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">usertreequery error</xsl:attribute>
				</xsl:if>
				
				<script type="text/javascript">
					UserTree = {
					"PicLoading": "<xsl:value-of select="$staticcontent"/>/pics/ajax-loader.gif",
					"PicEmpty": "<xsl:value-of select="$staticcontent"/>/pics/user_gray.png",
					"PicFailed": "<xsl:value-of select="$staticcontent"/>/pics/warning.png",
					"PicUser": "<xsl:value-of select="$staticcontent"/>/pics/user.png",
					"PicUserDisabled": "<xsl:value-of select="$staticcontent"/>/pics/user_gray.png",
					
					"i18nLoading": "<xsl:value-of select="$i18n.UserTree.Loading"/>",
					"i18nEmpty": "<xsl:value-of select="$i18n.UserTree.Empty"/>",
					"i18nFailed": "<xsl:value-of select="$i18n.UserTree.Failed"/>",
					"i18nEmailMissing": "<xsl:value-of select="$i18n.UserTree.EmailMissing"/>",
					"i18nInitialNodeTitle": "<xsl:value-of select="$i18n.UserTree.InitialNodeTitle"/>",
					};
				</script>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="UserTreeQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="UserTreeQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="UserTreeQueryInstance/UserTreeQuery/helpText">		
						<xsl:apply-templates select="UserTreeQueryInstance/UserTreeQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:if test="UserTreeQueryInstance/UserTreeQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
				
				<fieldset>
				
					<xsl:choose>
						<xsl:when test="UserTreeQueryInstance/FullTree">
						
							<div class="marginbottom">

								<xsl:call-template name="FancyTree">
									<xsl:with-param name="Query" select="UserTreeQueryInstance/UserTreeQuery"/>
									<xsl:with-param name="TreeNodes" select="UserTreeQueryInstance/FullTree/TreeNode"/>
								</xsl:call-template>
								
							</div>
							
						</xsl:when>
						<xsl:when test="UserTreeQueryInstance/selectedNodeKey">
						
							<div class="marginbottom">

								<xsl:call-template name="FancyTree">
									<xsl:with-param name="Query" select="UserTreeQueryInstance/UserTreeQuery"/>
									<xsl:with-param name="TreeNodes" select="UserTreeQueryInstance/SelectedTreeNodes/TreeNode[not(parentNodeKey)]"/>
								</xsl:call-template>
								
							</div>
					
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="$i18n.Error.UnableToGetTrees"/>
							
						</xsl:otherwise>
					</xsl:choose>
					
				</fieldset>
				
				<script type="text/javascript">initUserTreeQuery('<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/queryID" />');</script>
				
				<xsl:choose>
					<xsl:when test="requestparameters">
					
						<xsl:variable name="inputName">
							<xsl:value-of select="'q'"/>
							<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/queryID"/>
							<xsl:value-of select="'_selectedkey'"/>
						</xsl:variable>
						
						<xsl:if test="requestparameters/parameter[name = $inputName]">
							<script type="text/javascript">userTreeQuerySetSelected('<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/queryID" />', '<xsl:value-of select="requestparameters/parameter[name = $inputName]/value"/>');</script>
						</xsl:if>
						
					</xsl:when>
					<xsl:when test="UserTreeQueryInstance/selectedNodeKey">
					
						<script type="text/javascript">userTreeQuerySetSelected('<xsl:value-of select="UserTreeQueryInstance/UserTreeQuery/queryID" />', '<xsl:value-of select="UserTreeQueryInstance/selectedNodeKey"/>');</script>
					
					</xsl:when>
				</xsl:choose>
			</article>
			
			<div style="display: none;">
			
				<!-- Preload images -->
				<img src="{$staticcontent}/pics/ajax-loader.gif"/>
				<img src="{$staticcontent}/pics/warning.png"/>
				<img src="{$staticcontent}/pics/user_gray.png"/>
			
			</div>
		
		</div>
		
	</xsl:template>
	
	<xsl:template name="FancyTreeShow">
		<xsl:param name="QueryID"/>
		<xsl:param name="TreeNodes"/>
		
		<div id="tree{$QueryID}"/>
		
		<script type="text/javascript">var UserTreeQueryTree<xsl:value-of select="$QueryID"/> = [<xsl:apply-templates select="$TreeNodes"/>];</script>
	
	</xsl:template>
	
	<xsl:template name="FancyTree">
		<xsl:param name="Query"/>
		<xsl:param name="TreeNodes"/>
		
		<div class="floatleft fifty">
		
			<div class="floatright full marginbottom">
			
				<div class="floatright fifty">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id">
							<xsl:value-of select="'filterField'"/>
							<xsl:value-of select="$Query/queryID"/>
						</xsl:with-param>
						<xsl:with-param name="name">
							<xsl:value-of select="'filterField'"/>
							<xsl:value-of select="$Query/queryID"/>
						</xsl:with-param>
					</xsl:call-template>
				</div>
				
				<div class="floatright margintop">
					<label for="filterField{$Query/queryID}" class="floatright marginright">
						<xsl:value-of select="$i18n.Filter" />
					</label>
	    	</div>
	    	
			</div>
			
			<div id="tree{$Query/queryID}" data-nofolders="{$Query/onlyAllowSelectingLeafs}"/>
			
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="name">
					<xsl:value-of select="'q'"/>
					<xsl:value-of select="$Query/queryID"/>
					<xsl:value-of select="'_selectedkey'"/>
				</xsl:with-param>
				<xsl:with-param name="class" select="'selectedkey'" />
			</xsl:call-template>
			
			<script type="text/javascript">var UserTreeQueryTree<xsl:value-of select="$Query/queryID"/> = [<xsl:apply-templates select="$TreeNodes"/>];</script>
		</div>
		
		<div class="floatleft fifty">
		
			<div class="floatright full marginbottom">
			
				<div class="floatright fifty">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id">
							<xsl:value-of select="'filterField2'"/>
							<xsl:value-of select="$Query/queryID"/>
						</xsl:with-param>
						<xsl:with-param name="name">
							<xsl:value-of select="'filterField2'"/>
							<xsl:value-of select="$Query/queryID"/>
						</xsl:with-param>
					</xsl:call-template>
				</div>
				
				<div class="floatright margintop">
					<label for="filterField2{$Query/queryID}" class="floatright marginright">
						<xsl:value-of select="$i18n.Filter" />
					</label>
	    	</div>
	    	
			</div>
			
			<div id="usertree{$Query/queryID}" data-fetchurl="{queryRequestURL}" data-requireemail="{$Query/requireEmail}"/>
			
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="name">
					<xsl:value-of select="'q'"/>
					<xsl:value-of select="$Query/queryID"/>
					<xsl:value-of select="'_selecteduser'"/>
				</xsl:with-param>
				<xsl:with-param name="class" select="'selecteduser'" />
				<xsl:with-param name="value" select="$Query/../providerUserID"/>
			</xsl:call-template>
		</div>
	
	</xsl:template>
	
	<xsl:template match="TreeNode[parent::SelectedTreeNodes]">
	
		<xsl:variable name="key" select="key"/>
	
		<xsl:variable name="escapedKey">
			<xsl:call-template name="escapeForJavascript">
				<xsl:with-param name="text" select="key"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="escapedName">
			<xsl:call-template name="escapeForJavascript">
				<xsl:with-param name="text" select="name"/>
			</xsl:call-template>
		</xsl:variable>
	
		<xsl:text>{</xsl:text>
		
		<xsl:text>title:'</xsl:text>
		<xsl:value-of select="$escapedName"/>
		<xsl:text>'</xsl:text>
		
		<xsl:text>,key:'</xsl:text>
		<xsl:value-of select="$escapedKey"/>
		<xsl:text>'</xsl:text>
		
		<xsl:if test="icon">
			<xsl:text>,icon:'</xsl:text>
			<xsl:value-of select="icon"/>
			<xsl:text>'</xsl:text>
		</xsl:if>
		
		<xsl:if test="folder">
			<xsl:text>,folder:</xsl:text>
			<xsl:value-of select="folder"/>
		</xsl:if>
		
		<xsl:if test="../TreeNode[parentNodeKey = $key]">
			
			<xsl:text>,children:[</xsl:text>
			<xsl:apply-templates select="../TreeNode[parentNodeKey = $key]"/>
			<xsl:text>]</xsl:text>
			
		</xsl:if>
		
		<xsl:text>}</xsl:text>
	
	</xsl:template>
	
	<xsl:template match="TreeNode">
	
		<xsl:variable name="position">
			<xsl:number/>
		</xsl:variable>
	
		<xsl:if test="not($position = 1)">
			<xsl:text>,</xsl:text>
		</xsl:if>
		
		<xsl:variable name="escapedKey">
			<xsl:call-template name="escapeForJavascript">
				<xsl:with-param name="text" select="key"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="escapedName">
			<xsl:call-template name="escapeForJavascript">
				<xsl:with-param name="text" select="name"/>
			</xsl:call-template>
		</xsl:variable>
	
		<xsl:text>{</xsl:text>
		
		<xsl:text>title:'</xsl:text>
		<xsl:value-of select="$escapedName"/>
		<xsl:text>'</xsl:text>
		
		<xsl:text>,key:'</xsl:text>
		<xsl:value-of select="$escapedKey"/>
		<xsl:text>'</xsl:text>
		
		<xsl:if test="icon">
			<xsl:text>,icon:'</xsl:text>
			<xsl:value-of select="icon"/>
			<xsl:text>'</xsl:text>
		</xsl:if>
		
		<xsl:if test="folder">
			<xsl:text>,folder:</xsl:text>
			<xsl:value-of select="folder"/>
		</xsl:if>
		
		<xsl:if test="Children">
			
			<xsl:text>,children:[</xsl:text>
			<xsl:apply-templates select="Children/TreeNode"/>
			<xsl:text>]</xsl:text>
			
		</xsl:if>
		
		<xsl:text>}</xsl:text>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredTreeNode']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.Required"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredUser']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.Required"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'InvalidTreeNode']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.InvalidTreeNode"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'InvalidUser']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.Error.InvalidUser"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<xsl:choose>
			<xsl:when test="fieldName != ''">
				
				<i data-icon-after="!" title="{$i18n.Error.UnknownValidationError}"></i>
				
			</xsl:when>
			<xsl:otherwise>
				
				<span>
					<strong data-icon-before="!">
						<xsl:value-of select="$i18n.Error.UnknownValidationError"/>
					</strong>
				</span>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
		
</xsl:stylesheet>