package com.nordicpeak.flowengine.evaluators.settextfieldweightevaluator;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.RequiredIfNotSet;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_textfield_weight_evaluators")
@XMLElement
public class SetTextFieldWeightEvaluator extends BaseEvaluator {

	private static final EnumPopulator<EvaluationMode> EVALUATION_MODE_POPULATOR = new EnumPopulator<EvaluationMode>(EvaluationMode.class);
	
	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer textFieldID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "evaluationMode", paramValues = "USE_INTERVALL")
	@XMLElement
	private Integer fromValue;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer toValue;

	@DAOManaged
	@WebPopulate
	@RequiredIfNotSet(paramNames = "evaluationMode", paramValues = "USE_FIELD_VALUE")
	@XMLElement
	private Integer weight;
	
	@DAOManaged
	@WebPopulate(required=true)
	@XMLElement
	private EvaluationMode evaluationMode;

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public BigDecimal getWeight(String textValue) {
		
		if (!StringUtils.isEmpty(textValue)) {
			
			if (evaluationMode == EvaluationMode.USE_FIXED_VALUE) {
				
				return new BigDecimal(weight);
			}
			
			BigDecimal value = NumberUtils.toBigDecimalLocaleIndependent(textValue);
			
			if (value != null) {
				
				if (evaluationMode == EvaluationMode.USE_INTERVALL) {
					
					if (isInIntervall(value)) {
						
						return new BigDecimal(weight);
					}
					
				} else if (evaluationMode == EvaluationMode.USE_FIELD_VALUE) {
					
					return value;
				}
			}
		}
		
		return BigDecimal.ZERO;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		textFieldID = XMLValidationUtils.validateParameter("textFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		Boolean useIntervalWeight = xmlParser.getBoolean("useIntervallWeights");

		//Backwards compability
		if(useIntervalWeight != null){
			
			if(useIntervalWeight){
				
				evaluationMode = EvaluationMode.USE_INTERVALL;
				
			}else{
				
				evaluationMode = EvaluationMode.USE_FIELD_VALUE;
			}
			
		}else{
			
			evaluationMode = XMLValidationUtils.validateParameter("evaluationMode", xmlParser, true, EVALUATION_MODE_POPULATOR, errors);
		}
		
		if(evaluationMode != null){

			weight = XMLValidationUtils.validateParameter("weight", xmlParser, evaluationMode != EvaluationMode.USE_FIELD_VALUE, IntegerPopulator.getPopulator(), errors);
			fromValue = XMLValidationUtils.validateParameter("fromValue", xmlParser, evaluationMode == EvaluationMode.USE_INTERVALL, IntegerPopulator.getPopulator(), errors);
			toValue = XMLValidationUtils.validateParameter("toValue", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getTextFieldID() {

		return textFieldID;
	}

	public void setTextFieldID(Integer textFieldID) {

		this.textFieldID = textFieldID;
	}

	public boolean isInIntervall(BigDecimal value) {

		if (value.compareTo(BigDecimal.valueOf(fromValue)) >= 0 && (toValue == null || value.compareTo(BigDecimal.valueOf(toValue)) <= 0)) {

			return true;
		}

		return false;
	}

	public Integer getFromValue() {

		return fromValue;
	}

	public void setFromValue(Integer fromValue) {

		this.fromValue = fromValue;
	}

	public Integer getToValue() {

		return toValue;
	}

	public void setToValue(Integer toValue) {

		this.toValue = toValue;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	
	public EvaluationMode getEvaluationMode() {
	
		return evaluationMode;
	}

	
	public void setEvaluationMode(EvaluationMode evaluationMode) {
	
		this.evaluationMode = evaluationMode;
	}
}
