package com.nordicpeak.flowengine.evaluators.settextfieldweightfromapievaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.EvaluatorDescriptor;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.evaluators.weightapi.WeightAPIAdminModule;
import com.nordicpeak.flowengine.evaluators.weightapi.beans.WeightAPISource;
import com.nordicpeak.flowengine.evaluators.weightapi.interfaces.WeightAPIEvaluator;
import com.nordicpeak.flowengine.evaluators.weightapi.interfaces.WeightAPISourceEvaluatorProvider;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldValue;

public class SetTextFieldWeightFromAPIEvaluationProviderModule extends BaseEvaluationProviderModule<SetTextFieldWeightFromAPIEvaluator> implements WeightAPISourceEvaluatorProvider {

	@XSLVariable(prefix = "java.")
	private String missingSource = "Missing";

	protected SetTextFieldWeightFromAPIEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetTextFieldWeightFromAPIEvaluator> evaluatorDAO;
	private AnnotatedDAOWrapper<EvaluatorDescriptor, Integer> evaluatorDescriptorDAOWrapper;

	protected QueryParameterFactory<SetTextFieldWeightFromAPIEvaluator, Integer> evaluatorIDParamFactory;
	protected QueryParameterFactory<SetTextFieldWeightFromAPIEvaluator, String> sourceNameParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	protected WeightAPIAdminModule sourceAdminModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetTextFieldWeightFromAPIEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetTextFieldWeightFromAPIEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		sourceNameParamFactory = evaluatorDAO.getParamFactory("sourceName", String.class);

		evaluatorCRUD = new SetTextFieldWeightFromAPIEvaluatorCRUD(SetTextFieldWeightFromAPIEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);

		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		evaluatorDescriptorDAOWrapper = flowDAOFactory.getEvaluatorDescriptorDAO().getWrapper(Integer.class);
		evaluatorDescriptorDAOWrapper.addRelations(EvaluatorDescriptor.QUERY_DESCRIPTOR_RELATION, QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		evaluatorDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetTextFieldWeightFromAPIEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (evaluator.getWeightType() == null || evaluator.getSourceName() == null) {
			return null;
		}

		if (!(queryInstance instanceof TextFieldQueryInstance)) {

			log.warn("Query instance " + queryInstance + " is not of the type " + TextFieldQueryInstance.class.getName());
			return null;
		}

		ImmutableFlow flow = queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getStep().getFlow();

		WeightAPISource source;

		try {
			source = sourceAdminModule.getSource(evaluator.getSourceName(), flow);

			if (source == null) {
				log.warn("Source with name " + evaluator.getSourceName() + " not found");
				return null;
			}

		} catch (SQLException e) {
			log.error("Error getting source with name " + evaluator.getSourceName(), e);
			return null;
		}

		BigDecimal weight = BigDecimal.ZERO;

		List<TextFieldValue> textFieldValues = ((TextFieldQueryInstance) queryInstance).getValues();

		if (textFieldValues != null) {

			for (TextFieldValue textFieldValue : textFieldValues) {

				if (textFieldValue.getTextField().getTextFieldID().equals(evaluator.getTextFieldID())) {

					weight = sourceAdminModule.getWeight(source, textFieldValue.getValue(), poster, attributeHandler);
					break;
				}
			}
		}

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetTextFieldWeightFromAPIEvaluator evaluator = new SetTextFieldWeightFromAPIEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return TextFieldQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetTextFieldWeightFromAPIEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		List<TextField> sourceTextFields = ((TextFieldQuery) sourceQuery).getFields();
		List<TextField> copyTextFields = ((TextFieldQuery) copyQuery).getFields();

		if (sourceTextFields != null && copyTextFields != null) {

			int textFieldIndex = 0;

			while (textFieldIndex < sourceTextFields.size()) {

				if (sourceTextFields.get(textFieldIndex).getTextFieldID().equals(evaluator.getTextFieldID())) {

					evaluator.setTextFieldID(copyTextFields.get(textFieldIndex).getTextFieldID());

					break;
				}

				textFieldIndex++;
			}

		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	protected Class<SetTextFieldWeightFromAPIEvaluator> getEvaluatorClass() {

		return SetTextFieldWeightFromAPIEvaluator.class;
	}

	protected AnnotatedDAO<SetTextFieldWeightFromAPIEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetTextFieldWeightFromAPIEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws SQLException {

		SetTextFieldWeightFromAPIEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	@Override
	public SetTextFieldWeightFromAPIEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetTextFieldWeightFromAPIEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		if (evaluator.getTextFieldID() != null && query instanceof TextFieldQuery && ((TextFieldQuery) query).getTextFieldConversionMap() != null) {

			Map<Integer, Integer> conversionMap = ((TextFieldQuery) query).getTextFieldConversionMap();

			Integer newTextFieldID = conversionMap.get(evaluator.getTextFieldID());

			evaluator.setTextFieldID(newTextFieldID);
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	@Override
	public Collection<? extends WeightAPIEvaluator> getEvaluatorsUsingAPISource(WeightAPISource source) {

		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(evaluatorDAO.getDataSource(), "SELECT evaluatorID FROM " + evaluatorDAO.getTableName() + " WHERE sourceName = ?", IntegerPopulator.getPopulator());
			idQuery.setString(1, source.getName());

			List<Integer> evaluatorIDs = idQuery.executeQuery();

			if (!CollectionUtils.isEmpty(evaluatorIDs)) {

				List<WeightAPIEvaluator> evaluators = new ArrayList<>(evaluatorIDs.size());

				for (Integer evaluatorID : evaluatorIDs) {

					EvaluatorDescriptor queryDescriptor = evaluatorDescriptorDAOWrapper.get(evaluatorID);

					if (queryDescriptor != null) {

						evaluators.add((WeightAPIEvaluator) getEvaluator(queryDescriptor, null));
					}
				}

				return evaluators;
			}

		} catch (SQLException e) {

			log.error("Unable to get evaluators for API source " + source, e);
		}

		return null;
	}

	@Override
	public boolean hasEvaluatorsUsingAPISource(WeightAPISource source) {

		try {
			HighLevelQuery<SetTextFieldWeightFromAPIEvaluator> query = new HighLevelQuery<>();
			
			query.addParameter(sourceNameParamFactory.getParameter(source.getName()));
			query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
			
			return evaluatorDAO.getBoolean(query);
			
		} catch (SQLException e) {

			log.error("Unable to get evaluators for API source " + source, e);
		}

		return false;
	}
	
	public WeightAPIAdminModule getSourceAdminModule() {

		return sourceAdminModule;
	}

	public String getMissingAPISource() {

		return missingSource;
	}

}
