package com.nordicpeak.flowengine.evaluators.settextfieldweightfromapievaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.evaluators.weightapi.beans.WeightAPISource;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;

public class SetTextFieldWeightFromAPIEvaluatorCRUD extends BaseEvaluatorCRUD<SetTextFieldWeightFromAPIEvaluator, SetTextFieldWeightFromAPIEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SetTextFieldWeightFromAPIEvaluator> POPULATOR = new AnnotatedRequestPopulator<>(SetTextFieldWeightFromAPIEvaluator.class);

	public SetTextFieldWeightFromAPIEvaluatorCRUD(Class<SetTextFieldWeightFromAPIEvaluator> beanClass, AnnotatedDAOWrapper<SetTextFieldWeightFromAPIEvaluator, Integer> evaluatorDAO, SetTextFieldWeightFromAPIEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetTextFieldWeightFromAPIEvaluator", "set text field weight evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(SetTextFieldWeightFromAPIEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<TextField> fields = query.getFields();

		if (fields != null) {

			Element fieldsElement = doc.createElement("TextFields");
			updateTypeElement.appendChild(fieldsElement);

			for (TextField field : fields) {

				Element fieldElement = doc.createElement("TextField");
				fieldsElement.appendChild(fieldElement);

				XMLUtils.appendNewElement(doc, fieldElement, "fieldID", field.getTextFieldID());
				XMLUtils.appendNewCDATAElement(doc, fieldElement, "name", field.getLabel());
			}

		}

		Element sourcesElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Sources");
		
		ImmutableFlow flow = evaluator.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow();
		
		List<WeightAPISource> sources = callback.getSourceAdminModule().getSources(flow);
		
		if(sources != null) {
		
			for (WeightAPISource source : sources) {

				Element providerElement = XMLUtils.appendNewElement(doc, sourcesElement, "APISource");

				XMLUtils.appendNewElement(doc, providerElement, "Name", source.getName());
				XMLUtils.appendNewElement(doc, providerElement, "ID", source.getName());
			}
		}

		if (evaluator.getSourceName() != null) {

			WeightAPISource currentSource = callback.getSourceAdminModule().getSource(evaluator.getSourceName(),flow);

			if (currentSource == null) {

				Element providerElement = XMLUtils.appendNewElement(doc, sourcesElement, "APISource");

				XMLUtils.appendNewElement(doc, providerElement, "Name", callback.getMissingAPISource() + evaluator.getSourceName());
				XMLUtils.appendNewElement(doc, providerElement, "ID", evaluator.getSourceName());
			}
		}

	}

	@Override
	protected SetTextFieldWeightFromAPIEvaluator populateFromUpdateRequest(SetTextFieldWeightFromAPIEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		SetTextFieldWeightFromAPIEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> errors = new ArrayList<>();

		Integer textFieldID = bean.getTextFieldID();

		boolean found = false;

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		if (query.getFields() != null) {

			for (TextField field : query.getFields()) {

				if (field.getTextFieldID().equals(textFieldID)) {

					found = true;
					break;
				}
			}
		}

		if (!found) {
			errors.add(new ValidationError("TextFieldNotFound"));
		}

		if (!errors.isEmpty()) {
			throw new ValidationException(errors);
		}

		return evaluator;
	}

}
