package com.nordicpeak.flowengine.evaluators.setweightbycontactfieldevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.ContactFieldQuery;
import com.nordicpeak.flowengine.interfaces.ContactFieldQueryInstance;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

public class SetWeightByContactFieldEvaluationProviderModule extends BaseEvaluationProviderModule<SetWeightByContactFieldEvaluator> {

	@XSLVariable(name = "i18n.Value.MatchedValue")
	private String matchedValueDisplayName;

	@XSLVariable(name = "i18n.Value.Weight")
	private String weightDisplayName;

	protected SetWeightByContactFieldEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetWeightByContactFieldEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetWeightByContactFieldEvaluator, Integer> evaluatorIDParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetWeightByContactFieldEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetWeightByContactFieldEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetWeightByContactFieldEvaluatorCRUD(SetWeightByContactFieldEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetWeightByContactFieldEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (!evaluator.isConfigured()) {

			return null;
		}

		if (!(queryInstance instanceof ContactFieldQueryInstance)) {

			log.warn("Query instance " + queryInstance + " does not implement interface " + ContactFieldQueryInstance.class.getName());

			return null;
		}

		ContactFieldQueryInstance contactFieldQueryInstance = (ContactFieldQueryInstance) queryInstance;

		BigDecimal weight = getWeight(contactFieldQueryInstance, evaluator);

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	private BigDecimal getWeight(ContactFieldQueryInstance contactFieldQueryInstance, SetWeightByContactFieldEvaluator evaluator) {

		String contactFieldValue = getContactFieldValue(contactFieldQueryInstance, evaluator);

		if (contactFieldValue == null) {

			if (evaluator.isSetWeightIfContactFieldIsEmpty() && evaluator.getWeightIfContactFieldIsEmpty() != null) {

				return BigDecimal.valueOf(evaluator.getWeightIfContactFieldIsEmpty());
			}

			return BigDecimal.ZERO;
		}

		SetWeightByContactFieldEvaluatorValue value = CollectionUtils.find(evaluator.getValues(), v -> v.getMatchedValue().equals(contactFieldValue));

		if (value == null) {

			if (evaluator.isSetWeightIfNoValueMatches() && evaluator.getWeightIfNoValueMatches() != null) {

				return BigDecimal.valueOf(evaluator.getWeightIfNoValueMatches());
			}

			return BigDecimal.ZERO;
		}

		return BigDecimal.valueOf(value.getWeight());
	}

	private String getContactFieldValue(ContactFieldQueryInstance contactFieldQueryInstance, SetWeightByContactFieldEvaluator evaluator) {

		try {

			return contactFieldQueryInstance.getContactFieldValue(evaluator.getContactField());

		} catch (IllegalArgumentException | IllegalAccessException e) {

			log.error("Error getting contact field value for evaluator " + evaluator, e);
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetWeightByContactFieldEvaluator evaluator = new SetWeightByContactFieldEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return ContactFieldQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetWeightByContactFieldEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		if (evaluator.getValues() != null) {
			
			evaluator.getValues().forEach(value -> value.setValueID(null));
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	public SetWeightByContactFieldEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetWeightByContactFieldEvaluator evaluator = new SetWeightByContactFieldEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	protected Class<SetWeightByContactFieldEvaluator> getEvaluatorClass() {

		return SetWeightByContactFieldEvaluator.class;
	}

	protected AnnotatedDAO<SetWeightByContactFieldEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetWeightByContactFieldEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetWeightByContactFieldEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = super.createDocument(req, uriParser, user);

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));

		return doc;
	}

	public String getMatchedValueDisplayName() {

		return matchedValueDisplayName;
	}

	public void setMatchedValueDisplayName(String matchedValueDisplayName) {

		this.matchedValueDisplayName = matchedValueDisplayName;
	}

	public String getWeightDisplayName() {

		return weightDisplayName;
	}

	public void setWeightDisplayName(String weightDisplayName) {

		this.weightDisplayName = weightDisplayName;
	}

}
