package com.nordicpeak.flowengine.evaluators.setweightbytextfieldvalueevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class SetWeightByTextFieldValueEvaluationProviderModule extends BaseEvaluationProviderModule<SetWeightByTextFieldValueEvaluator> {
	
	protected SetWeightByTextFieldValueEvaluatorCRUD evaluatorCRUD;
	
	protected AnnotatedDAO<SetWeightByTextFieldValueEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<SetWeightByTextFieldValueEvaluator, Integer> evaluatorIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetWeightByTextFieldValueEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetWeightByTextFieldValueEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new SetWeightByTextFieldValueEvaluatorCRUD(SetWeightByTextFieldValueEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetWeightByTextFieldValueEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (!(queryInstance instanceof TextFieldQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " is not of the type " + TextFieldQueryInstance.class.getName());
			
			return null;
		}
		
		TextFieldQueryInstance textFieldQueryInstance = ((TextFieldQueryInstance) queryInstance);
		
		BigDecimal weight = BigDecimal.ZERO;

		boolean matches = textFieldValueMatches(textFieldQueryInstance.getFieldValue(evaluator.getTextFieldID()), evaluator);
		
		if((matches && evaluator.isInclusive()) || (!matches && !evaluator.isInclusive())) {
			
			weight = BigDecimal.valueOf(evaluator.getWeight());
		}
		
		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), false);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}
		
		return null;
	}
	
	private boolean textFieldValueMatches(String textFieldValue, SetWeightByTextFieldValueEvaluator evaluator) {
		
		if(textFieldValue == null || evaluator.getAllowedValues() == null || !evaluator.getAllowedValues().contains(textFieldValue)) {
			
			return false;
		}
		
		return true;
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		SetWeightByTextFieldValueEvaluator evaluator = new SetWeightByTextFieldValueEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return TextFieldQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		SetWeightByTextFieldValueEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetWeightByTextFieldValueEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		List<TextField> sourceTextFields = ((TextFieldQuery) sourceQuery).getFields();
		List<TextField> copyTextFields = ((TextFieldQuery) copyQuery).getFields();
		
		if (sourceTextFields != null && copyTextFields != null) {
			
			int textFieldIndex = 0;
			
			while (textFieldIndex < sourceTextFields.size()) {
				
				if (sourceTextFields.get(textFieldIndex).getTextFieldID().equals(evaluator.getTextFieldID())) {
					
					evaluator.setTextFieldID(copyTextFields.get(textFieldIndex).getTextFieldID());
					
					break;
				}
				
				textFieldIndex++;
			}
			
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	protected Class<SetWeightByTextFieldValueEvaluator> getEvaluatorClass() {
		
		return SetWeightByTextFieldValueEvaluator.class;
	}
	
	protected AnnotatedDAO<SetWeightByTextFieldValueEvaluator> getEvaluatorDAO() {
		
		return evaluatorDAO;
	}
	
	protected QueryParameterFactory<SetWeightByTextFieldValueEvaluator, Integer> getEvaluatorIDParameterFactory() {
		
		return evaluatorIDParamFactory;
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		SetWeightByTextFieldValueEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetWeightByTextFieldValueEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			if (instanceMetadata != null && instanceMetadata.getSiteProfile() != null) {
				
				TextTagReplacer.replaceTextTags(evaluator, instanceMetadata.getSiteProfile());
			}
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<SetWeightByTextFieldValueEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	public QueryHandler getQueryHandler() {
		
		return this.queryHandler;
	}
	
	@Override
	public SetWeightByTextFieldValueEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		SetWeightByTextFieldValueEvaluator evaluator = getEvaluatorClass().newInstance();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		if (evaluator.getTextFieldID() != null && query instanceof TextFieldQuery && ((TextFieldQuery) query).getTextFieldConversionMap() != null) {
			
			Map<Integer, Integer> conversionMap = ((TextFieldQuery) query).getTextFieldConversionMap();
			
			Integer newTextFieldID = conversionMap.get(evaluator.getTextFieldID());
			
			evaluator.setTextFieldID(newTextFieldID);
		}
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
