package com.nordicpeak.flowengine.evaluators.setweightfromapievaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.evaluators.weightapi.beans.WeightAPISource;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;

public class SetWeightFromAPIEvaluatorCRUD extends BaseEvaluatorCRUD<SetWeightFromAPIEvaluator, SetWeightFromAPIEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SetWeightFromAPIEvaluator> POPULATOR = new AnnotatedRequestPopulator<>(SetWeightFromAPIEvaluator.class);

	public SetWeightFromAPIEvaluatorCRUD(Class<SetWeightFromAPIEvaluator> beanClass, AnnotatedDAOWrapper<SetWeightFromAPIEvaluator, Integer> evaluatorDAO, SetWeightFromAPIEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetWeightFromAPIEvaluator", "set text field weight evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(SetWeightFromAPIEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);

		Element sourcesElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Sources");
		
		ImmutableFlow flow = evaluator.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow();
		
		List<WeightAPISource> sources = callback.getSourceAdminModule().getSources(flow);
		
		if(sources != null) {
		
			for (WeightAPISource source : sources) {

				Element providerElement = XMLUtils.appendNewElement(doc, sourcesElement, "APISource");

				XMLUtils.appendNewElement(doc, providerElement, "Name", source.getName());
				XMLUtils.appendNewElement(doc, providerElement, "ID", source.getName());
			}
		}

		if (evaluator.getSourceName() != null) {

			WeightAPISource currentSource = callback.getSourceAdminModule().getSource(evaluator.getSourceName(),flow);

			if (currentSource == null) {

				Element providerElement = XMLUtils.appendNewElement(doc, sourcesElement, "APISource");

				XMLUtils.appendNewElement(doc, providerElement, "Name", callback.getMissingAPISource() + evaluator.getSourceName());
				XMLUtils.appendNewElement(doc, providerElement, "ID", evaluator.getSourceName());
			}
		}

	}

	@Override
	protected SetWeightFromAPIEvaluator populateFromUpdateRequest(SetWeightFromAPIEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		SetWeightFromAPIEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> errors = new ArrayList<>();

		if (!errors.isEmpty()) {
			throw new ValidationException(errors);
		}

		return evaluator;
	}

}
